// swad_text.c: text messages on screen, in several languages

/*
    SWAD (Shared Workspace At a Distance in Spanish),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.

    This file is part of SWAD core.
    Copyright (C) 1999-2019 Antonio Caas Vargas

    Catal translation:
	Antonio Caas Vargas
	Joan Llus Daz Rodrguez
    German translation:
	Antonio Caas Vargas
	Rafael Barranco-Droege
    English translation (finished):
	Antonio Caas Vargas
    Spanish translation (finished):
	Antonio Caas Vargas
    French translation:
	Antonio Caas Vargas
    Guarani translation:
	Antonio Caas Vargas
    Italian translation (finished):
	Antonio Caas Vargas
	Nicola Comunale Rizzo
	Francisco Manuel Herrero Prez
	Giuseppe Antonio Pagin
	Antonella Grande
    Polish translation:
	Antonio Caas Vargas
	Wojtek Kieca
	Tomasz Olechowski
	Mateusz Stanko
    Portuguese translation:
	Antonio Caas Vargas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*****************************************************************************/
/********************************* Headers ***********************************/
/*****************************************************************************/

#include <linux/stddef.h>	// For NULL

#include "swad_action.h"
#include "swad_assignment.h"
#include "swad_centre.h"
#include "swad_classroom.h"
#include "swad_config.h"
#include "swad_country.h"
#include "swad_course.h"
#include "swad_date.h"
#include "swad_degree.h"
#include "swad_degree_type.h"
#include "swad_department.h"
#include "swad_figure.h"
#include "swad_file_browser.h"
#include "swad_forum.h"
#include "swad_holiday.h"
#include "swad_info.h"
#include "swad_institution.h"
#include "swad_language.h"
#include "swad_mail.h"
#include "swad_menu.h"
#include "swad_notification.h"
#include "swad_photo.h"
#include "swad_place.h"
#include "swad_privacy.h"
#include "swad_project.h"
#include "swad_record.h"
#include "swad_role.h"
#include "swad_statistic.h"
#include "swad_survey.h"
#include "swad_syllabus.h"
#include "swad_tab.h"
#include "swad_test.h"
#include "swad_timeline.h"
#include "swad_timetable.h"
#include "swad_user.h"

/*****************************************************************************/
/******************************** Constants **********************************/
/*****************************************************************************/

#ifndef L
#define L 3	// English
#endif

const unsigned Txt_Current_CGI_SWAD_Language = ((unsigned) L);

const char *Txt_NEW_LINE = "\r\n";	// End of line in a file. If we put only \n the file does not look good in some Windows text editors

// The HTML entity for "" is &szlig; It stands for "S-Z ligature", because this symbol comes from the ligature of a Gothic S and a Z.
// For polish HTML entities, see https://pl.wikipedia.org/wiki/Alfabet_polski
// Guarani:
// E with tilde: &#7868;
// e with tilde: &#7869;
// I with tilde: &#296;
// i with tilde: &#297;

/***** Languages *****/
const char *Txt_STR_LANG_NAME[1 + Lan_NUM_LANGUAGES] =
	{
	"",
	"Catal&agrave;",						// ca
	"Deutsch",							// de
	"English",							// en
	"Espa&ntilde;ol",						// es
	"Fran&ccedil;ais",						// fr
	"Ava&ntilde;e'&#7869;",						// gn
	"Italiano",							// it
	"Polski",							// pl
	"Portugu&ecirc;s",						// pt
	};
const char *Txt_Do_you_want_to_change_the_language_to_LANGUAGE[1 + Lan_NUM_LANGUAGES] =
	{
	"",
	"Voleu canviar l'idioma a catal&agrave;?",			// ca
	"Wollen Sie die Sprache auf Deutsch umstellen?",		// de
	"Do you want to change the language to English?",		// en
	"&iquest;Desea cambiar el idioma a espa&ntilde;ol?",		// es
	"Voulez-vous changer la langue au fran&ccedil;ais?",		// fr
	"&iquest;Oipota moambue pe &ntilde;e'&#7869; pe ava&ntilde;e'&#7869;?",	// gn
	"Vuoi cambiare la lingua in italiano?",				// it
	"Czy chcesz zmieni&cacute; j&eogon;zyk na polski?",		// pl
	"Voc&ecirc; quer mudar o idioma para portugu&ecirc;s?",		// pt
	};
const char *Txt_Do_you_want_to_change_your_language_to_LANGUAGE[1 + Lan_NUM_LANGUAGES] =
	{
	"",
	"Voleu canviar el seu idioma a catal&agrave;?",			// ca
	"Wollen Sie Ihre Sprache auf Deutsch umstellen?",		// de
	"Do you want to change your language to English?",		// en
	"&iquest;Desea cambiar su idioma a espa&ntilde;ol?",		// es
	"Voulez-vous changer votre langue au fran&ccedil;ais?",		// fr
	"&iquest;Oipota moambue pe &ntilde;e'&#7869; pe ava&ntilde;e'&#7869;?",	// gn
	"Vuoi cambiare la tua lingua in italiano?",			// it
	"Czy chcesz zmieni&cacute; j&eogon;zyk na polski?",		// pl
	"Voc&ecirc; quer mudar suo idioma para portugu&ecirc;s?",	// pt
	};
const char *Txt_Switch_to_LANGUAGE[1 + Lan_NUM_LANGUAGES] =
	{
	"",
	"Canviar a catal&agrave;",					// ca
	"Umschalten auf Deutsch",					// de
	"Switch to English",						// en
	"Cambiar a espa&ntilde;ol",					// es
	"Passer au fran&ccedil;ais",					// fr
	"Moambue pe &ntilde;e'&#7869; ava&ntilde;e'&#7869;",		// gn
	"Passare a italiano",						// it
	"Prze&lstrok;&aogon;cz na polski",				// pl
	"Mudar para portugu&ecirc;s",					// pt
	};
const char *Txt_Switching_to_LANGUAGE[1 + Lan_NUM_LANGUAGES] =
	{
	"",
	"Canviant a catal&agrave;&hellip;",				// ca
	"Umschaltung auf Deutsch&hellip;",				// de
	"Switching to English&hellip;",					// en
	"Cambiando a espa&ntilde;ol&hellip;",				// es
	"Passant au fran&ccedil;ais&hellip;",				// fr
	"Moambue pe &ntilde;e'&#7869; ava&ntilde;e'&#7869;&hellip;",	// gn
	"Passando a italiano&hellip;",					// it
	"Prze&lstrok;&aogon;cz na polski&hellip;",			// pl
	"Mudando para portugu&ecirc;s&hellip;",				// pt
	};

/***** Messages translated (use HTML entities) *****/

const char *Txt_A_face_marked_in_green_has_been_detected_ =
#if   L==1	// ca
	"S'ha detectat un rostre (senyalat en verd) en posici&oacute; frontal"
	" i amb el fons prou clar.<br />"
	"Premeu sobre ell per confirmar la actualizaci&oacute; de la fotografia.";
#elif L==2	// de
	"Ein Gesicht (gr&uuml;n markiert) wurde vor Position erfasst"
	" und der Hintergrund ist genug hell.<br />"
	"Klicken Sie darauf, um das Update des Bildes zu best&auml;tigen.";
#elif L==3	// en
	"A face (marked in green) has been detected in front position"
	" and the background is light enough.<br />"
	"Click on it to confirm the update of the picture.";
#elif L==4	// es
	"Se ha detectado un rostro (se&ntilde;alado en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Pulse sobre &eacute;l para confirmar la actualizaci&oacute;n de la fotograf&iacute;a.";
#elif L==5	// fr
	"Un visage (marqu&eacute; en vert) a &eacute;t&eacute; d&eacute;tect&eacute; en position avant"
	" et l'arri&egrave;re-plan est assez clair.<br />"
	 "Cliquez sur pour confirmer la mise &agrave; jour de l'image.";
#elif L==6	// gn
	"Se ha detectado un rostro (se&ntilde;alado en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Pulse sobre &eacute;l para confirmar la actualizaci&oacute;n de la fotograf&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"Un viso (segnato in verde) &egrave; stato individuato nella posizione centrale"
	" e lo sfondo &egrave; abbastanza chiaro.<br />"
	"Clicca su di esso per confermare l'aggiornamento dell'immagine.";
#elif L==8	// pl
	"Twarz (zaznaczone na zielono) zosta&lstrok; wykryty w przednim po&lstrok;o&zdot;eniu,"
	" a t&lstrok;o jest wystarczaj&aogon;co du&zdot;o &sacute;wiat&lstrok;a.<br />"
	"Kliknij go, aby potwierdzi&cacute; aktualizacj&eogon; obrazu.";
#elif L==9	// pt
	"Um rosto (marcado em verde) foi detectado em posi&ccedil;&atilde;o frontal"
	" eo fundo &eacute; claro o suficiente.<br />"
	"Clique nele para confirmar a atualiza&ccedil;&atilde;o da imagem.";
#endif

const char *Txt_A_face_marked_in_red_has_been_detected_ =
#if   L==1	// ca
	"S'ha detectat un rostre (senyalat en vermell) en posici&oacute; frontal,"
	" per&ograve; el fons &eacute;s massa fosc.";
#elif L==2	// de
	"Ein Gesicht (rot markiert) wurde vor Position erkannt,"
	" aber der Hintergrund ist zu dunkel.";
#elif L==3	// en
	"A face (marked in red) has been detected in front position,"
	" but the background is too dark.";
#elif L==4	// es
	"Se ha detectado un rostro (se&ntilde;alado en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";
#elif L==5	// fr
	"Un visage (marqu&eacute; en rouge) a &eacute;t&eacute; d&eacute;tect&eacute; en position avant,"
	" mais le fond est trop sombre.";
#elif L==6	// gn
	"Se ha detectado un rostro (se&ntilde;alado en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";	// Okoteve traduccin
#elif L==7	// it
	"Un viso (segnato in rosso) &egrave; stato individuato nella posizione centrale,"
	" ma lo sfondo &egrave; troppo scuro.";
#elif L==8	// pl
	"Twarz (zaznaczone na czerwono) zosta&lstrok; wykryty w przednim po&lstrok;o&zdot;eniu,"
	" ale t&lstrok;o jest zbyt ciemne.";
#elif L==9	// pt
	"Um rosto (marcado em vermelho) foi detectado em posi&ccedil;&atilde;o frontal,"
	" mas o plano de fundo &eacute; demasiado escuro.";
#endif

const char *Txt_A_message_has_been_sent_to_email_address_X_to_confirm_that_address =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo <strong>%s</strong>"
	" para confirmar dicha direcci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";			// Need bersetzung
#elif L==3	// en
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";
#elif L==4	// es
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo <strong>%s</strong>"
	" para confirmar dicha direcci&oacute;n.";
#elif L==5	// fr
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";			// Besoin de traduction
#elif L==6	// gn
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo <strong>%s</strong>"
	" para confirmar dicha direcci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";			// Bisogno di traduzione
#elif L==8	// pl
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Uma mensagem foi enviada"
	" para o endere&ccedil;o de email <strong>%s</strong>"
	" para confirmar esse endere&ccedil;o.";
#endif

const char *Txt_A_student_can_belong_to_several_groups =
#if   L==1	// ca
	"Un estudiant pot pert&agrave;nyer a diversos grups";
#elif L==2	// de
	"Ein Student kann mehreren Gruppen angehren";
#elif L==3	// en
	"A student can belong to several groups";
#elif L==4	// es
	"Un estudiante puede pertenecer a varios grupos";
#elif L==5	// fr
	"Un &eacute;tudiant peut appartenir &agrave; plusieurs groupes";
#elif L==6	// gn
	"Un estudiante puede pertenecer a varios grupos";	// Okoteve traduccin
#elif L==7	// it
	"Uno studente pu&ograve; appartenere a pi&ugrave; gruppi";
#elif L==8	// pl
	"Kazdy student moze nalezec do wielu grup";
#elif L==9	// pt
	"Um estudante pode pertencer a v&aacute;rios grupos";
#endif

const char *Txt_A_student_can_only_belong_to_one_group =
#if   L==1	// ca
	"Un estudiant nom&eacute;s pot pert&agrave;nyer a un grup";
#elif L==2	// de
	"Ein Student kann nur einer Gruppe angeh&ouml;ren";
#elif L==3	// en
	"A student can only belong to one group";
#elif L==4	// es
	"Un estudiante solo puede pertenecer a un grupo";
#elif L==5	// fr
	"Un &eacute;tudiant ne peut appartenir &agrave; un groupe";
#elif L==6	// gn
	"Un estudiante solo puede pertenecer a un grupo";	// Okoteve traduccin
#elif L==7	// it
	"Uno studente pu&ograve; appartenere solo a un gruppo";
#elif L==8	// pl
	"student moze nalezec tylko do jednej grupy";
#elif L==9	// pt
	"Um estudante pode pertencer a um grupo";
#endif

const char *Txt_A_teacher_or_administrator_has_enroled_you_as_X_into_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Un professor/a o un administrador/a l'ha inscrit"
	" com a <strong>%s</strong>"
	" en l'assignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;"
	" o prefereix ser eliminat de l'assignatura?";
#elif L==2	// de
	"Ein Lehrer oder ein Administrator hat Sie"
	" als <strong>%s</strong>"
	" in den Kurs <strong>%s</strong> eingeschrieben."
	" M&ouml;chten Sie die Anmeldung best&auml;tigen"
	" oder ziehen Sie es vor,"
	" aus dem Kurs entfernt zu werden?";
#elif L==3	// en
	"A teacher or an administrator has enroled you"
	" as a <strong>%s</strong>"
	" into the course <strong>%s</strong>."
	" Do you want to confirm the enrolment"
	" or do you prefer to be removed from the course?";
#elif L==4	// es
	"Un profesor/a o un administrador/a le ha inscrito"
	" como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;n"
	" o prefiere ser eliminado/a de la asignatura?";
#elif L==5	// fr
	"Un professeur ou un administrateur vous a enregistr&eacute;"
	" en tant que <strong>%s</strong>"
	" dans la mati&egrave;re <strong>%s</strong>."
	" Voulez-vous confirmer l'inscription"
	" ou pr&eacute;f&eacute;rez-vous &ecirc;tre retir&eacute; de la mati&egrave;re?";
#elif L==6	// gn
	"Un profesor/a o un administrador/a le ha inscrito"
	" como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;n"
	" o prefiere ser eliminado/a de la asignatura?";		// Okoteve traduccin
#elif L==7	// it
	"Un insegnante o amministratore ti a registrato"
	" come <strong>%s</strong>"
	" nel corso <strong>%s</strong>."
	" Vuoi confermare la registrazione"
	" o preferisci essere rimosso da questo corso?";
#elif L==8	// pl
	"Nauczyciel lub administrator zapisa&lstrok; ci&eogon;"
	" jako <strong>%s</strong>"
	" do kursu <strong>%s</strong>."
	" Czy chcesz, aby potwierdzi&cacute; rejestracj&eogon;"
	" czy wolisz zosta&cacute; usuni&eogon;te z kursu?";
#elif L==9	// pt
	"Um professor/a ou administrador/a tem registrado voc&ecirc;"
	" como <strong>%s</strong>"
	" na disciplina <strong>%s</strong>."
	" Voc&ecirc; quer para confirmar a inscri&ccedil;&atilde;o"
	" ou voc&ecirc; prefere para ser removido/a da disciplina?";
#endif

const char *Txt_A_user_can_not_have_more_than_X_IDs =
#if   L==1	// ca
	"Un usuari no pot tenir m&eacute;s de %u IDs (DNI/c&egrave;dulas)";
#elif L==2	// de
	"Ein Benutzer kann nicht mehr als %u Ausweis-Nr.";
#elif L==3	// en
	"A user can not have more than %u IDs.";
#elif L==4	// es
	"Un usuario no puede tener m&aacute;s de %u ID (DNI/c&eacute;dulas).";
#elif L==5	// fr
	"Un utilisateur ne peut pas avoir plus de %u IDs.";
#elif L==6	// gn
	"Un usuario no puede tener m&aacute;s de %u ID (DNI/c&eacute;dulas).";	// Okoteve traduccin
#elif L==7	// it
	"Un utente non pu&ograve; avere pi&ugrave; di %u ID.";
#elif L==8	// pl
	"U&zdot;ytkownik nie mo&zdot;e mie&cacute; wi&eogon;cej ni&zdot; %u identyfikatory.";
#elif L==9	// pt
	"Um usu&aacute;rio n&atilde;o pode ter mais de %u n&ordm;s. identif.";
#endif

const char *Txt_About_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Quant a %s";
#elif L==2	// de
	"&Uuml;ber uns %s";
#elif L==3	// en
	"About %s";
#elif L==4	// es
	"Acerca de %s";
#elif L==5	// fr
	"&Agrave; propos de %s";
#elif L==6	// gn
	"%s rehegua";
#elif L==7	// it
	"Riguardo %s";
#elif L==8	// pl
	"O %s";
#elif L==9	// pt
	"Sobre %s";
#endif

const char *Txt_Absent =
#if   L==1	// ca
	"Absent";
#elif L==2	// de
	"Nicht vorhanden";
#elif L==3	// en
	"Absent";
#elif L==4	// es
	"Ausente";
#elif L==5	// fr
	"Absent";
#elif L==6	// gn
	"Ausente";	// Okoteve traduccin
#elif L==7	// it
	"Assente";
#elif L==8	// pl
	"Nieobecny";
#elif L==9	// pt
	"Assente";
#endif

const char *Txt_Absents =
#if   L==1	// ca
	"Absents";
#elif L==2	// de
	"Nicht vorhanden";
#elif L==3	// en
	"Absents";
#elif L==4	// es
	"Ausentes";
#elif L==5	// fr
	"Absents";
#elif L==6	// gn
	"Ausentes";	// Okoteve traduccin
#elif L==7	// it
	"Assenti";
#elif L==8	// pl
	"Nieobecny";
#elif L==9	// pt
	"Assentes";
#endif

const char *Txt_Accept_third_party_cookies_to_view_multimedia_content_from_other_websites =
#if   L==1	// ca
	"Accepta cookies de tercers"
	" per veure contingut multim&iacute;dia de outros sites";
#elif L==2	// de
	"Akzeptieren Sie Cookies von Drittanbietern,"
	" um Multimedia-Inhalte von anderen Websites anzuzeigen";
#elif L==3	// en
	"Accept third-party cookies"
	" to view multimedia content from other websites";
#elif L==4	// es
	"Aceptar cookies de terceros"
	" para ver contenido multimedia de otros sitios web";
#elif L==5	// fr
	"Accepter les cookies de tiers"
	" pour afficher le contenu multim&eacute;dia d'autres sites Web";
#elif L==6	// gn
	"Aceptar cookies de terceros"
	" para ver contenido multimedia de otros sitios web";	// Okoteve traduccin
#elif L==7	// it
	"Accetta i cookie di terze parti"
	" per visualizzare contenuti multimediali da altri siti Web";
#elif L==8	// pl
	"Akceptuj pliki cookie innych firm,"
	" aby wy&sacute;wietla&cacute; tre&sacute;ci multimedialne z innych stron internetowych";
#elif L==9	// pt
	"Aceite cookies de terceiros"
	" para visualizar conte&uacute;do multimdia de outros sites";
#endif

const char *Txt_accessible_for_reading_and_writing_by_administrators_of_the_centre =
#if   L==1	// ca
	"accessible per a lectura i escriptura per administradors del centre";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Administratoren der Lehrinstitut";
#elif L==3	// en
	"accessible for reading and writing by administrators of the centre";
#elif L==4	// es
	"accesible para lectura y escritura por administradores del centro";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les administrateurs du centre";
#elif L==6	// gn
	"accesible para lectura y escritura por administradores del centro";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte dei amministratori del centro";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez administratorzy centrum";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos administradores do centro";
#endif

const char *Txt_accessible_for_reading_and_writing_by_administrators_of_the_degree =
#if   L==1	// ca
	"accessible per a lectura i escriptura"
	" per administradors de la titulaci&oacute;";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich"
	" f&uuml;r Administratoren der Studiengang";
#elif L==3	// en
	"accessible for reading and writing"
	" by administrators of the degree";
#elif L==4	// es
	"accesible para lectura y escritura"
	" por administradores de la titulaci&oacute;n";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture"
	" par les administrateurs de l'&eacute;tude";
#elif L==6	// gn
	"accesible para lectura y escritura"
	" por administradores de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura"
	" da parte dei amministratori della laurea";
#elif L==8	// pl
	"dostep do odczytu i zapisu"
	" przez administratorzy stopnia";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita"
	" pelos administradores do grau";
#endif

const char *Txt_accessible_for_reading_and_writing_by_administrators_of_the_institution =
#if   L==1	// ca
	"accessible per a lectura i escriptura per administradors de l'instituci&oacute;";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Administratoren der Hochschule";
#elif L==3	// en
	"accessible for reading and writing by administrators of the institution";
#elif L==4	// es
	"accesible para lectura y escritura por administradores de la instituci&oacute;n";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les administrateurs du &eacute;tablissement";
#elif L==6	// gn
	"accesible para lectura y escritura por administradores de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte dei amministratori della istituzione";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez administratorzy instytucji";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos administradores da institui&ccedil;&atilde;o";
#endif

const char *Txt_accessible_for_reading_and_writing_by_project_members =
#if   L==1	// ca
	"accessible per a lectura i escriptura per membres del projecte";
#elif L==2	// de
	"zug&auml;nglich zum Lesen und Schreiben von Projektmitgliedern";
#elif L==3	// en
	"accessible for reading and writing by project members";
#elif L==4	// es
	"accesible para lectura y escritura por miembros del proyecto";
#elif L==5	// fr
	"accessible pour la lecture et l'&eacute;criture par les membres du projet";
#elif L==6	// gn
	"accesible para lectura y escritura por miembros del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte dei membri del progetto";
#elif L==8	// pl
	"dost&eogon;pne do czytania i pisania przez cz&lstrok;onk&oacute;w projektu";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita por membros do projeto";
#endif

const char *Txt_accessible_for_reading_and_writing_by_project_tutors_and_evaluators =
#if   L==1	// ca
	"accessible per a lectura i escriptura per tutors i avaluadors del projecte";
#elif L==2	// de
	"zug&auml;nglich zum Lesen und Schreiben f&uuml;r Tutoren und Auswerter der Projekt";
#elif L==3	// en
	"accessible for reading and writing by project tutors and evaluators";
#elif L==4	// es
	"accesible para lectura y escritura por tutores y evaluadores del proyecto";
#elif L==5	// fr
	"accessible pour la lecture et l'&eacute;criture par tuteurs et &eacute;valuateurs du projet";
#elif L==6	// gn
	"accesible para lectura y escritura por tutores y evaluadores del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte dei precettori e dei valutatori del progetto";
#elif L==8	// pl
	"dost&eogon;pne do czytania i pisania przez nauczyciele i ewaluator&oacute;w projektu";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita por tutores e avaliadores do projeto";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_centre =
#if   L==1	// ca
	"accessible per a lectura i escriptura per estudiants i professors del centre";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Lehrinstitut";
#elif L==3	// en
	"accessible for reading and writing by students and teachers of the centre";
#elif L==4	// es
	"accesible para lectura y escritura por estudiantes y profesores del centro";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants du centre";
#elif L==6	// gn
	"accesible para lectura y escritura por estudiantes y profesores del centro";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori del centro";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli centrum";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos estudantes e professores do centro";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_course =
#if   L==1	// ca
	"accessible per a lectura i escriptura per estudiants i professors de l'assignatura";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible for reading and writing by students and teachers of the course";
#elif L==4	// es
	"accesible para lectura y escritura por estudiantes y profesores de la asignatura";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible para lectura y escritura por estudiantes y profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori del corso";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli zajec";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos estudantes e professores da disciplina";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_degree =
#if   L==1	// ca
	"accessible per a lectura i escriptura"
	" per estudiants i professors de la titulaci&oacute;";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich"
	" f&uuml;r Studenten und Lehrkr&auml;fte der Studiengang";
#elif L==3	// en
	"accessible for reading and writing"
	" by students and teachers of the degree";
#elif L==4	// es
	"accesible para lectura y escritura"
	" por estudiantes y profesores de la titulaci&oacute;n";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture"
	" par les &eacute;tudiants et les enseignants de l'&eacute;tude";
#elif L==6	// gn
	"accesible para lectura y escritura"
	" por estudiantes y profesores de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura"
	" da parte degli studenti e dei professori della laurea";
#elif L==8	// pl
	"dostep do odczytu i zapisu"
	" przez uczni&oacute;w i nauczycieli stopnia";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita"
	" pelos estudantes e professores do grau";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_institution =
#if   L==1	// ca
	"accessible per a lectura i escriptura per estudiants i professors de la instituci&oacute;";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Hochschule";
#elif L==3	// en
	"accessible for reading and writing by students and teachers of the institution";
#elif L==4	// es
	"accesible para lectura y escritura por estudiantes y profesores de la instituci&oacute;n";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants du &eacute;tablissement";
#elif L==6	// gn
	"accesible para lectura y escritura por estudiantes y profesores de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori della istituzione";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli instytucji";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos estudantes e professores da institui&ccedil;&atilde;o";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_of_the_group_and_teachers_of_the_course =
#if   L==1	// ca
	"accesible per a lectura i escriptura per estudiants del grup i professors de l'assignatura";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten der Gruppe und Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible for reading and writing by students of the group and teachers of the course";
#elif L==4	// es
	"accesible para lectura y escritura por estudiantes del grupo y profesores de la asignatura";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants du groupe et les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible para lectura y escritura por estudiantes del grupo y profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte degli studenti del gruppo e dei professori del corso";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez student&oacute;w i nauczycieli, grupy kursu";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos estudantes do grupo e professores da disciplina";
#endif

const char *Txt_accessible_for_reading_and_writing_by_teachers_of_the_course =
#if   L==1	// ca
	"accessible per a lectura i escriptura per professors de l'assignatura";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible for reading and writing by teachers of the course";
#elif L==4	// es
	"accesible para lectura y escritura por profesores de la asignatura";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible para lectura y escritura por profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura da parte dei professori del corso";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez nauczycieli zajec";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita pelos professores da disciplina";
#endif

const char *Txt_accessible_for_reading_and_writing_by_you_and_the_teachers_of_the_course =
#if   L==1	// ca
	"accessible per a lectura per vost&egrave; i els professors de l'assignatura";
#elif L==2	// de
	"zum Schreiben zug&auml;nglich f&uuml;r Sie und die Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible for reading and writing by you and the teachers of the course";
#elif L==4	// es
	"accesible para lectura por usted y los profesores de la asignatura";
#elif L==5	// fr
	"accessible en lecture et en &eacute;criture par vous et par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible para lectura por usted y los profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile per la lettura e la scrittura per te e i professori del corso";
#elif L==8	// pl
	"dostep do odczytu i zapisu przez Ciebie i nauczycieli zajec";
#elif L==9	// pt
	"acess&iacute;vel para leitura e escrita por voc&ecirc; e os professores da disciplina";
#endif

const char *Txt_accessible_only_for_reading_by_you_and_the_teachers_of_the_course =
#if   L==1	// ca
	"accesible nom&egrave; per a lectura per vost&eacute; i els professors de l'assignatura";
#elif L==2	// de
	"zum Lesen zug&auml;nglich f&uuml;r Sie und die Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible only for reading by you and the teachers of the course";
#elif L==4	// es
	"accesible solo para lectura por usted y los profesores de la asignatura";
#elif L==5	// fr
	"accessible uniquement pour la lecture par vous et par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible solo para lectura por usted y los profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura per te e i professori del corso";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania przez Ciebie i nauczycieli prowadzacych ten kurs";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura por voc&ecirc; e os professores da disciplina";
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_centre =
#if   L==1	// ca
	"accessible nom&eacute;s per a lectura per estudiants i professors del centre";
#elif L==2	// de
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Lehrinstitut";
#elif L==3	// en
	"accessible only for reading by students and teachers of the centre";
#elif L==4	// es
	"accesible solo para lectura por estudiantes y profesores del centro";
#elif L==5	// fr
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants du centre";
#elif L==6	// gn
	"accesible solo para lectura por estudiantes y profesores del centro";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura da parte degli studenti e dei professori del centro";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli centrum";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura pelos estudantes e professores do centro";
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_course =
#if   L==1	// ca
	"accessible nom&eacute;s per a lectura per estudiants i professors de l'assignatura";
#elif L==2	// de
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible only for reading by students and teachers of the course";
#elif L==4	// es
	"accesible solo para lectura por estudiantes y profesores de la asignatura";
#elif L==5	// fr
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible solo para lectura por estudiantes y profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura da parte degli studenti e dei professori del corso";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli zajec";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura pelos estudantes e professores da disciplina";
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_degree =
#if   L==1	// ca
	"accessible nom&eacute;s per a lectura"
	" per estudiants i professors de la titulaci&oacute;";
#elif L==2	// de
	"zum Lesen zug&auml;nglich"
	" f&uuml;r Studenten und Lehrkr&auml;fte der Studiengang";
#elif L==3	// en
	"accessible only for reading"
	" by students and teachers of the degree";
#elif L==4	// es
	"accesible solo para lectura"
	" por estudiantes y profesores de la titulaci&oacute;n";
#elif L==5	// fr
	"accessible uniquement pour la lecture"
	" par les &eacute;tudiants et les enseignants de l'&eacute;tude";
#elif L==6	// gn
	"accesible solo para lectura"
	" por estudiantes y profesores de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura"
	" da parte degli studenti e dei professori della laurea";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania"
	" przez uczni&oacute;w i nauczycieli stopnia";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura"
	" pelos estudantes e professores do grau";
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_institution =
#if   L==1	// ca
	"accessible nom&eacute;s per a lectura per estudiants i professors de l'instituci&oacute;";
#elif L==2	// de
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Hochschule";
#elif L==3	// en
	"accessible only for reading by students and teachers of the institution";
#elif L==4	// es
	"accesible solo para lectura por estudiantes y profesores de la instituci&oacute;n";
#elif L==5	// fr
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants du &eacute;tablissement";
#elif L==6	// gn
	"accesible solo para lectura por estudiantes y profesores de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura da parte degli studenti e dei professori della istituzione";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli instytucji";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura pelos estudantes e professores da institui&ccedil;&atilde;o";
#endif

const char *Txt_accessible_only_for_reading_by_students_of_the_group_and_teachers_of_the_course =
#if   L==1	// ca
	"accesible nom&eacute;s per a lectura per estudiants del grup i professors de l'assignatura";
#elif L==2	// de
	"zum Lesen zug&auml;nglich f&uuml;r Studenten der Gruppe und Lehrkr&auml;fte der Kurs";
#elif L==3	// en
	"accessible only for reading by students of the group and teachers of the course";
#elif L==4	// es
	"accesible solo para lectura por estudiantes del grupo y profesores de la asignatura";
#elif L==5	// fr
	"accessible uniquement pour la lecture par les &eacute;tudiants du groupe et les enseignants de la mati&egrave;re";
#elif L==6	// gn
	"accesible solo para lectura por estudiantes del grupo y profesores de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"accessibile solo per la lettura da parte degli studenti del gruppo e dei professori del corso";
#elif L==8	// pl
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli grupy kursu";
#elif L==9	// pt
	"acess&iacute;vel apenas para leitura pelos estudantes do grupo e professores da disciplina";
#endif

const char *Txt_Action =
#if   L==1	// ca
	"Acci&oacute;";
#elif L==2	// de
	"Handlung";
#elif L==3	// en
	"Action";
#elif L==4	// es
	"Acci&oacute;n";
#elif L==5	// fr
	"Action";
#elif L==6	// gn
	"Tembiapo";
#elif L==7	// it
	"Azione";
#elif L==8	// pl
	"Dzia&lstrok;anie";
#elif L==9	// pt
	"A&ccedil;&atilde;o";
#endif

const char *Txt_Administer_me =
#if   L==1	// ca
	"Administrarme";
#elif L==2	// de
	"Verwalten Sie mich";
#elif L==3	// en
	"Administer me";
#elif L==4	// es
	"Administrarme";
#elif L==5	// fr
	"G&eacute;rer moi";
#elif L==6	// gn
	"Administrarme";	// Okoteve traduccin
#elif L==7	// it
	"Gestire mi";
#elif L==8	// pl
	"Zarz&aogon;dzaj mnie";
#elif L==9	// pt
	"Gerenciar me";
#endif

const char *Txt_Administer_multiple_students =
#if   L==1	// ca
	"Administrar diversos estudiants";
#elif L==2	// de
	"Verwalten Sie mehrere Studenten";
#elif L==3	// en
	"Administer multiple students";
#elif L==4	// es
	"Administrar varios estudiantes";
#elif L==5	// fr
	"G&eacute;rer plusieurs &eacute;tudiants";
#elif L==6	// gn
	"Administrar varios estudiantes";	// Okoteve traduccin
#elif L==7	// it
	"Gestire pi&ugrave; studenti";
#elif L==8	// pl
	"Zarz&aogon;dzaj wielu studenci";
#elif L==9	// pt
	"Gerenciar v&aacute;rios estudantes";
#endif

const char *Txt_Administer_multiple_non_editing_teachers =
#if   L==1	// ca
	"Administrar diversos professors no editors";
#elif L==2	// de
	"Verwalten Sie mehrere nicht bearbeiteter Lehrkr&auml;fte";
#elif L==3	// en
	"Administer multiple non-editing teachers";
#elif L==4	// es
	"Administrar varios profesores no editores";
#elif L==5	// fr
	"G&eacute;rer plusieurs enseignants non-&eacute;diteurs";
#elif L==6	// gn
	"Administrar varios profesores no editores";	// Okoteve traduccin
#elif L==7	// it
	"Gestire pi&ugrave; professori non-editing";
#elif L==8	// pl
	"Zarz&aogon;dzaj wielu nauczycieli nie edytuj&aogon;cy";
#elif L==9	// pt
	"Gerenciar v&aacute;rios professores n&atilde;o editores";
#endif

const char *Txt_Administer_multiple_teachers =
#if   L==1	// ca
	"Administrar diversos professors";
#elif L==2	// de
	"Verwalten Sie mehrere Lehrkr&auml;fte";
#elif L==3	// en
	"Administer multiple teachers";
#elif L==4	// es
	"Administrar varios profesores";
#elif L==5	// fr
	"G&eacute;rer plusieurs enseignants";
#elif L==6	// gn
	"Administrar varios profesores";	// Okoteve traduccin
#elif L==7	// it
	"Gestire pi&ugrave; professori";
#elif L==8	// pl
	"Zarz&aogon;dzaj wielu nauczycieli";
#elif L==9	// pt
	"Gerenciar v&aacute;rios professores";
#endif

const char *Txt_Administer_one_user =
#if   L==1	// ca
	"Administrar un usuari";
#elif L==2	// de
	"Verwalten eines Benutzer";
#elif L==3	// en
	"Administer one user";
#elif L==4	// es
	"Administrar un usuario";
#elif L==5	// fr
	"G&eacute;rer un utilisateur";
#elif L==6	// gn
	"Administrar un usuario";	// Okoteve traduccin
#elif L==7	// it
	"Gestire un utente";
#elif L==8	// pl
	"Zarz&aogon;dzaj u&zdot;ytkownika";
#elif L==9	// pt
	"Gerenciar um utilizador";
#endif

const char *Txt_Administer_user =
#if   L==1	// ca
	"Administrar usuari";
#elif L==2	// de
	"Verwalten Benutzer";
#elif L==3	// en
	"Administer user";
#elif L==4	// es
	"Administrar usuario";
#elif L==5	// fr
	"G&eacute;rer utilisateur";
#elif L==6	// gn
	"Administrar usuario";	// Okoteve traduccin
#elif L==7	// it
	"Gestire utente";
#elif L==8	// pl
	"Zarz&aogon;dzaj u&zdot;ytkownika";
#elif L==9	// pt
	"Gerenciar utilizador";
#endif

const char *Txt_AGENDA_PAST___FUTURE_EVENTS[2] =
	{
	// Agd_PAST___EVENTS
#if   L==1	// ca
	"Esdeveniments passats i d'avui"
#elif L==2	// de
	"Vergangenheit und Gegenwart Ereignisse"
#elif L==3	// en
	"Past and present events"
#elif L==4	// es
	"Eventos pasados y de hoy"
#elif L==5	// fr
	"&Eacute;v&eacute;nements pass&eacute;s et pr&eacute;sents"
#elif L==6	// gn
	"Eventos pasados y de hoy"	// Okoteve traduccin
#elif L==7	// it
	"Eventi passati e presenti"
#elif L==8	// pl
	"Wydarzenia minione i obecne"
#elif L==9	// pt
	"Eventos passados e presentes"
#endif
	,
	// Agd_FUTURE_EVENTS
#if   L==1	// ca
	"Esdeveniments d'avui i futurs"
#elif L==2	// de
	"Aktuelle und zuk&uuml;nftige Ereignisse"
#elif L==3	// en
	"Current and future events"
#elif L==4	// es
	"Eventos de hoy y futuros"
#elif L==5	// fr
	"&Eacute;v&eacute;nements actuels et futurs"
#elif L==6	// gn
	"Eventos de hoy y futuros"	// Okoteve traduccin
#elif L==7	// it
	"Eventi attuali e futuri"
#elif L==8	// pl
	"Obecne i przysz&lstrok;e wydarzenia"
#elif L==9	// pt
	"Eventos atuais e futuros"
#endif
	};

const char *Txt_AGENDA_PRIVAT_PUBLIC_EVENTS[2] =
	{
	// Agd_PRIVAT_EVENTS
#if   L==1	// ca
	"Esdeveniments privados"
#elif L==2	// de
	"Private Ereignisse"
#elif L==3	// en
	"Private events"
#elif L==4	// es
	"Eventos privados"
#elif L==5	// fr
	"&Eacute;v&eacute;nements priv&eacute;s"
#elif L==6	// gn
	"Eventos privados"	// Okoteve traduccin
#elif L==7	// it
	"Eventi privati"
#elif L==8	// pl
	"Prywatne wydarzenia"
#elif L==9	// pt
	"Eventos privados"
#endif
	,
	// Agd_PUBLIC_EVENTS
#if   L==1	// ca
	"Esdeveniments p&uacute;blics"
#elif L==2	// de
	"&Ouml;ffentliche Ereignisse"
#elif L==3	// en
	"Public events"
#elif L==4	// es
	"Eventos p&uacute;blicos"
#elif L==5	// fr
	"&Eacute;v&eacute;nements publics"
#elif L==6	// gn
	"Eventos p&uacute;blicos"	// Okoteve traduccin
#elif L==7	// it
	"Eventi pubblici"
#elif L==8	// pl
	"Publiczne wydarzenia"
#elif L==9	// pt
	"Eventos p&uacute;blicos"
#endif
	};

const char *Txt_AGENDA_HIDDEN_VISIBL_EVENTS[2] =
	{
	// Agd_HIDDEN_EVENTS
#if   L==1	// ca
	"Esdeveniments ocults"
#elif L==2	// de
	"Unsichtbare Ereignisse"
#elif L==3	// en
	"Hidden events"
#elif L==4	// es
	"Eventos ocultos"
#elif L==5	// fr
	"&Eacute;v&eacute;nements cach&eacute;s"
#elif L==6	// gn
	"Eventos ocultos"	// Okoteve traduccin
#elif L==7	// it
	"Eventi nascosti"
#elif L==8	// pl
	"Ukryte wydarzenia"
#elif L==9	// pt
	"Eventos ocultos"
#endif
	,
	// Agd_VISIBL_EVENTS
#if   L==1	// ca
	"Esdeveniments visibles"
#elif L==2	// de
	"Sichtbar Ereignisse"
#elif L==3	// en
	"Visible events"
#elif L==4	// es
	"Eventos visibles"
#elif L==5	// fr
	"&Eacute;v&eacute;nements visibles"
#elif L==6	// gn
	"Eventos visibles"	// Okoteve traduccin
#elif L==7	// it
	"Eventi visibili"
#elif L==8	// pl
	"Widoczne wydarzenia"
#elif L==9	// pt
	"Eventos vis&iacute;veis"
#endif
	};

const char *Txt_all =
#if   L==1	// ca
	"tot";
#elif L==2	// de
	"alle";
#elif L==3	// en
	"all";
#elif L==4	// es
	"todo";
#elif L==5	// fr
	"tout";
#elif L==6	// gn
	"opakatu";
#elif L==7	// it
	"tutto";
#elif L==8	// pl
	"wszystko";
#elif L==9	// pt
	"tudo";
#endif

const char *Txt_all_degrees =
#if   L==1	// ca
	"Totes les titulacions";
#elif L==2	// de
	"all degrees";			// Need bersetzung
#elif L==3	// en
	"all degrees";
#elif L==4	// es
	"todas las titulaciones";
#elif L==5	// fr
	"tous les degr&eacute;s";
#elif L==6	// gn
	"todas las titulaciones";	// Okoteve traduccin
#elif L==7	// it
	"tutte le lauree";
#elif L==8	// pl
	"wszystkich stopni";
#elif L==9	// pt
	"todos os graus";
#endif

const char *Txt_all_files_inside_the_root_folder =
#if   L==1	// ca
	"tots els arxius dins de la carpeta arrel";
#elif L==2	// de
	"alle Dateien im Stammordner";
#elif L==3	// en
	"all files inside the root folder";
#elif L==4	// es
	"todos los archivos dentro de la carpeta ra&iacute;z";
#elif L==5	// fr
	"tous les fichiers dans le r&eacute;pertoire racine";
#elif L==6	// gn
	"todos los archivos dentro de la carpeta ra&iacute;z";	// Okoteve traduccin
#elif L==7	// it
	"tutti i file all'interno della cartella principale";
#elif L==8	// pl
	"wszystkie pliki znajduj&aogon;ce si&eogon; w folderze g&lstrok;&oacute;wnym";
#elif L==9	// pt
	"todos os arquivos dentro do diret&oacute;rio raiz";
#endif

const char *Txt_All_groups =
#if   L==1	// ca
	"Tots els grups";
#elif L==2	// de
	"Alle Gruppen";
#elif L==3	// en
	"All groups";
#elif L==4	// es
	"Todos los grupos";
#elif L==5	// fr
	"Tous les groupes";
#elif L==6	// gn
	"Todos los grupos";	// Okoteve traduccin
#elif L==7	// it
	"Tutti i gruppi";
#elif L==8	// pl
	"Wszystkie grupy";
#elif L==9	// pt
	"Todos os grupos";
#endif

const char *Txt_All_places =
#if   L==1	// ca
	"Tots els llocs";
#elif L==2	// de
	"Alle Standorte";
#elif L==3	// en
	"All places";
#elif L==4	// es
	"Todos los lugares";
#elif L==5	// fr
	"Tous les emplacements";
#elif L==6	// gn
	"Todos los lugares";	// Okoteve traduccin
#elif L==7	// it
	"Tutte le localit&agrave;";
#elif L==8	// pl
	"Wszystkie miejsca";
#elif L==9	// pt
	"Todas as localiza&ccedil;oes";
#endif

const char *Txt_All_tags =		// Tags used in test
#if   L==1	// ca
	"Tots els descriptors";
#elif L==2	// de
	"Alle Tags";
#elif L==3	// en
	"All tags";
#elif L==4	// es
	"Todos los descriptores";
#elif L==5	// fr
	"Tous les descripteurs";
#elif L==6	// gn
	"Todos los descriptores";	// Okoteve traduccin
#elif L==7	// it
	"Tutte le etichette";
#elif L==8	// pl
	"Wszystkie tagi";
#elif L==9	// pt
	"Todos os descritores";
#endif

const char *Txt_All_types_of_answers =		// Answers of a test
#if   L==1	// ca
	"Tots els tipus de respostes";
#elif L==2	// de
	"Alle Antworttypen";
#elif L==3	// en
	"All types of answers";
#elif L==4	// es
	"Todos los tipos de respuestas";
#elif L==5	// fr
	"Tous les types de r&eacute;ponses";
#elif L==6	// gn
	"Todos los tipos de respuestas";	// Okoteve traduccin
#elif L==7	// it
	"Tutti i tipi di risposte";
#elif L==8	// pl
	"Wszystkie typy odpowiedzi";
#elif L==9	// pt
	"Todos os tipos de respostas";
#endif

const char *Txt_Allow_teachers_to_consult_this_test =
#if   L==1	// ca
	"Permetre als professors consultar aquest test";
#elif L==2	// de
	"Erm&ouml;glichen es den Lehrern, diese Test zu konsultieren";
#elif L==3	// en
	"Allow teachers to consult this test";
#elif L==4	// es
	"Permitir a los profesores consultar este test";
#elif L==5	// fr
	"Permettre aux enseignants de consulter ce test";
#elif L==6	// gn
	"Permitir a los profesores consultar este test";	// Okoteve traduccin
#elif L==7	// it
	"Lasciare gli insegnanti di consultare questo test";
#elif L==8	// pl
	"Pozwalaj&aogon; nauczycielom skonsultowa&cacute; ten test";
#elif L==9	// pt
	"Permitir que os professores de consultar este teste";
#endif

const char *Txt_Alphabetic_BR_code_BR_ISO_3166_1 =
#if   L==1	// ca
	"Codi<br />alfab&egrave;tic<br />ISO 3166-1";
#elif L==2	// de
	"Alphabetischer<br />Code<br />ISO 3166-1";
#elif L==3	// en
	"Alphabetic<br />code<br />ISO 3166-1";
#elif L==4	// es
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";
#elif L==5	// fr
	"Code<br />alphab&eacute;tique<br />ISO 3166-1";
#elif L==6	// gn
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";	// Okoteve traduccin
#elif L==7	// it
	"Codice<br />alfabetico<br />ISO 3166-1";
#elif L==8	// pl
	"Alfabetycznego<br />kodu<br />ISO 3166-1";
#elif L==9	// pt
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";
#endif

const char *Txt_Already_existed_a_game_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia un joc amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Spiel mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"Already existed a game with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a un juego con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un jeu du titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a un juego con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un gioco con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"By&lstrok;o ju&zdot; gra z tytu&lstrok;em <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe um jogo com o t&iacute;tulo <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_a_project_with_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia un projecte amb la carpeta <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Projekt im Verzeichnis <strong>%s</strong>.";
#elif L==3	// en
	"There is already a project with folder <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a un proyecto con la carpeta <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un projet avec le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a un proyecto con la carpeta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un progetto con la cartella <strong>%s</strong>.";
#elif L==8	// pl
	"Istnia&lstrok; projekt z folderem <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe um projeto com o diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_a_survey_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia una enquesta amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Umfrage mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"Already existed a survey with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a una encuesta con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un sondage du titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a una encuesta con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un sondaggio con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"By&lstrok;o ju&zdot; ankieta z tytu&lstrok;em <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe um inqu&eacute;rito com o t&iacute;tulo <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_an_assignment_with_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia una activitat amb la carpeta <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Aufgabe im Verzeichnis <strong>%s</strong>.";
#elif L==3	// en
	"There is already an assignment with folder <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a una actividad con la carpeta <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; une activit&eacute; avec le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a una actividad con la carpeta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un'attivit&agrave; con la cartella <strong>%s</strong>.";
#elif L==8	// pl
	"Jest juz zadanie z folderu <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe uma atividade com o diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_an_assignment_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia una activitat amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Aufgabe mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"Already existed an assignment with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a una actividad con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; une activit&eacute; du titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a una actividad con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un'attivit&agrave; con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"Istniala juz przydzialu z tytulu <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe uma atividade com o t&iacute;tulo <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_an_event_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ja existia un esdeveniment amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt bereits einen Ereignis mit dem Namen <strong>%s</strong>.";
#elif L==3	// en
	"Already existed an event with the title <strong>%s</strong>.";
#elif L==4	// es
	"Ya exist&iacute;a un evento con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5	// fr
	"Il existe d&eacute;j&agrave; un &eacute;v&eacute;nement du titre <strong>%s</strong>.";
#elif L==6	// gn
	"Ya exist&iacute;a un evento con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Esiste gi&agrave; un evento con il titolo <strong>%s</strong>.";
#elif L==8	// pl
	"Istniala juz wydarzenie z tytulu <strong>%s</strong>.";
#elif L==9	// pt
	"J&aacute; existe um evento com o t&iacute;tulo <strong>%s</strong>.";
#endif

const char *Txt_and =
#if   L==1	// ca
	"i";
#elif L==2	// de
	"und";
#elif L==3	// en
	"and";
#elif L==4	// es
	"y";
#elif L==5	// fr
	"et";
#elif L==6	// gn
	"y";	// Okoteve traduccin
#elif L==7	// it
	"e";
#elif L==8	// pl
	"i";
#elif L==9	// pt
	"e";
#endif

const char *Txt_and_X_other_recipients =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"&hellip;i altres %u destinataris";
#elif L==2	// de
	"&hellip;und %u andere Empf&auml;nger";
#elif L==3	// en
	"&hellip;and %u other recipients";
#elif L==4	// es
	"&hellip;y otros %u destinatarios";
#elif L==5	// fr
	"&hellip;et %u autres destinataires";
#elif L==6	// gn
	"&hellip;y otros %u destinatarios";	// Okoteve traduccin
#elif L==7	// it
	"&hellip;e altri %u destinatari";
#elif L==8	// pl
	"&hellip;i %u innych odbiorc&oacute;w";
#elif L==9	// pt
	"&hellip;e %u outros destinat&aacute;rios";
#endif

const char *Txt_Announcements =
#if   L==1	// ca
	"Anuncis";
#elif L==2	// de
	"Bekanntmachungen";
#elif L==3	// en
	"Announcements";
#elif L==4	// es
	"Anuncios";
#elif L==5	// fr
	"Annonces";
#elif L==6	// gn
	"Anuncios";	// Okoteve traduccin
#elif L==7	// it
	"Annunci";
#elif L==8	// pl
	"Og&lstrok;oszenia";
#elif L==9	// pt
	"An&uacute;ncios";
#endif

const char *Txt_Announcement_of_exam_removed =
#if   L==1	// ca
	"Convocat&ograve;ria d'examen eliminada.";
#elif L==2	// de
	"Aufruf zur Pr&uuml;fung entfernt.";
#elif L==3	// en
	"Announcement of exam removed.";
#elif L==4	// es
	"Convocatoria de examen eliminada.";
#elif L==5	// fr
	"Convocation pour l'examen supprim&eacute;.";
#elif L==6	// gn
	"Convocatoria de examen eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Appello d'esame rimosso.";
#elif L==8	// pl
	"Ogloszenie egzamin usuniete.";
#elif L==9	// pt
	"Chamada para exame removida.";
#endif

const char *Txt_Announcements_of_exams =
#if   L==1	// ca
	 "Convocat&ograve;ries d'ex&agrave;mens";
#elif L==2	// de
	 "Aufrufe f&uuml;r Pr&uuml;fung";
#elif L==3	// en
	 "Announcements of exams";
#elif L==4	// es
	 "Convocatorias de ex&aacute;menes";
#elif L==5	// fr
	 "Convocations &agrave; examens";
#elif L==6	// gn
	 "Convocatorias de ex&aacute;menes";	// Okoteve traduccin
#elif L==7	// it
	 "Appelli d'esame";
#elif L==8	// pl
	 "Ogloszenia egzamin&oacute;w";
#elif L==9	// pt
	 "Chamadas para exames";
#endif

const char *Txt_Another_centre =
#if   L==1	// ca
	"Un altre centre";
#elif L==2	// de
	"Ein weiteres Lehrinstitut";
#elif L==3	// en
	"Another centre";
#elif L==4	// es
	"Otro centro";
#elif L==5	// fr
	"Un autre centre";
#elif L==6	// gn
	"Otro centro";	// Okoteve traduccin
#elif L==7	// it
	"Un altro centro";
#elif L==8	// pl
	"Innym centrum";
#elif L==9	// pt
	"Outro centro";
#endif

const char *Txt_Another_classroom =
#if   L==1	// ca
	"Una altra aula";
#elif L==2	// de
	"Eine weitere Klassenzimmer";
#elif L==3	// en
	"Another classroom";
#elif L==4	// es
	"Otra aula";
#elif L==5	// fr
	"Un autre salle de classe";
#elif L==6	// gn
	"Otra aula";	// Okoteve traduccin
#elif L==7	// it
	"Un'altra aula";
#elif L==8	// pl
	"Kolejna klasa";
#elif L==9	// pt
	"Outra sala de aula";
#endif

const char *Txt_Another_country =
#if   L==1	// ca
	"Un altre pa&iacute;s";
#elif L==2	// de
	"Ein weiterer Land";
#elif L==3	// en
	"Another country";
#elif L==4	// es
	"Otro pa&iacute;s";
#elif L==5	// fr
	"Un autre pays";
#elif L==6	// gn
	"Otro pa&iacute;s";	// Okoteve traduccin
#elif L==7	// it
	"Un altro paese";
#elif L==8	// pl
	"Kolejny kraj";
#elif L==9	// pt
	"Outro pa&iacute;s";
#endif

const char *Txt_Another_department =
#if   L==1	// ca
	"Un altre departament";
#elif L==2	// de
	"Eine weitere Abteilung";
#elif L==3	// en
	"Another department";
#elif L==4	// es
	"Otro departamento";
#elif L==5	// fr
	"Une autre d&eacute;partement";
#elif L==6	// gn
	"Otro departamento";	// Okoteve traduccin
#elif L==7	// it
	"Un altro dipartimento";
#elif L==8	// pl
	"Kolejny dzia&lstrok;";
#elif L==9	// pt
	"Outro departamento";
#endif

const char *Txt_Another_ID =
#if   L==1	// ca
	"Un altre ID";
#elif L==2	// de
	"Ein anderer Ausweis-Nr.";
#elif L==3	// en
	"Another ID";
#elif L==4	// es
	"Otro ID";
#elif L==5	// fr
	"Un autre num&eacute;ro d'identit&eacute;";
#elif L==6	// gn
	"Otro ID";		// Okoteve traduccin
#elif L==7	// it
	"Un'altra carta d'identit&agrave;";
#elif L==8	// pl
	"Another ID";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Outro n&ordm; de identif.";
#endif

const char *Txt_Another_institution =
#if   L==1	// ca
	"Una altra instituci&oacute;";
#elif L==2	// de
	"Eine weitere Hochschule";
#elif L==3	// en
	"Another institution";
#elif L==4	// es
	"Otra instituci&oacute;n";
#elif L==5	// fr
	"Un autre &eacute;tablissement";
#elif L==6	// gn
	"Otra instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Un'altra istituzione";
#elif L==8	// pl
	"Kolejna instytucja";
#elif L==9	// pt
	"Outra institui&ccedil;&atilde;o";
#endif

const char *Txt_Another_place =
#if   L==1	// ca
	"Un altre lloc";
#elif L==2	// de
	"Ein weiterer Standort";
#elif L==3	// en
	"Another place";
#elif L==4	// es
	"Otro lugar";
#elif L==5	// fr
	"Un autre emplacement";
#elif L==6	// gn
	"Otro lugar";	// Okoteve traduccin
#elif L==7	// it
	"Un'altra localit&grave;";
#elif L==8	// pl
	"Innym miejscem";
#elif L==9	// pt
	"Outra localiza&ccedil;&atilde;o";
#endif

const char *Txt_Another_user_s_profile =
#if   L==1	// ca
	"Perfil d'un altre usuari";
#elif L==2	// de
	"Profil eines anderen Benutzers";
#elif L==3	// en
	"Another user's profile";
#elif L==4	// es
	"Perfil de otro usuario";
#elif L==5	// fr
	"Profil d'un autre utilisateur";
#elif L==6	// gn
	"Perfil de otro usuario";	// Okoteve traduccin
#elif L==7	// it
	"Altro profilo di utente";
#elif L==8	// pl
	"Profil innego u&zdot;ytkownika";
#elif L==9	// pt
	"Perfil de outro usu&aacute;rio";
#endif

const char *Txt_Add_questions =
#if   L==1	// ca
	"Afegir preguntes";
#elif L==2	// de
	"F&uuml;gen Fragen";
#elif L==3	// en
	"Add questions";
#elif L==4	// es
	"A&ntilde;adir preguntas";
#elif L==5	// fr
	"Ajouter questions";
#elif L==6	// gn
	"A&ntilde;adir preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Aggiungere domande";
#elif L==8	// pl
	"Dodaj pytania";
#elif L==9	// pt
	"Adicionar quest&otilde;es";
#endif

const char *Txt_Add_this_ID =
#if   L==1	// ca
	"Afegir aquest ID";
#elif L==2	// de
	"F&uuml;gen Sie diese Ausweis-Nr.";
#elif L==3	// en
	"Add this ID";
#elif L==4	// es
	"A&ntilde;adir este ID";
#elif L==5	// fr
	"Ajouter ce num&eacute;ro d'identit&eacute;";
#elif L==6	// gn
	"A&ntilde;adir este ID";	// Okoteve traduccin
#elif L==7	// it
	"Aggiungere questa carta d'identit&agrave;";
#elif L==8	// pl
	"Dodaj ten identyfikator";
#elif L==9	// pt
	"Adicionar este n&ordm; identif.";
#endif

const char *Txt_Add_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Afegir %s";
#elif L==2	// de
	"F&uuml;gen %s";
#elif L==3	// en
	"Add %s";
#elif L==4	// es
	"A&ntilde;adir %s";
#elif L==5	// fr
	"Ajouter %s";
#elif L==6	// gn
	"A&ntilde;adir %s";	// Okoteve traduccin
#elif L==7	// it
	"Aggiungere %s";
#elif L==8	// pl
	"Dodaj %s";
#elif L==9	// pt
	"Adicionar %s";
#endif

const char *Txt_Announcement_created =
#if   L==1	// ca
	"Anunci creat.";
#elif L==2	// de
	"Bekanntmachung erstellt.";
#elif L==3	// en
	"Announcement created.";
#elif L==4	// es
	"Anuncio creado.";
#elif L==5	// fr
	"Annonce cr&eacute;&eacute;.";
#elif L==6	// gn
	"Anuncio creado.";		// Okoteve traduccin
#elif L==7	// it
	"Annuncio creato.";
#elif L==8	// pl
	"Og&lstrok;oszenia stworzony.";
#elif L==9	// pt
	"An&uacute;ncio criado.";
#endif

const char *Txt_Announcement_removed =
#if   L==1	// ca
	"Anunci eliminat.";
#elif L==2	// de
	"Bekanntmachung entfernt.";
#elif L==3	// en
	"Announcement removed.";
#elif L==4	// es
	"Anuncio eliminado.";
#elif L==5	// fr
	"Annonce supprim&eacute;.";
#elif L==6	// gn
	"Anuncio eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Annuncio rimosso.";
#elif L==8	// pl
	"Og&lstrok;oszenia usuniete.";
#elif L==9	// pt
	"An&uacute;ncio removido.";
#endif

const char *Txt_Answer_survey =
#if   L==1	// ca
	"Respondre enquesta";
#elif L==2	// de
	"Beantwortung Umfrage";
#elif L==3	// en
	"Answer survey";
#elif L==4	// es
	"Responder encuesta";
#elif L==5	// fr
	"R&eacute;pondre sondage";
#elif L==6	// gn
	"Responder encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Rispondere sondaggio";
#elif L==8	// pl
	"Badanie odpowiedzi";
#elif L==9	// pt
	"Responder inqu&eacute;rito";
#endif

const char *Txt_Answers =		// Answers of test
#if   L==1	// ca
	"Respostes";
#elif L==2	// de
	"Antworten";
#elif L==3	// en
	"Answers";
#elif L==4	// es
	"Respuestas";
#elif L==5	// fr
	"R&eacute;ponses";
#elif L==6	// gn
	"&Ntilde;embohov&aacute;i";
#elif L==7	// it
	"Risposte";
#elif L==8	// pl
	"Odpowiedzi";
#elif L==9	// pt
	"Respostas";
#endif

const char *Txt_any_course =
#if   L==1	// ca
	"qualsevol assignatura";
#elif L==2	// de
	"alle Kursen";
#elif L==3	// en
	"any course";
#elif L==4	// es
	"cualquier asignatura";
#elif L==5	// fr
	"toute mati&egrave;re";
#elif L==6	// gn
	"cualquier asignatura";	// Okoteve traduccin
#elif L==7	// it
	"qualsiasi corso";
#elif L==8	// pl
	"kurs";
#elif L==9	// pt
	"qualquer disciplina";
#endif

const char *Txt_Any_type_of_degree =
#if   L==1	// ca
	"Qualsevol tipus de titulaci&oacute;";
#elif L==2	// de
	"Alle Abschlussart";
#elif L==3	// en
	"grau";
#elif L==4	// es
	"Cualquier tipo de titulaci&oacute;n";
#elif L==5	// fr
	"Tout type d'&eacute;tude";
#elif L==6	// gn
	"Cualquier tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Qualsiasi tipo di laurea";
#elif L==8	// pl
	"Dowolny rodzaj stopnia";
#elif L==9	// pt
	"Qualquer tipo de grau";
#endif

const char *Txt_Any_department =
#if   L==1	// ca
	"Qualsevol departament";
#elif L==2	// de
	"Alle Abteilung";
#elif L==3	// en
	"Any department";
#elif L==4	// es
	"Cualquier departamento";
#elif L==5	// fr
	"Tout d&eacute;partement";
#elif L==6	// gn
	"Cualquier departamento";	// Okoteve traduccin
#elif L==7	// it
	"Qualsiasi dipartimento";
#elif L==8	// pl
	"Wszystkie dzia&lstrok;y";
#elif L==9	// pt
	"Qualquer departamento";
#endif

const char *Txt_any_user =
#if   L==1	// ca
	"qualsevol usuari";
#elif L==2	// de
	"alle Benutzer";
#elif L==3	// en
	"any user";
#elif L==4	// es
	"cualquier usuario";
#elif L==5	// fr
	"n'importe qu'utilisateur";
#elif L==6	// gn
	"cualquier usuario";	// Okoteve traduccin
#elif L==7	// it
	"qualsiasi utente";
#elif L==8	// pl
	"wszystkich uzytkownik&oacute;w";
#elif L==9	// pt
	"qualquer utilizador";
#endif

const char *Txt_Application_key =
#if   L==1	// ca
	"Clau de aplicaci&oacute;";
#elif L==2	// de
	"Application key";		// Need bersetzung
#elif L==3	// en
	"Application key";
#elif L==4	// es
	"Clave de aplicaci&oacute;n";
#elif L==5	// fr
	"Cl&eacute; d'application";
#elif L==6	// gn
	"Clave de aplicaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Codice di applicazione";
#elif L==8	// pl
	"Klawisz aplikacji";
#elif L==9	// pt
	"Chave de aplica&ccedil;&atilde;o";
#endif

const char *Txt_Assessment_criteria =
#if   L==1	// ca
	"Criteris d'avaluaci&oacute;";
#elif L==2	// de
	"Bewertungskriterien";
#elif L==3	// en
	"Assessment criteria";
#elif L==4	// es
	"Criterios de evaluaci&oacute;n";
#elif L==5	// fr
	"Crit&egrave;res d'&eacute;valuation";
#elif L==6	// gn
	"Criterios de evaluaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Criteri di valutazione";
#elif L==8	// pl
	"Kryteria oceny";
#elif L==9	// pt
	"Crit&eacute;rios de avalia&ccedil;&atilde;o";
#endif

const char *Txt_Assignment =
#if   L==1	// ca
	"Activitat";
#elif L==2	// de
	"Aufgabe";
#elif L==3	// en
	"Assignment";
#elif L==4	// es
	"Actividad";
#elif L==5	// fr
	"Activit&eacute;";
#elif L==6	// gn
	"Tar&eacute;a";
#elif L==7	// it
	"Attivit&agrave;";
#elif L==8	// pl
	"Przyporzadkowanie";
#elif L==9	// pt
	"Atividade";
#endif

const char *Txt_Assignment_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La activitat <strong>%s</strong> ara est&agrave; oculta.";
#elif L==2	// de
	"Die Aufgabe <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==3	// en
	"Assignment <strong>%s</strong> is now hidden.";
#elif L==4	// es
	"La actividad <strong>%s</strong> ahora est&aacute; oculta.";
#elif L==5	// fr
	"L'activit&eacute; <strong>%s</strong> est maintenant cach&eacute;e.";
#elif L==6	// gn
	"La actividad <strong>%s</strong> ahora est&aacute; oculta.";	// Okoteve traduccin
#elif L==7	// it
	"L'attivit&agrave; <strong>%s</strong> &egrave; ora nascosta.";
#elif L==8	// pl
	"Zadanie <strong>%s</strong> jest ukryte.";
#elif L==9	// pt
	"A atividade <strong>%s</strong> &eacute; agora oculta.";
#endif

const char *Txt_Assignment_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La activitat <strong>%s</strong> ara est&agrave; visible.";
#elif L==2	// de
	"Die Aufgabe <strong>%s</strong> ist jetzt sichtbar.";
#elif L==3	// en
	"Assignment <strong>%s</strong> is now visible.";
#elif L==4	// es
	"La actividad <strong>%s</strong> ahora est&aacute; visible.";
#elif L==5	// fr
	"L'activit&eacute; <strong>%s</strong> est maintenant visible.";
#elif L==6	// gn
	"La actividad <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7	// it
	"L'attivit&agrave; <strong>%s</strong> &egrave; ora visibile.";
#elif L==8	// pl
	"Zadanie <strong>%s</strong> jest teraz widoczny.";
#elif L==9	// pt
	"A atividade <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_Assignment_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Activitat <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Aufgabe <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Assignment <strong>%s</strong> removed.";
#elif L==4	// es
	"Actividad <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Activit&eacute; <strong>%s</strong> supprim&eacute;e.";
#elif L==6	// gn
	"Actividad <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Attivit&agrave; <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Zadanie <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Atividade <strong>%s</strong> removida.";
#endif

const char *Txt_Assignments =
#if   L==1	// ca
	"Activitats";
#elif L==2	// de
	"Aufgaben";
#elif L==3	// en
	"Assignments";
#elif L==4	// es
	"Actividades";
#elif L==5	// fr
	"Activit&eacute;s";
#elif L==6	// gn
	"Tar&eacute;a";
#elif L==7	// it
	"Attivit&agrave;";
#elif L==8	// pl
	"Zadania";
#elif L==9	// pt
	"Atividades";
#endif

const char *Txt_Assignments_and_other_works =
#if   L==1	// ca
	"Activitats i altres treballs";
#elif L==2	// de
	"Aufgaben und andere Hausarbeiten";
#elif L==3	// en
	"Assignments and other works";
#elif L==4	// es
	"Actividades y otros trabajos";		// Okoteve traduccin
#elif L==5	// fr
	"Activit&eacute;s et autres travaux";
#elif L==6	// gn
	"Actividades y otros trabajos";
#elif L==7	// it
	"Attivit&agrave; e altri lavori";
#elif L==8	// pl
	"Assignments and other works";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Atividades e outros trabalhos";
#endif

const char *Txt_START_END_TIME_HELP[Dat_NUM_START_END_TIME] =
   {
#if   L==1	// ca
	 "Ordenar per data de comen&ccedil;ament"
#elif L==2	// de
	 "Sortieren nach Startdatum"
#elif L==3	// en
	 "Sort by start date"
#elif L==4	// es
	 "Ordenar por fecha de comienzo"
#elif L==5	// fr
	 "Trier par date de d&eacute;but"
#elif L==6	// gn
	 "Ordenar por fecha de comienzo"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per data di inizio"
#elif L==8	// pl
	 "Sortuj wedlug daty rozpoczecia"
#elif L==9	// pt
	 "Classificar por data de in&iacute;cio"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar per data de finalitzaci&oacute;"
#elif L==2	// de
	 "Sortieren nach Enddatum"
#elif L==3	// en
	 "Sort by end date"
#elif L==4	// es
	 "Ordenar por fecha de finalizaci&oacute;n"
#elif L==5	// fr
	 "Trier par date de fin"
#elif L==6	// gn
	 "Ordenar por fecha de finalizaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per data di fine"
#elif L==8	// pl
	 "Sortuj wedlug daty zakonczenia"
#elif L==9	// pt
	 "Classificar por data de t&eacute;rmino"
#endif
	};

const char *Txt_START_END_TIME[Dat_NUM_START_END_TIME] =
   {
#if   L==1	// ca
	 "Inici"
#elif L==2	// de
	 "Start"
#elif L==3	// en
	 "Start"
#elif L==4	// es
	 "Inicio"
#elif L==5	// fr
	 "D&eacute;but"
#elif L==6	// gn
	 "&Ntilde;epyr&utilde;"
#elif L==7	// it
	 "Inizio"
#elif L==8	// pl
	 "Pocz&aogon;tek"
#elif L==9	// pt
	 "In&iacute;cio"
#endif
	 ,
#if   L==1	// ca
	 "Final"
#elif L==2	// de
	 "Ende"
#elif L==3	// en
	 "End"
#elif L==4	// es
	 "Final"
#elif L==5	// fr
	 "Fin"
#elif L==6	// gn
	 "Paha"
#elif L==7	// it
	 "Fine"
#elif L==8	// pl
	 "Koniec"
#elif L==9	// pt
	 "Fim"
#endif
	};

const char *Txt_Assignments_area =
#if   L==1	// ca
	"Zona d'activitats";
#elif L==2	// de
	"Aufgaben-Bereich";
#elif L==3	// en
	"Assignments area";
#elif L==4	// es
	"Zona de actividades";
#elif L==5	// fr
	"Zone d'activit&eacute;s";
#elif L==6	// gn
	"Zona de actividades";	// Okoteve traduccin
#elif L==7	// it
	"Area di attivit&agrave;";
#elif L==8	// pl
	"Obszarze Zadania";
#elif L==9	// pt
	"Zona de atividades";
#endif

const char *Txt_assignments_area =
#if   L==1	// ca
	"zona d'activitats";	// Necessita traduccio
#elif L==2	// de
	"Aufgaben-Bereich";
#elif L==3	// en
	"assignments area";
#elif L==4	// es
	"zona de actividades";
#elif L==5	// fr
	"zone d'activit&eacute;s";
#elif L==6	// gn
	"zona de actividades";	// Okoteve traduccin
#elif L==7	// it
	"area di attivit&agrave;";
#elif L==8	// pl
	"obszarze zadan";
#elif L==9	// pt
	"zona de atividades";
#endif

const char *Txt_Attendance =	// Number of times a students has attended to face-to-face class
#if   L==1	// ca
	"Assist&egrave;ncia";
#elif L==2	// de
	"Anwesenheit";
#elif L==3	// en
	"Attendance";
#elif L==4	// es
	"Asistencia";
#elif L==5	// fr
	"Pr&eacute;sence";
#elif L==6	// gn
	"Asistencia";	// Okoteve traduccin
#elif L==7	// it
	"Presenza";
#elif L==8	// pl
	"Bytno&sacute;&cacute;";
#elif L==9	// pt
	"Presen&ccedil;a";
#endif

const char *Txt_Attendance_list =
#if   L==1	// ca
	"Llista assistencia";
#elif L==2	// de
	"Anwesenheitsliste";
#elif L==3	// en
	"Attendance list";
#elif L==4	// es
	"Lista asistencia";
#elif L==5	// fr
	"Liste fr&eacute;quentation";
#elif L==6	// gn
	"Lista asistencia";	// Okoteve traduccin
#elif L==7	// it
	"Elenco presenze";
#elif L==8	// pl
	"Lista obecno&sacute;ci";
#elif L==9	// pt
	"Lista assiduidade";
#endif

const char *Txt_Availability =
#if   L==1	// ca
	"Disponibilitat";
#elif L==2	// de
	"Verf&uuml;gbarkeit";
#elif L==3	// en
	"Availability";
#elif L==4	// es
	"Disponibilidad";
#elif L==5	// fr
	"Disponibilit&eacute;";
#elif L==6	// gn
	"Disponibilidad";	// Okoteve traduccin
#elif L==7	// it
	"Disponibilit&agrave;";
#elif L==8	// pl
	"Dost&eogon;pno&sacute;&cacute;";
#elif L==9	// pt
	"Disponibilidade";
#endif

const char *Txt_Average_number_BR_of_ASSIG_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&acute;<br />d'activitats<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Aufgaben<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of assignments<br />per course";
#elif L==4	// es
	"N&ordm; medio<br />de actividades<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />d'activit&eacute;s<br />par mati&egrave;re";
#elif L==6	// gn
	"N&ordm; medio<br />de actividades<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di attivit&agrave;<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />zadania<br />na kurs";
#elif L==9	// pt
	"N&ordm; medio<br />de atividades<br />por disciplina";
#endif

const char *Txt_Average_number_BR_of_games_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&acute;<br />de jocs<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Spiele<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of games<br />per course";
#elif L==4	// es
	"N&ordm; medio<br />de juegos<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />de jeux<br />par mati&egrave;re";
#elif L==6	// gn
	"N&ordm; medio<br />de juegos<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di giochi<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />gry<br />na kurs";
#elif L==9	// pt
	"N&ordm; medio<br />de jogos<br />por disciplina";
#endif

const char *Txt_Average_number_BR_of_projects_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&acute;<br />de projectes<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Projekte<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of projects<br />per course";
#elif L==4	// es
	"N&ordm; medio<br />de proyectos<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />de projets<br />par mati&egrave;re";
#elif L==6	// gn
	"N&ordm; medio<br />de proyectos<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di progetti<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />projekty<br />na kurs";
#elif L==9	// pt
	"N&ordm; medio<br />de projetos<br />por disciplina";
#endif

const char *Txt_Average_number_BR_of_questions_BR_per_survey =
#if   L==1	// ca
	"Nombre mitj&acute;<br />de preguntes<br />per enquesta";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Fragen<br />pro Umfrage";
#elif L==3	// en
	"Average number<br />of questions<br />per survey";
#elif L==4	// es
	"N&ordm; medio<br />de preguntas<br />por encuesta";
#elif L==5	// fr
	"Nombre moyen<br />de questions<br />par sondage";
#elif L==6	// gn
	"N&ordm; medio<br />de preguntas<br />por encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di domande<br />per sondaggio";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />pyta&nacute;<br />na ankiety";
#elif L==9	// pt
	"N&ordm; medio<br />de quest&otilde;es<br />por inqu&eacute;rito";
#endif

const char *Txt_Average_BR_number_BR_of_test_BR_questions_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&acute;<br />de preguntes<br />de test per<br />assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Testfragen<br />pro Kurs";
#elif L==3	// en
	"Average<br />number<br />of test<br />questions<br />per course";
#elif L==4	// es
	"N&ordm; medio<br />de preguntas<br />de test por<br />asignatura";
#elif L==5	// fr
	"Nombre moyen<br />de questions<br />de test par<br />mati&egrave;re";
#elif L==6	// gn
	"N&ordm; medio<br />de preguntas<br />de test por<br />asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di domande<br />di test<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />pyta&nacute;<br />testowych<br />na kurs";
#elif L==9	// pt
	"N&ordm; medio<br />de quest&otilde;es<br />de test por<br />disciplina";
#endif

const char *Txt_Average_number_BR_of_surveys_BR_per_course =
#if   L==1	// ca
	"Nombre mitj&acute;<br />d'enquestes<br />per assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl<br />von Umfragen<br />pro Kurs";
#elif L==3	// en
	"Average number<br />of surveys<br />per course";
#elif L==4	// es
	"N&ordm; medio<br />de encuestas<br />por asignatura";
#elif L==5	// fr
	"Nombre moyen<br />de sondages<br />par mati&egrave;re";
#elif L==6	// gn
	"N&ordm; medio<br />de encuestas<br />por asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio<br />di sondaggi<br />per corso";
#elif L==8	// pl
	"&Sacute;rednia liczba<br />badania<br />na kurs";
#elif L==9	// pt
	"N&ordm; medio<br />de inqu&eacute;ritos<br />por disciplina";
#endif

const char *Txt_Average_number_of_courses_to_which_a_user_belongs =
#if   L==1	// ca
	"Nombre mitj&acute; d'assignatures a las quals pertany cada usuari";
#elif L==2	// de
	"Durchschnittliche Anzahl von Kursen, an denen der Benutzer teilnimmt";
#elif L==3	// en
	"Average number of courses to which a user belongs";
#elif L==4	// es
	"N&ordm; medio de asignaturas a las que pertenece cada usuario";
#elif L==5	// fr
	"Nombre moyen de cours &agrave; qui appartient un utilisateur";
#elif L==6	// gn
	"N&ordm; medio de asignaturas a las que pertenece cada usuario";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio di corsi a cui appartiene un utente";
#elif L==8	// pl
	"&Sacute;rednia liczba kurs&oacute;w do kt&oacute;rej nalezy uzytkownik";
#elif L==9	// pt
	"N&ordm; medio de disciplinas em que cada utilizador pertence";
#endif

const char *Txt_Average_BR_number_of_BR_times_that_BR_a_question_BR_has_been_BR_responded =
#if   L==1	// ca
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";	// Necessita traduccio
#elif L==2	// de
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Need bersetzung
#elif L==3	// en
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";
#elif L==4	// es
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";
#elif L==5	// fr
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Besoin de traduction
#elif L==6	// gn
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";	// Okoteve traduccin
#elif L==7	// it
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Bisogno di traduzione
#elif L==8	// pl
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&ordm; m&eacute;dio<br />de vezes<br />que uma<br />pergunta<br />foi respondida";
#endif

const char *Txt_Average_BR_number_of_BR_times_that_BR_questions_BR_have_been_BR_responded_BR_per_course =
#if   L==1	// ca
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";	// Necessita traduccio
#elif L==2	// de
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Need bersetzung
#elif L==3	// en
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";
#elif L==4	// es
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";
#elif L==5	// fr
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Besoin de traduction
#elif L==6	// gn
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";			// Okoteve traduccin
#elif L==7	// it
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Bisogno di traduzione
#elif L==8	// pl
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&ordm; m&eacute;dio<br />de vezes<br />que uma<br />pergunta<br />foi respondida<br />por disciplina";
#endif

const char *Txt_Average_number_of_users_belonging_to_a_course =
#if   L==1	// ca
	"Nombre mitj&agrave; de usuaris que pertanyen a cada assignatura";
#elif L==2	// de
	"Durchschnittliche Anzahl der Benutzer, die an einer Kurs teilnehmen";
#elif L==3	// en
	"Average number of users belonging to a course";
#elif L==4	// es
	"N&ordm; medio de usuarios que pertenecen a cada asignatura";
#elif L==5	// fr
	"Nombre moyen d'utilisateurs appartenant &agrave; une mati&egrave;re";
#elif L==6	// gn
	"N&ordm; medio de usuarios que pertenecen a cada asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Numero medio di utenti appartenenti a un corso";
#elif L==8	// pl
	"&Sacute;rednia liczba uzytkownik&oacute;w nalezacych do kursu";
#elif L==9	// pt
	"N&ordm; medio de utilizadores que pertencem a uma disciplina";
#endif

const char *Txt_AVERAGE_PHOTO_TYPES[Pho_NUM_AVERAGE_PHOTO_TYPES] =
   {
#if   L==1	// ca
	 "mitjana de totes les fotos"
#elif L==2	// de
	 "Median aller Fotos"
#elif L==3	// en
	 "median of all the photos"
#elif L==4	// es
	 "mediana de todas las fotos"
#elif L==5	// fr
	 "m&eacute;dian de toutes les photos"
#elif L==6	// gn
	 "mediana de todas las fotos"			// Okoteve traduccin
#elif L==7	// it
	 "mediana di tutte le foto"
#elif L==8	// pl
	 "mediana wszystkich zdjec"
#elif L==9	// pt
	 "mediana de todas as fotos"
#endif
	 ,
#if   L==1	// ca
	 "mitjana aritm&egrave;tica de totes les fotos"
#elif L==2	// de
	 "arithmetischer Mittelwert aller Fotos"
#elif L==3	// en
	 "arithmetic mean of all the photos"
#elif L==4	// es
	 "media aritm&eacute;tica de todas las fotos"
#elif L==5	// fr
	 "moyenne arithm&eacute;tique de toutes les photos"
#elif L==6	// gn
	 "media aritm&eacute;tica de todas las fotos"	// Okoteve traduccin
#elif L==7	// it
	 "media aritmetica di tutte le foto"
#elif L==8	// pl
	 "&sacute;rednia arytmetyczna wszystkich zdjec"
#elif L==9	// pt
	 "m&eacute;dia aritm&eacute;tica de todas as fotos"
#endif
	};

const char *Txt_Average_BR_score_BR_per_question_BR_from_0_to_1 =
#if   L==1	// ca
	"Puntuaci&oacute;n<br />media<br />por pregunta<br />(de 0 a 1)";	// Necessita traduccio
#elif L==2	// de
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Need bersetzung
#elif L==3	// en
	"Average<br />score<br />per question<br />(from 0 to 1)";
#elif L==4	// es
	"Puntuaci&oacute;n<br />media<br />por pregunta<br />(de 0 a 1)";
#elif L==5	// fr
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Besoin de traduction
#elif L==6	// gn
	"Puntuaci&oacute;n<br />media<br />por pregunta<br />(de 0 a 1)";	// Okoteve traduccin
#elif L==7	// it
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Bisogno di traduzione
#elif L==8	// pl
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o<br />m&eacute;dia por<br />pergunta<br />(de 0 a 1)";
#endif

const char *Txt_Average_type =
#if   L==1	// ca
	"Tipus de mitjana";
#elif L==2	// de
	"Art des Mittelwerts";
#elif L==3	// en
	"Average type";
#elif L==4	// es
	"Tipo de promedio";
#elif L==5	// fr
	"Type de moyenne";
#elif L==6	// gn
	"Tipo de promedio";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di media";
#elif L==8	// pl
	"&Sacute;rednia typu";
#elif L==9	// pt
	"Tipo de m&eacute;dia";
#endif

const char *Txt_Banned_users =
#if   L==1	// ca
	"Usuaris bloquejats";
#elif L==2	// de
	"Gebannt Benutzer";
#elif L==3	// en
	"Banned users";
#elif L==4	// es
	"Usuarios bloqueados";
#elif L==5	// fr
	"Utilisateurs bannis";
#elif L==6	// gn
	"Usuarios bloqueados";	// Okoteve traduccin
#elif L==7	// it
	"Utenti vietati";
#elif L==8	// pl
	"Zablokowani u&zdot;ytkownicy";
#elif L==9	// pt
	"Usu&aacute;rios banidos";
#endif

const char *Txt_Banner =
#if   L==1	// ca
	"Banner";
#elif L==2	// de
	"Werbebanner";
#elif L==3	// en
	"Banner";
#elif L==4	// es
	"Banner";
#elif L==5	// fr
	"Banni&eacute;re";
#elif L==6	// gn
	"Banner";
#elif L==7	// it
	"Banner";
#elif L==8	// pl
	"Banner";
#elif L==9	// pt
	"Banner";
#endif

const char *Txt_Banner_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Banner <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Werbebanner <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Banner <strong>%s</strong> removed.";
#elif L==4	// es
	"Banner <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Banni&eacute;re <strong>%s</strong> supprim&eacute;e.";
#elif L==6	// gn
	"Banner <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Banner <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Banner <strong>%s</strong> usuniety.";
#elif L==9	// pt
	"Banner <strong>%s</strong> removido.";
#endif

const char *Txt_Banners =
#if   L==1	// ca
	"Banners";
#elif L==2	// de
	"Werbebanners";
#elif L==3	// en
	"Banners";
#elif L==4	// es
	"Banners";
#elif L==5	// fr
	"Banni&eacute;res";
#elif L==6	// gn
	"Banners";
#elif L==7	// it
	"Banners";
#elif L==8	// pl
	"Banners";
#elif L==9	// pt
	"Banners";
#endif

const char *Txt_Basic_public_profile =
#if   L==1	// ca
	"Perfil p&uacute;blic b&agrave;sic";
#elif L==2	// de
	"Grundlegendes &ouml;ffentliches Profile";
#elif L==3	// en
	"Basic public profile";
#elif L==4	// es
	"Perfil p&uacute;blico b&aacute;sico";
#elif L==5	// fr
	"Profil public de base";
#elif L==6	// gn
	"Perfil p&uacute;blico b&aacute;sico";	// Okoteve traduccin
#elif L==7	// it
	"Profilo pubblico di base";
#elif L==8	// pl
	"Podstawowy profil publiczny";
#elif L==9	// pt
	"Perfil p&uacute;blico b&aacute;sico";
#endif

const char *Txt_Before_creating_a_new_account_check_if_you_have_been_already_registered =
#if   L==1	// ca
	"Abans de crear un compte nou,"
	" comprovi si ja li han inscrit";
#elif L==2	// de
	"Bevor ein neues Benutzerkonto erstellen,"
	" wenn Sie bereits registriert worden sind";
#elif L==3	// en
	"Before creating a new account,"
	" check if you have been already registered";
#elif L==4	// es
	"Antes de crear una nueva cuenta,"
	" compruebe si ya le han inscrito";
#elif L==5	// fr
	"Avant de cr&eacute;er un nouveau compte,"
	" v&eacute;rifiez si vous avez d&eacute;j&agrave; enregistr&eacute;";
#elif L==6	// gn
	"Antes de crear una nueva cuenta,"
	" compruebe si ya le han inscrito";	// Okoteve traduccin
#elif L==7	// it
	"Prima di creare un nuovo account,"
	" verificare se sono stati gi&agrave; registrato";
#elif L==8	// pl
	"Przed utworzeniem nowego konta sprawd&zacute;,"
	" czy zosta&lstrok;e&sacute; ju&zdot; zarejestrowany";
#elif L==9	// pt
	"Antes de criar uma nova conta,"
	" verifique se voc&ecirc; j&aacute; foi registrado/a";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_nickname =
#if   L==1	// ca
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de crear el seu sobrenom.";
#elif L==2	// de
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihr Benutzernamen erstellen.";
#elif L==3	// en
	"Before going to any other option"
	" you must create your nickname.";
#elif L==4	// es
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su apodo.";
#elif L==5	// fr
	"Avant d'aller &agrave; une autre option"
	" vous devez cr&eacute;er votre surnom.";
#elif L==6	// gn
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su apodo.";			// Okoteve traduccin
#elif L==7	// it
	"Prima di scegliere qualsiasi altra opzione"
	" devi creare il tuo soprannome.";
#elif L==8	// pl
	"Przed udaniem si&eogon; do innej opcji"
	" nale&zdot;y wype&lstrok;ni&cacute; sw&oacute;j nick";
#elif L==9	// pt
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve criar sua alcunha.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_create_your_password =
#if   L==1	// ca
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de crear la seva contrasenya.";
#elif L==2	// de
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihr Passwort erstellen.";
#elif L==3	// en
	"Before going to any other option"
	" you must create your password.";
#elif L==4	// es
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su contrase&ntilde;a.";
#elif L==5	// fr
	"Avant d'aller &agrave; une autre option"
	" vous devez cr&eacute;er votre mot de passe.";
#elif L==6	// gn
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su contrase&ntilde;a.";		// Okoteve traduccin
#elif L==7	// it
	"Prima di scegliere qualsiasi altra opzione"
	" devi creare una password.";
#elif L==8	// pl
	"Przed p&oacute;jsciem do jakiejkolwiek innej opcji"
	" nalezy utworzy&cacute; has&lstrok;o.";
#elif L==9	// pt
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	"voc&ecirc; deve criar sua senha.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_in_your_email_address =
#if   L==1	// ca
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de omplir la seva adre&ccedil;a de correu.";
#elif L==2	// de
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihr E-Mail-Adresse ausf&uuml;llen.";
#elif L==3	// en
	"Before going to any other option"
	" you must fill in your email address.";
#elif L==4	// es
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su direcci&oacute;n de correo.";
#elif L==5	// fr
	"Avant d'aller &agrave; une autre option"
	" vous devez indiquer votre adresse email.";
#elif L==6	// gn
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe rellenar su direcci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7	// it
	"Prima di scegliere qualsiasi altra opzione"
	" devi inserire il tuo indirizzo email.";
#elif L==8	// pl
	"Przed p&oacute;jsciem do jakiejkolwiek innej opcji"
	" wype&lstrok;ni&cacute; sw&oacute;j adres e-mail.";
#elif L==9	// pt
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	"voc&ecirc; deve preencher seu endere&ccedil;o de email.";
#endif

const char *Txt_Briefcase_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Briefcase of <strong>%s</strong> has been removed.";		// Need bersetzung
#elif L==3	// en
	"Briefcase of <strong>%s</strong> has been removed.";
#elif L==4	// es
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";
#elif L==5	// fr
	"Briefcase of <strong>%s</strong> has been removed.";		// Besoin de traduction
#elif L==6	// gn
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La penna USB de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==8	// pl
	"Briefcase of <strong>%s</strong> has been removed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pasta de <strong>%s</strong> foi removida.";
#endif

const char *Txt_Briefcases =
#if   L==1	// ca
	"Maletins";
#elif L==2	// de
	"Aktentasche";
#elif L==3	// en
	"Briefcases";
#elif L==4	// es
	"Maletines";
#elif L==5	// fr
	"Porte-documents";
#elif L==6	// gn
	"Maletines";	// Okoteve traduccin
#elif L==7	// it
	"Penna USB";
#elif L==8	// pl
	"Briefcases";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pasta";
#endif

const char *Txt_Calculate =
#if   L==1	// ca
	"Calcular";
#elif L==2	// de
	"Berechnen";
#elif L==3	// en
	"Calculate";
#elif L==4	// es
	"Calcular";
#elif L==5	// fr
	"Calculer";
#elif L==6	// gn
	"Calcular";	// Okoteve traduccin
#elif L==7	// it
	"Calcolare";
#elif L==8	// pl
	"Oblicza&cacute;";
#elif L==9	// pt
	"Calcular";
#endif

const char *Txt_Calculate_average_photo_of_THE_DEGREE_X =
#if   L==1	// ca
	"Calcular fotografia mitjana de";
#elif L==2	// de
	"Calculate average photo of";			// Need bersetzung
#elif L==3	// en
	"Calculate average photo of";
#elif L==4	// es
	"Calcular fotograf&iacute;a promedio de";
#elif L==5	// fr
	"Calculate average photo of";			// Besoin de traduction
#elif L==6	// gn
	"Calcular fotograf&iacute;a promedio de";	// Okoteve traduccin
#elif L==7	// it
	"Calcolare la media della foto di";
#elif L==8	// pl
	"Obliczyc &sacute;rednia zdjecie";
#elif L==9	// pt
	"Calcular a foto m&eacute;dia de";
#endif

const char *Txt_Calendar =
#if   L==1	// ca
	"Calendari";
#elif L==2	// de
	"Kalender";
#elif L==3	// en
	"Calendar";
#elif L==4	// es
	"Calendario";
#elif L==5	// fr
	"Calendrier";
#elif L==6	// gn
	"Arapapaha";
#elif L==7	// it
	"Calendario";
#elif L==8	// pl
	"Kalendarz";
#elif L==9	// pt
	"Calend&aacute;rio";
#endif

const char *Txt_Call_the_roll =
#if   L==1	// ca
	"Passar llista";
#elif L==2	// de
	"Anwesenheit feststellen";
#elif L==3	// en
	"Call the roll";
#elif L==4	// es
	"Pasar lista";
#elif L==5	// fr
	"Faire l'appel";
#elif L==6	// gn
	"Pasar lista";	// Okoteve traduccin
#elif L==7	// it
	"Appello";
#elif L==8	// pl
	"Imienne";
#elif L==9	// pt
	"Lista de presen&ccedil;a";
#endif

const char *Txt_Can_not_create_the_folder_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No s'ha creat la carpeta <strong>%s</strong>"
	" perque s'excediria la quota de disc.";
#elif L==2	// de
	"Verzeichnis <strong>%s</strong> kann aus Platzgrnden"
	" nicht erstellt werden.";
#elif L==3	// en
	"Cannot create the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4	// es
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5	// fr
	"Impossible de cr&eacute;er le r&eacute;pertoire <strong>%s</strong>"
	" parce qu'il d&eacute;passerait le quota de disque.";
#elif L==6	// gn
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7	// it
	"Non si pu&ograve; creare la cartella <strong>%s</strong>"
	" perch&egrave; si &egrave; superata la quota del disco.";
#elif L==8	// pl
	"Nie mozna utworzy&cacute; folder <strong>%s</strong>,"
	" poniewaz to przekracza kontyngentu na dysku.";
#elif L==9	// pt
	"N&atilde;o &eacute; poss&iacute;vel criar o diret&oacute;rio <strong>%s</strong>"
	" porque iria exceder a cota de disco.";
#endif

const char *Txt_Can_not_create_the_folder_X_because_there_is_already_a_folder_or_a_file_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No s'ha creat la carpeta <strong>%s</strong>"
	" perque ja existeix una carpeta o un fitxer amb aquest nom.";
#elif L==2	// de
	"Cannot create the folder <strong>%s</strong>"
	" because there is already a folder or a file with that name.";	// Need bersetzung
#elif L==3	// en
	"Cannot create the folder <strong>%s</strong>"
	" because there is already a folder or a file with that name.";
#elif L==4	// es
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque ya existe una carpeta o un archivo con ese nombre.";
#elif L==5	// fr
	"Impossible de cr&eacute;er le r&eacute;pertoire <strong>%s</strong>"
	" car il existe d&eacute;j&agrave; un r&eacute;pertoire ou un fichier avec ce nom.";
#elif L==6	// gn
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque ya existe una carpeta o un archivo con ese nombre.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi creare la cartella <strong>%s</strong>"
	" perch&egrave; gi&agrave; esiste una cartella o un file con quel nome.";
#elif L==8	// pl
	"Nie mozna utworzy&cacute; folder <strong>%s</strong>"
	" poniewaz istnieje juz folder lub plik o tej nazwie.";
#elif L==9	// pt
	"N&atilde;o &eacute; poss&iacute;vel criar o diret&oacute;rio <strong>%s</strong>"
	" porque j&aacute; existe um diret&oacute;rio ou um arquivo com esse nome.";
#endif

const char *Txt_Can_not_create_the_link_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No s'ha creat l'enlla&ccedil; <strong>%s</strong>"
	" perque s'excediria la quota de disc.";
#elif L==2	// de
	"Link <strong>%s</strong> kann aus Platzgr&uuml;nden"
	" nicht erstellt werden.";
#elif L==3	// en
	"Cannot create the link <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4	// es
	"No se ha creado el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5	// fr
	"Impossible de cr&eacute;er le lien <strong>%s</strong>"
	" parce qu'il d&eacute;passerait le quota de disque.";
#elif L==6	// gn
	"No se ha creado el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7	// it
	"Non si pu&ograve; creare il link <strong>%s</strong>"
	" perch&egrave; si &egrave; superata la quota del disco.";
#elif L==8	// pl
	"Nie mozna utworzy&cacute; link <strong>%s</strong>,"
	" poniewaz to przekracza kontyngentu na dysku.";
#elif L==9	// pt
	"N&atilde;o &eacute; poss&iacute;vel criar a liga&ccedil;&atilde;o <strong>%s</strong>"
	" porque iria exceder a cota de disco.";
#endif

const char *Txt_Can_not_create_the_link_X_because_there_is_already_a_folder_or_a_link_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No s'ha creat l'enlla&ccedil; <strong>%s</strong>"
	" perque ja existeix una carpeta o un enlla&ccedil; amb aquest nom.";
#elif L==2	// de
	"Cannot create the link <strong>%s</strong>"
	" because there is already a folder or a link with that name.";	// Need bersetzung
#elif L==3	// en
	"Cannot create the link <strong>%s</strong>"
	" because there is already a folder or a link with that name.";
#elif L==4	// es
	"No se ha creado el enlace <strong>%s</strong>"
	" porque ya existe una carpeta o un enlace con ese nombre.";
#elif L==5	// fr
	"Impossible de cr&eacute;er le lien <strong>%s</strong>"
	" car il existe d&eacute;j&agrave; un r&eacute;pertoire ou un lien avec ce nom.";
#elif L==6	// gn
	"No se ha creado el enlace <strong>%s</strong>"
	" porque ya existe una carpeta o un enlace con ese nombre.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi creare il link <strong>%s</strong>"
	" perch&egrave; gi&agrave; esiste una cartella o un link con quel nome.";
#elif L==8	// pl
	"Nie mozna utworzy&cacute; link <strong>%s</strong>"
	" poniewaz istnieje juz folder lub link o tej nazwie.";
#elif L==9	// pt
	"N&atilde;o &eacute; poss&iacute;vel criar a liga&ccedil;&atilde;o <strong>%s</strong>"
	" porque j&aacute; existe um diret&oacute;rio ou uma liga&ccedil;&atilde;o com esse nome.";
#endif

const char *Txt_Can_not_rename_a_folder_of_assignment =
#if   L==1	// ca
	"No es pot canviar el nom de una carpeta de activitat.";
#elif L==2	// de
	"Das Verzeichnis der Aufgabe kann nicht umbenannt werden.";
#elif L==3	// en
	"Cannot rename a folder of assignment.";
#elif L==4	// es
	"No se puede cambiar el nombre de una carpeta de actividad.";
#elif L==5	// fr
	"Impossible de renommer un r&eacute;pertoire d'activit&eacute;.";
#elif L==6	// gn
	"No se puede cambiar el nombre de una carpeta de actividad.";	// Okoteve traduccin
#elif L==7	// it
	"Non &egrave; possibile rinominare una cartella di attivit&agrave;.";
#elif L==8	// pl
	"Nie mozna zmienic nazwy folderu zadania.";
#elif L==9	// pt
	"N&atilde;o &eacute; poss&iacute;vel renomear um diret&oacute;rio de actividade.";
#endif

const char *Txt_Capacity_OF_A_CLASSROOM =
#if   L==1	// ca
	"Aforo";
#elif L==2	// de
	"Kapazit&auml;t";
#elif L==3	// en
	"Capacity";
#elif L==4	// es
	"Aforo";
#elif L==5	// fr
	"Capacit&eacute;";
#elif L==6	// gn
	"Aforo";				// Okoteve traduccin
#elif L==7	// it
	"Capacit&agrave;";
#elif L==8	// pl
	"Pojemno&sacute;&cacute;";
#elif L==9	// pt
	"Capacidade";
#endif

const char *Txt_Centre =
#if   L==1	// ca
	"Centre";
#elif L==2	// de
	"Lehrinstitut";
#elif L==3	// en
	"Centre";
#elif L==4	// es
	"Centro";
#elif L==5	// fr
	"Centre";
#elif L==6	// gn
	"Mbo'ehao";
#elif L==7	// it
	"Centro";
#elif L==8	// pl
	"Centrum";
#elif L==9	// pt
	"Centro";
#endif

const char *Txt_centre =
#if   L==1	// ca
	"centre";
#elif L==2	// de
	"Lehrinstitut";
#elif L==3	// en
	"centre";
#elif L==4	// es
	"centro";
#elif L==5	// fr
	"centre";
#elif L==6	// gn
	"mbo'ehao";
#elif L==7	// it
	"centro";
#elif L==8	// pl
	"centrum";
#elif L==9	// pt
	"centro";
#endif

const char *Txt_Centre_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Centre <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Lehrinstitut <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Centre <strong>%s</strong> removed.";
#elif L==4	// es
	"Centro <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Centre <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6	// gn
	"Centro <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Centro <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Centrum <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Centro <strong>%s</strong> removido.";
#endif

const char *Txt_CENTRE_STATUS[Crs_NUM_STATUS_TXT] =
   {
#if   L==1	// ca	// Ctr_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==2	// de
	 "Unbekannt"
#elif L==3	// en
	 "Unknown"
#elif L==4	// es
	 "Desconocido"
#elif L==5	// fr
	 "Inconnu"
#elif L==6	// gn
	 "Desconocido"			// Okoteve traduccin
#elif L==7	// it
	 "Sconosciuto"
#elif L==8	// pl
	 "Nieznany"
#elif L==9	// pt
	 "Desconhecido"
#endif
	 ,
#if   L==1	// ca	// Ctr_STATUS_ACTIVE
	 "Activo"
#elif L==2	// de
	 "Aktive"
#elif L==3	// en
	 "Active"
#elif L==4	// es
	 "Activo"
#elif L==5	// fr
	 "Actif"
#elif L==6	// gn
	 "Activo"	// Okoteve traduccin
#elif L==7	// it
	 "Attivo"
#elif L==8	// pl
	 "Aktywne"
#elif L==9	// pt
	 "Ativo"
#endif
	 ,
#if   L==1	// ca	// Ctr_STATUS_PENDING
	 "Novo"
#elif L==2	// de
	 "Neu"
#elif L==3	// en
	 "New"
#elif L==4	// es
	 "Nuevo"
#elif L==5	// fr
	 "Nouveau"
#elif L==6	// gn
	 "Pyahu"
#elif L==7	// it
	 "Nuovo"
#elif L==8	// pl
	 "Nowy"
#elif L==9	// pt
	 "Novo"
#endif
	 ,
#if   L==1	// ca	// Ctr_STATUS_REMOVED
	 "Eliminado"
#elif L==2	// de
	 "Entfernt"
#elif L==3	// en
	 "Removed"
#elif L==4	// es
	 "Eliminado"
#elif L==5	// fr
	 "Supprim&eacute;"
#elif L==6	// gn
	 "Eliminado"	// Okoteve traduccin
#elif L==7	// it
	 "Rimosso"
#elif L==8	// pl
	 "Usuniete"
#elif L==9	// pt
	 "Suprimido"
#endif
	};

const char *Txt_Centres =
#if   L==1	// ca
	"Centres";
#elif L==2	// de
	"Lehrinstitute";
#elif L==3	// en
	"Centres";
#elif L==4	// es
	"Centros";
#elif L==5	// fr
	"Centres";
#elif L==6	// gn
	"Mbo'ehao";
#elif L==7	// it
	"Centri";
#elif L==8	// pl
	"Centra";
#elif L==9	// pt
	"Centros";
#endif

const char *Txt_Centres_ABBREVIATION =
#if   L==1	// ca
	"Cent.";
#elif L==2	// de
	"Lehrin.";
#elif L==3	// en
	"Cent.";
#elif L==4	// es
	"Cent.";
#elif L==5	// fr
	"Cent.";
#elif L==6	// gn
	"Mbo'ehao";
#elif L==7	// it
	"Cent.";
#elif L==8	// pl
	"Cent.";
#elif L==9	// pt
	"Cent.";
#endif

const char *Txt_centres =
#if   L==1	// ca
	"centres";
#elif L==2	// de
	"Lehrinstitute";
#elif L==3	// en
	"centres";
#elif L==4	// es
	"centros";
#elif L==5	// fr
	"centres";
#elif L==6	// gn
	"mbo'ehao";
#elif L==7	// it
	"centri";
#elif L==8	// pl
	"centra";
#elif L==9	// pt
	"centros";
#endif

const char *Txt_CENTRES_HELP_ORDER[Ctr_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Ordenar per centre"
#elif L==2	// de
	 "Nach Lehrinstitut Sortieren"
#elif L==3	// en
	 "Sort by centre"
#elif L==4	// es
	 "Ordenar por centro"
#elif L==5	// fr
	 "Trier par centre"
#elif L==6	// gn
	 "Ordenar por centro"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per centro"
#elif L==8	// pl
	 "Sortuj centrum"
#elif L==9	// pt
	 "Classificar por centro"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar per n&ordm; d'usuaris"
#elif L==2	// de
	 "Nach Anzahl der Benutzer sortieren"
#elif L==3	// en
	 "Sort by no. of users"
#elif L==4	// es
	 "Ordenar por n&ordm; de usuarios"
#elif L==5	// fr
	 "Trier par nombre d'utilisateurs"
#elif L==6	// gn
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per numero di utenti"
#elif L==8	// pl
	 "Sortuj wedlug liczby uzytkownikow"
#elif L==9	// pt
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_Centres_of_INSTITUTION_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Centres de %s";
#elif L==2	// de
	"Lehrinstitute von %s";
#elif L==3	// en
	"Centres of %s";
#elif L==4	// es
	"Centros de %s";
#elif L==5	// fr
	"Centres de %s";
#elif L==6	// gn
	"Centros de %s";	// Okoteve traduccin
#elif L==7	// it
	"Centri di %s";
#elif L==8	// pl
	"Centra %s";
#elif L==9	// pt
	"Centros de %s";
#endif

const char *Txt_CENTRES_ORDER[Ctr_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Centre"
#elif L==2	// de
	 "Lehrinstitut"
#elif L==3	// en
	 "Centre"
#elif L==4	// es
	 "Centro"
#elif L==5	// fr
	 "Centre"
#elif L==6	// gn
	 "Mbo'ehao"
#elif L==7	// it
	 "Centro"
#elif L==8	// pl
	 "Centrum"
#elif L==9	// pt
	 "Centro"
#endif
	 ,
#if   L==1	// ca
	 "Usuaris del centre"
#elif L==2	// de
	 "Benutzer der Lehrinstitut"
#elif L==3	// en
	 "Users of the centre"
#elif L==4	// es
	 "Usuarios del centro"
#elif L==5	// fr
	 "Utilisateurs du centre"
#elif L==6	// gn
	 "Usuarios del centro"	// Okoteve traduccin
#elif L==7	// it
	 "Utenti del centro"
#elif L==8	// pl
	 "U&zdot;ytkownicy centrum"
#elif L==9	// pt
	 "Utilizadores do centro"
#endif
	};

const char *Txt_Centres_with_pending_degrees =
#if   L==1	// ca
	"Centres amb titulacions pendents d'activar";
#elif L==2	// de
	"Lehrinstitute mit anh&auml;ngig Studiengang";
#elif L==3	// en
	"Centres with pending degrees";
#elif L==4	// es
	"Centros con titulaciones pendientes de activar";
#elif L==5	// fr
	"Centres avec des &eacute;tudes en attente";
#elif L==6	// gn
	"Centros con titulaciones pendientes de activar";	// Okoteve traduccin
#elif L==7	// it
	"Centri con lauree in sospeso";
#elif L==8	// pl
	"Centra z stopnie oczekuj&aogon;cy";
#elif L==9	// pt
	"Centros com graus pendentes";
#endif

const char *Txt_Change_account =
#if   L==1	// ca
	"Canviar compte";
#elif L==2	// de
	"Benutzerkonto &auml;ndern";
#elif L==3	// en
	"Change account";
#elif L==4	// es
	"Cambiar cuenta";
#elif L==5	// fr
	"Changer compte";
#elif L==6	// gn
	"Cambiar cuenta";	// Okoteve traduccin
#elif L==7	// it
	"Cambiare account";
#elif L==8	// pl
	"Zmie&nacute; konto";
#elif L==9	// pt
	"Alterar conta";
#endif

const char *Txt_Change_email =
#if   L==1	// ca
	"Canviar correu";
#elif L==2	// de
	"E-Mail &auml;ndern";
#elif L==3	// en
	"Change email";
#elif L==4	// es
	"Cambiar correo";
#elif L==5	// fr
	"Changer courrier";
#elif L==6	// gn
	"Cambiar correo";	// Okoteve traduccin
#elif L==7	// it
	"Cambiare email";
#elif L==8	// pl
	"Zmie&nacute; email";
#elif L==9	// pt
	"Alterar email";
#endif

const char *Txt_Change_IDs =
#if   L==1	// ca
	"Canviar IDs";
#elif L==2	// de
	"Ausweis-Nr. &auml;ndern";
#elif L==3	// en
	"Change IDs";
#elif L==4	// es
	"Cambiar ID";
#elif L==5	// fr
	"Changer IDs";
#elif L==6	// gn
	"Cambiar ID";		// Okoteve traduccin
#elif L==7	// it
	"Cambiare ID";
#elif L==8	// pl
	"Zmie&nacute; identyfikatory";
#elif L==9	// pt
	"Alterar n&ordm;s. identif.";
#endif

const char *Txt_Change_image_video =
#if   L==1	// ca
	"Canviar imatge / v&iacute;deo";
#elif L==2	// de
	"Abbild / Video &auml;ndern";
#elif L==3	// en
	"Change image / video";
#elif L==4	// es
	"Cambiar imagen / v&iacute;deo";
#elif L==5	// fr
	"Changer image / vid&eacute;o";
#elif L==6	// gn
	"Moambue ta'&atilde;nga / video";
#elif L==7	// it
	"Cambiare immagine / video";
#elif L==8	// pl
	"Zmie&nacute; obraz / wideo";
#elif L==9	// pt
	"Alterar imagem / v&iacute;deo";
#endif

const char *Txt_Change_logo =
#if   L==1	// ca
	"Canviar logo";
#elif L==2	// de
	"Logo &auml;ndern";
#elif L==3	// en
	"Change logo";
#elif L==4	// es
	"Cambiar logo";
#elif L==5	// fr
	"Changer logo";
#elif L==6	// gn
	"Moambue logo";
#elif L==7	// it
	"Cambiare logo";
#elif L==8	// pl
	"Zmie&nacute; logo";
#elif L==9	// pt
	"Alterar logotipo";
#endif

const char *Txt_Change_my_groups =
#if   L==1	// ca
	"Canviar de grups";
#elif L==2	// de
	"Gruppen &auml;ndern";
#elif L==3	// en
	"Change my groups";
#elif L==4	// es
	"Cambiar de grupos";
#elif L==5	// fr
	"Modifier mes groupes";
#elif L==6	// gn
	"Cambiar de grupos";	// Okoteve traduccin
#elif L==7	// it
	"Cambia i miei gruppi";
#elif L==8	// pl
	"Zmie&nacute;moje grupy";
#elif L==9	// pt
	"Alterar meus grupos";
#endif

const char *Txt_Change_nickname =
#if   L==1	// ca
	"Canviar sobrenom";
#elif L==2	// de
	"Benutzername &auml;ndern";
#elif L==3	// en
	"Change nickname";
#elif L==4	// es
	"Cambiar apodo";
#elif L==5	// fr
	"Changer le surnom";
#elif L==6	// gn
	"Cambiar apodo";	// Okoteve traduccin
#elif L==7	// it
	"Cambiare nome utente";
#elif L==8	// pl
	"Zmie&nacute; nick";
#elif L==9	// pt
	"Alterar alcunha";
#endif

const char *Txt_Change_password =
#if   L==1	// ca
	"Canviar contrasenya";
#elif L==2	// de
	"Passwort &auml;ndern";
#elif L==3	// en
	"Change password";
#elif L==4	// es
	"Cambiar contrase&ntilde;a";
#elif L==5	// fr
	"Changer mot de passe";
#elif L==6	// gn
	"Cambiar contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Cambiare password";
#elif L==8	// pl
	"Zmie&nacute; has&lstrok;o";
#elif L==9	// pt
	"Alterar senha";
#endif

const char *Txt_Change_photo =
#if   L==1	// ca
	"Canviar foto";
#elif L==2	// de
	"Foto &auml;ndern";
#elif L==3	// en
	"Change photo";
#elif L==4	// es
	"Cambiar foto";
#elif L==5	// fr
	"Changer photo";
#elif L==6	// gn
	"Cambiar foto";		// Okoteve traduccin
#elif L==7	// it
	"Cambiare foto";
#elif L==8	// pl
	"Zmie&nacute; zdjecie";
#elif L==9	// pt
	"Alterar foto";
#endif

const char *Txt_CHAT_Room =
#if   L==1	// ca
	"Sala";
#elif L==2	// de
	"Raum";
#elif L==3	// en
	"Room";
#elif L==4	// es
	"Sala";
#elif L==5	// fr
	"Salle";
#elif L==6	// gn
	"Sala";	// Okoteve traduccin
#elif L==7	// it
	"Room";
#elif L==8	// pl
	"Pok&oacute;j";
#elif L==9	// pt
	"Room";
#endif

const char *Txt_CHAT_Room_code =
#if   L==1	// ca
	"Codi de sala";
#elif L==2	// de
	"Raum-code";
#elif L==3	// en
	"Room code";
#elif L==4	// es
	"C&oacute;digo de sala";
#elif L==5	// fr
	"Code de salle";
#elif L==6	// gn
	"C&oacute;digo de sala";	// Okoteve traduccin
#elif L==7	// it
	"Chiave di room";
#elif L==8	// pl
	"Kod pokoju";
#elif L==9	// pt
	"C&oacute;digo de room";
#endif

const char *Txt_Chat_rooms =
#if   L==1	// ca
	"Sales de xat";
#elif L==2	// de
	"Chat-R&auml;ume";
#elif L==3	// en
	"Chat rooms";
#elif L==4	// es
	"Salas de chat";
#elif L==5	// fr
	"Salles de chat";
#elif L==6	// gn
	"Salas de chat";	// Okoteve traduccin
#elif L==7	// it
	"Chat rooms";
#elif L==8	// pl
	"Czat";
#elif L==9	// pt
	"Chat rooms";
#endif

const char *Txt_Check =
#if   L==1	// ca
	"Comprovar";
#elif L==2	// de
	"&Uuml;berpr&uuml;fen";
#elif L==3	// en
	"Check";
#elif L==4	// es
	"Comprobar";
#elif L==5	// fr
	"V&eacute;rifier";
#elif L==6	// gn
	"Comprobar";	// Okoteve traduccin
#elif L==7	// it
	"Verificare";
#elif L==8	// pl
	"Sprawd&zacute;";
#elif L==9	// pt
	"Verificar";
#endif

const char *Txt_Check_another_ID =
#if   L==1	// ca
	"Comprovar altre ID?";
#elif L==2	// de
	"&Uuml;berpr&uuml;fen Sie eine andere ID?";
#elif L==3	// en
	"Check another ID?";
#elif L==4	// es
	"&iquest;Comprobar otro ID?";
#elif L==5	// fr
	"V&eacute;rifier un autre ID?";
#elif L==6	// gn
	"&iquest;Comprobar otro ID?";	// Okoteve traduccin
#elif L==7	// it
	"Verificare un altro ID?";
#elif L==8	// pl
	"Sprawdzi&cacute; inny identyfikator?";
#elif L==9	// pt
	"Verificar outra ID?";
#endif

const char *Txt_Check_marks_in_file_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Consultar qualificacions en el fitxer %s";
#elif L==2	// de
	"Bewertung in Datei %s abrufen";
#elif L==3	// en
	"Check marks in file %s";
#elif L==4	// es
	"Consultar calificaciones en el archivo %s";
#elif L==5	// fr
	"Check marks in file %s";			// Besoin de traduction
#elif L==6	// gn
	"Consultar calificaciones en el archivo %s";	// Okoteve traduccin
#elif L==7	// it
	"Consultare i risultati nel file %s";
#elif L==8	// pl
	"Zaznaczenia w pliku %s";
#elif L==9	// pt
	"Verifique notas no arquivo %s";
#endif

const char *Txt_Classroom =
#if   L==1	// ca
	"Aula";
#elif L==2	// de
	"Klassenzimmer";
#elif L==3	// en
	"Classroom";
#elif L==4	// es
	"Aula";
#elif L==5	// fr
	"Salle de classe";
#elif L==6	// gn
	"Mbo'ehakoty";
#elif L==7	// it
	"Aula";
#elif L==8	// pl
	"Klasa";
#elif L==9	// pt
	"Sala de aula";
#endif

const char *Txt_Classroom_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Aula <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Klassenzimmer <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Classroom <strong>%s</strong> removed.";
#elif L==4	// es
	"Aula <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Salle de classe <strong>%s</strong> supprim&eacute;e.";
#elif L==6	// gn
	"Aula <strong>%s</strong> eliminada.";		// Okoteve traduccin
#elif L==7	// it
	"Aula <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Klasa <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Sala de aula <strong>%s</strong> removida.";
#endif

const char *Txt_CLASSROOMS_HELP_ORDER[Cla_NUM_ORDERS] =
   {		// Cla_ORDER_BY_SHRT_NAME
#if   L==1	// ca
	 "Ordenar per nom breu"
#elif L==2	// de
	 "Nach Kurzname sortieren"
#elif L==3	// en
	 "Sort by short name"
#elif L==4	// es
	 "Ordenar por nombre breve"
#elif L==5	// fr
	 "Trier par nom abr&eacute;g&eacute;"
#elif L==6	// gn
	 "Ordenar por nombre breve"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per nome breve"
#elif L==8	// pl
	 "Sortuj wed&lstrok;ug kr&oacute;tkiej nazwy"
#elif L==9	// pt
	 "Classificar por nome abreviado"
#endif
	 ,	// Cla_ORDER_BY_FULL_NAME
#if   L==1	// ca
	 "Ordenar per nom complet"
#elif L==2	// de
	 "Nach vollst&auml;ndiger Name sortieren"
#elif L==3	// en
	 "Sort by full name"
#elif L==4	// es
	 "Ordenar por nombre completo"
#elif L==5	// fr
	 "Trier par nom complet"
#elif L==6	// gn
	 "Ordenar por nombre completo"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per nome completo"
#elif L==8	// pl
	 "Sortuj wed&lstrok;ug pe&lstrok;na nazwa"
#elif L==9	// pt
	 "Classificar por nome completo"
#endif
	 ,	// Cla_ORDER_BY_CAPACITY
#if   L==1	// ca
	 "Ordenar per aforo"
#elif L==2	// de
	 "Sortieren nach Sitzplatzkapazit&auml;t"
#elif L==3	// en
	 "Sort by seating capacity"
#elif L==4	// es
	 "Ordenar por aforo"
#elif L==5	// fr
	 "Trier par nombre de places"
#elif L==6	// gn
	 "Ordenar por aforo"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per capacit&agrave; di posti"
#elif L==8	// pl
	 "Sortuj wed&lsgtrok;ug pojemno&sacute;ci miejsc"
#elif L==9	// pt
	 "Ordenar por capacidade"
#endif
	 ,	// Cla_ORDER_BY_LOCATION
#if   L==1	// ca
	 "Ordenar per ubicaci&oacute;"
#elif L==2	// de
	 "Nach Standort sortieren"
#elif L==3	// en
	 "Sort by location"
#elif L==4	// es
	 "Ordenar por ubicaci&oacute;n"
#elif L==5	// fr
	 "Trier par emplacement"
#elif L==6	// gn
	 "Ordenar por ubicaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per posizione"
#elif L==8	// pl
	 "Sortuj wed&lstrok;ug lokacja"
#elif L==9	// pt
	 "Classificar por localiza&ccedil;&atilde;o"
#endif
	};

const char *Txt_CLASSROOMS_ORDER[Cla_NUM_ORDERS] =
   {		// Cla_ORDER_BY_SHRT_NAME
#if   L==1	// ca
	 "Nom breu"
#elif L==2	// de
	 "Kurzname"
#elif L==3	// en
	 "Short name"
#elif L==4	// es
	 "Nombre breve"
#elif L==5	// fr
	 "Nom abr&eacute;g&eacute;"
#elif L==6	// gn
	 "Nombre breve"	// Okoteve traduccin
#elif L==7	// it
	 "Nome breve"
#elif L==8	// pl
	 "Kr&oacute;tkiej nazwy"
#elif L==9	// pt
	 "Nome abreviado"
#endif
	 ,	// Cla_ORDER_BY_FULL_NAME
#if   L==1	// ca
	 "Nom complet"
#elif L==2	// de
	 "Vollst&auml;ndiger Name"
#elif L==3	// en
	 "Full name"
#elif L==4	// es
	 "Nombre completo"
#elif L==5	// fr
	 "Nom complet"
#elif L==6	// gn
	 "Nombre completo"	// Okoteve traduccin
#elif L==7	// it
	 "Nome completo"
#elif L==8	// pl
	 "Pe&lstrok;na nazwa"
#elif L==9	// pt
	 "Nome completo"
#endif
	 ,	// Cla_ORDER_BY_CAPACITY
#if   L==1	// ca
	"Aforo"
#elif L==2	// de
	"Kapazit&auml;t"
#elif L==3	// en
	"Capacity"
#elif L==4	// es
	"Aforo"
#elif L==5	// fr
	"Capacit&eacute;"
#elif L==6	// gn
	"Aforo"				// Okoteve traduccin
#elif L==7	// it
	"Capacit&agrave;"
#elif L==8	// pl
	"Pojemno&sacute;&cacute;"
#elif L==9	// pt
	"Capacidade"
#endif
	 ,	// // Cla_ORDER_BY_LOCATION
#if   L==1	// ca
	"Ubicaci&oacute;"
#elif L==2	// de
	"Standort"
#elif L==3	// en
	"Location"
#elif L==4	// es
	"Ubicaci&oacute;n"
#elif L==5	// fr
	"Emplacement"
#elif L==6	// gn
	"&Ntilde;emohenda"
#elif L==7	// it
	"Posizione"
#elif L==8	// pl
	"Lokacja"
#elif L==9	// pt
	"Localiza&ccedil;&atilde;o"
#endif
	};

const char *Txt_Classrooms =
#if   L==1	// ca
	"Aules";
#elif L==2	// de
	"Klassenzimmer";
#elif L==3	// en
	"Classrooms";
#elif L==4	// es
	"Aulas";
#elif L==5	// fr
	"Salles de classe";
#elif L==6	// gn
	"Mbo'ehakoty";
#elif L==7	// it
	"Aule";
#elif L==8	// pl
	"Klasy";
#elif L==9	// pt
	"Salas de aula";
#endif

const char *Txt_Click =
#if   L==1	// ca
	"Clic";
#elif L==2	// de
	"Klick";
#elif L==3	// en
	"Click";
#elif L==4	// es
	"Clic";
#elif L==5	// fr
	"Clic";
#elif L==6	// gn
	"Clic";	// Okoteve traduccin
#elif L==7	// it
	"Clic";
#elif L==8	// pl
	"Kliknij";
#elif L==9	// pt
	"Clique";
#endif

const char *Txt_Clicks =
#if   L==1	// ca
	"Clics";
#elif L==2	// de
	"Klicks";
#elif L==3	// en
	"Clicks";
#elif L==4	// es
	"Clics";
#elif L==5	// fr
	"Clics";
#elif L==6	// gn
	"Clics";	// Okoteve traduccin
#elif L==7	// it
	"Clic";
#elif L==8	// pl
	"Klikni&eogon;cia";
#elif L==9	// pt
	"Cliques";
#endif

const char *Txt_clicks =
#if   L==1	// ca
	"clics";
#elif L==2	// de
	"Klicks";
#elif L==3	// en
	"clicks";
#elif L==4	// es
	"clics";
#elif L==5	// fr
	"clics";
#elif L==6	// gn
	"clics";	// Okoteve traduccin
#elif L==7	// it
	"clic";
#elif L==8	// pl
	"klikni&eogon;cia";
#elif L==9	// pt
	"cliques";
#endif

const char *Txt_Clicks_per_day =
#if   L==1	// ca
	"Clics per dia";
#elif L==2	// de
	"Klicks pro Tag";
#elif L==3	// en
	"Clicks per day";
#elif L==4	// es
	"Clics por d&iacute;a";
#elif L==5	// fr
	"Clics par jour";
#elif L==6	// gn
	"Clics por d&iacute;a";	// Okoteve traduccin
#elif L==7	// it
	"Clic al giorno";
#elif L==8	// pl
	"Klikni&eogon;&cacute; dziennie";
#elif L==9	// pt
	"Cliques por dia";
#endif

const char *Txt_Close =
#if   L==1	// ca
	"Tancar";
#elif L==2	// de
	"Schlie&szlig;en";
#elif L==3	// en
	"Close";
#elif L==4	// es
	"Cerrar";
#elif L==5	// fr
	"Fermer";
#elif L==6	// gn
	"S&#7869;";
#elif L==7	// it
	"Chiudi";
#elif L==8	// pl
	"Zamknij";
#elif L==9	// pt
	"Fechar";
#endif

const char *Txt_Closed_game =
#if   L==1	// ca
	"Joc tancat";
#elif L==2	// de
	"Geschlossene Spiel";
#elif L==3	// en
	"Closed game";
#elif L==4	// es
	"Juego cerrado";
#elif L==5	// fr
	"Jeu ferm&eacute;";
#elif L==6	// gn
	"Juego cerrado";		// Okoteve traduccin
#elif L==7	// it
	"Gioco chiuso";
#elif L==8	// pl
	"Zamkniete gra";
#elif L==9	// pt
	"Jogo fechado";
#endif

const char *Txt_Closed_survey =
#if   L==1	// ca
	"Enquesta tancada";
#elif L==2	// de
	"Geschlossene Umfrage";
#elif L==3	// en
	"Closed survey";
#elif L==4	// es
	"Encuesta cerrada";
#elif L==5	// fr
	"Sondage ferm&eacute;";
#elif L==6	// gn
	"Encuesta cerrada";		// Okoteve traduccin
#elif L==7	// it
	"Sondaggio chiuso";
#elif L==8	// pl
	"Zamkniete badania";
#elif L==9	// pt
	"Inqu&eacute;rito fechado";
#endif

const char *Txt_Code =
#if   L==1	// ca
	"Codi";
#elif L==2	// de
	"Code";
#elif L==3	// en
	"Code";
#elif L==4	// es
	"C&oacute;d.";
#elif L==5	// fr
	"Code";
#elif L==6	// gn
	"Papanga";
#elif L==7	// it
	"Cod.";
#elif L==8	// pl
	"Kod";
#elif L==9	// pt
	"C&oacute;d.";
#endif

const char *Txt_Color_of_the_graphic =
#if   L==1	// ca
	"Color de la gr&agrave;fica";
#elif L==2	// de
	"Farbe der Grafik";
#elif L==3	// en
	"Color of the graphic";
#elif L==4	// es
	"Color de la gr&aacute;fica";
#elif L==5	// fr
	"Couleur de l'image";
#elif L==6	// gn
	"Color de la gr&aacute;fica";	// Okoteve traduccin
#elif L==7	// it
	"Colore del grafico";
#elif L==8	// pl
	"Kolor grafiki";
#elif L==9	// pt
	"Cor do gr&aacute;fico";
#endif

const char *Txt_Columns =
#if   L==1	// ca
	"Columnes";
#elif L==2	// de
	"Spalten";
#elif L==3	// en
	"Columns";
#elif L==4	// es
	"Columnas";
#elif L==5	// fr
	"Colonnes";
#elif L==6	// gn
	"Columnas";	// Okoteve traduccin
#elif L==7	// it
	"Colonne";
#elif L==8	// pl
	"Kolumny";
#elif L==9	// pt
	"Colunas";
#endif

const char *Txt_columns =
#if   L==1	// ca
	"columnes";
#elif L==2	// de
	"Spalten";
#elif L==3	// en
	"columns";
#elif L==4	// es
	"columnas";
#elif L==5	// fr
	"colonnes";
#elif L==6	// gn
	"columnas";	// Okoteve traduccin
#elif L==7	// it
	"colonne";
#elif L==8	// pl
	"kolumny";
#elif L==9	// pt
	"colunas";
#endif

const char *Txt_Comment =	// As a verb
#if   L==1	// ca
	"Comentar";
#elif L==2	// de
	"Kommentieren";
#elif L==3	// en
	"Comment";
#elif L==4	// es
	"Comentar";
#elif L==5	// fr
	"Commenter";
#elif L==6	// gn
	"Comentar";	// Okoteve traduccin
#elif L==7	// it
	"Commentare";
#elif L==8	// pl
	"Komentowa&cacute;";
#elif L==9	// pt
	"Comentar";
#endif

const char *Txt_Comment_removed =
#if   L==1	// ca
	"Comentari eliminat.";
#elif L==2	// de
	"Kommentar entfernt.";
#elif L==3	// en
	"Comment removed.";
#elif L==4	// es
	"Comentario eliminado.";
#elif L==5	// fr
	"Commentaire supprim&eacute;.";
#elif L==6	// gn
	"Comentario eliminado.";		// Okoteve traduccin
#elif L==7	// it
	"Commento rimosso.";
#elif L==8	// pl
	"Komentarz usuniety.";
#elif L==9	// pt
	"Coment&aacute;rio removido.";
#endif

const char *Txt_Comments =
#if   L==1	// ca
	"Comentaris";
#elif L==2	// de
	"Kommentare";
#elif L==3	// en
	"Comments";
#elif L==4	// es
	"Comentarios";
#elif L==5	// fr
	"Commentaires";
#elif L==6	// gn
	"Comentarios";	// Okoteve traduccin
#elif L==7	// it
	"Commenti";
#elif L==8	// pl
	"Komentarze";
#elif L==9	// pt
	"Coment&aacute;rios";
#endif

const char *Txt_Teachers_comment =
#if   L==1	// ca
	"Comentari de professors";
#elif L==2	// de
	"Lehrern-Kommentar";
#elif L==3	// en
	"Teachers' comment";
#elif L==4	// es
	"Comentario de profesores";
#elif L==5	// fr
	"Commentaire des enseignants";
#elif L==6	// gn
	"Comentario de profesores";	// Okoteve traduccin
#elif L==7	// it
	"Commento degli professori";
#elif L==8	// pl
	"Komentarz nauczycieli";
#elif L==9	// pt
	"Coment&aacute;rio dos professores";
#endif

const char *Txt_Configure =
#if   L==1	// ca
	"Configurar";
#elif L==2	// de
	"Konfigurieren";
#elif L==3	// en
	"Configure";
#elif L==4	// es
	"Configurar";
#elif L==5	// fr
	"Configurer";
#elif L==6	// gn
	"Configurar";	// Okoteve traduccin
#elif L==7	// it
	"Configurare";
#elif L==8	// pl
	"Konfiguracja";
#elif L==9	// pt
	"Configurar";
#endif

const char *Txt_Configure_tests =
#if   L==1	// ca
	"Configurar tests";
#elif L==2	// de
	"Konfigurieren Tests";
#elif L==3	// en
	"Configure tests";
#elif L==4	// es
	"Configurar test";
#elif L==5	// fr
	"Configurer tests";
#elif L==6	// gn
	"Configurar test";	// Okoteve traduccin
#elif L==7	// it
	"Configurare test";
#elif L==8	// pl
	"Test Konfiguracja";
#elif L==9	// pt
	"Configurar testes";
#endif

const char *Txt_Confirm =
#if   L==1	// ca
	"Confirmar";
#elif L==2	// de
	"Best&auml;tigen";
#elif L==3	// en
	"Confirm";
#elif L==4	// es
	"Confirmar";
#elif L==5	// fr
	"Confirmer";
#elif L==6	// gn
	"Confirmar";	// Okoteve traduccin
#elif L==7	// it
	"Confermare";
#elif L==8	// pl
	"Potwierdzenie";
#elif L==9	// pt
	"Confirmar";
#endif

const char *Txt_Confirm_email =
#if   L==1	// ca
	"Confirmar correu";
#elif L==2	// de
	"E-Mail best&auml;tigen";
#elif L==3	// en
	"Confirm email";
#elif L==4	// es
	"Confirmar correo";
#elif L==5	// fr
	"Confirmer courrier";
#elif L==6	// gn
	"Confirmar correo";	// Okoteve traduccin
#elif L==7	// it
	"Confermare email";
#elif L==8	// pl
	"Potwierdzenie email";
#elif L==9	// pt
	"Confirmar email";
#endif

const char *Txt_Confirm_ID =
#if   L==1	// ca
	"Confirmar ID";
#elif L==2	// de
	"ID best&auml;tigen";
#elif L==3	// en
	"Confirm ID";
#elif L==4	// es
	"Confirmar ID";
#elif L==5	// fr
	"Confirmer ID";
#elif L==6	// gn
	"Confirmar ID";	// Okoteve traduccin
#elif L==7	// it
	"Confermare ID";
#elif L==8	// pl
	"Potwierdzenie ID";
#elif L==9	// pt
	"Confirmar ID";
#endif

const char *Txt_Confirm_my_enrolment =
#if   L==1	// ca
	"Confirmar la meva inscripci&oacute;";
#elif L==2	// de
	"Best&auml;tigen Sie meine Registrierung";
#elif L==3	// en
	"Confirm my enrolment";
#elif L==4	// es
	"Confirmar mi inscripci&oacute;n";
#elif L==5	// fr
	"Confirmer mon inscription";
#elif L==6	// gn
	"Confirmar mi inscripci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Confermare la mia iscrizione";
#elif L==8	// pl
	"Potwierdzenie mojej rejestracji";
#elif L==9	// pt
	"Confirmar o meu registo";
#endif

const char *Txt_Congratulations_You_have_created_your_account_X_Now_Y_will_request_you_ =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Enhorabona! Ha creat el seu compte @%s."
	" Ara %s li demanar&agrave; algunes dades per completar el seu perfil."
	" Despr&eacute;s podr&agrave; sol&middot;licitar la seva inscripci&oacute; en assignatures."
	" Si no hi ha una assignatura,"
	" vost&egrave; pot crear abans de sol&middot;licitar la seva inscripci&oacute; en ella"
	" (el mateix &eacute;s aplicable a institucions, centres i titulacions).";
#elif L==2	// de
	"Herzlichen Gl&uuml;ckwunsch! Du hast Dein Benutzerkonto <strong>@%s</strong> erstellt."
	" Jetzt %s finden Sie einige Daten in Ihrem Profil."
	" Vervollst&auml;ndigen anfordern und dann werden Sie in der Lage,"
	" f&uuml;r die Einschreibung in Kurse an."
	" Wenn ein Kurs nicht vorhanden ist,"
	" k&ouml;nnen Sie es vor der Anwendung f&uuml;r die Immatrikulation in es zu erstellen"
	" (das gleiche gilt f&uuml;r Einrichtungen, Zentren und Grad).";
#elif L==3	// en
	"Congratulations! You have created your account <strong>@%s</strong>."
	" Now %s will request you some data to complete your profile."
	" Then you will be able to apply for enrolment in courses."
	" If a course does not exist,"
	" you can create it before applying for enrolment in it"
	" (the same applies to institutions, centres and degrees).";
#elif L==4	// es
	"&iexcl;Enhorabuena! Ha creado su cuenta <strong>@%s</strong>."
	" Ahora %s le solicitar&aacute; algunos datos para completar su perfil."
	" Usted podr&aacute; despu&eacute;s solicitar su inscripci&oacute;n en asignaturas."
	" Si una asignatura no existe,"
	" puede crearla antes de solicitar su inscripci&oacute;n en ella"
	" (lo mismo es aplicable a instituciones, centros y titulaciones).";
#elif L==5	// fr
	"F&eacute;licitations! Vous avez cr&eacute;&eacute; votre compte <strong>@%s</strong>."
	" Maintenant %s vous demander&aacute; certaines donn&eacute;es pour compl&eacute;ter votre profil."
	" Vous serez alors en mesure de demander l'inscription &agrave; des mati&egrave;res."
	" Si une mati&egrave;re ne existe pas,"
	" vous pouvez le cr&eacute;er avant de demander l'inscription en elle"
	" (la m&ecirc;me chose se applique aux institutions, centres et degr&eacute;s).";
#elif L==6	// gn
	"&iexcl;Enhorabuena! Ha creado su cuenta <strong>@%s</strong>."
	" Ahora %s le solicitar&aacute; algunos datos para completar su perfil."
	" Usted podr&aacute; despu&eacute;s solicitar su inscripci&oacute;n en asignaturas."
	" Si una asignatura no existe,"
	" puede crearla antes de solicitar su inscripci&oacute;n en ella"
	" (lo mismo es aplicable a instituciones, centros y titulaciones).";	// Okoteve traduccin
#elif L==7	// it
	"Congratulazioni! Avete creato il vostro account <strong>@%s</strong>."
	" Ora %s chieder&agrave; alcuni dati per completare il tuo profilo."
	" Allora si sar&agrave; in grado di applicare per l'iscrizione ai corsi."
	" Se un corso non esiste, &egrave; possibile crearlo prima di applicare per l'iscrizione in esso"
	" (lo stesso vale per le istituzioni, centri e gradi).";
#elif L==8	// pl
	"Gratulujemy! Utworzeniu konta <strong>@%s</strong>."
	" Teraz %s zwr&oacute;ci Ci kilka danych, aby zako&nacute;czy&cacute; sw&oacute;j profil."
	" Nast&eogon;pnie b&eogon;dzie mo&zdot;na ubiega&cacute; si&eogon; o przyj&eogon;cie na kursy."
	" Je&sacute;li kurs nie istnieje,"
	" mo&zdot;na go utworzy&cacute; przed ubieganiem si&egrave; o przyj&eogon;cie w nim"
	" (to samo odnosi si&eogon; do instytucji, plac&oacute;wek i stopni).";
#elif L==9	// pt
	"Parab&eacute;ns! Voc&ecirc; criou seu conta <strong>@%s</strong>."
	" Agora %s vai pedir-lhe alguns dados para completar o seu perfil."
	" Em seguida, voc&ecirc; ser&aacute; capaz de aplicar para a matrcula em cursos."
	" Se uma disciplina n&atilde;o existir,"
	" voc&ecirc; pode cri&aacute;-lo antes de solicitar a inscri&ccedil;&atilde;o do mesmo"
	" (o mesmo se aplica &agrave;s institui&ccedil;&otilde;es, centros e graus).";
#endif

const char *Txt_connected_PLURAL =
#if   L==1	// ca
	"connectats";
#elif L==2	// de
	"angemeldet";
#elif L==3	// en
	"connected";
#elif L==4	// es
	"conectados";
#elif L==5	// fr
	"connect&eacute;s";
#elif L==6	// gn
	"conectados";	// Okoteve traduccin
#elif L==7	// it
	"connessi";
#elif L==8	// pl
	"online";
#elif L==9	// pt
	"conectados";
#endif

const char *Txt_connected_SINGULAR =
#if   L==1	// ca
	"connectat";
#elif L==2	// de
	"angemeldet";
#elif L==3	// en
	"connected";
#elif L==4	// es
	"conectado";
#elif L==5	// fr
	"connect&eacute;";
#elif L==6	// gn
	"conectado";	// Okoteve traduccin
#elif L==7	// it
	"connesso";
#elif L==8	// pl
	"online";
#elif L==9	// pt
	"conectado";
#endif

const char *Txt_Connected_users =
#if   L==1	// ca
	"Usuaris connectats";
#elif L==2	// de
	"Angemeldet Benutzer";
#elif L==3	// en
	"Connected users";
#elif L==4	// es
	"Usuarios conectados";
#elif L==5	// fr
	"Utilisateurs connect&eacute;s";
#elif L==6	// gn
	"Usuarios conectados";	// Okoteve traduccin
#elif L==7	// it
	"Utenti connessi";
#elif L==8	// pl
	"U&zdot;ytkownicy online";
#elif L==9	// pt
	"Utilizadores conectados";
#endif

const char *Txt_Continue =
#if   L==1	// ca
	"Continuar";
#elif L==2	// de
	"Weiter";
#elif L==3	// en
	"Continue";
#elif L==4	// es
	"Continuar";
#elif L==5	// fr
	"Continuer";
#elif L==6	// gn
	"Hemo&ntilde;a";
#elif L==7	// it
	"Continua";
#elif L==8	// pl
	"Kontynuuj";
#elif L==9	// pt
	"Continuar";
#endif

const char *Txt_Contract =	// As a verb
#if   L==1	// ca
	"Contreure";
#elif L==2	// de
	"Zusammenziehen";
#elif L==3	// en
	"Contract";
#elif L==4	// es
	"Contraer";
#elif L==5	// fr
	"Contracter";
#elif L==6	// gn
	"Contraer";	// Okoteve traduccin
#elif L==7	// it
	"Contratto";
#elif L==8	// pl
	"Umowy";
#elif L==9	// pt
	"Contrair";
#endif

const char *Txt_Control_of_class_attendance =
#if   L==1	// ca
	"Control d'assistencia a classe";
#elif L==2	// de
	"Anwesenheitskontrolle";
#elif L==3	// en
	"Control of class attendance";
#elif L==4	// es
	"Control de asistencia a clase";
#elif L==5	// fr
	"Contr&ouml;le de la fr&eacute;quentation scolaire";
#elif L==6	// gn
	"Control de asistencia a clase";	// Okoteve traduccin
#elif L==7	// it
	"Controllo delle presenze in classe";
#elif L==8	// pl
	"Kontrola obecnosc na zajeciach";
#elif L==9	// pt
	"Controlo de assiduidade de classe";
#endif

const char *Txt_Cookies =
#if   L==1	// ca
	"Cookies";
#elif L==2	// de
	"Cookies";
#elif L==3	// en
	"Cookies";
#elif L==4	// es
	"Cookies";
#elif L==5	// fr
	"Cookies";
#elif L==6	// gn
	"Cookies";
#elif L==7	// it
	"Cookies";
#elif L==8	// pl
	"Cookies";
#elif L==9	// pt
	"Cookies";
#endif

const char *Txt_Copy_FOLDER_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Copiar %s";
#elif L==2	// de
	"Kopieren %s";
#elif L==3	// en
	"Copy %s";
#elif L==4	// es
	"Copiar %s";
#elif L==5	// fr
	"Copier %s";
#elif L==6	// gn
	"Ekopia %s";
#elif L==7	// it
	"Copiare %s";
#elif L==8	// pl
	"Kopia %s";
#elif L==9	// pt
	"Copiar %s";
#endif

const char *Txt_Copy_not_allowed =
#if   L==1	// ca
	"C&ograve;pia no perm&egrave;s";
#elif L==2	// de
	"Kopier nicht erlaubt";
#elif L==3	// en
	"Copy not allowed";
#elif L==4	// es
	"Copia no permitida";
#elif L==5	// fr
	"Copie pas autoris&eacute;e";
#elif L==6	// gn
	"Copia no permitida";	// Okoteve traduccin
#elif L==7	// it
	"Copia non ammessi";
#elif L==8	// pl
	"Kopiowanie nie wolno";
#elif L==9	// pt
	"Copia n&atilde;o permitida";
#endif

const char *Txt_Copy_source =
#if   L==1	// ca
	"Origen de c&ograve;pia";
#elif L==2	// de
	"Kopier-Quelle";
#elif L==3	// en
	"Copy source";
#elif L==4	// es
	"Origen de copia";
#elif L==5	// fr
	"Source de copie";
#elif L==6	// gn
	"Origen de copia";	// Okoteve traduccin
#elif L==7	// it
	"Copia sorgente";
#elif L==8	// pl
	"Kopiowanie zr&oacute;dla";
#elif L==9	// pt
	"Origem da copia";
#endif

const char *Txt_Could_not_detect_any_face_in_front_position_ =
#if   L==1	// ca
	"No s'ha detectat cap rostre en posici&oacute; frontal.";
#elif L==2	// de
	"Could not detect any face in front position.";				// Need bersetzung
#elif L==3	// en
	"Could not detect any face in front position.";
#elif L==4	// es
	"No se ha detectado ning&uacute;n rostro en posici&oacute;n frontal.";
#elif L==5	// fr
	"Could not detect any face in front position.";				// Besoin de traduction
#elif L==6	// gn
	"No se ha detectado ning&uacute;n rostro en posici&oacute;n frontal.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi rilevare qualsiasi viso in posizione frontale.";
#elif L==8	// pl
	"Nie wykryto zadnej twarzy w przednim.";
#elif L==9	// pt
	"N&atilde;o foi poss&iacute;vel detectar nenhum rosto na posi&ccedil;&atilde;o frontal.";
#endif

const char *Txt_Countries =
#if   L==1	// ca
	"Pa&iuml;sos";
#elif L==2	// de
	"L&auml;nder";
#elif L==3	// en
	"Countries";
#elif L==4	// es
	"Pa&iacute;ses";
#elif L==5	// fr
	"Pays";
#elif L==6	// gn
	"Tet&atilde;";
#elif L==7	// it
	"Paesi";
#elif L==8	// pl
	"Kraje";
#elif L==9	// pt
	"Pa&iacute;ses";
#endif

const char *Txt_countries =
#if   L==1	// ca
	"pa&iuml;sos";
#elif L==2	// de
	"L&auml;nder";
#elif L==3	// en
	"countries";
#elif L==4	// es
	"pa&iacute;ses";
#elif L==5	// fr
	"pays";
#elif L==6	// gn
	"tet&atilde;";
#elif L==7	// it
	"paesi";
#elif L==8	// pl
	"kraje";
#elif L==9	// pt
	"pa&iacute;ses";
#endif

const char *Txt_COUNTRIES_HELP_ORDER[Cty_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Ordenar por pa&iacute;s"	// Necessita traduccio
#elif L==2	// de
	 "Nach Land sortieren"
#elif L==3	// en
	 "Sort by country"
#elif L==4	// es
	 "Ordenar por pa&iacute;s"
#elif L==5	// fr
	 "Trier par pays"
#elif L==6	// gn
	 "Ordenar por pa&iacute;s"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per paese"
#elif L==8	// pl
	 "Sortuj wedlug kraju"
#elif L==9	// pt
	 "Classificar por pa&iacute;s"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar per n&ordm; d'usuaris"
#elif L==2	// de
	 "Nach Anzahl der Benutzer sortieren"
#elif L==3	// en
	 "Sort by no. of users"
#elif L==4	// es
	 "Ordenar por n&ordm; de usuarios"
#elif L==5	// fr
	 "Trier par nombre d'utilisateurs"
#elif L==6	// gn
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per numero di utenti"
#elif L==8	// pl
	 "Sortuj wedlug liczby uzytkownikow"
#elif L==9	// pt
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_Countries_with_pending_institutions =
#if   L==1	// ca
	"Pa&iuml;sos amb institucions pendents d'activar";
#elif L==2	// de
	"L&auml;nder mit anh&auml;ngig Hochschulen";
#elif L==3	// en
	"Countries with pending institutions";
#elif L==4	// es
	"Pa&iacute;ses con instituciones pendientes de activar";
#elif L==5	// fr
	"Pays avec des &eacute;tablissements en attente";
#elif L==6	// gn
	"Pa&iacute;ses con instituciones pendientes de activar";	// Okoteve traduccin
#elif L==7	// it
	"Paesi con istituzioni in sospeso";
#elif L==8	// pl
	"Kraje z instytucje oczekuj&aogon;cy";
#elif L==9	// pt
	"Pa&iacute;ses com institu&ccedil;&otilde;es pendentes";
#endif

const char *Txt_COUNTRIES_ORDER[Cty_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Pa&iacute;s"
#elif L==2	// de
	 "Land"
#elif L==3	// en
	 "Country"
#elif L==4	// es
	 "Pa&iacute;s"
#elif L==5	// fr
	 "Pays"
#elif L==6	// gn
	 "Tet&atilde;"
#elif L==7	// it
	 "Paese"
#elif L==8	// pl
	 "Kraj"
#elif L==9	// pt
	 "Pa&iacute;s"
#endif
	 ,
#if   L==1	// ca
	 "Usuaris del pa&iacute;s"
#elif L==2	// de
	 "Benutzer des Landes"
#elif L==3	// en
	 "Users of the country"
#elif L==4	// es
	 "Usuarios del pa&iacute;s"
#elif L==5	// fr
	 "Utilisateurs du pays"
#elif L==6	// gn
	 "Usuarios del pa&iacute;s"	// Okoteve traduccin
#elif L==7	// it
	 "Utenti del paese"
#elif L==8	// pl
	 "U&zdot;ytkownicy kraju"
#elif L==9	// pt
	 "Utilizadores do pa&iacute;s"
#endif
	};

const char *Txt_Country =
#if   L==1	// ca
	"Pa&iacute;s";
#elif L==2	// de
	"Land";
#elif L==3	// en
	"Country";
#elif L==4	// es
	"Pa&iacute;s";
#elif L==5	// fr
	"Pays;";
#elif L==6	// gn
	"Tet&atilde;";
#elif L==7	// it
	"Paese";
#elif L==8	// pl
	"Kraj";
#elif L==9	// pt
	"Pa&iacute;s";
#endif

const char *Txt_country =
#if   L==1	// ca
	"pa&iacute;s";
#elif L==2	// de
	"Land";
#elif L==3	// en
	"country";
#elif L==4	// es
	"pa&iacute;s";
#elif L==5	// fr
	"pays;";
#elif L==6	// gn
	"tet&atilde;";
#elif L==7	// it
	"paese";
#elif L==8	// pl
	"kraj";
#elif L==9	// pt
	"pa&iacute;s";
#endif

const char *Txt_Country_unspecified =
#if   L==1	// ca
	"Pa&iacute;s sense especificar";
#elif L==2	// de
	"Land nicht angegeben";
#elif L==3	// en
	"Country unspecified";
#elif L==4	// es
	"Pa&iacute;s sin especificar";
#elif L==5	// fr
	"Pays non sp&eacute;cifi&eacute;";
#elif L==6	// gn
	"Pa&iacute;s sin especificar";	// Okoteve traduccin
#elif L==7	// it
	"Paese non specificato";
#elif L==8	// pl
	"Kraj nieokreslony";
#elif L==9	// pt
	"Pa&iacute;s indeterminado";
#endif

const char *Txt_Country_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Pa&iacute;s <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2	// de
	"Land <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Country <strong>%s</strong> removed.";
#elif L==4	// es
	"Pa&iacute;s <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Pays <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6	// gn
	"Pa&iacute;s <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Paese <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Kraj <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Pa&iacute;s <strong>%s</strong> removido.";
#endif

const char *Txt_Course =
#if   L==1	// ca
	"Assignatura";
#elif L==2	// de
	"Kurs";
#elif L==3	// en
	"Course";
#elif L==4	// es
	"Asignatura";
#elif L==5	// fr
	"Mati&egrave;re";
#elif L==6	// gn
	"Mbo'esyry";
#elif L==7	// it
	"Corso";
#elif L==8	// pl
	"Kurs";
#elif L==9	// pt
	"Disciplina";
#endif

const char *Txt_course =
#if   L==1	// ca
	"assignatura";
#elif L==2	// de
	"Kurs";
#elif L==3	// en
	"course";
#elif L==4	// es
	"asignatura";
#elif L==5	// fr
	"mati&egrave;re";
#elif L==6	// gn
	"mbo'esyry";
#elif L==7	// it
	"corso";
#elif L==8	// pl
	"kurs";
#elif L==9	// pt
	"disciplina";
#endif

const char *Txt_COURSE_STATUS[Crs_NUM_STATUS_TXT] =
   {
#if   L==1	// ca	// Crs_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==2	// de
	 "Unbekannt"
#elif L==3	// en
	 "Unknown"
#elif L==4	// es
	 "Desconocido"
#elif L==5	// fr
	 "Inconnu"
#elif L==6	// gn
	 "Desconocido"			// Okoteve traduccin
#elif L==7	// it
	 "Sconosciuto"
#elif L==8	// pl
	 "Nieznany"
#elif L==9	// pt
	 "Desconhecido"
#endif
	 ,
#if   L==1	// ca	// Crs_STATUS_ACTIVE
	 "Activa"
#elif L==2	// de
	 "Aktive"
#elif L==3	// en
	 "Active"
#elif L==4	// es
	 "Activa"
#elif L==5	// fr
	 "Actif"
#elif L==6	// gn
	 "Activa"	// Okoteve traduccin
#elif L==7	// it
	 "Attivo"
#elif L==8	// pl
	 "Aktywne"
#elif L==9	// pt
	 "Ativa"
#endif
	 ,
#if   L==1	// ca	// Crs_STATUS_PENDING
	 "Nova"
#elif L==2	// de
	 "Neu"
#elif L==3	// en
	 "New"
#elif L==4	// es
	 "Nueva"
#elif L==5	// fr
	 "Nouvelle"
#elif L==6	// gn
	 "Pyahu"
#elif L==7	// it
	 "Nuovo"
#elif L==8	// pl
	 "Nowy"
#elif L==9	// pt
	 "Nova"
#endif
	 ,
#if   L==1	// ca	// Crs_STATUS_REMOVED
	 "Eliminada"
#elif L==2	// de
	 "Entfernt"
#elif L==3	// en
	 "Removed"
#elif L==4	// es
	 "Eliminada"
#elif L==5	// fr
	 "Supprim&eacute;"
#elif L==6	// gn
	 "Eliminada"	// Okoteve traduccin
#elif L==7	// it
	 "Rimosso"
#elif L==8	// pl
	 "Usuniete"
#elif L==9	// pt
	 "Suprimida"
#endif
	};

const char *Txt_COURSE_With_users =
#if   L==1	// ca
	"Amb usuaris";
#elif L==2	// de
	"Mit Benutzer";
#elif L==3	// en
	"With users";
#elif L==4	// es
	"Con usuarios";
#elif L==5	// fr
	"Avec utilisateurs";
#elif L==6	// gn
	"Con usuarios";	// Okoteve traduccin
#elif L==7	// it
	"Con utenti";
#elif L==8	// pl
	"Z uzytkownikow";
#elif L==9	// pt
	"Com utilizadores";
#endif

const char *Txt_COURSE_Without_users =
#if   L==1	// ca
	"Sense usuaris";
#elif L==2	// de
	"Ohne Benutzer";
#elif L==3	// en
	"Without users";
#elif L==4	// es
	"Sin usuarios";
#elif L==5	// fr
	"Sans utilisateurs";
#elif L==6	// gn
	"Sin usuarios";	// Okoteve traduccin
#elif L==7	// it
	"Senza utenti";
#elif L==8	// pl
	"Bez uzytkownikow";
#elif L==9	// pt
	"Sem utilizadores";
#endif

const char *Txt_Course_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Asignatura <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2	// de
	"Kurs <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Course <strong>%s</strong> removed.";
#elif L==4	// es
	"Asignatura <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Mati&egrave;re <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Asignatura <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Corso <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Kurs <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Disciplina <strong>%s</strong> removida.";
#endif

const char *Txt_Courses =
#if   L==1	// ca
	"Assignatures";
#elif L==2	// de
	"Kursen";
#elif L==3	// en
	"Courses";
#elif L==4	// es
	"Asignaturas";
#elif L==5	// fr
	"Mati&egrave;res";
#elif L==6	// gn
	"Mbo'esyry";
#elif L==7	// it
	"Corsi";
#elif L==8	// pl
	"Kurs&oacute;w";
#elif L==9	// pt
	"Disciplinas";
#endif

const char *Txt_Courses_ABBREVIATION =
#if   L==1	// ca
	"Assig.";
#elif L==2	// de
	"Veran.";
#elif L==3	// en
	"Cours.";
#elif L==4	// es
	"Asig.";
#elif L==5	// fr
	"Mat.";
#elif L==6	// gn
	"Asig.";	// Okoteve traduccin
#elif L==7	// it
	"Corsi";
#elif L==8	// pl
	"Kurs.";
#elif L==9	// pt
	"Disc.";
#endif

const char *Txt_courses =
#if   L==1	// ca
	"assignatures";
#elif L==2	// de
	"Kursen";
#elif L==3	// en
	"courses";
#elif L==4	// es
	"asignaturas";
#elif L==5	// fr
	"mati&egrave;res";
#elif L==6	// gn
	"mbo'esyry";
#elif L==7	// it
	"corsi";
#elif L==8	// pl
	"kurs&oacute;w";
#elif L==9	// pt
	"disciplinas";
#endif

const char *Txt_courses_ABBREVIATION =
#if   L==1	// ca
	"assig.";
#elif L==2	// de
	"Veran.";
#elif L==3	// en
	"cours.";
#elif L==4	// es
	"asig.";
#elif L==5	// fr
	"mat.";
#elif L==6	// gn
	"asig.";	// Okoteve traduccin
#elif L==7	// it
	"corsi";
#elif L==8	// pl
	"kurs.";
#elif L==9	// pt
	"disc.";
#endif

const char *Txt_X_courses_have_been_eliminated =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Se ha(n) eliminado %u asignatura(s).";		// Necessita traduccio
#elif L==2	// de
	"%u course(s) have been eliminated.";		// Need bersetzung
#elif L==3	// en
	"%u course(s) have been eliminated.";
#elif L==4	// es
	"Se ha(n) eliminado %u asignatura(s).";
#elif L==5	// fr
	"%u course(s) have been eliminated.";		// Besoin de traduction
#elif L==6	// gn
	"Se ha(n) eliminado %u asignatura(s).";		// Okoteve traduccin
#elif L==7	// it
	"%u corsi sono stati rimossi.";
#elif L==8	// pl
	"%u course(s) have been eliminated.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"%u disciplina(s) foram eliminadas.";
#endif

const char *Txt_Courses_of_DEGREE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Assignatures de %s";
#elif L==2	// de
	"Kursen der %s";
#elif L==3	// en
	"Courses of %s";
#elif L==4	// es
	"Asignaturas de %s";
#elif L==5	// fr
	"Mati&egrave;res de %s";
#elif L==6	// gn
	"Asignaturas de %s";	// Okoteve traduccin
#elif L==7	// it
	"Corsi di %s";
#elif L==8	// pl
	"Kurs&oacute;w %s";
#elif L==9	// pt
	"Disciplinas de %s";
#endif

const char *Txt_Create_account =
#if   L==1	// ca
	"Crea compte";
#elif L==2	// de
	"Benutzerkonto anlegen";
#elif L==3	// en
	"Create account";
#elif L==4	// es
	"Crear cuenta";
#elif L==5	// fr
	"Cr&eacute;er compte";
#elif L==6	// gn
	"Crear cuenta";		// Okoteve traduccin
#elif L==7	// it
	"Registrati";
#elif L==8	// pl
	"Za&lstrok;&oacute;&zdot; konto";
#elif L==9	// pt
	"Criar conta";
#endif

const char *Txt_Create_announcement =
#if   L==1	// ca
	"Crear anunci";
#elif L==2	// de
	"Erstelle Bekanntmachung";
#elif L==3	// en
	"Create announcement";
#elif L==4	// es
	"Crear anuncio";
#elif L==5	// fr
	"Cr&eacute;er annonce";
#elif L==6	// gn
	"Crear anuncio";		// Okoteve traduccin
#elif L==7	// it
	"Crea annuncio";
#elif L==8	// pl
	"Stw&oacute;rz og&lstrok;oszenia";
#elif L==9	// pt
	"Criar an&uacute;ncio";
#endif

const char *Txt_Create_another_centre =
#if   L==1	// ca
	"Crear un altre centre";
#elif L==2	// de
	"Erstellen Sie ein anderes Lehrinstitut";
#elif L==3	// en
	"Create another centre";
#elif L==4	// es
	"Crear otro centro";
#elif L==5	// fr
	"Cr&eacute;er un autre centre";
#elif L==6	// gn
	"Crear otro centro";	// Okoteve traduccin
#elif L==7	// it
	"Crea un altro centro";
#elif L==8	// pl
	"Utw&oacute;rz inny centrum";
#elif L==9	// pt
	"Criar outro centro";
#endif

const char *Txt_Create_another_course =
#if   L==1	// ca
	"Crear una altra assignatura";
#elif L==2	// de
	"Erstellen Sie ein anderes Kurs";
#elif L==3	// en
	"Create another course";
#elif L==4	// es
	"Crear otra asignatura";
#elif L==5	// fr
	"Cr&eacute;er une autre mati&egrave;re";
#elif L==6	// gn
	"Crear otra asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Crea un altro corso";
#elif L==8	// pl
	"Utw&oacute;rz inny kursu";
#elif L==9	// pt
	"Criar outra disciplina";
#endif

const char *Txt_Create_another_degree =
#if   L==1	// ca
	"Crear una altra titulaci&oacute;";
#elif L==2	// de
	"Erstellen Sie ein anderes Studiengang";
#elif L==3	// en
	"Create another degree";
#elif L==4	// es
	"Crear otra titulaci&oacute;n";
#elif L==5	// fr
	"Cr&eacute;er un autre &eacute;tude";
#elif L==6	// gn
	"Crear otra titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Crea una altra laurea";
#elif L==8	// pl
	"Utw&oacute;rz inny stopnia";
#elif L==9	// pt
	"Criar outro grau";
#endif

const char *Txt_Create_another_institution =
#if   L==1	// ca
	"Crear una altra instituci&oacute;";
#elif L==2	// de
	"Erstellen Sie ein anderes Hochschule";
#elif L==3	// en
	"Create another institution";
#elif L==4	// es
	"Crear otra instituci&oacute;n";
#elif L==5	// fr
	"Cr&eacute;er un autre &eacute;tablissement";
#elif L==6	// gn
	"Crear otra instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Crea una altra istituzione";
#elif L==8	// pl
	"Utw&oacute;rz inny instytucji";
#elif L==9	// pt
	"Criar outra institu&ccedil;&atilde;o";
#endif

const char *Txt_Create_another_type_of_degree =
#if   L==1	// ca
	"Crear otro tipo de titulaci&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Erstellen Sie ein anderes Abschlussart";
#elif L==3	// en
	"Create another type of degree";
#elif L==4	// es
	"Crear otro tipo de titulaci&oacute;n";
#elif L==5	// fr
	"Cr&eacute;er un autre type d'&eacute;tude";
#elif L==6	// gn
	"Crear otro tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Crea un altro tipo di laurea";
#elif L==8	// pl
	"Utw&oacute;rz inny typu stopnia";
#elif L==9	// pt
	"Criar outro tipo de grau";
#endif

const char *Txt_Create_assignment =
#if   L==1	// ca
	"Crear actividad";	// Necessita traduccio
#elif L==2	// de
	"Aufgabe erstellen";
#elif L==3	// en
	"Create assignment";
#elif L==4	// es
	"Crear actividad";
#elif L==5	// fr
	"Cr&eacute;er activit&eacute;";
#elif L==6	// gn
	"Crear actividad";	// Okoteve traduccin
#elif L==7	// it
	"Crea attivit&agrave;";
#elif L==8	// pl
	"Utw&oacute;rz zadanie";
#elif L==9	// pt
	"Criar atividade";
#endif

const char *Txt_Create_banner =
#if   L==1	// ca
	"Crear banner";
#elif L==2	// de
	"Werbebanner erstellen";
#elif L==3	// en
	"Create banner";
#elif L==4	// es
	"Crear banner";
#elif L==5	// fr
	"Cr&eacute;er banni&eacute;re";
#elif L==6	// gn
	"Crear banner";	// Okoteve traduccin
#elif L==7	// it
	"Crea banner";
#elif L==8	// pl
	"Utw&oacute;rz banner";
#elif L==9	// pt
	"Criar banner";
#endif

const char *Txt_Create_centre =
#if   L==1	// ca
	"Crear centre";
#elif L==2	// de
	"Lehrinstitut eingeben";
#elif L==3	// en
	"Create centre";
#elif L==4	// es
	"Crear centro";
#elif L==5	// fr
	"Cr&eacute;er centre";
#elif L==6	// gn
	"Crear centro";		// Okoteve traduccin
#elif L==7	// it
	"Crea centro";
#elif L==8	// pl
	"Utw&oacute;rz centrum";
#elif L==9	// pt
	"Criar centro";
#endif

const char *Txt_Create_classroom =
#if   L==1	// ca
	"Crear aula";
#elif L==2	// de
	"Klassenzimmer eingeben";
#elif L==3	// en
	"Create classroom";
#elif L==4	// es
	"Crear aula";
#elif L==5	// fr
	"Cr&eacute;er salle de classe";
#elif L==6	// gn
	"Crear aula";	// Okoteve traduccin
#elif L==7	// it
	"Crea aula";
#elif L==8	// pl
	"Utw&oacute;rz klasa";
#elif L==9	// pt
	"Criar sala de aula";
#endif

const char *Txt_Create_country =
#if   L==1	// ca
	"Crear pa&iacute;s";	// Necessita traduccio
#elif L==2	// de
	"Land eingeben";
#elif L==3	// en
	"Create country";
#elif L==4	// es
	"Crear pa&iacute;s";
#elif L==5	// fr
	"Cr&eacute;er pays";
#elif L==6	// gn
	"Crear pa&iacute;s";	// Okoteve traduccin
#elif L==7	// it
	"Crea paese";
#elif L==8	// pl
	"Utw&oacute;rz kraju";
#elif L==9	// pt
	"Criar pa&iacute;s";
#endif

const char *Txt_Create_course =
#if   L==1	// ca
	"Crear assignatura";
#elif L==2	// de
	"Kurs eingeben";
#elif L==3	// en
	"Create course";
#elif L==4	// es
	"Crear asignatura";
#elif L==5	// fr
	"Cr&eacute;er mati&egrave;re";
#elif L==6	// gn
	"Crear asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Crea corso";
#elif L==8	// pl
	"Utw&oacute;rz kursu";
#elif L==9	// pt
	"Criar disciplina";
#endif

const char *Txt_Create_degree =
#if   L==1	// ca
	"Crear titulaci&oacute;";
#elif L==2	// de
	"Studiengang eingeben";
#elif L==3	// en
	"Create degree";
#elif L==4	// es
	"Crear titulaci&oacute;n";
#elif L==5	// fr
	"Cr&eacute;er &eacute;tude";
#elif L==6	// gn
	"Crear titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Crea laurea";
#elif L==8	// pl
	"Utw&oacute;rz stopnia";
#elif L==9	// pt
	"Criar grau";
#endif

const char *Txt_Create_department =
#if   L==1	// ca
	"Crear departamento";	// Necessita traduccio
#elif L==2	// de
	"Abteilung eingeben";
#elif L==3	// en
	"Create department";
#elif L==4	// es
	"Crear departamento";
#elif L==5	// fr
	"Cr&eacute;er d&eacute;partement";
#elif L==6	// gn
	"Crear departamento";	// Okoteve traduccin
#elif L==7	// it
	"Crea dipartimento";
#elif L==8	// pl
	"Utw&oacute;rz dzia&lstrok;u";
#elif L==9	// pt
	"Criar departamento";
#endif

const char *Txt_Create_email_domain =
#if   L==1	// ca
	"Crear dominio de correo";	// Necessita traduccio
#elif L==2	// de
	"E-Mail-Dom&auml;ne anlegen";
#elif L==3	// en
	"Create email domain";
#elif L==4	// es
	"Crear dominio de correo";
#elif L==5	// fr
	"Cr&eacute;er domaine de messagerie";
#elif L==6	// gn
	"Crear dominio de correo";	// Okoteve traduccin
#elif L==7	// it
	"Crea campo mail";
#elif L==8	// pl
	"Utw&oacute;rz domeny mail";
#elif L==9	// pt
	"Criar dom&iacute;nio de email";
#endif

const char *Txt_Create_email_message =
#if   L==1	// ca
	"Crear mensaje de correo";	// Necessita traduccio
#elif L==2	// de
	"Neue E-Mail-Nachricht";
#elif L==3	// en
	"Create email message";
#elif L==4	// es
	"Crear mensaje de correo";
#elif L==5	// fr
	"Cr&eacute;er un email message";
#elif L==6	// gn
	"Crear mensaje de correo";	// Okoteve traduccin
#elif L==7	// it
	"Crea messaggio email";
#elif L==8	// pl
	"Utw&oacute;rz wiadomosci email";
#elif L==9	// pt
	"Criar uma mensagem de email";
#endif

const char *Txt_Create_event =
#if   L==1	// ca
	"Crear evento";	// Necessita traduccio
#elif L==2	// de
	"Ereignis erstellen";
#elif L==3	// en
	"Create event";
#elif L==4	// es
	"Crear evento";
#elif L==5	// fr
	"Cr&eacute;er &eacute;v&eacute;nement";
#elif L==6	// gn
	"Crear evento";	// Okoteve traduccin
#elif L==7	// it
	"Crea evento";
#elif L==8	// pl
	"Utw&oacute;rz wydarzenie";
#elif L==9	// pt
	"Criar evento";
#endif

const char *Txt_Create_folder =
#if   L==1	// ca
	"Crear carpeta";	// Necessita traduccio
#elif L==2	// de
	"Verzeichnis anlegen";
#elif L==3	// en
	"Create folder";
#elif L==4	// es
	"Crear carpeta";
#elif L==5	// fr
	"Cr&eacute;er r&eacute;pertoire";
#elif L==6	// gn
	"Crear carpeta";	// Okoteve traduccin
#elif L==7	// it
	"Crea cartella";
#elif L==8	// pl
	"Utw&oacute;rz folder";
#elif L==9	// pt
	"Criar diret&oacute;rio";
#endif

const char *Txt_Create_game =
#if   L==1	// ca
	"Crear joc";
#elif L==2	// de
	"Spiel erstellen";
#elif L==3	// en
	"Create game";
#elif L==4	// es
	"Crear juego";
#elif L==5	// fr
	"Cr&eacute;er jeu";
#elif L==6	// gn
	"Crear juego";	// Okoteve traduccin
#elif L==7	// it
	"Crea gioco";
#elif L==8	// pl
	"Utw&oacute;rz gra";
#elif L==9	// pt
	"Criar jogo";
#endif

const char *Txt_Create_group =
#if   L==1	// ca
	"Crear grup";
#elif L==2	// de
	"Gruppe erstellen";
#elif L==3	// en
	"Create group";
#elif L==4	// es
	"Crear grupo";
#elif L==5	// fr
	"Cr&eacute;er groupe";
#elif L==6	// gn
	"Crear grupo";		// Okoteve traduccin
#elif L==7	// it
	"Crea gruppo";
#elif L==8	// pl
	"Utw&oacute;rz grup&eogon;";
#elif L==9	// pt
	"Criar grupo";
#endif

const char *Txt_Create_holiday =
#if   L==1	// ca
	"Crear festivitat";
#elif L==2	// de
	"Feiertag eingeben";
#elif L==3	// en
	"Create holiday";
#elif L==4	// es
	"Crear festividad";
#elif L==5	// fr
	"Cr&eacute;er f&ecirc;te";
#elif L==6	// gn
	"Crear festividad";	// Okoteve traduccin
#elif L==7	// it
	"Crea festivit&agrave;";
#elif L==8	// pl
	"Utw&oacute;rz wakacje";
#elif L==9	// pt
	"Criar f&eacute;ria";
#endif

const char *Txt_Create_institution =
#if   L==1	// ca
	"Crear instituci&oacute;";
#elif L==2	// de
	"Hochschule eingeben";
#elif L==3	// en
	"Create institution";
#elif L==4	// es
	"Crear instituci&oacute;n";
#elif L==5	// fr
	"Cr&eacute;er &eacute;tablissement";
#elif L==6	// gn
	"Crear instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Crea istituzione";
#elif L==8	// pl
	"Utw&oacute;rz instytucji";
#elif L==9	// pt
	"Criar institu&ccedil;&atilde;o";
#endif

const char *Txt_Create_link =
#if   L==1	// ca
	"Crear enlla&ccedil;";
#elif L==2	// de
	"Link erstellen";
#elif L==3	// en
	"Create link";
#elif L==4	// es
	"Crear enlace";
#elif L==5	// fr
	"Cr&eacute;er lien";
#elif L==6	// gn
	"Crear enlace";	// Okoteve traduccin
#elif L==7	// it
	"Crea link";
#elif L==8	// pl
	"Utw&oacute;rz link";
#elif L==9	// pt
	"Criar liga&ccedil;&atilde;o";
#endif

const char *Txt_Create_notice =
#if   L==1	// ca
	"Crear av&iacute;s";
#elif L==2	// de
	"Erstelle Ank&uuml;ndigung";
#elif L==3	// en
	"Create notice";
#elif L==4	// es
	"Crear aviso";
#elif L==5	// fr
	"Cr&eacute;er avis";
#elif L==6	// gn
	"Crear aviso";		// Okoteve traduccin
#elif L==7	// it
	"Crea avviso";
#elif L==8	// pl
	"Utw&oacute;rz powiadomienie";
#elif L==9	// pt
	"Criar aviso";
#endif

const char *Txt_Create_BR_notification =
#if   L==1	// ca
	"Crear<br />notificaci&oacute;";
#elif L==2	// de
	"Anmeldung<br />eingeben";
#elif L==3	// en
	"Create<br />notification";
#elif L==4	// es
	"Crear<br />notificaci&oacute;n";
#elif L==5	// fr
	"Cr&eacute;er<br />notification";
#elif L==6	// gn
	"Crear<br />notificaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Crea<br />notifica";
#elif L==8	// pl
	"Utw&oacute;rz<br />powiadomienie";
#elif L==9	// pt
	"Criar<br />notifica&ccedil;&atilde;o";
#endif

const char *Txt_Create_password =
#if   L==1	// ca
	"Crear contrasenya";
#elif L==2	// de
	"Passwort erstellen";
#elif L==3	// en
	"Crear password";
#elif L==4	// es
	"Cambiar contrase&ntilde;a";
#elif L==5	// fr
	"Cr&eacute;er le mot de passe";
#elif L==6	// gn
	"Crear contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Cambiare password";
#elif L==8	// pl
	"Utw&oacute;rz has&lstrok;o";
#elif L==9	// pt
	"Criar senha";
#endif

const char *Txt_Create_place =
#if   L==1	// ca
	"Crear lloc";
#elif L==2	// de
	"Standort eingeben";
#elif L==3	// en
	"Create place";
#elif L==4	// es
	"Crear lugar";
#elif L==5	// fr
	"Cr&eacute;er emplacement";
#elif L==6	// gn
	"Crear lugar";	// Okoteve traduccin
#elif L==7	// it
	"Crea localit&grave;";
#elif L==8	// pl
	"Utw&oacute;rz miejsce";
#elif L==9	// pt
	"Criar localiza&ccedil;&atilde;o";
#endif

const char *Txt_Create_plugin =
#if   L==1	// ca
	"Crear complemento";	// Necessita traduccio
#elif L==2	// de
	"Plugin erstellen";
#elif L==3	// en
	"Create plugin";
#elif L==4	// es
	"Crear complemento";
#elif L==5	// fr
	"Cr&eacute;er plugin";
#elif L==6	// gn
	"Crear complemento";	// Okoteve traduccin
#elif L==7	// it
	"Crea plugin";
#elif L==8	// pl
	"Utw&oacute;rz plugin";
#elif L==9	// pt
	"Criar plugin";
#endif

const char *Txt_Create_project =
#if   L==1	// ca
	"Crear projecte";
#elif L==2	// de
	"Projekt eingeben";
#elif L==3	// en
	"Create project";
#elif L==4	// es
	"Crear proyecto";
#elif L==5	// fr
	"Cr&eacute;er projet";
#elif L==6	// gn
	"Crear proyecto";	// Okoteve traduccin
#elif L==7	// it
	"Crea progetto";
#elif L==8	// pl
	"Utw&oacute;rz projekt";
#elif L==9	// pt
	"Criar projeto";
#endif

const char *Txt_Create_question =
#if   L==1	// ca
	"Crear pregunta";
#elif L==2	// de
	"Frage erstellen";
#elif L==3	// en
	"Create question";
#elif L==4	// es
	"Crear pregunta";
#elif L==5	// fr
	"Cr&eacute;er question";
#elif L==6	// gn
	"Crear pregunta";	// Okoteve traduccin
#elif L==7	// it
	"Crea domanda";
#elif L==8	// pl
	"Utw&oacute;rz pytanie";
#elif L==9	// pt
	"Criar quest&atilde;o";
#endif

const char *Txt_Create_record_field =
#if   L==1	// ca
	"Crear campo de ficha";	// Necessita traduccio
#elif L==2	// de
	"Datenfeld erstellen";
#elif L==3	// en
	"Create record field";
#elif L==4	// es
	"Crear campo de ficha";
#elif L==5	// fr
	"Cr&eacute;er champ de carte";
#elif L==6	// gn
	"Crear campo de ficha";	// Okoteve traduccin
#elif L==7	// it
	"Crea campo di scheda";
#elif L==8	// pl
	"Utw&oacute;rz rekord pola";
#elif L==9	// pt
	"Criar campo de cart&atilde;o";
#endif

const char *Txt_Create_survey =
#if   L==1	// ca
	"Crear enquesta";
#elif L==2	// de
	"Umfrage erstellen";
#elif L==3	// en
	"Create survey";
#elif L==4	// es
	"Crear encuesta";
#elif L==5	// fr
	"Cr&eacute;er sondage";
#elif L==6	// gn
	"Crear encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Crea sondaggio";
#elif L==8	// pl
	"Utw&oacute;rz ankiety";
#elif L==9	// pt
	"Criar inqu&eacute;rito";
#endif

const char *Txt_Create_type_of_degree =
#if   L==1	// ca
	"Crear tipo de titulaci&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Abschlussart definieren";
#elif L==3	// en
	"Create type of degree";
#elif L==4	// es
	"Crear tipo de titulaci&oacute;n";
#elif L==5	// fr
	"Cr&eacute;er type d'&eacute;tude";
#elif L==6	// gn
	"Crear tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Crea tipo di laurea";
#elif L==8	// pl
	"Utw&oacute;rz typu stopnia";
#elif L==9	// pt
	"Criar tipo de grau";
#endif

const char *Txt_Create_type_of_group =
#if   L==1	// ca
	"Crear tipo de grupo";	// Necessita traduccio
#elif L==2	// de
	"Gruppen-Typ definieren";
#elif L==3	// en
	"Create type of group";
#elif L==4	// es
	"Crear tipo de grupo";
#elif L==5	// fr
	"Cr&eacute;er type de groupe";
#elif L==6	// gn
	"Crear tipo de grupo";	// Okoteve traduccin
#elif L==7	// it
	"Crea tipo di gruppo";
#elif L==8	// pl
	"Utw&oacute;rz typu grupy";
#elif L==9	// pt
	"Criar tipo de grupo";
#endif

const char *Txt_Create_ZIP_file =
#if   L==1	// ca
	"Crear archivo ZIP";	// Necessita traduccio
#elif L==2	// de
	"ZIP-Datei erstellen";
#elif L==3	// en
	"Create ZIP file";
#elif L==4	// es
	"Crear archivo ZIP";
#elif L==5	// fr
	"Cr&eacute;er fichier ZIP";
#elif L==6	// gn
	"Crear archivo ZIP";	// Okoteve traduccin
#elif L==7	// it
	"Crea file ZIP";
#elif L==8	// pl
	"Utw&oacute;rz pliku ZIP";
#elif L==9	// pt
	"Criar arquivo ZIP";
#endif

const char *Txt_Created_new_announcement_of_exam =
#if   L==1	// ca
	"Creada nueva convocatoria de examen.";	// Necessita traduccio
#elif L==2	// de
	"Neuer Aufruf zur Pr&uuml;fung erstellt.";
#elif L==3	// en
	"Created new announcement of exam.";
#elif L==4	// es
	"Creada nueva convocatoria de examen.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvelle convocation &agrave; un examen.";
#elif L==6	// gn
	"Creada nueva convocatoria de examen.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo appello d'esame.";
#elif L==8	// pl
	"Utworzono nowe ogloszenie egzamin.";
#elif L==9	// pt
	"Criado nova chamada para exame.";
#endif

const char *Txt_Created_new_assignment_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova activitat <strong>%s</strong>.";
#elif L==2	// de
	"Neue Aufgabe <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new assignment <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva actividad <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute;e nouvelle activit&eacute; <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva actividad <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova attivit&agrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowa zadania <strong>%s</strong>.";
#elif L==9	// pt
	"Criada nova atividade <strong>%s</strong>.";
#endif

const char *Txt_Created_new_banner_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou banner <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Werbebanner <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new banner <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo banner <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvelle banni&eacute;re <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo banner <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo banner <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowy banner <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo banner <strong>%s</strong>.";
#endif

const char *Txt_Created_new_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou centre <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neues Lehrinstitut <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new centre <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo centro <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau centre <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo centro <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo centro <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe centrum <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo centro <strong>%s</strong>.";
#endif

const char *Txt_Created_new_classroom_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova aula <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Klassenzimmer <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new classroom <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva aula <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvelle salle de classe <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva aula <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova aula <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe klasa <strong>%s</strong>.";
#elif L==9	// pt
	"Criada nova sala de aula <strong>%s</strong>.";
#endif

const char *Txt_Created_new_country_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neues Land <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new country <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau pays <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo paese <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowa kraju <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo pa&iacute;s <strong>%s</strong>.";
#endif

const char *Txt_Created_new_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nueva asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neue Kurs <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new course <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute;e nouvelle mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo corso <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowa oczywiscie <strong>%s</strong>.";
#elif L==9	// pt
	"Criada nova disciplina <strong>%s</strong>.";
#endif

const char *Txt_Created_new_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neuer Studiengang <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new degree <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute;e nouvel &eacute;tude <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova laurea <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowy poziom <strong>%s</strong>.";
#elif L==9	// pt
	"Criada novo grau <strong>%s</strong>.";
#endif

const char *Txt_Created_new_department_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creado nuevo departamento <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neue Abteilung <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new department <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo departamento <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau d&eacute;partement <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo departamento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo dipartimento <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowy dzia&lstrok; <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo departamento <strong>%s</strong>.";
#endif

const char *Txt_Created_new_email_domain_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creado nuevo dominio de correo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neue E-Mail-Dom&auml;ne <strong>%s</strong> angelegt.";
#elif L==3	// en
	"Created new email domain <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo dominio de correo <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvel domaine de messagerie <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo dominio de correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo campo mail <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowa domena mail <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo dom&iacute;nio de email <strong>%s</strong>.";
#endif

const char *Txt_Created_new_event_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creado nuevo evento <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neues Ereignis <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new event <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo evento <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau &eacute;v&eacute;nement <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo evento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo evento <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe wydarzenie <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo evento <strong>%s</strong>.";
#endif

const char *Txt_Created_new_game_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada novo joc <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Spiel <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new game <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo juego <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau jeu <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo juego <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo gioco <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe gra <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo jogo <strong>%s</strong>.";
#endif

const char *Txt_Created_new_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creado nuevo grupo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neue Gruppe <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new group <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo grupo <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau groupe <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo grupo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo gruppo <strong>%s</strong>.";
#elif L==8	// pl
	"Stworzyl nowa grupe <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo grupo <strong>%s</strong>.";
#endif

const char *Txt_Created_new_holiday_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nueva festividad <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neuer Feiertag <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new holiday <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva festividad <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvelle f&ecirc;te <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva festividad <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova festivit&agrave; <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe swieto <strong>%s</strong>.";
#elif L==9	// pt
	"Criado nova f&eacute;ria <strong>%s</strong>.";
#endif

const char *Txt_Created_new_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neue Hochschule <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new institution <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute;e nouvel &eacute;tablissement <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova istituzione <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowa instytucje <strong>%s</strong>.";
#elif L==9	// pt
	"Criada nova institu&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_link_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creado nuevo enlace <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neuer Link <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new link <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo enlace <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau lien <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo enlace <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo link <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowy link <strong>%s</strong>.";
#elif L==9	// pt
	"Criado nova liga&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_place_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou lloc <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Standort <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new place <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo lugar <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouvel emplacement <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo lugar <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creata nuova localit&grave; <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe miejsce <strong>%s</strong>.";
#elif L==9	// pt
	"Criado nova localiza&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_plugin_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creado nuevo complemento <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neuer Plugin <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new plugin <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo complemento <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau plugin <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo complemento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo plugin <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowy plugin <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo plugin <strong>%s</strong>.";
#endif

const char *Txt_Created_new_project_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creat nou projecte <strong>%s</strong>.";
#elif L==2	// de
	"Neues Projekt <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"Created new project <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo proyecto <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau projet <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo proyecto <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo progetto <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe projekt <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo projeto <strong>%s</strong>.";
#endif

const char *Txt_Created_new_record_field_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creado nuevo campo de ficha <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neuer Datenfeld <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new record field <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo campo de ficha <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau champ de carte <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo campo de ficha <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo campo di scheda <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe pola rekordu <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo campo de cart&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creada nova enquesta <strong>%s</strong>.";
#elif L==2	// de
	"Neuer Umfrage <strong>%s</strong> erstellt.";
#elif L==3	// en
	"Created new survey <strong>%s</strong>.";
#elif L==4	// es
	"Creada nueva encuesta <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau sondage <strong>%s</strong>.";
#elif L==6	// gn
	"Creada nueva encuesta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo sondaggio <strong>%s</strong>.";
#elif L==8	// pl
	"Utworzono nowe badania <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo inqu&eacute;rito <strong>%s</strong>.";
#endif

const char *Txt_Created_new_type_of_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neue Abschlussart <strong>%s</strong> definiert.";
#elif L==3	// en
	"Created new type of degree <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau type d'&eacute;tude <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo tipo di laurea <strong>%s</strong>.";
#elif L==8	// pl
	"Stworzyl nowy typ stopnia <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo tipo de grau <strong>%s</strong>.";
#endif

const char *Txt_Created_new_type_of_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Creado nuevo tipo de grupo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Neuer Gruppen-Typ <strong>%s</strong> definiert.";
#elif L==3	// en
	"Created new type of group <strong>%s</strong>.";
#elif L==4	// es
	"Creado nuevo tipo de grupo <strong>%s</strong>.";
#elif L==5	// fr
	"Cr&eacute;&eacute; nouveau type de groupe <strong>%s</strong>.";
#elif L==6	// gn
	"Creado nuevo tipo de grupo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Creato nuovo tipo di gruppo <strong>%s</strong>.";
#elif L==8	// pl
	"Stworzyl nowy typ grupy <strong>%s</strong>.";
#elif L==9	// pt
	"Criado novo tipo de grupo <strong>%s</strong>.";
#endif

const char *Txt_Created_tables_in_the_database_that_did_not_exist =
#if   L==1	// ca
	"Creadas las tablas de la base de datos que no exist&iacute;an.";	// Necessita traduccio
#elif L==2	// de
	"Neue Tabellen in der Datenbank erstellt.";
#elif L==3	// en
	"Created tables in the database that did not exist.";
#elif L==4	// es
	"Creadas las tablas de la base de datos que no exist&iacute;an.";
#elif L==5	// fr
	"Tables cr&eacute;&eacute;es dans la base de donn&eacute;es qui n'existaient pas.";
#elif L==6	// gn
	"Creadas las tablas de la base de datos que no exist&iacute;an.";	// Okoteve traduccin
#elif L==7	// it
	"Tabelle create in un database inesistente.";
#elif L==8	// pl
	"Wykonawca tabel w bazie danych, kt&oacute;re nie istnialy.";
#elif L==9	// pt
	"Criadas tabelas no banco de dados que n&atilde;o existiam.";
#endif

const char *Txt_Creating_database_tables_if_they_do_not_exist =
#if   L==1	// ca
	"Creando tablas de la base de datos si no existen&hellip;";	// Necessita traduccio
#elif L==2	// de
	"Creating database tables if they do not exist&hellip;";	// Need bersetzung
#elif L==3	// en
	"Creating database tables if they do not exist&hellip;";
#elif L==4	// es
	"Creando tablas de la base de datos si no existen&hellip;";
#elif L==5	// fr
	"Creating database tables if they do not exist&hellip;";	// Besoin de traduction
#elif L==6	// gn
	"Creando tablas de la base de datos si no existen&hellip;";	// Okoteve traduccin
#elif L==7	// it
	"Creazione tabelle della base di dati se non esistono&hellip;";
#elif L==8	// pl
	"Utw&oacute;rz tabel bazy danych, jesli ich nie ma &hellip;";
#elif L==9	// pt
	"Criando tabelas da base de dados, se elas n&atilde;o existirem&hellip;";
#endif

const char *Txt_Current_email =
#if   L==1	// ca
	"Correu actual";
#elif L==2	// de
	"Aktueller E-Mail";
#elif L==3	// en
	"Current email";
#elif L==4	// es
	"Correo actual";
#elif L==5	// fr
	"Courrier actuel";
#elif L==6	// gn
	"Correo actual";	// Okoteve traduccin
#elif L==7	// it
	"Email corrente";
#elif L==8	// pl
	"Obecny email";
#elif L==9	// pt
	"Email atual";
#endif

const char *Txt_Current_image_video =
#if   L==1	// ca
	"Imatge / v&iacute;deo actual";
#elif L==2	// de
	"Aktuelles Abbild / Video";
#elif L==3	// en
	"Current image / video";
#elif L==4	// es
	"Imagen / v&iacute;deo actual";
#elif L==5	// fr
	"Image / vid&eacute;o actuelle";
#elif L==6	// gn
	"Imagen / v&iacute;deo actual";	// Okoteve traduccin
#elif L==7	// it
	"Immagine / video corrente";
#elif L==8	// pl
	"Bie&zdot;&aogon;cy obraz / wideo";
#elif L==9	// pt
	"Imagem / v&iacute;deo atual";
#endif

const char *Txt_Current_nickname =
#if   L==1	// ca
	"Sobrenom actual";
#elif L==2	// de
	"Aktueller Benutzername";
#elif L==3	// en
	"Current nickname";
#elif L==4	// es
	"Apodo actual";
#elif L==5	// fr
	"Surnom actuel";
#elif L==6	// gn
	"Apodo actual";	// Okoteve traduccin
#elif L==7	// it
	"Nome utente corrente";
#elif L==8	// pl
	"Obecny pseudonim";
#elif L==9	// pt
	"Alcunha atual";
#endif

const char *Txt_Current_password =
#if   L==1	// ca
	"Contrasenya actual";
#elif L==2	// de
	"Aktuelles Passwort";
#elif L==3	// en
	"Current password";
#elif L==4	// es
	"Contrase&ntilde;a actual";
#elif L==5	// fr
	"Mot de passe actuel";
#elif L==6	// gn
	"Contrase&ntilde;a actual";	// Okoteve traduccin
#elif L==7	// it
	"Password corrente";
#elif L==8	// pl
	"Obecne has&lstrok;o";
#elif L==9	// pt
	"Senha atual";
#endif

const char *Txt_Cut =	// First part of "Cut-Paste"
#if   L==1	// ca
	"Tallar";
#elif L==2	// de
	"Ausschneiden";
#elif L==3	// en
	"Cut";
#elif L==4	// es
	"Cortar";
#elif L==5	// fr
	"Couper";
#elif L==6	// gn
	"Ikyt&itilde;";
#elif L==7	// it
	"Taglia";
#elif L==8	// pl
	"Wytnij";
#elif L==9	// pt
	"Cortar";
#endif

const char *Txt_Data =	// Project data
#if   L==1	// ca
	"Dades";
#elif L==2	// de
	"Daten";
#elif L==3	// en
	"Data";
#elif L==4	// es
	"Datos";
#elif L==5	// fr
	"Donn&eacute;es";
#elif L==6	// gn
	"Datos";	// Okoteve traduccin
#elif L==7	// it
	"Dati";
#elif L==8	// pl
	"Dane";
#elif L==9	// pt
	"Dados";
#endif

const char *Txt_DATA_PROTECTION_CLAUSE =
#if   L==1	// ca
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Necessita traduccio
#elif L==2	// de
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Need bersetzung
#elif L==3	// en
	"Data protection clause";
#elif L==4	// es
	"Pol&iacute;tica de protecci&oacute;n de datos";
#elif L==5	// fr
	"Clause de protection des donn&eacute;es";
#elif L==6	// gn
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Okoteve traduccin
#elif L==7	// it
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Bisogno di traduzione
#elif L==8	// pl
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pol&iacute;tica de prote&ccedil;&atilde;o de dados";
#endif

const char *Txt_Date =
#if   L==1	// ca
	"Data";
#elif L==2	// de
	"Datum";
#elif L==3	// en
	"Date";
#elif L==4	// es
	"Fecha";
#elif L==5	// fr
	"Date";
#elif L==6	// gn
	"Arange";
#elif L==7	// it
	"Data";
#elif L==8	// pl
	"Data";
#elif L==9	// pt
	"Data";
#endif

const char *Txt_Date_of_birth =
#if   L==1	// ca
	"Data naixement";
#elif L==2	// de
	"Geburtsdatum";
#elif L==3	// en
	"Date of birth";
#elif L==4	// es
	"Fecha nacimiento";
#elif L==5	// fr
	"Date naissance";
#elif L==6	// gn
	"Fecha nacimiento";	// Okoteve traduccin
#elif L==7	// it
	"Data di nascita";
#elif L==8	// pl
	"Data urodzenia";
#elif L==9	// pt
	"Data nascimento";
#endif

const char *Txt_Date_of_creation =
#if   L==1	// ca
	"Data de creaci&oacute;";
#elif L==2	// de
	"Datum der Erstellung";
#elif L==3	// en
	"Date of creation";
#elif L==4	// es
	"Fecha de creaci&oacute;n";
#elif L==5	// fr
	"Date de cr&eacute;ation";
#elif L==6	// gn
	"Fecha de creaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Data di creazione";
#elif L==8	// pl
	"Data utworzenia";
#elif L==9	// pt
	"Data da cria&ccedil;&atilde;o";
#endif

const char *Txt_Dates =
#if   L==1	// ca
	"Dates";
#elif L==2	// de
	"Datum";
#elif L==3	// en
	"Dates";
#elif L==4	// es
	"Fechas";
#elif L==5	// fr
	"Dates";
#elif L==6	// gn
	"Arange";
#elif L==7	// it
	"Date";
#elif L==8	// pl
	"Daty";
#elif L==9	// pt
	"Datas";
#endif

const char *Txt_Day =
#if   L==1	// ca
	"D&iacute;a";	// Necessita traduccio
#elif L==2	// de
	"Tag";
#elif L==3	// en
	"Day";
#elif L==4	// es
	"D&iacute;a";
#elif L==5	// fr
	"Jour";
#elif L==6	// gn
	"D&iacute;a";	// Okoteve traduccin
#elif L==7	// it
	"Giorno";
#elif L==8	// pl
	"Dzien";
#elif L==9	// pt
	"Dia";
#endif

const char *Txt_day =
#if   L==1	// ca
	"dia";
#elif L==2	// de
	"Tag";
#elif L==3	// en
	"day";
#elif L==4	// es
	"d&iacute;a";
#elif L==5	// fr
	"jour";
#elif L==6	// gn
	"&aacute;ra";
#elif L==7	// it
	"giorno";
#elif L==8	// pl
	"dzie&nacute;";
#elif L==9	// pt
	"dia";
#endif

const char *Txt_days =
#if   L==1	// ca
	"dies";
#elif L==2	// de
	"Tage";
#elif L==3	// en
	"days";
#elif L==4	// es
	"d&iacute;as";
#elif L==5	// fr
	"jours";
#elif L==6	// gn
	"&aacute;ra";
#elif L==7	// it
	"giorni";
#elif L==8	// pl
	"dni;";
#elif L==9	// pt
	"dias";
#endif

const char *Txt_DAYS_CAPS[7] =
   {
#if   L==1	// ca
	 "DILLUNS"
#elif L==2	// de
	 "MONTAG"
#elif L==3	// en
	 "MONDAY"
#elif L==4	// es
	 "LUNES"
#elif L==5	// fr
	 "LUNDI"
#elif L==6	// gn
	 "ARAKOI"
#elif L==7	// it
	 "LUNED&Igrave;"
#elif L==8	// pl
	 "Poniedzialek"
#elif L==9	// pt
	 "SEGUNDA-FEIRA"
#endif
	 ,
#if   L==1	// ca
	 "DIMARTS"
#elif L==2	// de
	 "DIENSTAG"
#elif L==3	// en
	 "TUESDAY"
#elif L==4	// es
	 "MARTES"
#elif L==5	// fr
	 "MARDI"
#elif L==6	// gn
	 "ARAAPY"
#elif L==7	// it
	 "MARTED&Igrave;"
#elif L==8	// pl
	 "Wtorek"
#elif L==9	// pt
	 "TER&Ccedil;A-FEIRA"
#endif
	 ,
#if   L==1	// ca
	 "DIMECRES"
#elif L==2	// de
	 "MITTWOCH"
#elif L==3	// en
	 "WEDNESDAY"
#elif L==4	// es
	 "MI&Eacute;RCOLES"
#elif L==5	// fr
	 "MERCREDI"
#elif L==6	// gn
	 "ARARUNDY"
#elif L==7	// it
	 "MERCOLED&Igrave;"
#elif L==8	// pl
	 "Sroda"
#elif L==9	// pt
	 "QUARTA-FEIRA"
#endif
	 ,
#if   L==1	// ca
	 "DIJOUS"
#elif L==2	// de
	 "DONNERSTAG"
#elif L==3	// en
	 "THURSDAY"
#elif L==4	// es
	 "JUEVES"
#elif L==5	// fr
	 "JEUDI"
#elif L==6	// gn
	 "ARAPO"
#elif L==7	// it
	 "GIOVED&Igrave;"
#elif L==8	// pl
	 "Czwartek"
#elif L==9	// pt
	 "QUINTA-FEIRA"
#endif
	 ,
#if   L==1	// ca
	 "DIVENDRES"
#elif L==2	// de
	 "FREITAG"
#elif L==3	// en
	 "FRIDAY"
#elif L==4	// es
	 "VIERNES"
#elif L==5	// fr
	 "VENDREDI"
#elif L==6	// gn
	 "ARAPOTE&#296;"
#elif L==7	// it
	 "VENERD&Igrave;"
#elif L==8	// pl
	 "Piatek"
#elif L==9	// pt
	 "SEXTA-FEIRA"
#endif
	 ,
#if   L==1	// ca
	 "DISSABTE"
#elif L==2	// de
	 "SAMSTAG"
#elif L==3	// en
	 "SATURDAY"
#elif L==4	// es
	 "S&Aacute;BADO"
#elif L==5	// fr
	 "SAMEDI"
#elif L==6	// gn
	 "ARAPOK&Otilde;I"
#elif L==7	// it
	 "SABATO"
#elif L==8	// pl
	 "Sobota"
#elif L==9	// pt
	 "SABADO"
#endif
	 ,
#if   L==1	// ca
	 "DIUMENGE"
#elif L==2	// de
	 "SONNTAG"
#elif L==3	// en
	 "SUNDAY"
#elif L==4	// es
	 "DOMINGO"
#elif L==5	// fr
	 "DIMANCHE"
#elif L==6	// gn
	 "ARATE&#296;"
#elif L==7	// it
	 "DOMENICA"
#elif L==8	// pl
	 "Niedziela"
#elif L==9	// pt
	 "DOMINGO"
#endif
	};

const char *Txt_DAYS_SMALL[7] =
   {
#if   L==1	// ca
	 "dilluns"
#elif L==2	// de
	 "Montag"
#elif L==3	// en
	 "Monday"
#elif L==4	// es
	 "lunes"
#elif L==5	// fr
	 "Lundi"
#elif L==6	// gn
	 "arak&otilde;i"
#elif L==7	// it
	 "Luned&igrave;"
#elif L==8	// pl
	 "Poniedzialek"
#elif L==9	// pt
	 "Segunda-feira"
#endif
	 ,
#if   L==1	// ca
	 "dimarts"
#elif L==2	// de
	 "Dienstag"
#elif L==3	// en
	 "Tuesday"
#elif L==4	// es
	 "martes"
#elif L==5	// fr
	 "Mardi"
#elif L==6	// gn
	 "araapy"
#elif L==7	// it
	 "Marted&igrave;"
#elif L==8	// pl
	 "Wtorek"
#elif L==9	// pt
	 "Ter&ccedil;a-feira"
#endif
	 ,
#if   L==1	// ca
	 "dimecres"
#elif L==2	// de
	 "Mittwoch"
#elif L==3	// en
	 "Wednesday"
#elif L==4	// es
	 "mi&eacute;rcoles"
#elif L==5	// fr
	 "Mercredi"
#elif L==6	// gn
	 "ararundy"
#elif L==7	// it
	 "Mercoled&igrave;"
#elif L==8	// pl
	 "Sroda"
#elif L==9	// pt
	 "Quarta-feira"
#endif
	 ,
#if   L==1	// ca
	 "dijous"
#elif L==2	// de
	 "Donnerstag"
#elif L==3	// en
	 "Thursday"
#elif L==4	// es
	 "jueves"
#elif L==5	// fr
	 "Jeudi"
#elif L==6	// gn
	 "arapo"
#elif L==7	// it
	 "Gioved&igrave;"
#elif L==8	// pl
	 "Czwartek"
#elif L==9	// pt
	 "Quinta-feira"
#endif
	 ,
#if   L==1	// ca
	 "divendres"
#elif L==2	// de
	 "Freitag"
#elif L==3	// en
	 "Friday"
#elif L==4	// es
	 "viernes"
#elif L==5	// fr
	 "Vendredi"
#elif L==6	// gn
	 "arapote&#297;"
#elif L==7	// it
	 "Venerd&igrave;"
#elif L==8	// pl
	 "Piatek"
#elif L==9	// pt
	 "Sexta-feira"
#endif
	 ,
#if   L==1	// ca
	 "dissabte"
#elif L==2	// de
	 "Samstag"
#elif L==3	// en
	 "Saturday"
#elif L==4	// es
	 "s&aacute;bado"
#elif L==5	// fr
	 "Samedi"
#elif L==6	// gn
	 "arapok&otilde;i"
#elif L==7	// it
	 "Sabato"
#elif L==8	// pl
	 "Sobota"
#elif L==9	// pt
	 "Sabado"
#endif
	 ,
#if   L==1	// ca
	 "diumenge"
#elif L==2	// de
	 "Sonntag"
#elif L==3	// en
	 "Sunday"
#elif L==4	// es
	 "domingo"
#elif L==5	// fr
	 "Dimanche"
#elif L==6	// gn
	 "arate&#297;"
#elif L==7	// it
	 "Domenica"
#elif L==8	// pl
	 "Niedziela"
#elif L==9	// pt
	 "Domingo"
#endif
	};

const char *Txt_Decrease_level_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Disminuir nivel de %s";	// Necessita traduccio
#elif L==2	// de
	"Decrease level of %s";		// Need bersetzung
#elif L==3	// en
	"Decrease level of %s";
#elif L==4	// es
	"Disminuir nivel de %s";
#elif L==5	// fr
	"Decrease level of %s";		// Besoin de traduction
#elif L==6	// gn
	"Disminuir nivel de %s";	// Okoteve traduccin
#elif L==7	// it
	"Decrementare livello di %s";
#elif L==8	// pl
	"Spadek poziomu %s";
#elif L==9	// pt
	"Diminuir o n&iacute;vel de %s";
#endif

const char *Txt_default =
#if   L==1	// ca
	"predeterminado";	// Necessita traduccio
#elif L==2	// de
	"Vorgabe";
#elif L==3	// en
	"default";
#elif L==4	// es
	"predeterminado";
#elif L==5	// fr
	"d&eacute;faut";
#elif L==6	// gn
	"predeterminado";	// Okoteve traduccin
#elif L==7	// it
	"predefinito";
#elif L==8	// pl
	"Domyslnie";
#elif L==9	// pt
	"padr&atilde;o";
#endif

const char *Txt_Degree =
#if   L==1	// ca
	"Titulaci&oacute;";
#elif L==2	// de
	"Studiengang";
#elif L==3	// en
	"Degree";
#elif L==4	// es
	"Titulaci&oacute;n";
#elif L==5	// fr
	"&Eacute;tude";
#elif L==6	// gn
	"Arandur&atilde;";
#elif L==7	// it
	"Laurea";
#elif L==8	// pl
	"Stopien";
#elif L==9	// pt
	"Grau";
#endif

const char *Txt_degree =
#if   L==1	// ca
	"titulaci&oacute;";
#elif L==2	// de
	"Studiengang";
#elif L==3	// en
	"degree";
#elif L==4	// es
	"titulaci&oacute;n";
#elif L==5	// fr
	"&eacute;tude";
#elif L==6	// gn
	"arandur&atilde;";
#elif L==7	// it
	"laurea";
#elif L==8	// pl
	"stopien";
#elif L==9	// pt
	"grau";
#endif

const char *Txt_DEGREE_STATUS[Deg_NUM_STATUS_TXT] =
   {
#if   L==1	// ca	// Deg_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==2	// de
	 "Unbekannt"
#elif L==3	// en
	 "Unknown"
#elif L==4	// es
	 "Desconocido"
#elif L==5	// fr
	 "Inconnu"
#elif L==6	// gn
	 "Desconocido"			// Okoteve traduccin
#elif L==7	// it
	 "Sconosciuto"
#elif L==8	// pl
	 "Nieznany"
#elif L==9	// pt
	 "Desconhecido"
#endif
	 ,
#if   L==1	// ca	// Deg_STATUS_ACTIVE
	 "Activa"
#elif L==2	// de
	 "Aktive"
#elif L==3	// en
	 "Active"
#elif L==4	// es
	 "Activa"
#elif L==5	// fr
	 "Actif"
#elif L==6	// gn
	 "Activa"	// Okoteve traduccin
#elif L==7	// it
	 "Attiva"
#elif L==8	// pl
	 "Aktywne"
#elif L==9	// pt
	 "Ativa"
#endif
	 ,
#if   L==1	// ca	// Deg_STATUS_PENDING
	 "Nova"
#elif L==2	// de
	 "Neu"
#elif L==3	// en
	 "New"
#elif L==4	// es
	 "Nueva"
#elif L==5	// fr
	 "Nouveau"
#elif L==6	// gn
	 "Pyahu"
#elif L==7	// it
	 "Nuovo"
#elif L==8	// pl
	 "Nowy"
#elif L==9	// pt
	 "Nova"
#endif
	 ,
#if   L==1	// ca	// Deg_STATUS_REMOVED
	 "Eliminada"
#elif L==2	// de
	 "Entfernt"
#elif L==3	// en
	 "Removed"
#elif L==4	// es
	 "Eliminada"
#elif L==5	// fr
	 "Supprim&eacute;"
#elif L==6	// gn
	 "Eliminada"	// Okoteve traduccin
#elif L==7	// it
	 "Rimossa"
#elif L==8	// pl
	 "Usuniete"
#elif L==9	// pt
	 "Suprimida"
#endif
	};

const char *Txt_DEGREE_TYPES_HELP_ORDER[DT_NUM_ORDERS] =
	{
#if   L==1	// ca
	 "Ordenar por tipus de titulaci&oacute;"
#elif L==2	// de
	 "Nach Art des Grades sortieren"
#elif L==3	// en
	 "Sort by type of degree"
#elif L==4	// es
	 "Ordenar por tipo de titulaci&oacute;n"
#elif L==5	// fr
	 "Trier par type d'&eacute;tude"
#elif L==6	// gn
	 "Ordenar por tipo de titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per tipo di laurea"
#elif L==8	// pl
	 "Sortuj wed&lstrok;ug rodzaju stopnia"
#elif L==9	// pt
	 "Classificar por tipo de grau"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar per nombre de titulacions"
#elif L==2	// de
	 "Nach Anzahl der Studieng&auml;nge sortieren"
#elif L==3	// en
	 "Sort by no. of degrees"
#elif L==4	// es
	 "Ordenar por n&ordm; de titulaciones"
#elif L==5	// fr
	 "Trier par nombre d'&eacute;tudes"
#elif L==6	// gn
	 "Ordenar por n&ordm; de titulaciones"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per numero di lauree"
#elif L==8	// pl
	 "Sortuj wedlug stopni"
#elif L==9	// pt
	 "Classificar por n&ordm; de graus"
#endif
	};

const char *Txt_DEGREE_TYPES_ORDER[DT_NUM_ORDERS] =
	{
#if   L==1	// ca
	"Tipus de titulaci&oacute;"
#elif L==2	// de
	"Abschlussart"
#elif L==3	// en
	"Types of degree"
#elif L==4	// es
	"Tipos de titulaci&oacute;n"
#elif L==5	// fr
	"Types d'&eacute;tude"
#elif L==6	// gn
	"Arandur&atilde; l&aacute;ja"
#elif L==7	// it
	"Tipi di laurea"
#elif L==8	// pl
	"Rodzaje stopni"
#elif L==9	// pt
	"Tipos de grau"
#endif
	 ,
#if   L==1	// ca
	"Titulacions"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"Degrees"
#elif L==4	// es
	"Titulaciones"
#elif L==5	// fr
	"&Eacute;tudes"
#elif L==6	// gn
	"Arandur&atilde;"
#elif L==7	// it
	"Lauree"
#elif L==8	// pl
	"Stopnie"
#elif L==9	// pt
	"Graus"
#endif
	};

const char *Txt_DEGREE_With_courses =
#if   L==1	// ca
	"Amb assignatures";
#elif L==2	// de
	"Mit Kursen";
#elif L==3	// en
	"With courses";
#elif L==4	// es
	"Con asignaturas";
#elif L==5	// fr
	"Avec des mati&egrave;res";
#elif L==6	// gn
	"Con asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Con corsi";
#elif L==8	// pl
	"Z kurs&oacute;w";
#elif L==9	// pt
	"Com disciplinas";
#endif

const char *Txt_DEGREE_Without_courses =
#if   L==1	// ca
	"Sense assignatures";
#elif L==2	// de
	"Ohne Kursen";
#elif L==3	// en
	"Without courses";
#elif L==4	// es
	"Sin asignaturas";
#elif L==5	// fr
	"Sans mati&egrave;res";
#elif L==6	// gn
	"Sin asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Senza corsi";
#elif L==8	// pl
	"Bez kurs&oacute;w";
#elif L==9	// pt
	"Sem disciplinas";
#endif

const char *Txt_Degree_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2	// de
	"Studiengang <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Degree <strong>%s</strong> removed.";
#elif L==4	// es
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Degree <strong>%s</strong> removed.";			// Besoin de traduction
#elif L==6	// gn
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Laurea <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Studia <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Grau <strong>%s</strong> removido.";
#endif

const char *Txt_Degrees =
#if   L==1	// ca
	"Titulacions";
#elif L==2	// de
	"Studiengang";
#elif L==3	// en
	"Degrees";
#elif L==4	// es
	"Titulaciones";
#elif L==5	// fr
	"&Eacute;tudes";
#elif L==6	// gn
	"Arandur&atilde;";
#elif L==7	// it
	"Lauree";
#elif L==8	// pl
	"Stopnie";
#elif L==9	// pt
	"Graus";
#endif

const char *Txt_Degrees_ABBREVIATION =
#if   L==1	// ca
	"Titul.";
#elif L==2	// de
	"Studieng.";
#elif L==3	// en
	"Degrees";
#elif L==4	// es
	"Titul.";
#elif L==5	// fr
	"&Eacute;tudes";
#elif L==6	// gn
	"Titul.";	// Okoteve traduccin
#elif L==7	// it
	"Lauree";
#elif L==8	// pl
	"Stopni";
#elif L==9	// pt
	"Titul.";
#endif

const char *Txt_degrees =
#if   L==1	// ca
	"titulacions";
#elif L==2	// de
	"Studiengang";
#elif L==3	// en
	"degrees";
#elif L==4	// es
	"titulaciones";
#elif L==5	// fr
	"&eacute;tudes";
#elif L==6	// gn
	"arandur&atilde;";
#elif L==7	// it
	"lauree";
#elif L==8	// pl
	"stopnie";
#elif L==9	// pt
	"graus";
#endif

const char *Txt_Degrees_of_CENTRE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Titulacions de %s";
#elif L==2	// de
	"Studiengang von %s";
#elif L==3	// en
	"Degrees of %s";
#elif L==4	// es
	"Titulaciones de %s";
#elif L==5	// fr
	"&Eacute;tudes de %s";
#elif L==6	// gn
	"Titulaciones de %s";	// Okoteve traduccin
#elif L==7	// it
	"Lauree di %s";
#elif L==8	// pl
	"Stopnie %s";
#elif L==9	// pt
	"Graus de %s";
#endif

const char *Txt_Degrees_with_pending_courses =
#if   L==1	// ca
	"Titulacions amb assignatures pendents d'activar";
#elif L==2	// de
	"Studiengang mit anh&auml;ngig Kursen";
#elif L==3	// en
	"Degrees with pending courses";
#elif L==4	// es
	"Titulaciones con asignaturas pendientes de activar";
#elif L==5	// fr
	"&Eacute;tudes avec des mati&egrave;res en attente";
#elif L==6	// gn
	"Titulaciones con asignaturas pendientes de activar";	// Okoteve traduccin
#elif L==7	// it
	"Lauree con corsi in sospeso";
#elif L==8	// pl
	"Stopnie z kurs&otilde;w oczekuj&aogon;cy";
#elif L==9	// pt
	"Graus com disciplinas pendentes";
#endif

const char *Txt_Delete_messages_received =
#if   L==1	// ca
	"Eliminar missatges rebuts";
#elif L==2	// de
	"Entfernen empfangene Nachrichten";
#elif L==3	// en
	"Delete messages received";
#elif L==4	// es
	"Eliminar mensajes recibidos";
#elif L==5	// fr
	"&Eacute;liminer messages re&ccedil;us";
#elif L==6	// gn
	"Eliminar mensajes recibidos";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere messaggi ricevuti";
#elif L==8	// pl
	"Usu&nacute; wiadomo&sacute;ci otrzymane";
#elif L==9	// pt
	"Eliminar mensagens recebidas";
#endif

const char *Txt_Delete_messages_sent =
#if   L==1	// ca
	"Eliminar mensajes enviados";	// Necessita traduccio
#elif L==2	// de
	"Entfernen gesendete Nachrichten";
#elif L==3	// en
	"Delete messages sent";
#elif L==4	// es
	"Eliminar mensajes enviados";
#elif L==5	// fr
	"&Eacute;liminer messages envoy&eacute;s";
#elif L==6	// gn
	"Eliminar mensajes enviados";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere messaggi inviati";
#elif L==8	// pl
	"Usu&nacute; wiadomo&sacute;ci wyslanych";
#elif L==9	// pt
	"Eliminar mensagens enviadas";
#endif

const char *Txt_Department =
#if   L==1	// ca
	"Departament";
#elif L==2	// de
	"Abteilung";
#elif L==3	// en
	"Department";
#elif L==4	// es
	"Departamento";
#elif L==5	// fr
	"D&eacute;partement";
#elif L==6	// gn
	"Peh&#7869;";
#elif L==7	// it
	"Dipartimento";
#elif L==8	// pl
	"Wydzia&lstrok;";
#elif L==9	// pt
	"Departamento";
#endif

const char *Txt_Department_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Departamento <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2	// de
	"Abteilung <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Department <strong>%s</strong> removed.";
#elif L==4	// es
	"Departamento <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"D&eacute;partement <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Departamento <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Dipartamento <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Zaklad <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Departamento <strong>%s</strong> removido.";
#endif

const char *Txt_Department_unspecified =
#if   L==1	// ca
	"Departament sense especificar";
#elif L==2	// de
	"Abteilung nicht angegeben";
#elif L==3	// en
	"Department unspecified";
#elif L==4	// es
	"Departamento sin especificar";
#elif L==5	// fr
	"D&eacute;partement non sp&eacute;cifi&eacute;";
#elif L==6	// gn
	"Departamento sin especificar";	// Okoteve traduccin
#elif L==7	// it
	"Dipartimento non specificato";
#elif L==8	// pl
	"Departament nieokreslone";
#elif L==9	// pt
	"Departamento indeterminado";
#endif

const char *Txt_Departments =
#if   L==1	// ca
	"Departaments";
#elif L==2	// de
	"Abteilungen";
#elif L==3	// en
	"Departments";
#elif L==4	// es
	"Departamentos";
#elif L==5	// fr
	"D&eacute;partements";
#elif L==6	// gn
	"Peh&#7869;";
#elif L==7	// it
	"Dipartimenti";
#elif L==8	// pl
	"Wydzia&lstrok;y";
#elif L==9	// pt
	"Departamentos";
#endif

const char *Txt_Departments_ABBREVIATION =
#if   L==1	// ca
	"Depar.";
#elif L==2	// de
	"Abteilung";
#elif L==3	// en
	"Depar";
#elif L==4	// es
	"Depar.";
#elif L==5	// fr
	"Depar.";
#elif L==6	// gn
	"Depar.";	// Okoteve traduccin
#elif L==7	// it
	"Dipar.";
#elif L==8	// pl
	"Wydzialy";
#elif L==9	// pt
	"Depar.";
#endif

const char *Txt_DEPARTMENTS_HELP_ORDER[Dpt_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Ordenar por departamento"	// Necessita traduccio
#elif L==2	// de
	 "Sortieren nach Abteilung"
#elif L==3	// en
	 "Sort by department"
#elif L==4	// es
	 "Ordenar por departamento"
#elif L==5	// fr
	 "Trier par d&eacute;partement"
#elif L==6	// gn
	 "Ordenar por departamento"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per dipartimento"
#elif L==8	// pl
	 "Sortuj wedlug dzial&oacute;w"
#elif L==9	// pt
	 "Classificar por departamento"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar por n&ordm; de profesores"	// Necessita traduccio
#elif L==2	// de
	 "Sortieren nach Anzahl der Lehrkr&auml;fte"
#elif L==3	// en
	 "Sort by no. of teachers"
#elif L==4	// es
	 "Ordenar por n&ordm; de profesores"
#elif L==5	// fr
	 "Trier par nombre d'enseignants"
#elif L==6	// gn
	 "Ordenar por n&ordm; de profesores"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per numero di professori"
#elif L==8	// pl
	 "Sortuj wedlug liczby nauczycieli"
#elif L==9	// pt
	 "Classificar por n&ordm; de professores"
#endif
	};

const char *Txt_DEPARTMENTS_ORDER[Dpt_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Departament"
#elif L==2	// de
	 "Abteilung"
#elif L==3	// en
	 "Department"
#elif L==4	// es
	 "Departamento"
#elif L==5	// fr
	 "D&eacute;partement"
#elif L==6	// gn
	 "Peh&#7869;"
#elif L==7	// it
	 "Dipartimento"
#elif L==8	// pl
	 "Wydzia&lstrok;"
#elif L==9	// pt
	 "Departamento"
#endif
	 ,
#if   L==1	// ca
	 "Professors"
#elif L==2	// de
	 "Lehrkr&auml;fte"
#elif L==3	// en
	 "Teachers"
#elif L==4	// es
	 "Profesores"
#elif L==5	// fr
	 "Enseignants"
#elif L==6	// gn
	 "Profesores"	// Okoteve traduccin
#elif L==7	// it
	 "Professori"
#elif L==8	// pl
	 "Nauczycieli"
#elif L==9	// pt
	 "Professores"
#endif
	};

const char *Txt_Description =
#if   L==1	// ca
	"Descripci&oacute;";
#elif L==2	// de
	"Beschreibung";
#elif L==3	// en
	"Description";
#elif L==4	// es
	"Descripci&oacute;n";
#elif L==5	// fr
	"Description";
#elif L==6	// gn
	"Descripci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Descrizione";
#elif L==8	// pl
	"Opis";
#elif L==9	// pt
	"Descri&ccedil;&atilde;o";
#endif

const char *Txt_Design =
#if   L==1	// ca
	"Disseny";
#elif L==2	// de
	"Design";
#elif L==3	// en
	"Design";
#elif L==4	// es
	"Dise&ntilde;o";
#elif L==5	// fr
	"Design";
#elif L==6	// gn
	"Ta'&atilde;ngahai";
#elif L==7	// it
	"Disegno";
#elif L==8	// pl
	"Design";
#elif L==9	// pt
	"Design";
#endif

const char *Txt_Details =
#if   L==1	// ca
	"Detalls";
#elif L==2	// de
	"Einzelheiten";
#elif L==3	// en
	"Details";
#elif L==4	// es
	"Detalles";
#elif L==5	// fr
	"D&eacute;tails";
#elif L==6	// gn
	"Detalles";	// Okoteve traduccin
#elif L==7	// it
	"Dettagli";
#elif L==8	// pl
	"Szczeg&oacute;&lstrok;y";
#elif L==9	// pt
	"Detalhes";
#endif

const char *Txt_Disclaimer_the_files_hosted_here_ =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Aviso legal: los archivos alojados aqu&iacute;,"
	" salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";	// Necessita traduccio
#elif L==2	// de
	"Disclaimer: the files hosted here,"
	" unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Need bersetzung
#elif L==3	// en
	"Disclaimer: the files hosted here,"
	" unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";
#elif L==4	// es
	"Aviso legal: los archivos alojados aqu&iacute;,"
	" salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";
#elif L==5	// fr
	"Disclaimer: the files hosted here,"
	" unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Besoin de traduction
#elif L==6	// gn
	"Aviso legal: los archivos alojados aqu&iacute;,"
	" salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";	// Okoteve traduccin
#elif L==7	// it
	"Disclaimer: the files hosted here,"
	" unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Bisogno di traduzione
#elif L==8	// pl
	"Disclaimer: the files hosted here,"
	" unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Isen&ccedil;&atilde;o de responsabilidade: os arquivos hospedados aqui,"
	" salvo indica&ccedil;&atilde;o em contr&aacute;rio,"
	" est&atilde;o sujeitos a direitos de propriedade intelectual"
	" e sua propriedade pertence aos usu&aacute;rios que fizeram o upload deles."
	" %s n&atilde;o &eacute; respons&aacute;vel pelas informa&ccedil;&otilde;es contidas nesses arquivos."
	" Se voc&ecirc; acha conveniente remover qualquer arquivo cujo conte&uacute;do perten&ccedil;a a voc&ecirc;"
	" ou que viole a lei, entre em contato com %s.";
#endif

const char *Txt_Display_photos =
#if   L==1	// ca
	"Incloure fotos";
#elif L==2	// de
	"Bilder anzeigen";
#elif L==3	// en
	"Display photos";
#elif L==4	// es
	"Incluir fotos";
#elif L==5	// fr
	"Inclure des photos";
#elif L==6	// gn
	"Incluir fotos";	// Okoteve traduccin
#elif L==7	// it
	"Mostra foto";
#elif L==8	// pl
	"Wyswietlanie zdjec";
#elif L==9	// pt
	"Incluir fotos";
#endif

const char *Txt_distributed_by =
#if   L==1	// ca
	"distribu&iuml;t per";
#elif L==2	// de
	"vertrieben von";
#elif L==3	// en
	"distributed by";
#elif L==4	// es
	"distribuido por";
#elif L==5	// fr
	"distribu&eacute; par";
#elif L==6	// gn
	"distribuido por";	// Okoteve traduccin
#elif L==7	// it
	"distribuito da";
#elif L==8	// pl
	"wyplacanych przez";
#elif L==9	// pt
	"distribu&iacute;do por";
#endif

const char *Txt_Do_not_show_again =
#if   L==1	// ca
	"No mostrar de nou";
#elif L==2	// de
	"Nicht mehr anzeigen";
#elif L==3	// en
	"Do not show again";
#elif L==4	// es
	"No mostrar de nuevo";
#elif L==5	// fr
	"Ne plus afficher";
#elif L==6	// gn
	"No mostrar de nuevo";	// Okoteve traduccin
#elif L==7	// it
	"Non mostrare pi&ugrave;";
#elif L==8	// pl
	"Nie pokazuj ponownie";
#elif L==9	// pt
	"N&atilde;o mostrar novamente";
#endif

const char *Txt_Do_you_really_want_to_be_removed_as_an_administrator_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea dejar de ser administrador/a"
	" de <strong>%s</strong>?";		// Necessita traduccio
#elif L==2	// de
	"Do you really want to be removed as an administrator"
	" of <strong>%s</strong>?";		// Need bersetzung
#elif L==3	// en
	"Do you really want to be removed as an administrator"
	" of <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea dejar de ser administrador/a"
	" de <strong>%s</strong>?";
#elif L==5	// fr
	"Do you really want to be removed as an administrator"
	" of <strong>%s</strong>?";		// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea dejar de ser administrador/a"
	" de <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7	// it
	"Vuole realmente essere rimosso come amministratore"
	" de <strong>%s</strong>?";
#elif L==8	// pl
	"Czy naprawde chcesz byc usuniety jako administrator"
	" <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer ser removido/a como administrador/a"
	" de <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_be_removed_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea ser eliminado de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrolment in groups, etc.)?";			// Need bersetzung
#elif L==3	// en
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrolment in groups, etc.)?";
#elif L==4	// es
	"&iquest;Realmente desea ser eliminado/a de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";
#elif L==5	// fr
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrolment in groups, etc.)?";			// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea ser eliminado/a de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente essere rimosso dal corso <strong>%s</strong>"
	" (rimuovendo i tuoi lavori, registrazione in gruppi, etc.)?";
#elif L==8	// pl
	"Czy na pewno chcesz zostac usuniete z kursu <strong>%s</strong>"
	" (usuwanie swoja prace, rejestracja w grupach, itp.)?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer ser removido/a da disciplina <strong>%s</strong>"
	" (removendo seus trabalhos, inscri&ccedil;&atilde;o em grupos, etc.)?";
#endif

const char *Txt_Do_you_really_want_to_be_removed_as_a_X_from_the_project_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu ser eliminat"
	" como a %s del projecte <strong>%s</strong>?";
#elif L==2	// de
        "M&ouml;chten Sie wirklich"
        " als %s aus dem Projekt <strong>%s</strong> entfernt werden?";
#elif L==3	// en
	"Do you really want to be removed"
	" as a %s from the project <strong>%s</strong>";
#elif L==4	// es
	"&iquest;Realmente desea ser eliminado"
	" como %s del proyecto <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment &ecirc;tre retir&eacute;"
	" en tant que %sdu projet <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea ser eliminado"
	" como %s del proyecto <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere"
	" come %s dal progetto <strong>%s</strong>?";
#elif L==8	// pl
	"Czy naprawd&eogon; chcesz zosta&cacute; usuni&eogon;ty"
	" jako %s z projektu <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer ser removido"
	" como %s do projeto <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_completely_eliminate_the_following_user =
#if   L==1	// ca
	"&iquest;Realmente desea eliminar completamente la cuenta del siguiente usuario?";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to completely eliminate the following user?";			// Need bersetzung
#elif L==3	// en
	"Do you really want to completely eliminate the following user?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar por completo la cuenta del siguiente usuario/a?";
#elif L==5	// fr
	"Do you really want to completely eliminate the following user?";			// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar por completo la cuenta del siguiente usuario/a?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere totalmente il seguente utente?";
#elif L==8	// pl
	"Czy na pewno chcesz, aby calkowicie wyeliminowac nastepujacych uzytkownik&oacute;w?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer eliminar completamente o seguinte usu&aacute;rio/a?";
#endif

const char *Txt_Do_you_really_want_to_completely_eliminate_your_user_account =
#if   L==1	// ca
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario/a?";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to completely eliminate your user account?";		// Need bersetzung
#elif L==3	// en
	"Do you really want to completely eliminate your user account?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario/a?";
#elif L==5	// fr
	"Do you really want to completely eliminate your user account?";		// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario/a?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere totalmente il tuo account?";
#elif L==8	// pl
	"Czy na pewno chcesz, aby calkowicie wyeliminowac konta uzytkownika?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer eliminar completamente sua conta de usu&aacute;rio?";
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_received_from_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to delete all messages received from %s from %s?";			// Need bersetzung
#elif L==3	// en
	"Do you really want to delete all messages received from %s from %s?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";
#elif L==5	// fr
	"Do you really want to delete all messages received from %s from %s?";			// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi ricevuti da %s da %s?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac wszystkie wiadomosci otrzymane od %s z %s?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar todas as mensagens recebidas de %s de %s?";
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_received_from_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s relacionados con <strong>%s</strong>?";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";			// Need bersetzung
#elif L==3	// en
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s relacionados con <strong>%s</strong>?";
#elif L==5	// fr
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";			// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi ricevuti da %s da %s relativi a <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac wszystkie wiadomosci otrzymane od %s z %s podobne do <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar todas as mensagens recebidas de %s de %s relacionadas com <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_sent_to_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to delete all messages sent to %s from %s?";		// Need bersetzung
#elif L==3	// en
	"Do you really want to delete all messages sent to %s from %s?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";
#elif L==5	// fr
	"Do you really want to delete all messages sent to %s from %s?";		// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi inviati a %s da %s?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac wszystkie wiadomosci wyslane do %s z %s?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar todas as mensagens enviadas para %s de %s?";
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_sent_to_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";				// Need bersetzung
#elif L==3	// en
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";
#elif L==5	// fr
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";				// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi inviati a %s da %s relativi a<strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac wszystkie wiadomosci wyslane do %s z %s podobne do <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar todas as mensagens enviadas para %s de %s relacionadas com <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_delete_the_unread_messages_received_from_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to delete the unread messages received from %s from %s?";			// Need bersetzung
#elif L==3	// en
	"Do you really want to delete the unread messages received from %s from %s?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";
#elif L==5	// fr
	"Do you really want to delete the unread messages received from %s from %s?";			// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi non letti ricevuti da %s da %s?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac nieprzeczytanych wiadomosci otrzymane od %s z %s?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar as mensagens n&atilde;o lidas recebidas de %s de %s?";
#endif

const char *Txt_Do_you_really_want_to_delete_the_unread_messages_received_from_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos"
	" recibidos de %s desde %s relacionados con <strong>%s</strong>?";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to delete the unread messages"
	" received from %s from %s related to <strong>%s</strong>?";		// Need bersetzung
#elif L==3	// en
	"Do you really want to delete the unread messages"
	" received from %s from %s related to <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos"
	" recibidos de %s desde %s relacionados con <strong>%s</strong>?";
#elif L==5	// fr
	"Do you really want to delete the unread messages"
	" received from %s from %s related to <strong>%s</strong>?";		// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos"
	" recibidos de %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere tutti i messaggi non letti"
	" ricevuti da %s da %s relativi a <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac nieprzeczytanych wiadomosci"
	" otrzymane od %s z %s podobne do <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja eliminar as mensagens n&atilde;o lidas"
	" recebidas de %s de %s relacionadas com <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_register_the_following_user_as_an_administrator_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea inscribir el siguiente usuario"
	" como administrador/a de <strong>%s</strong>?";		// Necessita traduccio
#elif L==2	// de
	"Do you really want to register the following user"
	" as an administrator of <strong>%s</strong>?";		// Need bersetzung
#elif L==3	// en
	"Do you really want to register the following user"
	" as an administrator of <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea inscribir el siguiente usuario/a"
	" como administrador/a de <strong>%s</strong>?";
#elif L==5	// fr
	"Do you really want to register the following user"
	" as an administrator of <strong>%s</strong>?";		// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea inscribir el siguiente usuario/a"
	" como administrador/a de <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente registrare il seguente utente"
	" come amministratore de <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz sie zarejestrowac nastepujace u&zdot;ytkownika"
	" jako administratora <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja registrar o seguinte usu&aacute;rio/a"
	" como administrador/a de <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_reject_the_enrolment_request_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";		// Necessita traduccio
#elif L==2	// de
	"Do you really want to reject the enrolment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Need bersetzung
#elif L==3	// en
	"Do you really want to reject the enrolment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";
#elif L==4	// es
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";
#elif L==5	// fr
	"Do you really want to reject the enrolment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Besoin de traduction
#elif L==6	// gn
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7	// it
	"Do you really want to reject the enrolment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Bisogno di traduzione
#elif L==8	// pl
	"Do you really want to reject the enrolment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; realmente quer rejeitar a solicita&ccedil;&atilde;o de inscri&ccedil;&atilde;o"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" na disciplina <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_X_students_from_the_course_Y_ =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y su inscripci&oacute;n en grupos de esta asignatura.</strong>.";		// Necessita traduccio
#elif L==2	// de
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enroled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrolment in groups of this course will be removed.</strong>.";	// Need bersetzung
#elif L==3	// en
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enroled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrolment in groups of this course will be removed.</strong>.";
#elif L==4	// es
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y su inscripci&oacute;n en grupos de esta asignatura.</strong>.";
#elif L==5	// fr
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enroled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrolment in groups of this course will be removed.</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y su inscripci&oacute;n en grupos de esta asignatura.</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere gli %u studenti dal corso <strong>%s</strong>?<br />"
	"Sebbene i dati degli studenti non saranno rimossi in altri corsi,"
	" essi non saranno pi&ugrave; registrati in questo corso,"
	" e cos <strong>tutti i lavori presentati da loro in questo corso,"
	" le loro schede per questo corso,"
	" e la loro registrazione nei gruppi di questo corso saranno rimossi.</strong>.";
#elif L==8	// pl
	"Czy na pewno chcesz usunac %u student&oacute;w z kursu <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" nie beda juz zarejestrowane w tym kurs"
	" a takze <strong>wszystkie prace zlozone przez nich w tym kursie,"
	" swoich kart zapisu na ten kurs,"
	" i ich rejestracja w grupach zajec zostana usuniete.</strong>.";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover os %u estudantes do curso <strong>%s</strong>?<br />"
	"Embora os dados dos estudantes n&atilde;o sejam removidos em outras disciplinas,"
	" eles n&atilde;o ser&atilde;o mais inscritos nesta disciplina,"
	" e tamb&eacute;m ser&atilde;o removidos todos os trabalhos enviados por eles nesta disciplina,"
	" suos cart&otilde;es para esta disciplina,"
	" e sua inscri&ccedil;&atilde;o em grupos desta disciplina.";
#endif

const char *Txt_Do_you_really_want_to_remove_the_assignment_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu eliminar l'activitat <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Aufgabe <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the assignment <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'activit&eacute; <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere l'attivit&agrave; <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac przypisanie <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover a atividade <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_entire_thread =
#if   L==1	// ca
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n?";	// Necessita traduccio
#elif L==2	// de
	"Wollen Sie die gesamten Thread wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the entire thread?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer tout le fil?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere l'intera discussione?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac caly watek?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover toda a discuss&atilde;o?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_entire_thread_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";	// Necessita traduccio
#elif L==2	// de
	"Wollen Sie die gesamten Thread <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the entire thread <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer tout le fil <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere l'intera discussione <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac caly watek <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover toda a discuss&atilde;o <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_event_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";	// Necessita traduccio
#elif L==2	// de
	"Wollen Sie der Ereignis <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the event <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'&eacute;v&eacute;nement <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere l'evento <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac wydarzenie <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o evento <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_field_X_from_the_records_of_Y_Z_ =	// Warning: it is very important to include two %s and one %u in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong>"
	" de las fichas de <strong>%s</strong>"
	" (dicho campo est&aacute; relleno en las fichas de %u estudiante(s))?";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to remove the field <strong>%s</strong>"
	" from the records of <strong>%s</strong>"
	" (this field is filled in the records of %u student(s))?";			// Need bersetzung
#elif L==3	// en
	"Do you really want to remove the field <strong>%s</strong>"
	" from the records of <strong>%s</strong>"
	" (this field is filled in the records of %u student(s))?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong>"
	" de las fichas de <strong>%s</strong>"
	" (dicho campo est&aacute; relleno en las fichas de %u estudiante(s))?";
#elif L==5	// fr
	"Do you really want to remove the field <strong>%s</strong>"
	" from the records of <strong>%s</strong>"
	" (this field is filled in the records of %u student(s))?";		// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong>"
	" de las fichas de <strong>%s</strong>"
	" (dicho campo est&aacute; relleno en las fichas de %u estudiante(s))?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il campo <strong>%s</strong>"
	" dalle schede di <strong>%s</strong>"
	" (questo campo &egrave; inserito nella scheda di %u studenti)?";
#elif L==8	// pl
	"Do you really want to remove the field <strong>%s</strong>"
	" from the records of <strong>%s</strong>"
	" (this field is filled in the records of %u student(s))?";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; realmente quer remover o campo <strong>%s</strong>"
	" dos cart&otilde;es de <strong>%s</strong>"
	" (este campo &eacute; preenchido nos cart&otilde;es de %u estudante(s))?";
#endif

const char *Txt_Do_you_really_want_to_remove_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";	// Necessita traduccio
#elif L==2	// de
	"Wollen Sie <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar"
	" la carpeta <strong>%s</strong> y todo su contenido?";		// Necessita traduccio
#elif L==2	// de
	"Wollen Sie wirklich"
	" das Verzeichnis <strong>%s</strong> und alle Inhalte entfernen?";
#elif L==3	// en
	"Do you really want to remove"
	" the folder <strong>%s</strong> and all its contents?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar"
	" la carpeta <strong>%s</strong> y todo su contenido?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer"
	" le r&eacute;pertoire <strong>%s</strong> et l'ensemble de son contenu?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar"
	" la carpeta <strong>%s</strong> y todo su contenido?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere"
	" la cartella <strong>%s</strong> e tutto il suo contenuto?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac"
	" folder <strong>%s</strong> i wszystkie jego skladniki?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover"
	" o diret&oacute;rio <strong>%s</strong> e todo o seu conte&uacute;do?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_announcement_of_exam =
#if   L==1	// ca
	"De veres voleu eliminar la seg&uuml;ent convocat&ograve;ria d'examen?";
#elif L==2	// de
	"Wollen Sie wirklich, um die folgende Aufruf f&uuml;r Pr&uuml;fung?";
#elif L==3	// en
	"Do you really want to remove the following announcement of exam?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la siguiente convocatoria de examen?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer la convocation &agrave; un examen suivante?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la siguiente convocatoria de examen?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente appello d'esame?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nast&eogon;puj&aogon;c&aogon; zapowied&zacute; egzaminu?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover a seguinte chamada para exame?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_notice =
#if   L==1	// ca
	"De veres voleu eliminar el seg&uuml;ent av&iacute;s?";
#elif L==2	// de
	"Wollen Sie wirklich, um die folgende Ank&uuml;ndigung zu entfernen?";
#elif L==3	// en
	"Do you really want to remove the following notice?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el siguiente aviso?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'avis suivant?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el siguiente aviso?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente avviso?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac nastepujace obwieszczenie?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o seguinte aviso?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_comment =
#if   L==1	// ca
	"De veres voleu eliminar el seg&uuml;ent comentari?";
#elif L==2	// de
	"Wollen Sie wirklich, um die folgende Kommentar zu entfernen?";
#elif L==3	// en
	"Do you really want to remove the following comment?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el siguiente comentario?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le commentaire suivant?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el siguiente comentario?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente commento?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac nastepujace komentarz?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o seguinte coment&aacute;rio?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_post =
#if   L==1	// ca
	"De veres voleu eliminar la seg&uuml;ent publicaci&oacute;?";
#elif L==2	// de
	"Wollen Sie wirklich, um die folgende Beitrag zu entfernen?";
#elif L==3	// en
	"Do you really want to remove the following post?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la siguiente publicaci&oacute;n?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le post suivant?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la siguiente publicaci&oacute;n?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente post?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac nastepujace post?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o seguinte post?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_as_an_administrator_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador/a de <strong>%s</strong>?";		// Necessita traduccio
#elif L==2	// de
	"Do you really want to remove the following user"
	" as an administrator of <strong>%s</strong>?";			// Need bersetzung
#elif L==3	// en
	"Do you really want to remove the following user"
	" as an administrator of <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador/a de <strong>%s</strong>?";
#elif L==5	// fr
	"Do you really want to remove the following user"
	" as an administrator of <strong>%s</strong>?";			// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador/a de <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente utente"
	" come amministratore de <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac nastepujace u&zdot;ytkownika"
	" jako administratora <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o seguinte usu&aacute;rio"
	" como administrador de <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu eliminar el seg&uuml;ent usuari de l'assignatura <strong>%s</strong>"
	" (eliminant els seus treballs, adscripci&oacute; a grups, etc.)?";
#elif L==2	// de
        "M&ouml;chten Sie den folgenden Benutzer wirklich aus dem Kurs <strong>%s</strong> entfernen"
	" (Beseitigung ihrer Werke, Einschreibung in Gruppen usw.)?";
#elif L==3	// en
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrolment in groups, etc.)?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el siguiente usuario de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'utilisateur suivant de la mati&egrave;re <strong>%s</strong>"
	" (&eacute;liminant ses travaux, inscription en groupes, etc.)?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el siguiente usuario de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente utente dal corso <strong>%s</strong>"
	" (rimuovendo i suoi lavori, registrazione in gruppi, etc.)?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nast&eogon;puj&aogon;cy u&zdot;ytkownik z kursu <strong>%s</strong>"
	" (usuni&eogon;cie jego utwor&oacute;w, zapisanie si&eogon; w grupach itp.)?";
#elif L==9	// pt
	"Voc&ecirc; deseja realmente remover o seguinte usu&aacute;rio da disciplina <strong>%s</strong>"
	" (removendo seus trabalhos, inscri&ccedil;&atilde;o em grupos, etc.)?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_as_a_X_from_the_project_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu eliminar el seg&uuml;ent usuari"
	" com a %s del projecte <strong>%s</strong>?";
#elif L==2	// de
        "M&ouml;chten Sie den folgenden Benutzer wirklich"
        " als %s aus dem Projekt <strong>%s</strong> entfernen?";
#elif L==3	// en
	"Do you really want to remove the following user"
	" as a %s from the project <strong>%s</strong>";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como %s del proyecto <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer l'utilisateur suivant"
	" en tant que %s du projet <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como %s del proyecto <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il seguente utente"
	" como %s dal progetto <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; nast&eogon;puj&aogon;cy u&zdot;ytkownik"
	" jako %s z projektu <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; deseja realmente remover o seguinte usu&aacute;rio"
	" como %s do projeto <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_game_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar le joc <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Spiel <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the game <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el juego <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le jeu <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el juego <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il gioco <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac gra <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o jogo <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar el grup <strong>%s</strong>?";
#elif L==2	// de
	"Do you really want to remove the group <strong>%s</strong>?";		// Need bersetzung
#elif L==3	// en
	"Do you really want to remove the group <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?";
#elif L==5	// fr
	"Do you really want to remove the group <strong>%s</strong>?";		// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il gruppo <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac grupe <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente quer remover o grupo <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_group_X_1_student_ =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar el grup <strong>%s</strong>?<br />"
	"Si ho fa, un/a estudiant deixar&agrave; de pert&agrave;nyer a aquest grup.";
#elif L==2	// de
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";			// Need bersetzung
#elif L==3	// en
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, un/a estudiante dejar&aacute; de pertenecer a ese grupo.";
#elif L==5	// fr
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";			// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, un/a estudiante dejar&aacute; de pertenecer a ese grupo.";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai uno studente da questo gruppo.";
#elif L==8	// pl
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"W ten spos&oacute;b usunie jeden uczen z tej grupy.";
#elif L==9	// pt
	"Voc&ecirc; realmente quer remover o grupo <strong>%s</strong>?<br />"
	"Ao fazer isso, voc&ecirc; remover&aacute; um estudante desse grupo.";
#endif

const char *Txt_Do_you_really_want_to_remove_the_group_X_Y_students_ =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar el grup <strong>%s</strong>?<br />"
	"Si ho fa, %u estudiants deixaran de pert&agrave;nyer a aquest grup.";
#elif L==2	// de
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";			// Need bersetzung
#elif L==3	// en
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, %u estudiantes dejar&aacute;n de pertenecer a ese grupo.";
#elif L==5	// fr
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";			// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, %u estudiantes dejar&aacute;n de pertenecer a ese grupo.";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai %u studenti da questo gruppo.";
#elif L==8	// pl
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"W ten spos&oacute;b usunie %u uczni&oacute;w z tej grupy.";
#elif L==9	// pt
	"Voc&ecirc; realmente quer remover o grupo <strong>%s</strong>?<br />"
	"Ao fazer isso, voc&ecirc; remover&aacute; %u estudantes desse grupo.";
#endif

const char *Txt_Do_you_really_want_to_remove_the_photo_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar la foto de <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die wirklich das Foto von <strong>%s</strong> zu entfernen?";
#elif L==3	// en
	"Do you really want to remove the photo of <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la foto de <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer la photo de <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la foto de <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere la foto di <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; zdj&eogon;cie <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover la foto de <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_project_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;De veres voleu eliminar el projecte <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Projekt <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the project <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el proyecto <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le projet <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el proyecto <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il progetto <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac projekt <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o projeto <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_question_X =	// Warning: it is very important to include %lu in the following sentences
#if   L==1	// ca
	"De veres voleu eliminar pregunta <strong>%lu</strong>?";
#elif L==2	// de
	"Wollen Sie die Frage <strong>%lu</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the question <strong>%lu</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la pregunta <strong>%lu</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer la question <strong>%lu</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la pregunta <strong>%lu</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere la domanda <strong>%lu</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac pytanie <strong>%lu</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover a quest&atilde;o <strong>%lu</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Realment voleu eliminar l'enquesta <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Umfrage <strong>%s</strong> wirklich entfernen?";
#elif L==3	// en
	"Do you really want to remove the survey <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar la encuesta <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer le sondage <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar la encuesta <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il sondaggio <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz usunac badania <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover o inqu&eacute;rito <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_type_of_group_X_1_group_ =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n un grupo de ese tipo.";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";			// Need bersetzung
#elif L==3	// en
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n un grupo de ese tipo.";
#elif L==5	// fr
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";			// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n un grupo de ese tipo.";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il tipo di gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai anche un gruppo di questo tipo.";
#elif L==8	// pl
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Spowoduje to takze usuniecie jednej grupie tego typu.";
#elif L==9	// pt
	"Voc&ecirc; realmente quer remover o tipo de grupo <strong>%s</strong>?<br />"
	"Isso tamb&eacute;m remover&aacute; um grupo desse tipo.";
#endif

const char *Txt_Do_you_really_want_to_remove_the_type_of_group_X_Y_groups_ =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1	// ca
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n %u grupos de ese tipo.";	// Necessita traduccio
#elif L==2	// de
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";			// Need bersetzung
#elif L==3	// en
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";
#elif L==4	// es
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n %u grupos de ese tipo.";
#elif L==5	// fr
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";			// Besoin de traduction
#elif L==6	// gn
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n %u grupos de ese tipo.";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere il tipo di gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai anche %u gruppi di questo tipo.";
#elif L==8	// pl
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Spowoduje to r&oacute;wniez usuniecie grup %u tego typu.";
#elif L==9	// pt
	"Voc&ecirc; realmente quer remover o tipo de grupo <strong>%s</strong>?<br />"
	"Isso tamb&eacute;m remover&aacute; %u grupos desse tipo.";
#endif

const char *Txt_Do_you_really_want_to_remove_your_photo =
#if   L==1	// ca
	"De veres voleu eliminar la seva foto?";
#elif L==2	// de
	"Wollen Sie die Ihr Foto zu entfernen?";
#elif L==3	// en
	"Do you really want to remove your photo?";
#elif L==4	// es
	"&iquest;Realmente desea eliminar su foto?";
#elif L==5	// fr
	"Voulez-vous vraiment supprimer votre photo?";
#elif L==6	// gn
	"&iquest;Realmente desea eliminar su foto?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente rimuovere la tua foto?";
#elif L==8	// pl
	"Czy na pewno chcesz usun&aogon;&cacute; swoje zdj&eogon;cie?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja remover a sua foto?";
#endif

const char *Txt_Do_you_really_want_to_reset_the_game_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu posar a zero le joc <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Spiel <strong>%s</strong> wirklich reset?";
#elif L==3	// en
	"Do you really want to reset the game <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea poner a cero el juego <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment reset le jeu <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea poner a cero el juego <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente resettare il gioco <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz zresetowac gra <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja reiniciar o jogo <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_reset_the_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"De veres voleu posar a zero l'enquesta <strong>%s</strong>?";
#elif L==2	// de
	"Wollen Sie die Umfrage <strong>%s</strong> wirklich reset?";
#elif L==3	// en
	"Do you really want to reset the survey <strong>%s</strong>?";
#elif L==4	// es
	"&iquest;Realmente desea poner a cero la encuesta <strong>%s</strong>?";
#elif L==5	// fr
	"Voulez-vous vraiment reset le sondage <strong>%s</strong>?";
#elif L==6	// gn
	"&iquest;Realmente desea poner a cero la encuesta <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi realmente resettare il sondaggio <strong>%s</strong>?";
#elif L==8	// pl
	"Czy na pewno chcesz zresetowac badania <strong>%s</strong>?";
#elif L==9	// pt
	"Voc&ecirc; realmente deseja reiniciar o inqu&eacute;rito <strong>%s</strong>?";
#endif

const char *Txt_Do_you_think_you_are_one_of_these_users =
#if   L==1	// ca
	"Creu que vost&egrave; &eacute;s un d'aquests usuaris?";
#elif L==2	// de
	"Denken Sie, Sie einer dieser Benutzer sind?";
#elif L==3	// en
	"Do you think you are one of these users?";
#elif L==4	// es
	"&iquest;Cree que usted es uno de estos usuarios?";
#elif L==5	// fr
	"Pensez-vous que vous &ecirc;tes un de ces utilisateurs?";
#elif L==6	// gn
	"&iquest;Cree que usted es uno de estos usuarios?";	// Okoteve traduccin
#elif L==7	// it
	"Pensi che sei uno di questi utenti?";
#elif L==8	// pl
	"My&sacute;lisz, &zdot;e jeste&sacute; jednym z tych u&zdot;ytkownik&oacute;w?";
#elif L==9	// pt
	"Voc&ecirc; acha que voc&ecirc; &eacute; um desses usu&aacute;rios?";
#endif

const char *Txt_Do_you_think_you_are_this_user =
#if   L==1	// ca
	"Creu que vost&egrave; &eacute;s aquest usuari?";
#elif L==2	// de
	"Denken Sie, Sie dieser Benutzer?";
#elif L==3	// en
	"Do you think you are this user?";
#elif L==4	// es
	"&iquest;Cree que usted es este usuario/a?";
#elif L==5	// fr
	"Pensez-vous que vous &ecirc;tes cet utilisateur?";
#elif L==6	// gn
	"&iquest;Cree que usted es este usuario/a?";	// Okoteve traduccin
#elif L==7	// it
	"Pensi che sei questo utente?";
#elif L==8	// pl
	"Czy uwa&zdot;asz, &zdot;e ta osoba?";
#elif L==9	// pt
	"Voc&ecirc; acha que voc&ecirc; &eacute; deste usu&aacute;rio?";
#endif

const char *Txt_Do_you_want_to_follow_the_selected_user_whom_you_do_not_follow_yet =
#if   L==1	// ca
	"Voleu seguir a l'usuari seleccionat a qui encara no segueix?";
#elif L==2	// de
	"M&ouml;chten Sie dem ausgew&auml;hlten Benutzer folgen, dem Sie noch nicht folgen?";
#elif L==3	// en
	"Do you want to follow the selected user whom you do not follow yet?";
#elif L==4	// es
	"&iquest;Desea seguir al usuario seleccionado que a&uacute;n no sigue?";
#elif L==5	// fr
	"Voulez-vous suivre l'utilisateur s&eacute;lectionn&eacute; que vous ne suivez pas encore?";
#elif L==6	// gn
	"&iquest;Desea seguir al usuario seleccionado que a&uacute;n no sigue?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi seguire l'utente selezionato che non segui ancora?";
#elif L==8	// pl
	"Czy chcesz &sacute;ledzi&cacute; wybranego u&zdot;ytkownika, kt&oacute;rego jeszcze nie &sacute;ledzisz?";
#elif L==9	// pt
	"Voc&ecirc; quer seguir o usu&aacute;rio selecionado que voc&ecirc; n&atilde;o segue ainda?";
#endif

const char *Txt_Do_you_want_to_follow_the_X_selected_users_whom_you_do_not_follow_yet =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Voleu seguir als %u usuaris seleccionats als que encara no segueix?";
#elif L==2	// de
	"M&ouml;chten Sie den %u ausgew&auml;hlten Benutzern folgen, denen Sie noch nicht folgen?";
#elif L==3	// en
	"Do you want to follow the %u selected users whom you do not follow yet?";
#elif L==4	// es
	"&iquest;Desea seguir a los %u usuarios seleccionados que a&uacute;n no sigue?";
#elif L==5	// fr
	"Voulez-vous suivre les %u utilisateurs s&eacute;lectionn&eacute;s que vous ne suivez pas encore?";
#elif L==6	// gn
	"&iquest;Desea seguir a los %u usuarios seleccionados que a&uacute;n no sigue?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi seguire i %u utenti selezionati che non segui ancora?";
#elif L==8	// pl
	"Czy chcesz &sacute;ledzi&cacute; %u wybranych u&zdot;ytkownik&oacute;w, kt&oacute;rych jeszcze nie &sacute;ledzisz?";
#elif L==9	// pt
	"Voc&ecirc; quer seguir os %u usu&aacute;rios selecionados que voc&ecirc; n&atilde;o segue ainda?";
#endif

const char *Txt_Do_you_want_to_stop_following_the_selected_user_whom_you_follow =
#if   L==1	// ca
	"Voleu deixar de seguir a l'usuari seleccionat a qui segueix?";
#elif L==2	// de
	"M&ouml;chten Sie dem ausgew&auml;hlten Benutzer, dem Sie folgen, nicht mehr folgen?";
#elif L==3	// en
	"Do you want to stop following the selected user whom you follow?";
#elif L==4	// es
	"&iquest;Desea dejar de seguir al usuario seleccionado que sigue?";
#elif L==5	// fr
	"Voulez-vous arr&ecirc;ter de suivre l'utilisateur s&eacute;lectionn&eacute; que vous suivez?";
#elif L==6	// gn
	"&iquest;Desea dejar de seguir al usuario seleccionado que sigue?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi smettere di seguire l'utente selezionato che segui?";
#elif L==8	// pl
	"Czy chcesz przesta&cacute; &sacute;ledzi&cacute; wybranego u&zdot;ytkownika, kt&oacute;rego &sacute;ledzisz?";
#elif L==9	// pt
	"Voc&ecirc; quer parar de seguir o usu&aacute;rio selecionado que voc&ecirc; segue?";
#endif

const char *Txt_Do_you_want_to_stop_following_the_X_selected_users_whom_you_follow =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Voleu deixar de seguir als %u usuaris seleccionats als que segueix?";
#elif L==2	// de
	"M&ouml;chten Sie nicht mehr den %u ausgew&auml;hlten Nutzern folgen, denen Sie folgen?";
#elif L==3	// en
	"Do you want to stop following the %u selected users whom you follow?";
#elif L==4	// es
	"&iquest;Desea dejar de seguir a los %u usuarios seleccionados que sigue?";
#elif L==5	// fr
	"Voulez-vous arr&ecirc;ter de suivre les %u utilisateurs s&eacute;lectionn&eacute;s que vous suivez?";
#elif L==6	// gn
	"&iquest;Desea dejar de seguir a los %u usuarios seleccionados que sigue?";	// Okoteve traduccin
#elif L==7	// it
	"Vuoi smettere di seguire i %u utenti selezionati che segui?";
#elif L==8	// pl
	"Czy chcesz przesta&cacute; &sacute;ledzi&cacute; %u wybranych u&zdot;ytkownik&oacute;w, kt&oacute;rych &sacute;ledzisz?";
#elif L==9	// pt
	"Voc&ecirc; quer parar de seguir os %u usu&aacute;rios selecionados que voc&ecirc; segue?";
#endif

const char *Txt_Document =
#if   L==1	// ca
	"Document";
#elif L==2	// de
	"Dokument";
#elif L==3	// en
	"Document";
#elif L==4	// es
	"Documento";
#elif L==5	// fr
	"Document";
#elif L==6	// gn
	"Documento";	// Okoteve traduccin
#elif L==7	// it
	"Documento";
#elif L==8	// pl
	"Document";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Documento";
#endif

const char *Txt_document_in_my_courses =
#if   L==1	// ca
	"document en les meves assignatures";
#elif L==2	// de
	"Dokument in meinen Kursen";
#elif L==3	// en
	"document in my courses";
#elif L==4	// es
	"documento en mis asignaturas";
#elif L==5	// fr
	"document &agrave; mes mati&egrave;res";
#elif L==6	// gn
	"documento en mis asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"documento in miei corsi";
#elif L==8	// pl
	"dokument w moich kursach";
#elif L==9	// pt
	"documento em minha disciplinas";
#endif

const char *Txt_Documents =
#if   L==1	// ca
	"Documents";
#elif L==2	// de
	"Dokumente";
#elif L==3	// en
	"Documents";
#elif L==4	// es
	"Documentos";
#elif L==5	// fr
	"Documents";
#elif L==6	// gn
	"Documentos";	// Okoteve traduccin
#elif L==7	// it
	"Documenti";
#elif L==8	// pl
	"Documents";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Documentos";
#endif

const char *Txt_document_from_me =
#if   L==1	// ca
	"document meu";
#elif L==2	// de
	"Dokument von mir";
#elif L==3	// en
	"document from me";
#elif L==4	// es
	"documento m&iacute;o";
#elif L==5	// fr
	"document de moi";
#elif L==6	// gn
	"documento m&iacute;o";	// Okoteve traduccin
#elif L==7	// it
	"documento da me";
#elif L==8	// pl
	"dokument ode mnie";
#elif L==9	// pt
	"documento de mim";
#endif

const char *Txt_documents_from_me =
#if   L==1	// ca
	"documents meus";
#elif L==2	// de
	"Dokumente von mir";
#elif L==3	// en
	"documents from me";
#elif L==4	// es
	"documentos m&iacute;os";
#elif L==5	// fr
	"documents de moi";
#elif L==6	// gn
	"documentos m&iacute;os";	// Okoteve traduccin
#elif L==7	// it
	"documenti da me";
#elif L==8	// pl
	"dokumenty ode mnie";
#elif L==9	// pt
	"documentos de mim";
#endif

const char *Txt_documents_in_my_courses =
#if   L==1	// ca
	"documents en les meves assignatures";
#elif L==2	// de
	"Dokumente in meinen Kursen";
#elif L==3	// en
	"documents in my courses";
#elif L==4	// es
	"documentos en mis asignaturas";
#elif L==5	// fr
	"documents &agrave; mes mati&egrave;res";
#elif L==6	// gn
	"documentos en mis asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"documenti in miei corsi";
#elif L==8	// pl
	"dokumenty w moich kursach";
#elif L==9	// pt
	"documentos em minha disciplinas";
#endif

const char *Txt_Documents_area =
#if   L==1	// ca
	"Zona de documents";
#elif L==2	// de
	"Dokumente-Bereich";
#elif L==3	// en
	"Documents area";
#elif L==4	// es
	"Zona de documentos";
#elif L==5	// fr
	"Zone de documents";
#elif L==6	// gn
	"Zona de documentos";	// Okoteve traduccin
#elif L==7	// it
	"Area documenti";
#elif L==8	// pl
	"Obszar Dokument&oacute;w";
#elif L==9	// pt
	"Zona de documentos";
#endif

const char *Txt_Documents_management_area =
#if   L==1	// ca
	"Zona d'administraci&oacute; de documents";
#elif L==2	// de
	"Dokumente-Verwaltung";
#elif L==3	// en
	"Documents management area";
#elif L==4	// es
	"Zona de administraci&oacute;n de documentos";
#elif L==5	// fr
	"Zone de gestion de documents";
#elif L==6	// gn
	"Zona de administraci&oacute;n de documentos";	// Okoteve traduccin
#elif L==7	// it
	"Area di gestione dei documenti";
#elif L==8	// pl
	"Obszar zarzadzania Dokumenty";
#elif L==9	// pt
	"Zona de gest&atilde;o de documentos";
#endif

const char *Txt_documents_management_area =
#if   L==1	// ca
	"zona de administraci&oacute;n de documentos";	// Necessita traduccio
#elif L==2	// de
	"Dokumente-Verwaltung";
#elif L==3	// en
	"documents management area";
#elif L==4	// es
	"zona de administraci&oacute;n de documentos";
#elif L==5	// fr
	"zone de gestion de documents";
#elif L==6	// gn
	"zona de administraci&oacute;n de documentos";	// Okoteve traduccin
#elif L==7	// it
	"area di gestione dei documenti";
#elif L==8	// pl
	"obszaru zarzadzania dokumentami";
#elif L==9	// pt
	"zona de gest&atilde;o de documentos";
#endif

const char *Txt_Domains =
#if   L==1	// ca
	"Dominis";
#elif L==2	// de
	"Dom&auml;nen";
#elif L==3	// en
	"Domains";
#elif L==4	// es
	"Dominios";
#elif L==5	// fr
	"Domaines";
#elif L==6	// gn
	"Dominios";	// Okoteve traduccin
#elif L==7	// it
	"Campi";
#elif L==8	// pl
	"Domen";
#elif L==9	// pt
	"Dom&iacute;nios";
#endif

const char *Txt_Done =
#if   L==1	// ca
	"Fet";
#elif L==2	// de
	"Gemacht";
#elif L==3	// en
	"Done";
#elif L==4	// es
	"Hecho";
#elif L==5	// fr
	"Termin&eacute;";
#elif L==6	// gn
	"Hecho";	// Okoteve traduccin
#elif L==7	// it
	"Fatto";
#elif L==8	// pl
	"Gotowe";
#elif L==9	// pt
	"Feito";
#endif

const char *Txt_Done_assess_test =
#if   L==1	// ca
	"He acabat; corregir test";
#elif L==2	// de
	"Geschehen; beurteilen Test";
#elif L==3	// en
	"Done; assess test";
#elif L==4	// es
	"He terminado; corregir test";
#elif L==5	// fr
	"Fait; corriger test";
#elif L==6	// gn
	"He terminado; corregir test";	// Okoteve traduccin
#elif L==7	// it
	"Finito; correggere test";
#elif L==8	// pl
	"Sporz&aogon;dzono; oceny test";
#elif L==9	// pt
	"Feito; corrigir teste";
#endif

const char *Txt_Download =	// As a verb
#if   L==1	// ca
	"Descarregar";
#elif L==2	// de
	"Download";
#elif L==3	// en
	"Download";
#elif L==4	// es
	"Descargar";
#elif L==5	// fr
	"T&eacute;l&eacute;charger";
#elif L==6	// gn
	"Descargar";	// Okoteve traduccin
#elif L==7	// it
	"Scarica";
#elif L==8	// pl
	"Pobierz";
#elif L==9	// pt
	"Baixar";
#endif

const char *Txt_Download_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Descarregar %s";
#elif L==2	// de
	"Laden Sie %s";
#elif L==3	// en
	"Download %s";
#elif L==4	// es
	"Descargar %s";
#elif L==5	// fr
	"T&eacute;l&eacute;charger %s";
#elif L==6	// gn
	"Descargar %s";		// Okoteve traduccin
#elif L==7	// it
	"Scarica %s";
#elif L==8	// pl
	"Pobierz %s";
#elif L==9	// pt
	"Download %s";
#endif

const char *Txt_download =	// As a noun
#if   L==1	// ca
	"desc&agrave;rrega";
#elif L==2	// de
	"Download";
#elif L==3	// en
	"download";
#elif L==4	// es
	"descarga";
#elif L==5	// fr
	"t&eacute;l&eacute;chargement";
#elif L==6	// gn
	"descarga";	// Okoteve traduccin
#elif L==7	// it
	"download";
#elif L==8	// pl
	"pobrania";
#elif L==9	// pt
	"download";
#endif

const char *Txt_Downloads =	// As a noun
#if   L==1	// ca
	"Desc&agrave;rregues";
#elif L==2	// de
	"Downloads";
#elif L==3	// en
	"Downloads";
#elif L==4	// es
	"Descargas";
#elif L==5	// fr
	"T&eacute;l&eacute;chargements";
#elif L==6	// gn
	"Descargas";	// Okoteve traduccin
#elif L==7	// it
	"Downloads";
#elif L==8	// pl
	"Pobrania";
#elif L==9	// pt
	"Downloads";
#endif

const char *Txt_downloads =	// As a noun
#if   L==1	// ca
	"desc&agrave;rregues";
#elif L==2	// de
	"Downloads";
#elif L==3	// en
	"downloads";
#elif L==4	// es
	"descargas";
#elif L==5	// fr
	"t&eacute;l&eacute;chargements";
#elif L==6	// gn
	"descargas";	// Okoteve traduccin
#elif L==7	// it
	"downloads";
#elif L==8	// pl
	"pobrania";
#elif L==9	// pt
	"downloads";
#endif

const char *Txt_Duplicate_USERS =
#if   L==1	// ca
	"Duplicats";
#elif L==2	// de
	"Doppelte";
#elif L==3	// en
	"Duplicates";
#elif L==4	// es
	"Duplicados";
#elif L==5	// fr
	"En double";
#elif L==6	// gn
	"Duplicados";	// Okoteve traduccin
#elif L==7	// it
	"Duplicati";
#elif L==8	// pl
	"Duplikaty";
#elif L==9	// pt
	"Duplicados";
#endif

const char *Txt_Edit =
#if   L==1	// ca
	"Editar";
#elif L==2	// de
	"Bearbeiten";
#elif L==3	// en
	"Edit";
#elif L==4	// es
	"Editar";
#elif L==5	// fr
	"&Eacute;diter";
#elif L==6	// gn
	"Jehaijey";
#elif L==7	// it
	"Editare";
#elif L==8	// pl
	"Edytuj";
#elif L==9	// pt
	"Editar";
#endif

const char *Txt_Edit_assignment =
#if   L==1	// ca
	"Editar activitat";
#elif L==2	// de
	"Aufgabe bearbeiten";
#elif L==3	// en
	"Edit assignment";
#elif L==4	// es
	"Editar actividad";
#elif L==5	// fr
	"&Eacute;diter activit&eacute;";
#elif L==6	// gn
	"Editar actividad";	// Okoteve traduccin
#elif L==7	// it
	"Editare attivit&agrave;";
#elif L==8	// pl
	"Edytuj zadanie";
#elif L==9	// pt
	"Editar atividade";
#endif

const char *Txt_Edit_event =
#if   L==1	// ca
	"Editar evento";	// Necessita traduccio
#elif L==2	// de
	"Ereignis bearbeiten";
#elif L==3	// en
	"Edit event";
#elif L==4	// es
	"Editar evento";
#elif L==5	// fr
	"&Eacute;diter &eacute;v&eacute;nement";
#elif L==6	// gn
	"Editar evento";	// Okoteve traduccin
#elif L==7	// it
	"Editare evento";
#elif L==8	// pl
	"Edytuj wydarzenie";
#elif L==9	// pt
	"Editar evento";
#endif

const char *Txt_Edit_game =
#if   L==1	// ca
	"Editar joc";
#elif L==2	// de
	"Spiel bearbeiten";
#elif L==3	// en
	"Edit game";
#elif L==4	// es
	"Editar juego";
#elif L==5	// fr
	"&Eacute;diter jeu";
#elif L==6	// gn
	"Editar juego";	// Okoteve traduccin
#elif L==7	// it
	"Editare gioco";
#elif L==8	// pl
	"Edycja gra";
#elif L==9	// pt
	"Editar jogo";
#endif

const char *Txt_Edit_my_personal_data =
#if   L==1	// ca
	"Edita meves dades personals";
#elif L==2	// de
	"Bearbeiten meiner pers&ouml;nlichen Daten";
#elif L==3	// en
	"Edit my personal data";
#elif L==4	// es
	"Editar mis datos personales";
#elif L==5	// fr
	"&Eacute;diter mes donn&eacute;es personnelles";
#elif L==6	// gn
	"Editar mis datos personales";		// Okoteve traduccin
#elif L==7	// it
	"Editare i miei dati personali";
#elif L==8	// pl
	"Edycja moich danych osobowych";
#elif L==9	// pt
	"Editar os meus dados pessoais";
#endif

const char *Txt_Edit_plain_text =
#if   L==1	// ca
	"Editar text sense format";
#elif L==2	// de
	"Plain Text bearbeiten";
#elif L==3	// en
	"Edit plain text";
#elif L==4	// es
	"Editar texto sin formato";
#elif L==5	// fr
	"&Eacute;diter texte simple";
#elif L==6	// gn
	"Editar texto sin formato";	// Okoteve traduccin
#elif L==7	// it
	"Editare testo normale";
#elif L==8	// pl
	"Edycja tekstu zwyk&lstrok;y";
#elif L==9	// pt
	"Editar texto simples";
#endif

const char *Txt_Edit_project =
#if   L==1	// ca
	"Editar projecte";
#elif L==2	// de
	"Projekt bearbeiten";
#elif L==3	// en
	"Edit project";
#elif L==4	// es
	"Editar proyecto";
#elif L==5	// fr
	"&Eacute;diter projet";
#elif L==6	// gn
	"Editar proyecto";	// Okoteve traduccin
#elif L==7	// it
	"Editare progetto";
#elif L==8	// pl
	"Edycja projekt";
#elif L==9	// pt
	"Editar projeto";
#endif

const char *Txt_Edit_record_fields =
#if   L==1	// ca
	"Editar camps de fitxes";
#elif L==2	// de
	"Datensatzfelder bearbeiten";
#elif L==3	// en
	"Edit record fields";
#elif L==4	// es
	"Editar campos de fichas";
#elif L==5	// fr
	"Modifier champs de cartes";
#elif L==6	// gn
	"Editar campos de fichas";	// Okoteve traduccin
#elif L==7	// it
	"Editare campo schede";
#elif L==8	// pl
	"Edycja rekord w dziedzinie";
#elif L==9	// pt
	"Editar campos cart&otilde;es";
#endif

const char *Txt_Edit_survey =
#if   L==1	// ca
	"Editar enquesta";
#elif L==2	// de
	"Umfrage bearbeiten";
#elif L==3	// en
	"Edit survey";
#elif L==4	// es
	"Editar encuesta";
#elif L==5	// fr
	"&Eacute;diter sondage";
#elif L==6	// gn
	"Editar encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Editare sondaggio";
#elif L==8	// pl
	"Edycja ankiety";
#elif L==9	// pt
	"Editar inqu&eacute;rito";
#endif

const char *Txt_Edit_rich_text =
#if   L==1	// ca
	"Editar text enriquit";
#elif L==2	// de
	"Rich Text bearbeiten";
#elif L==3	// en
	"Edit rich text";
#elif L==4	// es
	"Editar texto enriquecido";
#elif L==5	// fr
	"&Eacute;diter texte enrichi";
#elif L==6	// gn
	"Editar texto enriquecido";	// Okoteve traduccin
#elif L==7	// it
	"Editare testo formattato";
#elif L==8	// pl
	"Edycja tekstu sformatowany";
#elif L==9	// pt
	"Editar texto rico";
#endif

const char *Txt_eg_A_B =
#if   L==1	// ca
	"per ex.: A, B,&hellip;";
#elif L==2	// de
	"z. B.: A, B,&hellip;";
#elif L==3	// en
	"e.g., A, B,&hellip;";
#elif L==4	// es
	"por ej.: A, B,&hellip;";
#elif L==5	// fr
	"par exemple: A, B,&hellip;";
#elif L==6	// gn
	"por ej.: A, B,&hellip;";	// Okoteve traduccin
#elif L==7	// it
	"ad esempio: A, B,&hellip;";
#elif L==8	// pl
	"np. A, B, &hellip;";
#elif L==9	// pt
	"ex.: A, B,&hellip;";
#endif

const char *Txt_eg_Lectures_Practicals =
#if   L==1	// ca
	"per ex.: Teoria, Pr&agrave;ctiques,&hellip;";
#elif L==2	// de
	"z. B.: Vorlesung, &Uuml;bung,&hellip;";
#elif L==3	// en
	"e.g., Lectures, Practicals,&hellip;";
#elif L==4	// es
	"por ej.: Teor&iacute;a, Pr&aacute;cticas,&hellip;";
#elif L==5	// fr
	"par exemple: Th&eacute;orie, Pratique,&hellip;";
#elif L==6	// gn
	"por ej.: Teor&iacute;a, Pr&aacute;cticas,&hellip;";	// Okoteve traduccin
#elif L==7	// it
	"ad esempio: Teoria, Pratica,&hellip;";
#elif L==8	// pl
	"np. Wyklady, Praktyki &hellip;";
#elif L==9	// pt
	"ex.: Teoria, Pr&aacute;tica,&hellip;";
#endif

const char *Txt_ELAPSED_TIME =
#if   L==1	// ca
	"Fa";
#elif L==2	// de
	"Vor";
#elif L==3	// en
	"Ago";
#elif L==4	// es
	"Hace";
#elif L==5	// fr
	"Y a";
#elif L==6	// gn
	"Hace";	// Okoteve traduccin
#elif L==7	// it
	"Fa";
#elif L==8	// pl
	"Temu";
#elif L==9	// pt
	"Atr&aacute;s";
#endif

const char *Txt_Eliminate =
#if   L==1	// ca
	"Eliminar";
#elif L==2	// de
	"Entfernen";
#elif L==3	// en
	"Eliminate";
#elif L==4	// es
	"Eliminar";
#elif L==5	// fr
	"&Eacute;liminer";
#elif L==6	// gn
	"Mbogue";
#elif L==7	// it
	"Rimuovere";
#elif L==8	// pl
	"Wyeliminowac";
#elif L==9	// pt
	"Eliminar";
#endif

const char *Txt_Eliminate_all_courses_whithout_users_PART_1_OF_2 =
#if   L==1	// ca
	"Elimineu tots els cursos sense usuaris i sense acc&eacute;s en els &uacute;ltims";
#elif L==2	// de
	"Beseitigen Sie alle Kurse ohne Benutzer und ohne Zugriffe in den letzten";
#elif L==3	// en
	"Eliminate all courses without users and without accesses in the last";
#elif L==4	// es
	"Eliminar todas las asignaturas sin usuarios y sin accesos en los &uacute;ltimos";
#elif L==5	// fr
	"&Eacute;liminer tous les mati&egrave;res sans utilisateurs et sans acc&egrave;s au cours des";
#elif L==6	// gn
	"Eliminar todas las asignaturas sin usuarios y sin accesos en los &uacute;ltimos";	// Okoteve traduccin
#elif L==7	// it
	"Elimina tutti i corsi senza utenti e senza accessi negli ultimi";
#elif L==8	// pl
	"Wyeliminuj wszystkie kursy bez u&zdot;ytkownik&oacute;w i bez dost&eogon;pu w ci&aogon;gu ostatnich";
#elif L==9	// pt
	"Eliminar todas as disciplinas sem usu&aacute;rios e sem acessos nos &uacute;ltimos";
#endif

const char *Txt_Eliminate_all_courses_whithout_users_PART_2_OF_2 =
#if   L==1	// ca
	"mesos.";
#elif L==2	// de
	"Monaten.";
#elif L==3	// en
	"months.";
#elif L==4	// es
	"meses.";
#elif L==5	// fr
	"derniers mois.";
#elif L==6	// gn
	"meses.";		// Okoteve traduccin
#elif L==7	// it
	"mesi.";
#elif L==8	// pl
	"miesi&eogon;cy.";
#elif L==9	// pt
	"meses.";
#endif

const char *Txt_Eliminate_all_users_who_are_not_enroled_on_any_courses_PART_1_OF_2 =
#if   L==1	// ca
	"Elimineu tots els usuaris"
	" que no estiguin inscrits en cap curs"
	" i amb m&eacute;s de";
#elif L==2	// de
	"Beseitigen Sie alle Benutzer"
	" die nicht an Kursen angemeldet sind"
	" und mehr als";
#elif L==3	// en
	"Eliminate all users"
	" who are not enroled on any courses"
	" and with more than";
#elif L==4	// es
	"Eliminar todos los usuarios"
	" que no est&eacute;n inscritos en ninguna asignatura"
	" y que lleven m&aacute;s de";
#elif L==5	// fr
	"&Eacute;liminer tous les utilisateurs"
	" qui ne sont inscrits &agrave; aucune mati&egrave;re"
	" et qui ont plus de";
#elif L==6	// gn
	"Eliminar todos los usuarios"
	" que no est&eacute;n inscritos en ninguna asignatura"
	" y que lleven m&aacute;s de";				// Okoteve traduccin
#elif L==7	// it
	"Rimuovere tutti gli utenti"
	" che non sono registrati ad alcun corso"
	" e con pi&ugrave; di";
#elif L==8	// pl
	"Wyeliminuj wszystkich u&zdot;ytkownik&oacute;w"
	" kt&oacute;rzy nie s&aogon; zapisani na &zdot;adne kursy"
	" i d&lstrok;u&zdot;ej ni&zdot;";
#elif L==9	// pt
	"Eliminar todos os usu&aacute;rios"
	" que n&atilde,o est&atilde;o matriculados em nenhuma disciplina"
	" e com mais de";
#endif

const char *Txt_Eliminate_all_users_who_are_not_enroled_on_any_courses_PART_2_OF_2 =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"mesos sense acc&eacute;s a %s.";
#elif L==2	// de
	"Monate ohne Zugriff auf %s.";
#elif L==3	// en
	"months without access to %s.";
#elif L==4	// es
	"meses sin acceder a %s.";
#elif L==5	// fr
	"mois sans acc&egrave;s &agrave; %s.";
#elif L==6	// gn
	"meses sin acceder a %s.";		// Okoteve traduccin
#elif L==7	// it
	"mesi senza accedere a %s.";
#elif L==8	// pl
	"miesi&aogon;ce bez dost&eogon;pu do %s.";
#elif L==9	// pt
	"meses sem acesso a %s.";
#endif

const char *Txt_Eliminate_from_the_platform_the_users_indicated_in_step_1 =
#if   L==1	// ca
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";		// Necessita traduccio
#elif L==2	// de
	"Die Benutzer auf der Liste aus dem System entfernen";
#elif L==3	// en
	"Eliminate from the platform"
	" the users indicated on step 1";
#elif L==4	// es
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";
#elif L==5	// fr
	"&Eacute;liminer de la plate-forme"
	" les utilisateurs qui sont sur la liste";
#elif L==6	// gn
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";		// Okoteve traduccin
#elif L==7	// it
	"Rimuovere dalla piattaforma"
	" gli utenti che sono in lista";
#elif L==8	// pl
	"Wyeliminuj z platformy"
	" uzytkownikow kt&oacute;rzy sa na liscie";
#elif L==9	// pt
	"Eliminar da plataforma"
	" os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Eliminate_old_courses =
#if   L==1	// ca
	"Eliminar assignatures antics";
#elif L==2	// de
	"Fr&uuml;here Kursen entfernen";
#elif L==3	// en
	"Eliminate old courses";
#elif L==4	// es
	"Eliminar asignaturas antiguas";
#elif L==5	// fr
	"&Eacute;liminer anciens mati&egrave;res";
#elif L==6	// gn
	"Eliminar asignaturas antiguas";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere vecchi corsi";
#elif L==8	// pl
	"Wyeleminuj starych kurs&oacute;w";
#elif L==9	// pt
	"Eliminar antigas disciplinas";
#endif

const char *Txt_Eliminate_old_users =
#if   L==1	// ca
	"Eliminar usuaris antics";
#elif L==2	// de
	"Fr&uuml;here Benutzer entfernen";
#elif L==3	// en
	"Eliminate old users";
#elif L==4	// es
	"Eliminar usuarios antiguos";
#elif L==5	// fr
	"&Eacute;liminer anciens utilisateurs";
#elif L==6	// gn
	"Eliminar usuarios antiguos";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere vecchi utenti";
#elif L==8	// pl
	"Usu&nacute; starych uzytkownik&oacute;w";
#elif L==9	// pt
	"Eliminar antigos utilizadores";
#endif

const char *Txt_Eliminating_X_courses_whithout_users_and_with_more_than_Y_months_without_access =	// Warning: it is very important to include %lu and %u in the following sentences
#if   L==1	// ca
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";	// Necessita traduccio
#elif L==2	// de
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Need bersetzung
#elif L==3	// en
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";
#elif L==4	// es
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";
#elif L==5	// fr
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Besoin de traduction
#elif L==6	// gn
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";	// Okoteve traduccin
#elif L==7	// it
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Bisogno di traduzione
#elif L==8	// pl
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Eliminando %lu disciplina(s) sem usu&aacute;rios"
	" e com mais de %u meses sem acesso.";
#endif

const char *Txt_Eliminate_my_user_account =
#if   L==1	// ca
	"Eliminar la meva compte d'usuari";
#elif L==2	// de
	"Entfernen mein Benutzerkonto";
#elif L==3	// en
	"Eliminate my user account";
#elif L==4	// es
	"Eliminar mi cuenta de usuario/a";
#elif L==5	// fr
	"Supprimer mon compte d'utilisateur";
#elif L==6	// gn
	"Eliminar mi cuenta de usuario/a";		// Okoteve traduccin
#elif L==7	// it
	"Rimuovere il mio account utente";
#elif L==8	// pl
	"Usu&nacute; moje konto u&zdot;ytkownika";
#elif L==9	// pt
	"Remover minha conta de usu&aacute;rio";
#endif

const char *Txt_Eliminate_user_account =
#if   L==1	// ca
	"Eliminar compte d'usuari";
#elif L==2	// de
	"Entfernen Benutzerkonto";
#elif L==3	// en
	"Eliminate user account";
#elif L==4	// es
	"Eliminar cuenta de usuario/a";
#elif L==5	// fr
	"Supprimer compte d'utilisateur";
#elif L==6	// gn
	"Eliminar cuenta de usuario/a";		// Okoteve traduccin
#elif L==7	// it
	"Rimuovere account utente";
#elif L==8	// pl
	"Usu&nacute; konto u&zdot;ytkownika";
#elif L==9	// pt
	"Remover conta de usu&aacute;rio";
#endif

const char *Txt_Eliminating_X_users_who_were_not_enroled_in_any_course_and_with_more_than_Y_months_without_access_to_Z =	// Warning: it is very important to include %lu, %u and %s in the following sentences
#if   L==1	// ca
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";	// Necessita traduccio
#elif L==2	// de
	"Eliminating %lu user(s) who were not enroled in any course"
	" and with more than %u months without access to %s.";		// Need bersetzung
#elif L==3	// en
	"Eliminating %lu user(s) who were not enroled in any course"
	" and with more than %u months without access to %s.";
#elif L==4	// es
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";
#elif L==5	// fr
	"Eliminating %lu user(s) who were not enroled in any course"
	" and with more than %u months without access to %s.";		// Besoin de traduction
#elif L==6	// gn
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovendo %lu utenti che non siano registrati in alcun corso"
	" e con pi&ugrave; di %u mesi senza accedere a %s.";
#elif L==8	// pl
	"Eliminowanie %lu u&zdot;ytkownik&oacute;w, kt&oacute;rzy nie byli zapisani na &zdot;aden kurs"
	" i d&lstrok;u&zdor;j ni&zdot; %u miesi&aogon;ce bez dost&eogon;pu do %s.";
#elif L==9	// pt
	"Eliminando %lu usu&aacute;rio(s) que n&atilde;o estavam matriculados em nenhuma disciplina"
	" e com mais de %u meses sem acesso a %s.";
#endif

const char *Txt_Email =
#if   L==1	// ca
	"Correu";
#elif L==2	// de
	"E-Mail";
#elif L==3	// en
	"Email";
#elif L==4	// es
	"Correo";
#elif L==5	// fr
	"Courrier";
#elif L==6	// gn
	"&Ntilde;e'&#7869;veve";
#elif L==7	// it
	"Email";
#elif L==8	// pl
	"Email";
#elif L==9	// pt
	"Email";
#endif

const char *Txt_Email_domain_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Dominio de correo <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2	// de
	"E-Mail-Dom&auml;ne <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Email domain <strong>%s</strong> removed.";
#elif L==4	// es
	"Dominio de correo <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Mail domain <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6	// gn
	"Dominio de correo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Campo mail <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Domeny poczty <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Dom&iacute;nio de email <strong>%s</strong> removido.";
#endif

const char *Txt_Email_domains_allowed_for_notifications =
#if   L==1	// ca
	"Dominios de correo permitidos para notificaciones";	// Necessita traduccio
#elif L==2	// de
	"E-Mail-Dom&auml;nen, an die Benachrichtigungen gesendet werden k&ouml;nnen";
#elif L==3	// en
	"Email domains allowed for notifications";
#elif L==4	// es
	"Dominios de correo permitidos para notificaciones";
#elif L==5	// fr
	"Email domains allowed for notifications";		// Besoin de traduction
#elif L==6	// gn
	"Dominios de correo permitidos para notificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Campo mail permesso per notifiche";
#elif L==8	// pl
	"Mail domen dozwolone powiadomien";
#elif L==9	// pt
	"Dom&iacute;nios de email permitidos para notifica&ccedil;&otilde;es";
#endif

const char *Txt_EMAIL_DOMAIN_HELP_ORDER[Mai_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Ordenar por dominio de correo"	// Necessita traduccio
#elif L==2	// de
	 "Nach E-Mail-Dom&auml;ne sortieren"
#elif L==3	// en
	 "Sort by email domain"
#elif L==4	// es
	 "Ordenar por dominio de correo"
#elif L==5	// fr
	 "Trier par domaine de messagerie"
#elif L==6	// gn
	 "Ordenar por dominio de correo"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per campo mail"
#elif L==8	// pl
	 "Sortuj wedlug domeny poczty"
#elif L==9	// pt
	 "Classificar por dom&iacute;nio de email"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar por informaci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	 "Nach Information sortieren"
#elif L==3	// en
	 "Sort by information"
#elif L==4	// es
	 "Ordenar por informaci&oacute;n"
#elif L==5	// fr
	 "Trier par information"
#elif L==6	// gn
	 "Ordenar por informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per informazione"
#elif L==8	// pl
	 "Sortuj wedlug informacji"
#elif L==9	// pt
	 "Classificar por informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar por n&ordm; de usuarios"	// Necessita traduccio
#elif L==2	// de
	 "Nach Anzahl der Benutzer sortieren"
#elif L==3	// en
	 "Sort by no. of users"
#elif L==4	// es
	 "Ordenar por n&ordm; de usuarios"
#elif L==5	// fr
	 "Trier par nombre d'utilisateurs"
#elif L==6	// gn
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per numero di utenti"
#elif L==8	// pl
	 "Sortuj wedlug numerow uzytkownikow"
#elif L==9	// pt
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_EMAIL_DOMAIN_ORDER[Mai_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Domini de correu"
#elif L==2	// de
	 "E-Mail-Dom&auml;ne"
#elif L==3	// en
	 "Email domain"
#elif L==4	// es
	 "Dominio de correo"
#elif L==5	// fr
	 "Domaine de messagerie"
#elif L==6	// gn
	 "Dominio de correo"	// Okoteve traduccin
#elif L==7	// it
	 "Campo mail"
#elif L==8	// pl
	 "Domeny poczty"
#elif L==9	// pt
	 "Dom&iacute;nio de email"
#endif
	 ,
#if   L==1	// ca
	 "Informaci&oacute;"
#elif L==2	// de
	 "Information"
#elif L==3	// en
	 "Information"
#elif L==4	// es
	 "Informaci&oacute;n"
#elif L==5	// fr
	 "Information"
#elif L==6	// gn
	 "Informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Informazione"
#elif L==8	// pl
	 "Informacja"
#elif L==9	// pt
	 "Informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "Usuaris"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "Users"
#elif L==4	// es
	 "Usuarios"
#elif L==5	// fr
	 "Utilisateurs"
#elif L==6	// gn
	 "Usuarios"	// Okoteve traduccin
#elif L==7	// it
	 "Utenti"
#elif L==8	// pl
	 "Uzytkownikow"
#elif L==9	// pt
	 "Utilizadores"
#endif
	};

const char *Txt_Email_X_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Correo %s confirmado";	// Necessita traduccio
#elif L==2	// de
	"Email %s confirmed";	// Need bersetzung
#elif L==3	// en
	"Email %s confirmed";
#elif L==4	// es
	"Correo %s confirmado";
#elif L==5	// fr
	"Email %s confirmed";	// Besoin de traduction
#elif L==6	// gn
	"Correo %s confirmado";	// Okoteve traduccin
#elif L==7	// it
	"Email %s confirmed";	// Bisogno di traduzione
#elif L==8	// pl
	"Email %s confirmed";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Email %s confirmado";
#endif

const char *Txt_Email_X_has_already_been_confirmed_before =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El correo <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";	// Necessita traduccio
#elif L==2	// de
	"Email <strong>%s</strong>"
	" has already been confirmed before.";			// Need bersetzung
#elif L==3	// en
	"Email <strong>%s</strong>"
	" has already been confirmed before.";
#elif L==4	// es
	"El correo <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";
#elif L==5	// fr
	"Email <strong>%s</strong>"
	" has already been confirmed before.";			// Besoin de traduction
#elif L==6	// gn
	"El correo <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";	// Okoteve traduccin
#elif L==7	// it
	"Email <strong>%s</strong>"
	" has already been confirmed before.";			// Bisogno di traduzione
#elif L==8	// pl
	"Email <strong>%s</strong>"
	" has already been confirmed before.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Email <strong>%s</strong>"
	" j&aacute; foi confirmado antes.";
#endif

const char *Txt_Email_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Correu <strong>%s</strong> eliminat.";
#elif L==2	// de
	"E-Mail <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Email <strong>%s</strong> removed.";
#elif L==4	// es
	"Correo <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Courrier <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Correo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Email <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Email <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Email <strong>%s</strong> removido.";
#endif

const char *Txt_End_date =
#if   L==1	// ca
	"Data final";
#elif L==2	// de
	"Enddatum";
#elif L==3	// en
	"End date";
#elif L==4	// es
	"Fecha final";
#elif L==5	// fr
	"Date finale";
#elif L==6	// gn
	"Fecha final";	// Okoteve traduccin
#elif L==7	// it
	"Data finale";
#elif L==8	// pl
	"Data koncowa";
#elif L==9	// pt
	"Data final";
#endif

const char *Txt_Enrol_in_groups =
#if   L==1	// ca
	"Inscriure en grups";
#elif L==2	// de
	"Anmeldung mich in Gruppen";
#elif L==3	// en
	"Enrol in groups";
#elif L==4	// es
	"Inscribirme en grupos";
#elif L==5	// fr
	"Inscriver moi dans groupes";
#elif L==6	// gn
	"Inscribirme en grupos";	// Okoteve traduccin
#elif L==7	// it
	"Iscriver mi in gruppi";
#elif L==8	// pl
	"Zarejestruj mnie w grupach";
#elif L==9	// pt
	"Inscreva-me em grupos ";
#endif

const char *Txt_Enrolment =
#if   L==1	// ca
	"Inscripci&oacute;";
#elif L==2	// de
	"Einschreibung";
#elif L==3	// en
	"Enrolment";
#elif L==4	// es
	"Inscripci&oacute;n";
#elif L==5	// fr
	"Inscription";
#elif L==6	// gn
	"Inscripci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Registrazione";
#elif L==8	// pl
	"Rejestracja";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o";
#endif

const char *Txt_Enrolment_confirmed =
#if   L==1	// ca
	"Inscripci&oacute;n confirmada";	// Necessita traduccio
#elif L==2	// de
	"Enrolment confirmed";			// Need bersetzung
#elif L==3	// en
	"Enrolment confirmed";
#elif L==4	// es
	"Inscripci&oacute;n confirmada";
#elif L==5	// fr
	"Enrolment confirmed";			// Besoin de traduction
#elif L==6	// gn
	"Inscripci&oacute;n confirmada";	// Okoteve traduccin
#elif L==7	// it
	"Registrazione confermata";
#elif L==8	// pl
	"Rejestracja potwierdzona";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o confirmada";
#endif

const char *Txt_Enrolment_of_X_rejected =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";	// Necessita traduccio
#elif L==2	// de
	"Enrolment of <strong>%s</strong> rejected.";		// Need bersetzung
#elif L==3	// en
	"Enrolment of <strong>%s</strong> rejected.";
#elif L==4	// es
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";
#elif L==5	// fr
	"Enrolment of <strong>%s</strong> rejected.";		// Besoin de traduction
#elif L==6	// gn
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";	// Okoteve traduccin
#elif L==7	// it
	"Enrolment of <strong>%s</strong> rejected.";		// Bisogno di traduzione
#elif L==8	// pl
	"Enrolment of <strong>%s</strong> rejected.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o de <strong>%s</strong> rejeitada.";
#endif

const char *Txt_Enrolment_not_confirmed =
#if   L==1	// ca
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Enrolment not confirmed";				// Need bersetzung
#elif L==3	// en
	"Enrolment not confirmed";
#elif L==4	// es
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";
#elif L==5	// fr
	"Enrolment not confirmed";				// Besoin de traduction
#elif L==6	// gn
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Registrazione non confermata";
#elif L==8	// pl
	"Rejestracja nie potwierdzona";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o n&atilde;o confirmada";
#endif

const char *Txt_Enrolment_requests =
#if   L==1	// ca
	"Peticions d'inscripci&oacute;";
#elif L==2	// de
	"Registrierungsanforderungen";
#elif L==3	// en
	"Enrolment requests";
#elif L==4	// es
	"Solicitudes de inscripci&oacute;n";
#elif L==5	// fr
	"Demandes d'inscription";
#elif L==6	// gn
	"Solicitudes de inscripci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Richieste di registrazione";
#elif L==8	// pl
	"Pro&sacute;by o rejestracji";
#elif L==9	// pt
	"Pedidos de inscri&ccedil;&atilde;o";
#endif

const char *Txt_Enter_a_new_item_here =
#if   L==1	// ca
	"Escriba aqu&iacute; un nuevo apartado";	// Necessita traduccio
#elif L==2	// de
	"Neuen Eintrag hier eingeben";
#elif L==3	// en
	"Enter a new item here";
#elif L==4	// es
	"Escriba aqu&iacute; un nuevo apartado";
#elif L==5	// fr
	"Ecrivez ici un nouveau point";
#elif L==6	// gn
	"Escriba aqu&iacute; un nuevo apartado";	// Okoteve traduccin
#elif L==7	// it
	"Inserisci qui un nuovo oggetto(tambi&egrave;n item est&agrave; bien)";
#elif L==8	// pl
	"Wpisz nowy element o";
#elif L==9	// pt
	"Escreva aqui uma nova rubrica";
#endif

const char *Txt_Enter_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Entrar des %s";
#elif L==2	// de
	"Geben Sie %s";
#elif L==3	// en
	"Enter from %s";
#elif L==4	// es
	"Entrar desde %s";
#elif L==5	// fr
	"Entrez du %s";
#elif L==6	// gn
	"Entrar desde %s";	// Okoteve traduccin
#elif L==7	// it
	"Inserisci da %s";
#elif L==8	// pl
	"Wprowadzi&cacute; z %s";
#elif L==9	// pt
	"Digite partir %s";
#endif

const char *Txt_Equation_centered =
#if   L==1	// ca
	"Equaci&oacute; centrada";
#elif L==2	// de
	"Gleichung zentriert";
#elif L==3	// en
	"Equation centered";
#elif L==4	// es
	"Ecuaci&oacute;n centrada";
#elif L==5	// fr
	"Equation centr&eacute;";
#elif L==6	// gn
	"Ecuaci&oacute;n centrada";	// Okoteve traduccin
#elif L==7	// it
	"Equazione centrato";
#elif L==8	// pl
	"R&oacute;wnanie centered";
#elif L==9	// pt
	"equa&ccedil;&atilde;o centrado";
#endif

const char *Txt_Error_getting_data_from_a_recipient =
#if   L==1	// ca
	"Error obteniendo datos de un destinatario.";	// Necessita traduccio
#elif L==2	// de
	"Error getting data from a recipient.";		// Need bersetzung
#elif L==3	// en
	"Error getting data from a recipient.";
#elif L==4	// es
	"Error obteniendo datos de un destinatario.";
#elif L==5	// fr
	"Error getting data from a recipient.";		// Besoin de traduction
#elif L==6	// gn
	"Error obteniendo datos de un destinatario.";	// Okoteve traduccin
#elif L==7	// it
	"Errore acquisizione dati da un destinatario.";
#elif L==8	// pl
	"Blad podczas pobierania danych z odbiorca.";
#elif L==9	// pt
	"Erro ao obter dados de um destinat&aacute;rio.";
#endif

const char *Txt_Error_sending_or_processing_image_video =
#if   L==1	// ca
	"Error enviant o processant imatge / v&iacute;deo.";
#elif L==2	// de
	"Fehler beim Senden oder Verarbeiten eines Abbild / Video.";
#elif L==3	// en
	"Error sending or processing image / video.";
#elif L==4	// es
	"Error enviando o procesando imagen / v&iacute;deo.";
#elif L==5	// fr
	"Erreur d'envoi ou de traitement de l'image / vid&eacute;o.";
#elif L==6	// gn
	"Error enviando o procesando imagen / v&iacute;deo.";	// Okoteve traduccin
#elif L==7	// it
	"Errore nell'invio o nell'elaborazione dell'immagine / video.";
#elif L==8	// pl
	"B&lstrok;&aogon;d podczas wysy&lstrok;ania lub przetwarzania obrazu / wideo.";
#elif L==9	// pt
	"Erro ao enviar ou processar imagem / v&iacute;deo.";
#endif

const char *Txt_Event =
#if   L==1	// ca
	"Esdeveniment";
#elif L==2	// de
	"Ereignis";
#elif L==3	// en
	"Event";
#elif L==4	// es
	"Evento";
#elif L==5	// fr
	"&Eacute;v&eacute;nement";
#elif L==6	// gn
	"Evento";	// Okoteve traduccin
#elif L==7	// it
	"Evento";
#elif L==8	// pl
	"Wydarzenie";
#elif L==9	// pt
	"Evento";
#endif

const char *Txt_Event_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'esdeveniment <strong>%s</strong> ara est&agrave; ocult.";
#elif L==2	// de
	"Der Ereignis <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==3	// en
	"Event <strong>%s</strong> is now hidden.";
#elif L==4	// es
	"El evento <strong>%s</strong> ahora est&aacute; oculto.";
#elif L==5	// fr
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==6	// gn
	"El evento <strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==7	// it
	"L'evento <strong>%s</strong> &egrave; ora nascosto.";
#elif L==8	// pl
	"Wydarzenie <strong>%s</strong> jest ukryte.";
#elif L==9	// pt
	"O evento <strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_Event_X_is_now_private =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'esdevenimen <strong>%s</strong> ara &eacute;s privat.";
#elif L==2	// de
	"Der Ereignis <strong>%s</strong> ist jetzt privat.";
#elif L==3	// en
	"Event <strong>%s</strong> is now private.";
#elif L==4	// es
	"El evento <strong>%s</strong> ahora es privado.";
#elif L==5	// fr
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant priv&eacute;.";
#elif L==6	// gn
	"El evento <strong>%s</strong> ahora es privado.";	// Okoteve traduccin
#elif L==7	// it
	"L'evento <strong>%s</strong> &egrave; ora privato.";
#elif L==8	// pl
	"Wydarzenie <strong>%s</strong> jest obecnie prywatne.";
#elif L==9	// pt
	"O evento <strong>%s</strong> &eacute; agora privado.";
#endif

const char *Txt_Event_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'esdevenimen <strong>%s</strong> ara est&agrave; visible.";
#elif L==2	// de
	"Der Ereignis <strong>%s</strong> ist jetzt sichtbar.";
#elif L==3	// en
	"Event <strong>%s</strong> is now visible.";
#elif L==4	// es
	"El evento <strong>%s</strong> ahora est&aacute; visible.";
#elif L==5	// fr
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant visible.";
#elif L==6	// gn
	"El evento <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7	// it
	"L'evento <strong>%s</strong> &egrave; ora visibile.";
#elif L==8	// pl
	"Wydarzenie <strong>%s</strong> jest teraz widoczny.";
#elif L==9	// pt
	"O evento <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_Event_X_is_now_visible_to_users_of_your_courses =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'esdevenimen <strong>%s</strong> ara &eacute;s visible"
	" pels usuaris de les seves assignatures.";
#elif L==2	// de
	"Der Ereignis <strong>%s</strong> ist jetzt sichtbar"
	" f&uuml;r Benutzer Ihrer Kurse.";
#elif L==3	// en
	"Event <strong>%s</strong> is now visible"
	" to users of your courses.";
#elif L==4	// es
	"El evento <strong>%s</strong> ahora es visible"
	" por los usuarios de sus asignaturas.";
#elif L==5	// fr
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant visible"
	" pour les utilisateurs de vos mati&egrave;res.";
#elif L==6	// gn
	"El evento <strong>%s</strong> ahora es visible"
	" por los usuarios de sus asignaturas.";	// Okoteve traduccin
#elif L==7	// it
	"L'evento <strong>%s</strong> &egrave; ora visibile"
	" agli utenti dei corsi.";
#elif L==8	// pl
	"Wydarzenie <strong>%s</strong> jest teraz widoczne"
	" dla u&zdot;ytkownik&oacute;w swoich kursach.";
#elif L==9	// pt
	"O evento <strong>%s</strong> &eacute; agora vis&iacute;vel"
	" para os usu&aacute;rios de as suas disciplinas.";
#endif

const char *Txt_Event_private_click_to_make_it_visible_to_the_users_of_your_courses =
#if   L==1	// ca
	"Esdeveniment privat; feu clic per fer-lo visible pels usuaris de les seves assignatures";
#elif L==2	// de
	"Ereignis privat; klicken Sie, um es f&uuml;r die Nutzer Ihrer Kurse sichtbar zu machen";
#elif L==3	// en
	"Event private, click to make it visible to the users of your courses";
#elif L==4	// es
	"Evento privado; pulse para hacerlo visible por los usuarios de sus asignaturas";
#elif L==5	// fr
	"Event privat, cliquez pour le rendre visible aux utilisateurs de vos mati&egrave;res";
#elif L==6	// gn
	"Evento privado; pulse para hacerlo visible por los usuarios de sus asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Evento privato, fare clic per renderlo visibile agli utenti dei corsi";
#elif L==8	// pl
	"Event privat, kliknij aby by&lstrok; on widoczny dla u&zdot;ytkownik&oacute;w swoich kursach";
#elif L==9	// pt
	"Evento privat, clique para torn&aacute;-lo vis&iacute;vel para os usu&aacute;rios de as suas disciplinas";
#endif

const char *Txt_Event_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Esdeveniment <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Ereignis <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Event <strong>%s</strong> removed.";
#elif L==4	// es
	"Evento <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"&Eacute;v&eacute;nement <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Event <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Evento <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Wydarzenie <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Evento <strong>%s</strong> removido.";
#endif

const char *Txt_Event_visible_to_the_users_of_your_courses_click_to_make_it_private =
#if   L==1	// ca
	"Esdeveniment visible pels usuaris de les seves assignatures; feu clic per fer-lo privat";
#elif L==2	// de
	"Ereignis Nutzer Ihrer Kurse sichtbar zu machen; klicken Sie, um es f&uuml;r die privat";
#elif L==3	// en
	"Event visible to the users of your courses, click to make it private";
#elif L==4	// es
	"Evento visible por los usuarios de sus asignaturas; pulse para hacerlo privado";
#elif L==5	// fr
	"Event visible aux utilisateurs de vos mati&egrave;res, cliquez pour le rendre privat";
#elif L==6	// gn
	"Evento visible por los usuarios de sus asignaturas; pulse para hacerlo privado";	// Okoteve traduccin
#elif L==7	// it
	"Evento visibile agli utenti dei corsi, fare clic per renderlo privato";
#elif L==8	// pl
	"Event widoczny dla u&zdot;ytkownik&oacute;w swoich kurs&oacute;w, kliknij, aby prywatne";
#elif L==9	// pt
	"Evento vis&iacute;vel para os usu&aacute;rios de as suas disciplinas, clique para torn&aacute;-lo privat";
#endif

const char *Txt_Events =
#if   L==1	// ca
	 "Esdeveniments";
#elif L==2	// de
	 "Ereignisse";
#elif L==3	// en
	 "Events";
#elif L==4	// es
	 "Eventos";
#elif L==5	// fr
	 "&Eacute;v&eacute;nements";
#elif L==6	// gn
	 "Eventos";	// Okoteve traduccin
#elif L==7	// it
	 "Eventi";
#elif L==8	// pl
	 "Wydarzen";
#elif L==9	// pt
	 "Eventos";
#endif

const char *Txt_Exam =
#if   L==1	// ca
	"ex&agrave;men";
#elif L==2	// de
	"Pr&uuml;fung";
#elif L==3	// en
	"Exam";
#elif L==4	// es
	"Examen";
#elif L==5	// fr
	"Examen";
#elif L==6	// gn
	"Kuaara'&atilde;";
#elif L==7	// it
	"Esame";
#elif L==8	// pl
	"Egzamin";
#elif L==9	// pt
	"Exame";
#endif

const char *Txt_EXAM_ANNOUNCEMENT =
#if   L==1	// ca
	"CONVOCAT&Ograve;RIA D'EXAMEN";
#elif L==2	// de
	"PR&Uuml;FUNGSAUFRUF";
#elif L==3	// en
	"ANNOUNCEMENT OF EXAM";
#elif L==4	// es
	"CONVOCATORIA DE EXAMEN";
#elif L==5	// fr
	"CONVOCATION &Agrave; UN EXAMEN";
#elif L==6	// gn
	"CONVOCATORIA DE EXAMEN";	// Okoteve traduccin
#elif L==7	// it
	"APPELLO D'ESAME";
#elif L==8	// pl
	"OGLOSZENIE egzaminu";
#elif L==9	// pt
	"CHAMADA DE EXAME";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Approximate_duration =
#if   L==1	// ca
	"Duraci&oacute;n aproximada";	// Necessita traduccio
#elif L==2	// de
	"Voraussichtliche Dauer";
#elif L==3	// en
	"Approximate duration";
#elif L==4	// es
	"Duraci&oacute;n aproximada";
#elif L==5	// fr
	"Dur&eacute;e approximative";
#elif L==6	// gn
	"Duraci&oacute;n aproximada";	// Okoteve traduccin
#elif L==7	// it
	"Durata approssimativa";
#elif L==8	// pl
	"Przyblizony czas trwania";
#elif L==9	// pt
	"Dura&ccedil;&atilde;o aproximada";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Course =
#if   L==1	// ca
	"Assignatura";
#elif L==2	// de
	"Kurs";
#elif L==3	// en
	"Course";
#elif L==4	// es
	"Asignatura";
#elif L==5	// fr
	"Mati&egrave;re";
#elif L==6	// gn
	"Mbo'esyry";
#elif L==7	// it
	"Corso";
#elif L==8	// pl
	"Kurs";
#elif L==9	// pt
	"Disciplina";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Documentation_required =
#if   L==1	// ca
	"Documentaci&oacute;n exigida";	// Necessita traduccio
#elif L==2	// de
	"Ausweis erforderlich";
#elif L==3	// en
	"ID required";
#elif L==4	// es
	"Documentaci&oacute;n exigida";
#elif L==5	// fr
	"Pi&egrave;ce d'identit obligatoire";
#elif L==6	// gn
	"Documentaci&oacute;n exigida";	// Okoteve traduccin
#elif L==7	// it
	"Documentazione richiesta";
#elif L==8	// pl
	"ID wymagane";
#elif L==9	// pt
	"Documenta&ccedil;&atilde;o necess&aacute;ria";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Exam_date =
#if   L==1	// ca
	"Fecha del examen";	// Necessita traduccio
#elif L==2	// de
	"Pr&uuml;fungsdatum";
#elif L==3	// en
	"Exam date";
#elif L==4	// es
	"Fecha del examen";
#elif L==5	// fr
	"Date de l'examen";
#elif L==6	// gn
	"Fecha del examen";	// Okoteve traduccin
#elif L==7	// it
	"Data d'esame";
#elif L==8	// pl
	"Termin egzaminu";
#elif L==9	// pt
	"Exame data";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Material_allowed =
#if   L==1	// ca
	"Material permitido";	// Necessita traduccio
#elif L==2	// de
	"Erlaubte Unterlagen";
#elif L==3	// en
	"Material allowed";
#elif L==4	// es
	"Material permitido";
#elif L==5	// fr
	"Mat&eacute;riel autoris&eacute;";
#elif L==6	// gn
	"Material permitido";	// Okoteve traduccin
#elif L==7	// it
	"Materiale permesso";
#elif L==8	// pl
	"Material dozwolone";
#elif L==9	// pt
	"Material autorizado";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Material_required =
#if   L==1	// ca
	"Material obligatorio";	// Necessita traduccio
#elif L==2	// de
	"Erforderliche Unterlagen";
#elif L==3	// en
	"Material required";
#elif L==4	// es
	"Material obligatorio";
#elif L==5	// fr
	"Mat&eacute;riel n&eacute;cessaire";
#elif L==6	// gn
	"Material obligatorio";	// Okoteve traduccin
#elif L==7	// it
	"Materiale richiesto";
#elif L==8	// pl
	"Materialy niezbedne do";
#elif L==9	// pt
	"Material obrigat&oacute;rio";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Mode =
#if   L==1	// ca
	"Modalidad<br />(escrito, oral,&hellip;)";	// Necessita traduccio
#elif L==2	// de
	"Pr&uuml;fungsart<br />(schriftlich, m&uuml;ndlich,&hellip;)";
#elif L==3	// en
	"Mode<br />(written, oral,&hellip;)";
#elif L==4	// es
	"Modalidad<br />(escrito, oral,&hellip;)";
#elif L==5	// fr
	"Mode<br />(&eacute;crit, oral,&hellip;)";
#elif L==6	// gn
	"Modalidad<br />(escrito, oral,&hellip;)";	// Okoteve traduccin
#elif L==7	// it
	"Modalit&agrave;<br />(scritto, orale,&hellip;)";
#elif L==8	// pl
	"Tryb<br />(pisemnej, ustnej,&hellip;)";
#elif L==9	// pt
	"Modalidade<br />(escrita, oral,&hellip;)";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Other_information =
#if   L==1	// ca
	"Otras indicaciones";	// Necessita traduccio
#elif L==2	// de
	"Weitere Informationen";
#elif L==3	// en
	"Other information";
#elif L==4	// es
	"Otras indicaciones";
#elif L==5	// fr
	"Autres informations";
#elif L==6	// gn
	"Otras indicaciones";	// Okoteve traduccin
#elif L==7	// it
	"Altre informazioni";
#elif L==8	// pl
	"Inne informacje";
#elif L==9	// pt
	"Outras informa&ccedil;&otilde;es";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Place_of_exam =
#if   L==1	// ca
	"Lugar de realizaci&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Pr&uuml;fungsort";
#elif L==3	// en
	"Place of exam";
#elif L==4	// es
	"Lugar de realizaci&oacute;n";
#elif L==5	// fr
	"Lieu de l'examen";
#elif L==6	// gn
	"Lugar de realizaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Luogo d'esame";
#elif L==8	// pl
	"Miejsce egzaminu";
#elif L==9	// pt
	"Local do exame";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Session =
#if   L==1	// ca
	"Convocatoria<br />(junio, septiembre,&hellip;)";	// Necessita traduccio
#elif L==2	// de
	"Einberufung<br />(Juni, September,&hellip;)";
#elif L==3	// en
	"Session<br />(june, september,&hellip;)";
#elif L==4	// es
	"Convocatoria<br />(junio, septiembre,&hellip;)";
#elif L==5	// fr
	"P&eacute;riode<br />(juin, septembre,&hellip;)";
#elif L==6	// gn
	"Convocatoria<br />(junio, septiembre,&hellip;)";	// Okoteve traduccin
#elif L==7	// it
	"Sessione<br />(giugno, settembre,&hellip;)";
#elif L==8	// pl
	"Sesja<br />(czerwiec, wrzesien,&hellip;)";
#elif L==9	// pt
	"Per&iacute;odo<br />(junho, setembro,&hellip;)";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Start_time =
#if   L==1	// ca
	"Hora de inicio";	// Necessita traduccio
#elif L==2	// de
	"Beginn um";
#elif L==3	// en
	"Start time";
#elif L==4	// es
	"Hora de inicio";
#elif L==5	// fr
	"Heure de d&eacute;but";
#elif L==6	// gn
	"Hora de inicio";	// Okoteve traduccin
#elif L==7	// it
	"Ora d'inizio";
#elif L==8	// pl
	"Czas rozpoczecia";
#elif L==9	// pt
	"Hora de in&iacute;cio";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Structure_of_the_exam =
#if   L==1	// ca
	"Estructura del examen";	// Necessita traduccio
#elif L==2	// de
	"Pr&uuml;fungsstruktur";
#elif L==3	// en
	"Structure of the exam";
#elif L==4	// es
	"Estructura del examen";
#elif L==5	// fr
	"Structure de l'examen";
#elif L==6	// gn
	"Estructura del examen";	// Okoteve traduccin
#elif L==7	// it
	"Struttura d'esame";
#elif L==8	// pl
	"Struktura egzaminu";
#elif L==9	// pt
	"Estrutura do exame";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Year_or_semester =	// 1, 2, 3..., not 1984, 2038...
#if   L==1	// ca
	"Any o semestre";
#elif L==2	// de
	"Jahr oder Semester";
#elif L==3	// en
	"Year or semester";
#elif L==4	// es
	"A&ntilde;o o semestre";
#elif L==5	// fr
	"Ann&eacute;e ou semestre";
#elif L==6	// gn
	"A&ntilde;o o semestre";	// Okoteve traduccin
#elif L==7	// it
	"Anno o semestre";
#elif L==8	// pl
	"Rok lub semestr";
#elif L==9	// pt
	"Ano ou semestre";
#endif

const char *Txt_Exam_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Examen de %s";
#elif L==2	// de
	"Pr&uuml;fung der %s";
#elif L==3	// en
	"Exam of %s";
#elif L==4	// es
	"Examen de %s";
#elif L==5	// fr
	"Examen de %s";
#elif L==6	// gn
	"Examen de %s";		// Okoteve traduccin
#elif L==7	// it
	"Esame di %s";
#elif L==8	// pl
	"Egzamin z %s";
#elif L==9	// pt
	"Exame de %s";
#endif

const char *Txt_Existing_question =
#if   L==1	// ca
	"Pregunta ja existent";
#elif L==2	// de
	"Bestehende Frage";
#elif L==3	// en
	"Existing question";
#elif L==4	// es
	"Pregunta ya existente";
#elif L==5	// fr
	"Question existante";
#elif L==6	// gn
	"Pregunta ya existente";	// Okoteve traduccin
#elif L==7	// it
	"Domanda esistente";
#elif L==8	// pl
	"Istniej&aogon;ca pytanie";
#elif L==9	// pt
	"Quest&atilde;o existente";
#endif

const char *Txt_Expand =
#if   L==1	// ca
	"Expandir";
#elif L==2	// de
	"Erweitern";
#elif L==3	// en
	"Expand";
#elif L==4	// es
	"Expandir";
#elif L==5	// fr
	"&Eacute;largir";
#elif L==6	// gn
	"Expandir";	// Okoteve traduccin
#elif L==7	// it
	"Espandere";
#elif L==8	// pl
	"Rozwin";
#elif L==9	// pt
	"Expandir";
#endif

const char *Txt_Export_questions =
#if   L==1	// ca
	"Exportar preguntas";	// Necessita traduccio
#elif L==2	// de
	"Export questions";	// Need bersetzung
#elif L==3	// en
	"Export questions";
#elif L==4	// es
	"Exportar preguntas";
#elif L==5	// fr
	"Exporter questions";
#elif L==6	// gn
	"Exportar preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Export questions";	// Bisogno di traduzione
#elif L==8	// pl
	"Export questions";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Exportar perguntas";
#endif

const char *Txt_Extended_public_profile =
#if   L==1	// ca
	"Perfil p&uacute;blic ampliat";
#elif L==2	// de
	"Erweitertes &ouml;ffentliches Profile";
#elif L==3	// en
	"Extended public profile";
#elif L==4	// es
	"Perfil p&uacute;blico extendido";
#elif L==5	// fr
	"Profil public &eacute;tendu";
#elif L==6	// gn
	"Perfil p&uacute;blico extendido";	// Okoteve traduccin
#elif L==7	// it
	"Profilo pubblico esteso";
#elif L==8	// pl
	"Rozszerzony profil publiczny";
#elif L==9	// pt
	"Perfil p&uacute;blico estendido";
#endif

const char *Txt_X_faces_have_been_detected_in_front_position_1_Z_ =	// Warning: it is very important to include two %u in the following sentences
#if   L==1	// ca
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";	// Necessita traduccio
#elif L==2	// de
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";			// Need bersetzung
#elif L==3	// en
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";
#elif L==4	// es
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";
#elif L==5	// fr
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";			// Besoin de traduction
#elif L==6	// gn
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==7	// it
	"%u visi sono stati individuati in posizione centrale:<br />"
	"1 con lo sfondo abbastanza chiaro (segnato in verde).<br />"
	"%u con lo sfondo troppo scuro (segnato in rosso).<br />"
	"Se vuoi scegliere il viso segnato in verde, clicca su di esso.";
#elif L==8	// pl
	"%u twarze zostaly wykryte w przednim:<br />"
	"1 z dosc jasnym tle (zaznaczone na zielono).<br />"
	"%u z tlem zbyt ciemny (zaznaczone na czerwono).<br />"
	"Jesli chcesz wybrac twarz zaznaczone na zielono, kliknij na niego.";
#elif L==9	// pt
	"%u faces foram detectadas na posi&ccedil;&atilde;o frontal:<br />"
	"1 com o fundo claro o suficiente (marcado em verde).<br />"
	"%u com o fundo muito escuro (marcado em vermelho).<br />"
	"Se voc&ecirc; quiser escolher o rosto marcado em verde, clique nele.";
#endif

const char *Txt_X_faces_have_been_detected_in_front_position_Y_Z_ =	// Warning: it is very important to include three %u in the following sentences
#if   L==1	// ca
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";	// Necessita traduccio
#elif L==2	// de
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";			// Need bersetzung
#elif L==3	// en
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";
#elif L==4	// es
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";
#elif L==5	// fr
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";			// Besoin de traduction
#elif L==6	// gn
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==7	// it
	"%u visi sono stati individuati in posizione centrale:<br />"
	"%u con lo sfondo abbastanza chiaro (segnato il verde).<br />"
	"%u con lo sfondo troppo scuro (segnato in rosso).<br />"
	"Se vuoi scegliere uno dei visi segnati in verde, clicca su di esso.";
#elif L==8	// pl
	"%u twarze zostaly wykryte w przednim:<br />"
	"%u z dosc jasnym tlem (zaznaczone na zielono).<br />"
	"%u z tlem zbyt ciemnym (zaznaczone na czerwono).<br />"
	"Jesli chcesz wybrac jedna z twarzy zaznaczone na zielono, kliknij na niego.";
#elif L==9	// pt
	"%u faces foram detectadas na posi&ccedil;&atilde;o frontal:<br />"
	"%u com o fundo claro o suficiente (marcado em verde).<br />"
	"%u com o fundo muito escuro (marcado em vermelho).<br />"
	"Se voc&ecirc; quiser escolher um dos rostos marcados em verde, clique nele.";
#endif

const char *Txt_X_faces_marked_in_green_have_been_detected_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'han detectat %u cares (assenyalats en verd) en posici&oacute; frontal"
	" i amb el fons prou clar.<br />"
	"Si voleu triar un d'aquests rostres, premi sobre ell.";
#elif L==2	// de
	"%u Fl&auml;chen (gr&uuml;n markiert) wurden vor Position erfasst,"
	" und der Hintergrund ist leicht genug.<br />"
	"Wenn Sie eine dieser Fl&auml;chen zu w&auml;hlen, klicken Sie darauf.";
#elif L==3	// en
	"%u faces (marked in green) have been detected in front position,"
	" and the background is light enough.<br />"
	"If you want to choose one of those faces, click on it.";
#elif L==4	// es
	"Se han detectado %u rostros (se&ntilde;alados en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Si desea elegir uno de esos rostros, pulse sobre &eacute;l.";
#elif L==5	// fr
	"%u visages ont &eacute;t&eacute; d&eacute;tect&eacute;s (indiqu&eacute;s en vert) dans la position avant"
	" et avec un fond suffisamment claire.<br />"
	"Si vous voulez choisir un de ces visages, cliquez dessus.";
#elif L==6	// gn
	"Se han detectado %u rostros (se&ntilde;alados en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Si desea elegir uno de esos rostros, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==7	// it
	"%u visi (segnati in verde) sono stati individuati in posizione frontale,"
	" e lo sfondo &egrave; abbastanza chiaro.<br />"
	"Se vuoi scegliere uno di questi visi, clicca su di esso.";
#elif L==8	// pl
	"%u powierzchnie zosta&lstrok;y wykryte (oznaczony na zielono) w po&lstrok;o&zdot;eniu przednim,"
	" o dostatecznie jasne t&lstrok;o.<br />"
	"Je&zdot;eli chcesz wybra&cacute; jedn&aogon; z tych twarzy, kliknij na ni&aogon;.";
#elif L==9	// pt
	"%u faces foram detectados (indicado em verde) na posi&ccedil;&atilde;o frontal"
	" e com fundo suficientemente claro.<br />"
	"Se voc&ecirc; quer escolher um daqueles rostos, clique sobre ele.";
#endif

const char *Txt_X_faces_marked_in_red_have_been_detected_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"S'han detectat %u cares (assenyalats en vermell) en posici&oacute; frontal,"
	" per&ograve; el fons &eacute;s massa fosc.";
#elif L==2	// de
	"%u wurden Fl&auml;chen (in rot abgebildet) in der vorderen Position erkannt,"
	" aber der Hintergrund ist zu dunkel.";
#elif L==3	// en
	"%u faces (marked in red) have been detected in front position,"
	" but the background is too dark.";
#elif L==4	// es
	"Se han detectado %u rostros (se&ntilde;alados en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";
#elif L==5	// fr
	"%u faces (en rouge) ont &eacute;t&eacute; d&eacute;tect&eacute;s dans la position avant,"
	" mais le fond est trop sombre.";
#elif L==6	// gn
	"Se han detectado %u rostros (se&ntilde;alados en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";	// Okoteve traduccin
#elif L==7	// it
	"%u visi (segnati in rosso) sono stati individuati in posizione centrale,"
	" ma lo sfondo &egrave; troppo scuro.";
#elif L==8	// pl
	"%u wykryto twarze (pokazane na czerwono) w przednim po&lstrok;o&ztot;eniu,"
	" ale t&lstrok;o jest zbyt ciemne.";
#elif L==9	// pt
	"%u rostos foram detectados (mostrados em vermelho) na posi&ccedil;&atilde;o frontal,"
	" mas o fundo &eacute; muito escuro.";
#endif

const char *Txt_Faces_detected =
#if   L==1	// ca
	"Rostres detectats";
#elif L==2	// de
	"Erkannte Gesichter";
#elif L==3	// en
	"Faces detected";
#elif L==4	// es
	"Rostros detectados";
#elif L==5	// fr
	"Visages d&eacute;tect&eacute;s";
#elif L==6	// gn
	"Rostros detectados";		// Okoteve traduccin
#elif L==7	// it
	"Volti rilevati";
#elif L==8	// pl
	"Wykryto twarze";
#elif L==9	// pt
	"Rostos detectados";
#endif

const char *Txt_Failed_email_confirmation_key =
#if   L==1	// ca
	"Error en la clave de confirmaci&oacute;n de correo.";	// Necessita traduccio
#elif L==2	// de
	"Failed email confirmation key.";			// Need bersetzung
#elif L==3	// en
	"Failed email confirmation key.";
#elif L==4	// es
	"Error en la clave de confirmaci&oacute;n de correo.";
#elif L==5	// fr
	"Failed email confirmation key.";			// Besoin de traduction
#elif L==6	// gn
	"Error en la clave de confirmaci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7	// it
	"Failed email confirmation key.";			// Bisogno di traduzione
#elif L==8	// pl
	"Failed email confirmation key.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Erro na chave de confirma&ccedil;&atilde;o de email.";
#endif

const char *Txt_Family_address =
#if   L==1	// ca
	"Domicilio familiar";	// Necessita traduccio
#elif L==2	// de
	"Heimatadresse";
#elif L==3	// en
	"Family address";
#elif L==4	// es
	"Domicilio familiar";
#elif L==5	// fr
	"Adresse famille";
#elif L==6	// gn
	"Domicilio familiar";	// Okoteve traduccin
#elif L==7	// it
	"Domicilio familiare";
#elif L==8	// pl
	"Adres rodziny";
#elif L==9	// pt
	"Endere&ccedil;o Fam&iacute;lia";
#endif

const char *Txt_Feedback =
#if   L==1	// ca
	"Realimentaci&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Feedback";
#elif L==3	// en
	"Feedback";
#elif L==4	// es
	"Realimentaci&oacute;n";
#elif L==5	// fr
	"Feedback";
#elif L==6	// gn
	"Realimentaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Feedback";
#elif L==8	// pl
	"Opinie";
#elif L==9	// pt
	"Feedback";
#endif

const char *Txt_Feedback_to_students =
#if   L==1	// ca
	"Realimentaci&oacute;n para estudiantes";	// Necessita traduccio
#elif L==2	// de
	"Feedback an die Studenten";
#elif L==3	// en
	"Feedback to students";
#elif L==4	// es
	"Realimentaci&oacute;n para estudiantes";
#elif L==5	// fr
	"Feedback pour les &eacute;tudiants";
#elif L==6	// gn
	"Realimentaci&oacute;n para estudiantes";	// Okoteve traduccin
#elif L==7	// it
	"Feedback per gli studenti";
#elif L==8	// pl
	"Opinie student&oacute;w";
#elif L==9	// pt
	"Feedback para estudantes";
#endif

const char *Txt_Field_BR_name =
#if   L==1	// ca
	"Nombre<br />del campo";	// Necessita traduccio
#elif L==2	// de
	"Name des Feldes";
#elif L==3	// en
	"Field<br />name";
#elif L==4	// es
	"Nombre<br />del campo";
#elif L==5	// fr
	"Nom<br />du champ";
#elif L==6	// gn
	"Nombre<br />del campo";	// Okoteve traduccin
#elif L==7	// it
	"Nome<br />del campo";
#elif L==8	// pl
	"Nazwa<br />pola";
#elif L==9	// pt
	"Nome<br />do campo";
#endif

const char *Txt_Figures =
#if   L==1	// ca
	"Xifres";
#elif L==2	// de
	"Zahlen";
#elif L==3	// en
	"Figures";
#elif L==4	// es
	"Cifras";
#elif L==5	// fr
	"Chiffres";
#elif L==6	// gn
	"Cifras";	// Okoteve traduccin
#elif L==7	// it
	"Cifre";
#elif L==8	// pl
	"Cyfry";
#elif L==9	// pt
	"N&uacute;meros";
#endif

const char *Txt_File =
#if   L==1	// ca
	"Arxiu";
#elif L==2	// de
	"Datei";
#elif L==3	// en
	"File";
#elif L==4	// es
	"Archivo";
#elif L==5	// fr
	"Fichier";
#elif L==6	// gn
	"Jehai'aty";
#elif L==7	// it
	"File";
#elif L==8	// pl
	"Plik";
#elif L==9	// pt
	"Arquivo";
#endif

const char *Txt_file =
#if   L==1	// ca
	"arxiu";
#elif L==2	// de
	"Datei";
#elif L==3	// en
	"file";
#elif L==4	// es
	"archivo";
#elif L==5	// fr
	"fichier";
#elif L==6	// gn
	"jehai'aty";
#elif L==7	// it
	"file";
#elif L==8	// pl
	"Plik";
#elif L==9	// pt
	"arquivo";
#endif

const char *Txt_X_file =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Arxiu %s";
#elif L==2	// de
	"%s-Datei";
#elif L==3	// en
	"%s file";
#elif L==4	// es
	"Archivo %s";
#elif L==5	// fr
	"Fichier %s";
#elif L==6	// gn
	"Archivo %s";	// Okoteve traduccin
#elif L==7	// it
	"File %s";
#elif L==8	// pl
	"Plik %s";
#elif L==9	// pt
	"Arquivo %s";
#endif

const char *Txt_File_not_found =
#if   L==1	// ca
	"Arxiu no trobat";
#elif L==2	// de
	"Datei nicht gefunden";
#elif L==3	// en
	"File not found";
#elif L==4	// es
	"Archivo no encontrado";
#elif L==5	// fr
	"Fichier non trouv&eacute;";
#elif L==6	// gn
	"Archivo no encontrado";	// Okoteve traduccin
#elif L==7	// it
	"File non trovato";
#elif L==8	// pl
	"Nie znaleziono pliku";
#elif L==9	// pt
	"Arquivo n&atilde;o encontrado";
#endif

const char *Txt_FILE_uncompressed =
#if   L==1	// ca
	"sense comprimir";
#elif L==2	// de
	"unkomprimierte";
#elif L==3	// en
	"uncrompressed";
#elif L==4	// es
	"sin comprimir";
#elif L==5	// fr
	"non compress&eacute;";
#elif L==6	// gn
	"sin comprimir";	// Okoteve traduccin
#elif L==7	// it
	"non compresso";
#elif L==8	// pl
	"nieskompresowane";
#elif L==9	// pt
	"descompactado";
#endif

const char *Txt_Filename =
#if   L==1	// ca
	"Nom del arxiu";
#elif L==2	// de
	"Dateiname";
#elif L==3	// en
	"Filename";
#elif L==4	// es
	"Nombre del archivo";
#elif L==5	// fr
	"Nom du fichier";
#elif L==6	// gn
	"Nombre del archivo";	// Okoteve traduccin
#elif L==7	// it
	"Nome del file";
#elif L==8	// pl
	"Nazwa";
#elif L==9	// pt
	"Nome do arquivo";
#endif

const char *Txt_file_folder =
#if   L==1	// ca
	"arxiu/carpeta";
#elif L==2	// de
	"Datei/Verzeichnis";
#elif L==3	// en
	"file/folder";
#elif L==4	// es
	"archivo/carpeta";
#elif L==5	// fr
	"fichier/r&eacute;pertoire";
#elif L==6	// gn
	"archivo/carpeta";	// Okoteve traduccin
#elif L==7	// it
	"file/cartella";
#elif L==8	// pl
	"Plik/folder";
#elif L==9	// pt
	"arquivo/diret&oacute;rio";
#endif

const char *Txt_FILE_FOLDER_OR_LINK_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ahora est&aacute; oculto.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==3	// en
	"<strong>%s</strong> is now hidden.";
#elif L==4	// es
	"<strong>%s</strong> ahora est&aacute; oculto.";
#elif L==5	// fr
	"<strong>%s</strong> est maintenant cach&eacute;.";
#elif L==6	// gn
	"<strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; ora nascosto.";
#elif L==8	// pl
	"<strong>%s</strong> jest ukryty.";
#elif L==9	// pt
	"<strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_FILE_FOLDER_OR_LINK_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ahora est&aacute; visible.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> ist jetzt sichtbar.";
#elif L==3	// en
	"<strong>%s</strong> is now visible.";
#elif L==4	// es
	"<strong>%s</strong> ahora est&aacute; visible.";
#elif L==5	// fr
	"<strong>%s</strong> est maintenant visible.";
#elif L==6	// gn
	"<strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; ora visibile.";
#elif L==8	// pl
	"<strong>%s</strong> jest teraz widoczny.";
#elif L==9	// pt
	"<strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_FILE_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> eliminat.";
#elif L==2	// de
	"<strong>%s</strong> entfernt.";
#elif L==3	// en
	"<strong>%s</strong> removed.";
#elif L==4	// es
	"<strong>%s</strong> eliminado.";
#elif L==5	// fr
	"<strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"<strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> rimosso.";
#elif L==8	// pl
	"<strong>%s</strong> usuniety.";
#elif L==9	// pt
	"<strong>%s</strong> removido.";
#endif

const char *Txt_File_size =
#if   L==1	// ca
	"Mida del fitxer";
#elif L==2	// de
	"Dateigr&ouml;&szlig;e";
#elif L==3	// en
	"File size";
#elif L==4	// es
	"Tama&ntilde;o del archivo";
#elif L==5	// fr
	"Taille du fichier";
#elif L==6	// gn
	"Tama&ntilde;o del archivo";	// Okoteve traduccin
#elif L==7	// it
	"Dimensioni del file";
#elif L==8	// pl
	"Rozmiar pliku";
#elif L==9	// pt
	"Tamanho do arquivo";
#endif

const char *Txt_File_with_the_logo =
#if   L==1	// ca
	"Fitxer amb el logo";
#elif L==2	// de
	"Datei mit dem Logo";
#elif L==3	// en
	"File with the logo";
#elif L==4	// es
	"Archivo con el logo";
#elif L==5	// fr
	"Fichier avec le logo";
#elif L==6	// gn
	"Archivo con el logo";	// Okoteve traduccin
#elif L==7	// it
	"File con il logo";
#elif L==8	// pl
	"Plik ze logo";
#elif L==9	// pt
	"Arquivo com o logotipo";
#endif

const char *Txt_File_with_the_photo =
#if   L==1	// ca
	"Fitxer amb la foto";
#elif L==2	// de
	"Datei mit dem Foto";
#elif L==3	// en
	"File with the photo";
#elif L==4	// es
	"Archivo con la foto";
#elif L==5	// fr
	"Fichier avec la photo";
#elif L==6	// gn
	"Archivo con la foto";	// Okoteve traduccin
#elif L==7	// it
	"File con la foto";
#elif L==8	// pl
	"Plik ze zdjeciem";
#elif L==9	// pt
	"Arquivo com a foto";
#endif

const char *Txt_File_zone =
#if   L==1	// ca
	"Zona d'arxius";
#elif L==2	// de
	"Dateienbereich";
#elif L==3	// en
	"File zone";
#elif L==4	// es
	"Zona de archivos";
#elif L==5	// fr
	"Zone de fichiers";
#elif L==6	// gn
	"Zona de archivos";	// Okoteve traduccin
#elif L==7	// it
	"Area file";
#elif L==8	// pl
	"File zone";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Zona de arquivos";
#endif

const char *Txt_File_zones =
#if   L==1	// ca
	"Zones d'arxius";
#elif L==2	// de
	"Dateienbereiche";
#elif L==3	// en
	"File zones";
#elif L==4	// es
	"Zonas de archivos";
#elif L==5	// fr
	"Zones de fichiers";
#elif L==6	// gn
	"Zonas de archivos";	// Okoteve traduccin
#elif L==7	// it
	"Area file";
#elif L==8	// pl
	"Plik obszar&oacute;w";
#elif L==9	// pt
	"Zonas de arquivos";
#endif

const char *Txt_File_zones_disabled =
#if   L==1	// ca
	"Zonas de archivos inhabilitadas";	// Necessita traduccio
#elif L==2	// de
	"File zones disabled";			// Need bersetzung
#elif L==3	// en
	"File zones disabled";
#elif L==4	// es
	"Zonas de archivos inhabilitadas";
#elif L==5	// fr
	"File zones disabled";			// Besoin de traduction
#elif L==6	// gn
	"Zonas de archivos inhabilitadas";	// Okoteve traduccin
#elif L==7	// it
	"Area file disabilitata";
#elif L==8	// pl
	"Plik strefy niedostepnej";
#elif L==9	// pt
	"Zonas de arquivos desativadas";
#endif

const char *Txt_File_zones_of_the_group_X_are_now_disabled =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";	// Necessita traduccio
#elif L==2	// de
	"File zones of the group %s are now disabled.";				// Need bersetzung
#elif L==3	// en
	"File zones of the group %s are now disabled.";
#elif L==4	// es
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";
#elif L==5	// fr
	"File zones of the group %s are now disabled.";				// Besoin de traduction
#elif L==6	// gn
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";	// Okoteve traduccin
#elif L==7	// it
	"Le aree file del gruppo %s sono ora disabilitate.";
#elif L==8	// pl
	"Plik strefy grupy %s jest teraz niedostepny.";
#elif L==9	// pt
	"As zonas de arquivos do grupo %s est&atilde;o agora desativadas.";
#endif

const char *Txt_File_zones_of_the_group_X_are_now_enabled =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";	// Necessita traduccio
#elif L==2	// de
	"File zones of the group %s are now enabled.";				// Need bersetzung
#elif L==3	// en
	"File zones of the group %s are now enabled.";
#elif L==4	// es
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";
#elif L==5	// fr
	"File zones of the group %s are now enabled.";				// Besoin de traduction
#elif L==6	// gn
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";	// Okoteve traduccin
#elif L==7	// it
	"Le aree file del gruppo %s sono ora abilitate.";
#elif L==8	// pl
	"Plik strefy grupa %s sa obecnie dost&eogon;pne.";
#elif L==9	// pt
	"As zonas de arquivos do grupo %s est&atilde;o agora ativadas.";
#endif

const char *Txt_File_zones_of_the_group_X_enabled_click_to_disable_them =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Zonas de archivos del grupo %s habilitadas; pulse para inhabilitarlas";	// Necessita traduccio
#elif L==2	// de
	"File zones of the group %s enabled, click to disable them";			// Need bersetzung
#elif L==3	// en
	"File zones of the group %s enabled, click to disable them";
#elif L==4	// es
	"Zonas de archivos del grupo %s habilitadas; pulse para inhabilitarlas";
#elif L==5	// fr
	"File zones of the group %s enabled, click to disable them";			// Besoin de traduction
#elif L==6	// gn
	"Zonas de archivos del grupo %s habilitadas; pulse para inhabilitarlas";	// Okoteve traduccin
#elif L==7	// it
	"Aree file del gruppo %s abilitate, clicca per disabilitarle";
#elif L==8	// pl
	"Plik strefy grupy %s aktywny, kliknij, aby je wylaczyc";
#elif L==9	// pt
	"Zonas de arquivos do grupo %s ativadas, clique para desativ&aacute;-las";
#endif

const char *Txt_File_zones_of_the_group_X_disabled_click_to_enable_them =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Zonas de archivos del grupo %s inhabilitadas; pulse para habilitarlas";	// Necessita traduccio
#elif L==2	// de
	"File zones of the group %s disabled, click to enable them";			// Need bersetzung
#elif L==3	// en
	"File zones of the group %s disabled, click to enable them";
#elif L==4	// es
	"Zonas de archivos del grupo %s inhabilitadas; pulse para habilitarlas";
#elif L==5	// fr
	"File zones of the group %s disabled, click to enable them";			// Besoin de traduction
#elif L==6	// gn
	"Zonas de archivos del grupo %s inhabilitadas; pulse para habilitarlas";	// Okoteve traduccin
#elif L==7	// it
	"Aree file del gruppo %s disabilitate, clicca per abilitarle";
#elif L==8	// pl
	"Plik strefy grupy %s jes niedostepny, kliknij aby wlaczyc";
#elif L==9	// pt
	"Zonas de arquivos do grupo %s desativadas, clique para ativ&aacute;-las";
#endif

const char *Txt_Files =
#if   L==1	// ca
	"Fitxers";
#elif L==2	// de
	"Dateien";
#elif L==3	// en
	"Files";
#elif L==4	// es
	"Archivos";
#elif L==5	// fr
	"Fichiers";
#elif L==6	// gn
	"Archivos";	// Okoteve traduccin
#elif L==7	// it
	"File";
#elif L==8	// pl
	"Pliki";
#elif L==9	// pt
	"Arquivos";
#endif

const char *Txt_files =
#if   L==1	// ca
	"fitxers";
#elif L==2	// de
	"Dateien";
#elif L==3	// en
	"files";
#elif L==4	// es
	"archivos";
#elif L==5	// fr
	"fichiers";
#elif L==6	// gn
	"archivos";	// Okoteve traduccin
#elif L==7	// it
	"file";
#elif L==8	// pl
	"pliki";
#elif L==9	// pt
	"arquivos";
#endif

const char *Txt_Files_assignments =
#if   L==1	// ca
	"Archivos actividades";	// Necessita traduccio
#elif L==2	// de
	"Dateien der Aufgaben";
#elif L==3	// en
	"Files assignments";
#elif L==4	// es
	"Archivos actividades";
#elif L==5	// fr
	"Fichiers activites";
#elif L==6	// gn
	"Archivos actividades";	// Okoteve traduccin
#elif L==7	// it
	"File attivit&agrave";
#elif L==8	// pl
	"Pliki zadan";
#elif L==9	// pt
	"Arquivos atividades";
#endif

const char *Txt_Files_copied =
#if   L==1	// ca
	"Arxius copiats";
#elif L==2	// de
	"Dateien kopiert";
#elif L==3	// en
	"Files copied";
#elif L==4	// es
	"Archivos copiados";
#elif L==5	// fr
	"Fichiers copi&eacute;s";
#elif L==6	// gn
	"Archivos copiados";	// Okoteve traduccin
#elif L==7	// it
	"File copiati";
#elif L==8	// pl
	"Pliki kopiowane";
#elif L==9	// pt
	"Arquivos copiados";
#endif

const char *Txt_Files_uploaded =
#if   L==1	// ca
	"Fitxers pujats";
#elif L==2	// de
	"Dateien hochgeladen";
#elif L==3	// en
	"Files uploaded";
#elif L==4	// es
	"Archivos subidos";
#elif L==5	// fr
	"Fichiers t&eacute;l&eacute;charg&eacute;s";
#elif L==6	// gn
	"Archivos subidos";	// Okoteve traduccin
#elif L==7	// it
	"File caricati";
#elif L==8	// pl
	"Pliki dodane";
#elif L==9	// pt
	"Arquivos enviados";
#endif

const char *Txt_Files_of_marks_must_contain_a_table_in_HTML_format_ =
#if   L==1	// ca
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";			// Necessita traduccio
#elif L==2	// de
	"Bewertungsdateien mssen eine Tabelle im <strong>HTML-Format</strong> ohne Bilder enthalten."
	" Die HTML-Dateien k&ouml;nnen z.B. aus einer Kalkulationstabelle erstellt werden,"
	" indem der gewnschte Bereich aus der Tabelle ausgew&auml;hlt und als HTML gespeichert wird.<br />"
	" Eine Tabelle kann <strong>mehrere Kopf- und Fuzeilen enthalten</strong>."
	" Zwischen solchen Zeilen muss eine <strong>einzige Zeile f&uuml; jeden Studenten bestehen</strong>,"
	" mit einer beliebigen Anzahl von Spalten, <strong>wovon die erste die Ausweis-Nr. enthalten muss</strong>.<br />"
	" Ruft ein Student die Tabelle auf, sieht er nur die f&uuml;r ihn bestimmte Zeile.";
#elif L==3	// en
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows,"
	" followed by the row with his/her marks,"
	" and footer rows.";
#elif L==4	// es
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";
#elif L==5	// fr
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows,"
	" followed by the row with his/her marks,"
	" and footer rows.";				// Besoin de traduction
#elif L==6	// gn
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";			// Okoteve traduccin
#elif L==7	// it
	"I file dei risultati devono apparire in una tabella in <strong>formato HTML</strong> senza immagini."
	" Ogni file HTML pu&ograve; essere generato, per esempio, da un foglio di calcolo elettronico,"
	" selezionando la desiderata porzione di tabella e salvandolo come una pagina web.<br />"
	" Una tabella pu&ograve; avere <strong>intestazioni multiple e righe a pi&egrave; di pagina</strong>."
	" Tra di loro dovrebbe esserci <strong>una singola riga per ogni studente</strong>,"
	" con alcuni numeri di colonne, <strong>la prima colonna contiene l'ID dello studente</strong>.<br />"
	" Uno studente vedr&agrave; una tabella con righe intestate,"
	" seguite da una riga con i loro risultati,"
	" e righe a pi&egrave; di pagina.";
#elif L==8	// pl
	"Pliki znak&oacute;w musza zawierac tabele <strong>w formacie HTML</strong> bez zdjecia."
	" Kazdy plik HTML moze zostac wygenerowany, na przyklad z arkusza kalkulacyjnego,"
	" wybierajac odpowiednia czesc tabeli i zapisanie go jako strone sieci Web.<br />"
	" Tabela moze miec <strong>wielu wiersze nagl&oacute;wka i stopki</strong>."
	" Pomiedzy nimi nalezy <strong>jednym wierszu dla kazdego ucznia</strong>,"
	" z dowolna iloscia kolumn, <strong>pierwszej kolumnie zawierajacej studenta ID</strong>.<br />"
	" Student zobaczy tabele z wierszy nagl&oacute;wka, a nastepnie przez rzad z ich znak&oacute;w towarowych i stopki.";
#elif L==9	// pt
	"Os arquivos de notas devem conter uma tabela no <strong>formato HTML</strong> sem imagens."
	" Cada arquivo HTML pode ser gerado, por exemplo, a partir de uma planilha eletr&ocirc;nica,"
	" selecionando a parte desejada da tabela e salvando-a como p&aacute;gina web.<br />"
	" Uma tabela pode ter <strong>v&aacute;rias linhas de cabe&ccedil;alho e rodap&eacute;</strong>."
	" Entre eles deve haver <strong>uma &uacute;nica linha para cada estudante</strong>,"
	" com qualquer n&uacute;mero de colunas, <strong>a primeira coluna contendo o ID do estudante</strong>.<br />"
	" Um estudante ver&aacute; uma tabela com linhas de cabe&ccedil;alho,"
	" seguida pela linha com suas notas,"
	" e linhas de rodap&eacute;.";
#endif

const char *Txt_Files_removed =
#if   L==1	// ca
	"Arxius eliminats";
#elif L==2	// de
	"Dateien entfernt";
#elif L==3	// en
	"Files removed";
#elif L==4	// es
	"Archivos eliminados";
#elif L==5	// fr
	"Fichiers supprim&eacute;s";
#elif L==6	// gn
	"Archivos eliminados";	// Okoteve traduccin
#elif L==7	// it
	"File rimossi";
#elif L==8	// pl
	"Pliki usuni&eogon;te";
#elif L==9	// pt
	"Arquivos removidos";
#endif

const char *Txt_Files_works =
#if   L==1	// ca
	"Archivos trabajos";	// Necessita traduccio
#elif L==2	// de
	"Dateien Hausarbeiten";
#elif L==3	// en
	"Files homework";
#elif L==4	// es
	"Archivos trabajos";
#elif L==5	// fr
	"Fichiers travaux";
#elif L==6	// gn
	"Archivos trabajos";	// Okoteve traduccin
#elif L==7	// it
	"File lavori";
#elif L==8	// pl
	"Pliki domowe";
#elif L==9	// pt
	"Arquivos trabalhos";
#endif

const char *Txt_Filter =	// As a noun
#if   L==1	// ca
	"Filtre";
#elif L==2	// de
	"Filter";
#elif L==3	// en
	"Filter";
#elif L==4	// es
	"Filtro";
#elif L==5	// fr
	"Filtre";
#elif L==6	// gn
	"Ayga";
#elif L==7	// it
	"Filtro";
#elif L==8	// pl
	"Filtr";
#elif L==9	// pt
	"Filtro";
#endif

const char *Txt_First_day_of_the_week =
#if   L==1	// ca
	"Primer dia de la setmana";
#elif L==2	// de
	"Erster Tag der Woche";
#elif L==3	// en
	"First day of the week";
#elif L==4	// es
	"Primer d&iacute;a de la semana";
#elif L==5	// fr
	"Premier jour de la semaine";
#elif L==6	// gn
	"Primer d&iacute;a de la semana";	// Okoteve traduccin
#elif L==7	// it
	"Primo giorno della settimana";
#elif L==8	// pl
	"Pierwszy dzie&nacute; tygodnia";
#elif L==9	// pt
	"Primeiro dia da semana";
#endif

const char *Txt_First_name =
#if   L==1	// ca
	"Nom";
#elif L==2	// de
	"Vorname";
#elif L==3	// en
	"First name";
#elif L==4	// es
	"Nombre";
#elif L==5	// fr
	"Pr&eacute;nom";
#elif L==6	// gn
	"T&eacute;ra";
#elif L==7	// it
	"Primo nome";
#elif L==8	// pl
	"Imi&eogon;";
#elif L==9	// pt
	"Nome";
#endif

const char *Txt_Folder =
#if   L==1	// ca
	"Carpeta";
#elif L==2	// de
	"Verzeichnis";
#elif L==3	// en
	"Folder";
#elif L==4	// es
	"Carpeta";
#elif L==5	// fr
	"R&eacute;pertoire";
#elif L==6	// gn
	"Carpeta";	// Okoteve traduccin
#elif L==7	// it
	"Cartella";
#elif L==8	// pl
	"Folder";
#elif L==9	// pt
	"Diret&oacute;rio";
#endif

const char *Txt_folder =
#if   L==1	// ca
	"carpeta";
#elif L==2	// de
	"Verzeichnis";
#elif L==3	// en
	"folder";
#elif L==4	// es
	"carpeta";
#elif L==5	// fr
	"r&eacute;pertoire";
#elif L==6	// gn
	"carpeta";	// Okoteve traduccin
#elif L==7	// it
	"cartella";
#elif L==8	// pl
	"folder";
#elif L==9	// pt
	"diret&oacute;rio";
#endif

const char *Txt_Folder_X_and_all_its_contents_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";	// Necessita traduccio
#elif L==2	// de
	"Verzeichnis <strong>%s</strong> und alle Inhalte wurden entfernt.";
#elif L==3	// en
	"Folder <strong>%s</strong> and all its contents removed.";
#elif L==4	// es
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";
#elif L==5	// fr
	"R&eacute;pertoire <strong>%s</strong> et l'ensemble de son contenu supprim&eacute;s.";
#elif L==6	// gn
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";	// Okoteve traduccin
#elif L==7	// it
	"Cartella <strong>%s</strong> e tutto il suo contenuto rimossi.";
#elif L==8	// pl
	"Folder <strong>%s</strong> i cala jego zawartosc usunieta.";
#elif L==9	// pt
	"Diret&oacute;rio <strong>%s</strong> e todo o seu conte&uacute;do removidos.";
#endif

const char *Txt_Folder_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Carpeta <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2	// de
	"Verzeichnis <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Folder <strong>%s</strong> removed.";
#elif L==4	// es
	"Carpeta <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"R&eacute;pertoire <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Carpeta <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Cartella <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Folder <strong>%s</strong> usuniety.";
#elif L==9	// pt
	"Diret&oacute;rio <strong>%s</strong> removido.";
#endif

const char *Txt_Folders =
#if   L==1	// ca
	"Carpetes";
#elif L==2	// de
	"Verzeichnisse";
#elif L==3	// en
	"Folders";
#elif L==4	// es
	"Carpetas";
#elif L==5	// fr
	"R&eacute;pertoires";
#elif L==6	// gn
	"Carpetas";	// Okoteve traduccin
#elif L==7	// it
	"Cartelle";
#elif L==8	// pl
	"Foldery";
#elif L==9	// pt
	"Diret&oacute;rios";
#endif

const char *Txt_folders =
#if   L==1	// ca
	"carpetes";
#elif L==2	// de
	"Verzeichnisse";
#elif L==3	// en
	"folders";
#elif L==4	// es
	"carpetas";
#elif L==5	// fr
	"r&eacute;pertoires";
#elif L==6	// gn
	"carpetas";	// Okoteve traduccin
#elif L==7	// it
	"cartelle";
#elif L==8	// pl
	"foldery";
#elif L==9	// pt
	"diret&oacute;rios";
#endif

const char *Txt_Folders_copied =
#if   L==1	// ca
	"Carpetes copiades";
#elif L==2	// de
	"Verzeichnisse kopiert";
#elif L==3	// en
	"Folders copied";
#elif L==4	// es
	"Carpetas copiadas";
#elif L==5	// fr
	"R&eacute;pertoires copi&eacute;s";
#elif L==6	// gn
	"Carpetas copiadas";	// Okoteve traduccin
#elif L==7	// it
	"Cartelle copiate";
#elif L==8	// pl
	"foldery skopiowane";
#elif L==9	// pt
	"Diret&oacute;rios copiados";
#endif

const char *Txt_Folders_renamed =
#if   L==1	// ca
	"Carpetes anomenades";
#elif L==2	// de
	"Verzeichnisse umbenannt";
#elif L==3	// en
	"Folders renamed";
#elif L==4	// es
	"Carpetas renombradas";
#elif L==5	// fr
	"R&eacute;pertoires renomm&eacute;s";
#elif L==6	// gn
	"Carpetas renombradas";	// Okoteve traduccin
#elif L==7	// it
	"Cartelle rinominate";
#elif L==8	// pl
	"Foldery zmienione";
#elif L==9	// pt
	"Diret&oacute;rios renomeados";
#endif

const char *Txt_Folders_removed =
#if   L==1	// ca
	"Carpetes eliminades";
#elif L==2	// de
	"Verzeichnisse entfernt";
#elif L==3	// en
	"Folders removed";
#elif L==4	// es
	"Carpetas eliminadas";
#elif L==5	// fr
	"R&eacute;pertoires supprim&eacute;s";
#elif L==6	// gn
	"Carpetas eliminadas";	// Okoteve traduccin
#elif L==7	// it
	"Cartelle rimossi";
#elif L==8	// pl
	"Foldery usuni&eogon;te";
#elif L==9	// pt
	"Diret&oacute;rios removidos";
#endif

const char *Txt_Folders_not_renamed =
#if   L==1	// ca
	"Carpetes no anomenades";
#elif L==2	// de
	"Verzeichnisse nicht umbenannt";
#elif L==3	// en
	"Folders not renamed";
#elif L==4	// es
	"Carpetas no renombradas";
#elif L==5	// fr
	"R&eacute;pertoires pas renomm&eacute;s";
#elif L==6	// gn
	"Carpetas no renombradas";	// Okoteve traduccin
#elif L==7	// it
	"Cartelle non rinominate";
#elif L==8	// pl
	"Foldery nie zmienione";
#elif L==9	// pt
	"Diret&oacute;rios n&atilde;o renomeados";
#endif

const char *Txt_Follow =
#if   L==1	// ca
	"Seguir";
#elif L==2	// de
	"Folgen";
#elif L==3	// en
	"Follow";
#elif L==4	// es
	"Seguir";
#elif L==5	// fr
	"Suivre";
#elif L==6	// gn
	"Seguir";	// Okoteve traduccin
#elif L==7	// it
	"Seguire";
#elif L==8	// pl
	"&Sacute;led&zacute;";
#elif L==9	// pt
	"Seguir";
#endif

const char *Txt_FollowPerFollow[2] =
      {
#if   L==1	// ca
	"Mitjana de seguits per cada seguidor"
#elif L==2	// de
	"Durchschnittliche gefolgt pro anh&auml;nger"
#elif L==3	// en
	"Average followed per follower"
#elif L==4	// es
	"Promedio de seguidos por cada seguidor"
#elif L==5	// fr
	"Moyenne de suivis par suiveur"
#elif L==6	// gn
	"Promedio de seguidos por cada seguidor"	// Okoteve traduccin
#elif L==7	// it
	"Media di seguiti per seguace"
#elif L==8	// pl
        "&Sacute;rednia liczba obserwowani za obserwuj&aogon;"
#elif L==9	// pt
	 "M&eacute;dia por seguidos por seguidor"
#endif
      ,
#if   L==1	// ca
	"Mitjana de seguidors per cada seguit"
#elif L==2	// de
	"Durchschnittliche anh&auml;nger pro gefolgt"
#elif L==3	// en
	"Average followers per followed"
#elif L==4	// es
	"Promedio de seguidores por cada seguido"
#elif L==5	// fr
	"Moyenne de suiveurs par suivie"
#elif L==6	// gn
	"Promedio de seguidores por cada seguido"	// Okoteve traduccin
#elif L==7	// it
	"Media di seguace per seguito"
#elif L==8	// pl
        "&Sacute;rednia liczba obserwuj&aogon; za obserwowani"
#elif L==9	// pt
	"M&eacute;dia por seguidores por seguido"
#endif
      };

const char *Txt_Followed =
#if   L==1	// ca
	"Seguits";
#elif L==2	// de
	"Gefolgt";
#elif L==3	// en
	"Followed";
#elif L==4	// es
	"Seguidos";
#elif L==5	// fr
	"Suivis";
#elif L==6	// gn
	"Seguidos";	// Okoteve traduccin
#elif L==7	// it
	"Seguiti";
#elif L==8	// pl
	"Obserwowani";
#elif L==9	// pt
	"Seguidos";
#endif

const char *Txt_Followers =
#if   L==1	// ca
	"Seguidors";
#elif L==2	// de
	"Anh&auml;nger";
#elif L==3	// en
	"Followers";
#elif L==4	// es
	"Seguidores";
#elif L==5	// fr
	"Suiveurs";
#elif L==6	// gn
	"Seguidores";	// Okoteve traduccin
#elif L==7	// it
	"Seguaci";
#elif L==8	// pl
	"Obserwuj&aogon;";
#elif L==9	// pt
	"Seguidores";
#endif

const char *Txt_Following =
#if   L==1	// ca
	"Seg&uuml;ent";
#elif L==2	// de
	"Folgende";
#elif L==3	// en
	"Following";
#elif L==4	// es
	"Siguiendo";
#elif L==5	// fr
	"Suivant";
#elif L==6	// gn
	"Siguiendo";	// Okoteve traduccin
#elif L==7	// it
	"Seguendo";
#elif L==8	// pl
	"Nast&eogon;puj&aogon;cy";
#elif L==9	// pt
	"Seguinte";
#endif

const char *Txt_Following_unfollow =
#if   L==1	// ca
	"Seg&uuml;ent, deixar de seguir";
#elif L==2	// de
	"Folgende, entfolgen";
#elif L==3	// en
	"Following, unfollow";
#elif L==4	// es
	"Siguiendo, dejar de seguir";
#elif L==5	// fr
	"Suivant, se d&eacute;sabonner";
#elif L==6	// gn
	"Siguiendo, dejar de seguir";	// Okoteve traduccin
#elif L==7	// it
	"Seguendo, smetti";
#elif L==8	// pl
	"Nast&eogon;puj&aogon;cy, nie obserwuj";
#elif L==9	// pt
	"Seguinte, deixar de seguir";
#endif

const char *Txt_FOLLOWS_YOU =
#if   L==1	// ca
	"ET SEGUEIX";
#elif L==2	// de
	"FOLGT DIR";
#elif L==3	// en
	"FOLLOWS YOU";
#elif L==4	// es
	"TE SIGUE";
#elif L==5	// fr
	"VOUS SUIT";
#elif L==6	// gn
	"TE SIGUE";	// Okoteve traduccin
#elif L==7	// it
	"TI SEGUE";
#elif L==8	// pl
	"OBSERWUJE CI&Eogon;";
#elif L==9	// pt
	"SEGUE VOC&Ecirc;";
#endif

const char *Txt_For_security_enter_your_password =
#if   L==1	// ca
	"Per a m&eacute;s seguretat, introdueixi la seva contrasenya";
#elif L==2	// de
	"Aus Sicherheitsgr&uuml;nden geben Sie Ihr Passwort";
#elif L==3	// en
	"For security, enter your password";
#elif L==4	// es
	"Para mayor seguridad, introduzca su contrase&ntilde;a";
#elif L==5	// fr
	"Pour plus de s&eacute;curit&eacute;, entrez votre mot de passe";
#elif L==6	// gn
	"Para mayor seguridad, introduzca su contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Per motivi di sicurezza, immettere la password";
#elif L==8	// pl
	"Dla bezpiecze&nacute;stwa, wprowad&zacute; has&lstrok;o";
#elif L==9	// pt
	"Por seguran&ccedil;a, digite a sua senha";
#endif

const char *Txt_Force_students_to_read_this_information =
#if   L==1	// ca
	"Obligar els estudiants a llegir aquesta informaci&oacute;";
#elif L==2	// de
	"Erzwingen Studenten lesen diese Informationen";
#elif L==3	// en
	"Force students to read this information";
#elif L==4	// es
	"Obligar a los estudiantes a leer esta informaci&oacute;n";
#elif L==5	// fr
	"Forcer les &eacute;tudiants &agrave; lire cette information";
#elif L==6	// gn
	"Obligar a los estudiantes a leer esta informaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Obbligare gli studenti a leggere questa informazione";
#elif L==8	// pl
	"Studenci maja przeczytac ta informacje";
#elif L==9	// pt
	"For&ccedil;ar os estudantes a ler esta informa&ccedil;&atilde;o";
#endif

const char *Txt_Forgotten_password =
#if   L==1	// ca
	"Contrasenya oblidada?";
#elif L==2	// de
	"Passwort vergessen?";
#elif L==3	// en
	"Forgotten password?";
#elif L==4	// es
	"&iquest;Contrasea olvidada?";
#elif L==5	// fr
	"Mot de passe oubli&eacute;?";
#elif L==6	// gn
	"&iquest;Contrasea olvidada?";		// Okoteve traduccin
#elif L==7	// it
	"Password dimenticata?";
#elif L==8	// pl
	"Zapomniane has&lstrok;o?";
#elif L==9	// pt
	"Senha esquecida?";
#endif

const char *Txt_Format =
#if   L==1	// ca
	"Format";
#elif L==2	// de
	"Format";
#elif L==3	// en
	"Format";
#elif L==4	// es
	"Formato";
#elif L==5	// fr
	"Format";
#elif L==6	// gn
	"Formato";		// Okoteve traduccin
#elif L==7	// it
	"Formato";
#elif L==8	// pl
	"Format";
#elif L==9	// pt
	"Formato";
#endif

const char *Txt_Forum =
#if   L==1	// ca
	"Foro";		// Necessita traduccio
#elif L==2	// de
	"Forum";
#elif L==3	// en
	"Forum";
#elif L==4	// es
	"Foro";
#elif L==5	// fr
	"Forum";
#elif L==6	// gn
	"Foro";		// Okoteve traduccin
#elif L==7	// it
	"Forum";
#elif L==8	// pl
	"Forum";
#elif L==9	// pt
	"F&oacute;rum";
#endif

const char *Txt_Forum_posts =
#if   L==1	// ca
	"Comentarios en foros";	// Necessita traduccio
#elif L==2	// de
	"Forum-Beitr&auml;ge";
#elif L==3	// en
	"Forum posts";
#elif L==4	// es
	"Comentarios en foros";
#elif L==5	// fr
	"Posts de forum";
#elif L==6	// gn
	"Comentarios en foros";	// Okoteve traduccin
#elif L==7	// it
	"Post del forum";
#elif L==8	// pl
	"Forum postow";
#elif L==9	// pt
	"Posts de f&oacute;rum";
#endif

const char *Txt_FORUM_post =
#if   L==1	// ca
	"comentari";
#elif L==2	// de
	"Post";
#elif L==3	// en
	"post";
#elif L==4	// es
	"comentario";
#elif L==5	// fr
	"post";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"post";
#elif L==8	// pl
	"post";
#elif L==9	// pt
	"post";
#endif

const char *Txt_FORUM_posts =
#if   L==1	// ca
	"comentaris";
#elif L==2	// de
	"Nachrichten";
#elif L==3	// en
	"posts";
#elif L==4	// es
	"comentarios";
#elif L==5	// fr
	"posts";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"post";
#elif L==8	// pl
	"posty";
#elif L==9	// pt
	"posts";
#endif

const char *Txt_FORUM_Post_X_allowed =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Comentario %u permitido.";	// Necessita traduccio
#elif L==2	// de
	"Post %u allowed.";		// Need bersetzung
#elif L==3	// en
	"Post %u allowed.";
#elif L==4	// es
	"Comentario %u permitido.";
#elif L==5	// fr
	"Post %u allowed.";		// Besoin de traduction
#elif L==6	// gn
	"Comentario %u permitido.";	// Okoteve traduccin
#elif L==7	// it
	"Post %u autorizzato.";
#elif L==8	// pl
	"Post %u dozwolone.";
#elif L==9	// pt
	"Post %u permitido.";
#endif

const char *Txt_FORUM_Post_X_allowed_Click_to_ban_it =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Comentario %u permitido. Pulse para censurarlo.";	// Necessita traduccio
#elif L==2	// de
	"Post %u allowed. Click to ban it.";			// Need bersetzung
#elif L==3	// en
	"Post %u allowed. Click to ban it.";
#elif L==4	// es
	"Comentario %u permitido. Pulse para censurarlo.";
#elif L==5	// fr
	"Post %u allowed. Click to ban it.";			// Besoin de traduction
#elif L==6	// gn
	"Comentario %u permitido. Pulse para censurarlo.";	// Okoteve traduccin
#elif L==7	// it
	"Post %u autorizzato. Clicca per censurarlo.";
#elif L==8	// pl
	"Post% u dozwolone. Kliknij, aby to zakazac.";
#elif L==9	// pt
	"Post %u permitido. Clique para censur&aacute;-lo.";
#endif

const char *Txt_FORUM_Post_and_thread_removed =
#if   L==1	// ca
	"Comentario y discusi&oacute;n eliminados.";	// Necessita traduccio
#elif L==2	// de
	"Nachricht und Faden entfernt.";
#elif L==3	// en
	"Post and thread removed.";
#elif L==4	// es
	"Comentario y discusi&oacute;n eliminados.";
#elif L==5	// fr
	"Post and thread removed.";			// Besoin de traduction
#elif L==6	// gn
	"Comentario y discusi&oacute;n eliminados.";	// Okoteve traduccin
#elif L==7	// it
	"Post e discussione rimossi.";
#elif L==8	// pl
	"Post watku usuniety.";
#elif L==9	// pt
	"Post e discuss&atilde;o removidos.";
#endif

const char *Txt_FORUM_Post_banned =
#if   L==1	// ca
	"Comentario no permitido";	// Necessita traduccio
#elif L==2	// de
	"Post banned.";			// Need bersetzung
#elif L==3	// en
	"Post banned.";
#elif L==4	// es
	"Comentario no permitido";
#elif L==5	// fr
	"Post banned.";			// Besoin de traduction
#elif L==6	// gn
	"Comentario no permitido";	// Okoteve traduccin
#elif L==7	// it
	"Post censurato.";
#elif L==8	// pl
	"Post zakazany.";
#elif L==9	// pt
	"Post n&atilde;o permitido.";
#endif

const char *Txt_FORUM_Post_X_banned =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Comentario %u no permitido";	// Necessita traduccio
#elif L==2	// de
	"Post %u banned.";		// Need bersetzung
#elif L==3	// en
	"Post %u banned.";
#elif L==4	// es
	"Comentario %u no permitido";
#elif L==5	// fr
	"Post %u banned.";		// Besoin de traduction
#elif L==6	// gn
	"Comentario %u no permitido";	// Okoteve traduccin
#elif L==7	// it
	"Post %u censurato.";
#elif L==8	// pl
	"Post %u zakazany.";
#elif L==9	// pt
	"Post %u n&atilde;o permitido.";
#endif

const char *Txt_FORUM_Post_X_banned_Click_to_unban_it =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Comentario %u no permitido. Pulse para permitirlo.";	// Necessita traduccio
#elif L==2	// de
	"Post %u banned. Click to unban it.";			// Need bersetzung
#elif L==3	// en
	"Post %u banned. Click to unban it.";
#elif L==4	// es
	"Comentario %u no permitido. Pulse para permitirlo.";
#elif L==5	// fr
	"Post %u banned. Click to unban it.";			// Besoin de traduction
#elif L==6	// gn
	"Comentario %u no permitido. Pulse para permitirlo.";	// Okoteve traduccin
#elif L==7	// it
	"Post %u censurato. Clicca per autorizzarlo.";
#elif L==8	// pl
	"Post %u zakazany.Kliknij, aby to odbanowac.";
#elif L==9	// pt
	"Post %u n&atilde;o permitido. Clique para permiti-lo.";
#endif

const char *Txt_FORUM_Post_sent =
#if   L==1	// ca
	"Comentario enviado.";	// Necessita traduccio
#elif L==2	// de
	"Post sent.";		// Need bersetzung
#elif L==3	// en
	"Post sent.";
#elif L==4	// es
	"Comentario enviado.";
#elif L==5	// fr
	"Post sent.";		// Besoin de traduction
#elif L==6	// gn
	"Comentario enviado.";	// Okoteve traduccin
#elif L==7	// it
	"Post inviato.";
#elif L==8	// pl
	"Post wyslany.";
#elif L==9	// pt
	"Post enviado.";
#endif

const char *Txt_FORUM_Post_removed =
#if   L==1	// ca
	"Comentari eliminat.";
#elif L==2	// de
	"Nachricht entfernt.";
#elif L==3	// en
	"Post removed.";
#elif L==4	// es
	"Comentario eliminado.";
#elif L==5	// fr
	"Message supprim&eacute;.";
#elif L==6	// gn
	"Comentario eliminado.";		// Okoteve traduccin
#elif L==7	// it
	"Post rimosso.";
#elif L==8	// pl
	"Post usuniety.";
#elif L==9	// pt
	"Post removido.";
#endif

const char *Txt_FORUM_Post_unbanned =
#if   L==1	// ca
	"Comentario permitido";		// Necessita traduccio
#elif L==2	// de
	"Post unbanned.";		// Need bersetzung
#elif L==3	// en
	"Post unbanned";
#elif L==4	// es
	"Comentario permitido";
#elif L==5	// fr
	"Post unbanned.";		// Besoin de traduction
#elif L==6	// gn
	"Comentario permitido";		// Okoteve traduccin
#elif L==7	// it
	"Post autorizzato.";
#elif L==8	// pl
	"Post odbanowany";
#elif L==9	// pt
	"Post permitido.";
#endif

const char *Txt_Forum_threads =
#if   L==1	// ca
	"Discusiones de foro";	// Necessita traduccio
#elif L==2	// de
	"Forum-Threads";
#elif L==3	// en
	"Forum threads";
#elif L==4	// es
	"Discusiones de foro";
#elif L==5	// fr
	"Fils de forum";
#elif L==6	// gn
	"Discusiones de foro";	// Okoteve traduccin
#elif L==7	// it
	"Discussione del forum";
#elif L==8	// pl
	"Watki forum";
#elif L==9	// pt
	"Discuss&otilde;es de f&oacute;rum";
#endif

const char *Txt_FORUM_THREAD_HELP_ORDER[For_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Ordenar por fecha del primer mensaje"	// Necessita traduccio
#elif L==2	// de
	 "Nach Datum der Nachrichten sortieren (aufsteigend)"
#elif L==3	// en
	 "Sort by date of first message"
#elif L==4	// es
	 "Ordenar por fecha del primer mensaje"
#elif L==5	// fr
	 "Trier par date du premier message"
#elif L==6	// gn
	 "Ordenar por fecha del primer mensaje"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per data del primo messaggio"
#elif L==8	// pl
	 "Sortuj wedlug daty pierwszej wiadomosci"
#elif L==9	// pt
	 "Classificar por data da primeira mensagem"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar por fecha del &uacute;ltimo mensaje"	// Necessita traduccio
#elif L==2	// de
	 "Nach Datum der Nachrichten sortieren (absteigend)"
#elif L==3	// en
	 "Sort by date of the last message"
#elif L==4	// es
	 "Ordenar por fecha del &uacute;ltimo mensaje"
#elif L==5	// fr
	 "Trier par date du dernier message"
#elif L==6	// gn
	 "Ordenar por fecha del &uacute;ltimo mensaje"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per data dell'ultimo messaggio"
#elif L==8	// pl
	 "Sortuj wedlug daty ostatniego komunikatu"
#elif L==9	// pt
	 "Classificar pela data da &uacute;ltima mensagem"
#endif
	};

const char *Txt_FORUM_THREAD_ORDER[For_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Primer comentari"
#elif L==2	// de
	 "Erste Post"
#elif L==3	// en
	 "First post"
#elif L==4	// es
	 "Primer comentario"
#elif L==5	// fr
	 "Premier post"
#elif L==6	// gn
	 "Primer comentario"	// Okoteve traduccin
#elif L==7	// it
	 "Primo post"
#elif L==8	// pl
	 "Pierwsza post"
#elif L==9	// pt
	 "Primeira post"
#endif
	 ,
#if   L==1	// ca
	 "&Uacute;ltim comentari"
#elif L==2	// de
	 "Letzte Post"
#elif L==3	// en
	 "Last post"
#elif L==4	// es
	 "&Uacute;ltimo comentario"
#elif L==5	// fr
	 "Dernier post"
#elif L==6	// gn
	 "&Uacute;ltimo comentario"	// Okoteve traduccin
#elif L==7	// it
	 "Ultimo post"
#elif L==8	// pl
	 "Ostatnia post"
#elif L==9	// pt
	 "&Uacute;ltima post"
#endif
	};

const char *Txt_FORUM_WHICH_FORUM[For_NUM_FORUM_SETS] =
   {
	 // For_ONLY_CURRENT_FORUMS
#if   L==1	// ca
	 "F&ograve;rums d'aquest lloc"
#elif L==2	// de
	 "Foren zu diesem Standort"
#elif L==3	// en
	 "Forums of this location"
#elif L==4	// es
	 "Foros de este sitio"
#elif L==5	// fr
	 "Forums de cet emplacement"
#elif L==6	// gn
	 "Foros de este sitio"	// Okoteve traduccin
#elif L==7	// it
	 "Forum di questo sito"
#elif L==8	// pl
	 "Forum tej lokalizacji"
#elif L==9	// pt
	 "F&oacute;runs deste site"
#endif
	 ,
	 // For_ALL_MY_FORUMS
#if   L==1	// ca
	 "Tots els meus f&ograve;rums"
#elif L==2	// de
	 "Alle meine Foren"
#elif L==3	// en
	 "All my forums"
#elif L==4	// es
	 "Todos mis foros"
#elif L==5	// fr
	 "Tous mes forums"
#elif L==6	// gn
	 "Todos mis foros"	// Okoteve traduccin
#elif L==7	// it
	 "Tutti i miei forum"
#elif L==8	// pl
	 "Wszystkie moje forum"
#elif L==9	// pt
	 "Todos os meus f&oacute;runs"
#endif
	};

const char *Txt_Forums =
#if   L==1	// ca
	"F&ograve;rums";
#elif L==2	// de
	"Foren";
#elif L==3	// en
	"Forums";
#elif L==4	// es
	"Foros";
#elif L==5	// fr
	"Forums";
#elif L==6	// gn
	"Foros";	// Okoteve traduccin
#elif L==7	// it
	"Forum";
#elif L==8	// pl
	"Forum";
#elif L==9	// pt
	"F&oacute;runs";
#endif

const char *Txt_Found_an_index_htm_file =
#if   L==1	// ca
	"Se ha encontrado un archivo <em>index.htm</em>.";	// Necessita traduccio
#elif L==2	// de
	"Found an <em>index.htm</em> file.";			// Need bersetzung
#elif L==3	// en
	"Found an <em>index.htm</em> file.";
#elif L==4	// es
	"Se ha encontrado un archivo <em>index.htm</em>.";
#elif L==5	// fr
	"Found an <em>index.htm</em> file.";			// Besoin de traduction
#elif L==6	// gn
	"Se ha encontrado un archivo <em>index.htm</em>.";	// Okoteve traduccin
#elif L==7	// it
	"Si &egrave; trovato un file <em>index.htm</em>.";
#elif L==8	// pl
	"Znaleziono <em>index.htm</em> plik.";
#elif L==9	// pt
	"Um arquivo <em>index.htm</em> foi encontrado.";
#endif

const char *Txt_Found_an_index_html_file =
#if   L==1	// ca
	"Se ha encontrado un archivo <em>index.html</em>.";	// Necessita traduccio
#elif L==2	// de
	"Found an <em>index.html</em> file.";			// Need bersetzung
#elif L==3	// en
	"Found an <em>index.html</em> file.";
#elif L==4	// es
	"Se ha encontrado un archivo <em>index.html</em>.";
#elif L==5	// fr
	"Found an <em>index.html</em> file.";			// Besoin de traduction
#elif L==6	// gn
	"Se ha encontrado un archivo <em>index.html</em>.";	// Okoteve traduccin
#elif L==7	// it
	"Si &egrave; trovato un file <em>index.html</em>.";
#elif L==8	// pl
	"Znaleziono <em>index.html</em> plik.";
#elif L==9	// pt
	"Um arquivo <em>index.html</em> foi encontrado.";
#endif

const char *Txt_Frequent_ACTIONS =	// Check that this sentence fits within the allocated space
#if   L==1	// ca
	"Freq&uuml;ents";
#elif L==2	// de
	"H&auml;ufige";
#elif L==3	// en
	"Frequent";
#elif L==4	// es
	"Frecuentes";
#elif L==5	// fr
	"Frequent";
#elif L==6	// gn
	"Frecuentes";	// Okoteve traduccin
#elif L==7	// it
	"Frequenti";
#elif L==8	// pl
	"Cz&eogon;ste";
#elif L==9	// pt
	"Freq&uuml;entes";
#endif

const char *Txt_from =
#if   L==1	// ca
	"de";	// Necessita traduccio
#elif L==2	// de
	"von";
#elif L==3	// en
	"from";
#elif L==4	// es
	"de";
#elif L==5	// fr
	"de";
#elif L==6	// gn
	"de";	// Okoteve traduccin
#elif L==7	// it
	"da";
#elif L==8	// pl
	"z";
#elif L==9	// pt
	"de";
#endif

const char *Txt_From_now_on_the_number_of_editing_lines_of_the_field_X_is_Y =	// Warning: it is very important to include %s and %u (in this order) in the following sentences
#if   L==1	// ca
	"A partir de ahora"
	" el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong>"
	" es <strong>%u</strong>.";	// Necessita traduccio
#elif L==2	// de
	"From now on"
	" the number of editing lines of the field <strong>%s</strong>"
	" is <strong>%u</strong>.";	// Need bersetzung
#elif L==3	// en
	"From now on"
	" the number of editing lines of the field <strong>%s</strong>"
	" is <strong>%u</strong>.";
#elif L==4	// es
	"A partir de ahora"
	" el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong>"
	" es <strong>%u</strong>.";
#elif L==5	// fr
	"From now on"
	" the number of editing lines of the field <strong>%s</strong>"
	" is <strong>%u</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"A partir de ahora"
	" el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong>"
	" es <strong>%u</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"A partire da ora"
	" il numero di linee editate <strong>%s</strong>"
	" &egrave; <strong>%u</strong>.";
#elif L==8	// pl
	"Od tej pory"
	" liczba linii w polu edycji <strong>%s</strong>"
	" jest <strong>%u</strong>.";
#elif L==9	// pt
	"A partir de agora"
	" o n&uacute;mero de linhas de edi&ccedil;&atilde;o no campo <strong>%s</strong>"
	" &eacute; <strong>%u</strong>.";
#endif

const char *Txt_from_this_course =
#if   L==1	// ca
	"desde esta asignatura";	// Necessita traduccio
#elif L==2	// de
	"von dieser Kurs";
#elif L==3	// en
	"from this course";
#elif L==4	// es
	"desde esta asignatura";
#elif L==5	// fr
	"de cette mati&egrave;re";
#elif L==6	// gn
	"desde esta asignatura";	// Okoteve traduccin
#elif L==7	// it
	"da questo corso";
#elif L==8	// pl
	"z tego kursu";
#elif L==9	// pt
	"desta disciplina";
#endif

const char *Txt_From_this_time_you_can_receive_messages_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"A partir de este momento, puede recibir mensajes de %s.";	// Necessita traduccio
#elif L==2	// de
	"From this time, you can receive messages from %s.";		// Need bersetzung
#elif L==3	// en
	"From this time, you can receive messages from %s.";
#elif L==4	// es
	"A partir de este momento, puede recibir mensajes de %s.";
#elif L==5	// fr
	"From this time, you can receive messages from %s.";		// Besoin de traduction
#elif L==6	// gn
	"A partir de este momento, puede recibir mensajes de %s.";	// Okoteve traduccin
#elif L==7	// it
	"From this time, you can receive messages from %s.";		// Bisogno di traduzione
#elif L==8	// pl
	"From this time, you can receive messages from %s.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A partir deste momento, voc&ecirc; pode receber mensagens de %s.";
#endif

const char *Txt_From_this_time_you_will_not_receive_messages_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"A partir de este momento, no recibir&aacute; mensajes de %s.";	// Necessita traduccio
#elif L==2	// de
	"From this time, you will not receive messages from %s.";	// Need bersetzung
#elif L==3	// en
	"From this time, you will not receive messages from %s.";
#elif L==4	// es
	"A partir de este momento, no recibir&aacute; mensajes de %s.";
#elif L==5	// fr
	"From this time, you will not receive messages from %s.";	// Besoin de traduction
#elif L==6	// gn
	"A partir de este momento, no recibir&aacute; mensajes de %s.";	// Okoteve traduccin
#elif L==7	// it
	"From this time, you will not receive messages from %s.";	// Bisogno di traduzione
#elif L==8	// pl
	"From this time, you will not receive messages from %s.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A partir deste momento, voc&ecirc; n&atilde;o receber&aacute; mensagens de %s.";
#endif

const char *Txt_Full_name =
#if   L==1	// ca
	"Nom complet";
#elif L==2	// de
	"Vollst&auml;ndiger Name";
#elif L==3	// en
	"Full name";
#elif L==4	// es
	"Nombre completo";
#elif L==5	// fr
	"Nom complet";
#elif L==6	// gn
	"Nombre completo";	// Okoteve traduccin
#elif L==7	// it
	"Nome completo";
#elif L==8	// pl
	"Pe&lstrok;na nazwa";
#elif L==9	// pt
	"Nome completo";
#endif

const char *Txt_Full_name_of_the_degree =
#if   L==1	// ca
	"Nom complet de la titulaci&oacute;";
#elif L==2	// de
	"Vollst&auml;ndiger Name der Studiengang";
#elif L==3	// en
	"Full name of the degree";
#elif L==4	// es
	"Nombre completo de la titulaci&oacute;n";
#elif L==5	// fr
	"Nom complet du &eacute;tude";
#elif L==6	// gn
	"Nombre completo de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nome completo della laurea";
#elif L==8	// pl
	"Pe&lstrok;na nazwa stopie&nacute;";
#elif L==9	// pt
	"Nome completo do grau";
#endif

const char *Txt_Full_name_of_the_centre =
#if   L==1	// ca
	"Nom complet del centre";
#elif L==2	// de
	"Vollst&auml;ndiger Name der Lehrinstitut";
#elif L==3	// en
	"Full name of the centre";
#elif L==4	// es
	"Nombre completo del centro";
#elif L==5	// fr
	"Nom complet du centre";
#elif L==6	// gn
	"Nombre completo del centro";	// Okoteve traduccin
#elif L==7	// it
	"Nome completo del centro";
#elif L==8	// pl
	"Pe&lstrok;na nazwa centrum";
#elif L==9	// pt
	"Nome completo do centro";
#endif

const char *Txt_Full_name_of_the_course =
#if   L==1	// ca
	"Nom complet de l'assignatura";
#elif L==2	// de
	"Vollst&auml;ndiger Name der Kurs";
#elif L==3	// en
	"Full name of the course";
#elif L==4	// es
	"Nombre completo de la asignatura";
#elif L==5	// fr
	"Nom complet de la mati&egrave;re";
#elif L==6	// gn
	"Nombre completo de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Nome completo del corso";
#elif L==8	// pl
	"Pe&lstrok;na nazwa kurs";
#elif L==9	// pt
	"Nome completo da disciplina";
#endif

const char *Txt_Full_name_of_the_institution =
#if   L==1	// ca
	"Nom complet de la instituci&oacute;";
#elif L==2	// de
	"Vollst&auml;ndiger Name der Institution";
#elif L==3	// en
	"Full name of the institution";
#elif L==4	// es
	"Nombre completo de la instituci&oacute;n";
#elif L==5	// fr
	"Nom complet de l'institution";
#elif L==6	// gn
	"Nombre completo de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nome completo dell'istituzione";
#elif L==8	// pl
	"Pe&lstrok;na nazwa instytucji";
#elif L==9	// pt
	"Nome completo da institui&ccedil;&atilde;o";
#endif

const char *Txt_Function =
#if   L==1	// ca
	"Funci&oacute;";
#elif L==2	// de
	"Funktion";
#elif L==3	// en
	"Function";
#elif L==4	// es
	"Funci&oacute;n";
#elif L==5	// fr
	"Fonction";
#elif L==6	// gn
	"Funci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Funzione";
#elif L==8	// pl
	"Funkcja";
#elif L==9	// pt
	"Fun&ccedil;&atilde;o";
#endif

const char *Txt_Game =
#if   L==1	// ca
	"Joc";
#elif L==2	// de
	"Spiel";
#elif L==3	// en
	"Game";
#elif L==4	// es
	"Juego";
#elif L==5	// fr
	"Jeu";
#elif L==6	// gn
	"Juego";	// Okoteve traduccin
#elif L==7	// it
	"Gioco";
#elif L==8	// pl
	"Gra";
#elif L==9	// pt
	"Jogo";
#endif

const char *Txt_Game_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El joc <strong>%s</strong> ara est&agrave; oculto.";
#elif L==2	// de
	"Das Spiel <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==3	// en
	"Game <strong>%s</strong> is now hidden.";
#elif L==4	// es
	"El juego <strong>%s</strong> ahora est&aacute; oculto.";
#elif L==5	// fr
	"Le jeu <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==6	// gn
	"El juego <strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==7	// it
	"Il gioco <strong>%s</strong> &egrave; ora nascosto.";
#elif L==8	// pl
	"Gra <strong>%s</strong> jest teraz ukryta.";
#elif L==9	// pt
	"O jogo <strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_Game_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El joc <strong>%s</strong> ara est&agrave; visible.";
#elif L==2	// de
	"Das Spiel <strong>%s</strong> ist jetzt sichtbar.";
#elif L==3	// en
	"Game <strong>%s</strong> is now visible.";
#elif L==4	// es
	"El juego <strong>%s</strong> ahora est&aacute; visible.";
#elif L==5	// fr
	"Le jeu <strong>%s</strong> est maintenant visible.";
#elif L==6	// gn
	"El juego <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7	// it
	"Il gioco <strong>%s</strong> &egrave; ora visibile.";
#elif L==8	// pl
	"Badanie <strong>%s</strong> jest teraz widoczne.";
#elif L==9	// pt
	"O jogo <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_Game_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Joc <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Spiel <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Game <strong>%s</strong> removed.";
#elif L==4	// es
	"Juego <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Jeu <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Juego <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Gioco <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Gra <strong>%s</strong> usuni&eogon;te.";
#elif L==9	// pt
	"Jogo <strong>%s</strong> removido.";
#endif

const char *Txt_Game_X_reset =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Joc <strong>%s</strong> posat a zero..";
#elif L==2	// de
	"Spiel <strong>%s</strong> zur&uuml;ckgesetzt.";
#elif L==3	// en
	"Game <strong>%s</strong> reset.";
#elif L==4	// es
	"Juego <strong>%s</strong> puesto a cero.";
#elif L==5	// fr
	"Jeu <strong>%s</strong> remis &agrave; z&eacute;ro.";
#elif L==6	// gn
	"Juego <strong>%s</strong> puesto a cero.";	// Okoteve traduccin
#elif L==7	// it
	"Gioco <strong>%s</strong> resettato.";
#elif L==8	// pl
	"Gra <strong>%s</strong> zresetowane.";
#elif L==9	// pt
	"Jogo <strong>%s</strong> fixado em zero.";
#endif

const char *Txt_GAME_You_belong_to_the_scope_of_the_game =
#if   L==1	// ca
	"Pertany a l'&agrave;mbit del joc";
#elif L==2	// de
	"Sie geh&ouml;ren zum Umfang des Spiels";
#elif L==3	// en
	"You belong to the scope of the game";
#elif L==4	// es
	"Pertenece al &aacute;mbito del juego";
#elif L==5	// fr
	"Vous appartenez &agrave; la port&eacute;e du jeu";
#elif L==6	// gn
	"Pertenece al &aacute;mbito del juego";		// Okoteve traduccin
#elif L==7	// it
	"Voi appartenete al campo del gioco";
#elif L==8	// pl
	"Nale&zdot;ysz do zakresu gry";
#elif L==9	// pt
	"Voc&ecirc; pertence ao &acirc;mbito do jogo";
#endif

const char *Txt_GAME_You_dont_belong_to_the_scope_of_the_game =
#if   L==1	// ca
	"No pertany a l'&agrave;mbit del joc";
#elif L==2	// de
	"Sie geh&ouml;ren nicht zum Umfang des Spiels";
#elif L==3	// en
	"You don't belong to the scope the game";
#elif L==4	// es
	"No pertenece al &aacute;mbito del juego";
#elif L==5	// fr
	"Vous n'appartenez pas &agrave; la port&eacute;e du jeu";
#elif L==6	// gn
	"No pertenece al &aacute;mbito del juego";	// Okoteve traduccin
#elif L==7	// it
	"Voi non appartieni al campo del gioco";
#elif L==8	// pl
	"Nie nale&zdot;&aogon; do zakresu gry";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pertence ao &acirc;mbito do jogo";
#endif

const char *Txt_Games =
#if   L==1	// ca
	"Jocs";
#elif L==2	// de
	"Spiele";
#elif L==3	// en
	"Games";
#elif L==4	// es
	"Juegos";
#elif L==5	// fr
	"Jeux";
#elif L==6	// gn
	"Juegos";	// Okoteve traduccin
#elif L==7	// it
	"Giochi";
#elif L==8	// pl
	"Gry";
#elif L==9	// pt
	"Jogos";
#endif

const char *Txt_General =
#if   L==1	// ca
	"General";
#elif L==2	// de
	"Allgemein";
#elif L==3	// en
	"General";
#elif L==4	// es
	"General";
#elif L==5	// fr
	"G&eacute;n&eacute;ral";
#elif L==6	// gn
	"General";	// Okoteve traduccin
#elif L==7	// it
	"Generale";
#elif L==8	// pl
	"Ogolne";
#elif L==9	// pt
	"Geral";
#endif

const char *Txt_general_topics =
#if   L==1	// ca
	"temas generales";	// Necessita traduccio
#elif L==2	// de
	"allgemeine Themen";
#elif L==3	// en
	"general topics";
#elif L==4	// es
	"temas generales";
#elif L==5	// fr
	"affaires g&eacute;n&eacute;rales";
#elif L==6	// gn
	"temas generales";	// Okoteve traduccin
#elif L==7	// it
	"temi generali";
#elif L==8	// pl
	"oglne tematy";
#elif L==9	// pt
	"temas gerais";
#endif

const char *Txt_Generate_report =
#if   L==1	// ca
	"Generar informe";
#elif L==2	// de
	"Generieren Nutzungsbericht";
#elif L==3	// en
	"Generate report";
#elif L==4	// es
	"Generar informe";
#elif L==5	// fr
	"G&eacute;n&eacute;rer le rapport";
#elif L==6	// gn
	"Generar informe";	// Okoteve traduccin
#elif L==7	// it
	"Crea rapporto";
#elif L==8	// pl
	"Wygeneruj raport";
#elif L==9	// pt
	"Gerar relat&oacute;rio";
#endif

const char *Txt_Generate_test =
#if   L==1	// ca
	"Generar test";
#elif L==2	// de
	"Generieren Test";
#elif L==3	// en
	"Generate test";
#elif L==4	// es
	"Generar test";
#elif L==5	// fr
	"G&eacute;n&eacute;rer test";
#elif L==6	// gn
	"Generar test";	// Okoteve traduccin
#elif L==7	// it
	"Crea test";
#elif L==8	// pl
	"Wygeneruj test";
#elif L==9	// pt
	"Gerar teste";
#endif

const char *Txt_Get_a_new_password =
#if   L==1	// ca
	"Rebre una nova contrasenya";
#elif L==2	// de
	"Neues Passwort beantragen";
#elif L==3	// en
	"Get a new password";
#elif L==4	// es
	"Recibir una nueva contrase&ntilde;a";
#elif L==5	// fr
	"Recevoir un nouveau mot de passe";
#elif L==6	// gn
	"Recibir una nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Ottenere una nuova password";
#elif L==8	// pl
	"Uzyskaj nowe has&lstrok;o";
#elif L==9	// pt
	"Obter uma nova senha";
#endif

const char *Txt_Global_visits =
#if   L==1	// ca
	"Accessos globals";
#elif L==2	// de
	"Globale Anmeldungen";
#elif L==3	// en
	"Global visits";
#elif L==4	// es
	"Accesos globales";
#elif L==5	// fr
	"Visites globales";
#elif L==6	// gn
	"Accesos globales";	// Okoteve traduccin
#elif L==7	// it
	"Accessi globali";
#elif L==8	// pl
	"Globalne wizyty";
#elif L==9	// pt
	"Visitas globais";
#endif

const char *Txt_Go_back =
#if   L==1	// ca
	"Tornar enrere";
#elif L==2	// de
	"Zur&uuml;ckgehen";
#elif L==3	// en
	"Go back";
#elif L==4	// es
	"Volver atr&aacute;s";
#elif L==5	// fr
	"Retour";
#elif L==6	// gn
	"Volver atr&aacute;s";	// Okoteve traduccin
#elif L==7	// it
	"Tornare";
#elif L==8	// pl
	"Wraca&cacute;";
#elif L==9	// pt
	"Voltar";
#endif

const char *Txt_Go_to_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ir a %s";	// Necessita traduccio
#elif L==2	// de
	"Gehe zum %s";
#elif L==3	// en
	"Go to %s";
#elif L==4	// es
	"Ir a %s";
#elif L==5	// fr
	"Aller &agrave; %s";
#elif L==6	// gn
	"Ir a %s";	// Okoteve traduccin
#elif L==7	// it
	"Vai a %s";
#elif L==8	// pl
	"Przejdz do %s";
#elif L==9	// pt
	"Ir para %s";
#endif

const char *Txt_Go_to_course_and_reply =
#if   L==1	// ca
	"Ir a asignatura y responder";		// Necessita traduccio
#elif L==2	// de
	"Kurs einblenden und antworten";
#elif L==3	// en
	"Go to course and reply";
#elif L==4	// es
	"Ir a asignatura y responder";
#elif L==5	// fr
	"Aller &agrave; la mati&egrave;re et r&eacute;pondre";
#elif L==6	// gn
	"Ir a asignatura y responder";		// Okoteve traduccin
#elif L==7	// it
	"Vai a corso e rispondere";
#elif L==8	// pl
	"Idz do kursu i odpowiedz";
#elif L==9	// pt
	"Ir para a disciplina e responder";
#endif

const char *Txt_Go_to_course_and_reply_again =
#if   L==1	// ca
	"Ir a asignatura y responder otra vez";		// Necessita traduccio
#elif L==2	// de
	"Kurs einblenden und erneut antworten";
#elif L==3	// en
	"Go to course and reply again";
#elif L==4	// es
	"Ir a asignatura y responder otra vez";
#elif L==5	// fr
	"Aller &agrave; la mati&egrave;re et r&eacute;pondre &agrave; nouveau";
#elif L==6	// gn
	"Ir a asignatura y responder otra vez";		// Okoteve traduccin
#elif L==7	// it
	"Vai a corso e rispondere nuovamente";
#elif L==8	// pl
	"Idz do kursu i odpowiedz ponownie";
#elif L==9	// pt
	"Ir para a disciplina e responder novamente";
#endif

const char *Txt_Go_to_one_of_my_courses =
#if   L==1	// ca
	"Anar a una de les meves assignatures";
#elif L==2	// de
	"Gehen Sie zu einem meiner Kurse";
#elif L==3	// en
	"Go to one of my courses";
#elif L==4	// es
	"Ir a una de mis asignaturas";
#elif L==5	// fr
	"Allez &agrave; l'une de mes mati&egrave;res";
#elif L==6	// gn
	"Ir a una de mis asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Vai a uno dei miei corsi";
#elif L==8	// pl
	"Id&zacute; do jednej z moich kurs&oacute;w";
#elif L==9	// pt
	"V&aacute; para uma das minha disciplinas";
#endif

const char *Txt_Group =
#if   L==1	// ca
	"Grup";
#elif L==2	// de
	"Gruppe";
#elif L==3	// en
	"Group";
#elif L==4	// es
	"Grupo";
#elif L==5	// fr
	"Groupe";
#elif L==6	// gn
	"Aty";
#elif L==7	// it
	"Gruppo";
#elif L==8	// pl
	"Grupa";
#elif L==9	// pt
	"Grupo";
#endif

const char *Txt_group =
#if   L==1	// ca
	"grup";
#elif L==2	// de
	"Gruppe";
#elif L==3	// en
	"group";
#elif L==4	// es
	"grupo";
#elif L==5	// fr
	"groupe";
#elif L==6	// gn
	"aty";
#elif L==7	// it
	"gruppo";
#elif L==8	// pl
	"grupa";
#elif L==9	// pt
	"grupo";
#endif

const char *Txt_Group_closed =
#if   L==1	// ca
	"Grup tancat";
#elif L==2	// de
	"Gruppe geschlossen";
#elif L==3	// en
	"Group closed";
#elif L==4	// es
	"Grupo cerrado";
#elif L==5	// fr
	"Groupe ferm&eacute;";
#elif L==6	// gn
	"Grupo cerrado";	// Okoteve traduccin
#elif L==7	// it
	"Gruppo chiuso";
#elif L==8	// pl
	"Grupa zamknieta";
#elif L==9	// pt
	"Grupo fechado";
#endif

const char *Txt_Group_X_closed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Grup %s tancat";
#elif L==2	// de
	"Gruppe %s geschlossen";
#elif L==3	// en
	"Group %s closed";
#elif L==4	// es
	"Grupo %s cerrado";
#elif L==5	// fr
	"Groupe %s ferm&eacute;";
#elif L==6	// gn
	"Grupo %s cerrado";		// Okoteve traduccin
#elif L==7	// it
	"Gruppo %s chiuso";
#elif L==8	// pl
	"Grupa %s zamknieta";
#elif L==9	// pt
	"Grupo %s fechado";
#endif

const char *Txt_Group_X_closed_click_to_open_it =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Grup %s tancat; feu clic per obrir-lo";
#elif L==2	// de
	"Gruppe %s geschlossen, klicke zum &Ouml;ffnen";
#elif L==3	// en
	"Group %s closed, click to open it";
#elif L==4	// es
	"Grupo %s cerrado; pulse para abrirlo";
#elif L==5	// fr
	"Groupe %s ferm&eacute;, cliquez pour l'ouvrir";
#elif L==6	// gn
	"Grupo %s cerrado; pulse para abrirlo";		// Okoteve traduccin
#elif L==7	// it
	"Gruppo %s chiuso, clicca per aprirlo";
#elif L==8	// pl
	"Grupa %s zamknieta, kliknij zeby otworzyc";
#elif L==9	// pt
	"Grupo %s fechado, clique para abri-lo";
#endif

const char *Txt_Group_name =
#if   L==1	// ca
	"Nombre del grupo";	// Necessita traduccio
#elif L==2	// de
	"Name der Gruppe";
#elif L==3	// en
	"Group name";
#elif L==4	// es
	"Nombre del grupo";
#elif L==5	// fr
	"Nom du groupe";
#elif L==6	// gn
	"Nombre del grupo";	// Okoteve traduccin
#elif L==7	// it
	"Nome del gruppo";
#elif L==8	// pl
	"Nazwa grupy";
#elif L==9	// pt
	"Nome do grupo";
#endif

const char *Txt_Group_X_open =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Grup %s obert";
#elif L==2	// de
	"Gruppe %s &ouml;ffnen";
#elif L==3	// en
	"Group %s open";
#elif L==4	// es
	"Grupo %s abierto";
#elif L==5	// fr
	"Groupe %s ouvert";
#elif L==6	// gn
	"Grupo %s abierto";		// Okoteve traduccin
#elif L==7	// it
	"Gruppo %s aperto";
#elif L==8	// pl
	"Grupa %s otwarta";
#elif L==9	// pt
	"Grupo %s aberto";
#endif

const char *Txt_Group_X_open_click_to_close_it =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Grup %s obert, feu clic per tancar-lo";
#elif L==2	// de
	"Gruppe %s &ouml;ffnen, klicken Sie, um es zu schlie&szlig;en";
#elif L==3	// en
	"Group %s open, click to close it";
#elif L==4	// es
	"Grupo %s abierto; pulse para cerrarlo";
#elif L==5	// fr
	"Groupe %s ouvert, cliquez pour le fermer";
#elif L==6	// gn
	"Grupo %s abierto; pulse para cerrarlo";	// Okoteve traduccin
#elif L==7	// it
	"Gruppo %s aperto, clicca per chiuderlo";
#elif L==8	// pl
	"Grupa %s otwarta, kliknij zeby zamknac";
#elif L==9	// pt
	"Grupo %s aberto, clique para fech&aacute;-lo";
#endif

const char *Txt_Group_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Grup <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Gruppe <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Group <strong>%s</strong> removed.";
#elif L==4	// es
	"Grupo <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Groupe <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Grupo <strong>%s</strong> eliminado.";		// Okoteve traduccin
#elif L==7	// it
	"Gruppo <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Grupa <strong>%s</strong> usunieta.";
#elif L==9	// pt
	"Grupo <strong>%s</strong> removido.";
#endif

const char *Txt_GROUP_WHICH_GROUPS[2] =
   {
#if   L==1	// ca
	"Nom&eacute;s els meus grups"
#elif L==2	// de
	"Nur meine Gruppen"
#elif L==3	// en
	"Only my groups"
#elif L==4	// es
	"Solo mis grupos"
#elif L==5	// fr
	"Uniquement mes groupes"
#elif L==6	// gn
	"Solo mis grupos"	// Okoteve traduccin
#elif L==7	// it
	"Solo i miei gruppi"
#elif L==8	// pl
	"Tylko moje grupy"
#elif L==9	// pt
	"Apenas os meus grupos"
#endif
	,
#if   L==1	// ca
	"Tots els grups"
#elif L==2	// de
	"Alle Gruppen"
#elif L==3	// en
	"All groups"
#elif L==4	// es
	"Todos los grupos"
#elif L==5	// fr
	"Tous les groupes"
#elif L==6	// gn
	"Todos los grupos"	// Okoteve traduccin
#elif L==7	// it
	"Tutti i gruppi"
#elif L==8	// pl
	"Wszystkie grupy"
#elif L==9	// pt
	"Todos os grupos"
#endif
	};

const char *Txt_Groups =
#if   L==1	// ca
	"Grups";
#elif L==2	// de
	"Gruppen";
#elif L==3	// en
	"Groups";
#elif L==4	// es
	"Grupos";
#elif L==5	// fr
	"Groupes";
#elif L==6	// gn
	"Aty";
#elif L==7	// it
	"Gruppi";
#elif L==8	// pl
	"Grupy";
#elif L==9	// pt
	"Grupos";
#endif

const char *Txt_Groups_OF_A_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Grups de %s";
#elif L==2	// de
	"Gruppen von %s";
#elif L==3	// en
	"%s groups";
#elif L==4	// es
	"Grupos de %s";
#elif L==5	// fr
	"Groupes de %s";
#elif L==6	// gn
	"Grupos de %s";		// Okoteve traduccin
#elif L==7	// it
	"Gruppi di %s";
#elif L==8	// pl
	"%s grupy";
#elif L==9	// pt
	"Grupos de %s";
#endif

const char *Txt_Guided_academic_assignments =
#if   L==1	// ca
	"Actividades acad&eacute;micas dirigidas";	// Necessita traduccio
#elif L==2	// de
	"Gef&uuml;hrte akademische Aufgaben";
#elif L==3	// en
	"Guided academic assignments";
#elif L==4	// es
	"Actividades acad&eacute;micas dirigidas";
#elif L==5	// fr
	"Activit&eacute;s acad&eacute;miques guid&eacute;es";
#elif L==6	// gn
	"Actividades acad&eacute;micas dirigidas";	// Okoteve traduccin
#elif L==7	// it
	"Attivit&agrave; accademiche guidate";
#elif L==8	// pl
	"Zwiedzanie dyplomowe";
#elif L==9	// pt
	"Atividades acad&eacute;micas guided";
#endif

const char *Txt_Happy_birthday_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Feli&ccedil; aniversari, %s!";
#elif L==2	// de
	"Gute zum Geburtstag, %s!";
#elif L==3	// en
	"Happy birthday, %s!";
#elif L==4	// es
	"&iexcl;Feliz cumplea&ntilde;os, %s!";
#elif L==5	// fr
	"Joyeux anniversaire, %s!";
#elif L==6	// gn
	"&iexcl;Feliz cumplea&ntilde;os, %s!";	// Okoteve traduccin
#elif L==7	// it
	"Buon compleanno, %s!";
#elif L==8	// pl
	"Wszystkiego najlepszego, %s!";
#elif L==9	// pt
	"Feliz anivers&aacute;rio, %s!";
#endif

const char *Txt_Help =
#if   L==1	// ca
	"Ajuda";
#elif L==2	// de
	"Hilfe";
#elif L==3	// en
	"Help";
#elif L==4	// es
	"Ayuda";
#elif L==5	// fr
	"Aide";
#elif L==6	// gn
	"Pytyv&otilde;";
#elif L==7	// it
	"Aiuto";
#elif L==8	// pl
	"Pomoc";
#elif L==9	// pt
	"Ajuda";
#endif

const char *Txt_HELP_email =
#if   L==1	// ca
	"Exemple: garcia@exemplemail.com";
#elif L==2	// de
	"Beispiel: muller@beispielmail.com";
#elif L==3	// en
	"Example: smith@examplemail.com";
#elif L==4	// es
	"Ejemplo: gonzalez@ejemplomail.com";
#elif L==5	// fr
	"Exemple: dubois@exemplemail.com";
#elif L==6	// gn
	"Ejemplo: gonzalez@ejemplomail.com";	// Okoteve traduccin
#elif L==7	// it
	"Esempio: rossi@esempiomail.com";
#elif L==8	// pl
	"Przyk&lstrok;ad: nowak@przykladmail.com";
#elif L==9	// pt
	"Exemplo: silva@exemplomail.com";
#endif

const char *Txt_HELP_nickname =
#if   L==1	// ca
	"Exemple: @garcia";
#elif L==2	// de
	"Beispiel: @muller";
#elif L==3	// en
	"Example: @smith";
#elif L==4	// es
	"Ejemplo: @gonzalez";
#elif L==5	// fr
	"Exemple: @dubois";
#elif L==6	// gn
	"Ejemplo: @gonzalez";	// Okoteve traduccin
#elif L==7	// it
	"Esempio: @rossi";
#elif L==8	// pl
	"Przyk&lstrok;ad: @nowak";
#elif L==9	// pt
	"Exemplo: @silva";
#endif

const char *Txt_HELP_password =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"%u o m&eacute;s car&agrave;cters";
#elif L==2	// de
	"%u oder mehr Zeichen";
#elif L==3	// en
	"%u or more characters";
#elif L==4	// es
	"%u o m&aacute;s caracteres";
#elif L==5	// fr
	"%u ou plusieurs caract&egrave;res";
#elif L==6	// gn
	"%u o m&aacute;s caracteres";	// Okoteve traduccin
#elif L==7	// it
	"%u o pi&ugrave; caratteri";
#elif L==8	// pl
	"%u lub wi&eogon;cej znak&oacute;w";
#elif L==9	// pt
	"%u ou mais caracteres";
#endif

const char *Txt_Hidden_MALE_PLURAL =
#if   L==1	// ca
	"Ocultos";	// Necessita traduccio
#elif L==2	// de
	"Versteckt";
#elif L==3	// en
	"Hidden";
#elif L==4	// es
	"Ocultos";
#elif L==5	// fr
	"Cach&eacute;s";
#elif L==6	// gn
	"Ocultos";	// Okoteve traduccin
#elif L==7	// it
	"Nascosti";
#elif L==8	// pl
	"Ukryty";
#elif L==9	// pt
	"Ocultos";
#endif

const char *Txt_hidden_document =
#if   L==1	// ca
	"document ocult";
#elif L==2	// de
	"Dokument verborgen";
#elif L==3	// en
	"hidden document";
#elif L==4	// es
	"documento oculto";
#elif L==5	// fr
	"document cach&eacute;";
#elif L==6	// gn
	"documento oculto";	// Okoteve traduccin
#elif L==7	// it
	"documento nascosto";
#elif L==8	// pl
	"hidden document";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"documento oculto";
#endif

const char *Txt_hidden_documents =
#if   L==1	// ca
	"documents ocults";
#elif L==2	// de
	"Dokumente verborgen";
#elif L==3	// en
	"hidden documents";
#elif L==4	// es
	"documentos ocultos";
#elif L==5	// fr
	"documents cach&eacute;s";
#elif L==6	// gn
	"documentos ocultos";	// Okoteve traduccin
#elif L==7	// it
	"documenti nascosti";
#elif L==8	// pl
	"hidden documents";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"documentos ocultos";
#endif

const char *Txt_Hidden_game =
#if   L==1	// ca
	"Joc ocult";
#elif L==2	// de
	"Ausgeblendet Spiel";
#elif L==3	// en
	"Hidden game";
#elif L==4	// es
	"Juego oculto";
#elif L==5	// fr
	"Jeu cach&eacute;";
#elif L==6	// gn
	"Juego oculto";	// Okoteve traduccin
#elif L==7	// it
	"Gioco nascosto";
#elif L==8	// pl
	"Ukryte gra";
#elif L==9	// pt
	"Jogo oculto";
#endif

const char *Txt_Hidden_survey =
#if   L==1	// ca
	"Enquesta oculta";
#elif L==2	// de
	"Ausgeblendet Umfrage";
#elif L==3	// en
	"Hidden survey";
#elif L==4	// es
	"Encuesta oculta";
#elif L==5	// fr
	"Sondage cach&eacute;";
#elif L==6	// gn
	"Encuesta oculta";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio nascosto";
#elif L==8	// pl
	"Ukryte badania";
#elif L==9	// pt
	"Inqu&eacute;rito oculto";
#endif

const char *Txt_Hide =
#if   L==1	// ca
	"Amagar";
#elif L==2	// de
	"Ausblenden";
#elif L==3	// en
	"Hide";
#elif L==4	// es
	"Ocultar";
#elif L==5	// fr
	"Masquer";
#elif L==6	// gn
	"Ocultar";	// Okoteve traduccin
#elif L==7	// it
	"Nascondi";
#elif L==8	// pl
	"Ukryj";
#elif L==9	// pt
	"Ocultar";
#endif

const char *Txt_Hide_FOLDER_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Amagar %s";
#elif L==2	// de
	"Ausblenden %s";
#elif L==3	// en
	"Hide %s";
#elif L==4	// es
	"Ocultar %s";
#elif L==5	// fr
	"Masquer %s";
#elif L==6	// gn
	"Ocultar %s";	// Okoteve traduccin
#elif L==7	// it
	"Nascondi %s";
#elif L==8	// pl
	"Ukryj %s";
#elif L==9	// pt
	"Ocultar %s";
#endif

const char *Txt_Hide_left_column =
#if   L==1	// ca
	"Ocultar columna izquierda";	// Necessita traduccio
#elif L==2	// de
	"Links Spalte ausblenden";
#elif L==3	// en
	"Hide left column";
#elif L==4	// es
	"Ocultar columna izquierda";
#elif L==5	// fr
	"Masquer la colonne de gauche";
#elif L==6	// gn
	"Ocultar columna izquierda";	// Okoteve traduccin
#elif L==7	// it
	"Nascondi colonna di sinistra";
#elif L==8	// pl
	"Ukryj lewa kolumne";
#elif L==9	// pt
	"Ocultar coluna da esquerda";
#endif

const char *Txt_Hide_message =
#if   L==1	// ca
	"Amaga missatge";
#elif L==2	// de
	"Ausblende Nachricht";
#elif L==3	// en
	"Hide message";
#elif L==4	// es
	"Ocultar mensaje";
#elif L==5	// fr
	"Masquer message";
#elif L==6	// gn
	"Ocultar mensaje";	// Okoteve traduccin
#elif L==7	// it
	"Nascondi messaggio";
#elif L==8	// pl
	"Hide message";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ocultar mensagem";
#endif

const char *Txt_Hide_right_column =
#if   L==1	// ca
	"Ocultar columna derecha";	// Necessita traduccio
#elif L==2	// de
	"Rechte Spalte ausblenden";
#elif L==3	// en
	"Hide right column";
#elif L==4	// es
	"Ocultar columna derecha";
#elif L==5	// fr
	"Masquer la colonne de droite";
#elif L==6	// gn
	"Ocultar columna derecha";	// Okoteve traduccin
#elif L==7	// it
	"Nascondi colonna di destra";
#elif L==8	// pl
	"Ukryj prawa kolumne";
#elif L==9	// pt
	"Ocultar coluna da direita";
#endif

const char *Txt_historical_log =
#if   L==1	// ca
	"registre hist&ograve;ric";
#elif L==2	// de
	"historische log";
#elif L==3	// en
	"historical log";
#elif L==4	// es
	"registro hist&oacute;rico";
#elif L==5	// fr
	"journal historique";
#elif L==6	// gn
	"registro hist&oacute;rico";		// Okoteve traduccin
#elif L==7	// it
	"registro storico";
#elif L==8	// pl
	"dziennik historyczny";
#elif L==9	// pt
	"log hist&oacute;rico";
#endif

const char *Txt_Hits =	// hits = visits, clicks, page views...
#if   L==1	// ca
	"Accessos";
#elif L==2	// de
	"Anmeldungen";
#elif L==3	// en
	"Hits";
#elif L==4	// es
	"Accesos (clics)";
#elif L==5	// fr
	"Visites";
#elif L==6	// gn
	"Accesos (clics)";		// Okoteve traduccin
#elif L==7	// it
	"Visite";
#elif L==8	// pl
	"Ods&lstrok;on";
#elif L==9	// pt
	"Visitas";
#endif

const char *Txt_Hits_as_a_USER =	// hits = visits, clicks, page views... Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Accessos (clics) com a %s";
#elif L==2	// de
	"Anmeldungen (Klicks) als %s";
#elif L==3	// en
	"Hits (clicks) as a %s";
#elif L==4	// es
	"Accesos (clics) como %s";
#elif L==5	// fr
	"Visites (clics) en tant que %s";
#elif L==6	// gn
	"Accesos (clics) como %s";		// Okoteve traduccin
#elif L==7	// it
	"Visite (scatti) come %s";
#elif L==8	// pl
	"Ods&lstrok;on (klikni&eogon;cia) jako %s";
#elif L==9	// pt
	"Visitas (cliques) como %s";
#endif

const char *Txt_Hits_per_action =	// hits = visits, clicks, page views...
#if   L==1	// ca
	"Accessos (clics) per acci&oacute;";
#elif L==2	// de
	"Anmeldungen (Klicks) pro Handlung";
#elif L==3	// en
	"Hits (clicks) per action";
#elif L==4	// es
	"Accesos (clics) por acci&oacute;n";
#elif L==5	// fr
	"Visites (clics) par action";
#elif L==6	// gn
	"Accesos (clics) por acci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Visite (clic) per azione";
#elif L==8	// pl
	"Ods&lstrok;on (klikni&eogon;cia) dzia&lstrok;anie";
#elif L==9	// pt
	"Visitas (cliques) por a&ccedil;&atilde;o";
#endif

const char *Txt_Hits_per_year =	// hits = visits, clicks, page views...
#if   L==1	// ca
	"Accessos (clics) per any";
#elif L==2	// de
	"Anmeldungen (Klicks) pro Jahr";
#elif L==3	// en
	"Hits (clicks) per year";
#elif L==4	// es
	"Accesos (clics) por a&ntilde;o";
#elif L==5	// fr
	"Visites (clics) par an";
#elif L==6	// gn
	"Accesos (clics) por a&ntilde;o";	// Okoteve traduccin
#elif L==7	// it
	"Visite (clic) all'anno";
#elif L==8	// pl
	"Ods&lstrok;on (klikni&eogon;cia) rocznie";
#elif L==9	// pt
	"Visitas (cliques) por ano";
#endif

const char *Txt_Hits_without_course_selected =	// hits = visits, clicks, page views...
#if   L==1	// ca
	"Accessos (clics) sense assignatura seleccionada";
#elif L==2	// de
	"Anmeldungen (Klicks) ohne Kurs ausgew&auml;hlt";
#elif L==3	// en
	"Hits (clicks) without course selected";
#elif L==4	// es
	"Accesos (clics) sin asignatura seleccionada";
#elif L==5	// fr
	"Visites (clics) sans mati&egrave;re s&eacute;lectionn&eacute;e";
#elif L==6	// gn
	"Accesos (clics) sin asignatura seleccionada";		// Okoteve traduccin
#elif L==7	// it
	"Visite (scatti) senza corso selezionato";
#elif L==8	// pl
	"Ods&lstrok;on (klikni&eogon;cia) bez oczywi&sacute;cie wybrana";
#elif L==9	// pt
	"Visitas (cliques) sem disciplina selecionada";
#endif

const char *Txt_Holiday =
#if   L==1	// ca
	"Festivitat";
#elif L==2	// de
	"Feiertag";
#elif L==3	// en
	"Holiday";
#elif L==4	// es
	"Festividad";
#elif L==5	// fr
	"F&ecirc;te";
#elif L==6	// gn
	"Festividad";
#elif L==7	// it
	"Festivit&agrave;";
#elif L==8	// pl
	"Wakacje";
#elif L==9	// pt
	"F&eacute;ria";
#endif

const char *Txt_Holiday_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Festividad <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2	// de
	"Feiertag <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Holiday <strong>%s</strong> removed.";
#elif L==4	// es
	"Festividad <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Holiday <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6	// gn
	"Festividad <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Festivit&agrave; <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Wakacje <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Festividade <strong>%s</strong> eliminada.";
#endif

const char *Txt_HOLIDAY_TYPES[Hld_NUM_TYPES_HOLIDAY] =
   {
#if   L==1	// ca
	 "D&iacute;a festivo"	// Necessita traduccio
#elif L==2	// de
	 "Urlaub"
#elif L==3	// en
	 "Holiday"
#elif L==4	// es
	 "D&iacute;a festivo"
#elif L==5	// fr
	 "Jour f&eacute;ri&eacute;"
#elif L==6	// gn
	 "Arete"
#elif L==7	// it
	 "Festivit&agrave;"
#elif L==8	// pl
	 "wakacje"
#elif L==9	// pt
	 "Feriado"
#endif
	 ,
#if   L==1	// ca
	 "Per&iacute;odo no lectivo"	// Necessita traduccio
#elif L==2	// de
	 "Non-Schule Zeitraum"
#elif L==3	// en
	 "Non school period"
#elif L==4	// es
	 "Per&iacute;odo no lectivo"
#elif L==5	// fr
	 "P&eacute;riode sans &eacute;cole"
#elif L==6	// gn
	 "Per&iacute;odo no lectivo"	// Okoteve traduccin
#elif L==7	// it
	 "Periodo senza lezione"
#elif L==8	// pl
	 "Okres nauki szkolnej dla"
#elif L==9	// pt
	 "Per&iacute;odo sem escola"
#endif
	};

const char *Txt_Holidays =
#if   L==1	// ca
	"Dies festius";
#elif L==2	// de
	"Feiertage";
#elif L==3	// en
	"Holidays";
#elif L==4	// es
	"D&iacute;as festivos";
#elif L==5	// fr
	"Jours f&eacute;ri&eacute;s";
#elif L==6	// gn
	"Arete";
#elif L==7	// it
	"Festivit&agrave;";
#elif L==8	// pl
	"Noclegi";
#elif L==9	// pt
	"F&eacute;rias";
#endif

const char *Txt_HOLIDAYS_HELP_ORDER[Hld_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Ordenar por lugar"	// Necessita traduccio
#elif L==2	// de
	 "Nach Standort sortieren"
#elif L==3	// en
	 "Sort by place"
#elif L==4	// es
	 "Ordenar por lugar"
#elif L==5	// fr
	 "Trier par emplacement"
#elif L==6	// gn
	 "Ordenar por lugar"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per localit&grave;"
#elif L==8	// pl
	 "Sortuj wedlug miejsca"
#elif L==9	// pt
	 "Classificar por localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar por fecha"	// Necessita traduccio
#elif L==2	// de
	 "Sortieren nach Datum"
#elif L==3	// en
	 "Sort by date"
#elif L==4	// es
	 "Ordenar por fecha"
#elif L==5	// fr
	 "Trier par date"
#elif L==6	// gn
	 "Ordenar por fecha"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per data"
#elif L==8	// pl
	 "Sortuj wedlug daty"
#elif L==9	// pt
	 "Classificar por data"
#endif
	};

const char *Txt_HOLIDAYS_ORDER[Hld_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Lloc"
#elif L==2	// de
	 "Standort"
#elif L==3	// en
	 "Place"
#elif L==4	// es
	 "Lugar"
#elif L==5	// fr
	 "Emplacement"
#elif L==6	// gn
	 "Tenda"
#elif L==7	// it
	 "Luogo"
#elif L==8	// pl
	 "Miejsce"
#elif L==9	// pt
	 "Localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "Data inicial"
#elif L==2	// de
	 "Startdatum"
#elif L==3	// en
	 "Start date"
#elif L==4	// es
	 "Fecha inicial"
#elif L==5	// fr
	 "Date initiale"
#elif L==6	// gn
	 "Fecha inicial"	// Okoteve traduccin
#elif L==7	// it
	 "Data iniziale"
#elif L==8	// pl
	 "Data rozpoczecia"
#elif L==9	// pt
	 "Data de in&iacute;cio"
#endif
	};

const char *Txt_Home_PAGE =
#if   L==1	// ca
	"Inicio";	// Necessita traduccio
#elif L==2	// de
	"Home";
#elif L==3	// en
	"Home";
#elif L==4	// es
	"Inicio";
#elif L==5	// fr
	"Accueil";
#elif L==6	// gn
	"Inicio";	// Okoteve traduccin
#elif L==7	// it
	"Home";
#elif L==8	// pl
	"Dom";
#elif L==9	// pt
	"Home";
#endif

const char *Txt_hour =
#if   L==1	// ca
	"hora";
#elif L==2	// de
	"Stunde";
#elif L==3	// en
	"hour";
#elif L==4	// es
	"hora";
#elif L==5	// fr
	"heure";
#elif L==6	// gn
	"aravo";
#elif L==7	// it
	"ora";
#elif L==8	// pl
	"godzina";
#elif L==9	// pt
	"hora";
#endif

const char *Txt_hours =
#if   L==1	// ca
	"horas";	// Necessita traduccio
#elif L==2	// de
	"Stunden";
#elif L==3	// en
	"hours";
#elif L==4	// es
	"horas";
#elif L==5	// fr
	"heures";
#elif L==6	// gn
	"aravo";
#elif L==7	// it
	"ore";
#elif L==8	// pl
	"godziny";
#elif L==9	// pt
	"horas";
#endif

const char *Txt_hours_ABBREVIATION =	// Abbreviation of "hours"
#if   L==1	// ca
	"h";	// Necessita traduccio
#elif L==2	// de
	"S";
#elif L==3	// en
	"h";
#elif L==4	// es
	"h";
#elif L==5	// fr
	"h";
#elif L==6	// gn
	"a";
#elif L==7	// it
	"o";
#elif L==8	// pl
	"g";
#elif L==9	// pt
	"h";
#endif

const char *Txt_I_have_read_this_information =
#if   L==1	// ca
	"He le&iacute;do esta informaci&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Ich habe diese Information lesen";
#elif L==3	// en
	"I have read this information";
#elif L==4	// es
	"He le&iacute;do esta informaci&oacute;n";
#elif L==5	// fr
	"J'ai lu cette information";
#elif L==6	// gn
	"He le&iacute;do esta informaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Ho letto questa informazione";
#elif L==8	// pl
	"Czytalem te informacje";
#elif L==9	// pt
	"Eu li esta informa&ccedil;&atilde;o";
#endif

const char *Txt_I_understand_that_this_action_can_not_be_undone =
#if   L==1	// ca
	"Entenc que aquesta acci&oacute; no es pot desfer.";
#elif L==2	// de
	"Ich verstehe, dass diese Aktion kann nicht r&uuml;ckg&auml;ngig gemacht werden.";
#elif L==3	// en
	"I understand that this action can not be undone.";
#elif L==4	// es
	"Entiendo que esta acci&oacute;n no se puede deshacer.";
#elif L==5	// fr
	"Je comprends que cette action ne peut &ecirc;tre annul&eacute;e.";
#elif L==6	// gn
	"Entiendo que esta acci&oacute;n no se puede deshacer.";		// Okoteve traduccin
#elif L==7	// it
	"Mi rendo conto che questa azione non pu&ograve; essere annullata.";
#elif L==8	// pl
	"Rozumiem, &zdot;e dzia&lstrok;anie nie mo&zdot;na cofn&aogon;&cacute;.";
#elif L==9	// pt
	"Eu entendo que esta a&ccedil;&atilde;o n&atilde;o pode ser desfeita.";
#endif

const char *Txt_Icons =
#if   L==1	// ca
	"Icones";
#elif L==2	// de
	"Symbole";
#elif L==3	// en
	"Icons";
#elif L==4	// es
	"Iconos";
#elif L==5	// fr
	"Ic&ocirc;nes";
#elif L==6	// gn
	"Iconos";	// Okoteve traduccin
#elif L==7	// it
	"Icone";
#elif L==8	// pl
	"Ikony";
#elif L==9	// pt
	"&Iacute;cones";
#endif

const char *Txt_ID =
#if   L==1	// ca
	"ID (DNI/c&egrave;dula)";
#elif L==2	// de
	"Ausweis-Nr.";
#elif L==3	// en
	"ID (identity number/passport)";
#elif L==4	// es
	"ID (DNI/c&eacute;dula)";
#elif L==5	// fr
	"Num. d'identit&eacute;";
#elif L==6	// gn
	"ID (DNI/c&eacute;dula)";	// Okoteve traduccin
#elif L==7	// it
	"Numero carta d'identit&agrave;";
#elif L==8	// pl
	"Dow&oacute;d osobisty";
#elif L==9	// pt
	"N&ordm; identif.";
#endif

const char *Txt_ID_X_had_already_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El ID (DNI/c&eacute;dula) %s ja havia estat confirmat.";
#elif L==2	// de
	"ID %s hatte bereits best&auml;tigt worden.";
#elif L==3	// en
	"ID %s had already been confirmed.";
#elif L==4	// es
	"El ID (DNI/c&eacute;dula) %s ya hab&iacute;a sido confirmado.";
#elif L==5	// fr
	"L'ID %s avait d&eacute;j&agrave; &eacute;t&eacute; confirm&eacute;e.";
#elif L==6	// gn
	"El ID (DNI/c&eacute;dula) %s ya hab&iacute;a sido confirmado.";		// Okoteve traduccin
#elif L==7	// it
	"L'ID %s era gi&agrave; stato confermato.";
#elif L==8	// pl
	"ID %s zosta&lstrok;a ju&zdot; potwierdzona.";
#elif L==9	// pt
	"O ID %s j&aacute; havia sido confirmada.";
#endif

const char *Txt_ID_X_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"ID (DNI/c&eacute;dula) %s confirmat";
#elif L==2	// de
	"ID %s best&auml;tigt";
#elif L==3	// en
	"ID %s confirmed";
#elif L==4	// es
	"ID (DNI/c&eacute;dula) %s confirmado";
#elif L==5	// fr
	"ID %s confirm&eacute;";
#elif L==6	// gn
	"ID (DNI/c&eacute;dula) %s confirmado";		// Okoteve traduccin
#elif L==7	// it
	"ID %s confermato";
#elif L==8	// pl
	"ID %s potwierdzone";
#elif L==9	// pt
	"ID %s confirmado";
#endif

const char *Txt_ID_X_not_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"ID (DNI/c&eacute;dula) %s no confirmat";
#elif L==2	// de
	"ID %s nicht best&auml;tigt";
#elif L==3	// en
	"ID %s not confirmed";
#elif L==4	// es
	"ID (DNI/c&eacute;dula) %s no confirmado";
#elif L==5	// fr
	"ID %s pas confirm&eacute;";
#elif L==6	// gn
	"ID (DNI/c&eacute;dula) %s no confirmado";		// Okoteve traduccin
#elif L==7	// it
	"ID %s non confermato";
#elif L==8	// pl
	"ID %s nie potwierdzone";
#elif L==9	// pt
	"ID %s n&atilde;o confirmado";
#endif

const char *Txt_ID_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminat.";
#elif L==2	// de
	"ID <strong>%s</strong> entfernt.";
#elif L==3	// en
	"ID <strong>%s</strong> removed.";
#elif L==4	// es
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"ID <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"ID <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"ID <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Email <strong>%s</strong> removido.";
#endif

const char *Txt_Identified_views =
#if   L==1	// ca
	"Accessos identificats";
#elif L==2	// de
	"Identifizierte Ansichten";
#elif L==3	// en
	"Identified views";
#elif L==4	// es
	"Accesos identificados";
#elif L==5	// fr
	"Vues identifi&eacute;es";
#elif L==6	// gn
	"Accesos identificados";	// Okoteve traduccin
#elif L==7	// it
	"Visualizzazioni identificati";
#elif L==8	// pl
	"Zidentyfikowane ods&lstrok;on";
#elif L==9	// pt
	"Acessos identificados";
#endif

const char *Txt_If_this_is_a_new_user_in_X_you_should_indicate_her_his_ID =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";	// Necessita traduccio
#elif L==2	// de
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// bersetzung notwendig
#elif L==3	// en
	"If this is a new user in %s,"
	" you should indicate her/his ID.";
#elif L==4	// es
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";
#elif L==5	// fr
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Besoin de traduction
#elif L==6	// gn
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==7	// it
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Bisogno di traduzione
#elif L==8	// pl
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Se este for um novo usu&aacute;rio em %s,"
	" voc&ecirc; deve indicar seu ID.";
#endif

const char *Txt_If_you_can_not_find_your_institution_your_centre_your_degree_or_your_courses_you_can_create_them =
#if   L==1	// ca
	"Si no trobes la seva instituci&oacute;, el seu centre,"
	" la seva titulaci&oacute; o les seves assignatures, pot crear-los.";
#elif L==2	// de
	"Wenn Sie Ihre Institution, Ihre Zentrum, Ihr Studium"
	" oder Ihre Kurse nicht finden k&oacute;nnen, k&oacute;nnen Sie sie erstellen.";
#elif L==3	// en
	"If you can not find your institution, your centre,"
	" your degree or your courses, you can create them.";
#elif L==4	// es
	"Si no encuentra su instituci&oacute;n, su centro,"
	" su titulaci&oacute;n o sus asignaturas, puede crearlos.";
#elif L==5	// fr
	"Si vous ne trouvez pas votre &eacute;tablissement, votre centre,"
	" votre &eacute;tude ou de vos mati&egrave;res, vous pouvez les cr&eacute;er.";
#elif L==6	// gn
	"Si no encuentra su instituci&oacute;n, su centro,"
	" su titulaci&oacute;n o sus asignaturas, puede crearlos.";	// Okoteve traduccin
#elif L==7	// it
	"Se non riesci a trovare il vostro istituto, il centro,"
	" il vostro grado o vostri corsi, &egrave; possibile creare.";
#elif L==8	// pl
	"Je&sacute;li nie mo&zdot;esz znale&zacute;&cacute; instytucj&eogo, Centrum,"
	" studia lub kursy swoich, mo&zdot;na je tworzy&cacute;.";
#elif L==9	// pt
	"Se voc&ecirc; n&atilde;o consegue encontrar a sua institui&ccedil;&atilde;o, o seu centro,"
	" o seu grau ou as suas disciplinas, voc&ecirc; pode cri&aacute;-los.";
#endif

const char *Txt_If_you_have_an_Android_device_try_SWADroid =
#if   L==1	// ca
	"Si teniu un dispositiu Android, provi SWADroid";
#elif L==2	// de
	"Wenn Sie ein Android-Ger&auml;t haben, versuchen SWADroid";
#elif L==3	// en
	"If you have an Android device, try SWADroid";
#elif L==4	// es
	"Si tiene un dispositivo Android, pruebe SWADroid";
#elif L==5	// fr
	"Si vous avez un appareil Android, essayer SWADroid";
#elif L==6	// gn
	"Si tiene un dispositivo Android, pruebe SWADroid";	// Okoteve traduccin
#elif L==7	// it
	"Se avete un dispositivo Android, provate SWADroid";
#elif L==8	// pl
	"Je&sacute;li masz telefon z systemem Android, spr&oacute;buj SWADroid";
#elif L==9	// pt
	"Se voc&ecirc; tiver um dispositivo Android, tente SWADroid";
#endif

const char *Txt_If_you_have_forgotten_your_password_ =
#if   L==1	// ca
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva"
	" a la direcci&oacute;n de correo"
	" que escribi&oacute; en su ficha personal.";	// Necessita traduccio
#elif L==2	// de
	"Wenn Sie Ihr Passwort vergessen haben,"
	" eine neue wird"
	" die E-Mail von Ihnen angegebene Adresse"
	" in Ihren Profil gesendet werden vergessen.";
#elif L==3	// en
	"If you have forgotten your password,"
	" a new one will be sent"
	" to the email address"
	" you entered in your record.";
#elif L==4	// es
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva"
	" a la direcci&oacute;n de correo"
	" que escribi&oacute; en su ficha personal.";
#elif L==5	// fr
	"Si vous avez oubli&eacute; votre mot de passe,"
	" une nouvelle sera envoy&eacute;"
	" &agrave; l'adresse &eacute;lectronique"
	" que vous avez entr&eacute; dans votre carte.";
#elif L==6	// gn
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva"
	" a la direcci&oacute;n de correo"
	" que escribi&oacute; en su ficha personal.";	// Okoteve traduccin
#elif L==7	// it
	"Se hai dimenticato la tua password,"
	" una nuova verr&agrave; inviata"
	" all'indirizzo email"
	" che hai inserito nella tua scheda.";
#elif L==8	// pl
	"Jesli nie pamietasz hasla,"
	" nowe zostanie wyslane na adres email,"
	" kt&oacute;ry wszedl w rekordzie.";
#elif L==9	// pt
	"Se voc&ecirc; tiver esquecido sua senha,"
	" uma nova ser&aacute; enviada"
	" para o email"
	" que voc&ecirc; inseriu em seu cart&atilde;o.";
#endif

const char *Txt_If_you_have_written_your_ID_nickname_or_email_correctly_ =
#if   L==1	// ca
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y el correo en su ficha es correcto,"
	" recibir&aacute; un mensaje con una nueva contrase&ntilde;a.";	// Necessita traduccio
#elif L==2	// de
	"Wenn Sie Ihre @Benutzername, Email oder Ausweis-Nr.,"
	" korrekt geschrieben wurden, und die E-Mail-Adresse auf Ihren Profil korrekt ist,"
	" erhalten Sie eine E-Mail mit einem neuen Passwort.";
#elif L==3	// en
	"If you have written your @nick, email or ID correctly,"
	" and the email address on your record is correct,"
	" you will receive an email with a new password.";
#elif L==4	// es
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y el correo en su ficha es correcto,"
	" recibir&aacute; un mensaje con una nueva contrase&ntilde;a.";
#elif L==5	// fr
	"Si vous avez &eacute;crit votre @surnom, courrier ou num&eacute;ro d'identit&eacute; correctement,"
	" ainsi que l'adresse email sur votre carte est correct,"
	" vous recevrez un email avec un nouveau mot de passe.";
#elif L==6	// gn
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y el correo en su ficha es correcto,"
	" recibir&aacute; un mensaje con una nueva contrase&ntilde;a.";	// Okoteve traduccin
#elif L==7	// it
	"Se hai scritto il tuo nome utente, email o numero di carta d'identit&agrave; correttamente,"
	" e l'indirizzo email della tua scheda &egrave; corretto,"
	" riceverai una email con una nuova password.";
#elif L==8	// pl
	"Jesli napisales swoja nazwe lub nick poprawnie,"
	" a adres email na zapis jest poprawny,"
	" otrzymasz maila z nowym haslem.";
#elif L==9	// pt
	"Se voc&ecirc; ter escrito o suo alcunha, email ou n&ordm; identif. corretamente,"
	" bem como o endere&ccedil;o de email em seu cart&atilde;o est&aacute; correto,"
	" voc&ecirc; receber&aacute; um email com uma nova senha.";
#endif

const char *Txt_If_you_think_you_may_have_been_registered_ =
#if   L==1	// ca
	"Si creu que pot haver estat inscrit/a"
	" per un professor o administrador,"
	" comprovi escrivint algun dels seus possibles ID"
	" (DNI amb lletra final o sense, passaport, etc.).";
#elif L==2	// de
	"Wenn Sie glauben, dass Sie"
	" von einem Lehrer oder Administrator registriert worden sind,"
	" &uuml;berpr&uuml;fen Sie ihn, indem Sie eine Ihrer m&ouml;glichen IDs"
	" (ID, Reisepass usw.) schreiben.";
#elif L==3	// en
	"If you think you may have been registered"
	" by a teacher or administrator,"
	" check it by writing one of your possible IDs (ID, passport, etc.).";
#elif L==4	// es
	"Si cree que puede haber sido inscrito/a"
	" por un profesor o administrador,"
	" compru&eacute;belo escribiendo alguno de sus posibles ID"
	" (DNI con letra final o sin ella, c&eacute;dula, pasaporte, etc.).";
#elif L==5	// fr
	"Si vous pensez que vous avez &eacute;t&eacute; enregistr&eacute;"
	" par un enseignant ou un administrateur,"
	" v&eacute;rifiez-le en &eacute;crivant l'un de vos identifiants possibles"
	" (carte d'identit&eacute;, passeport, etc.).";
#elif L==6	// gn
	"Si cree que puede haber sido inscrito/a"
	" por un profesor o administrador,"
	" compru&eacute;belo escribiendo alguno de sus posibles ID"
	" (DNI con letra final o sin ella, c&eacute;dula, pasaporte, etc.).";	// Okoteve traduccin
#elif L==7	// it
	"Se pensi di essere stato registrato"
	" da un insegnante o da un amministratore,"
	" controllarlo scrivendo uno dei tuoi possibili ID"
	" (ID, passaporto, ecc.).";
#elif L==8	// pl
	"Je&sacute;li uwa&zdot;asz, &zdot;e mo&zdot;esz zosta&cacute; zarejestrowany"
	" przez nauczyciela lub administratora,"
	" sprawd&zacute; go, wpisuj&aogon;c jeden z mo&zdot;liwych identyfikator&oacute;w"
	" (dowodu to&zdot;samo&sacute;ci, paszportu itp.).";
#elif L==9	// pt
	"Se voc&ecirc; acha que pode ter sido registrado"
	" por um professor ou administrador,"
	" verifique-o escrevendo uma das suas poss&iacute;veis IDs"
	" (identifica&ccedil;&atilde;o, passaporte, etc.).";
#endif

const char *Txt_Image =
#if   L==1	// ca
	"Imatge";
#elif L==2	// de
	"Abbild";
#elif L==3	// en
	"Image";
#elif L==4	// es
	"Imagen";
#elif L==5	// fr
	"Image";
#elif L==6	// gn
	"Ta'&atilde;nga";
#elif L==7	// it
	"Immagine";
#elif L==8	// pl
	"Obraz";
#elif L==9	// pt
	"Imagem";
#endif

const char *Txt_Image_video =
#if   L==1	// ca
	"Imatge / v&iacute;deo";
#elif L==2	// de
	"Abbild / Video";
#elif L==3	// en
	"Image / video";
#elif L==4	// es
	"Imagen / v&iacute;deo";
#elif L==5	// fr
	"Image / vid&eacute;o";
#elif L==6	// gn
	"Ta'&atilde;nga / video";
#elif L==7	// it
	"Immagine / video";
#elif L==8	// pl
	"Obraz / wideo";
#elif L==9	// pt
	"Imagem / v&iacute;deo";
#endif

const char *Txt_Import_questions =
#if   L==1	// ca
	"Importa preguntes";
#elif L==2	// de
	"Fragen importieren";
#elif L==3	// en
	"Import questions";
#elif L==4	// es
	"Importar preguntas";
#elif L==5	// fr
	"Import questions";	// Besoin de traduction
#elif L==6	// gn
	"Importar preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Import questions";	// Bisogno di traduzione
#elif L==8	// pl
	"Import questions";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Importar perguntas";
#endif

const char *Txt_Imported_questions =
#if   L==1	// ca
	"Preguntas importadas";	// Necessita traduccio
#elif L==2	// de
	"Imported questions";	// Need bersetzung
#elif L==3	// en
	"Imported questions";
#elif L==4	// es
	"Preguntas importadas";
#elif L==5	// fr
	"Imported questions";	// Besoin de traduction
#elif L==6	// gn
	"Preguntas importadas";	// Okoteve traduccin
#elif L==7	// it
	"Imported questions";	// Bisogno di traduzione
#elif L==8	// pl
	"Imported questions";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Perguntas importadas";
#endif

const char *Txt_SEARCH_X_in_Y =	// Search string "in" documentos, users, etc.
#if   L==1	// ca
	"en";
#elif L==2	// de
	"im";
#elif L==3	// en
	"in";
#elif L==4	// es
	"en";
#elif L==5	// fr
	"dans";
#elif L==6	// gn
	"en";	// Okoteve traduccin
#elif L==7	// it
	"in";
#elif L==8	// pl
	"w";
#elif L==9	// pt
	"em";
#endif

const char *Txt_In_a_type_of_group_with_single_enrolment_students_can_not_be_registered_in_more_than_one_group =
#if   L==1	// ca
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";	// Necessita traduccio
#elif L==2	// de
	"In a type of group with single enrolment,"
	" students can not be registered in more than one group.";	// Need bersetzung
#elif L==3	// en
	"In a type of group with single enrolment,"
	" tudents can not be registered in more than one group.";
#elif L==4	// es
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";
#elif L==5	// fr
	"In a type of group with single enrolment,"
	" students can not be registered in more than one group.";	// Besoin de traduction
#elif L==6	// gn
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";	// Okoteve traduccin
#elif L==7	// it
	"In un tipo di gruppo a singola registrazione,"
	" non puoi registrare studenti in pi&ugrave; di un gruppo.";
#elif L==8	// pl
	"W rodzaju grupy z jednej rejestracji,"
	" nie mozesz rejestrowac uczni&oacute;w w wiecej niz jednej grupy.";
#elif L==9	// pt
	"Em um tipo de grupo com inscri&ccedil;&atilde;o &uacute;nica,"
	" os estudantes n&atilde;o podem ser registrados em mais de um grupo.";
#endif

const char *Txt_In_the_year_X_of_the_degree_Y_already_existed_a_course_with_the_name_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"En el a&ntilde;o/semestre <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"In the year/semester <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"In the year/semester <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";
#elif L==4	// es
	"En el a&ntilde;o/semestre <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";
#elif L==5	// fr
	"In the year/semester <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"En el a&ntilde;o/semestre <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Nell'anno/semester <strong>%s</strong> di laurea <strong>%s</strong>"
	" gi&agrave; esiste un corso con il nome <strong>%s</strong>.";
#elif L==8	// pl
	"W roku/semestr <strong>%s</strong> stopnia <strong>%s</strong>"
	" istnialy juz kurs&oacute;w z nazwa <strong>%s</strong>.";
#elif L==9	// pt
	"No ano/semestre <strong>%s</strong> do grau <strong>%s</strong>"
	" j&aacute; existia uma disciplina com o nome <strong>%s</strong>.";
#endif

const char *Txt_Incomplete_timetable_for_lack_of_space =
#if   L==1	// ca
	"Horari incomplet per falta d'espai.";
#elif L==2	// de
	"Unvollst&auml;ndige Studenplan f&uuml;r Platzmangel.";
#elif L==3	// en
	"Incomplete timetable for lack of space.";
#elif L==4	// es
	"Horario incompleto por falta de espacio.";
#elif L==5	// fr
	"Horaire incompl&egrave;te par manque de place.";
#elif L==6	// gn
	"Horario incompleto por falta de espacio.";	// Okoteve traduccin
#elif L==7	// it
	"Orario incompleto per mancanza di spazio.";
#elif L==8	// pl
	"Niekompletne harmonogram brak miejsca.";
#elif L==9	// pt
	"Hor&aacute;rio incompleto por falta de espa&ccedil;o.";
#endif

const char *Txt_Increase_level_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Aumentar nivel de %s";		// Necessita traduccio
#elif L==2	// de
	"Increase level of %s";		// Need bersetzung
#elif L==3	// en
	"Increase level of %s";
#elif L==4	// es
	"Aumentar nivel de %s";
#elif L==5	// fr
	"Increase level of %s";		// Besoin de traduction
#elif L==6	// gn
	"Aumentar nivel de %s";		// Okoteve traduccin
#elif L==7	// it
	"Incrementare livello di %s";
#elif L==8	// pl
	"Wzrost poziomu %s";
#elif L==9	// pt
	"Aumentar o n&iacute;vel de %s";
#endif

const char *Txt_Indicators =
#if   L==1	// ca
	"Indicadors";
#elif L==2	// de
	"Indikatoren";
#elif L==3	// en
	"Indicators";
#elif L==4	// es
	"Indicadores";
#elif L==5	// fr
	"Indicateurs";
#elif L==6	// gn
	"Indicadores";	// Okoteve traduccin
#elif L==7	// it
	"Indicatori";
#elif L==8	// pl
	"Wska&zacute;niki";
#elif L==9	// pt
	"Indocadores";
#endif

const char *Txt_Indicators_of_courses =
#if   L==1	// ca
	"Indicadors d'assignatures";
#elif L==2	// de
	"Anzeigen von Kursen";
#elif L==3	// en
	"Indicators of courses";
#elif L==4	// es
	"Indicadores de asignaturas";
#elif L==5	// fr
	"Indicateurs de mati&egrave;res";
#elif L==6	// gn
	"Indicadores de asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Indicatori di corsi";
#elif L==8	// pl
	"Wska&zacute;niki kurs&oacute;w";
#elif L==9	// pt
	"Indicadores de disciplinas";
#endif

const char *Txt_Info =
#if   L==1	// ca
	 "Info";
#elif L==2	// de
	 "Info";
#elif L==3	// en
	 "Info";
#elif L==4	// es
	 "Info";
#elif L==5	// fr
	 "Info";
#elif L==6	// gn
	 "Info";	// Okoteve traduccin
#elif L==7	// it
	 "Informazione";
#elif L==8	// pl
	 "Info";
#elif L==9	// pt
	 "Info";
#endif

const char *Txt_INFO_TITLE[Inf_NUM_INFO_TYPES] =
   {
   // Inf_INTRODUCTION
#if   L==1	// ca
	 "Informaci&oacute;"
#elif L==2	// de
	 "Information"
#elif L==3	// en
	 "Information"
#elif L==4	// es
	 "Informaci&oacute;n"
#elif L==5	// fr
	 "Information"
#elif L==6	// gn
	 "Informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Informazione"
#elif L==8	// pl
	 "Informacja"
#elif L==9	// pt
	 "Informa&ccedil;&atilde;o"
#endif
	 ,
	 // Inf_TEACHING_GUIDE
#if   L==1	// ca
	 "Guia docent"
#elif L==2	// de
	 "Hinweise zur Kurs"
#elif L==3	// en
	 "Teaching guide"
#elif L==4	// es
	 "Gu&iacute;a docente"
#elif L==5	// fr
	 "Guide p&eacute;dagogique"
#elif L==6	// gn
	 "Gu&iacute;a docente"	// Okoteve traduccin
#elif L==7	// it
	 "Guida didattica"
#elif L==8	// pl
	 "Nauczanie przypomnienie"
#elif L==9	// pt
	 "Guia de ensino"
#endif
	 ,
	 // Inf_LECTURES
#if   L==1	// ca
	 "Programa teoria"
#elif L==2	// de
	 "Studienplan (Vorlesungen)"
#elif L==3	// en
	 "Syllabus (lectures)"
#elif L==4	// es
	 "Programa teor&iacute;a"
#elif L==5	// fr
	 "Plan th&eacute;orie"
#elif L==6	// gn
	 "Programa teor&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	 "Programma teoria"
#elif L==8	// pl
	 "Syllabus (wyklady)"
#elif L==9	// pt
	 "Programa teoria"
#endif
	 ,
	 // Inf_PRACTICALS
#if   L==1	// ca
	 "Programa pr&agrave;ctiques"
#elif L==2	// de
	 "Studienplan (&Uuml;bungen)"
#elif L==3	// en
	 "Syllabus (practicals)"
#elif L==4	// es
	 "Programa pr&aacute;cticas"
#elif L==5	// fr
	 "Plan pratique"
#elif L==6	// gn
	 "Programa pr&aacute;cticas"	// Okoteve traduccin
#elif L==7	// it
	 "Programma pratica"
#elif L==8	// pl
	 "Syllabus (practicals)"
#elif L==9	// pt
	 "Programa pr&aacute;tica"
#endif
	 ,
	 // Inf_BIBLIOGRAPHY
#if   L==1	// ca
	 "Bibliografia"
#elif L==2	// de
	 "Literatur"
#elif L==3	// en
	 "Bibliography"
#elif L==4	// es
	 "Bibliograf&iacute;a"
#elif L==5	// fr
	 "Bibliographie"
#elif L==6	// gn
	 "Arandukapurupyre"
#elif L==7	// it
	 "Bibliografia"
#elif L==8	// pl
	 "Bibliografia"
#elif L==9	// pt
	 "Bibliografia"
#endif
	 ,
	 // Inf_FAQ
#if   L==1	// ca
	 "FAQ"
#elif L==2	// de
	 "FAQ"
#elif L==3	// en
	 "FAQ"
#elif L==4	// es
	 "FAQ"
#elif L==5	// fr
	 "FAQ"
#elif L==6	// gn
	 "FAQ"	// Okoteve traduccin
#elif L==7	// it
	 "FAQ"
#elif L==8	// pl
	 "FAQ"
#elif L==9	// pt
	 "FAQ"
#endif
	 ,
	 // Inf_LINKS
#if   L==1	// ca
	 "Enlla&ccedil;os"
#elif L==2	// de
	 "Links"
#elif L==3	// en
	 "Links"
#elif L==4	// es
	 "Enlaces"
#elif L==5	// fr
	 "Liens"
#elif L==6	// gn
	 "Ambu&eacute;re joaju"
#elif L==7	// it
	 "Link"
#elif L==8	// pl
	 "Linki"
#elif L==9	// pt
	 "Liga&ccedil;&otilde;es"
#endif
	 ,
	 // Inf_ASSESSMENT
#if   L==1	// ca
	 "Sistema d'avaluaci&oacute;"
#elif L==2	// de
	 "Bewertungssystem"
#elif L==3	// en
	 "Assessment system"
#elif L==4	// es
	 "Sistema de evaluaci&oacute;n"
#elif L==5	// fr
	 "Syst&egrave;me d'&eacute;valuation"
#elif L==6	// gn
	 "Sistema de evaluaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Sistema di valutazione"
#elif L==8	// pl
	 "System oceniania"
#elif L==9	// pt
	 "Sistema de avalia&ccedil;&atilde;o"
#endif
	};

const char *Txt_INFO_SRC_FULL_TEXT[Inf_NUM_INFO_SOURCES] =
   {
#if   L==1	// ca
	 "No ofrecer ninguna informaci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	 "Geben keine Auskunft"
#elif L==3	// en
	 "Do not give any information"
#elif L==4	// es
	 "No ofrecer ninguna informaci&oacute;n"
#elif L==5	// fr
	 "Ne pas donner d'information"
#elif L==6	// gn
	 "No ofrecer ninguna informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Non dare alcuna informazione"
#elif L==8	// pl
	 "Nie podawaj zadnych informacji"
#elif L==9	// pt
	 "N&atilde;o d&ecirc; nenhuma informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "Editor interno"	// Necessita traduccio
#elif L==2	// de
	 "Interner Editor"
#elif L==3	// en
	 "Internal editor"
#elif L==4	// es
	 "Editor interno"
#elif L==5	// fr
	 "&Eacute;diteur interne"
#elif L==6	// gn
	 "Editor interno"	// Okoteve traduccin
#elif L==7	// it
	 "Editor interno"
#elif L==8	// pl
	 "Wewnetrzny edytor"
#elif L==9	// pt
	 "Interno editor"
#endif
	 ,
#if   L==1	// ca
	 "Text sense format"
#elif L==2	// de
	 "Plain Text"
#elif L==3	// en
	 "Plain text"
#elif L==4	// es
	 "Texto sin formato"
#elif L==5	// fr
	 "Texte brut"
#elif L==6	// gn
	 "Texto sin formato"	// Okoteve traduccin
#elif L==7	// it
	 "Testo normale"
#elif L==8	// pl
	 "Zwykly tekst"
#elif L==9	// pt
	 "Texto plano"
#endif
	 ,
#if   L==1	// ca
	 "Text enriquit"
#elif L==2	// de
	 "Rich Text"
#elif L==3	// en
	 "Rich text"
#elif L==4	// es
	 "Texto enriquecido"
#elif L==5	// fr
	 "Texte enrichi"
#elif L==6	// gn
	 "Texto enriquecido"	// Okoteve traduccin
#elif L==7	// it
	 "Testo formattato"
#elif L==8	// pl
	 "Rich text"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Texto enriquecido"
#endif
	 ,
#if   L==1	// ca
	 "P&aacute;gina web alojada aqu&iacute;"	// Necessita traduccio
#elif L==2	// de
	 "Webseite gehostet hier"
#elif L==3	// en
	 "Web page hosted here"
#elif L==4	// es
	 "P&aacute;gina web alojada aqu&iacute;"
#elif L==5	// fr
	 "Page web h&eacute;berg&eacute;e ici"
#elif L==6	// gn
	 "P&aacute;gina web alojada aqu&iacute;"	// Okoteve traduccin
#elif L==7	// it
	 "Pagina web ospitata qui"
#elif L==8	// pl
	 "Strona internetowa hostowana tutaj"
#elif L==9	// pt
	 "P&aacute;gina Web hospedada aqui"
#endif
	 ,
#if   L==1	// ca
	 "P&aacute;gina web alojada en otro servidor"	// Necessita traduccio
#elif L==2	// de
	 "Webseite auf einem anderen Server gehostet"
#elif L==3	// en
	 "Web page hosted on another server"
#elif L==4	// es
	 "P&aacute;gina web alojada en otro servidor"
#elif L==5	// fr
	 "Page Web h&eacute;berg&eacute;e sur un autre serveur"
#elif L==6	// gn
	 "P&aacute;gina web alojada en otro servidor"	// Okoteve traduccin
#elif L==7	// it
	 "Pagina Web ospitata su un altro server"
#elif L==8	// pl
	 "Strona internetowa umieszczona na innym serwerze"
#elif L==9	// pt
	 "P&aacute;gina Web hospedada em outro servidor"
#endif
	};

const char *Txt_INFO_SRC_HELP[Inf_NUM_INFO_SOURCES] =
	{
	NULL,
#if   L==1	// ca
	"editor integrado, solo para los temarios de teor&iacute;a y pr&aacute;cticas"// Necessita traduccio
#elif L==2	// de
	"Interner Editor, nur f&uuml;r Studienplan (Vorlesung und &Uuml;bungen)"
#elif L==3	// en
	"internal editor, only for syllabus (lectures and practicals)"
#elif L==4	// es
	"editor integrado, solo para los temarios de teor&iacute;a y pr&aacute;cticas"
#elif L==5	// fr
	"&eacute;diteur interne, que pour les plans (th&eacute;orie et pratique)"
#elif L==6	// gn
	"editor integrado, solo para los temarios de teor&iacute;a y pr&aacute;cticas"// Okoteve traduccin
#elif L==7	// it
	"editor interno, solo per programmi di teoria e di pratica"
#elif L==8	// pl
	"Wewnetrzny edytor, tylko program (wyklady i practicals)"
#elif L==9	// pt
	"interno editor, apenas para programas de teoria y pr&aacute;tica"
#endif
	,
#if   L==1	// ca
	"editor de text sense format"
#elif L==2	// de
	"Plain Text Editor"
#elif L==3	// en
	"plain text editor"
#elif L==4	// es
	"editor de texto sin formato"
#elif L==5	// fr
	"&eacute;diteur de texte simple"
#elif L==6	// gn
	"editor de texto sin formato"		// Okoteve traduccin
#elif L==7	// it
	"editor di testo normale"
#elif L==8	// pl
	"edytor tekstu zwyk&lstrok;y"
#elif L==9	// pt
	"editor de texto simples"
#endif
	,
#if   L==1	// ca
	"editor de text enriquit"
#elif L==2	// de
	"Rich Text Editor"
#elif L==3	// en
	"rich text editor"
#elif L==4	// es
	"editor de texto enriquecido"
#elif L==5	// fr
	"&eacute;diteur de texte enrichi"
#elif L==6	// gn
	"editor de texto enriquecido"	// Okoteve traduccin
#elif L==7	// it
	"editor di testo formattato"
#elif L==8	// pl
	"edytor tekstu sformatowany"
#elif L==9	// pt
	"editor de texto rico"
#endif
	,
#if   L==1	// ca
	"env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"// Necessita traduccio
#elif L==2	// de
	"senden Sie eine HTML-Datei, oder als ZIP-Datei mit einer Datei index.html / index.htm und Bild-Dateien"
#elif L==3	// en
	"send an HTML file, or a ZIP file containing a file index.html / index.htm and image files"
#elif L==4	// es
	"env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"
#elif L==5	// fr
	"envoyer un fichier HTML, ou un fichier ZIP contenant un fichier index.html / index.htm et les fichiers d'images"
#elif L==6	// gn
	"env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"// Okoteve traduccin
#elif L==7	// it
	"inviare un file HTML, o un file ZIP contenente un file index.html / index.htm e file di immagine"
#elif L==8	// pl
	"wyslac plik HTML lub plik ZIP zawierajacy plik index.html / index.htm i pliki graficzne"
#elif L==9	// pt
	"enviar um arquivo HTML, ou um arquivo ZIP que cont&eacute;m um arquivo index.html / index.htm e limas da imagem"
#endif
	,
#if   L==1	// ca
	"escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"	// Necessita traduccio
#elif L==2	// de
	"Geben Sie die Adresse einer Web-Seite geleitet werden, der sich auf einem anderen Server"
#elif L==3	// en
	"enter the address of a web page that is hosted on another server"
#elif L==4	// es
	"escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"
#elif L==5	// fr
	"entrer l'adresse d'une page Web qui est h&eacute;berg&eacute;e sur un autre serveur"
#elif L==6	// gn
	"escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"	// Okoteve traduccin
#elif L==7	// it
	"immettere l'indirizzo di una pagina web che &egrave; ospitata su un altro server"
#elif L==8	// pl
	"wprowadzic adres strony internetowej, kt&oacute;ra znajduje sie na innym serwerze"
#elif L==9	// pt
	"introduza o endere&ccedil;o de uma p&aacute;gina Web que est&aacute; hospedada em outro servidor"
#endif
	};

const char *Txt_INFO_SRC_SHORT_TEXT[Inf_NUM_INFO_SOURCES] =
   {
#if   L==1	// ca
	 "Ninguna informaci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	 "Keine Angaben"
#elif L==3	// en
	 "No information"
#elif L==4	// es
	 "Ninguna informaci&oacute;n"
#elif L==5	// fr
	 "Aucune information"
#elif L==6	// gn
	 "Ninguna informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Nessuna informazione"
#elif L==8	// pl
	 "Brak informacji"
#elif L==9	// pt
	 "Sem informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "Editor interno"	// Necessita traduccio
#elif L==2	// de
	 "Interner Editor"
#elif L==3	// en
	 "Internal editor"
#elif L==4	// es
	 "Editor interno"
#elif L==5	// fr
	 "&Eacute;diteur interne"
#elif L==6	// gn
	 "Editor interno"	// Okoteve traduccin
#elif L==7	// it
	 "Editor interno"
#elif L==8	// pl
	 "Wewnetrzny edytor"
#elif L==9	// pt
	 "Interno editor"
#endif
	 ,
#if   L==1	// ca
	 "Text sense format"
#elif L==2	// de
	 "Plain Text"
#elif L==3	// en
	 "Plain text"
#elif L==4	// es
	 "Texto sin formato"
#elif L==5	// fr
	 "Plain text"
#elif L==6	// gn
	 "Texto sin formato"	// Okoteve traduccin
#elif L==7	// it
	 "Testo normale"
#elif L==8	// pl
	 "Zwykly tekst"
#elif L==9	// pt
	 "Texto plano"
#endif
	 ,
#if   L==1	// ca
	 "Text enriquit"
#elif L==2	// de
	 "Rich Text"
#elif L==3	// en
	 "Rich text"
#elif L==4	// es
	 "Texto enriquecido"
#elif L==5	// fr
	 "Texte enrichi"
#elif L==6	// gn
	 "Texto enriquecido"	// Okoteve traduccin
#elif L==7	// it
	 "Testo formattato"
#elif L==8	// pl
	 "Rich text"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Texto enriquecido"
#endif
	 ,
#if   L==1	// ca
	 "P&aacute;gina interna"	// Necessita traduccio
#elif L==2	// de
	 "Interne Webseite"
#elif L==3	// en
	 "Internal page"
#elif L==4	// es
	 "P&aacute;gina interna"
#elif L==5	// fr
	 "Page interne"
#elif L==6	// gn
	 "P&aacute;gina interna"	// Okoteve traduccin
#elif L==7	// it
	 "Pagina interna"
#elif L==8	// pl
	 "Strona wewnetrzna"
#elif L==9	// pt
	 "Interna p&aacute;gina"
#endif
	 ,
#if   L==1	// ca
	 "P&aacute;gina externa"	// Necessita traduccio
#elif L==2	// de
	 "Externe Webseite"
#elif L==3	// en
	 "External page"
#elif L==4	// es
	 "P&aacute;gina externa"
#elif L==5	// fr
	 "Page externe"
#elif L==6	// gn
	 "P&aacute;gina externa"	// Okoteve traduccin
#elif L==7	// it
	 "Pagina esterna"
#elif L==8	// pl
	 "Strona zewnetrzna"
#elif L==9	// pt
	 "Externa p&aacute;gina"
#endif
	};

const char *Txt_Informants =
#if   L==1	// ca
	"Informadors";
#elif L==2	// de
	"Informanten";
#elif L==3	// en
	"Informants";
#elif L==4	// es
	"Informantes";
#elif L==5	// fr
	"Informateurs";
#elif L==6	// gn
	"Informantes";	// Okoteve traduccin
#elif L==7	// it
	"Informatori";
#elif L==8	// pl
	"informator&oacute;w";
#elif L==9	// pt
	"Informantes";
#endif

const char *Txt_Inline_math =
#if   L==1	// ca
	"F&oacute;rmula";
#elif L==2	// de
	"Inline Mathematik";
#elif L==3	// en
	"Inline math";
#elif L==4	// es
	"F&oacute;rmula";
#elif L==5	// fr
	"Formule";
#elif L==6	// gn
	"F&oacute;rmula";	// Okoteve traduccin
#elif L==7	// it
	"Matematica inline";
#elif L==8	// pl
	"Jazda matematyki";
#elif L==9	// pt
	"F&oacute;rmula";
#endif

const char *Txt_Institution =
#if   L==1	// ca
	"Instituci&oacute;";
#elif L==2	// de
	"Hochschule";
#elif L==3	// en
	"Institution";
#elif L==4	// es
	"Instituci&oacute;n";
#elif L==5	// fr
	"&Eacute;tablissement";
#elif L==6	// gn
	"Mbo'ehao";
#elif L==7	// it
	"Istituzione";
#elif L==8	// pl
	"Instytucji";
#elif L==9	// pt
	"Institu&ccedil;&atilde;o";
#endif

const char *Txt_institution =
#if   L==1	// ca
	"instituci&oacute;";
#elif L==2	// de
	"Hochschule";
#elif L==3	// en
	"institution";
#elif L==4	// es
	"instituci&oacute;n";
#elif L==5	// fr
	"&eacute;tablissement";
#elif L==6	// gn
	"mbo'ehao";
#elif L==7	// it
	"istituzione";
#elif L==8	// pl
	"instytucji";
#elif L==9	// pt
	"institu&ccedil;&atilde;o";
#endif

const char *Txt_Institution_centre_and_department =
#if   L==1	// ca
	"Instituci&oacute;, centre i departament";
#elif L==2	// de
	"Hochschule, Lehrinstitut und Abteilung";
#elif L==3	// en
	"Institution, centre and department";
#elif L==4	// es
	"Instituci&oacute;n, centro y departamento";
#elif L==5	// fr
	"&Eacute;tablissement, centre et d&eacute;partement";
#elif L==6	// gn
	"Mbo'ehao, mbo'ehao, peh&#7869;";
#elif L==7	// it
	"Istituzione, centro e dipartimento";
#elif L==8	// pl
	"Instytucji, centrum i wydzia&lstrok;";
#elif L==9	// pt
	"Institu&ccedil;&atilde;o, centro e departamento";
#endif

const char *Txt_Institution_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Instituci&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2	// de
	"Hochschule <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Institution <strong>%s</strong> removed.";
#elif L==4	// es
	"Instituci&oacute;n <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Institution <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6	// gn
	"Instituci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Istituzione <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Instytucja <strong>%s</strong> usunieta.";
#elif L==9	// pt
	"Institui&ccedil;&atilde;o <strong>%s</strong> removida.";
#endif


const char *Txt_INSTITUTION_STATUS[Ins_NUM_STATUS_TXT] =
   {
#if   L==1	// ca	// Ins_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==2	// de
	 "Unbekannt"
#elif L==3	// en
	 "Unknown"
#elif L==4	// es
	 "Desconocido"
#elif L==5	// fr
	 "Inconnu"
#elif L==6	// gn
	 "Desconocido"			// Okoteve traduccin
#elif L==7	// it
	 "Sconosciuto"
#elif L==8	// pl
	 "Nieznany"
#elif L==9	// pt
	 "Desconhecido"
#endif
	 ,
#if   L==1	// ca	// Ins_STATUS_ACTIVE
	 "Activa"
#elif L==2	// de
	 "Aktive"
#elif L==3	// en
	 "Active"
#elif L==4	// es
	 "Activa"
#elif L==5	// fr
	 "Actif"
#elif L==6	// gn
	 "Activa"	// Okoteve traduccin
#elif L==7	// it
	 "Attiva"
#elif L==8	// pl
	 "Aktywne"
#elif L==9	// pt
	 "Ativa"
#endif
	 ,
#if   L==1	// ca	// Ins_STATUS_PENDING
	 "Nova"
#elif L==2	// de
	 "Neu"
#elif L==3	// en
	 "New"
#elif L==4	// es
	 "Nueva"
#elif L==5	// fr
	 "Nouvelle"
#elif L==6	// gn
	 "Pyahu"
#elif L==7	// it
	 "Nuovo"
#elif L==8	// pl
	 "Nowy"
#elif L==9	// pt
	 "Nova"
#endif
	 ,
#if   L==1	// ca	// Ins_STATUS_REMOVED
	 "Eliminada"
#elif L==2	// de
	 "Entfernt"
#elif L==3	// en
	 "Removed"
#elif L==4	// es
	 "Eliminada"
#elif L==5	// fr
	 "Supprim&eacute;"
#elif L==6	// gn
	 "Eliminada"	// Okoteve traduccin
#elif L==7	// it
	 "Rimossa"
#elif L==8	// pl
	 "Usuniete"
#elif L==9	// pt
	 "Suprimida"
#endif
	};

const char *Txt_Institutional_code =
#if   L==1	// ca
	"Codi institucional";
#elif L==2	// de
	"Institutionelle Code";
#elif L==3	// en
	"Institutional code";
#elif L==4	// es
	"C&oacute;digo institucional";
#elif L==5	// fr
	"Code institutionnel";
#elif L==6	// gn
	"C&oacute;digo institucional";	// Okoteve traduccin
#elif L==7	// it
	"Codice istituzionale";
#elif L==8	// pl
	"Kod instytucjonalna";
#elif L==9	// pt
	"C&oacute;digo institutional";
#endif

const char *Txt_Institutional_BR_code =
#if   L==1	// ca
	"Codi<br />instituc.";
#elif L==2	// de
	"Institutionelle<br />Code";
#elif L==3	// en
	"Institut.<br />code";
#elif L==4	// es
	"C&oacute;digo<br />instituc.";
#elif L==5	// fr
	"Code<br />institut.";
#elif L==6	// gn
	"C&oacute;digo<br />instituc.";	// Okoteve traduccin
#elif L==7	// it
	"Codice<br />istituz.";
#elif L==8	// pl
	"Kod<br />instytuc.";
#elif L==9	// pt
	"C&oacute;digo<br />instituc.";
#endif

const char *Txt_Institutions =
#if   L==1	// ca
	"Institucions";
#elif L==2	// de
	"Hochschulen";
#elif L==3	// en
	"Institutions";
#elif L==4	// es
	"Instituciones";
#elif L==5	// fr
	"&Eacute;tablissements";
#elif L==6	// gn
	"Mbo'ehao";
#elif L==7	// it
	"Istituzioni";
#elif L==8	// pl
	"Instytucje";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es";
#endif

const char *Txt_Institutions_ABBREVIATION =
#if   L==1	// ca
	"Instit.";
#elif L==2	// de
	"Hochsc.";
#elif L==3	// en
	"Instit.";
#elif L==4	// es
	"Instit.";
#elif L==5	// fr
	"&Eacute;tabl.";
#elif L==6	// gn
	"Mbo'ehao";
#elif L==7	// it
	"Istit.";
#elif L==8	// pl
	"Instyt.";
#elif L==9	// pt
	"Instit.";
#endif

const char *Txt_institutions =
#if   L==1	// ca
	"institucions";
#elif L==2	// de
	"Hochschulen";
#elif L==3	// en
	"institutions";
#elif L==4	// es
	"instituciones";
#elif L==5	// fr
	"&eacute;tablissements";
#elif L==6	// gn
	"mbo'ehao";
#elif L==7	// it
	"istituzioni";
#elif L==8	// pl
	"instytucje";
#elif L==9	// pt
	"institu&ccedil;&otilde;es";
#endif

const char *Txt_INSTITUTIONS_HELP_ORDER[Ins_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Ordenar por instituci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	 "Nach Hochschule sortieren"
#elif L==3	// en
	 "Sort by institution"
#elif L==4	// es
	 "Ordenar por instituci&oacute;n"
#elif L==5	// fr
	 "Trier par &eacute;tablissement"
#elif L==6	// gn
	 "Ordenar por instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per istituzione"
#elif L==8	// pl
	 "Sortuj wedlug instytucji"
#elif L==9	// pt
	 "Classificar por institui&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar por n&ordm; de usuarios"	// Necessita traduccio
#elif L==2	// de
	 "Nach Anzahl der Benutzer sortieren"
#elif L==3	// en
	 "Sort by no. of users"
#elif L==4	// es
	 "Ordenar por n&ordm; de usuarios"
#elif L==5	// fr
	 "Trier par nombre d'utilisateurs"
#elif L==6	// gn
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per numero di utenti"
#elif L==8	// pl
	 "Sortowanie wedlug liczby uzytkownikow"
#elif L==9	// pt
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_Institutions_by_number_of_centres =
#if   L==1	// ca
	"Institucions segons nombre de centres";
#elif L==2	// de
	"Hochschulen nach der Anzahl der Lehrinstitute";
#elif L==3	// en
	"Institutions by number of centres";
#elif L==4	// es
	"Instituciones seg&uacute;n n&uacute;mero de centros";
#elif L==5	// fr
	"&Eacute;tablissements par nombre de centres";
#elif L==6	// gn
	"Instituciones seg&uacute;n n&uacute;mero de centros";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni per numero di centri";
#elif L==8	// pl
	"Instytucje wed&lstrok;ug liczby o&sacute;rodk&oacute;w";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es por n&uacute;mero de centros";
#endif

const char *Txt_Institutions_by_number_of_degrees =
#if   L==1	// ca
	"Institucions segons nombre de titulacions";
#elif L==2	// de
	"Hochschulen nach der Anzahl der Studiengang";
#elif L==3	// en
	"Institutions by number of degrees";
#elif L==4	// es
	"Instituciones seg&uacute;n n&uacute;mero de titulaciones";
#elif L==5	// fr
	"&Eacute;tablissements par nombre d'&eacute;tudes";
#elif L==6	// gn
	"Instituciones seg&uacute;n n&uacute;mero de titulaciones";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni per numero di lauree";
#elif L==8	// pl
	"Instytucje wed&lstrok;ug liczby stopni";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es por n&uacute;mero de graus";
#endif

const char *Txt_Institutions_by_number_of_courses =
#if   L==1	// ca
	"Institucions segons nombre de assignatures";
#elif L==2	// de
	"Hochschulen nach der Anzahl der Kursen";
#elif L==3	// en
	"Institutions by number of courses";
#elif L==4	// es
	"Instituciones seg&uacute;n n&uacute;mero de asignaturas";
#elif L==5	// fr
	"&Eacute;tablissements par nombre de mati&egrave;res";
#elif L==6	// gn
	"Instituciones seg&uacute;n n&uacute;mero de asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni per numero di corsi";
#elif L==8	// pl
	"Instytucje wed&lstrok;ug liczby kurs&otilde;w";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es por n&uacute;mero de disciplinas";
#endif

const char *Txt_Institutions_by_number_of_users_in_courses =
#if   L==1	// ca
	"Institucions segons nombre d'usuaris en assignatures";
#elif L==2	// de
	"Hochschulen nach der Anzahl der Benutzer in Kurse";
#elif L==3	// en
	"Institutions by number of users in courses";
#elif L==4	// es
	"Instituciones seg&uacute;n n&uacute;mero de usuarios en asignaturas";
#elif L==5	// fr
	"&Eacute;tablissements par nombre d'utilisateurs dans mati&egrave;res";
#elif L==6	// gn
	"Instituciones seg&uacute;n n&uacute;mero de usuarios en asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni per numero di utenti in corsi";
#elif L==8	// pl
	"Instytucje wed&lstrok;ug liczby u&zdot;ytkownik&oacute;w w kurs&otilde;w";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es por n&uacute;mero de usu&aacute;rios em disciplinas";
#endif

const char *Txt_Institutions_by_number_of_users_who_claim_to_belong_to_them =
#if   L==1	// ca
	"Institucions segons nombre d'usuaris que diuen pert&agrave;nyer-hi";
#elif L==2	// de
	"Hochschulen nach der Anzahl der Benutzer, die behaupten, zu ihnen geh&ouml;ren";
#elif L==3	// en
	"Institutions by number of users who claim to belong to them";
#elif L==4	// es
	"Instituciones seg&uacute;n n&uacute;mero de usuarios que dicen pertenecer a ellas";
#elif L==5	// fr
	"&Eacute;tablissements par nombre d'utilisateurs qui pr&eacute;tendent leur appartiennent";
#elif L==6	// gn
	"Instituciones seg&uacute;n n&uacute;mero de usuarios que dicen pertenecer a ellas";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni per numero di utenti che affermano di appartenere a loro";
#elif L==8	// pl
	"Instytucje wed&lstrok;ug liczby u&zdot;ytkownik&oacute;w, kt&oacute;rzy twierdz&aogon;, &zdot;e nale&zdot;&aogon; do nich";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es por n&uacute;mero de usu&aacute;rios que afirmam pertencer a elas";
#endif

const char *Txt_Institutions_of_COUNTRY_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Institucions de %s";
#elif L==2	// de
	"Hochschulen von %s";
#elif L==3	// en
	"Institutions of %s";
#elif L==4	// es
	"Instituciones de %s";
#elif L==5	// fr
	"&Eacute;tablissements de %s";
#elif L==6	// gn
	"Instituciones de %s";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni di %s";
#elif L==8	// pl
	"Instytucje %s";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es de %s";
#endif

const char *Txt_INSTITUTIONS_ORDER[Ins_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Instituci&oacute;"
#elif L==2	// de
	 "Hochschule"
#elif L==3	// en
	 "Institution"
#elif L==4	// es
	 "Instituci&oacute;n"
#elif L==5	// fr
	 "&Eacute;tablissement"
#elif L==6	// gn
	 "Mbo'ehao"
#elif L==7	// it
	 "Istituzione"
#elif L==8	// pl
	 "Instytucji"
#elif L==9	// pt
	 "Institui&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "Usuaris de la instituci&oacute;"
#elif L==2	// de
	 "Benutzer der Hochschule"
#elif L==3	// en
	 "Users of the institution"
#elif L==4	// es
	 "Usuarios de la instituci&oacute;n"
#elif L==5	// fr
	 "Utilisateurs du &eacute;tablissement"
#elif L==6	// gn
	 "Usuarios de la instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Utenti della istituzione"
#elif L==8	// pl
	 "U&zdot;ytkownicy instytucji"
#elif L==9	// pt
	 "Utilizadores da institu&ccedil;&atilde;o"
#endif
	};

const char *Txt_Institutions_with_pending_centres =
#if   L==1	// ca
	"Institucions amb centres pendents d'activar";
#elif L==2	// de
	"Hochschulen mit anh&auml;ngig Lehrinstitute";
#elif L==3	// en
	"Institutions with pending centres";
#elif L==4	// es
	"Instituciones con centros pendientes de activar";
#elif L==5	// fr
	"&Eacute;tablissements avec des centres en attente";
#elif L==6	// gn
	"Instituciones con centros pendientes de activar";	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni con centri in sospeso";
#elif L==8	// pl
	"Instytucji z centra oczekuj&aogon;cy";
#elif L==9	// pt
	"Institu&ccedil;&otilde;es com centros pendentes";
#endif

const char *Txt_Integer_number =
#if   L==1	// ca
	"Nombre enter";
#elif L==2	// de
	"Ganze Zahl";
#elif L==3	// en
	"Integer number";
#elif L==4	// es
	"N&ordm; entero";
#elif L==5	// fr
	"Nombre entier";
#elif L==6	// gn
	"N&ordm; entero";	// Okoteve traduccin
#elif L==7	// it
	"Numero intero";
#elif L==8	// pl
	"Liczba calkowita";
#elif L==9	// pt
	"N&ordm; inteiro";
#endif

const char *Txt_Internal_code =
#if   L==1	// ca
	"Codi intern";
#elif L==2	// de
	"Interner Code";
#elif L==3	// en
	"Internal code";
#elif L==4	// es
	"C&oacute;digo interno";
#elif L==5	// fr
	"Code interne";
#elif L==6	// gn
	"C&oacute;digo interno";	// Okoteve traduccin
#elif L==7	// it
	"Codice interno";
#elif L==8	// pl
	"Kod wewn&eogon;trzny";
#elif L==9	// pt
	"C&oacute;digo interno";
#endif

const char *Txt_Internationalization =
#if   L==1	// ca
	"Internacionalitzaci&oacute;";
#elif L==2	// de
	"Internationalisierung";
#elif L==3	// en
	"Internationalization";
#elif L==4	// es
	"Internacionalizaci&oacute;n";
#elif L==5	// fr
	"Internationalisation";
#elif L==6	// gn
	"Internacionalizaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Internazionalizzazione";
#elif L==8	// pl
	"Umi&eogon;dzynarodowienie";
#elif L==9	// pt
	"Internacionaliza&ccedil;&atilde;o";
#endif

const char *Txt_IP =		// Internet Protocol address
#if   L==1	// ca
	"IP";
#elif L==2	// de
	"IP";
#elif L==3	// en
	"IP";
#elif L==4	// es
	"IP";
#elif L==5	// fr
	"IP";
#elif L==6	// gn
	"IP";	// Okoteve traduccin
#elif L==7	// it
	"IP";
#elif L==8	// pl
	"IP";
#elif L==9	// pt
	"IP";
#endif

const char *Txt_It_is_mandatory_to_choose_a_group =
#if   L==1	// ca
	"Elegir grupo es obligatorio";		// Necessita traduccio
#elif L==2	// de
	"Es muss eine Gruppe ausgew&auml;hlt werden";
#elif L==3	// en
	"It is mandatory to choose a group";
#elif L==4	// es
	"Elegir grupo es obligatorio";
#elif L==5	// fr
	"Il est obligatoire de choisir un groupe";
#elif L==6	// gn
	"Elegir grupo es obligatorio";		// Okoteve traduccin
#elif L==7	// it
	"E 'obbligatorio scegliere un gruppo";
#elif L==8	// pl
	"Jest to obowiazkowe, aby wybrac grupe";
#elif L==9	// pt
	"Escolher um grupo &eacute; obrigat&oacute;rio";
#endif

const char *Txt_It_is_optional_to_choose_a_group =
#if   L==1	// ca
	"Elegir grupo es voluntario";		// Necessita traduccio
#elif L==2	// de
	"Die Ausw&auml;hl einer Gruppe ist freiwillig";
#elif L==3	// en
	"It is optional to choose a group";
#elif L==4	// es
	"Elegir grupo es voluntario";
#elif L==5	// fr
	"Il est volontaire de choisir un groupe";
#elif L==6	// gn
	"Elegir grupo es voluntario";		// Okoteve traduccin
#elif L==7	// it
	"E' facoltativo scegliere un gruppo";
#elif L==8	// pl
	"Opcjonalnie mozna wybrac grupe";
#elif L==9	// pt
	"Escolher um grupo &eacute; volunt&aacute;rio";
#endif

const char *Txt_Its_me =
#if   L==1	// ca
	"S&oacute;c jo!";
#elif L==2	// de
	"Da ich bin!";
#elif L==3	// en
	"It's me!";
#elif L==4	// es
	"&iexcl;Soy yo!";
#elif L==5	// fr
	"C'est moi!";
#elif L==6	// gn
	"&iexcl;Soy yo!";	// Okoteve traduccin
#elif L==7	// it
	"Sono io!";
#elif L==8	// pl
	"To ja!";
#elif L==9	// pt
	"Sou eu!";
#endif

const char *Txt_Language =
#if   L==1	// ca
	"Idioma";
#elif L==2	// de
	"Sprache";
#elif L==3	// en
	"Language";
#elif L==4	// es
	"Idioma";
#elif L==5	// fr
	"Langue";
#elif L==6	// gn
	"&Ntilde;e'&#7869;";
#elif L==7	// it
	"Lingua";
#elif L==8	// pl
	"J&eogon;zyk";
#elif L==9	// pt
	"Lingua";
#endif

const char *Txt_Last_clicks =
#if   L==1	// ca
	"&Uacute;ltims clics";
#elif L==2	// de
	"Letzte Klicks";
#elif L==3	// en
	"Last clicks";
#elif L==4	// es
	"&Uacute;ltimos clics";
#elif L==5	// fr
	"Derni&egrave;re clics";
#elif L==6	// gn
	"&Uacute;ltimos clics";	// Okoteve traduccin
#elif L==7	// it
	"Ultimi clic";
#elif L==8	// pl
	"Ostatnie dwa klikni&eogon;cia";
#elif L==9	// pt
	"&Uacute;ltimos cliques";
#endif

const char *Txt_Last_clicks_in_real_time =
#if   L==1	// ca
	"&Uacute;ltims clics en temps real";
#elif L==2	// de
	"Letzte Klicks in Echtzeit";
#elif L==3	// en
	"Last clicks in real time";
#elif L==4	// es
	"&Uacute;ltimos clics en tiempo real";
#elif L==5	// fr
	"Derni&egrave;re clics en temps r&eacute;el";
#elif L==6	// gn
	"&Uacute;ltimos clics en tiempo real";	// Okoteve traduccin
#elif L==7	// it
	"Ultimi clic in tempo reale";
#elif L==8	// pl
	"Ostatnie dwa klikni&eogon;cia w czasie rzeczywistym";
#elif L==9	// pt
	"&Uacute;ltimos cliques em tempo real";
#endif

const char *Txt_LAYOUT_SIDE_COLUMNS[4] =
   {
#if   L==1	// ca
	 "Solo zona central"	// Necessita traduccio
#elif L==2	// de
	 "Nur mittleren Bereich anzeigen"
#elif L==3	// en
	 "Display central area only"
#elif L==4	// es
	 "Solo zona central"
#elif L==5	// fr
	 "Display central area only"	// Besoin de traduction
#elif L==6	// gn
	 "Solo zona central"	// Okoteve traduccin
#elif L==7	// it
	 "Mostra solo la parte centrale"
#elif L==8	// pl
	 "Wyswietl centralnej czesci tylko"
#elif L==9	// pt
	 "Somente &aacute;rea central"
#endif
	 ,
#if   L==1	// ca
	 "Columna derecha visible"	// Necessita traduccio
#elif L==2	// de
	 "Rechte Spalte anzeigen"
#elif L==3	// en
	 "Display right column"
#elif L==4	// es
	 "Columna derecha visible"
#elif L==5	// fr
	 "Display right column"		// Besoin de traduction
#elif L==6	// gn
	 "Columna derecha visible"	// Okoteve traduccin
#elif L==7	// it
	 "Mostra la colonna destra"
#elif L==8	// pl
	 "Wyswietl prawa kolumne"
#elif L==9	// pt
	 "Coluna direita vis&iacute;vel"
#endif
	 ,
#if   L==1	// ca
	 "Columna izquierda visible"	// Necessita traduccio
#elif L==2	// de
	 "Linke Spalte anzeigen"
#elif L==3	// en
	 "Display left column"
#elif L==4	// es
	 "Columna izquierda visible"
#elif L==5	// fr
	 "Display left column"		// Besoin de traduction
#elif L==6	// gn
	 "Columna izquierda visible"	// Okoteve traduccin
#elif L==7	// it
	 "Mostra la colonna sinistra"
#elif L==8	// pl
	 "Wyswietl lewa kolumne"
#elif L==9	// pt
	 "Coluna esquerda vis&iacute;vel"
#endif
	 ,
#if   L==1	// ca
	 "Columnas izquierda y derecha visibles"	// Necessita traduccio
#elif L==2	// de
	 "Linke und rechte Spalte anzeigen"
#elif L==3	// en
	 "Display left and right columns"
#elif L==4	// es
	 "Columnas izquierda y derecha visibles"
#elif L==5	// fr
	 "Display left and right columns"		// Besoin de traduction
#elif L==6	// gn
	 "Columnas izquierda y derecha visibles"	// Okoteve traduccin
#elif L==7	// it
	 "Mostra la colonna sinistra e destra"
#elif L==8	// pl
	 "Wyswietl lewa i prawa kolumne"
#elif L==9	// pt
	 "Colunas esquerda e direita vis&iacute;veis"
#endif
	};

const char *Txt_level =
#if   L==1	// ca
	"nivel";	// Necessita traduccio
#elif L==2	// de
	"Ebene";
#elif L==3	// en
	"level";
#elif L==4	// es
	"nivel";
#elif L==5	// fr
	"niveau";
#elif L==6	// gn
	"nivel";	// Okoteve traduccin
#elif L==7	// it
	"livello";
#elif L==8	// pl
	"poziom";
#elif L==9	// pt
	"n&iacute;vel";
#endif

const char *Txt_Levels =
#if   L==1	// ca
	"Niveles";	// Necessita traduccio
#elif L==2	// de
	"Ebenen";
#elif L==3	// en
	"Levels";
#elif L==4	// es
	"Niveles";
#elif L==5	// fr
	"Niveaux";
#elif L==6	// gn
	"Niveles";	// Okoteve traduccin
#elif L==7	// it
	"Livelli";
#elif L==8	// pl
	"Poziomy";
#elif L==9	// pt
	"N&iacute;veis";
#endif

const char *Txt_levels =
#if   L==1	// ca
	"niveles";	// Necessita traduccio
#elif L==2	// de
	"Ebenen";
#elif L==3	// en
	"levels";
#elif L==4	// es
	"niveles";
#elif L==5	// fr
	"niveaux";
#elif L==6	// gn
	"niveles";	// Okoteve traduccin
#elif L==7	// it
	"livelli";
#elif L==8	// pl
	"poziomy";
#elif L==9	// pt
	"n&iacute;veis";
#endif

const char *Txt_License =
#if   L==1	// ca
	"Llic&egrave;ncia";
#elif L==2	// de
	"Lizenz";
#elif L==3	// en
	"License";
#elif L==4	// es
	"Licencia";
#elif L==5	// fr
	"Licence";
#elif L==6	// gn
	"Licencia";	// Okoteve traduccin
#elif L==7	// it
	"Licenza";
#elif L==8	// pl
	"Pozwolenie";
#elif L==9	// pt
	"Licen&ccedil;a";
#endif

const char *Txt_LICENSES[Brw_NUM_LICENSES] =
	{
#if   L==1	// ca
	"desconeguda / altra"
#elif L==2	// de
	"Unbekannte / sonstige"
#elif L==3	// en
	"unknown / other"
#elif L==4	// es
	"desconocida / otra"
#elif L==5	// fr
	"inconnue / autre"
#elif L==6	// gn
	"desconocida / otra"			// Okoteve traduccin
#elif L==7	// it
	"sconosciuta / altri"
#elif L==8	// pl
	"nieznana / inny"
#elif L==9	// pt
	"desconhecida / outra"
#endif
	,
#if   L==1	// ca
	"Tots els drets reservats"		// All Rights Reserved
#elif L==2	// de
	"Alle Rechte vorbehalten"		// All Rights Reserved
#elif L==3	// en
	"All rights reserved"			// All Rights Reserved
#elif L==4	// es
	"Todos los derechos reservados"		// All Rights Reserved
#elif L==5	// fr
	"Tous droits r&eacute;serv&eacute;s"	// All Rights Reserved
#elif L==6	// gn
	"Todos los derechos reservados"		// All Rights Reserved	// Okoteve traduccin
#elif L==7	// it
	"Tutti i diritti riservati"		// All Rights Reserved
#elif L==8	// pl
	"Wszelkie prawa zastrze&zdot;one"	// All Rights Reserved
#elif L==9	// pt
	"Todos os direitos reservados"		// All Rights Reserved
#endif
	,
#if   L==1	// ca
	"CC Reconeixement"			// CC Attribution License
#elif L==2	// de
	"CC Namensnennung"			// CC Attribution License
#elif L==3	// en
	"CC Attribution"			// CC Attribution License
#elif L==4	// es
	"CC Reconocimiento"			// CC Attribution License
#elif L==5	// fr
	"CC Attribution"			// CC Attribution License
#elif L==6	// gn
	"CC Reconocimiento"			// CC Attribution License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione"			// CC Attribution License
#elif L==8	// pl
	"CC Uznanie autorstwa"			// CC Attribution License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o"		// CC Attribution License
#endif
	,
#if   L==1	// ca
	"CC Reconeixement - Compartir igual"				// CC Attribution-ShareAlike License
#elif L==2	// de
	"CC Namensnennung - Weitergabe unter gleichen Bedingungen"	// CC Attribution-ShareAlike License
#elif L==3	// en
	"CC Attribution - Share Alike"					// CC Attribution-ShareAlike License
#elif L==4	// es
	"CC Reconocimiento - Compartir bajo la misma licencia"		// CC Attribution-ShareAlike License
#elif L==5	// fr
	"CC Attribution - Partage &agrave; l'Identique"			// CC Attribution-ShareAlike License
#elif L==6	// gn
	"CC Reconocimiento - Compartir bajo la misma licencia"		// CC Attribution-ShareAlike License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione - Condividi allo stesso modo"			// CC Attribution-ShareAlike License
#elif L==8	// pl
	"CC Uznanie autorstwa - Na tych samych warunkach"		// CC Attribution-ShareAlike License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o - Compartilha Igual"		// CC Attribution-ShareAlike License
#endif
	,
#if   L==1	// ca
	"CC Reconeixement - Sense obra derivada"				// CC Attribution-NoDerivs License
#elif L==2	// de
	"CC Namensnennung - Keine Bearbeitung"					// CC Attribution-NoDerivs License
#elif L==3	// en
	"CC Attribution - No Derivative Works"					// CC Attribution-NoDerivs License
#elif L==4	// es
	"CC Reconocimiento - Sin obras derivadas"				// CC Attribution-NoDerivs License
#elif L==5	// fr
	"CC Attribution - Pas de travaux d&eacute;riv&eacute;s"			// CC Attribution-NoDerivs License
#elif L==6	// gn
	"CC Reconocimiento - Sin obras derivadas"				// CC Attribution-NoDerivs License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione - Non opere derivate"					// CC Attribution-NoDerivs License
#elif L==8	// pl
	"CC Uznanie autorstwa - Bez utwor&oacute;w zale&zdot;nych"		// CC Attribution-NoDerivs License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o - Trabalhos Derivados Pro&iacute;bidos"	// CC Attribution-NoDerivs License
#endif
	,
#if   L==1	// ca
	"CC Reconeixement - No comercial"			// CC Attribution-NonCommercial License
#elif L==2	// de
	"CC Namensnennung - Keine kommerzielle Nutzung"		// CC Attribution-NonCommercial License
#elif L==3	// en
	"CC Attribution - Noncommercial"			// CC Attribution-NonCommercial License
#elif L==4	// es
	"CC Reconocimiento - No comercial"			// CC Attribution-NonCommercial License
#elif L==5	// fr
	"CC Attribution - Pas d'Utilisation Commerciale"	// CC Attribution-NonCommercial License
#elif L==6	// gn
	"CC Reconocimiento - No comercial"			// CC Attribution-NonCommercial License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione - Non commerciale"			// CC Attribution-NonCommercial License
#elif L==8	// pl
	"CC Uznanie autorstwa - U&zdot;ycie niekomercyjne"	// CC Attribution-NonCommercial License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o - N&atilde;oComercial"	// CC Attribution-NonCommercial License
#endif
	,
#if   L==1	// ca
	"CC Reconeixement - No comercial - Compartir igual"					// CC Attribution-NonCommercial-ShareAlike License
#elif L==2	// de
	"CC Namensnennung - Keine kommerzielle Nutzung - Weitergabe unter gleichen Bedingungen"	// CC Attribution-NonCommercial-ShareAlike License
#elif L==3	// en
	"CC Attribution - Noncommercial - Share Alike"						// CC Attribution-NonCommercial-ShareAlike License
#elif L==4	// es
	"CC Reconocimiento - No comercial - Compartir bajo la misma licencia"			// CC Attribution-NonCommercial-ShareAlike License
#elif L==5	// fr
	"CC Attribution - Pas d'Utilisation Commerciale - Partage &agrave; l'Identique"		// CC Attribution-NonCommercial-ShareAlike License
#elif L==6	// gn
	"CC Reconocimiento - No comercial - Compartir bajo la misma licencia"			// CC Attribution-NonCommercial-ShareAlike License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione - Non commerciale - Condividi allo stesso modo"			// CC Attribution-NonCommercial-ShareAlike License
#elif L==8	// pl
	"CC Uznanie autorstwa - U&zdot;ycie niekomercyjne - Na tych samych warunkach"		// CC Attribution-NonCommercial-ShareAlike License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o - N&atilde;oComercial - Compartilha Igual"			// CC Attribution-NonCommercial-ShareAlike License
#endif
	,
#if   L==1	// ca
	"CC Reconeixement - No comercial - Sense obra derivada"						// CC Attribution-NonCommercial-NoDerivs License
#elif L==2	// de
	"CC Namensnennung - Keine kommerzielle Nutzung - Keine Bearbeitung"				// CC Attribution-NonCommercial-NoDerivs License
#elif L==3	// en
	"CC Attribution - Noncommercial - No Derivative Works"						// CC Attribution-NonCommercial-NoDerivs License
#elif L==4	// es
	"CC Reconocimiento - No comercial - Sin obras derivadas"					// CC Attribution-NonCommercial-NoDerivs License
#elif L==5	// fr
	"CC Attribution - Pas dUtilisation Commerciale - Pas de travaux d&eacute;riv&eacute;s"		// CC Attribution-NonCommercial-NoDerivs License
#elif L==6	// gn
	"CC Reconocimiento - No comercial - Sin obras derivadas"					// CC Attribution-NonCommercial-NoDerivs License	// Okoteve traduccin
#elif L==7	// it
	"CC Attribuzione - Non commerciale - Non opere derivate"					// CC Attribution-NonCommercial-NoDerivs License
#elif L==8	// pl
	"CC Uznanie autorstwa - U&zdot;ycie niekomercyjne - Bez utwor&oacute;w zale&zdot;nych"		// CC Attribution-NonCommercial-NoDerivs License
#elif L==9	// pt
	"CC Atribui&ccedil;&atilde;o - N&atilde;oComercial - Trabalhos Derivados Pro&iacute;bidos"	// CC Attribution-NonCommercial-NoDerivs License
#endif
	};

const char *Txt_Link =
#if   L==1	// ca
	"Enlla&ccedil;";
#elif L==2	// de
	"Link";
#elif L==3	// en
	"Link";
#elif L==4	// es
	"Enlace";
#elif L==5	// fr
	"Lien";
#elif L==6	// gn
	"Enlace";	// Okoteve traduccin
#elif L==7	// it
	"Link";
#elif L==8	// pl
	"Link";
#elif L==9	// pt
	"Liga&ccedil;&atilde;o.";
#endif

const char *Txt_link =
#if   L==1	// ca
	"enlla&ccedil;";
#elif L==2	// de
	"Link";
#elif L==3	// en
	"link";
#elif L==4	// es
	"enlace";
#elif L==5	// fr
	"lien";
#elif L==6	// gn
	"enlace";	// Okoteve traduccin
#elif L==7	// it
	"link";
#elif L==8	// pl
	"link";
#elif L==9	// pt
	"liga&ccedil;&atilde;o.";
#endif

const char *Txt_Link_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Enlla&ccedil; <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Link <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Link <strong>%s</strong> removed.";
#elif L==4	// es
	"Enlace <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Lien <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Enlace <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Link <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Link <strong>%s</strong> usuniety.";
#elif L==9	// pt
	"Liga&ccedil;&atilde;o <strong>%s</strong> removida.";
#endif

const char *Txt_Link_to_announcement_of_exam =
#if   L==1	// ca
	"Enlla&ccedil; a convocat&ograve;ria d'examen";
#elif L==2	// de
	"Link zu Aufruf f&uuml;r Pr&uuml;fung";
#elif L==3	// en
	"Link to announcement of exam";
#elif L==4	// es
	"Enlace a convocatoria de examen";
#elif L==5	// fr
	"Lien vers convocation &agrave; un examen";
#elif L==6	// gn
	"Enlace a convocatoria de examen";	// Okoteve traduccin
#elif L==7	// it
	"Link al appello d'esame";
#elif L==8	// pl
	"Link do ogloszenie egzaminu";
#elif L==9	// pt
	"Link para chamada para exame";
#endif

const char *Txt_List_edit_questions =
#if   L==1	// ca
	"Llistar / editar preguntes";
#elif L==2	// de
	"Liste / bearbeiten Fragen";
#elif L==3	// en
	"List / edit questions";
#elif L==4	// es
	"Listar / editar preguntas";
#elif L==5	// fr
	"Lister / modifier questions";
#elif L==6	// gn
	"Listar / editar preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Elencare / modificare domande";
#elif L==8	// pl
	"Wy&sacute;wietli&cacute; / edytowa&cacute; pytania";
#elif L==9	// pt
	"Listar / editar quest&otilde;es";
#endif

const char *Txt_List_of_detailed_clicks =
#if   L==1	// ca
	"Listado de clics detallados";		// Necessita traduccio
#elif L==2	// de
	"List of detailed clicks";		// Need bersetzung
#elif L==3	// en
	"List of detailed clicks";
#elif L==4	// es
	"Listado de clics detallados";
#elif L==5	// fr
	"List of detailed clicks";		// Besoin de traduction
#elif L==6	// gn
	"Listado de clics detallados";		// Okoteve traduccin
#elif L==7	// it
	"Lista di scatti dettagliati";
#elif L==8	// pl
	"Lista szczeg&oacute;lowa klikniec";
#elif L==9	// pt
	"Lista de cliques detalhados";
#endif

const char *Txt_List_of_nicks_emails_or_IDs =
#if   L==1	// ca
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";	// Necessita traduccio
#elif L==2	// de
	"Liste von @Benutzernamen, Emails oder Ausweis-Nr.";
#elif L==3	// en
	"List of @nicks, emails or IDs";
#elif L==4	// es
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";
#elif L==5	// fr
	"Liste de num&eacute;ros d'identit&eacute;";
#elif L==6	// gn
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";	// Okoteve traduccin
#elif L==7	// it
	"Lista di @nomi-utenti, email o numero carte d'identit&agrave;";
#elif L==8	// pl
	"List of @nicks, emails or IDs";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Lista de @alcunhas, emails ou n&ordm;s. identif.";
#endif

const char *Txt_Links =
#if   L==1	// ca
	"Enlla&ccedil;os";
#elif L==2	// de
	"Links";
#elif L==3	// en
	"Links";
#elif L==4	// es
	"Enlaces";
#elif L==5	// fr
	"Liens";
#elif L==6	// gn
	"Enlaces";	// Okoteve traduccin
#elif L==7	// it
	"Link";
#elif L==8	// pl
	"Linki";
#elif L==9	// pt
	"Liga&ccedil;&otilde;es";
#endif

const char *Txt_Links_copied =
#if   L==1	// ca
	"Enlla&ccedil;os copiats";
#elif L==2	// de
	"Links kopiert";
#elif L==3	// en
	"Links copied";
#elif L==4	// es
	"Enlaces copiados";
#elif L==5	// fr
	"Liens copi&eacute;s";
#elif L==6	// gn
	"Enlaces copiados";	// Okoteve traduccin
#elif L==7	// it
	"Link copiati";
#elif L==8	// pl
	"Linki kopiowane";
#elif L==9	// pt
	"Liga&ccedil;&otilde;es copiadas";
#endif

const char *Txt_Links_removed =
#if   L==1	// ca
	"Enlla&ccedil;os eliminats";
#elif L==2	// de
	"Links entfernt";
#elif L==3	// en
	"Links removed";
#elif L==4	// es
	"Enlaces eliminados";
#elif L==5	// fr
	"Liens supprim&eacute;s";
#elif L==6	// gn
	"Enlaces eliminados";	// Okoteve traduccin
#elif L==7	// it
	"Link rimossi";
#elif L==8	// pl
	"Linki usuni&eogon;te";
#elif L==9	// pt
	"Liga&ccedil;&otilde;es removidas";
#endif

const char *Txt_Local_address =
#if   L==1	// ca
	"Domicilio local";	// Necessita traduccio
#elif L==2	// de
	"Lokale Adresse";
#elif L==3	// en
	"Local address";
#elif L==4	// es
	"Domicilio local";
#elif L==5	// fr
	"Adresse locale";
#elif L==6	// gn
	"Domicilio local";	// Okoteve traduccin
#elif L==7	// it
	"Indirizzo locale";
#elif L==8	// pl
	"Adres lokalny";
#elif L==9	// pt
	"Local Address";
#endif

const char *Txt_Location =
#if   L==1	// ca
	"Ubicaci&oacute;";
#elif L==2	// de
	"Standort";
#elif L==3	// en
	"Location";
#elif L==4	// es
	"Ubicaci&oacute;n";
#elif L==5	// fr
	"Emplacement";
#elif L==6	// gn
	"Tenda";
#elif L==7	// it
	"Posizione";
#elif L==8	// pl
	"Lokacja";
#elif L==9	// pt
	"Localiza&ccedil;&atilde;o";
#endif

const char *Txt_Locations =
#if   L==1	// ca
	"Ubicacions";
#elif L==2	// de
	"Standorte";
#elif L==3	// en
	"Locations";
#elif L==4	// es
	"Ubicaciones";
#elif L==5	// fr
	"Emplacements";
#elif L==6	// gn
	"Tenda";
#elif L==7	// it
	"Posizioni";
#elif L==8	// pl
	"Lokalizacje";
#elif L==9	// pt
	"Localiza&ccedil;&otilde;es";
#endif

const char *Txt_LOG_More_info =
#if   L==1	// ca
	"Observaciones";	// Necessita traduccio
#elif L==2	// de
	"Weitere Informationen";
#elif L==3	// en
	"More info";
#elif L==4	// es
	"Observaciones";
#elif L==5	// fr
	"Plus d'informations";
#elif L==6	// gn
	"Observaciones";	// Okoteve traduccin
#elif L==7	// it
	"Maggiori informazioni";
#elif L==8	// pl
	"Wiecej informacji";
#elif L==9	// pt
	"Mais informa&ccedil;&atilde;o";
#endif

const char *Txt_Log_in =
#if   L==1	// ca
	"Iniciar sessi&oacute;";
#elif L==2	// de
	"Anmelden";
#elif L==3	// en
	"Log in";
#elif L==4	// es
	"Iniciar sesi&oacute;n";
#elif L==5	// fr
	"Entrer";
#elif L==6	// gn
	"Tera&ntilde;emboguapy";
#elif L==7	// it
	"Inizia sessione";
#elif L==8	// pl
	"Zaloguj si&eogon;";
#elif L==9	// pt
	"Entrar";
#endif

const char *Txt_Log_out =
#if   L==1	// ca
	"Tancar sessi&oacute;";
#elif L==2	// de
	"Abmelden";
#elif L==3	// en
	"Log out";
#elif L==4	// es
	"Cerrar sesi&oacute;n";
#elif L==5	// fr
	"Fermer session";
#elif L==6	// gn
	"S&#7869;";
#elif L==7	// it
	"Chiudi sessione";
#elif L==8	// pl
	"Wyloguj si&eogon;";
#elif L==9	// pt
	"Fechar sess&atilde;o";
#endif

const char *Txt_logged[Usr_NUM_SEXS] =
	{
#if   L==1	// ca	// Usr_SEX_UNKNOWN
	 "identificado/a"	// Necessita traduccio
#elif L==2	// de
	 "angemeldet"
#elif L==3	// en
	 "logged"
#elif L==4	// es
	 "identificado/a"
#elif L==5	// fr
	 "connect&eacute;/e"
#elif L==6	// gn
	 "identificado/a"	// Okoteve traduccin
#elif L==7	// it
	 "loggato/a"
#elif L==8	// pl
	 "zalogowany"
#elif L==9	// pt
	 "logado/a"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_FEMALE
	 "identificada"	// Necessita traduccio
#elif L==2	// de
	 "angemeldet"
#elif L==3	// en
	 "logged"
#elif L==4	// es
	 "identificada"
#elif L==5	// fr
	 "connect&eacute;e"
#elif L==6	// gn
	 "identificada"	// Okoteve traduccin
#elif L==7	// it
	 "loggata"
#elif L==8	// pl
	 "zalogowana"
#elif L==9	// pt
	 "logada"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_MALE
	 "identificado"	// Necessita traduccio
#elif L==2	// de
	 "angemeldet"
#elif L==3	// en
	 "logged"
#elif L==4	// es
	 "identificado"
#elif L==5	// fr
	 "connect&eacute;"
#elif L==6	// gn
	 "identificado"	// Okoteve traduccin
#elif L==7	// it
	 "loggato"
#elif L==8	// pl
	 "zalogowany"
#elif L==9	// pt
	 "logado"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_ALL
	 "identificado/a"	// Necessita traduccio
#elif L==2	// de
	 "angemeldet"
#elif L==3	// en
	 "logged"
#elif L==4	// es
	 "identificado/a"
#elif L==5	// fr
	 "connect&eacute;/e"
#elif L==6	// gn
	 "identificado/a"	// Okoteve traduccin
#elif L==7	// it
	 "loggato/a"
#elif L==8	// pl
	 "zalogowani"
#elif L==9	// pt
	 "logado/a"
#endif
	};

const char *Txt_Logo =
#if   L==1	// ca
	"Logo";	// Necessita traduccio
#elif L==2	// de
	"Logo";
#elif L==3	// en
	"Logo";
#elif L==4	// es
	"Logo";
#elif L==5	// fr
	"Logo";
#elif L==6	// gn
	"Logo";	// Okoteve traduccin
#elif L==7	// it
	"Logo";
#elif L==8	// pl
	"Logo";
#elif L==9	// pt
	"Logotipo";
#endif

const char *Txt_Mandatory_enrolment =		// (to a type of group)
#if   L==1	// ca
	"&iquest;Adscripci&oacute;n obligatoria?";	// Necessita traduccio
#elif L==2	// de
	"Obligatorische beitreten?";
#elif L==3	// en
	"Mandatory enrolment?";
#elif L==4	// es
	"&iquest;Adscripci&oacute;n obligatoria?";
#elif L==5	// fr
	"Inscription obligatoire?";
#elif L==6	// gn
	"&iquest;Adscripci&oacute;n obligatoria?";	// Okoteve traduccin
#elif L==7	// it
	"Iscrizione obbligatoria?";
#elif L==8	// pl
	"Obowiazkowa rejestracja?";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o obrigat&oacute;ria?";
#endif

const char *Txt_Mark_all_NOTIFICATIONS_as_read =
#if   L==1	// ca
	"Marca totes com llegides";
#elif L==2	// de
	"Markieren Sie alle als gelesen";
#elif L==3	// en
	"Mark all as read";
#elif L==4	// es
	"Marcar todas como le&iacute;das";
#elif L==5	// fr
	"Marquer tous comme lus";
#elif L==6	// gn
	"Marcar todas como le&iacute;das";	// Okoteve traduccin
#elif L==7	// it
	"Segnare tutte come gi&grave; letti";
#elif L==8	// pl
	"Oznacz wszystko jako czytasz";
#elif L==9	// pt
	"Marcar todas como lidas";
#endif

const char *Txt_Mark_as_favourite =
#if   L==1	// ca
	"Marca com favorit";
#elif L==2	// de
	"Markieren als Favorit";
#elif L==3	// en
	"Mark as favourite";
#elif L==4	// es
	"Marcar como favorito";
#elif L==5	// fr
	"Marquer comme favori";
#elif L==6	// gn
	"Marcar como favorito";	// Okoteve traduccin
#elif L==7	// it
	"Segnare come preferito";
#elif L==8	// pl
	"Oznacz jako ulubiony";
#elif L==9	// pt
	"Marcar como favorito";
#endif

const char *Txt_Marks_management_area =
#if   L==1	// ca
	"Zona d'administraci&oacute; de qualificacions";
#elif L==2	// de
	"Bewertungs-Verwaltung";
#elif L==3	// en
	"Marks management area";
#elif L==4	// es
	"Zona de administraci&oacute;n de calificaciones";
#elif L==5	// fr
	"Zone de gestion de notes";
#elif L==6	// gn
	"Zona de administraci&oacute;n de calificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Area di gestione dei risultati";
#elif L==8	// pl
	"Obszar zarzadzania znak&oacute;w";
#elif L==9	// pt
	"Zona de gest&atilde;o de notas";
#endif

const char *Txt_marks_management_area =
#if   L==1	// ca
	"zona de administraci&oacute;n de calificaciones";	// Necessita traduccio
#elif L==2	// de
	"Bewertungs-Verwaltung";
#elif L==3	// en
	"marks management area";
#elif L==4	// es
	"zona de administraci&oacute;n de calificaciones";
#elif L==5	// fr
	"zone de gestion de notes";
#elif L==6	// gn
	"zona de administraci&oacute;n de calificaciones";	// Okoteve traduccin
#elif L==7	// it
	"area di gestione dei risultati";
#elif L==8	// pl
	"obszar zarzadzania znak&oacute;w";
#elif L==9	// pt
	"zona de gest&atilde;o de notas";
#endif

const char *Txt_Marks_area =
#if   L==1	// ca
	"Zona de qualificacions";
#elif L==2	// de
	"Bewertungs-Bereich";
#elif L==3	// en
	"Marks area";
#elif L==4	// es
	"Zona de calificaciones";
#elif L==5	// fr
	"Zone de notes";
#elif L==6	// gn
	"Zona de calificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Area risultati";
#elif L==8	// pl
	"Powierzchnia znak&oacute;w";
#elif L==9	// pt
	"Zona de notas";
#endif

const char *Txt_Materials =
#if   L==1	// ca
	"Materiales";	// Necessita traduccio
#elif L==2	// de
	"Material";
#elif L==3	// en
	"Materials";
#elif L==4	// es
	"Materiales";
#elif L==5	// fr
	"Mat&eacute;riaux";
#elif L==6	// gn
	"Materiales";	// Okoteve traduccin
#elif L==7	// it
	"Materiale";
#elif L==8	// pl
	"Materialy";
#elif L==9	// pt
	"Materiais";
#endif

const char *Txt_Max_levels =
#if   L==1	// ca
	"M&aacute;x. niveles";	// Necessita traduccio
#elif L==2	// de
	"Max. Ebenen";
#elif L==3	// en
	"Max. levels";
#elif L==4	// es
	"M&aacute;x. niveles";
#elif L==5	// fr
	"Max. niveaux";
#elif L==6	// gn
	"M&aacute;x. niveles";	// Okoteve traduccin
#elif L==7	// it
	"Max. livelli";
#elif L==8	// pl
	"Max. poziom&oacute;w";
#elif L==9	// pt
	"M&aacute;x. n&iacute;veis";
#endif

const char *Txt_Max_BR_students =
#if   L==1	// ca
	"M&agrave;xim<br />d'est.";
#elif L==2	// de
	"Max.<br />Stud.";
#elif L==3	// en
	"Max.<br />stud.";
#elif L==4	// es
	"M&aacute;x.<br />estu.";
#elif L==5	// fr
	"Max.<br />&eacute;tud.";
#elif L==6	// gn
	"M&aacute;x.<br />estu.";	// Okoteve traduccin
#elif L==7	// it
	"Max.<br />stud.";
#elif L==8	// pl
	"Max.<br />stud.";
#elif L==9	// pt
	"M&aacute;x.<br />estu.";
#endif

const char *Txt_maximum =
#if   L==1	// ca
	"m&aacute;ximo";	// Necessita traduccio
#elif L==2	// de
	"Maximum";
#elif L==3	// en
	"maximum";
#elif L==4	// es
	"m&aacute;ximo";
#elif L==5	// fr
	"maximum";
#elif L==6	// gn
	"m&aacute;ximo";	// Okoteve traduccin
#elif L==7	// it
	"massimo";
#elif L==8	// pl
	"maksimum";
#elif L==9	// pt
	"m&aacute;ximo";
#endif

const char *Txt_Members =	// Project members
#if   L==1	// ca
	"Membres";
#elif L==2	// de
	"Mitarbeiter";
#elif L==3	// en
	"Members";
#elif L==4	// es
	"Miembros";
#elif L==5	// fr
	"Membres";
#elif L==6	// gn
	"Miembros";	// Okoteve traduccin
#elif L==7	// it
	"Membri";
#elif L==8	// pl
	"Cz&lstrok;onkowie";
#elif L==9	// pt
	"Membros";
#endif

const char *Txt_Menu =
#if   L==1	// ca
	"Men&uacute;";
#elif L==2	// de
	"Men&uuml;";
#elif L==3	// en
	"Menu";
#elif L==4	// es
	"Men&uacute;";
#elif L==5	// fr
	"Menu";
#elif L==6	// gn
	"Men&uacute;";	// Okoteve traduccin
#elif L==7	// it
	"Menu";
#elif L==8	// pl
	"Menu";
#elif L==9	// pt
	"Menu";
#endif

const char *Txt_MENU_NAMES[Mnu_NUM_MENUS] =
   {
#if   L==1	// ca
	 "Horitzontal"
#elif L==2	// de
	 "Horizontal"
#elif L==3	// en
	 "Horizontal"
#elif L==4	// es
	 "Horizontal"
#elif L==5	// fr
	 "Horizontal"
#elif L==6	// gn
	 "Horizontal"	// Okoteve traduccin
#elif L==7	// it
	 "Orizzontale"
#elif L==8	// pl
	 "Poziomy"
#elif L==9	// pt
	 "Horizontal"
#endif
	 ,
#if   L==1	// ca
	 "Vertical"
#elif L==2	// de
	 "Vertikal"
#elif L==3	// en
	 "Vertical"
#elif L==4	// es
	 "Vertical"
#elif L==5	// fr
	 "Vertical"
#elif L==6	// gn
	 "Vertical"		// Okoteve traduccin
#elif L==7	// it
	 "Verticale"
#elif L==8	// pl
	 "Pionowy"
#elif L==9	// pt
	 "Vertical"
#endif
	};

const char *Txt_MENU_TITLE[Tab_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB] =
{
	{
	// TabUnk ****************************************************
	NULL,	//  0
	NULL,	//  1
	NULL,	//  2
	NULL,	//  3
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabSta *****************************************************
	//  0: ActFrmLogIn
#if   L==1	// ca
	"Entrar"
#elif L==2	// de
	"Einloggen"
#elif L==3	// en
	"Log in"
#elif L==4	// es
	"Entrar"
#elif L==5	// fr
	"S'identifier"
#elif L==6	// gn
	"Entrar"	// Okoteve traduccin
#elif L==7	// it
	"Accesso"
#elif L==8	// pl
	"Zaloguj Si&eogon;"
#elif L==9	// pt
	"Entrar"
#endif
	,
	//  1: ActReqSch
#if   L==1	// ca
	"Cercar"
#elif L==2	// de
	"Suchen"
#elif L==3	// en
	"Search"
#elif L==4	// es
	"Buscar"
#elif L==5	// fr
	"Chercher"
#elif L==6	// gn
	"Heka"
#elif L==7	// it
	"Cercare"
#elif L==8	// pl
	"Szukaj"
#elif L==9	// pt
	"Procurar"
#endif
	,
	//  2: ActSeeSocTmlGbl
#if   L==1	// ca
	 "Timeline"
#elif L==2	// de
	 "Timeline"
#elif L==3	// en
	 "Timeline"
#elif L==4	// es
	 "Timeline"
#elif L==5	// fr
	 "Chronologie"
#elif L==6	// gn
	 "Timeline"	// Okoteve traduccin
#elif L==7	// it
	 "Timeline"
#elif L==8	// pl
	 "Timeline"
#elif L==9	// pt
	 "Timeline"
#endif
	,
	//  3: ActSeeSocPrf
#if   L==1	// ca
	 "Perfils"
#elif L==2	// de
	 "Profiles"
#elif L==3	// en
	 "Profiles"
#elif L==4	// es
	 "Perfiles"
#elif L==5	// fr
	 "Profils"
#elif L==6	// gn
	 "Che"
#elif L==7	// it
	 "Profili"
#elif L==8	// pl
	 "Profile"
#elif L==9	// pt
	 "Perfiles"
#endif
	,
	//  4: ActSeeCal
#if   L==1	// ca
	"Calendari"
#elif L==2	// de
	"Kalender"
#elif L==3	// en
	"Calendar"
#elif L==4	// es
	"Calendario"
#elif L==5	// fr
	"Calendrier"
#elif L==6	// gn
	"Arapapaha"
#elif L==7	// it
	"Calendario"
#elif L==8	// pl
	"Kalendarz"
#elif L==9	// pt
	"Calend&aacute;rio"
#endif
	,
	//  5: ActSeeNtf
#if   L==1	// ca
	"Notificacions"
#elif L==2	// de
	"Anmeldungen"
#elif L==3	// en
	"Notifications"
#elif L==4	// es
	"Notificaciones"
#elif L==5	// fr
	"Notifications"
#elif L==6	// gn
	"Momarandu"
#elif L==7	// it
	"Notifiche"
#elif L==8	// pl
	"Powiadomienia"
#elif L==9	// pt
	"Notifica&ccedil;&otilde;es"
#endif
	,
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabSys *****************************************************
	//  0: ActSeeCty
#if   L==1	// ca
	"Pa&iuml;sos"
#elif L==2	// de
	"L&auml;nder"
#elif L==3	// en
	"Countries"
#elif L==4	// es
	"Pa&iacute;ses"
#elif L==5	// fr
	"Pays"
#elif L==6	// gn
	"Tet&atilde;"
#elif L==7	// it
	"Paesi"
#elif L==8	// pl
	"Kraje"
#elif L==9	// pt
	"Pa&iacute;ses"
#endif
	,
	//  1: ActSeePen
#if   L==1	// ca
	 "Jerarquia"
#elif L==2	// de
	 "Hierarchie"
#elif L==3	// en
	 "Hierarchy"
#elif L==4	// es
	 "Jerarqu&iacute;a"
#elif L==5	// fr
	 "Hi&eacute;rarchie"
#elif L==6	// gn
	 "S&atilde;mbyhyha reko"
#elif L==7	// it
	 "Gerarchia"
#elif L==8	// pl
	 "Hierarchia"
#elif L==9	// pt
	 "Hierarquia"
#endif
	,
	//  2: ActSeeLnk
#if   L==1	// ca
	"Enlla&ccedil;os"
#elif L==2	// de
	"Links"
#elif L==3	// en
	"Links"
#elif L==4	// es
	"Enlaces"
#elif L==5	// fr
	"Liens"
#elif L==6	// gn
	"Ambu&eacute;re"
#elif L==7	// it
	"Link"
#elif L==8	// pl
	"Linki"
#elif L==9	// pt
	"Liga&ccedil;&otilde;es"
#endif
	,
	//  3: ActLstPlg
#if   L==1	// ca
	"Complements"
#elif L==2	// de
	"Plugins"
#elif L==3	// en
	"Plugins"
#elif L==4	// es
	"Complementos"
#elif L==5	// fr
	"Plugins"
#elif L==6	// gn
	"Complementos"	// Okoteve traduccin
#elif L==7	// it
	"Plugin"
#elif L==8	// pl
	"Plugin&oacute;w"
#elif L==9	// pt
	"Plugins"
#endif
	,
	//  4: ActSetUp
#if   L==1	// ca
	"Instal lar"
#elif L==2	// de
	"Einrichten"
#elif L==3	// en
	"Set up"
#elif L==4	// es
	"Instalar"
#elif L==5	// fr
	"Installer"
#elif L==6	// gn
	"Instalar"	// Okoteve traduccin
#elif L==7	// it
	"Setup"
#elif L==8	// pl
	"Set up"
#elif L==9	// pt
	"Instalar"
#endif
	,
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabCty *****************************************************
	//  0: ActSeeCtyInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#endif
	,
	//  1: ActSeeIns
#if   L==1	// ca
	"Institucions"
#elif L==2	// de
	"Hochschulen"
#elif L==3	// en
	"Institutions"
#elif L==4	// es
	"Instituciones"
#elif L==5	// fr
	"&Eacute;tablissements"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Istituzioni"
#elif L==8	// pl
	"Instytucje"
#elif L==9	// pt
	"Institu&ccedil;&otilde;es"
#endif
	,
	NULL,	//  2
	NULL,	//  3
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabIns *****************************************************
	//  0: ActSeeInsInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#endif
	,
	//  1: ActSeeCtr
#if   L==1	// ca
	"Centres"
#elif L==2	// de
	"Lehrinstitute"
#elif L==3	// en
	"Centres"
#elif L==4	// es
	"Centros"
#elif L==5	// fr
	"Centres"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Centri"
#elif L==8	// pl
	"Centra"
#elif L==9	// pt
	"Centros"
#endif
	,
	//  2: ActSeePlc
#if   L==1	// ca
	"Llocs"
#elif L==2	// de
	"Standorte"
#elif L==3	// en
	"Places"
#elif L==4	// es
	"Lugares"
#elif L==5	// fr
	"Emplacements"
#elif L==6	// gn
	"Tenda"
#elif L==7	// it
	"Localit&grave;"
#elif L==8	// pl
	"Miejsca"
#elif L==9	// pt
	"Localiza&ccedil;oes"
#endif
	,
	//  3: ActSeeDpt
#if   L==1	// ca
	"Departaments"
#elif L==2	// de
	"Abteilungen"
#elif L==3	// en
	"Departments"
#elif L==4	// es
	"Departamentos"
#elif L==5	// fr
	"D&eacute;partements"
#elif L==6	// gn
	"Peh&#7869;"
#elif L==7	// it
	"Dipartimenti"
#elif L==8	// pl
	"Wydzia&lstrok;y"
#elif L==9	// pt
	"Departamentos"
#endif
	,
	//  4: ActSeeHld
#if   L==1	// ca
	"Festius"
#elif L==2	// de
	"Feiertage"
#elif L==3	// en
	"Holidays"
#elif L==4	// es
	"Festivos"
#elif L==5	// fr
	"F&eacute;ri&eacute;s"
#elif L==6	// gn
	"Arete"
#elif L==7	// it
	"Festivit&agrave;"
#elif L==8	// pl
	"Noclegi"
#elif L==9	// pt
	"F&eacute;rias"
#endif
	,
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabCtr *****************************************************
	//  0: ActSeeCtrInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#endif
	,
	//  1: ActSeeDeg
#if   L==1	// ca
	"Titulacions"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"Degrees"
#elif L==4	// es
	"Titulaciones"
#elif L==5	// fr
	"&Eacute;tudes"
#elif L==6	// gn
	"Arandur&atilde;"
#elif L==7	// it
	"Lauree"
#elif L==8	// pl
	"Stopnie"
#elif L==9	// pt
	"Graus"
#endif
	,
	//  2: ActSeeCla
#if   L==1	// ca
	"Aules"
#elif L==2	// de
	"Klassenzimmer"
#elif L==3	// en
	"Classrooms"
#elif L==4	// es
	"Aulas"
#elif L==5	// fr
	"Salles"
#elif L==6	// gn
	"Mbo'ehakoty"
#elif L==7	// it
	"Aule"
#elif L==8	// pl
	"Sale"
#elif L==9	// pt
	"Salas"
#endif
	,
	NULL,	//  3
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabDeg *****************************************************
	//  0: ActSeeDegInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#endif
	,
	//  1: ActSeeCrs
#if   L==1	// ca
	"Assignatures"
#elif L==2	// de
	"Kursen"
#elif L==3	// en
	"Courses"
#elif L==4	// es
	"Asignaturas"
#elif L==5	// fr
	"Mati&egrave;res"
#elif L==6	// gn
	"Mbo'esyry"
#elif L==7	// it
	"Corsi"
#elif L==8	// pl
	"Kurs&oacute;w"
#elif L==9	// pt
	"Disciplinas"
#endif
	,
	NULL,	//  2
	NULL,	//  3
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabCrs *****************************************************
	//  0: ActSeeCrsInf
#if   L==1	// ca
	"Informaci&oacute;"
#elif L==2	// de
	"Information"
#elif L==3	// en
	"Information"
#elif L==4	// es
	"Informaci&oacute;n"
#elif L==5	// fr
	"Information"
#elif L==6	// gn
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Informazione"
#elif L==8	// pl
	"Informacja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o"
#endif
	,
	//  1: ActSeeTchGui
#if   L==1	// ca
	"Guia"
#elif L==2	// de
	"Hinweise"
#elif L==3	// en
	"Guide"
#elif L==4	// es
	"Gu&iacute;a"
#elif L==5	// fr
	"Guide"
#elif L==6	// gn
	"Gu&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	"Guida"
#elif L==8	// pl
	"Podr&eogon;cznik"
#elif L==9	// pt
	"Guia"
#endif
	,
	//  2: ActSeeSyl
#if   L==1	// ca
	"Programa"
#elif L==2	// de
	"Studienplan"
#elif L==3	// en
	"Syllabus"
#elif L==4	// es
	"Programa"
#elif L==5	// fr
	"Plan"
#elif L==6	// gn
	"Programa"	// Okoteve traduccin
#elif L==7	// it
	"Programma"
#elif L==8	// pl
	"Syllabus"
#elif L==9	// pt
	"Programa"
#endif
	,
	//  3: ActSeeBib
#if   L==1	// ca
	"Bibliografia"
#elif L==2	// de
	"Literatur"
#elif L==3	// en
	"Bibliography"
#elif L==4	// es
	"Bibliograf&iacute;a"
#elif L==5	// fr
	"Bibliographie"
#elif L==6	// gn
	"Arandukapurupyre"
#elif L==7	// it
	"Bibliografia"
#elif L==8	// pl
	"Bibliografia"
#elif L==9	// pt
	"Bibliografia"
#endif
	,
	//  4: ActSeeFAQ
#if   L==1	// ca
	"FAQ"	// Necessita traduccio
#elif L==2	// de
	"FAQ"
#elif L==3	// en
	"FAQ"
#elif L==4	// es
	"FAQ"
#elif L==5	// fr
	"FAQ"
#elif L==6	// gn
	"FAQ"	// Okoteve traduccin
#elif L==7	// it
	"FAQ"
#elif L==8	// pl
	"FAQ"
#elif L==9	// pt
	"FAQ"
#endif
	,
	//  5: ActSeeCrsLnk
#if   L==1	// ca
	"Enlla&ccedil;os"
#elif L==2	// de
	"Links"
#elif L==3	// en
	"Links"
#elif L==4	// es
	"Enlaces"
#elif L==5	// fr
	"Liens"
#elif L==6	// gn
	"Ambu&eacute;re"
#elif L==7	// it
	"Link"
#elif L==8	// pl
	"Linki"
#elif L==9	// pt
	"Liga&ccedil;&otilde;es"
#endif
	,
	//  6: ActSeeCrsTT
#if   L==1	// ca
	"Horari"
#elif L==2	// de
	"Stundenplan"
#elif L==3	// en
	"Timetable"
#elif L==4	// es
	"Horario"
#elif L==5	// fr
	"Horaire"
#elif L==6	// gn
	"Horario"	// Okoteve traduccin
#elif L==7	// it
	"Orario"
#elif L==8	// pl
	"Plan"
#elif L==9	// pt
	"Hor&aacute;rio"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabAss *****************************************************
	//  0: ActSeeAss
#if   L==1	// ca
	 "Sistema"
#elif L==2	// de
	 "System"
#elif L==3	// en
	 "System"
#elif L==4	// es
	 "Sistema"
#elif L==5	// fr
	 "Syst&egrave;me"
#elif L==6	// gn
	 "Sistema"	// Okoteve traduccin
#elif L==7	// it
	 "Sistema"
#elif L==8	// pl
	 "System"
#elif L==9	// pt
	 "Sistema"
#endif
	,
	//  1: ActSeeAsg
#if   L==1	// ca
	"Activitats"
#elif L==2	// de
	"Aufgaben"
#elif L==3	// en
	"Assignments"
#elif L==4	// es
	"Actividades"
#elif L==5	// fr
	"Activit&eacute;s"
#elif L==6	// gn
	"Tar&eacute;a"
#elif L==7	// it
	"Attivit&agrave;"
#elif L==8	// pl
	"Zadania"
#elif L==9	// pt
	"Atividades"
#endif
	,
	//  2: ActSeePrj
#if   L==1	// ca
	"Projectes"
#elif L==2	// de
	"Projekte"
#elif L==3	// en
	"Projects"
#elif L==4	// es
	"Proyectos"
#elif L==5	// fr
	"Projets"
#elif L==6	// gn
	"Proyectos"	// Okoteve traduccin
#elif L==7	// it
	"Progetti"
#elif L==8	// pl
	"Projekty"
#elif L==9	// pt
	"Projetos"
#endif
	,
	//  3: ActReqTst
#if   L==1	// ca
	"Tests"
#elif L==2	// de
	"Tests"
#elif L==3	// en
	"Tests"
#elif L==4	// es
	"Test"
#elif L==5	// fr
	"Tests"
#elif L==6	// gn
	"Test"	// Okoteve traduccin
#elif L==7	// it
	"Test"
#elif L==8	// pl
	"Test&oacute;w"
#elif L==9	// pt
	"Testes"
#endif
	,
	//  4: ActSeeAllGam
#if   L==1	// ca
	"Jocs"
#elif L==2	// de
	"Spiele"
#elif L==3	// en
	"Games"
#elif L==4	// es
	"Juegos"
#elif L==5	// fr
	"Jeux"
#elif L==6	// gn
	"Juegos"	// Okoteve traduccin
#elif L==7	// it
	"Giochi"
#elif L==8	// pl
	"Gry"
#elif L==9	// pt
	"Jogos"
#endif
	,
	//  5: ActSeeAllSvy
#if   L==1	// ca
	"Enquestes"
#elif L==2	// de
	"Umfragen"
#elif L==3	// en
	"Surveys"
#elif L==4	// es
	"Encuestas"
#elif L==5	// fr
	"Sondages"
#elif L==6	// gn
	"Encuestas"	// Okoteve traduccin
#elif L==7	// it
	"Sondaggi"
#elif L==8	// pl
	"Ankiety"
#elif L==9	// pt
	"Inqu&eacute;ritos"
#endif
	,
	//  6: ActSeeAllExaAnn
#if   L==1	// ca
	"Convocat&ograve;ries"
#elif L==2	// de
	"Pr&uuml;fungsaufrufe"
#elif L==3	// en
	"Announcements"
#elif L==4	// es
	"Convocatorias"
#elif L==5	// fr
	"Convocations"
#elif L==6	// gn
	"Convocatorias"	// Okoteve traduccin
#elif L==7	// it
	"Appelli"
#elif L==8	// pl
	"Ogloszenia"
#elif L==9	// pt
	"Chamadas"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabFil *****************************************************
	//  0: ActSeeAdmDocIns
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents"
#elif L==4	// es
	"Documentos"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Kuatia"
#elif L==7	// it
	"Documenti"
#elif L==8	// pl
	"Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#endif
	,
	//  1: ActAdmShaIns
#if   L==1	// ca
	"Compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"Shared"
#elif L==4	// es
	"Compartidos"
#elif L==5	// fr
	"Partag&eacute;s"
#elif L==6	// gn
	"Compartidos"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi"
#elif L==8	// pl
	"Udost&eogon;pnione"
#elif L==9	// pt
	"Compartilhados"
#endif
	,
	//  2: ActSeeAdmDocCtr
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents"
#elif L==4	// es
	"Documentos"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Kuatia"
#elif L==7	// it
	"Documenti"
#elif L==8	// pl
	"Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#endif
	,
	//  3: ActAdmShaCtr
#if   L==1	// ca
	"Compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"Shared"
#elif L==4	// es
	"Compartidos"
#elif L==5	// fr
	"Partag&eacute;s"
#elif L==6	// gn
	"Compartidos"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi"
#elif L==8	// pl
	"Udost&eogon;pnione"
#elif L==9	// pt
	"Compartilhados"
#endif
	,
	//  4: ActSeeAdmDocDeg
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents"
#elif L==4	// es
	"Documentos"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Kuatia"
#elif L==7	// it
	"Documenti"
#elif L==8	// pl
	"Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#endif
	,
	//  5: ActAdmShaDeg
#if   L==1	// ca
	"Compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"Shared"
#elif L==4	// es
	"Compartidos"
#elif L==5	// fr
	"Partag&eacute;s"
#elif L==6	// gn
	"Compartidos"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi"
#elif L==8	// pl
	"Udost&eogon;pnione"
#elif L==9	// pt
	"Compartilhados"
#endif
	,
	//  6: ActSeeAdmDocCrsGrp
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents"
#elif L==4	// es
	"Documentos"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Kuatia"
#elif L==7	// it
	"Documenti"
#elif L==8	// pl
	"Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#endif
	,
	//  7: ActAdmTchCrsGrp
#if   L==1	// ca
	"Privats"
#elif L==2	// de
	"Private"
#elif L==3	// en
	"Private"
#elif L==4	// es
	"Privados"
#elif L==5	// fr
	"Priv&eacute;s"
#elif L==6	// gn
	"Privados"	// Okoteve traduccin
#elif L==7	// it
	"Privati"
#elif L==8	// pl
	"Prywatne"
#elif L==9	// pt
	"Privados"
#endif
	,
	//  8: ActAdmShaCrsGrp
#if   L==1	// ca
	"Compartits"
#elif L==2	// de
	"Freigegebene"
#elif L==3	// en
	"Shared"
#elif L==4	// es
	"Compartidos"
#elif L==5	// fr
	"Partag&eacute;s"
#elif L==6	// gn
	"Compartidos"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi"
#elif L==8	// pl
	"Udost&eogon;pnione"
#elif L==9	// pt
	"Compartilhados"
#endif
	,
	//  9: ActAdmAsgWrkUsr
#if   L==1	// ca
	"Treballs"
#elif L==2	// de
	"Hausarbeiten"
#elif L==3	// en
	"Homework"
#elif L==4	// es
	"Trabajos"
#elif L==5	// fr
	"Travaux"
#elif L==6	// gn
	"Tembiapo"
#elif L==7	// it
	"Lavori"
#elif L==8	// pl
	"Zadanie"
#elif L==9	// pt
	"Trabalhos"
#endif
	,
	// 10: ActReqAsgWrkCrs
#if   L==1	// ca
	"Treballs"
#elif L==2	// de
	"Hausarbeiten"
#elif L==3	// en
	"Homework"
#elif L==4	// es
	"Trabajos"
#elif L==5	// fr
	"Travaux"
#elif L==6	// gn
	"Tembiapo"
#elif L==7	// it
	"Lavori"
#elif L==8	// pl
	"Zadanie"
#elif L==9	// pt
	"Trabalhos"
#endif
	,
	// 11: ActSeeMrk
#if   L==1	// ca
	"Qualificacions"
#elif L==2	// de
	"Bewertungen"
#elif L==3	// en
	"Marks"
#elif L==4	// es
	"Calificaciones"
#elif L==5	// fr
	"Notes"
#elif L==6	// gn
	"Calificaciones"	// Okoteve traduccin
#elif L==7	// it
	"Risultati"
#elif L==8	// pl
	"Marks"
#elif L==9	// pt
	"Notas"
#endif
	,
	// 12: ActAdmBrf
#if   L==1	// ca
	"Malet&iacute;"
#elif L==2	// de
	"Aktentasche"
#elif L==3	// en
	"Briefcase"
#elif L==4	// es
	"Malet&iacute;n"
#elif L==5	// fr
	"Portedocuments"
#elif L==6	// gn
	"Malet&iacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Cartella"
#elif L==8	// pl
	"Teczka"
#elif L==9	// pt
	"Pasta"
#endif
	},
	{
	// TabUsr *****************************************************
	//  0: ActReqSelGrp
#if   L==1	// ca
	"Grups"
#elif L==2	// de
	"Gruppen"
#elif L==3	// en
	"Groups"
#elif L==4	// es
	"Grupos"
#elif L==5	// fr
	"Groupes"
#elif L==6	// gn
	"Aty"
#elif L==7	// it
	"Gruppi"
#elif L==8	// pl
	"Grupy"
#elif L==9	// pt
	"Grupos"
#endif
	,
	//  1: ActLstStd
#if   L==1	// ca
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiants"
#elif L==6	// gn
	"Estudiantes"	// Okoteve traduccin
#elif L==7	// it
	"Studenti"
#elif L==8	// pl
	"Student&oacute;w"
#elif L==9	// pt
	"Estudantes"
#endif
	,
	//  2: ActLstTch
#if   L==1	// ca
	"Professors"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesores"
#elif L==5	// fr
	"Enseignants"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Professores"
#endif
	,
	//  3: ActLstOth
#if   L==1	// ca
	"Altres"
#elif L==2	// de
	"Andere"
#elif L==3	// en
	"Others"
#elif L==4	// es
	"Otros"
#elif L==5	// fr
	"Autres"
#elif L==6	// gn
	"Otros"		// Okoteve traduccin
#elif L==7	// it
	"Altri"
#elif L==8	// pl
	"Inne"
#elif L==9	// pt
	"Outros"
#endif
	,
	//  4: ActSeeAtt
#if   L==1	// ca
	"Assist&egrave;ncia"
#elif L==2	// de
	"Anwesenheit"
#elif L==3	// en
	"Attendance"
#elif L==4	// es
	"Asistencia"
#elif L==5	// fr
	"Pr&eacute;sence"
#elif L==6	// gn
	"Asistencia"	// Okoteve traduccin
#elif L==7	// it
	"Frequenza"
#elif L==8	// pl
	"Frekwencji"
#elif L==9	// pt
	"Presen&ccedil;a"
#endif
	,
	//  5: ActReqSignUp
#if   L==1	// ca
	"Inscriure"
#elif L==2	// de
	"Anmeldung"
#elif L==3	// en
	"Sign up"
#elif L==4	// es
	"Inscribirme"
#elif L==5	// fr
	"Signer"
#elif L==6	// gn
	"Inscribirme"	// Okoteve traduccin
#elif L==7	// it
	"Iscrivi"
#elif L==8	// pl
	"Zapisa&cacute;"
#elif L==9	// pt
	"Inscrever"
#endif
	,
	//  6: ActSeeSignUpReq
#if   L==1	// ca
	"Peticions"
#elif L==2	// de
	"Ersuchen"
#elif L==3	// en
	"Requests"
#elif L==4	// es
	"Solicitudes"
#elif L==5	// fr
	"Demandes"
#elif L==6	// gn
	"Solicitudes"// Okoteve traduccin
#elif L==7	// it
	"Richieste"
#elif L==8	// pl
	"Upraszanie"
#elif L==9	// pt
	"Solicita&ccedil;&otilde;es"
#endif
	,
	//  7: ActLstCon
#if   L==1	// ca
	"Connectats"
#elif L==2	// de
	"Angemeldet"
#elif L==3	// en
	"Connected"
#elif L==4	// es
	"Conectados"
#elif L==5	// fr
	"Connect&eacute;s"
#elif L==6	// gn
	"Conectados"	// Okoteve traduccin
#elif L==7	// it
	"Connessi"
#elif L==8	// pl
	"Online"
#elif L==9	// pt
	"Conectados"
#endif
	,
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabMsg *****************************************************
	//  0: ActSeeAnn
#if   L==1	// ca
	"Anuncis"
#elif L==2	// de
	"Bekanntmachungen"
#elif L==3	// en
	"Announcements"
#elif L==4	// es
	"Anuncios"
#elif L==5	// fr
	"Annonces"
#elif L==6	// gn
	"Anuncios"	// Okoteve traduccin
#elif L==7	// it
	"Annunci"
#elif L==8	// pl
	"Og&lstrok;oszenia"
#elif L==9	// pt
	"An&uacute;ncios"
#endif
	,
	//  1: ActSeeAllNot
#if   L==1	// ca
	"Avisos"
#elif L==2	// de
	"Ank&uuml;ndigungen"
#elif L==3	// en
	"Notices"
#elif L==4	// es
	"Avisos"
#elif L==5	// fr
	"Avis"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Avvisi"
#elif L==8	// pl
	"Ogloszenia"
#elif L==9	// pt
	"Avisos"
#endif
	,
	//  2: ActSeeFor
#if   L==1	// ca
	"F&ograve;rums"
#elif L==2	// de
	"Foren"
#elif L==3	// en
	"Forums"
#elif L==4	// es
	"Foros"
#elif L==5	// fr
	"Forums"
#elif L==6	// gn
	"Foros"	// Okoteve traduccin
#elif L==7	// it
	"Forum"
#elif L==8	// pl
	"Forum"
#elif L==9	// pt
	"F&oacute;runs"
#endif
	,
	//  3: ActSeeChtRms
#if   L==1	// ca
	"Xat"
#elif L==2	// de
	"Chat"
#elif L==3	// en
	"Chat"
#elif L==4	// es
	"Chat"
#elif L==5	// fr
	"Chat"
#elif L==6	// gn
	"&Ntilde;omongeta"
#elif L==7	// it
	"Chat"
#elif L==8	// pl
	"Chat"
#elif L==9	// pt
	"Chat"
#endif
	,
	//  4: ActReqMsgUsr
#if   L==1	// ca
	"Redactar"
#elif L==2	// de
	"Schreiben"
#elif L==3	// en
	"Write"
#elif L==4	// es
	"Redactar"
#elif L==5	// fr
	"Ecrire"
#elif L==6	// gn
	"Redactar"	// Okoteve traduccin
#elif L==7	// it
	"Scrivi"
#elif L==8	// pl
	"Napisz"
#elif L==9	// pt
	"Escrever"
#endif
	,
	//  5: ActSeeRcvMsg
#if   L==1	// ca
	"Rebuts"
#elif L==2	// de
	"Empfangene"
#elif L==3	// en
	"Received"
#elif L==4	// es
	"Recibidos"
#elif L==5	// fr
	"Re&ccedil;us"
#elif L==6	// gn
	"Recibidos"	// Okoteve traduccin
#elif L==7	// it
	"Ricevuti"
#elif L==8	// pl
	"Odebrane"
#elif L==9	// pt
	"Recebidas"
#endif
	,
	//  6: ActSeeSntMsg
#if   L==1	// ca
	"Enviats"
#elif L==2	// de
	"Gesendete"
#elif L==3	// en
	"Sent"
#elif L==4	// es
	"Enviados"
#elif L==5	// fr
	"Envoy&eacute;s"
#elif L==6	// gn
	"Enviados"	// Okoteve traduccin
#elif L==7	// it
	"Inviati"
#elif L==8	// pl
	"Wysylane"
#elif L==9	// pt
	"Enviadas"
#endif
	,
	//  7: ActMaiStd
#if   L==1	// ca
	"Correu"
#elif L==2	// de
	"E-Mail"
#elif L==3	// en
	"Email"
#elif L==4	// es
	"Correo"
#elif L==5	// fr
	"Courrier"
#elif L==6	// gn
	"Correo"	// Okoteve traduccin
#elif L==7	// it
	"Email"
#elif L==8	// pl
	"Email"
#elif L==9	// pt
	"Email"
#endif
	,
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabAna **********************************************************************
	//  0: ActReqUseGbl
#if   L==1	// ca
	"Xifres"
#elif L==2	// de
	"Zahlen"
#elif L==3	// en
	"Figures"
#elif L==4	// es
	"Cifras"
#elif L==5	// fr
	"Chiffres"
#elif L==6	// gn
	"Cifras"	// Okoteve traduccin
#elif L==7	// it
	"Cifre"
#elif L==8	// pl
	"Cyfry"
#elif L==9	// pt
	"N&uacute;meros"
#endif
	,
	//  1: ActSeePhoDeg
#if   L==1	// ca
	"Titulacions"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"Degrees"
#elif L==4	// es
	"Titulaciones"
#elif L==5	// fr
	"&Eacute;tudes"
#elif L==6	// gn
	"Arandur&atilde;"
#elif L==7	// it
	"Lauree"
#elif L==8	// pl
	"Stopnie"
#elif L==9	// pt
	"Graus"
#endif
	,
	//  2: ActReqStaCrs
#if   L==1	// ca
	"Indicadors"
#elif L==2	// de
	"Anzeigen"
#elif L==3	// en
	"Indicators"
#elif L==4	// es
	"Indicadores"
#elif L==5	// fr
	"Indicateurs"
#elif L==6	// gn
	"Indicadores"	// Okoteve traduccin
#elif L==7	// it
	"Indicatori"
#elif L==8	// pl
	"Wska&zacute;niki"
#elif L==9	// pt
	"Indicadores"
#endif
	,
	//  3: ActReqAccGbl
#if   L==1	// ca
	"Accessos"
#elif L==2	// de
	"Anmeldungen"
#elif L==3	// en
	"Visits"
#elif L==4	// es
	"Accesos"
#elif L==5	// fr
	"Visites"
#elif L==6	// gn
	"Accesos"	// Okoteve traduccin
#elif L==7	// it
	"Accessi"
#elif L==8	// pl
	"Logowanie"
#elif L==9	// pt
	"Visitas"
#endif
	,
	//  4: ActReqMyUsgRep
#if   L==1	// ca
	"Informe"
#elif L==2	// de
	"Nutzungsbericht"
#elif L==3	// en
	"Report"
#elif L==4	// es
	"Informe"
#elif L==5	// fr
	"Rapport"
#elif L==6	// gn
	"Informe"	// Okoteve traduccin
#elif L==7	// it
	"Rapporto"
#elif L==8	// pl
	"Raport"
#elif L==9	// pt
	"Relat&oacute;rio"
#endif
	,
	//  5: ActMFUAct
#if   L==1	// ca
	"Freq&uuml;ents"
#elif L==2	// de
	"H&auml;ufige"
#elif L==3	// en
	"Frequent"
#elif L==4	// es
	"Frecuentes"
#elif L==5	// fr
	"Frequent"
#elif L==6	// gn
	"Frecuentes"	// Okoteve traduccin
#elif L==7	// it
	"Frequenti"
#elif L==8	// pl
	"Cz&eogon;ste"
#elif L==9	// pt
	"Freq&uuml;entes"
#endif
	,
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabPrf *****************************************************
	//  0: ActFrmRolSes
#if   L==1	// ca
	"Sessi&oacute;"
#elif L==2	// de
	"Sitzung"
#elif L==3	// en
	"Session"
#elif L==4	// es
	"Sesi&oacute;n"
#elif L==5	// fr
	"Session"
#elif L==6	// gn
	"Sesi&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Sessione"
#elif L==8	// pl
	"Sesja"
#elif L==9	// pt
	"Sess&atilde;o"
#endif
	,
	//  1: ActMyCrs
#if   L==1	// ca
	"Assignatures"
#elif L==2	// de
	"Kursen"
#elif L==3	// en
	"Courses"
#elif L==4	// es
	"Asignaturas"
#elif L==5	// fr
	"Mati&egrave;res"
#elif L==6	// gn
	"Mbo'esyry"
#elif L==7	// it
	"Corsi"
#elif L==8	// pl
	"Kurs&oacute;w"
#elif L==9	// pt
	"Disciplinas"
#endif
	,
	//  2: ActSeeMyTT
#if   L==1	// ca
	"Horari"
#elif L==2	// de
	"Stundenplan"
#elif L==3	// en
	"Timetable"
#elif L==4	// es
	"Horario"
#elif L==5	// fr
	"Horaire"
#elif L==6	// gn
	"Horario"	// Okoteve traduccin
#elif L==7	// it
	"Orario"
#elif L==8	// pl
	"Plan"
#elif L==9	// pt
	"Hor&aacute;rio"
#endif
	,
	//  3: ActSeeMyAgd
#if   L==1	// ca
	"Agenda"
#elif L==2	// de
	"Organizer"
#elif L==3	// en
	"Agenda"
#elif L==4	// es
	"Agenda"
#elif L==5	// fr
	"Agenda"
#elif L==6	// gn
	"Aporyr&atilde;"
#elif L==7	// it
	"Agenda"
#elif L==8	// pl
	"Pami&eogon;tnik"
#elif L==9	// pt
	"Agenda"
#endif
	,
	//  4: ActFrmMyAcc
#if   L==1	// ca
	"Compte"
#elif L==2	// de
	"Benutzerkonto"
#elif L==3	// en
	"Account"
#elif L==4	// es
	"Cuenta"
#elif L==5	// fr
	"Compte"
#elif L==6	// gn
	"Cuenta"	// Okoteve traduccin
#elif L==7	// it
	"Account"
#elif L==8	// pl
	"Konto"
#elif L==9	// pt
	"Conta"
#endif
	,
	//  5: ActReqEdiRecSha
#if   L==1	// ca
	"Fitxa"
#elif L==2	// de
	"Profil"
#elif L==3	// en
	"Record"
#elif L==4	// es
	"Ficha"
#elif L==5	// fr
	"Carte"
#elif L==6	// gn
	"Ficha"	// Okoteve traduccin
#elif L==7	// it
	"Scheda"
#elif L==8	// pl
	"Karta"
#elif L==9	// pt
	"Cart&atilde;o"
#endif
	,
	//  6: ActReqEdiPrf
#if   L==1	// ca
	"Configuraci&oacute;"
#elif L==2	// de
	"Einstellungen"
#elif L==3	// en
	"Settings"
#elif L==4	// es
	"Ajustes"
#elif L==5	// fr
	"R&eacute;glages"
#elif L==6	// gn
	"Mbohoryha"
#elif L==7	// it
	"Impostazioni"
#elif L==8	// pl
	"Ustawienia"
#elif L==9	// pt
	"Configura&ccedil;&atilde;o"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	}
};

const char *Txt_MENU_SUBTITLE[Tab_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB] =
{
	{
	// TabUnk ****************************************************
	NULL,	//  0
	NULL,	//  1
	NULL,	//  2
	NULL,	//  3
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabSta *****************************************************
	//  0: ActFrmLogIn
#if   L==1	// ca
	"Rol en aquesta sessi&oacute;"
#elif L==2	// de
	"Rolle in dieser Sitzung"
#elif L==3	// en
	"Role in this session"
#elif L==4	// es
	"Rol en esta sesi&oacute;n"
#elif L==5	// fr
	"R&ocirc;le dans cette session"
#elif L==6	// gn
	"Rol en esta sesi&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ruolo in questa sessione"
#elif L==8	// pl
	"Rola w tej sesji"
#elif L==9	// pt
	"Papel nesta sess&atilde;o"
#endif
	,
	//  1: ActReqSch
#if   L==1	// ca
	"Cerca d'assignatures, professors i documents"
#elif L==2	// de
	"Suche Kursen, Lehrer und Dokumente"
#elif L==3	// en
	"Search subjects, teachers and documents"
#elif L==4	// es
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==5	// fr
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==6	// gn
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==7	// it
	"Cercare corsi, professori e documenti"
#elif L==8	// pl
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==9	// pt
	"Procurar disciplinas, professores e documentos"
#endif
	,
	//  2: ActSeeSocTmlGbl
#if   L==1	// ca
	 "Timeline dels usuaris que segueixo o de tots els usuaris"
#elif L==2	// de
	 "Timeline der Benutzer Ich folge oder alle Benutzern"
#elif L==3	// en
	 "Timeline of users I follow or all users"
#elif L==4	// es
	 "Timeline de los usuarios que sigo o de todos los usuarios"
#elif L==5	// fr
	 "Chronologie des utilisateurs que je suis ou de tous les utilisateurs"
#elif L==6	// gn
	 "Timeline de los usuarios que sigo o de todos los usuarios"	// Okoteve traduccin
#elif L==7	// it
	 "Timeline degli utenti che seguo o tutti gli utenti"
#elif L==8	// pl
	 "Timeline u&zdot;ytkownik&oacute;w &sacute;ledz&eogon; lub wszystkich u&zdot;ytkownik&oacute;w"
#elif L==9	// pt
	 "Timeline dos usu&aacute;rios que eu sigo ou de todos os usu&aacute;rios"
#endif
	,
	 //  3: ActSeeSocPrf
#if   L==1	// ca
	 "Perfils p&uacute;blics"
#elif L==2	// de
	 "&Ouml;ffentliche Profile"
#elif L==3	// en
	 "Public profiles"
#elif L==4	// es
	 "Perfiles p&uacute;blicos de usuarios"
#elif L==5	// fr
	 "Profils publics"
#elif L==6	// gn
	 "Perfiles p&uacute;blicos de usuarios"	// Okoteve traduccin
#elif L==7	// it
	 "Profili pubblici"
#elif L==8	// pl
	 "Profile publiczne"
#elif L==9	// pt
	 "Perfis p&uacute;blicos"
#endif
	,
	//  4: ActSeeCal
#if   L==1	// ca
	"Calendari acad&egrave;mic"
#elif L==2	// de
	"Kalender"
#elif L==3	// en
	"Academic calendar"
#elif L==4	// es
	"Calendario acad&eacute;mico"
#elif L==5	// fr
	"Calendrier"
#elif L==6	// gn
	"Calendario acad&eacute;mico"// Okoteve traduccin
#elif L==7	// it
	"Calendario del corso"
#elif L==8	// pl
	"Kalendarz akademicki"
#elif L==9	// pt
	"Calend&aacute;rio"
#endif
	,
	//  5: ActSeeNtf
#if   L==1	// ca
	"Esdeveniments recents que han estat o seran notificats"
#elif L==2	// de
	"Die j&uuml;ngsten Ereignisse wurden oder werden benachrichtigt"
#elif L==3	// en
	"Recent events that have been or will be notified"
#elif L==4	// es
	"Eventos recientes que han sido o ser&aacute;n notificados"
#elif L==5	// fr
	"Les &eacute;v&eacute;nements r&eacute;cents qui ont &eacute;t&eacute; ou seront avis&eacute;s"
#elif L==6	// gn
	"Eventos recientes que han sido o ser&aacute;n notificados"// Okoteve traduccin
#elif L==7	// it
	"Recenti eventi che sono stati o saranno notificati"
#elif L==8	// pl
	"Ostatnie wydarzenia, kt&oacute;re zostaly lub zostana powiadomieni"
#elif L==9	// pt
	"Os recentes acontecimentos que foram ou ser&atilde;o notificados"
#endif
	,
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabSys *****************************************************
	//  0: ActSeeCty
#if   L==1	// ca
	"Llista de pa&iuml;sos i del nombre d'usuaris de cada pa&iacute;s"
#elif L==2	// de
	"L&auml;nder"
#elif L==3	// en
	"Countries"
#elif L==4	// es
	"Lista de pa&iacute;ses y del n&uacute;mero de usuarios de cada pais"
#elif L==5	// fr
	"Pays"
#elif L==6	// gn
	"Lista de pa&iacute;ses y del n&uacute;mero de usuarios de cada pais"// Okoteve traduccin
#elif L==7	// it
	"Paesi"
#elif L==8	// pl
	"Kraje"
#elif L==9	// pt
	"Pa&iacute;ses"
#endif
	,
	//  1: ActSeePen
#if   L==1	// ca
	"Institutions, centres, titulacions i assignatures pendents d'activar"
#elif L==2	// de
	"Anh&auml;ngig Hochschulen, Lehrinstitute, Studiengang und Kursen"
#elif L==3	// en
	"Pending institutions, centres, degrees and courses"
#elif L==4	// es
	"Instituciones, centros, titulaciones y asignaturas pendientes de activar"
#elif L==5	// fr
	"&Eacute;tablissements, centres, &eacute;tudes et mati&egrave;res en attente"
#elif L==6	// gn
	"Instituciones, centros, titulaciones y asignaturas pendientes de activar"	// Okoteve traduccin
#elif L==7	// it
	"Istituzioni, centri, lauree e corsi in sospeso"
#elif L==8	// pl
	"Instytucje, centra, stopnie i kurs&otilde;w oczekuj&aogon;cy"
#elif L==9	// pt
	"Institu&ccedil;&otilde;es, centros, graus e disciplinas pendentes"
#endif
	,
	//  2: ActSeeLnk
#if   L==1	// ca
	"Llista d'enlla&ccedil;os institucionals"
#elif L==2	// de
	"Offizielle Links Liste"
#elif L==3	// en
	"Institutional links list"
#elif L==4	// es
	"Lista de enlaces institucionales"
#elif L==5	// fr
	"Liens institutionnels"
#elif L==6	// gn
	"Lista de enlaces institucionales"// Okoteve traduccin
#elif L==7	// it
	"Link istituzionali"
#elif L==8	// pl
	"Instytucjonalne linki"
#elif L==9	// pt
	"Liga&ccedil;&otilde;es institucionais"
#endif
	,
	//  3: ActLstPlg
#if   L==1	// ca
	"Llista de complements (versi&oacute; beta)"
#elif L==2	// de
	"Plugin-Liste"
#elif L==3	// en
	"List of plugins"
#elif L==4	// es
	"Lista de complementos (versi&oacute;n beta)"
#elif L==5	// fr
	"Liste de plugins"
#elif L==6	// gn
	"Lista de complementos (versi&oacute;n beta)"// Okoteve traduccin
#elif L==7	// it
	"Lista di plugin"
#elif L==8	// pl
	"Lista plugin&oacute;w"
#elif L==9	// pt
	"Lista de plugins"
#endif
	,
	//  4: ActSetUp
#if   L==1	// ca
	"Instal laci&oacute; de la plataforma (en proves)"
#elif L==2	// de
	"Einrichten"
#elif L==3	// en
	"Set up"
#elif L==4	// es
	"Instalaci&oacute;n de la plataforma (en pruebas)"
#elif L==5	// fr
	"Installer"
#elif L==6	// gn
	"Instalaci&oacute;n de la plataforma (en pruebas)"// Okoteve traduccin
#elif L==7	// it
	"Setup"
#elif L==8	// pl
	"Set up"
#elif L==9	// pt
	"Instalar"
#endif
	,
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabCty *****************************************************
	//  0: ActSeeCtyInf
#if   L==1	// ca
	"Informaci&oacute; d'aquest pa&iacute;s"
#elif L==2	// de
	"Information dieses Land"
#elif L==3	// en
	"Information on this country"
#elif L==4	// es
	"Informaci&oacute;n sobre este pa&iacute;s"
#elif L==5	// fr
	"Information de cet pays"
#elif L==6	// gn
	"Informaci&oacute;n sobre este pa&iacute;s"// Okoteve traduccin
#elif L==7	// it
	"Informazione di questo paese"
#elif L==8	// pl
	"Informacja na ten kraj"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o deste pa&iacute;s"
#endif
	,
	//  1: ActSeeIns
#if   L==1	// ca
	"Llista d'institucions"
#elif L==2	// de
	"Hochschulen"
#elif L==3	// en
	"List of institutions of the current country"
#elif L==4	// es
	"Lista de instituciones (universidades, institutos&hellip;) de este pa&iacute;s"
#elif L==5	// fr
	"&Eacute;tablissement"
#elif L==6	// gn
	"Lista de instituciones (universidades, institutos&hellip;) de este pa&iacute;s"// Okoteve traduccin
#elif L==7	// it
	"Istituzioni"
#elif L==8	// pl
	"Instytucje"
#elif L==9	// pt
	"Institu&ccedil;&otilde;es"
#endif
	,
	NULL,	//  2
	NULL,	//  3
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabIns *****************************************************
	//  0: ActSeeInsInf
#if   L==1	// ca
	"Informaci&oacute; d'aquesta instituci&oacute;"
#elif L==2	// de
	"Information dieses Hochschule"
#elif L==3	// en
	"Information on this institution"
#elif L==4	// es
	"Informaci&oacute;n sobre esta instituci&oacute;n"
#elif L==5	// fr
	"Information de cet &eacute;tablissement"
#elif L==6	// gn
	"Informaci&oacute;n sobre esta instituci&oacute;n"// Okoteve traduccin
#elif L==7	// it
	"Informazione di questa istituzione"
#elif L==8	// pl
	"Informacja na ten instytucja"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o desta institu&ccedil;&atilde;o"
#endif
	,
	//  1: ActSeeCtr
#if   L==1	// ca
	"Llista de centres"
#elif L==2	// de
	"Lehrinstitute"
#elif L==3	// en
	"List of centres of the current institution"
#elif L==4	// es
	"Lista de centros (facultades, escuelas&hellip;) de esta instituci&oacute;n"
#elif L==5	// fr
	"Centres"
#elif L==6	// gn
	"Lista de centros (facultades, escuelas&hellip;) de esta instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Centri"
#elif L==8	// pl
	"Centra"
#elif L==9	// pt
	"Centros"
#endif
	,
	//  2: ActSeePlc
#if   L==1	// ca
	"Llocs o seus de la instituci&oacute;"
	" on s'ubica cada un dels centres."
#elif L==2	// de
	"Standorte oder Hauptsitze der Institution,"
	" an der sich jedes Zentrum befindet."
#elif L==3	// en
	"Places or headquarters of the institution"
	" where each of the centres is located."
#elif L==4	// es
	"Lugares o sedes de la instituci&oacute;n"
	" donde se ubica cada uno de los centros."
#elif L==5	// fr
	"Emplacements ou si&egrave;ges de l'institution"
	" o&ugrave; est situ&eacute; chacun des centres."
#elif L==6	// gn
	"Lugares o sedes de la instituci&oacute;n"
	" donde se ubica cada uno de los centros."
#elif L==7	// it
	"Localit&grave; o sede dell'istituzione"
	" in cui si trova ciascun centro."
#elif L==8	// pl
	"Miejsca lub siedziba instytucji,"
	" w kt&oacute;rej znajduje si&eogon; ka&zdot;dy z o&sacute;rodk&oacute;w."
#elif L==9	// pt
	"Localiza&ccedil;oes ou sede da institui&ccedil;&atilde;o"
	" onde cada um dos centros est&aacute; localizado."
#endif
	,
	//  3: ActSeeDpt
#if   L==1	// ca
	"Llista de tots els departaments d'aquesta instituci&oacute;"
#elif L==2	// de
	"Abteilungen"
#elif L==3	// en
	"List all departments of this institution"
#elif L==4	// es
	"Lista de todos los departamentos de esta instituci&oacute;n"
#elif L==5	// fr
	"D&eacute;partements"
#elif L==6	// gn
	"Lista de todos los departamentos de esta instituci&oacute;n"// Okoteve traduccin
#elif L==7	// it
	"Dipartimenti"
#elif L==8	// pl
	"Wydzia&lstrok;y"
#elif L==9	// pt
	"Departamentos"
#endif
	,
	//  4: ActSeeHld
#if   L==1	// ca
	"Dies festius"
#elif L==2	// de
	"Feiertage"
#elif L==3	// en
	"Holidays"
#elif L==4	// es
	"D&iacute;as festivos"
#elif L==5	// fr
	"Jours f&eacute;ri&eacute;s"
#elif L==6	// gn
	"Arete"
#elif L==7	// it
	"Festivit&agrave;"
#elif L==8	// pl
	"Noclegi"
#elif L==9	// pt
	"F&eacute;rias"
#endif
	,
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabCtr *****************************************************
	//  0: ActSeeCtrInf
#if   L==1	// ca
	"Informaci&oacute; d'aquest centre"
#elif L==2	// de
	"Information dieses Lehrinstitut"
#elif L==3	// en
	"Information on this centre"
#elif L==4	// es
	"Informaci&oacute;n sobre este centro"
#elif L==5	// fr
	"Information de cet centre"
#elif L==6	// gn
	"Informaci&oacute;n sobre este centro"// Okoteve traduccin
#elif L==7	// it
	"Informazione di questo centro"
#elif L==8	// pl
	"Informacja na ten centrum"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o deste centro"
#endif
	,
	//  1: ActSeeDeg
#if   L==1	// ca
	"Titulacions d'aquest centre"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"List of degrees of the current centre"
#elif L==4	// es
	"Lista de titulaciones (carreras, m&aacute;steres&hellip;) de este centro"
#elif L==5	// fr
	"&Eacute;tudes"
#elif L==6	// gn
	"Lista de titulaciones (carreras, m&aacute;steres&hellip;) de este centro"// Okoteve traduccin
#elif L==7	// it
	"Lauree"
#elif L==8	// pl
	"Stopnie"
#elif L==9	// pt
	"Graus"
#endif
	,
	//  2: ActSeeCla
#if   L==1	// ca
	"Aules, laboratoris o altres llocs on s'imparteixen classes"
#elif L==2	// de
	"Klassenzimmer, Laboratorien oder andere Unterrichtsorte"
#elif L==3	// en
	"Classrooms, laboratories or other places where classes are taught"
#elif L==4	// es
	"Aulas, laboratorios u otros lugares donde se imparten clases"
#elif L==5	// fr
	"Salles de classe, laboratoires ou autres lieux de cours"
#elif L==6	// gn
	"Mbo'ehakoty"
#elif L==7	// it
	"Aule, laboratori o altri luoghi in cui vengono insegnate le lezioni"
#elif L==8	// pl
	"Klasy, laboratoria lub inne miejsca, w kt&oacute;rych odbywaj&aogon; si&eogon; zaj&eogon;cia"
#elif L==9	// pt
	"Salas de aula, laborat&oacute;rios ou outros locais onde as aulas s&atilde;o ministradas"
#endif
	,
	NULL,	//  3
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabDeg *****************************************************
	//  0: ActSeeDegInf
#if   L==1	// ca
	"Informaci&oacute; d'aquesta titulaci&oacute;"
#elif L==2	// de
	"Information dieses Studiengang"
#elif L==3	// en
	"Information on this degree"
#elif L==4	// es
	"Informaci&oacute;n sobre esta titulaci&oacute;n"
#elif L==5	// fr
	"Information de cet &eacute;tude"
#elif L==6	// gn
	"Informaci&oacute;n sobre esta titulaci&oacute;n"// Okoteve traduccin
#elif L==7	// it
	"Informazione di questa laurea"
#elif L==8	// pl
	"Informacja na ten stopie&nacute;"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o sobre este grau"
#endif
	,
	//  1: ActSeeCrs
#if   L==1	// ca
	"Assignatures d'aquesta titulaci&oacute;"
#elif L==2	// de
	"Kursen in diesem Studiengang"
#elif L==3	// en
	"List of courses of the current degree"
#elif L==4	// es
	"Lista de asignaturas de esta titulaci&oacute;n"
#elif L==5	// fr
	"Mati&egrave;res dans cet &eacute;tude"
#elif L==6	// gn
	"Lista de asignaturas de esta titulaci&oacute;n"// Okoteve traduccin
#elif L==7	// it
	"Corsi di questa laurea"
#elif L==8	// pl
	"Kurs&oacute;w w tym stopie&nacute;"
#elif L==9	// pt
	"Disciplinas deste grau"
#endif
	,
	NULL,	//  2
	NULL,	//  3
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabCrs *****************************************************
	//  0: ActSeeCrsInf
#if   L==1	// ca
	"Informaci&oacute; d'aquesta assignatura"
#elif L==2	// de
	"Information dieses Kurs"
#elif L==3	// en
	"Information on this course"
#elif L==4	// es
	"Informaci&oacute;n sobre esta asignatura"
#elif L==5	// fr
	"Information de cette mati&egrave;re"
#elif L==6	// gn
	"Informaci&oacute;n sobre esta asignatura"// Okoteve traduccin
#elif L==7	// it
	"Informazione di questo corso"
#elif L==8	// pl
	"Informacja na ten kurs"
#elif L==9	// pt
	"Informa&ccedil;&atilde;o desta disciplina"
#endif
	,
	//  1: ActSeeTchGui
#if   L==1	// ca
	"Guia docent de l'assignatura"
#elif L==2	// de
	"Hinweise zur Kurs"
#elif L==3	// en
	"Teaching guide"
#elif L==4	// es
	"Gu&iacute;a docente de la asignatura"
#elif L==5	// fr
	"Guide p&eacute;dagogique de la mati&egrave;re"
#elif L==6	// gn
	"Gu&iacute;a docente de la asignatura"// Okoteve traduccin
#elif L==7	// it
	"Guida didattica del corso"
#elif L==8	// pl
	"Nauczanie przypomnienie"
#elif L==9	// pt
	"Guia de ensino"
#endif
	,
	//  2: ActSeeSyl
#if   L==1	// ca
	"Programa de l'assignatura"
#elif L==2	// de
	"Studienplan"
#elif L==3	// en
	"Syllabus"
#elif L==4	// es
	"Programa de la asignatura"
#elif L==5	// fr
	"Plan"
#elif L==6	// gn
	"Programa de la asignatura"// Okoteve traduccin
#elif L==7	// it
	"Programma"
#elif L==8	// pl
	"Syllabus"
#elif L==9	// pt
	"Programa"
#endif
	,
	//  3: ActSeeBib
#if   L==1	// ca
	"Bibliografia relacionada amb els continguts de l'assignatura"
#elif L==2	// de
	"Literatur"
#elif L==3	// en
	"Bibliography"
#elif L==4	// es
	"Bibliograf&iacute;a relacionada con los contenidos de la asignatura"
#elif L==5	// fr
	"Bibliographie"
#elif L==6	// gn
	"Bibliograf&iacute;a relacionada con los contenidos de la asignatura"// Okoteve traduccin
#elif L==7	// it
	"Bibliografia"
#elif L==8	// pl
	"Bibliografia"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Bibliografia"
#endif
	,
	//  4: ActSeeFAQ
#if   L==1	// ca
	"FAQ (preguntes freq&uuml;entment formulades i les seves respostes)"
#elif L==2	// de
	"FAQ"
#elif L==3	// en
	"FAQ (Frequently Asked Questions)"
#elif L==4	// es
	"FAQ (preguntas frecuentemente formuladas y sus respuestas)"
#elif L==5	// fr
	"FAQ"
#elif L==6	// gn
	"FAQ (preguntas frecuentemente formuladas y sus respuestas)"// Okoteve traduccin
#elif L==7	// it
	"FAQ"
#elif L==8	// pl
	"FAQ (Frequently Asked Questions)"
#elif L==9	// pt
	"FAQ"
#endif
	,
	//  5: ActSeeCrsLnk
#if   L==1	// ca
	"Enlla&ccedil;os relacionats amb els continguts de l'assignatura"
#elif L==2	// de
	"Links"
#elif L==3	// en
	"Links"
#elif L==4	// es
	"Enlaces relacionados con los contenidos de la asignatura"
#elif L==5	// fr
	"Liens"
#elif L==6	// gn
	"Enlaces relacionados con los contenidos de la asignatura"// Okoteve traduccin
#elif L==7	// it
	"Link"
#elif L==8	// pl
	"Linki"
#elif L==9	// pt
	"Liga&ccedil;&otilde;es"
#endif
	,
	//  6: ActSeeCrsTT
#if   L==1	// ca
	"Horari de classes de l'assignatura"
#elif L==2	// de
	"Stundenplan"
#elif L==3	// en
	"Class timetable"
#elif L==4	// es
	"Horario de clases de la asignatura"
#elif L==5	// fr
	"Horaire"
#elif L==6	// gn
	"Horario de clases de la asignatura"	// Okoteve traduccin
#elif L==7	// it
	"Orario del corso"
#elif L==8	// pl
	"Harmonogram klasy"
#elif L==9	// pt
	"Hor&aacute;rio"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabAss *****************************************************
	//  0: ActSeeAss
#if   L==1	// ca
	"Sistema o criteris d'avaluaci&oacute;"
	" de l'estudiant en aquesta assignatura"
#elif L==2	// de
	"Bewertungssystem"
#elif L==3	// en
	"Assessment system"
#elif L==4	// es
	"Sistema o criterios de evaluaci&oacute;n"
	" del estudiante en esta asignatura"
#elif L==5	// fr
	"Syst&egrave;me d'&eacute;valuation"
#elif L==6	// gn
	"Sistema o criterios de evaluaci&oacute;n"
	" del estudiante en esta asignatura"// Okoteve traduccin
#elif L==7	// it
	"Sistema di valutazione"
#elif L==8	// pl
	"System oceny"
#elif L==9	// pt
	"Sistema de avalia&ccedil;&atilde;o"
#endif
	,
	//  1: ActSeeAsg
#if   L==1	// ca
	"Activitats obligat&ograve;ries o recomanades en aquesta assignatura"
#elif L==2	// de
	"Aufgaben"
#elif L==3	// en
	"Assignments"
#elif L==4	// es
	"Actividades obligatorias o recomendadas en esta asignatura"
#elif L==5	// fr
	"Activit&eacute;s"
#elif L==6	// gn
	"Actividades obligatorias o recomendadas en esta asignatura"// Okoteve traduccin
#elif L==7	// it
	"Attivit&agrave;"
#elif L==8	// pl
	"Zadania"
#elif L==9	// pt
	"Atividades"
#endif
	,
	//  2: ActSeePrj
#if   L==1	// ca
	"Projectes (treballs fi de grau, projectes final de carrera, tesi&hellip;)"
#elif L==2	// de
	"Projekte (Abschlussarbeiten, Diplomarbeiten&hellip;)"
#elif L==3	// en
	"Projects (end-of-degree projects, thesis&hellip;)"
#elif L==4	// es
	"Proyectos (trabajos fin de grado/m&aacute;ster, proyectos fin de carrera, tesis&hellip;)"
#elif L==5	// fr
	"Projets (projets de fin de cours, th&egrave;se&hellip;)"
#elif L==6	// gn
	"Proyectos (trabajos fin de grado/m&aacute;ster, proyectos fin de carrera, tesis&hellip;)"	// Okoteve traduccin
#elif L==7	// it
	"Progetti (progetti di fine corso, tesi&hellip;)"
#elif L==8	// pl
	"Projekty (projekty ko&nacute;cowe, praca dyplomowa&hellip;)"
#elif L==9	// pt
	"Projetos (projetos de fim de grau, tese&hellip;)"
#endif
	,
	//  3: ActReqTst
#if   L==1	// ca
	"Tests sobre els continguts d'aquesta assignatura"
#elif L==2	// de
	"Tests"
#elif L==3	// en
	"Tests on the contents of this course"
#elif L==4	// es
	"Test sobre los contenidos de esta asignatura"
#elif L==5	// fr
	"Tests sur le contenu de cette mati&egrave;re"
#elif L==6	// gn
	"Test sobre los contenidos de esta asignatura"// Okoteve traduccin
#elif L==7	// it
	"Test sui contenuti del corso"
#elif L==8	// pl
	"Test&oacute;w"
#elif L==9	// pt
	"Testes"
#endif
	,
	//  4: ActSeeAllGam
#if   L==1	// ca
	"Jocs (comandament a dist&agrave;ncia)"
#elif L==2	// de
	"Spiele (Fernbedienung)"
#elif L==3	// en
	"Games (remote control)"
#elif L==4	// es
	"Juegos (mando a distancia)"
#elif L==5	// fr
	"Jeux (t&eacute;l&eacute;commande)"
#elif L==6	// gn
	"Juegos (mando a distancia)"	// Okoteve traduccin
#elif L==7	// it
	"Giochi (telecomando)"
#elif L==8	// pl
	"Gry (pilot zdalnego sterowania)"
#elif L==9	// pt
	"Jogos (controlo remoto)"
#endif
	,
	//  5: ActSeeAllSvy
#if   L==1	// ca
	"Enquestes an&ograve;nimes"
#elif L==2	// de
	"Umfragen"
#elif L==3	// en
	"Surveys"
#elif L==4	// es
	"Encuestas an&oacute;nimas"
#elif L==5	// fr
	"Sondages"
#elif L==6	// gn
	"Encuestas an&oacute;nimas"	// Okoteve traduccin
#elif L==7	// it
	"Sondaggi"
#elif L==8	// pl
	"Ankiety"
#elif L==9	// pt
	"Inqu&eacute;ritos"
#endif
	,
	//  6: ActSeeAllExaAnn
#if   L==1	// ca
	"Llistat de convocat&ograve;ries d'ex&agrave;mens d'aquesta assignatura"
#elif L==2	// de
	"Pr&uuml;fungsaufruf"
#elif L==3	// en
	"Announcements of exams"
#elif L==4	// es
	"Listado de convocatorias de ex&aacute;menes de esta asignatura"
#elif L==5	// fr
	"Convocations &agrave; examens"
#elif L==6	// gn
	"Listado de convocatorias de ex&aacute;menes de esta asignatura"// Okoteve traduccin
#elif L==7	// it
	"Appelli d'esame"
#elif L==8	// pl
	"Ogloszenia egzamin&oacute;w"
#elif L==9	// pt
	"Chamadas para exames"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabFil *****************************************************
	//  0: ActSeeAdmDocIns
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents of the institution uploaded by administrators"
#elif L==4	// es
	"Documentos de la instituci&oacute;n subidos por administradores"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Documentos de la instituci&oacute;n subidos por administradores"	// Okoteve traduccin
#elif L==7	// it
	"Area documenti"
#elif L==8	// pl
	"Obszar Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#endif
	,
	//  1: ActAdmShaIns
#if   L==1	// ca
	"Arxius compartits per professors i estudiants"
	" de l'instituci&oacute;"
#elif L==2	// de
	"Freigegebene Dateien"
#elif L==3	// en
	"Files shared by teachers and students"
	" of the institution"
#elif L==4	// es
	"Archivos compartidos por profesores y estudiantes"
	" de la instituci&oacute;n"
#elif L==5	// fr
	"Fichiers partag&eacute;s"
#elif L==6	// gn
	"Archivos compartidos por profesores y estudiantes"
	" de la instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"File condivisi"
#elif L==8	// pl
	"Pliki udost&eogon;pnione"
#elif L==9	// pt
	"Arquivos compartilhados"
#endif
	,
	//  2: ActSeeAdmDocCtr
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents of the centre uploaded by administrators"
#elif L==4	// es
	"Documentos del centro subidos por administradores"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Documentos del centro subidos por administradores"	// Okoteve traduccin
#elif L==7	// it
	"Area documenti"
#elif L==8	// pl
	"Obszar Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#endif
	,
	//  3: ActAdmShaCtr
#if   L==1	// ca
	"Arxius compartits per professors i estudiants"
	" del centre"
#elif L==2	// de
	"Freigegebene Dateien"
#elif L==3	// en
	"Files shared by teachers and students"
	" of the centre"
#elif L==4	// es
	"Archivos compartidos por profesores y estudiantes"
	" del centro"
#elif L==5	// fr
	"Fichiers partag&eacute;s"
#elif L==6	// gn
	"Archivos compartidos por profesores y estudiantes"
	" del centro"	// Okoteve traduccin
#elif L==7	// it
	"File condivisi"
#elif L==8	// pl
	"Pliki udost&eogon;pnione"
#elif L==9	// pt
	"Arquivos compartilhados"
#endif
	,
	//  4: ActSeeAdmDocDeg
#if   L==1	// ca
	"Documents"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents of the degree uploaded by administrators"
#elif L==4	// es
	"Documentos de la titulaci&oacute;n subidos por administradores"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Documentos de la titulaci&oacute;n subidos por administradores"	// Okoteve traduccin
#elif L==7	// it
	"Area documenti"
#elif L==8	// pl
	"Obszar Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#endif
	,
	//  5: ActAdmShaDeg
#if   L==1	// ca
	"Arxius compartits per professors i estudiants"
	" de la titulaci&oacute;"
#elif L==2	// de
	"Freigegebene Dateien"
#elif L==3	// en
	"Files shared by teachers and students"
	" of the degree"
#elif L==4	// es
	"Archivos compartidos por profesores y estudiantes"
	" de la titulaci&oacute;n"
#elif L==5	// fr
	"Fichiers partag&eacute;s"
#elif L==6	// gn
	"Archivos compartidos por profesores y estudiantes"
	" de la titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"File condivisi"
#elif L==8	// pl
	"Pliki udost&eogon;pnione"
#elif L==9	// pt
	"Arquivos compartilhados"
#endif
	,
	//  6: ActSeeAdmDocCrsGrp
#if   L==1	// ca
	"Documents de l'assignatura o d'un grup enviats per professors"
#elif L==2	// de
	"Dokumente"
#elif L==3	// en
	"Documents of the course or a group uploaded by teachers"
#elif L==4	// es
	"Documentos de la asignatura o de un grupo subidos por profesores"
#elif L==5	// fr
	"Documents"
#elif L==6	// gn
	"Documentos de la asignatura o de un grupo subidos por profesores"// Okoteve traduccin
#elif L==7	// it
	"Area documenti"
#elif L==8	// pl
	"Obszar Dokument&oacute;w"
#elif L==9	// pt
	"Documentos"
#endif
	,
	//  7: ActAdmTchCrsGrp
#if   L==1	// ca
	"Arxius privats, nom&eacute;s per als professors de l'assignatura"
#elif L==2	// de
	"Privat-Dateien, nur f&uuml;r die Lehrer des Kurses"
#elif L==3	// en
	"Private files, only for teachers of the course"
#elif L==4	// es
	"Archivos privados, solo para los profesores de la asignatura"
#elif L==5	// fr
	"Fichiers priv&eacute;s, seulement pour les enseignants du cours"
#elif L==6	// gn
	"Archivos privados, solo para los profesores de la asignatura"// Okoteve traduccin
#elif L==7	// it
	"File privati, solo per gli insegnanti del corso"
#elif L==8	// pl
	"Pliki prywatne, tylko dla nauczycieli przedmiotu"
#elif L==9	// pt
	"Arquivos privados, apenas para os professores do curso"
#endif
	,
	//  8: ActAdmShaCrsGrp
#if   L==1	// ca
	"Arxius compartits per professors i estudiants"
	" de l'assignatura o d'un grup"
#elif L==2	// de
	"Freigegebene Dateien"
#elif L==3	// en
	"Files shared by teachers and students"
	" of the course or group"
#elif L==4	// es
	"Archivos compartidos por profesores y estudiantes"
	" de la asignatura o de un grupo"
#elif L==5	// fr
	"Fichiers partag&eacute;s"
#elif L==6	// gn
	"Archivos compartidos por profesores y estudiantes"
	" de la asignatura o de un grupo"// Okoteve traduccin
#elif L==7	// it
	"File condivisi"
#elif L==8	// pl
	"Pliki udost&eogon;pnione"
#elif L==9	// pt
	"Arquivos compartilhados"
#endif
	,
	//  9: ActAdmAsgWrkUsr
#if   L==1	// ca
	"Els meus documents d'activitats i altres treballs en aquesta assignatura"
#elif L==2	// de
	"Eingereichte Hausarbeiten"
#elif L==3	// en
	"My homework"
#elif L==4	// es
	"Mis documentos de actividades y otros trabajos en esta asignatura"
#elif L==5	// fr
	"Mes travaux"
#elif L==6	// gn
	"Mis documentos de actividades y otros trabajos en esta asignatura"// Okoteve traduccin
#elif L==7	// it
	"Miei lavori"
#elif L==8	// pl
	"Moje zadanie domowe"
#elif L==9	// pt
	"Os meus trabalhos"
#endif
	,
	// 10: ActReqAsgWrkCrs
#if   L==1	// ca
	"Activitats i altres treballs enviats pels usuaris d'aquesta assignatura"
#elif L==2	// de
	"Hausarbeiten der Benutzer"
#elif L==3	// en
	"Users' homework"
#elif L==4	// es
	"Actividades y otros trabajos enviados por los usuarios de esta asignatura"
#elif L==5	// fr
	"Travails d'utilisateurs"
#elif L==6	// gn
	"Actividades y otros trabajos enviados por los usuarios de esta asignatura"// Okoteve traduccin
#elif L==7	// it
	"Lavori di utenti"
#elif L==8	// pl
	"Uzytkownik&oacute;w domowych"
#elif L==9	// pt
	"Trabalhos de utilizadores"
#endif
	,
	// 11: ActSeeMrk
#if   L==1	// ca
	"Consulta individualitzada de qualificacions"
	" (cada estudiant nom&eacute;s veu les seves qualificacions)"
#elif L==2	// de
	"Individuelle Beratung von Bewertungen"
	" (jeder Student sieht nur seine Bewertungen)"
#elif L==3	// en
	"Individualized consultation of marks"
	"(each student only sees her/his marks)"
#elif L==4	// es
	"Consulta individualizada de calificaciones"
	" (cada estudiante solo ve sus calificaciones)"
#elif L==5	// fr
	"Consultation individualis&eacute; des notes"
	" (chaque &eacute;tudiant ne voit que ses notes)"
#elif L==6	// gn
	"Consulta individualizada de calificaciones"
	" (cada estudiante solo ve sus calificaciones)"	// Okoteve traduccin
#elif L==7	// it
	"Consultazione individualizzata dei risultati"
	" (ogni studente vede solo i suoi risultati)"
#elif L==8	// pl
	"Zindywidualizowane konsultacje znak&oacute;w"
	" (ka&zdot;dy student widzi tylko jego &sacute;lady)"
#elif L==9	// pt
	"Consulta individualizada de notas"
	" (somente cada estudante v&ecirc; suas notas)"
#endif
	,
	// 12: ActAdmBrf
#if   L==1	// ca
	"La meva zona privada d'emmagatzematge de fitxers"
#elif L==2	// de
	"Aktentasche"
#elif L==3	// en
	"Virtual pendrive"
#elif L==4	// es
	"Mi zona privada de almacenamiento de archivos"
#elif L==5	// fr
	"Portedocuments"
#elif L==6	// gn
	"Mi zona privada de almacenamiento de archivos"	// Okoteve traduccin
#elif L==7	// it
	"Penna USB"
#elif L==8	// pl
	"Pendrive wirtualne"
#elif L==9	// pt
	"Pasta"
#endif
	},
	{
	// TabUsr *****************************************************
	//  0: ActReqSelGrp
#if   L==1	// ca
	"Grups en aquesta assignatura"
#elif L==2	// de
	"Gruppen"
#elif L==3	// en
	"Groups"
#elif L==4	// es
	"Grupos en esta asignatura"
#elif L==5	// fr
	"Groupes"
#elif L==6	// gn
	"Grupos en esta asignatura"	// Okoteve traduccin
#elif L==7	// it
	"Gruppi"
#elif L==8	// pl
	"Grupy"
#elif L==9	// pt
	"Grupos"
#endif
	,
	//  1: ActLstStd
#if   L==1	// ca
	"Llistat i fitxes dels estudiants"
#elif L==2	// de
	"Studentenverzeichnis und profile"
#elif L==3	// en
	"Students' list and records"
#elif L==4	// es
	"Listado y fichas de los estudiantes"
#elif L==5	// fr
	"Liste et cartes d'&eacute;tudiants"
#elif L==6	// gn
	"Listado y fichas de los estudiantes"// Okoteve traduccin
#elif L==7	// it
	"Elenco e schede studenti"
#elif L==8	// pl
	"Listy i rekordy student&oacute;w"
#elif L==9	// pt
	"Lista e cart&otilde;es estudantes"
#endif
	,
	//  2: ActLstTch
#if   L==1	// ca
	"Llistat i fitxes de professors"
#elif L==2	// de
	"Lehrkr&auml;fteverzeichnis und profile"
#elif L==3	// en
	"Teachers' list and records"
#elif L==4	// es
	"Listado y fichas de profesores"
#elif L==5	// fr
	"Liste et cartes d'enseignants"
#elif L==6	// gn
	"Listado y fichas de profesores"// Okoteve traduccin
#elif L==7	// it
	"Elenco e schede professori"
#elif L==8	// pl
	"Lista i rekordy nauczycieli"
#elif L==9	// pt
	"Lista e cart&otilde;es professores"
#endif
	,
	//  3: ActLstOth
#if   L==1	// ca
	"Llistat i fitxes d'altres usuaris"
#elif L==2	// de
	"Eingeladenverzeichnis und profile von anderen Benutzer"
#elif L==3	// en
	"Other users' list and records"
#elif L==4	// es
	"Listado y fichas de otros usuarios"
#elif L==5	// fr
	"Liste et cartes d'autres utilisateurs"
#elif L==6	// gn
	"Listado y fichas de otros usuarios"	// Okoteve traduccin
#elif L==7	// it
	"Elenco e schede da altri utenti"
#elif L==8	// pl
	"Listy i rekordy innych u&zdot;ytkownik&oacute;w"
#elif L==9	// pt
	"Lista e cart&otilde;es de outros usu&aacute;rios"
#endif
	,
	//  4: ActSeeAtt
#if   L==1	// ca
	"Control d'assist&egrave;ncia dels estudiants"
	" a les classes d'aquesta assignatura"
#elif L==2	// de
	"Kontrolle der Studierenden Anwesenheit"
#elif L==3	// en
	"Control of students' attendance"
	" to classes in this course"
#elif L==4	// es
	"Control de asistencia de los estudiantes"
	" a las clases de esta asignatura"
#elif L==5	// fr
	"Contr&ocirc;le de pr&eacute;sence des &eacute;tudiants"
#elif L==6	// gn
	"Control de asistencia de los estudiantes"
	" a las clases de esta asignatura"// Okoteve traduccin
#elif L==7	// it
	"Controllo di frequenza degli studenti"
#elif L==8	// pl
	"Kontrola frekwencji"
#elif L==9	// pt
	"Lista de presen&ccedil;a"
#endif
	,
	//  5: ActReqSignUp
#if   L==1	// ca
	"Sol&middot;licitar inscripci&oacute;"
#elif L==2	// de
	"Registrierung beantragen"
#elif L==3	// en
	"Sign up in this course"
#elif L==4	// es
	"Solicitar mi inscripci&oacute;n en esta asignatura"
#elif L==5	// fr
	"Demander inscription"
#elif L==6	// gn
	"Solicitar mi inscripci&oacute;n en esta asignatura"// Okoteve traduccin
#elif L==7	// it
	"Richiedere iscrizione"
#elif L==8	// pl
	"Wniosku rejestracj&eogon;"
#elif L==9	// pt
	"Solicitar inscri&ccedil;&atilde;o"
#endif
	,
	//  6: ActSeeSignUpReq
#if   L==1	// ca
	"Peticions d'inscripci&oacute;"
#elif L==2	// de
	"Registrierungsanforderungen"
#elif L==3	// en
	"Enrolment requests in this course pending verification"
#elif L==4	// es
	"Peticiones de inscripci&oacute;n pendientes de verificar"
#elif L==5	// fr
	"Demandes d'inscription"
#elif L==6	// gn
	"Peticiones de inscripci&oacute;n pendientes de verificar"	// Okoteve traduccin
#elif L==7	// it
	"Richieste di iscrizione"
#elif L==8	// pl
	"&Zdot;&aogon;dania rejestrowania"
#elif L==9	// pt
	"Solicita&ccedil;&otilde;es de inscri&ccedil;&atilde;o"
#endif
	,
	//  7: ActLstCon
#if   L==1	// ca
	"Llista d'usuaris connectats en aquest moment"
#elif L==2	// de
	"Angemeldet"
#elif L==3	// en
	"Connected"
#elif L==4	// es
	"Lista de usuarios conectados en este momento"
#elif L==5	// fr
	"Connect&eacute;s"
#elif L==6	// gn
	"Lista de usuarios conectados en este momento"// Okoteve traduccin
#elif L==7	// it
	"Connessi"
#elif L==8	// pl
	"Online"
#elif L==9	// pt
	"Conectados"
#endif
	,
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabMsg *****************************************************
	//  0: ActSeeAnn
#if   L==1	// ca
	"Anuncis globals mostrat en iniciar sessi&oacute;"
#elif L==2	// de
	"Globale Bekanntmachungen bei der Anmeldung sichtbar"
#elif L==3	// en
	"Global announcements visible at login"
#elif L==4	// es
	"Anuncios globales visibles al iniciar sesi&oacute;n"
#elif L==5	// fr
	"Annonces globales visibles lors de la connexion"
#elif L==6	// gn
	"Anuncios globales visibles al iniciar sesi&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Annunci globali visibili al login"
#elif L==8	// pl
	"Globalne og&lstrok;oszenia widoczne przy logowaniu"
#elif L==9	// pt
	"An&uacute;ncios globais vis&iacute;veis no in&iacute;cio da sess&atilde;o"
#endif
	,
	//  1: ActSeeAllNot
#if   L==1	// ca
	"Breus avisos relacionats amb aquesta assignatura,"
	" visibles per qualsevol visitant, encara que no estigui identificat"
#elif L==2	// de
	"Ank&uuml;ndigungen"
#elif L==3	// en
	"Notices"
#elif L==4	// es
	"Breves avisos relacionados con esta asignatura,"
	" visibles por cualquier visitante, aunque no est&eacute; identificado"
#elif L==5	// fr
	"Avis"
#elif L==6	// gn
	"Breves avisos relacionados con esta asignatura,"
	" visibles por cualquier visitante, aunque no est&eacute; identificado"	// Okoteve traduccin
#elif L==7	// it
	"Avvisi"
#elif L==8	// pl
	"Ogloszenia"
#elif L==9	// pt
	"Avisos"
#endif
	,
	//  2: ActSeeFor
#if   L==1	// ca
	"F&ograve;rums de discussi&oacute;, distribu&iuml;ts"
	" segons la tem&agrave;tica (escolliu el f&ograve;rum adequat)"
#elif L==2	// de
	"Foren"
#elif L==3	// en
	"Forums"
#elif L==4	// es
	"Foros de discusi&oacute;n, distribuidos"
	" seg&uacute;n la tem&aacute;tica (elija el foro adecuado)"
#elif L==5	// fr
	"Forums"
#elif L==6	// gn
	"Foros de discusi&oacute;n, distribuidos"
	" seg&uacute;n la tem&aacute;tica (elija el foro adecuado)"// Okoteve traduccin
#elif L==7	// it
	"Forum"
#elif L==8	// pl
	"Forum"
#elif L==9	// pt
	"F&oacute;runs"
#endif
	,
	//  3: ActSeeChtRms
#if   L==1	// ca
	"Conversa en temps real amb altres usuaris"
#elif L==2	// de
	"Chat"
#elif L==3	// en
	"Chat"
#elif L==4	// es
	"Conversaci&oacute;n en tiempo real con otros usuarios"
#elif L==5	// fr
	"Chat"
#elif L==6	// gn
	"Conversaci&oacute;n en tiempo real con otros usuarios"	// Okoteve traduccin
#elif L==7	// it
	"Chat"
#elif L==8	// pl
	"Chat"
#elif L==9	// pt
	"Chat"
#endif
	,
	//  4: ActReqMsgUsr
#if   L==1	// ca
	"Enviament d'un nou missatge a altres usuaris"
#elif L==2	// de
	"Neue Nachricht an andere Benutzer senden"
#elif L==3	// en
	"Sending a new message to other users"
#elif L==4	// es
	"Env&iacute;o de un nuevo mensaje a otros usuarios"
#elif L==5	// fr
	"Envoi d'un nouveau message &agrave; d'autres utilisateurs"
#elif L==6	// gn
	"Env&iacute;o de un nuevo mensaje a otros usuarios"// Okoteve traduccin
#elif L==7	// it
	"Invio di un nuovo messaggio ad altri utenti"
#elif L==8	// pl
	"Wysy&lstrok;anie nowej wiadomo&sacute;ci do innych u&zdot;ytkownik&oacute;w"
#elif L==9	// pt
	"Envio de uma nova mensagem para outros usu&aacute;rios"
#endif
	,
	//  5: ActSeeRcvMsg
#if   L==1	// ca
	"Missatges rebuts d'altres usuaris"
#elif L==2	// de
	"Nachrichten, die von anderen Benutzern empfangen wurden"
#elif L==3	// en
	"Messages received from other users"
#elif L==4	// es
	"Mensajes recibidos de otros usuarios"
#elif L==5	// fr
	"Messages re&ccedil;us d'autres utilisateurs"
#elif L==6	// gn
	"Mensajes recibidos de otros usuarios"	// Okoteve traduccin
#elif L==7	// it
	"Messaggi ricevuti da altri utenti"
#elif L==8	// pl
	"Wiadomo&sacute;ci otrzymane od innych u&zdot;ytkownik&oacute;w"
#elif L==9	// pt
	"Mensagens recebidas de outros usu&aacute;rios"
#endif
	,
	//  6: ActSeeSntMsg
#if   L==1	// ca
	"Missatges enviats per mi a altres usuaris"
#elif L==2	// de
	"Nachrichten, die ich an andere Benutzer gesendet habe"
#elif L==3	// en
	"Messages sent by me to other users"
#elif L==4	// es
	"Mensajes enviados por m&iacute; a otros usuarios"
#elif L==5	// fr
	"Messages envoy&eacute;s par moi &agrave; d'autres utilisateurs"
#elif L==6	// gn
	"Mensajes enviados por m&iacute; a otros usuarios"// Okoteve traduccin
#elif L==7	// it
	"Messaggi inviati da me ad altri utenti"
#elif L==8	// pl
	"Wiadomo&sacute;ci wysy&lstrok;ane przeze mnie do innych u&zdot;ytkownik&oacute;w"
#elif L==9	// pt
	"Mensagens enviadas por mim para outros usu&aacute;rios"
#endif
	,
	//  7: ActMaiStd
#if   L==1	// ca
	"Creaci&oacute; d'un missatge de correu"
	" per enviar-lo a estudiants d'aquesta assignatura"
	" utilitzant el programa client de correu predeterminat en el meu sistema"
#elif L==2	// de
	"E-Mail"
#elif L==3	// en
	"Email"
#elif L==4	// es
	"Creaci&oacute;n de un mensaje de correo"
	" para enviarlo a estudiantes de esta asignatura"
	" usando el programa cliente de correo predeterminado en mi sistema"
#elif L==5	// fr
	"Courrier &eacute;lectronique"
#elif L==6	// gn
	"Creaci&oacute;n de un mensaje de correo"
	" para enviarlo a estudiantes de esta asignatura"
	" usando el programa cliente de correo predeterminado en mi sistema"// Okoteve traduccin
#elif L==7	// it
	"Email"
#elif L==8	// pl
	"Email"
#elif L==9	// pt
	"Email"
#endif
	,
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabAna *****************************************************
	//  0: ActReqUseGbl
#if   L==1	// ca
	"Grans nombres sobre l'&uacute;s actual de tota la plataforma,"
	" o d'aquesta titulaci&oacute; o assignatura"
#elif L==2	// de
	"Nutzung von SWAD"
#elif L==3	// en
	"Use of SWAD"
#elif L==4	// es
	"Grandes n&uacute;meros sobre el uso actual de toda la plataforma,"
	" o de esta titulaci&oacute;n o asignatura"
#elif L==5	// fr
	"Utilisation de SWAD"
#elif L==6	// gn
	"Grandes n&uacute;meros sobre el uso actual de toda la plataforma,"
	" o de esta titulaci&oacute;n o asignatura"// Okoteve traduccin
#elif L==7	// it
	"Uso di SWAD"
#elif L==8	// pl
	"Korzystanie z SWAD"
#elif L==9	// pt
	"Utiliza&ccedil;&atilde;o de SWAD"
#endif
	,
	//  1: ActSeePhoDeg
#if   L==1	// ca
	"Estad&iacute;stiques del nombre d'estudiants en cada titulaci&oacute;"
	" representades per les fotos mitjana dels estudiants"
#elif L==2	// de
	"Mischbilder der Studieng&auml;nge"
#elif L==3	// en
	"Photos of degrees"
#elif L==4	// es
	"Estad&iacute;sticas del n&uacute;mero de estudiantes en cada titulaci&oacute;n"
	" representadas por las fotos promedio de los estudiantes"
#elif L==5	// fr
	"Photographie &eacute;tudes"
#elif L==6	// gn
	"Estad&iacute;sticas del n&uacute;mero de estudiantes en cada titulaci&oacute;n"
	" representadas por las fotos promedio de los estudiantes"// Okoteve traduccin
#elif L==7	// it
	"Foto di lauree"
#elif L==8	// pl
	"Zdjecia stopni"
#elif L==9	// pt
	"Estat&iacute;sticas do n&uacute;mero de estudantes em cada grau"
	" representadas pela m&acute;dia das fotos dos estudantes"
#endif
	,
	//  2: ActReqStaCrs
#if   L==1	// ca
	"Indicadors d'assignatures per al contracte-programa"
#elif L==2	// de
	"Anzeigen von Kursen f&uuml;r den Vertrag Programm"
#elif L==3	// en
	"Indicators of courses for the contract program"
#elif L==4	// es
	"Indicadores de asignaturas para el contrato-programa"
#elif L==5	// fr
	"Indicateurs de mati&egrave;res pour le contrat-programme"
#elif L==6	// gn
	"Indicadores de asignaturas para el contrato-programa"// Okoteve traduccin
#elif L==7	// it
	"Indicatori di corsi per il contratto programma"
#elif L==8	// pl
	"Wska&zacute;niki kurs&oacute;w dla umowy programu"
#elif L==9	// pt
	"Indicadores de disciplinas para o contrato programa"
#endif
	,
	//  3: ActReqAccGbl
#if   L==1	// ca
	"Estad&iacute;stiques d'accesso a la plataforma"
#elif L==2	// de
	"Zugriffsstatistik auf die Plattform"
#elif L==3	// en
	"Statistics of access to the platform"
#elif L==4	// es
	"Estad&iacute;sticas de acceso a la plataforma"
#elif L==5	// fr
	"Statistiques de connexion &agrave; la plate-forme"
#elif L==6	// gn
	"Estad&iacute;sticas de acceso a la plataforma"	// Okoteve traduccin
#elif L==7	// it
	"Statistiche di accesso alla piattaforma"
#elif L==8	// pl
	"Statystyki dost&eogon;p do platformy"
#elif L==9	// pt
	"Estat&iacute;sticas de acesso &agrave; plataforma"
#endif
	,
	//  4: ActReqMyUsgRep
#if   L==1	// ca
	"Informe sobre el meu &uacute;s de la plataforma"
#elif L==2	// de
	"Bericht &uuml;ber meine Nutzung der Plattform"
#elif L==3	// en
	"Report on my use of the platform"
#elif L==4	// es
	"Informe sobre mi uso de la plataforma"
#elif L==5	// fr
	"Rapport sur mon utilisation de la plate-forme"
#elif L==6	// gn
	"Informe sobre mi uso de la plataforma"	// Okoteve traduccin
#elif L==7	// it
	"Relazione sulla mia utilizzo della piattaforma"
#elif L==8	// pl
	"Raport o moim korzystania z platformy"
#elif L==9	// pt
	"Relat&oacute;rio sobre o meu uso da plataforma"
#endif
	,
	//  5: ActMFUAct
#if   L==1	// ca
	"Acc&eacute;s directe a les meves accions m&eacute;s freq&uuml;ents"
#elif L==2	// de
	"Direkter Zugriff auf meine h&auml;ufige Aktionen"
#elif L==3	// en
	"Direct access to my most frequent actions"
#elif L==4	// es
	"Acceso directo a mis acciones m&aacute;s frecuentes"
#elif L==5	// fr
	"Acc&egrave;s direct &agrave; mes actions les plus fr&eacute;quentes"
#elif L==6	// gn
	"Acceso directo a mis acciones m&aacute;s frecuentes"// Okoteve traduccin
#elif L==7	// it
	"Accesso diretto alle mie azioni pi&ugrave; frequenti"
#elif L==8	// pl
	"Bezpo&sacute;redni dost&eogon;p do najcz&eogon;stszych dzia&lstrok;a&nacute;"
#elif L==9	// pt
	"Acesso direto das minhas a&ccedil;&otilde;es mais freq&uuml;entes"
#endif
	,
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	},
	{
	// TabPrf *****************************************************
	//  0: ActFrmRolSes
#if   L==1	// ca
	"Rol en aquesta sessi&oacute;"
#elif L==2	// de
	"Rolle in dieser Sitzung"
#elif L==3	// en
	"Role in this session"
#elif L==4	// es
	"Rol en esta sesi&oacute;n"
#elif L==5	// fr
	"R&ocirc;le dans cette session"
#elif L==6	// gn
	"Rol en esta sesi&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ruolo in questa sessione"
#elif L==8	// pl
	"Rola w tej sesji"
#elif L==9	// pt
	"Papel nesta sess&atilde;o"
#endif
	,
	//  1: ActMyCrs
#if   L==1	// ca
	"Assignatures a les quals pertanyo"
#elif L==2	// de
	"Kursen der ich angeh&ouml;re"
#elif L==3	// en
	"Courses to which I belong"
#elif L==4	// es
	"Asignaturas a las que pertenezco"
#elif L==5	// fr
	"Mati&egrave;res &agrave; laquelle je appartiens"
#elif L==6	// gn
	"Asignaturas a las que pertenezco"	// Okoteve traduccin
#elif L==7	// it
	"Corsi a cui appartengo"
#elif L==8	// pl
	"Kursy do kt&oacute;rego nale&zdot;&eogon;"
#elif L==9	// pt
	"Disciplinas a que perten&ccedil;o"
#endif
	,
	//  2: ActSeeMyTT
#if   L==1	// ca
	"Horari conjunt de totes les meves assignatures a la plataforma"
#elif L==2	// de
	"Stundenplan"
#elif L==3	// en
	"My timetable"
#elif L==4	// es
	"Horario conjunto de todas mis asignaturas en la plataforma"
#elif L==5	// fr
	"Horaire mes mati&egrave;res"
#elif L==6	// gn
	"Horario conjunto de todas mis asignaturas en la plataforma"// Okoteve traduccin
#elif L==7	// it
	"Orario dei miei corsi"
#elif L==8	// pl
	"M&oacute;j plan lekcji"
#elif L==9	// pt
	"Hor&aacute;rio minha disciplinas"
#endif
	,
	//  3: ActSeeMyAgd
#if   L==1	// ca
	"Agenda"
#elif L==2	// de
	"Organizer"
#elif L==3	// en
	"Agenda (personal organizer)"
#elif L==4	// es
	"Agenda"
#elif L==5	// fr
	"Agenda"
#elif L==6	// gn
	"Aporyr&atilde;"
#elif L==7	// it
	"Agenda"
#elif L==8	// pl
	"Pami&eogon;tnik"
#elif L==9	// pt
	"Agenda"
#endif
	,
	//  4: ActFrmMyAcc
#if   L==1	// ca
	"Creaci&oacute; o edici&oacute; del meu compte d'usuari"
#elif L==2	// de
	"Erstellen oder Bearbeiten meinem Benutzerkonto"
#elif L==3	// en
	"Create or edit my user account"
#elif L==4	// es
	"Creaci&oacute;n o edici&oacute;n de mi cuenta de usuario/a"
#elif L==5	// fr
	"Cr&eacute;er ou modifier mon compte d'utilisateur"
#elif L==6	// gn
	"Creaci&oacute;n o edici&oacute;n de mi cuenta de usuario/a"// Okoteve traduccin
#elif L==7	// it
	"Creare o modificare il mio account utente"
#elif L==8	// pl
	"Utworzy&cacute; lub edytowa&cacute; konto u&zdot;ytkownika"
#elif L==9	// pt
	"Criar ou editar minha conta de usu&aacute;rio"
#endif
	,
	//  5: ActReqEdiRecSha
#if   L==1	// ca
	"Dades personals, comuns per a totes les assignatures"
#elif L==2	// de
	"Profil"
#elif L==3	// en
	"Record"
#elif L==4	// es
	"Datos personales, comunes para todas las asignaturas"
#elif L==5	// fr
	"Carte"
#elif L==6	// gn
	"Datos personales, comunes para todas las asignaturas"// Okoteve traduccin
#elif L==7	// it
	"Scheda"
#elif L==8	// pl
	"Karta"
#elif L==9	// pt
	"Cart&atilde;o"
#endif
	,
	//  6: ActReqEdiPrf
#if   L==1	// ca
	"Les meves prefer&egrave;ncies (idioma, colors, notificacions &hellip;)"
#elif L==2	// de
	"Einstellungen"
#elif L==3	// en
	"Preferences"
#elif L==4	// es
	"Mis preferencias (idioma, colores, notificaciones&hellip;)"
#elif L==5	// fr
	"Pr&eacute;f&eacute;rences"
#elif L==6	// gn
	"Mis preferencias (idioma, colores, notificaciones&hellip;)"	// Okoteve traduccin
#elif L==7	// it
	"Preferenze"
#elif L==8	// pl
	"Preferencje"
#elif L==9	// pt
	"Prefer&ecirc;ncias"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	NULL,	// 12
	}
};

const char *Txt_message =
#if   L==1	// ca
	"missatge";
#elif L==2	// de
	"Nachricht";
#elif L==3	// en
	"message";
#elif L==4	// es
	"mensaje";
#elif L==5	// fr
	"message";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"messaggio";
#elif L==8	// pl
	"wiadomosc";
#elif L==9	// pt
	"mensagem";
#endif

const char *Txt_Message_deleted =
#if   L==1	// ca
	"Mensaje eliminado.";		// Necessita traduccio
#elif L==2	// de
	"Nachricht entfernt.";
#elif L==3	// en
	"Message deleted.";
#elif L==4	// es
	"Mensaje eliminado.";
#elif L==5	// fr
	"Message deleted.";		// Besoin de traduction
#elif L==6	// gn
	"Mensaje eliminado.";		// Okoteve traduccin
#elif L==7	// it
	"Messaggio rimosso.";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; usunieta.";
#elif L==9	// pt
	"Mensagem removida.";
#endif

const char *Txt_message_not_sent_to_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Missatge no enviat a %s.";
#elif L==2	// de
	"Nachricht nicht gesendet, um %s.";
#elif L==3	// en
	"Message not sent to %s.";
#elif L==4	// es
	"Mensaje no enviado a %s.";
#elif L==5	// fr
	"Message non envoy&eacute; &agrave; %s.";
#elif L==6	// gn
	"Mensaje no enviado a %s.";	// Okoteve traduccin
#elif L==7	// it
	"Messaggio non inviato a %s.";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; nie zostala wyslana do %s.";
#elif L==9	// pt
	"Mensagem n&atilde;o enviada para %s.";
#endif

const char *Txt_message_sent_to_X_not_notified_by_email =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Mensaje enviado a %s (no notificado por email).";	// Necessita traduccio
#elif L==2	// de
	"Message sent to %s (not notified by email).";	// Need bersetzung
#elif L==3	// en
	"Message sent to %s (not notified by email).";
#elif L==4	// es
	"Mensaje enviado a %s (no notificado por email).";
#elif L==5	// fr
	"Message sent to %s (not notified by email).";	// Besoin de traduction
#elif L==6	// gn
	"Mensaje enviado a %s (no notificado por email).";	// Okoteve traduccin
#elif L==7	// it
	"Messaggio inviato a %s (non notificato per email).";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; wyslana do %s (nie zgloszonych przez email).";	//
#elif L==9	// pt
	"Mensagem enviada para %s (n&atilde;o notificada por email).";
#endif

const char *Txt_message_sent_to_X_notified_by_email =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Mensaje enviado a %s (notificado por email).";	// Necessita traduccio
#elif L==2	// de
	"Message sent to %s (notified by email).";	// Need bersetzung
#elif L==3	// en
	"Message sent to %s (notified by email).";
#elif L==4	// es
	"Mensaje enviado a %s (notificado por email).";
#elif L==5	// fr
	"Message sent to %s (notified by email).";	// Besoin de traduction
#elif L==6	// gn
	"Mensaje enviado a %s (notificado por email).";	// Okoteve traduccin
#elif L==7	// it
	"Messaggio inviato a %s (notificato per email).";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; wyslana do %s (zgloszone przez email).";
#elif L==9	// pt
	"Mensagem enviada para %s (notificada por email).";
#endif

const char *Txt_Messages =
#if   L==1	// ca
	"Missatges";
#elif L==2	// de
	"Nachrichten";
#elif L==3	// en
	"Messages";
#elif L==4	// es
	"Mensajes";
#elif L==5	// fr
	"Messages";
#elif L==6	// gn
	"Marandu";
#elif L==7	// it
	"Messaggi";
#elif L==8	// pl
	"Wiadomo&sacute;ci";
#elif L==9	// pt
	"Mensagens";
#endif

const char *Txt_messages =
#if   L==1	// ca
	"missatges";
#elif L==2	// de
	"Nachrichten";
#elif L==3	// en
	"messages";
#elif L==4	// es
	"mensajes";
#elif L==5	// fr
	"messages";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"messaggi";
#elif L==8	// pl
	"wiadomo&sacute;ci";
#elif L==9	// pt
	"mensagens";
#endif

const char *Txt_X_messages_have_been_deleted =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Se han eliminado %u mensajes.";	// Necessita traduccio
#elif L==2	// de
	"%u messages have been deleted.";	// Need bersetzung
#elif L==3	// en
	"%u messages have been deleted.";
#elif L==4	// es
	"Se han eliminado %u mensajes.";
#elif L==5	// fr
	"%u messages have been deleted.";	// Besoin de traduction
#elif L==6	// gn
	"Se han eliminado %u mensajes.";	// Okoteve traduccin
#elif L==7	// it
	"%u messaggi sono stati rimossi.";
#elif L==8	// pl
	"wiadomosci% u zostaly usuniete.";
#elif L==9	// pt
	"%u mensagens foram removidas.";
#endif

const char *Txt_Messages_of_THE_USER_X_have_been_deleted =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Se han eliminado los mensajes de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Messages of <strong>%s</strong> have been deleted.";	// Need bersetzung
#elif L==3	// en
	"Messages of <strong>%s</strong> have been deleted.";
#elif L==4	// es
	"Se han eliminado los mensajes de <strong>%s</strong>.";
#elif L==5	// fr
	"Messages of <strong>%s</strong> have been deleted.";	// Besoin de traduction
#elif L==6	// gn
	"Se han eliminado los mensajes de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Messaggi de <strong>%s</strong> sono stati rimossi.";
#elif L==8	// pl
	"Wiadomo&sacute;ci od <strong>%s</strong> zostaly usuniete.";
#elif L==9	// pt
	"Mensagens de <strong>%s</strong> foram removidas.";
#endif

const char *Txt_message_received =
#if   L==1	// ca
	"missatge rebut";
#elif L==2	// de
	"Nachricht empfangen";
#elif L==3	// en
	"message received";
#elif L==4	// es
	"mensaje recibido";
#elif L==5	// fr
	"message re&ccedil;u";
#elif L==6	// gn
	"mensaje recibido";	// Okoteve traduccin
#elif L==7	// it
	"messaggio ricevuto";
#elif L==8	// pl
	"odebranej wiadomo&sacute;ci";
#elif L==9	// pt
	"mensagem recebida";
#endif

const char *Txt_messages_received =
#if   L==1	// ca
	"missatges rebuts";
#elif L==2	// de
	"empfangene Nachrichten";
#elif L==3	// en
	"messages received";
#elif L==4	// es
	"mensajes recibidos";
#elif L==5	// fr
	"messages re&ccedil;us";
#elif L==6	// gn
	"mensajes recibidos";	// Okoteve traduccin
#elif L==7	// it
	"messaggi ricevuti";
#elif L==8	// pl
	"wiadomo&sacute;ci odebrane";
#elif L==9	// pt
	"mensagens recebidas";
#endif

const char *Txt_Messages_received_from_A_COURSE =
#if   L==1	// ca
	"Missatges rebuts des de";
#elif L==2	// de
	"Empfangene Nachrichten aus";
#elif L==3	// en
	"Messages received from";
#elif L==4	// es
	"Mensajes recibidos desde";
#elif L==5	// fr
	"Messages re&ccedil;us de";
#elif L==6	// gn
	"Mensajes recibidos desde";	// Okoteve traduccin
#elif L==7	// it
	"Messaggi ricevuti da";
#elif L==8	// pl
	"Wiadomo&sacute;ci odebrane";
#elif L==9	// pt
	"Mensagens recebidas de";
#endif

const char *Txt_message_sent =
#if   L==1	// ca
	"missatge enviat";
#elif L==2	// de
	"Nachricht gesendet";
#elif L==3	// en
	"message sent";
#elif L==4	// es
	"mensaje enviado";
#elif L==5	// fr
	"message envoy&eacute;";
#elif L==6	// gn
	"mensaje enviado";	// Okoteve traduccin
#elif L==7	// it
	"messaggio inviato";
#elif L==8	// pl
	"wiadomo&sacute;&cacute; wys&lstrok;ana";
#elif L==9	// pt
	"mensagem enviada";
#endif

const char *Txt_messages_sent =
#if   L==1	// ca
	"missatges enviats";
#elif L==2	// de
	"gesendete Nachrichten";
#elif L==3	// en
	"messages sent";
#elif L==4	// es
	"mensajes enviados";
#elif L==5	// fr
	"messages envoy&eacute;s";
#elif L==6	// gn
	"mensajes enviados";	// Okoteve traduccin
#elif L==7	// it
	"messaggi inviati";
#elif L==8	// pl
	"wiadomo&sacute;ci wysylane";
#elif L==9	// pt
	"mensagens enviadas";
#endif

const char *Txt_Messages_sent =
#if   L==1	// ca
	"Missatges enviats";
#elif L==2	// de
	"Gesendete Nachrichten";
#elif L==3	// en
	"Messages sent";
#elif L==4	// es
	"Mensajes enviados";
#elif L==5	// fr
	"Messages envoy&eacute;s";
#elif L==6	// gn
	"Mensajes enviados";	// Okoteve traduccin
#elif L==7	// it
	"Messaggi inviati";
#elif L==8	// pl
	"Wiadomo&sacute;ci wysylane";
#elif L==9	// pt
	"Mensagens enviadas";
#endif

const char *Txt_Messages_sent_by_teachers =
#if   L==1	// ca
	"Missatges enviats per professors";
#elif L==2	// de
	"Gesendete Nachrichten, die von Lehrkr&auml;ften";
#elif L==3	// en
	"Messages sent by teachers";
#elif L==4	// es
	"Mensajes enviados por profesores";
#elif L==5	// fr
	"Messages envoy&eacute;s par enseignants";
#elif L==6	// gn
	"Mensajes enviados por profesores";	// Okoteve traduccin
#elif L==7	// it
	"Messaggi inviati dai professori";
#elif L==8	// pl
	"Wiadomo&sacute;ci wysylane przez nauczycieli";
#elif L==9	// pt
	"Mensagens enviadas por professores";
#endif

const char *Txt_Messages_sent_from_A_COURSE =
#if   L==1	// ca
	"Missatges enviats des de";
#elif L==2	// de
	"Gesendete Nachrichten aus";
#elif L==3	// en
	"Messages sent from";
#elif L==4	// es
	"Mensajes enviados desde";
#elif L==5	// fr
	"Messages envoy&eacute;s de";
#elif L==6	// gn
	"Mensajes enviados desde";	// Okoteve traduccin
#elif L==7	// it
	"Messaggi inviati da";
#elif L==8	// pl
	"Wiadomo&sacute;ci wyslane z";
#elif L==9	// pt
	"Mensagens enviadas de";
#endif

const char *Txt_minimum =
#if   L==1	// ca
	"m&iacute;nimo";	// Necessita traduccio
#elif L==2	// de
	"Mindest";
#elif L==3	// en
	"minimum";
#elif L==4	// es
	"m&iacute;nimo";
#elif L==5	// fr
	"minimum";
#elif L==6	// gn
	"m&iacute;nimo";	// Okoteve traduccin
#elif L==7	// it
	"minimo";
#elif L==8	// pl
	"minimum";
#elif L==9	// pt
	"m&iacute;nimo";
#endif

const char *Txt_Minimum_time_seconds_per_question_between_two_tests =
#if   L==1	// ca
	"Temps m&iacute;nim (segons)<br />per pregunta<br />entre dues tests";
#elif L==2	// de
	"Minimale Zeit (Sekunden)<br />pro Frage<br />zwischen zwei Tests";
#elif L==3	// en
	"Minimum time (seconds)<br />per question<br />between two tests";
#elif L==4	// es
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos test";
#elif L==5	// fr
	"Minimum de temps (en secondes)<br />par question<br />entre deux tests";
#elif L==6	// gn
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos test";	// Okoteve traduccin
#elif L==7	// it
	"Minimo tempo (in secondi)<br />per domanda<br />tra due test";
#elif L==8	// pl
	"Minimalny czas (w sekundach)<br />na pytanie miedzy<br />dwoma badaniami";
#elif L==9	// pt
	"Tempo m&iacute;nimo (segundos)<br />por quest&atilde;o<br />entre dois testes";
#endif

const char *Txt_minute =
#if   L==1	// ca
	"minuto";	// Necessita traduccio
#elif L==2	// de
	"Minute";
#elif L==3	// en
	"minute";
#elif L==4	// es
	"minuto";
#elif L==5	// fr
	"minute";
#elif L==6	// gn
	"aravo'i";
#elif L==7	// it
	"minuto";
#elif L==8	// pl
	"minut&eogon;";
#elif L==9	// pt
	"minuto";
#endif

const char *Txt_minutes =
#if   L==1	// ca
	"minutos";	// Necessita traduccio
#elif L==2	// de
	"Minute";
#elif L==3	// en
	"minutes";
#elif L==4	// es
	"minutos";
#elif L==5	// fr
	"minutes";
#elif L==6	// gn
	"aravo'i";
#elif L==7	// it
	"minuti";
#elif L==8	// pl
	"minuty";
#elif L==9	// pt
	"minutos";
#endif

const char *Txt_Modify_me_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Modificarme en l'assignatura <strong>%s</strong>";
#elif L==2	// de
	"&Auml;ndern Sie mir im Kurs <strong>%s</strong>";
#elif L==3	// en
	"Modify me in the course <strong>%s</strong>";
#elif L==4	// es
	"Modificarme en la asignatura <strong>%s</strong>";
#elif L==5	// fr
	"Modifier-moi dans la mati&egrave;re <strong>%s</strong>";
#elif L==6	// gn
	"Modificarme en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Modificami utente nel corso <strong>%s</strong>";
#elif L==8	// pl
	"Modyfikacja mnie w kurs <strong>%s</strong>";
#elif L==9	// pt
	"Modificar me na disciplina <strong>%s</strong>";
#endif

const char *Txt_Modify_user_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Modificar usuari en l'assignatura <strong>%s</strong>";
#elif L==2	// de
	"&Auml;ndern Benutzer im Kurs <strong>%s</strong>";
#elif L==3	// en
	"Modify user in the course <strong>%s</strong>";
#elif L==4	// es
	"Modificar usuario en la asignatura <strong>%s</strong>";
#elif L==5	// fr
	"Modifier utilisateur dans la mati&egrave;re <strong>%s</strong>";
#elif L==6	// gn
	"Modificar usuario en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Modifica utente nel corso <strong>%s</strong>";
#elif L==8	// pl
	"Modyfikacja u&zdot;ytkownika w kurs <strong>%s</strong>";
#elif L==9	// pt
	"Modificar utilizador na disciplina <strong>%s</strong>";
#endif

const char *Txt_Month =
#if   L==1	// ca
	"Mes";
#elif L==2	// de
	"Monat";
#elif L==3	// en
	"Month";
#elif L==4	// es
	"Mes";
#elif L==5	// fr
	"Mois";
#elif L==6	// gn
	"Jasy";
#elif L==7	// it
	"Mese";
#elif L==8	// pl
	"Miesi&aogon;c";
#elif L==9	// pt
	"M&ecirc;s";
#endif

const char *Txt_month =
#if   L==1	// ca
	"mes";
#elif L==2	// de
	"Monat";
#elif L==3	// en
	"month";
#elif L==4	// es
	"mes";
#elif L==5	// fr
	"mois";
#elif L==6	// gn
	"jasy";
#elif L==7	// it
	"mese";
#elif L==8	// pl
	"miesi&aogon;c";
#elif L==9	// pt
	"m&ecirc;s";
#endif

const char *Txt_months =
#if   L==1	// ca
	"mesos";
#elif L==2	// de
	"Monate";
#elif L==3	// en
	"months";
#elif L==4	// es
	"meses";
#elif L==5	// fr
	"mois";
#elif L==6	// gn
	"meses";	// Okoteve traduccin
#elif L==7	// it
	"mesi";
#elif L==8	// pl
	"miesi&aogon;ce";
#elif L==9	// pt
	"meses";
#endif

const char *Txt_MONTHS_SMALL[12] =
   {
#if   L==1	// ca
	 "gener"
#elif L==2	// de
	 "Januar"
#elif L==3	// en
	 "January"
#elif L==4	// es
	 "enero"
#elif L==5	// fr
	 "janvier"
#elif L==6	// gn
	 "jasyte&#297;"
#elif L==7	// it
	 "gennaio"
#elif L==8	// pl
	 "Stycznia"
#elif L==9	// pt
	 "janeiro"
#endif
	 ,
#if   L==1	// ca
	 "febrer"
#elif L==2	// de
	 "Februar"
#elif L==3	// en
	 "February"
#elif L==4	// es
	 "febrero"
#elif L==5	// fr
	 "f&eacute;vrier"
#elif L==6	// gn
	 "jasyk&otilde;i"
#elif L==7	// it
	 "febbraio"
#elif L==8	// pl
	 "Lutego"
#elif L==9	// pt
	 "fevereiro"
#endif
	 ,
#if   L==1	// ca
	 "mar&ccedil;"
#elif L==2	// de
	 "M;&auml;rz"
#elif L==3	// en
	 "March"
#elif L==4	// es
	 "marzo"
#elif L==5	// fr
	 "mars"
#elif L==6	// gn
	 "jasyapy"
#elif L==7	// it
	 "marzo"
#elif L==8	// pl
	 "Marca"
#elif L==9	// pt
	 "mar&ccedil;o"
#endif
	 ,
#if   L==1	// ca
	 "abril"
#elif L==2	// de
	 "April"
#elif L==3	// en
	 "April"
#elif L==4	// es
	 "abril"
#elif L==5	// fr
	 "avril"
#elif L==6	// gn
	 "jasyrundy"
#elif L==7	// it
	 "aprile"
#elif L==8	// pl
	 "Kwietnia"
#elif L==9	// pt
	 "abril"
#endif
	 ,
#if   L==1	// ca
	 "maig"
#elif L==2	// de
	 "Mai"
#elif L==3	// en
	 "May"
#elif L==4	// es
	 "mayo"
#elif L==5	// fr
	 "mai"
#elif L==6	// gn
	 "jasypo"
#elif L==7	// it
	 "maggio"
#elif L==8	// pl
	 "Maja"
#elif L==9	// pt
	 "maio"
#endif
	 ,
#if   L==1	// ca
	 "juny"
#elif L==2	// de
	 "Juni"
#elif L==3	// en
	 "June"
#elif L==4	// es
	 "junio"
#elif L==5	// fr
	 "juin"
#elif L==6	// gn
	 "jasypote&#297;"
#elif L==7	// it
	 "giugno"
#elif L==8	// pl
	 "Czerwca"
#elif L==9	// pt
	 "junho"
#endif
	 ,
#if   L==1	// ca
	 "juliol"
#elif L==2	// de
	 "Juli"
#elif L==3	// en
	 "July"
#elif L==4	// es
	 "julio"
#elif L==5	// fr
	 "juillet"
#elif L==6	// gn
	 "jasypok&otilde;i"
#elif L==7	// it
	 "luglio"
#elif L==8	// pl
	 "Lipca"
#elif L==9	// pt
	 "julho"
#endif
	 ,
#if   L==1	// ca
	 "agost"
#elif L==2	// de
	 "August"
#elif L==3	// en
	 "August"
#elif L==4	// es
	 "agosto"
#elif L==5	// fr
	 "ao&ucirc;t"
#elif L==6	// gn
	 "jasypoapy"
#elif L==7	// it
	 "agosto"
#elif L==8	// pl
	 "Sierpnia"
#elif L==9	// pt
	 "agosto"
#endif
	 ,
#if   L==1	// ca
	 "setembre"
#elif L==2	// de
	 "September"
#elif L==3	// en
	 "September"
#elif L==4	// es
	 "septiembre"
#elif L==5	// fr
	 "septembre"
#elif L==6	// gn
	 "jasyporundy"
#elif L==7	// it
	 "settembre"
#elif L==8	// pl
	 "Wrze&sacute;nia"
#elif L==9	// pt
	 "setembro"
#endif
	 ,
#if   L==1	// ca
	 "octubre"
#elif L==2	// de
	 "Oktober"
#elif L==3	// en
	 "October"
#elif L==4	// es
	 "octubre"
#elif L==5	// fr
	 "octobre"
#elif L==6	// gn
	 "jasypa"
#elif L==7	// it
	 "ottobre"
#elif L==8	// pl
	 "Pa&zacute;dziernika"
#elif L==9	// pt
	 "outubro"
#endif
	 ,
#if   L==1	// ca
	 "novembre"
#elif L==2	// de
	 "November"
#elif L==3	// en
	 "November"
#elif L==4	// es
	 "noviembre"
#elif L==5	// fr
	 "novembre"
#elif L==6	// gn
	 "jasypate&#297;"
#elif L==7	// it
	 "novembre"
#elif L==8	// pl
	 "Listopada"
#elif L==9	// pt
	 "novembro"
#endif
	 ,
#if   L==1	// ca
	 "desembre"
#elif L==2	// de
	 "Dezember"
#elif L==3	// en
	 "December"
#elif L==4	// es
	 "diciembre"
#elif L==5	// fr
	 "d&eacute;cembre"
#elif L==6	// gn
	 "jasypak&otilde;i"
#elif L==7	// it
	 "dicembre"
#elif L==8	// pl
	 "Grudnia"
#elif L==9	// pt
	 "dezembro"
#endif
	};

const char *Txt_MONTHS_SMALL_SHORT[12] =
   {
#if   L==1	// ca
	 "gen"
#elif L==2	// de
	 "Jan"
#elif L==3	// en
	 "Jan"
#elif L==4	// es
	 "ene"
#elif L==5	// fr
	 "jan"
#elif L==6	// gn
	 "ene"	// Okoteve traduccin
#elif L==7	// it
	 "gen"
#elif L==8	// pl
	 "Sty"
#elif L==9	// pt
	 "jan"
#endif
	 ,
#if   L==1	// ca
	 "feb"
#elif L==2	// de
	 "Feb"
#elif L==3	// en
	 "Feb"
#elif L==4	// es
	 "feb"
#elif L==5	// fr
	 "f&eacute;v"
#elif L==6	// gn
	 "feb"	// Okoteve traduccin
#elif L==7	// it
	 "feb"
#elif L==8	// pl
	 "Lut"
#elif L==9	// pt
	 "fev"
#endif
	 ,
#if   L==1	// ca
	 "mar"
#elif L==2	// de
	 "M;&auml;r"
#elif L==3	// en
	 "Mar"
#elif L==4	// es
	 "mar"
#elif L==5	// fr
	 "mar"
#elif L==6	// gn
	 "mar"	// Okoteve traduccin
#elif L==7	// it
	 "mar"
#elif L==8	// pl
	 "Mar"
#elif L==9	// pt
	 "mar"
#endif
	 ,
#if   L==1	// ca
	 "abr"
#elif L==2	// de
	 "Apr"
#elif L==3	// en
	 "Apr"
#elif L==4	// es
	 "abr"
#elif L==5	// fr
	 "avr"
#elif L==6	// gn
	 "abr"	// Okoteve traduccin
#elif L==7	// it
	 "apr"
#elif L==8	// pl
	 "Kwi"
#elif L==9	// pt
	 "abr"
#endif
	 ,
#if   L==1	// ca
	 "mai"
#elif L==2	// de
	 "Mai"
#elif L==3	// en
	 "May"
#elif L==4	// es
	 "may"
#elif L==5	// fr
	 "mai"
#elif L==6	// gn
	 "may"	// Okoteve traduccin
#elif L==7	// it
	 "mag"
#elif L==8	// pl
	 "Maj"
#elif L==9	// pt
	 "mai"
#endif
	 ,
#if   L==1	// ca
	 "jun"
#elif L==2	// de
	 "Jun"
#elif L==3	// en
	 "Jun"
#elif L==4	// es
	 "jun"
#elif L==5	// fr
	 "jui"
#elif L==6	// gn
	 "jun"	// Okoteve traduccin
#elif L==7	// it
	 "giu"
#elif L==8	// pl
	 "Cze"
#elif L==9	// pt
	 "jun"
#endif
	 ,
#if   L==1	// ca
	 "jul"
#elif L==2	// de
	 "Jul"
#elif L==3	// en
	 "Jul"
#elif L==4	// es
	 "jul"
#elif L==5	// fr
	 "jui"
#elif L==6	// gn
	 "jul"	// Okoteve traduccin
#elif L==7	// it
	 "lug"
#elif L==8	// pl
	 "Lip"
#elif L==9	// pt
	 "jul"
#endif
	 ,
#if   L==1	// ca
	 "ago"
#elif L==2	// de
	 "Aug"
#elif L==3	// en
	 "Aug"
#elif L==4	// es
	 "ago"
#elif L==5	// fr
	 "ao&ucirc;"
#elif L==6	// gn
	 "ago"	// Okoteve traduccin
#elif L==7	// it
	 "ago"
#elif L==8	// pl
	 "Sie"
#elif L==9	// pt
	 "ago"
#endif
	 ,
#if   L==1	// ca
	 "set"
#elif L==2	// de
	 "Sep"
#elif L==3	// en
	 "Sep"
#elif L==4	// es
	 "sep"
#elif L==5	// fr
	 "sep"
#elif L==6	// gn
	 "sep"	// Okoteve traduccin
#elif L==7	// it
	 "set"
#elif L==8	// pl
	 "Wrz"
#elif L==9	// pt
	 "set"
#endif
	 ,
#if   L==1	// ca
	 "oct"
#elif L==2	// de
	 "Okt"
#elif L==3	// en
	 "Oct"
#elif L==4	// es
	 "oct"
#elif L==5	// fr
	 "oct"
#elif L==6	// gn
	 "oct"	// Okoteve traduccin
#elif L==7	// it
	 "ott"
#elif L==8	// pl
	 "Pa&zacute;"
#elif L==9	// pt
	 "out"
#endif
	 ,
#if   L==1	// ca
	 "nov"
#elif L==2	// de
	 "Nov"
#elif L==3	// en
	 "Nov"
#elif L==4	// es
	 "nov"
#elif L==5	// fr
	 "nov"
#elif L==6	// gn
	 "nov"	// Okoteve traduccin
#elif L==7	// it
	 "nov"
#elif L==8	// pl
	 "Lis"
#elif L==9	// pt
	 "nov"
#endif
	 ,
#if   L==1	// ca
	 "des"
#elif L==2	// de
	 "Dez"
#elif L==3	// en
	 "Dec"
#elif L==4	// es
	 "dic"
#elif L==5	// fr
	 "d&eacute;c"
#elif L==6	// gn
	 "dic"	// Okoteve traduccin
#elif L==7	// it
	 "dic"
#elif L==8	// pl
	 "Gru"
#elif L==9	// pt
	 "dez"
#endif
	};

const char *Txt_Move_up_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Subir %s";		// Necessita traduccio
#elif L==2	// de
	"Move up %s";		// Need bersetzung
#elif L==3	// en
	"Move up %s";
#elif L==4	// es
	"Subir %s";
#elif L==5	// fr
	"Move up %s";		// Besoin de traduction
#elif L==6	// gn
	"Subir %s";		// Okoteve traduccin
#elif L==7	// it
	"Muovi su %s";
#elif L==8	// pl
	"Przesun w g&oacute;re %s";
#elif L==9	// pt
	"Mover %s para cima";
#endif

const char *Txt_Move_up_X_and_its_subsections =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Subir %s y sus subapartados";			// Necessita traduccio
#elif L==2	// de
	"Move up %s and its subsections";		// Need bersetzung
#elif L==3	// en
	"Move up %s and its subsections";
#elif L==4	// es
	"Subir %s y sus subapartados";
#elif L==5	// fr
	"Move up %s and its subsections";		// Besoin de traduction
#elif L==6	// gn
	"Subir %s y sus subapartados";			// Okoteve traduccin
#elif L==7	// it
	"Muovi su %s e le sue sottosezioni";
#elif L==8	// pl
	"Przenies sie %s, a jego podrozdzialy";
#elif L==9	// pt
	"Mover %s e suas subse&ccedil;&otilde;es para cima";
#endif

const char *Txt_Move_down_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Bajar %s";		// Necessita traduccio
#elif L==2	// de
	"Move down %s";		// Need bersetzung
#elif L==3	// en
	"Move down %s";
#elif L==4	// es
	"Bajar %s";
#elif L==5	// fr
	"Move down %s";		// Besoin de traduction
#elif L==6	// gn
	"Bajar %s";		// Okoteve traduccin
#elif L==7	// it
	"Muovi gi&ugrave; %s";
#elif L==8	// pl
	"Opuszczac %s";
#elif L==9	// pt
	"Mover %s para baixo";
#endif

const char *Txt_Move_down_X_and_its_subsections =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Bajar %s y sus subapartados";			// Necessita traduccio
#elif L==2	// de
	"Move down %s and its subsections";		// Need bersetzung
#elif L==3	// en
	"Move down %s and its subsections";
#elif L==4	// es
	"Bajar %s y sus subapartados";
#elif L==5	// fr
	"Move down %s and its subsections";		// Besoin de traduction
#elif L==6	// gn
	"Bajar %s y sus subapartados";			// Okoteve traduccin
#elif L==7	// it
	"Muovi gi&ugrave; %s e le sue sottosezioni";
#elif L==8	// pl
	"Przesun w d&oacute;l %s, a jego podrozdzialy";
#elif L==9	// pt
	"Mover %s e suas subse&ccedil;&otilde;es para baixo";
#endif

const char *Txt_Movement_not_allowed =
#if   L==1	// ca
	"Moviment no perm&egrave;s";
#elif L==2	// de
	"Bewegung nicht erlaubt";
#elif L==3	// en
	"Move up not allowed";
#elif L==4	// es
	"Movimiento no permitido";
#elif L==5	// fr
	"Mouvement pas autoris&eacute;";
#elif L==6	// gn
	"Movimiento no permitido";	// Okoteve traduccin
#elif L==7	// it
	"Movimento non consentito";
#elif L==8	// pl
	"Ruch nie jest dozwolone";
#elif L==9	// pt
	"Movimento n&atilde;o permitido";
#endif

const char *Txt_MSG_Content =	// of a post or message
#if   L==1	// ca
	"Contingut";
#elif L==2	// de
	"Inhalt";
#elif L==3	// en
	"Content";
#elif L==4	// es
	"Contenido";
#elif L==5	// fr
	"Contenu";
#elif L==6	// gn
	"Contenido";	// Okoteve traduccin
#elif L==7	// it
	"Contenuto";
#elif L==8	// pl
	"Tre&sacute;&cacute;";
#elif L==9	// pt
	"Conte&uacute;do";
#endif

const char *Txt_MSG_Deleted_without_opening =
#if   L==1	// ca
	"Eliminado sin abrir";		// Necessita traduccio
#elif L==2	// de
	"Gel&ouml;scht ohne &Ouml;ffnen";
#elif L==3	// en
	"Deleted without opening";
#elif L==4	// es
	"Eliminado sin abrir";
#elif L==5	// fr
	"Supprim&eacute; sans ouvrir";
#elif L==6	// gn
	"Eliminado sin abrir";		// Okoteve traduccin
#elif L==7	// it
	"Rimosso senza aprire";
#elif L==8	// pl
	"Usuniete bez otwierania";
#elif L==9	// pt
	"Suprimida sem abrir";
#endif

const char *Txt_MSG_From =
#if   L==1	// ca
	"De";		// Necessita traduccio
#elif L==2	// de
	"Von";
#elif L==3	// en
	"From";
#elif L==4	// es
	"De";
#elif L==5	// fr
	"De";
#elif L==6	// gn
	"De";		// Okoteve traduccin
#elif L==7	// it
	"Da";
#elif L==8	// pl
	"z";
#elif L==9	// pt
	"De";
#endif

const char *Txt_MSG_from =
#if   L==1	// ca
	"from";		// Necessita traduccio
#elif L==2	// de
	"von";
#elif L==3	// en
	"from";
#elif L==4	// es
	"de";
#elif L==5	// fr
	"de";
#elif L==6	// gn
	"de";	// Okoteve traduccin
#elif L==7	// it
	"da";
#elif L==8	// pl
	"z";
#elif L==9	// pt
	"de";
#endif

const char *Txt_MSG_New =
#if   L==1	// ca
	"Nuevo";	// Necessita traduccio
#elif L==2	// de
	"Neue";
#elif L==3	// en
	"New";
#elif L==4	// es
	"Nuevo";
#elif L==5	// fr
	"Nouveau";
#elif L==6	// gn
	"Nuevo";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo";
#elif L==8	// pl
	"nowe";
#elif L==9	// pt
	"Nova";
#endif

const char *Txt_MSG_Not_replied =
#if   L==1	// ca
	"No respondido";	// Necessita traduccio
#elif L==2	// de
	"Nicht geantwortet";
#elif L==3	// en
	"Not replied";
#elif L==4	// es
	"No respondido";
#elif L==5	// fr
	"Pas r&eacute;pondu";
#elif L==6	// gn
	"No respondido";	// Okoteve traduccin
#elif L==7	// it
	"Non risposto";
#elif L==8	// pl
	"Nie odpowiedzial";
#elif L==9	// pt
	"N&atilde;o respondida";
#endif

const char *Txt_MSG_Open =
#if   L==1	// ca
	"Abierto";	// Necessita traduccio
#elif L==2	// de
	"Ge&ouml;ffneten";
#elif L==3	// en
	"Open";
#elif L==4	// es
	"Abierto";
#elif L==5	// fr
	"Ouvert";
#elif L==6	// gn
	"Abierto";	// Okoteve traduccin
#elif L==7	// it
	"Aperto";
#elif L==8	// pl
	"otwarte";
#elif L==9	// pt
	"Aberta";
#endif

const char *Txt_MSG_Open_and_deleted =
#if   L==1	// ca
	"Abierto y eliminado";	// Necessita traduccio
#elif L==2	// de
	"Ge&ouml;ffneten und gel&ouml;scht";
#elif L==3	// en
	"Open and deleted";
#elif L==4	// es
	"Abierto y eliminado";
#elif L==5	// fr
	"Ouvert et supprim&eacute;";
#elif L==6	// gn
	"Abierto y eliminado";	// Okoteve traduccin
#elif L==7	// it
	"Aperto e rimosso";
#elif L==8	// pl
	"Open i usuniete";
#elif L==9	// pt
	"Aberta e suprimida";
#endif

const char *Txt_MSG_Replied =
#if   L==1	// ca
	"Respondido";	// Necessita traduccio
#elif L==2	// de
	"Geantwortet";
#elif L==3	// en
	"Replied";
#elif L==4	// es
	"Respondido";
#elif L==5	// fr
	"R&eacute;pondu";
#elif L==6	// gn
	"Respondido";	// Okoteve traduccin
#elif L==7	// it
	"Risposto";
#elif L==8	// pl
	"Odpowiedziano";
#elif L==9	// pt
	"Respondida";
#endif

const char *Txt_MSG_Sent =
#if   L==1	// ca
	"Enviado";	// Necessita traduccio
#elif L==2	// de
	"Gesendet";
#elif L==3	// en
	"Sent";
#elif L==4	// es
	"Enviado";
#elif L==5	// fr
	"Envoy&eacute;";
#elif L==6	// gn
	"Enviado";	// Okoteve traduccin
#elif L==7	// it
	"Inviato";
#elif L==8	// pl
	"Wyslane";
#elif L==9	// pt
	"Enviada";
#endif

const char *Txt_MSG_Sent_and_deleted =
#if   L==1	// ca
	"Enviado y eliminado";	// Necessita traduccio
#elif L==2	// de
	"Gesendet und gel&ouml;scht";
#elif L==3	// en
	"Sent and deleted";
#elif L==4	// es
	"Enviado y eliminado";
#elif L==5	// fr
	"Envoy&eacute; et supprim&eacute;";
#elif L==6	// gn
	"Enviado y eliminado";	// Okoteve traduccin
#elif L==7	// it
	"Inviato e rimosso";
#elif L==8	// pl
	"Wyslane i usuniete";
#elif L==9	// pt
	"Enviada e suprimida";
#endif

const char *Txt_MSG_Subject =
#if   L==1	// ca
	"Assumpte";
#elif L==2	// de
	"Betreff";
#elif L==3	// en
	"Subject";
#elif L==4	// es
	"Asunto";
#elif L==5	// fr
	"Objet";
#elif L==6	// gn
	"Asunto";	// Okoteve traduccin
#elif L==7	// it
	"Oggetto";
#elif L==8	// pl
	"Z zastrzezeniem";
#elif L==9	// pt
	"Assunto";
#endif

const char *Txt_MSG_To =
#if   L==1	// ca
	"Per";
#elif L==2	// de
	"An";
#elif L==3	// en
	"To";
#elif L==4	// es
	"Para";
#elif L==5	// fr
	"Pour";
#elif L==6	// gn
	"Para";	// Okoteve traduccin
#elif L==7	// it
	"A";
#elif L==8	// pl
	"Aby";
#elif L==9	// pt
	"Para";
#endif

const char *Txt_MSG_Unopened =
#if   L==1	// ca
	"Sin abrir";	// Necessita traduccio
#elif L==2	// de
	"Unge&ouml;ffnet";
#elif L==3	// en
	"Unopened";
#elif L==4	// es
	"Sin abrir";
#elif L==5	// fr
	"Non ouvert";
#elif L==6	// gn
	"Sin abrir";	// Okoteve traduccin
#elif L==7	// it
	"Non aperto";
#elif L==8	// pl
	"Otwarte";
#elif L==9	// pt
	"Fechada";
#endif

const char *Txt_MSGS_Deleted =
#if   L==1	// ca
	"Eliminados";	// Necessita traduccio
#elif L==2	// de
	"Gel&ouml;scht";
#elif L==3	// en
	"Deleted";
#elif L==4	// es
	"Eliminados";
#elif L==5	// fr
	"Supprim&eacute;s";
#elif L==6	// gn
	"Eliminados";	// Okoteve traduccin
#elif L==7	// it
	"Rimossi";
#elif L==8	// pl
	"Usuniete";
#elif L==9	// pt
	"Suprimidas";
#endif

const char *Txt_MSGS_Not_deleted =
#if   L==1	// ca
	"No eliminados";	// Necessita traduccio
#elif L==2	// de
	"Nicht Gel&ouml;scht";
#elif L==3	// en
	"Not deleted";
#elif L==4	// es
	"No eliminados";
#elif L==5	// fr
	"Non supprim&eacute;s";
#elif L==6	// gn
	"No eliminados";	// Okoteve traduccin
#elif L==7	// it
	"Non rimossi";
#elif L==8	// pl
	"Nie usuniete";
#elif L==9	// pt
	"N&atilde;o suprimidas";
#endif

const char *Txt_MSGS_Received =
#if   L==1	// ca
	"Rebuts";
#elif L==2	// de
	"Empfangene";
#elif L==3	// en
	"Received";
#elif L==4	// es
	"Recibidos";
#elif L==5	// fr
	"Re&ccedil;us";
#elif L==6	// gn
	"Recibidos";	// Okoteve traduccin
#elif L==7	// it
	"Ricevuti";
#elif L==8	// pl
	"Odebrane";
#elif L==9	// pt
	"Recebidas";
#endif

const char *Txt_MSGS_Sent =
#if   L==1	// ca
	"Enviats";
#elif L==2	// de
	"Gesendete";
#elif L==3	// en
	"Sent";
#elif L==4	// es
	"Enviados";
#elif L==5	// fr
	"Envoy&eacute;s";
#elif L==6	// gn
	"Enviados";	// Okoteve traduccin
#elif L==7	// it
	"Inviati";
#elif L==8	// pl
	"Wyslane";
#elif L==9	// pt
	"Enviadas";
#endif

const char *Txt_Multiple_enrolment =		// (to a type of group)
#if   L==1	// ca
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";	// Necessita traduccio
#elif L==2	// de
	"Mehrere beitreten?";
#elif L==3	// en
	"Multiple enrolment?";
#elif L==4	// es
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";
#elif L==5	// fr
	"Inscription multiple?";
#elif L==6	// gn
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";	// Okoteve traduccin
#elif L==7	// it
	"Registrazione multipla?";
#elif L==8	// pl
	"Rejestracja wielu element&oacute;w?";
#elif L==9	// pt
	"Inscri&ccedil;&atilde;o m&uacute;ltiplas?";
#endif

const char *Txt_My_agenda =
#if   L==1	// ca
	"La meva agenda";
#elif L==2	// de
	"Mein Organizer";
#elif L==3	// en
	"My agenda";
#elif L==4	// es
	"Mi agenda";
#elif L==5	// fr
	"Mon agenda";
#elif L==6	// gn
	"Che aporyr&atilde;";
#elif L==7	// it
	"La mia agenda";
#elif L==8	// pl
	"Moja pami&eogon;tnik";
#elif L==9	// pt
	"Minha agenda";
#endif

const char *Txt_My_courses =
#if   L==1	// ca
	"Les meves assignatures";
#elif L==2	// de
	"Meine Kursen";
#elif L==3	// en
	"My courses";
#elif L==4	// es
	"Mis asignaturas";
#elif L==5	// fr
	"Mes mati&egrave;res";
#elif L==6	// gn
	"Che mbo'esyry";
#elif L==7	// it
	"Miei corsi";
#elif L==8	// pl
	"Moje kurs&oacute;w";
#elif L==9	// pt
	"As minha disciplinas";
#endif

const char *Txt_my_documents =
#if   L==1	// ca
	"els meus documents";
#elif L==2	// de
	"meine Dokumente";
#elif L==3	// en
	"my documents";
#elif L==4	// es
	"mis documentos";
#elif L==5	// fr
	"mes documents";
#elif L==6	// gn
	"mis documentos";	// Okoteve traduccin
#elif L==7	// it
	"i miei documenti";
#elif L==8	// pl
	"moje dokumenty";
#elif L==9	// pt
	"meus documentos";
#endif

const char *Txt_My_frequent_actions =
#if   L==1	// ca
	"Les meves accions freq&uuml;ents";
#elif L==2	// de
	"Meine h&auml;ufigen Handlungen";
#elif L==3	// en
	"My frequent actions";
#elif L==4	// es
	"Mis acciones frecuentes";
#elif L==5	// fr
	"Mes actions fr&eacute;quentes";
#elif L==6	// gn
	"Mis acciones frecuentes";	// Okoteve traduccin
#elif L==7	// it
	"Le mie azioni frequenti";
#elif L==8	// pl
	"Moi cz&eogon;ste dzia&lstrok;ania";
#elif L==9	// pt
	"Minhas a&ccedil;&otilde;es freq&uuml;entes";
#endif

const char *Txt_My_groups =
#if   L==1	// ca
	"Meus grups";
#elif L==2	// de
	"Meine Gruppen";
#elif L==3	// en
	"My groups";
#elif L==4	// es
	"Mis grupos";
#elif L==5	// fr
	"Mes groupes";
#elif L==6	// gn
	"Mis grupos";	// Okoteve traduccin
#elif L==7	// it
	"Miei gruppi";
#elif L==8	// pl
	"Moje grupy";
#elif L==9	// pt
	"Meus grupos";
#endif

const char *Txt_My_public_profile =
#if   L==1	// ca
	"El meu perfil p&uacute;blic";
#elif L==2	// de
	"Mein &ouml;ffentliches Profil";
#elif L==3	// en
	"My public profile";
#elif L==4	// es
	"Mi perfil p&uacute;blico";
#elif L==5	// fr
	"Mon profil public";
#elif L==6	// gn
	"Mi perfil p&uacute;blico";	// Okoteve traduccin
#elif L==7	// it
	"Il mio profilo pubblico";
#elif L==8	// pl
	"M&oacute;j profil publiczny";
#elif L==9	// pt
	"Meu perfil p&uacute;blico";
#endif

const char *Txt_My_views =
#if   L==1	// ca
	"Els meus accessos";
#elif L==2	// de
	"Meine Ansichten";
#elif L==3	// en
	"My views";
#elif L==4	// es
	"Mis accesos";
#elif L==5	// fr
	"Mes vues";
#elif L==6	// gn
	"Mis accesos";	// Okoteve traduccin
#elif L==7	// it
	"Miei visualizzazioni";
#elif L==8	// pl
	"Moje ods&lstrok;on";
#elif L==9	// pt
	"Meus acessos";
#endif

const char *Txt_Name =
#if   L==1	// ca
	"Nom";
#elif L==2	// de
	"Name";
#elif L==3	// en
	"Name";
#elif L==4	// es
	"Nombre";
#elif L==5	// fr
	"Nom";
#elif L==6	// gn
	"T&eacute;ra";
#elif L==7	// it
	"Nome";
#elif L==8	// pl
	"Nazwa";
#elif L==9	// pt
	"Nome";
#endif

const char *Txt_New_assignment =
#if   L==1	// ca
	"Nova activitat";
#elif L==2	// de
	"Neue Aufgabe";
#elif L==3	// en
	"New assignment";
#elif L==4	// es
	"Nueva actividad";
#elif L==5	// fr
	"Nouvelle activit&eacute;";
#elif L==6	// gn
	"Nueva actividad";	// Okoteve traduccin
#elif L==7	// it
	"Nuova attivit&agrave;";
#elif L==8	// pl
	"Nowe zadania";
#elif L==9	// pt
	"Nova atividade";
#endif

const char *Txt_New_announcement =
#if   L==1	// ca
	"Nou anunci";
#elif L==2	// de
	"Neue Bekanntmachung";
#elif L==3	// en
	"New announcement";
#elif L==4	// es
	"Nuevo anuncio";
#elif L==5	// fr
	"Nouvel annonce";
#elif L==6	// gn
	"Nuevo anuncio";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo annuncio";
#elif L==8	// pl
	"Nowe og&lstrok;oszenia";
#elif L==9	// pt
	"Novo an&uacute;ncio";
#endif

const char *Txt_New_announcement_OF_EXAM =
#if   L==1	// ca
	"Nova convocat&ograve;ria";
#elif L==2	// de
	"Neuer Aufruf";
#elif L==3	// en
	"New announcement";
#elif L==4	// es
	"Nueva convocatoria";
#elif L==5	// fr
	"Nouvelle convocation";
#elif L==6	// gn
	"Nueva convocatoria";	// Okoteve traduccin
#elif L==7	// it
	"Nuova sessione";
#elif L==8	// pl
	"Nowe ogloszenie";
#elif L==9	// pt
	"Nova chamada";
#endif

const char *Txt_New_banner =
#if   L==1	// ca
	"Nou banner";
#elif L==2	// de
	"Neue Werbebanner";
#elif L==3	// en
	"New banner";
#elif L==4	// es
	"Nuevo banner";
#elif L==5	// fr
	"Nouvelle banni&eacute;re";
#elif L==6	// gn
	"Nuevo banner";		// Okoteve traduccin
#elif L==7	// it
	"Nuovo banner";
#elif L==8	// pl
	"Nowy banner";
#elif L==9	// pt
	"Novo banner";
#endif

const char *Txt_New_centre =
#if   L==1	// ca
	"Nou centre (facultat, escola, institut, divisi&oacute;, edifici...)";
#elif L==2	// de
	"Neues Lehrinstitut (Fakult&auml;t, Schule, Hochschule, Abteilung, Geb&auml;ude...)";
#elif L==3	// en
	"New centre (faculty, school, college, division, building...)";
#elif L==4	// es
	"Nuevo centro (facultad, escuela, instituto, divisi&oacute;n, edificio...)";
#elif L==5	// fr
	"Nouveau centre (facult&eacute;, &eacute;cole, universit&eacute;, division, b&acirc;timent...)";
#elif L==6	// gn
	"Nuevo centro (facultad, escuela, instituto, divisi&oacute;n, edificio...)";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo centro (facolt&agrave;, scuola, istituto, divisione, costruzione...)";
#elif L==8	// pl
	"Nowe centrum (wydzia&lstrok;, szko&lstrok;a, uczelnia, podzia&lstrok;, budowa...)";
#elif L==9	// pt
	"Novo centro (faculdade, escola, faculdade, divis&atilde;o, constru&ccedil;&atilde;o...)";
#endif

const char *Txt_New_classroom =
#if   L==1	// ca
	"Nova aula";
#elif L==2	// de
	"Neue Klassenzimmer";
#elif L==3	// en
	"New classroom";
#elif L==4	// es
	"Nueva aula";
#elif L==5	// fr
	"Nouvelle salle de classe";
#elif L==6	// gn
	"Nueva aula";	// Okoteve traduccin
#elif L==7	// it
	"Nuova aula";
#elif L==8	// pl
	"Nowe klasa";
#elif L==9	// pt
	"Nova sala de aula";
#endif

const char *Txt_New_country =
#if   L==1	// ca
	"Nuevo pa&iacute;s";	// Necessita traduccio
#elif L==2	// de
	"Neues Land";
#elif L==3	// en
	"New country";
#elif L==4	// es
	"Nuevo pa&iacute;s";
#elif L==5	// fr
	"Nouveau pays";
#elif L==6	// gn
	"Nuevo pa&iacute;s";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo paese";
#elif L==8	// pl
	"Nowy kraj";
#elif L==9	// pt
	"Novo pa&iacute;s";
#endif

const char *Txt_New_course =
#if   L==1	// ca
	"Nova assignatura (assignatura, curs, mat&egrave;ria, disciplina...)";
#elif L==2	// de
	"Neue Kurs (Kurs, Thema, Disziplin...)";
#elif L==3	// en
	"New course (course, subject, discipline...)";
#elif L==4	// es
	"Nueva asignatura (asignatura, curso, materia, disciplina...)";
#elif L==5	// fr
	"Nouvelle mati&egrave;re (mati&egrave;re, cours, discipline...)";
#elif L==6	// gn
	"Nueva asignatura (asignatura, curso, materia, disciplina...)";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo corso (corso, materia...)";
#elif L==8	// pl
	"Nowy kurs";
#elif L==9	// pt
	"Nova disciplina (disciplina, curso...)";
#endif

const char *Txt_New_degree =
#if   L==1	// ca
	"Nova titulaci&oacute; (carrera, llicenciatura, diplomatura, grau, postgrau, m&agrave;ster, cicle, nivell...)";
#elif L==2	// de
	"Neuer Studiengang (Karriere, Diplom, Grad, Graduierten, Master, Zyklus, Niveau...)";
#elif L==3	// en
	"New degree (career, diploma, degree, postgraduate, master, cycle, level...)";
#elif L==4	// es
	"Nueva titulaci&oacute;n (carrera, licenciatura, diplomatura, grado, posgrado, m&aacute;ster, ciclo, nivel...)";
#elif L==5	// fr
	"Nouvel &eacute;tude (carri&egrave;re, grade, dipl&ocirc;me, degr&eacute;, troisi&agrave;me cycle, master, cycle, niveau...)";
#elif L==6	// gn
	"Nueva titulaci&oacute;n (carrera, licenciatura, diplomatura, grado, posgrado, m&aacute;ster, ciclo, nivel...)";	// Okoteve traduccin
#elif L==7	// it
	"Nuova laurea (carriera, diploma, laurea, post-laurea, master, ciclo, livello...)";
#elif L==8	// pl
	"Nowy stopie&nacute; (zawodowe, stopie&nacute;, dyplom, magisterskie, podyplomowe, magisterskie, cykl, poziom...)";
#elif L==9	// pt
	"Novo grau (carreira, diploma, grau, p&oacute;s-grau, mestrado, ciclo, n&iacute;vel...)";
#endif

const char *Txt_New_department =
#if   L==1	// ca
	"Nuevo departamento";	// Necessita traduccio
#elif L==2	// de
	"Neue Abteilung";
#elif L==3	// en
	"New department";
#elif L==4	// es
	"Nuevo departamento";
#elif L==5	// fr
	"Nouveau d&eacute;partement";
#elif L==6	// gn
	"Nuevo departamento";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo dipartimento";
#elif L==8	// pl
	"Nowy dzia&lstrok;";
#elif L==9	// pt
	"Novo departamento";
#endif

const char *Txt_New_email =
#if   L==1	// ca
	"Nou correu";
#elif L==2	// de
	"Neues E-Mail";
#elif L==3	// en
	"New email";
#elif L==4	// es
	"Nuevo correo";
#elif L==5	// fr
	"Nouveau courrier";
#elif L==6	// gn
	"Nuevo correo";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo email";
#elif L==8	// pl
	"Nowy email";
#elif L==9	// pt
	"Novo email";
#endif

const char *Txt_New_email_domain =
#if   L==1	// ca
	"Nuevo dominio de correo";	// Necessita traduccio
#elif L==2	// de
	"Neue E-Mail-Dom&auml;ne";
#elif L==3	// en
	"New email domain";
#elif L==4	// es
	"Nuevo dominio de correo";
#elif L==5	// fr
	"Nouveau domaine de messagerie";
#elif L==6	// gn
	"Nuevo dominio de correo";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo campo mail";
#elif L==8	// pl
	"Nowe domeny mail";
#elif L==9	// pt
	"Novo dom&iacute;nio de email";
#endif

const char *Txt_New_event =
#if   L==1	// ca
	"Nou esdeveniment";
#elif L==2	// de
	"Neue Ereignis";
#elif L==3	// en
	"New event";
#elif L==4	// es
	"Nuevo evento";
#elif L==5	// fr
	"Nouveau &eacute;v&eacute;nement";
#elif L==6	// gn
	"Nuevo evento";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo evento";
#elif L==8	// pl
	"Nowe wydarzenie";
#elif L==9	// pt
	"Novo evento";
#endif

const char *Txt_New_FILE_OR_FOLDER =
#if   L==1	// ca
	"Nou";
#elif L==2	// de
	"Neue";
#elif L==3	// en
	"New";
#elif L==4	// es
	"Nuevo";
#elif L==5	// fr
	"Nouveau";
#elif L==6	// gn
	"Nuevo";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo";
#elif L==8	// pl
	"Nowe";
#elif L==9	// pt
	"Novo";
#endif

const char *Txt_New_game =
#if   L==1	// ca
	"Nou joc";
#elif L==2	// de
	"Neues Spiel";
#elif L==3	// en
	"New game";
#elif L==4	// es
	"Nuevo juego";
#elif L==5	// fr
	"Nouveau jeu";
#elif L==6	// gn
	"Nuevo juego";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo gioco";
#elif L==8	// pl
	"Nowa gra";
#elif L==9	// pt
	"Novo jogo";
#endif

const char *Txt_New_group =
#if   L==1	// ca
	"Nuevo grupo";	// Necessita traduccio
#elif L==2	// de
	"Neue Gruppe";
#elif L==3	// en
	"New group";
#elif L==4	// es
	"Nuevo grupo";
#elif L==5	// fr
	"Nouveau groupe";
#elif L==6	// gn
	"Nuevo grupo";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo gruppo";
#elif L==8	// pl
	"Nowa grupa";
#elif L==9	// pt
	"Novo grupo";
#endif

const char *Txt_New_holiday =
#if   L==1	// ca
	"Nova festivitat";
#elif L==2	// de
	"Neue Feiertag";
#elif L==3	// en
	"New holiday";
#elif L==4	// es
	"Nueva festividad";
#elif L==5	// fr
	"Nouvelle f&ecirc;te";
#elif L==6	// gn
	"Nueva festividad";	// Okoteve traduccin
#elif L==7	// it
	"Nuova festivit&agrave;";
#elif L==8	// pl
	"Nowe swieto";
#elif L==9	// pt
	"Nova f&eacute;ria";
#endif

const char *Txt_New_institution =
#if   L==1	// ca
	"Nova instituci&oacute;n (universitat, institut, escola, acad&egrave;mia, organitzaci&oacute;, empresa...)";
#elif L==2	// de
	"Neue Hochschule (Universit&auml;t, Hochschule, Schule, Hochschule, Organisation, Unternehmen...)";
#elif L==3	// en
	"New institution (university, college, school, academy, organization, company...)";
#elif L==4	// es
	"Nueva instituci&oacute;n (universidad, instituto, colegio, academia, organizaci&oacute;n, empresa...)";
#elif L==5	// fr
	"Nouvel &eacute;tablissement (universit&eacute;, coll&egrave;ge, &eacute;cole, acad&eacute;mie, organisation, entreprise...)";
#elif L==6	// gn
	"Nueva instituci&oacute;n (universidad, instituto, colegio, academia, organizaci&oacute;n, empresa...)";	// Okoteve traduccin
#elif L==7	// it
	"Nuova istituzione (universit&agrave;, istituto, scuola, accademia, organizzazione, societ&agrave;...)";
#elif L==8	// pl
	"Nowa instytucja (uniwersytet, uczelnia, szko&lstrok;a, akademia, organizacja, firma...)";
#elif L==9	// pt
	"Nova institu&ccedil;&atilde;o (universidade, faculdade, escola, academia, organiza&ccedil;&atilde;o, empresa...)";
#endif

const char *Txt_New_link =
#if   L==1	// ca
	"Nuevo enlace";		// Necessita traduccio
#elif L==2	// de
	"Neue Link";
#elif L==3	// en
	"New link";
#elif L==4	// es
	"Nuevo enlace";
#elif L==5	// fr
	"Nouveau lien";
#elif L==6	// gn
	"Nuevo enlace";		// Okoteve traduccin
#elif L==7	// it
	"Nuovo link";
#elif L==8	// pl
	"Nowy link";
#elif L==9	// pt
	"Nova liga&ccedil;&atilde;o";
#endif

const char *Txt_New_message =
#if   L==1	// ca
	"Nou missatge";
#elif L==2	// de
	"Neue Nachricht";
#elif L==3	// en
	"New message";
#elif L==4	// es
	"Nuevo mensaje";
#elif L==5	// fr
	"Nouveau message";
#elif L==6	// gn
	"Nuevo mensaje";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo messaggio";
#elif L==8	// pl
	"Nowa wiadomosc";
#elif L==9	// pt
	"Nova mensagem";
#endif

const char *Txt_New_nickname =
#if   L==1	// ca
	"Nou sobrenom";
#elif L==2	// de
	"Neues Benutzername";
#elif L==3	// en
	"New nickname";
#elif L==4	// es
	"Nuevo apodo";
#elif L==5	// fr
	"Nouveau surnom";
#elif L==6	// gn
	"Nuevo apodo";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo nome utente";
#elif L==8	// pl
	"Nowy nick";
#elif L==9	// pt
	"Nova alcunha";
#endif

const char *Txt_New_notice =
#if   L==1	// ca
	"Nou av&iacute;s";
#elif L==2	// de
	"Neue Ank&uuml;ndigung";
#elif L==3	// en
	"New notice";
#elif L==4	// es
	"Nuevo aviso";
#elif L==5	// fr
	"Nouvel avis";
#elif L==6	// gn
	"Nuevo aviso";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo avviso";
#elif L==8	// pl
	"Nowe obwieszczenie";
#elif L==9	// pt
	"Novo aviso";
#endif

const char *Txt_New_on_PLATFORM_Sign_up =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No t&eacute; compte a %s? Registra't ara";
#elif L==2	// de
	"Neu bei %s? Registriere Dich!";
#elif L==3	// en
	"New on %s? Sign up";
#elif L==4	// es
	"&iquest;No tiene cuenta en %s? Reg&iacute;strese";
#elif L==5	// fr
	"Nouveau sur %s? Inscrivez-vous";
#elif L==6	// gn
	"&iquest;No tiene cuenta en %s? Reg&iacute;strese";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo su %s? Iscriviti";
#elif L==8	// pl
	"Pierwszy raz na %s? Zarejestruj si&eogon;";
#elif L==9	// pt
	"Novo no %s? Inscreva-se";
#endif

const char *Txt_New_password =
#if   L==1	// ca
	"Nova contrasenya";
#elif L==2	// de
	"Neues Passwort";
#elif L==3	// en
	"New password";
#elif L==4	// es
	"Nueva contrase&ntilde;a";
#elif L==5	// fr
	"Nouveau mot de passe";
#elif L==6	// gn
	"Nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Nuova password";
#elif L==8	// pl
	"Nowe has&lstrok;o";
#elif L==9	// pt
	"Nova senha";
#endif

const char *Txt_New_place =
#if   L==1	// ca
	"Nuevo lugar";	// Necessita traduccio
#elif L==2	// de
	"Neue Standort";
#elif L==3	// en
	"New place";
#elif L==4	// es
	"Nuevo lugar";
#elif L==5	// fr
	"Nouvel emplacement";
#elif L==6	// gn
	"Nuevo lugar";	// Okoteve traduccin
#elif L==7	// it
	"Nuova localit&grave;";
#elif L==8	// pl
	"Nowe miejsce";
#elif L==9	// pt
	"Nova localiza&ccedil;&atilde;o";
#endif

const char *Txt_New_plugin =
#if   L==1	// ca
	"Nuevo complemento";	// Necessita traduccio
#elif L==2	// de
	"Neue Plugin";
#elif L==3	// en
	"New plugin";
#elif L==4	// es
	"Nuevo complemento";
#elif L==5	// fr
	"Nouveau plugin";
#elif L==6	// gn
	"Nuevo complemento";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo plugin";
#elif L==8	// pl
	"Nowy plugin";
#elif L==9	// pt
	"Novo plugin";
#endif

const char *Txt_New_post =
#if   L==1	// ca
	"Nou comentari";
#elif L==2	// de
	"Neue Post";
#elif L==3	// en
	"New post";
#elif L==4	// es
	"Nuevo comentario";
#elif L==5	// fr
	"Nouveau post";
#elif L==6	// gn
	"Nuevo comentario";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo post";
#elif L==8	// pl
	"Nowy post";
#elif L==9	// pt
	"Novo post";
#endif

const char *Txt_New_project =
#if   L==1	// ca
	"Nou projecte";
#elif L==2	// de
	"Neues Projekt";
#elif L==3	// en
	"New project";
#elif L==4	// es
	"Nuevo proyecto";
#elif L==5	// fr
	"Nouveau projet";
#elif L==6	// gn
	"Nuevo proyecto";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo progetto";
#elif L==8	// pl
	"Nowy projekt";
#elif L==9	// pt
	"Novo projeto";
#endif

const char *Txt_New_question =
#if   L==1	// ca
	"Nova pregunta";
#elif L==2	// de
	"Neuer Frage";
#elif L==3	// en
	"New question";
#elif L==4	// es
	"Nueva pregunta";
#elif L==5	// fr
	"Nouvelle question";
#elif L==6	// gn
	"Nueva pregunta";	// Okoteve traduccin
#elif L==7	// it
	"Nuova domanda";
#elif L==8	// pl
	"Nowe pytanie";
#elif L==9	// pt
	"Nova quest&atilde;o";
#endif

const char *Txt_New_record_field =
#if   L==1	// ca
	"Nuevo campo de ficha";		// Necessita traduccio
#elif L==2	// de
	"Neues Datenfeld";
#elif L==3	// en
	"New record field";
#elif L==4	// es
	"Nuevo campo de ficha";
#elif L==5	// fr
	"Nouveau champ de carte";
#elif L==6	// gn
	"Nuevo campo de ficha";		// Okoteve traduccin
#elif L==7	// it
	"Nuovo campo di scheda";
#elif L==8	// pl
	"Nowy rekord pola";
#elif L==9	// pt
	"Novo campo de cart&atilde;o";
#endif

const char *Txt_New_TIMELINE_comment =
#if   L==1	// ca
	"Nou comentari";
#elif L==2	// de
	"Neue Kommentar";
#elif L==3	// en
	"New comment";
#elif L==4	// es
	"Nuevo comentario";
#elif L==5	// fr
	"Nouveau commentaire";
#elif L==6	// gn
	"Nuevo comentario";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo commento";
#elif L==8	// pl
	"Nowy komentarz";
#elif L==9	// pt
	"Novo coment&aacute;rio";
#endif

const char *Txt_New_TIMELINE_post =
#if   L==1	// ca
	"Nova publicaci&oacute;";
#elif L==2	// de
	"Neue Beitrag";
#elif L==3	// en
	"New post";
#elif L==4	// es
	"Nueva publicaci&oacute;n";
#elif L==5	// fr
	"Nouveau post";
#elif L==6	// gn
	"Nueva publicaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo post";
#elif L==8	// pl
	"Nowy post";
#elif L==9	// pt
	"Novo post";
#endif

const char *Txt_New_survey =
#if   L==1	// ca
	"Nova enquesta";
#elif L==2	// de
	"Neuer Umfrage";
#elif L==3	// en
	"New survey";
#elif L==4	// es
	"Nueva encuesta";
#elif L==5	// fr
	"Nouveau sondage";
#elif L==6	// gn
	"Nueva encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo sondaggio";
#elif L==8	// pl
	"Nowe badania";
#elif L==9	// pt
	"Novo inqu&eacute;rito";
#endif

const char *Txt_new_tag =		// Tag used in test
#if   L==1	// ca
	"nuevo descriptor";	// Necessita traduccio
#elif L==2	// de
	"neuen Tag";
#elif L==3	// en
	"new tag";
#elif L==4	// es
	"nuevo descriptor";
#elif L==5	// fr
	"nouveau descripteur";
#elif L==6	// gn
	"nuevo descriptor";	// Okoteve traduccin
#elif L==7	// it
	"nuova etichetta";
#elif L==8	// pl
	"nowy tag";
#elif L==9	// pt
	"novo descritor";
#endif

const char *Txt_New_thread =
#if   L==1	// ca
	"Nueva discusi&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Neuen Thread";
#elif L==3	// en
	"New thread";
#elif L==4	// es
	"Nueva discusi&oacute;n";
#elif L==5	// fr
	"Nouveau fil";
#elif L==6	// gn
	"Nueva discusi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nuova discussione";
#elif L==8	// pl
	"Nowy temat";
#elif L==9	// pt
	"Nova discuss&atilde;o";
#endif

const char *Txt_New_type_of_degree =
#if   L==1	// ca
	"Nuevo tipo de titulaci&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Neue Abschlussart";
#elif L==3	// en
	"New type of degree";
#elif L==4	// es
	"Nuevo tipo de titulaci&oacute;n";
#elif L==5	// fr
	"Nouveau type d'&eacute;tude";
#elif L==6	// gn
	"Nuevo tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nuovo tipo di laurea";
#elif L==8	// pl
	"Nowy rodzaj stopnia";
#elif L==9	// pt
	"Novo tipo de grau";
#endif

const char *Txt_New_type_of_group =
#if   L==1	// ca
	"Nuevo tipo de grupo";		// Necessita traduccio
#elif L==2	// de
	"Neue Art der Gruppe";
#elif L==3	// en
	"New type of group";
#elif L==4	// es
	"Nuevo tipo de grupo";
#elif L==5	// fr
	"Nouveau type de groupe";
#elif L==6	// gn
	"Nuevo tipo de grupo";		// Okoteve traduccin
#elif L==7	// it
	"Nuovo tipo di gruppo";
#elif L==8	// pl
	"Nowy typ grupy";
#elif L==9	// pt
	"Novo tipo de grupo";
#endif

const char *Txt_NEW_YEAR_GREETING =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"&iexcl;Feliz %u!";	// Necessita traduccio
#elif L==2	// de
	"Happy %u!";
#elif L==3	// en
	"Happy %u!";
#elif L==4	// es
	"&iexcl;Feliz %u!";
#elif L==5	// fr
	"Happy %u!";
#elif L==6	// gn
	"&iexcl;Feliz %u!";	// Okoteve traduccin
#elif L==7	// it
	"Felice %u!";
#elif L==8	// pl
	"Happy %u!";
#elif L==9	// pt
	"Feliz %u!";
#endif

const char *Txt_nick_email_or_ID =
#if   L==1	// ca
	"@sobrenom, correu o ID (DNI/c&egrave;dula)";
#elif L==2	// de
	"@Benutzername, Email oder Ausweis-Nr.";
#elif L==3	// en
	"@nick, email or ID";
#elif L==4	// es
	"@apodo, correo o ID (DNI/c&eacute;dula)";
#elif L==5	// fr
	"@surnom, courrier ou num&eacute;ro d'identit&eacute;";
#elif L==6	// gn
	"@apodo, correo o ID (DNI/c&eacute;dula)";	// Okoteve traduccin
#elif L==7	// it
	"@nome-utente, email o numero carta d'identit&agrave;";
#elif L==8	// pl
	"@pseudonim, email lub ID";
#elif L==9	// pt
	"@alcunha, email ou n&ordm; identif.";
#endif

const char *Txt_nicks_emails_or_IDs_separated_by_commas =
#if   L==1	// ca
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";	// Necessita traduccio
#elif L==2	// de
	"@Benutzernamen, Emails oder Ausweis-Nr. durch Kommas getrennt";
#elif L==3	// en
	"@nicks, emails or IDs separated by commas";
#elif L==4	// es
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";
#elif L==5	// fr
	"@surnoms, courriers ou num&eacute;ros d'identit&eacute; s&eacute;par&eacute;s par des virgules";
#elif L==6	// gn
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";	// Okoteve traduccin
#elif L==7	// it
	"@nomi-utenti, email o numero carte d'identit&agrave; separati da virgole";
#elif L==8	// pl
	"@nicks, emails or IDs separated by commas";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"@alcunhas, emails ou n&ordm;s. identif. separados por v&iacute;rgulas";
#endif

const char *Txt_Nickname =
#if   L==1	// ca
	"Sobrenom";
#elif L==2	// de
	"Benutzername";
#elif L==3	// en
	"Nickname";
#elif L==4	// es
	"Apodo";
#elif L==5	// fr
	"Surnom";
#elif L==6	// gn
	"Apodo";	// Okoteve traduccin
#elif L==7	// it
	"Nome-utente";
#elif L==8	// pl
	"Pseudonim";
#elif L==9	// pt
	"Alcunha";
#endif

const char *Txt_Nickname_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Sobrenom <strong>@%s</strong> eliminat.";
#elif L==2	// de
	"Benutzername <strong>@%s</strong> entfernt.";
#elif L==3	// en
	"Nickname <strong>@%s</strong> removed.";
#elif L==4	// es
	"Apodo <strong>@%s</strong> eliminado.";
#elif L==5	// fr
	"Surnom <strong>@%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Apodo <strong>@%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Nome utente <strong>@%s</strong> rimosso.";
#elif L==8	// pl
	"Pseudonim <strong>@%s</strong> usuniete.";
#elif L==9	// pt
	"Alcunha <strong>@%s</strong> removido.";
#endif

const char *Txt_NO =
#if   L==1	// ca
	"NO";
#elif L==2	// de
	"NEIN";
#elif L==3	// en
	"NO";
#elif L==4	// es
	"NO";
#elif L==5	// fr
	"NON";
#elif L==6	// gn
	"NAH&aACUTE;NIRI";
#elif L==7	// it
	"NO";
#elif L==8	// pl
	"NO";
#elif L==9	// pt
	"N&Atilde;O";
#endif

const char *Txt_No =
#if   L==1	// ca
	"No";
#elif L==2	// de
	"Nein";
#elif L==3	// en
	"No";
#elif L==4	// es
	"No";
#elif L==5	// fr
	"Non";
#elif L==6	// gn
	"Nah&aacute;niri";
#elif L==7	// it
	"No";
#elif L==8	// pl
	"No";
#elif L==9	// pt
	"N&atilde;o";
#endif

const char *Txt_No_announcements =
#if   L==1	// ca
	"No hi ha anuncis";
#elif L==2	// de
	"Keine Bekanntmachungen";
#elif L==3	// en
	"No announcements";
#elif L==4	// es
	"No hay anuncios";
#elif L==5	// fr
	"Il n'y a pas d'annonces";
#elif L==6	// gn
	"No hay anuncios";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono annunci";
#elif L==8	// pl
	"Brak og&lstrok;oszenia";
#elif L==9	// pt
	"N&atilde;o h&aacute; an&uacute;ncios";
#endif

const char *Txt_No_announcements_of_exams_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No hay convocatorias de examen de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Keine Aufrufe f&uuml;r Pr&uuml;fung von <strong>%s</strong>.";
#elif L==3	// en
	"No announcements of exams of <strong>%s</strong>.";
#elif L==4	// es
	"No hay convocatorias de examen de <strong>%s</strong>.";
#elif L==5	// fr
	"Aucune convocation &agrave; un examen de <strong>%s</strong>.";
#elif L==6	// gn
	"No hay convocatorias de examen de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun appello d'esame di <strong>%s</strong>.";
#elif L==8	// pl
	"Brak ogloszen egzamin&oacute;w <strong>%s</strong>.";
#elif L==9	// pt
	"N&atilde;o chamadas para exame de <strong>%s</strong>.";
#endif

const char *Txt_No_assigned_classroom =
#if   L==1	// ca
	"Sense aula assignada";
#elif L==2	// de
	"Kein zugewiesenes Klassenzimmer";
#elif L==3	// en
	"No assigned classroom";
#elif L==4	// es
	"Sin aula asignada";
#elif L==5	// fr
	"Aucune classe assign&eacute;e";
#elif L==6	// gn
	"Sin aula asignada";	// Okoteve traduccin
#elif L==7	// it
	"Nessuna aula assegnata";
#elif L==8	// pl
	"Brak przydzielonej sali lekcyjnej";
#elif L==9	// pt
	"Nenhuma sala de aula atribu&iacute;da";
#endif

const char *Txt_No_assignments =
#if   L==1	// ca
	"No hi ha activitats.";
#elif L==2	// de
	"Keine Aufgaben.";
#elif L==3	// en
	"No assignments.";
#elif L==4	// es
	"No hay actividades.";
#elif L==5	// fr
	"Il n'y a pas d'activit&eacute;s.";
#elif L==6	// gn
	"No hay actividades.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono attivit&agrave;.";
#elif L==8	// pl
	"Brak zadania.";
#elif L==9	// pt
	"N&atilde;o h&aacute; atividades.";
#endif

const char *Txt_No_banners =
#if   L==1	// ca
	"No hi ha banners.";
#elif L==2	// de
	"Keine Werbebanners.";
#elif L==3	// en
	"No banners.";
#elif L==4	// es
	"No hay banners.";
#elif L==5	// fr
	"Il n'y a pas de banni&eacute;res.";
#elif L==6	// gn
	"No hay banners.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono banners.";
#elif L==8	// pl
	"Brak banners.";
#elif L==9	// pt
	"N&atilde;o h&aacute; banners.";
#endif

const char *Txt_no_course_of_origin =	// Means: "message sent from any course"
#if   L==1	// ca
	"sin asignatura de origen";	// Necessita traduccio
#elif L==2	// de
	"no course of origin";		// Need bersetzung
#elif L==3	// en
	"no course of origin";
#elif L==4	// es
	"sin asignatura de origen";
#elif L==5	// fr
	"no course of origin";		// Besoin de traduction
#elif L==6	// gn
	"sin asignatura de origen";	// Okoteve traduccin
#elif L==7	// it
	"senza corso di origine";
#elif L==8	// pl
	"kurs nie pochodzenia";
#elif L==9	// pt
	"nenhuma disciplina de origem";
#endif

const char *Txt_no_course_selected =
#if   L==1	// ca
	"cap assignatura seleccionada";
#elif L==2	// de
	"kein Kurs ausgew&auml;hlt";
#elif L==3	// en
	"no course selected";
#elif L==4	// es
	"ninguna asignatura seleccionada";
#elif L==5	// fr
	"pas de mati&egrave;re choisi";
#elif L==6	// gn
	"ninguna asignatura seleccionada";
#elif L==7	// it
	"nessun corso scelto";
#elif L==8	// pl
	"kurs nie jest wybrany";
#elif L==9	// pt
	"nenhuma disciplina seleccionada";
#endif

const char *Txt_No_centres =
#if   L==1	// ca
	"No hi ha centres.";
#elif L==2	// de
	"Keine Lehrinstitute.";
#elif L==3	// en
	"No centres.";
#elif L==4	// es
	"No hay centros.";
#elif L==5	// fr
	"Il n'y a pas de centres.";
#elif L==6	// gn
	"No hay centros.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono centri.";
#elif L==8	// pl
	"Brak centra.";
#elif L==9	// pt
	"N&atilde;o h&aacute; centros.";
#endif

const char *Txt_No_courses =
#if   L==1	// ca
	"No hi ha assignatures.";
#elif L==2	// de
	"Keine Kursen.";
#elif L==3	// en
	"No courses.";
#elif L==4	// es
	"No hay asignaturas.";
#elif L==5	// fr
	"Il n'y a pas de mati&egrave;res.";
#elif L==6	// gn
	"No hay asignaturas.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono laurea.";
#elif L==8	// pl
	"Brak kurs&oacute;w.";
#elif L==9	// pt
	"N&atilde;o h&aacute; disciplinas.";
#endif

const char *Txt_No_types_of_degree =
#if   L==1	// ca
	"No hay tipos de titulaci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"No types of degree.";			// Need bersetzung
#elif L==3	// en
	"No types of degree.";
#elif L==4	// es
	"No hay tipos de titulaci&oacute;n.";
#elif L==5	// fr
	"There are no types of degree.";	// Besoin de traduction
#elif L==6	// gn
	"No hay tipos de titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono tipi di lauree.";
#elif L==8	// pl
	"No types of degree.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; tipos de grau.";
#endif

const char *Txt_No_degrees =
#if   L==1	// ca
	"No hi ha titulacions.";
#elif L==2	// de
	"Keine Studieng&auml;nge.";
#elif L==3	// en
	"No degrees.";
#elif L==4	// es
	"No hay titulaciones.";
#elif L==5	// fr
	"Il n'y a pas d'&eacute;tudes.";
#elif L==6	// gn
	"No hay titulaciones.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono corsi.";
#elif L==8	// pl
	"Brak stopni.";
#elif L==9	// pt
	"N&atilde;o h&aacute; graus.";
#endif

const char *Txt_No_enrolment_requests =
#if   L==1	// ca
	"No hay solicitudes de inscripci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"No enrolment requests.";			// Need bersetzung
#elif L==3	// en
	"No enrolment requests.";
#elif L==4	// es
	"No hay solicitudes de inscripci&oacute;n.";
#elif L==5	// fr
	"No enrolment requests.";			// Besoin de traduction
#elif L==6	// gn
	"No hay solicitudes de inscripci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"No enrolment requests.";			// Bisogno di traduzione
#elif L==8	// pl
	"No enrolment requests.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; solicita&ccedil;&otilde;es de inscri&ccedil;&atilde;o.";
#endif

const char *Txt_No_events =
#if   L==1	// ca
	"No hi ha esdeveniments.";
#elif L==2	// de
	"Keine Ereignisse.";
#elif L==3	// en
	"No events.";
#elif L==4	// es
	"No hay eventos.";
#elif L==5	// fr
	"Il n'y a pas d'&eacute;v&eacute;nements.";
#elif L==6	// gn
	"No hay eventos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono eventi.";
#elif L==8	// pl
	"Brak wydarzen.";
#elif L==9	// pt
	"N&atilde;o h&aacute; eventos.";
#endif

const char *Txt_No_file_index_html_index_htm_found_within_the_ZIP_file =
#if   L==1	// ca
	"No se ha encontrado un archivo <em>index.html</em> / <em>index.htm</em>"
	" dentro del archivo <em>ZIP</em>.";		// Necessita traduccio
#elif L==2	// de
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";		// Need bersetzung
#elif L==3	// en
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";
#elif L==4	// es
	"No se ha encontrado un archivo <em>index.html</em> / <em>index.htm</em>"
	" dentro del archivo <em>ZIP</em>.";
#elif L==5	// fr
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";		// Besoin de traduction
#elif L==6	// gn
	"No se ha encontrado un archivo <em>index.html</em> / <em>index.htm</em>"
	" dentro del archivo <em>ZIP</em>.";		// Okoteve traduccin
#elif L==7	// it
	"Nessun documento <em>index.html</em> / <em>index.htm</em> trovato"
	" nel file <em>ZIP</em>.";
#elif L==8	// pl
	"Nr pliku <em>indeks.html</em> / <em>indeks.htm</em> znalezionych"
	" w <em>ZIP</em> plik.";
#elif L==9	// pt
	"Um arquivo <em>index.html</em> / <em>index.htm</em> n&atilde;o foi encontrado"
	" dentro do arquivo <em>ZIP</em>.";
#endif

const char *Txt_No_games =
#if   L==1	// ca
	"No hi ha jocs.";
#elif L==2	// de
	"Keine Spiele.";
#elif L==3	// en
	"No games.";
#elif L==4	// es
	"No hay juegos.";
#elif L==5	// fr
	"Il n'y a pas de jeux.";
#elif L==6	// gn
	"No hay juegos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono giochi.";
#elif L==8	// pl
	"Brak gry.";
#elif L==9	// pt
	"N&atilde;o h&aacute; jogos.";
#endif

const char *Txt_No_groups_have_been_created_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No se han creado grupos en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"No groups have been created in the course <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"No groups have been created in the course <strong>%s</strong>.";
#elif L==4	// es
	"No se han creado grupos en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"No groups have been created in the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"No se han creado grupos en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun gruppo &egrave; stato creato nel corso <strong>%s</strong>.";
#elif L==8	// pl
	"Nr grupy zostaly utworzone w ramach <strong>%s</strong>.";
#elif L==9	// pt
	"Nenhum grupo foi criado na disciplina <strong>%s</strong>.";
#endif

const char *Txt_No_groups_have_been_created_in_the_course_X_Therefore_ =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados ser&aacute;n inscritos/eliminados en/de la asignatura.";	// Necessita traduccio
#elif L==2	// de
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enroled/removed in/from the course.";				// Need bersetzung
#elif L==3	// en
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enroled/removed in/from the course.";
#elif L==4	// es
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados ser&aacute;n inscritos/eliminados en/de la asignatura.";
#elif L==5	// fr
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enroled/removed in/from the course.";				// Besoin de traduction
#elif L==6	// gn
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados ser&aacute;n inscritos/eliminados en/de la asignatura.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun gruppo &egrave; stato creato nel corso <strong>%s</strong>."
	"Dunque, gli utenti specificati saranno registrati/rimossi in/dal corso.";
#elif L==8	// pl
	"Nr grupy zostaly utworzone w ramach <strong>%s</strong>."
	"Dlatego okreslonych uzytkownik&oacute;w zostanie zarejestrowany/usuniete w/z kursu.";
#elif L==9	// pt
	"Nenhum grupo foi criado na disciplina <strong>%s</strong>."
	" Portanto, os usu&aacute;rios especificados ser&atilde;o inscritos/removidos da disciplina.";
#endif

const char *Txt_No_holidays =
#if   L==1	// ca
	"No hay d&iacute;as festivos.";	// Necessita traduccio
#elif L==2	// de
	"No holidays.";			// Need bersetzung
#elif L==3	// en
	"No holidays.";
#elif L==4	// es
	"No hay d&iacute;as festivos.";
#elif L==5	// fr
	"TNo holidays.";		// Besoin de traduction
#elif L==6	// gn
	"No hay d&iacute;as festivos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono festivit&agrave;.";
#elif L==8	// pl
	"No holidays.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; feriados.";
#endif

const char *Txt_No_image_video =	// Without any image
#if   L==1	// ca
	"Sense imatge / v&iacute;deo";
#elif L==2	// de
	"Ohne Abbild / Video";
#elif L==3	// en
	"No image / video";
#elif L==4	// es
	"Sin imagen / v&iacute;deo";
#elif L==5	// fr
	"Pas d'image / vid&eacute;o";
#elif L==6	// gn
	"Sin imagen / v&iacute;deo";	// Okoteve traduccin
#elif L==7	// it
	"Nessuna immagine / video";
#elif L==8	// pl
	"Brak obrazka / wideo";
#elif L==9	// pt
	"Sem imagem / v&iacute;deo";
#endif

const char *Txt_No_INDEX =	// Short version of "Number" (as an index)...
#if   L==1	// ca
	"N&ordm;";	// Necessita traduccio
#elif L==2	// de
	"Anzahl";
#elif L==3	// en
	"#";
#elif L==4	// es
	"N&ordm;";
#elif L==5	// fr
	"Nombre";
#elif L==6	// gn
	"Papanga";
#elif L==7	// it
	"Numero";
#elif L==8	// pl
	"Numer";
#elif L==9	// pt
	"N&ordm;";
#endif

const char *Txt_No_information =
#if   L==1	// ca
	"No hi ha informaci&oacute;.";
#elif L==2	// de
	"Keine Informationen.";
#elif L==3	// en
	"No information.";
#elif L==4	// es
	"No hay informaci&oacute;n.";
#elif L==5	// fr
	"Aucune information.";
#elif L==6	// gn
	"No hay informaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Nessuna informazione.";
#elif L==8	// pl
	"Brak informacji.";
#elif L==9	// pt
	"N&atilde;o h&aacute; informa&ccedil;&atilde;o.";
#endif

const char *Txt_No_institutions =
#if   L==1	// ca
	"No hi ha institucions.";
#elif L==2	// de
	"Keine Hochschulen.";
#elif L==3	// en
	"No institutions.";
#elif L==4	// es
	"No hay instituciones.";
#elif L==5	// fr
	"Il n'y a pas d'&eacute;tablissements.";
#elif L==6	// gn
	"No hay instituciones.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono istituzioni.";
#elif L==8	// pl
	"Brak instytucje.";
#elif L==9	// pt
	"N&atilde;o h&aacute; institu&ccedil;&otilde;es.";
#endif

const char *Txt_No_links =
#if   L==1	// ca
	"No hi ha enlla&ccedil;os.";
#elif L==2	// de
	"Keine Links.";
#elif L==3	// en
	"No links.";
#elif L==4	// es
	"No hay enlaces.";
#elif L==5	// fr
	"Il n'y a pas de liens.";
#elif L==6	// gn
	"No hay enlaces.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono link.";
#elif L==8	// pl
	"Brak linki.";
#elif L==9	// pt
	"N&atilde;o h&aacute; liga&ccedil;&otilde;es.";
#endif

const char *Txt_No_BR_msgs =
#if   L==1	// ca
	"N&ordm;<br />mens.";	// Necessita traduccio
#elif L==2	// de
	"Anzahl<br />Nachr.";
#elif L==3	// en
	"No.<br />msgs.";
#elif L==4	// es
	"N&ordm;<br />mens.";
#elif L==5	// fr
	"Nombre<br />msgs.";
#elif L==6	// gn
	"N&ordm;<br />mens.";	// Okoteve traduccin
#elif L==7	// it
	"Num.<br />msg.";
#elif L==8	// pl
	"Numer<br />msgs.";
#elif L==9	// pt
	"N&ordm;<br />mens.";
#endif

const char *Txt_No_new_posts =
#if   L==1	// ca
	"No hay comentarios nuevos";	// Necessita traduccio
#elif L==2	// de
	"Keine neuen Nachrichten";
#elif L==3	// en
	"No new posts";
#elif L==4	// es
	"No hay comentarios nuevos";
#elif L==5	// fr
	"Pas de nouveaux posts";
#elif L==6	// gn
	"No hay comentarios nuevos";	// Okoteve traduccin
#elif L==7	// it
	"Nessun nuovo post";
#elif L==8	// pl
	"Brak nowych post&oacute;w";
#elif L==9	// pt
	"Sem posts novos";
#endif

const char *Txt_No_notices =
#if   L==1	// ca
	"No hay avisos.";	// Necessita traduccio
#elif L==2	// de
	"Keine Ank&uuml;ndigungen.";
#elif L==3	// en
	"No notices.";
#elif L==4	// es
	"No hay avisos.";
#elif L==5	// fr
	"Il n'y a pas d'avis.";
#elif L==6	// gn
	"No hay avisos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono avvisi.";
#elif L==8	// pl
	"Brak ogloszen.";
#elif L==9	// pt
	"N&atilde;o h&aacute; avisos.";
#endif

const char *Txt_No_of_files =
#if   L==1	// ca
	"Nombre d'arxius";
#elif L==2	// de
	"Anzahl der Dateien";
#elif L==3	// en
	"Number of files";
#elif L==4	// es
	"N&ordm; de archivos";
#elif L==5	// fr
	"Nombre de fichiers";
#elif L==6	// gn
	"N&ordm; de archivos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di file";
#elif L==8	// pl
	"Liczba pliki";
#elif L==9	// pt
	"N&ordm; de arquivos";
#endif

const char *Txt_No_of_files_in_SHARE_zones =
#if   L==1	// ca
	"Nombre d'arxius compartits";
#elif L==2	// de
	"Anzahl der freigegebene Dateien";
#elif L==3	// en
	"Number of shared files";
#elif L==4	// es
	"N&ordm; de archivos compartidos";
#elif L==5	// fr
	"Nombre de fichiers partag&eacute;s";
#elif L==6	// gn
	"N&ordm; de archivos compartidos";		// Okoteve traduccin
#elif L==7	// it
	"Numero di file condivisi";
#elif L==8	// pl
	"Liczba pliki udost&eogon;pnione";
#elif L==9	// pt
	"N&ordm; de arquivos compartilhados";
#endif

const char *Txt_No_of_files_in_DOCUM_zones =
#if   L==1	// ca
	"N&ordm; de archivos en zonas de documentos";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der Dateien im Dokumente-Bereich";
#elif L==3	// en
	"Number of files in documents area";
#elif L==4	// es
	"N&ordm; de archivos en zonas de documentos";
#elif L==5	// fr
	"Nombre de fichiers dans les zones de documents";
#elif L==6	// gn
	"N&ordm; de archivos en zonas de documentos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di file nell'area documenti";
#elif L==8	// pl
	"Number of files in documents area";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&ordm; de arquivos em zonas de documentos";
#endif

const char *Txt_No_of_forums =
#if   L==1	// ca
	"N&ordm; de foros";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der Foren";
#elif L==3	// en
	"No. of forums";
#elif L==4	// es
	"N&ordm; de foros";
#elif L==5	// fr
	"Nombre de forums";
#elif L==6	// gn
	"N&ordm; de foros";	// Okoteve traduccin
#elif L==7	// it
	"Numero di forum";
#elif L==8	// pl
	"Liczba forum";
#elif L==9	// pt
	"N&ordm; de f&oacute;runs";
#endif

const char *Txt_No_of_BR_groups =
#if   L==1	// ca
	"N&ordm; de<br />grupos";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der<br />Gruppen";
#elif L==3	// en
	"No. of<br />groups";
#elif L==4	// es
	"N&ordm; de<br />grupos";
#elif L==5	// fr
	"Nombre de<br />groupes";
#elif L==6	// gn
	"N&ordm; de<br />grupos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />gruppi";
#elif L==8	// pl
	"Liczba z<br />grupy";
#elif L==9	// pt
	"N&ordm; de<br />grupos";
#endif

const char *Txt_No_of_indicators =
#if   L==1	// ca
	"N&ordm; de indicadores";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der Indikatoren";
#elif L==3	// en
	"No. of indicators";
#elif L==4	// es
	"N&ordm; de indicadores";
#elif L==5	// fr
	"Nombre d'indicateurs";
#elif L==6	// gn
	"N&ordm; de indicadores";	// Okoteve traduccin
#elif L==7	// it
	"Numero di indicatori";
#elif L==8	// pl
	"Liczba z wskazniki";
#elif L==9	// pt
	"N&ordm; de indocadores";
#endif

const char *Txt_No_of_BR_lines =
#if   L==1	// ca
	"N&ordm; de<br />l&iacute;neas";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der<br />Zeilen";
#elif L==3	// en
	"No. of<br />lines";
#elif L==4	// es
	"N&ordm; de<br />l&iacute;neas";
#elif L==5	// fr
	"Nombre de<br />lignes";
#elif L==6	// gn
	"N&ordm; de<br />l&iacute;neas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />linee";
#elif L==8	// pl
	"Liczba z<br />linie";
#elif L==9	// pt
	"N&ordm; de<br />linhas";
#endif

const char *Txt_No_of_posts =
#if   L==1	// ca
	"N&ordm; de comentarios";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der Nachrichten";
#elif L==3	// en
	"No. of posts";
#elif L==4	// es
	"N&ordm; de comentarios";
#elif L==5	// fr
	"Nombre de posts";
#elif L==6	// gn
	"N&ordm; de comentarios";	// Okoteve traduccin
#elif L==7	// it
	"Numero di post";
#elif L==8	// pl
	"Liczba wiadomosci";
#elif L==9	// pt
	"N&ordm; de posts";
#endif

const char *Txt_No_of_posts_BR_per_forum =
#if   L==1	// ca
	"N&ordm; de comentarios<br />por foro";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der Nachrichten<br />pro Forum";
#elif L==3	// en
	"No. of posts<br />per forum";
#elif L==4	// es
	"N&ordm; de comentarios<br />por foro";
#elif L==5	// fr
	"Nombre de posts<br />par forum";
#elif L==6	// gn
	"N&ordm; de comentarios<br />por foro";	// Okoteve traduccin
#elif L==7	// it
	"Numero di post<br />per forum";
#elif L==8	// pl
	"Liczba wiadomosci<br />per forum";
#elif L==9	// pt
	"N&ordm; de posts<br />por f&oacute;rum";
#endif

const char *Txt_No_of_posts_BR_per_user =
#if   L==1	// ca
	"N&ordm; de comentarios<br />por usuari";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der Nachrichten<br />pro Benutzer";
#elif L==3	// en
	"No. of posts<br />per user";
#elif L==4	// es
	"N&ordm; de comentarios<br />por usuario";
#elif L==5	// fr
	"Nombre de posts<br />par utilisateur";
#elif L==6	// gn
	"N&ordm; de comentarios<br />por puruh&aacute;ra";	// Okoteve traduccin
#elif L==7	// it
	"Numero di post<br />per utente";
#elif L==8	// pl
	"Liczba wiadomosci<br />per u&zdot;ytkownik";
#elif L==9	// pt
	"N&ordm; de posts<br />por utilizador";
#endif

const char *Txt_No_of_posts_BR_per_thread =
#if   L==1	// ca
	"N&ordm; de comentarios<br />por discusi&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der Nachrichten<br />pro Thread";
#elif L==3	// en
	"No. of posts<br />per thread";
#elif L==4	// es
	"N&ordm; de comentarios<br />por usuario";
#elif L==5	// fr
	"Nombre de posts<br />par fil";
#elif L==6	// gn
	"N&ordm; de comentarios<br />por discusi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Numero di post<br />per discussione";
#elif L==8	// pl
	"Liczba wiadomosci<br />per watku";
#elif L==9	// pt
	"N&ordm; de posts<br />por discuss&atilde;o";
#endif

const char *Txt_No_of_private_files =
#if   L==1	// ca
	"Nombre d'arxius privats";
#elif L==2	// de
	"Anzahl der private Dateien";
#elif L==3	// en
	"Number of private files";
#elif L==4	// es
	"N&ordm; de archivos privados";
#elif L==5	// fr
	"Nombre de fichiers priv&eacute;s";
#elif L==6	// gn
	"N&ordm; de archivos privados";	// Okoteve traduccin
#elif L==7	// it
	"Numero di file privati";
#elif L==8	// pl
	"Liczba prywatne pliki";
#elif L==9	// pt
	"N&ordm; de arquivos privados";
#endif

const char *Txt_No_of_public_files =
#if   L==1	// ca
	"Nombre d'arxius p&uacute;blics";
#elif L==2	// de
	"Anzahl der &ouml;ffentliche Dateien";
#elif L==3	// en
	"Number of public files";
#elif L==4	// es
	"N&ordm; de archivos p&uacute;blicos";
#elif L==5	// fr
	"Nombre de fichiers publics";
#elif L==6	// gn
	"N&ordm; de archivos p&uacute;blicos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di file pubblici";
#elif L==8	// pl
	"Liczba publicznych pliki";
#elif L==9	// pt
	"N&ordm; de arquivos p&uacute;blicos";
#endif

const char *Txt_No_of_questions =
#if   L==1	// ca
	"Nombre de preguntes";
#elif L==2	// de
	"Anzahl der Fragen";
#elif L==3	// en
	"No. of questions";
#elif L==4	// es
	"N&ordm; de preguntas";
#elif L==5	// fr
	"Nombre de questions";
#elif L==6	// gn
	"N&ordm; de preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di domande";
#elif L==8	// pl
	"Liczba pyta&nacute;";
#elif L==9	// pt
	"N&ordm; de quest&otilde;es";
#endif

const char *Txt_No_of_social_posts =
#if   L==1	// ca
	"Nombre de missatges socials";
#elif L==2	// de
	"Anzahl der sozialen Beitr&auml;ge";
#elif L==3	// en
	"No. of social posts";
#elif L==4	// es
	"N&ordm; de mensajes sociales";
#elif L==5	// fr
	"Nombre de messages sociaux";
#elif L==6	// gn
	"N&ordm; de mensajes sociales";	// Okoteve traduccin
#elif L==7	// it
	"Numero di post sociali";
#elif L==8	// pl
	"Liczba post spo&lstrok;ecznej";
#elif L==9	// pt
	"N&ordm; de post sociais";
#endif

const char *Txt_No_of_threads =
#if   L==1	// ca
	"N&ordm; de discusiones";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der Threads";
#elif L==3	// en
	"No. of threads";
#elif L==4	// es
	"N&ordm; de discusiones";
#elif L==5	// fr
	"Nombre de fils";
#elif L==6	// gn
	"N&ordm; de discusiones";	// Okoteve traduccin
#elif L==7	// it
	"Numero di discussioni";
#elif L==8	// pl
	"Liczba watk&oacute;w";
#elif L==9	// pt
	"N&ordm; de discuss&otilde;es";
#endif

const char *Txt_No_of_threads_BR_per_forum =
#if   L==1	// ca
	"N&ordm; de discusiones<br />por foro";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der Threads<br />pro Forum";
#elif L==3	// en
	"No. of threads<br />per forum";
#elif L==4	// es
	"N&ordm; de discusiones<br />por foro";
#elif L==5	// fr
	"Nombre de fils<br />par forum";
#elif L==6	// gn
	"N&ordm; de discusiones<br />por foro";	// Okoteve traduccin
#elif L==7	// it
	"Numero di discussioni<br />per forum";
#elif L==8	// pl
	"Liczba watk&oacute;w<br />per forum";
#elif L==9	// pt
	"N&ordm; de discuss&otilde;es<br />por f&oacute;rum";
#endif

const char *Txt_Number_of_BR_times_that_BR_questions_BR_have_been_BR_responded =
#if   L==1	// ca
	"N&ordm; de<br />veces<br />que se ha<br />respondido";	// Necessita traduccio
#elif L==2	// de
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Need bersetzung
#elif L==3	// en
	"Number of<br />times that<br />questions<br />have been<br />responded";
#elif L==4	// es
	"N&ordm; de<br />veces<br />que se ha<br />respondido";
#elif L==5	// fr
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Besoin de traduction
#elif L==6	// gn
	"N&ordm; de<br />veces<br />que se ha<br />respondido";	// Okoteve traduccin
#elif L==7	// it
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Bisogno di traduzione
#elif L==8	// pl
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&ordm; de<br />vezes que<br />as perguntas<br />foram<br />respondidas";
#endif

const char *Txt_No_of_users =
#if   L==1	// ca
	"Nombre d'usuaris";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der Benutzer";
#elif L==3	// en
	"No. of users";
#elif L==4	// es
	"N&ordm; de usuarios";
#elif L==5	// fr
	"Nombre d'utilisateurs";
#elif L==6	// gn
	"N&ordm; de usuarios";	// Okoteve traduccin
#elif L==7	// it
	"Numero di utenti";
#elif L==8	// pl
	"Liczba uzytkownik&oacute;w";
#elif L==9	// pt
	"N&ordm; de utilizadores";
#endif

const char *Txt_No_projects =
#if   L==1	// ca
	"No hi ha projectes.";
#elif L==2	// de
	"Keine Projekte.";
#elif L==3	// en
	"No projects.";
#elif L==4	// es
	"No hay proyectos.";
#elif L==5	// fr
	"Il n'y a pas de projets.";
#elif L==6	// gn
	"No hay proyectos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono progetti.";
#elif L==8	// pl
	"Brak projekty.";
#elif L==9	// pt
	"N&atilde;o h&aacute; projetos.";
#endif

const char *Txt_No_questions_found_matching_your_search_criteria =
#if   L==1	// ca
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";	// Necessita traduccio
#elif L==2	// de
	"Keine Testfragen gefunden, die mit Ihrer Suchanfrage entsprechen.";
#elif L==3	// en
	"No questions found matching your search criteria.";
#elif L==4	// es
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";
#elif L==5	// fr
	"Aucune question trouv&eacute;e correspondant &agrave; vos crit&egrave;res de recherche.";
#elif L==6	// gn
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono domande trovate con il criterio di ricerca selezionato.";
#elif L==8	// pl
	"Brak pyta&nacute; spelniajacych zadane kryteria wyszukiwania.";
#elif L==9	// pt
	"N&atilde;o quest&otilde;es com os seus crit&eacute;rios de pesquisa selecionados.";
#endif

const char *Txt_No_results =
#if   L==1	// ca
	"No hi ha resultats.";
#elif L==2	// de
	"Keine Ergebnisse.";
#elif L==3	// en
	"No results.";
#elif L==4	// es
	"No hay resultados.";
#elif L==5	// fr
	"Pas de r&eacute;sultats.";
#elif L==6	// gn
	"No hay resultados.";		// Okoteve traduccin
#elif L==7	// it
	"Nessun risultato.";
#elif L==8	// pl
	"Brak wynik&oacute;w.";
#elif L==9	// pt
	"Nenhum resultado.";
#endif

const char *Txt_no_subject =
#if   L==1	// ca
	"sin asunto";	// Necessita traduccio
#elif L==2	// de
	"kein Betreff";
#elif L==3	// en
	"no subject";
#elif L==4	// es
	"sin asunto";
#elif L==5	// fr
	"pas de sujet";
#elif L==6	// gn
	"sin asunto";	// Okoteve traduccin
#elif L==7	// it
	"nessun oggetto";
#elif L==8	// pl
	"bez tematu";
#elif L==9	// pt
	"sem assunto";
#endif

const char *Txt_No_surveys =
#if   L==1	// ca
	"No hi ha enquestes.";
#elif L==2	// de
	"Keine Umfragen.";
#elif L==3	// en
	"No surveys.";
#elif L==4	// es
	"No hay encuestas.";
#elif L==5	// fr
	"Il n'y a pas de sondages.";
#elif L==6	// gn
	"No hay encuestas.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono sondaggi.";
#elif L==8	// pl
	"Brak badania.";
#elif L==9	// pt
	"N&atilde;o h&aacute; inqu&eacute;ritos.";
#endif

const char *Txt_no_tags =		// Tags used in test
#if   L==1	// ca
	"sin descriptores";	// Necessita traduccio
#elif L==2	// de
	"keine Tags";
#elif L==3	// en
	"no tags";
#elif L==4	// es
	"sin descriptores";
#elif L==5	// fr
	"pas de descripteurs";
#elif L==6	// gn
	"sin descriptores";	// Okoteve traduccin
#elif L==7	// it
	"senza etichette";
#elif L==8	// pl
	"zadnych tag&oacute;w";
#elif L==9	// pt
	"sem descritores";
#endif

const char *Txt_No_teachers_or_students_found =
#if   L==1	// ca
	"No hay profesores ni estudiantes.";	// Necessita traduccio
#elif L==2	// de
	"Keine Lehrer oder Studenten gefunden.";
#elif L==3	// en
	"No teachers or students found.";
#elif L==4	// es
	"No hay profesores ni estudiantes.";
#elif L==5	// fr
	"Aucun enseignant ou &eacute;tudiant trouv&eacute;.";
#elif L==6	// gn
	"No hay profesores ni estudiantes.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun professore o studente trovato.";
#elif L==8	// pl
	"Nie znaleziono nauczycieli i uczni&oacute;w.";
#elif L==9	// pt
	"Nenhum professor ou estudante encontrado.";
#endif

const char *Txt_No_test_questions =
#if   L==1	// ca
	"No hay preguntas de test.";	// Necessita traduccio
#elif L==2	// de
	"Keine Testfragen.";
#elif L==3	// en
	"No test questions.";
#elif L==4	// es
	"No hay preguntas de test.";
#elif L==5	// fr
	"Il n'y a pas de questions de test.";
#elif L==6	// gn
	"No hay preguntas de test.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono domande di test.";
#elif L==8	// pl
	"Brak pyta&nacute; testowych.";
#elif L==9	// pt
	"N&atilde;o existem quest&otilde;es de test.";
#endif

const char *Txt_No_user_has_been_eliminated =
#if   L==1	// ca
	"No se ha eliminado ning&uacute;n usuario.";	// Necessita traduccio
#elif L==2	// de
	"No user has been eliminated.";			// Need bersetzung
#elif L==3	// en
	"No user has been eliminated.";
#elif L==4	// es
	"No se ha eliminado ning&uacute;n usuario.";
#elif L==5	// fr
	"No user has been eliminated.";			// Besoin de traduction
#elif L==6	// gn
	"No se ha eliminado ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun utente &egrave; stato rimosso.";
#elif L==8	// pl
	"Zaden uzytkownik nie zostal wyeliminowany.";
#elif L==9	// pt
	"Nenhum usu&aacute;rio foi removido.";
#endif

const char *Txt_No_user_has_been_enroled =
#if   L==1	// ca
	"No se ha inscrito ning&uacute;n usuario.";	// Necessita traduccio
#elif L==2	// de
	"No user has been enroled.";			// Need bersetzung
#elif L==3	// en
	"No user has been enroled.";
#elif L==4	// es
	"No se ha inscrito ning&uacute;n usuario.";
#elif L==5	// fr
	"No user has been enroled.";			// Besoin de traduction
#elif L==6	// gn
	"No se ha inscrito ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun utente &egrave; stato registrato.";
#elif L==8	// pl
	"Zaden uzytkownik nie zostal zarejestrowany.";
#elif L==9	// pt
	"Nenhum usu&aacute;rio se registrou.";
#endif

const char *Txt_No_user_has_been_removed =
#if   L==1	// ca
	"No se ha eliminado ning&uacute;n usuario.";	// Necessita traduccio
#elif L==2	// de
	"No user has been removed.";			// Need bersetzung
#elif L==3	// en
	"No user has been removed.";
#elif L==4	// es
	"No se ha eliminado ning&uacute;n usuario.";
#elif L==5	// fr
	"No user has been removed.";			// Besoin de traduction
#elif L==6	// gn
	"No se ha eliminado ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun utente &egrave; stato rimosso.";
#elif L==8	// pl
	"Zaden uzytkownik nie zostal usuniety.";
#elif L==9	// pt
	"Nenhum usu&aacute;rio foi removido.";
#endif

const char *Txt_No_user_to_whom_you_can_follow_Try_again_later =
#if   L==1	// ca
	"No hi ha cap usuari a qui vost&egrave; pugui seguir."
	" Intenteu-ho m&eacute;s endavant.";
#elif L==2	// de
	"Kein Benutzer dem Sie folgen k&ouml;nnen."
	" Versuchen Sie es sp&auml;ter noch einmal.";
#elif L==3	// en
	"No user to whom you can follow."
	" Try again later.";
#elif L==4	// es
	"No hay ning&uacute;n usuario a quien usted pueda seguir."
	" Int&eacute;ntelo m&aacute;s adelante.";
#elif L==5	// fr
	"Aucun utilisateur auquel vous pouvez suivre."
	" R&eacute;essayez plus tard.";
#elif L==6	// gn
	"No hay ning&uacute;n usuario a quien usted pueda seguir."
	" Int&eacute;ntelo m&aacute;s adelante.";	// Okoteve traduccin
#elif L==7	// it
	"Nessun utente a cui &egrave; possibile seguire."
	" Riprovare pi&ugrave; tardi.";
#elif L==8	// pl
	"No u&zdot;ytkownika, do kt&oacute;rego mo&zdot;na &sacute;ledzi&cacute;."
	" Spr&oacute;buj ponownie p&oacute;&zacute;niej.";
#elif L==9	// pt
	"Nenhum usu&aacute;rio a quem voc&ecirc; pode seguir."
	" Tente novamente mais tarde.";
#endif

const char *Txt_No_users_found[Rol_NUM_ROLES] =
   {
#if   L==1	// ca	// Rol_UNK
	 "No hi ha usuaris."
#elif L==2	// de
	 "Keine Benutzer gefunden."
#elif L==3	// en
	 "No users found."
#elif L==4	// es
	 "No hay usuarios."
#elif L==5	// fr
	 "Aucun utilisateur trouv&eacute;."
#elif L==6	// gn
	 "No hay usuarios."	// Okoteve traduccin
#elif L==7	// it
	 "Nessun utente trovato."
#elif L==8	// pl
	 "Nie znaleziono uzytkownik&oacute;w."
#elif L==9	// pt
	 "Nenhum utilizador encontrado."
#endif
	 ,
#if   L==1	// ca	// Rol_GST
	 "No hi ha convidats."
#elif L==2	// de
	 "Keine Eingeladen gefunden."
#elif L==3	// en
	 "No guests found."
#elif L==4	// es
	 "No hay invitados."
#elif L==5	// fr
	 "Aucun invit&eacute; trouv&eacute;."
#elif L==6	// gn
	 "No hay invitados."	// Okoteve traduccin
#elif L==7	// it
	 "Nessun ospite trovato."
#elif L==8	// pl
	 "Nie znaleziono zaproszeni."
#elif L==9	// pt
	 "Nenhum convidado encontrado."
#endif
	 ,
#if   L==1	// ca	// Rol_USR
	 "No hi ha usuaris."
#elif L==2	// de
	 "Keine Benutzer gefunden."
#elif L==3	// en
	 "No users found."
#elif L==4	// es
	 "No hay usuarios."
#elif L==5	// fr
	 "Aucun utilisateur trouv&eacute;."
#elif L==6	// gn
	 "No hay usuarios."	// Okoteve traduccin
#elif L==7	// it
	 "Nessun utente trovato."
#elif L==8	// pl
	 "Nie znaleziono uzytkownik&oacute;w."
#elif L==9	// pt
	 "Nenhum utilizador encontrado."
#endif
	 ,
#if   L==1	// ca	// Rol_STD
	 "No hi ha estudiants."
#elif L==2	// de
	 "Keine Studenten gefunden."
#elif L==3	// en
	 "No students found."
#elif L==4	// es
	 "No hay estudiantes."
#elif L==5	// fr
	 "Aucun &eacute;tudiant trouv&eacute;."
#elif L==6	// gn
	 "No hay estudiantes."	// Okoteve traduccin
#elif L==7	// it
	 "Non ci sono studenti."
#elif L==8	// pl
	 "Nie znaleziono uczni&oacute;w."
#elif L==9	// pt
	 "Nenhum estudante encontrado."
#endif
	 ,
#if   L==1	// ca	// Rol_NET
	 "No hi ha professors no editors."
#elif L==2	// de
	 "Keine nicht bearbeiteter Lehrkr&auml;ften gefunden."
#elif L==3	// en
	 "No non-editing teachers found."
#elif L==4	// es
	 "No hay profesores no editores."
#elif L==5	// fr
	 "Aucun enseignant non-&eacute;diteur trouv&eacute;."
#elif L==6	// gn
	 "No hay profesores no editores."	// Okoteve traduccin
#elif L==7	// it
	 "Non ci sono professori non-editing."
#elif L==8	// pl
	 "Nie znaleziono nauczycieli nie edytuj&aogon;cy."
#elif L==9	// pt
	 "Nenhum professor n&atilde;o editor encontrado."
#endif
	 ,
#if   L==1	// ca	// Rol_TCH
	 "No hi ha professors."
#elif L==2	// de
	 "Keine Lehrkr&auml;ften gefunden."
#elif L==3	// en
	 "No teachers found."
#elif L==4	// es
	 "No hay profesores."
#elif L==5	// fr
	 "Aucun enseignant trouv&eacute;."
#elif L==6	// gn
	 "No hay profesores."	// Okoteve traduccin
#elif L==7	// it
	 "Non ci sono professori."
#elif L==8	// pl
	 "Nie znaleziono nauczycieli."
#elif L==9	// pt
	 "Nenhum professor encontrado."
#endif
	 ,
#if   L==1	// ca	// Rol_DEG_ADM
	 "No hi ha administradors."
#elif L==2	// de
	 "Keine Administratoren gefunden."
#elif L==3	// en
	 "No administrators found."
#elif L==4	// es
	 "No hay administradores."
#elif L==5	// fr
	 "Aucun administrateur trouv&eacute;."
#elif L==6	// gn
	 "No hay administradores."	// Okoteve traduccin
#elif L==7	// it
	 "Non ci sono amministratori."
#elif L==8	// pl
	 "Nie znaleziono administrator&oacute;w."
#elif L==9	// pt
	 "Nenhum administrador encontrado."
#endif
	 ,
#if   L==1	// ca	// Rol_CTR_ADM
	 "No hi ha administradors."
#elif L==2	// de
	 "Keine Administratoren gefunden."
#elif L==3	// en
	 "No administrators found."
#elif L==4	// es
	 "No hay administradores."
#elif L==5	// fr
	 "Aucun administrateur trouv&eacute;."
#elif L==6	// gn
	 "No hay administradores."	// Okoteve traduccin
#elif L==7	// it
	 "Non ci sono amministratori."
#elif L==8	// pl
	 "Nie znaleziono administrator&oacute;w."
#elif L==9	// pt
	 "Nenhum administrador encontrado."
#endif
	 ,
#if   L==1	// ca	// Rol_INS_ADM
	 "No hi ha administradors."
#elif L==2	// de
	 "Keine Administratoren gefunden."
#elif L==3	// en
	 "No administrators found."
#elif L==4	// es
	 "No hay administradores."
#elif L==5	// fr
	 "Aucun administrateur trouv&eacute;."
#elif L==6	// gn
	 "No hay administradores."	// Okoteve traduccin
#elif L==7	// it
	 "Non ci sono amministratori."
#elif L==8	// pl
	 "Nie znaleziono administrator&oacute;w."
#elif L==9	// pt
	 "Nenhum administrador encontrado."
#endif
	 ,
#if   L==1	// ca	// Rol_SYS_ADM
	 "No hi ha superusuaris."
#elif L==2	// de
	 "Keine Superuser gefunden."
#elif L==3	// en
	 "No superusers found."
#elif L==4	// es
	 "No hay superusuarios."
#elif L==5	// fr
	 "Aucun superutilisateur trouv&eacute;."
#elif L==6	// gn
	 "No hay superusuarios."	// Okoteve traduccin
#elif L==7	// it
	 "Non ci sono superutenti."
#elif L==8	// pl
	 "Nie znaleziono superu&zdot;ytkownika."
#elif L==9	// pt
	 "Nenhum superutilizador encontrado."
#endif
	};

const char *Txt_nobody_else_can_access_this_content =
#if   L==1	// ca
	"ning&uacute; m&eacute;s pot accedir a aquest contingut";
#elif L==2	// de
	"Niemand sonst kann auf diese Inhalte";
#elif L==3	// en
	"nobody else can access this content";
#elif L==4	// es
	"nadie m&aacute;s puede acceder a este contenido";
#elif L==5	// fr
	"personne d'autre ne peut acc&eacute;der &agrave; ce contenu";
#elif L==6	// gn
	"nadie m&aacute;s puede acceder a este contenido";	// Okoteve traduccin
#elif L==7	// it
	"nessun altro pu&ograve; accedere a questo contenuto";
#elif L==8	// pl
	"nikt inny nie moze uzyskac dostepu do tej zawartosci";
#elif L==9	// pt
	"ningu&eacute;m mais pode acessar este conte&uacute;do";
#endif

const char *Txt_non_blank_QUESTIONS =
#if   L==1	// ca
	"contestades";
#elif L==2	// de
	"Non-blank";
#elif L==3	// en
	"non-blank";
#elif L==4	// es
	"contestadas";
#elif L==5	// fr
	"non vide";
#elif L==6	// gn
	"contestadas";	// Okoteve traduccin
#elif L==7	// it
	"risposte";
#elif L==8	// pl
	"niepustych";
#elif L==9	// pt
	"respondidas";
#endif

const char *Txt_Non_blank_BR_questions =
#if   L==1	// ca
	"Preguntes<br />contestades";
#elif L==2	// de
	"Non-blank<br />Fragen";
#elif L==3	// en
	"Non-blank<br />questions";
#elif L==4	// es
	"Preguntas<br />contestadas";
#elif L==5	// fr
	"Questions<br />non vide";
#elif L==6	// gn
	"Preguntas<br />contestadas";	// Okoteve traduccin
#elif L==7	// it
	"Domande<br />risposte";
#elif L==8	// pl
	"Niepustych<br />pytania";
#elif L==9	// pt
	"Quest&otilde;es<br />respondidas";
#endif

const char *Txt_Not_applicable =
#if   L==1	// ca
	"No aplicable";
#elif L==2	// de
	"Nicht anwendbar";
#elif L==3	// en
	"Not applicable";
#elif L==4	// es
	"No aplicable";
#elif L==5	// fr
	"Non applicable";
#elif L==6	// gn
	"No aplicable";		// Okoteve traduccin
#elif L==7	// it
	"Non applicabile";
#elif L==8	// pl
	"Nie dotyczy";
#elif L==9	// pt
	"N&atilde;o aplic&aacute;vel";
#endif

const char *Txt_not_available =
#if   L==1	// ca
	"no disponible";
#elif L==2	// de
	"Nicht verf&uuml;gbar";
#elif L==3	// en
	"not available";
#elif L==4	// es
	"no disponible";
#elif L==5	// fr
	"pas disponible";
#elif L==6	// gn
	"no disponible";		// Okoteve traduccin
#elif L==7	// it
	"non disponibile";
#elif L==8	// pl
	"niedost&eogon;pne";
#elif L==9	// pt
	"n&atilde;o dispon&iacute;vel";
#endif

const char *Txt_Not_duplicated =
#if   L==1	// ca
	"No &eacute;s duplicat";
#elif L==2	// de
	"Nicht dupliziert";
#elif L==3	// en
	"Not duplicated";
#elif L==4	// es
	"No es duplicado/a";
#elif L==5	// fr
	"Pas dupliqu&eacute;";
#elif L==6	// gn
	"No es duplicado/a";		// Okoteve traduccin
#elif L==7	// it
	"Non duplicato";
#elif L==8	// pl
	"Nie powiela&cacute;";
#elif L==9	// pt
	"N&atilde;o duplicado";
#endif

const char *Txt_Nothing_has_been_pasted_because_the_clipboard_is_empty_ =
#if   L==1	// ca
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";	// Necessita traduccio
#elif L==2	// de
	"Es konnten keine Daten eingef&uuml;gt werden. Der Zwischenspeicher ist leer."
	" Versuchen Sie den Kopieren-Einf&uuml;gen-Vorgang bitte erneut.";
#elif L==3	// en
	"Nothing has been pasted because the clipboard is empty."
	" Try copy-paste again.";
#elif L==4	// es
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";
#elif L==5	// fr
	"Nothing has been pasted because the clipboard is empty."
	" Try copy-paste again.";		// Besoin de traduction
#elif L==6	// gn
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";	// Okoteve traduccin
#elif L==7	// it
	"Niente &egrave; stato copiato perch&egrave; la bacheca &egrave; piena."
	" Prova copia-incolla nuovamente.";
#elif L==8	// pl
	"Nic nie zostal wklejony bo schowek jest pusty."
	" Spr&oacute;buj ponownie kopiuj-wklej.";
#elif L==9	// pt
	"Nada foi colado porque a &aacute;rea de transfer&ecirc;ncia est&aacute; vazia."
	" Tente copiar e colar novamente.";
#endif

const char *Txt_NOTICE_Active_SINGULAR =
#if   L==1	// ca
	"Activo";	// Necessita traduccio
#elif L==2	// de
	"Aktive";
#elif L==3	// en
	"Active";
#elif L==4	// es
	"Activo";
#elif L==5	// fr
	"Active";
#elif L==6	// gn
	"Activo";	// Okoteve traduccin
#elif L==7	// it
	"Attivo";
#elif L==8	// pl
	"Active";
#elif L==9	// pt
	"Ativo";
#endif

const char *Txt_NOTICE_Active_BR_notices =
#if   L==1	// ca
	"Avisos<br />activos";	// Necessita traduccio
#elif L==2	// de
	"Aktive<br />Ank&uuml;ndigungen";
#elif L==3	// en
	"Active<br />notices";
#elif L==4	// es
	"Avisos<br />activos";
#elif L==5	// fr
	"Avis<br />actives";
#elif L==6	// gn
	"Avisos<br />activos";	// Okoteve traduccin
#elif L==7	// it
	"Avvisi<br />attivi";
#elif L==8	// pl
	"Active<br />Ogloszenia";
#elif L==9	// pt
	"Avisos<br />ativos";
#endif

const char *Txt_NOTICE_Active_Mark_as_obsolete =
#if   L==1	// ca
	"Actiu; marcar com obsolet";
#elif L==2	// de
	"Aktive; marke als veraltet";
#elif L==3	// en
	"Active; mark as obsolete";
#elif L==4	// es
	"Activo; marcar como obsoleto";
#elif L==5	// fr
	"Active; marquer comme obsol&egrave;te";
#elif L==6	// gn
	"Activo; marcar como obsoleto";	// Okoteve traduccin
#elif L==7	// it
	"Attivo; segnare come obsoleto";
#elif L==8	// pl
	"Active; zaznaczy jako przestarzale";
#elif L==9	// pt
	"Ativo; marcar como obsoleto";
#endif

const char *Txt_NOTICE_Deleted_BR_notices =
#if   L==1	// ca
	"Avisos<br />eliminados";	// Necessita traduccio
#elif L==2	// de
	"Gel&ouml;scht<br />Ank&uuml;ndigungen";
#elif L==3	// en
	"Deleted<br />notices";
#elif L==4	// es
	"Avisos<br />eliminados";
#elif L==5	// fr
	"Avis<br />supprim&eacute;s";
#elif L==6	// gn
	"Avisos<br />eliminados";	// Okoteve traduccin
#elif L==7	// it
	"Avvisi<br />rimossi";
#elif L==8	// pl
	"Usuniete<br />notices";
#elif L==9	// pt
	"Avisos<br />suprimidos";
#endif

const char *Txt_NOTICE_Obsolete_SINGULAR =
#if   L==1	// ca
	"Obsoleto";	// Necessita traduccio
#elif L==2	// de
	"Veraltet";
#elif L==3	// en
	"Obsolete";
#elif L==4	// es
	"Obsoleto";
#elif L==5	// fr
	"Obsol&egrave;te";
#elif L==6	// gn
	"Obsoleto";	// Okoteve traduccin
#elif L==7	// it
	"Obsoleto";
#elif L==8	// pl
	"Przestarzaly";
#elif L==9	// pt
	"Obsoleto";
#endif

const char *Txt_NOTICE_Obsolete_BR_notices =
#if   L==1	// ca
	"Avisos<br />obsoletos";	// Necessita traduccio
#elif L==2	// de
	"Veraltet<br />Ank&uuml;ndigungen";
#elif L==3	// en
	"Obsolete<br />notices";
#elif L==4	// es
	"Avisos<br />obsoletos";
#elif L==5	// fr
	"Avis<br />obsol&egrave;tes";
#elif L==6	// gn
	"Avisos<br />obsoletos";	// Okoteve traduccin
#elif L==7	// it
	"Avvisi<br />obsoleti";
#elif L==8	// pl
	"Przestarzaly<br />Ogloszenia";
#elif L==9	// pt
	"Avisos<br />obsoletos";
#endif

const char *Txt_NOTICE_Obsolete_Mark_as_active =
#if   L==1	// ca
	"Obsolet; marcar com actiu";
#elif L==2	// de
	"Veraltet; marke als aktive";
#elif L==3	// en
	"Obsolete; mark as active";
#elif L==4	// es
	"Obsoleto; marcar como activo";
#elif L==5	// fr
	"Obsol&egrave;te; marquer comme active";
#elif L==6	// gn
	"Obsoleto; marcar como activo";	// Okoteve traduccin
#elif L==7	// it
	"Obsoleto; segnare come attivo";
#elif L==8	// pl
	"Przestarzaly; znak towarowy jako aktywnych";
#elif L==9	// pt
	"Obsoleto; marcar como ativo";
#endif

const char *Txt_Notice =
#if   L==1	// ca
	"Av&iacute;s";
#elif L==2	// de
	"Ank&uuml;ndigung";
#elif L==3	// en
	"Notice";
#elif L==4	// es
	"Aviso";
#elif L==5	// fr
	"Avis";
#elif L==6	// gn
	"Marandu";
#elif L==7	// it
	"Avviso";
#elif L==8	// pl
	"Informacja";
#elif L==9	// pt
	"Aviso";
#endif

const char *Txt_Notice_created =
#if   L==1	// ca
	"Av&iacute;s creat.";
#elif L==2	// de
	"Ank&uuml;ndigung erstellt.";
#elif L==3	// en
	"Notice created.";
#elif L==4	// es
	"Aviso creado.";
#elif L==5	// fr
	"Avis cr&eacute;&eacute;.";
#elif L==6	// gn
	"Aviso creado.";	// Okoteve traduccin
#elif L==7	// it
	"Avviso creato.";
#elif L==8	// pl
	"Zawiadomienie stworzony.";
#elif L==9	// pt
	"Aviso criado.";
#endif

const char *Txt_Notice_removed =
#if   L==1	// ca
	"Av&iacute;s eliminat.";
#elif L==2	// de
	"Ank&uuml;ndigung entfernt.";
#elif L==3	// en
	"Notice removed.";
#elif L==4	// es
	"Aviso eliminado";
#elif L==5	// fr
	"Avis supprim&eacute;.";
#elif L==6	// gn
	"Aviso eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Avviso rimosso.";
#elif L==8	// pl
	"Informacja usuniete.";
#elif L==9	// pt
	"Aviso removido.";
#endif

const char *Txt_Notices =
#if   L==1	// ca
	 "Avisos";
#elif L==2	// de
	 "Ank&uuml;ndigungen";
#elif L==3	// en
	 "Notices";
#elif L==4	// es
	 "Avisos";
#elif L==5	// fr
	 "Avis";
#elif L==6	// gn
	 "Marandu";
#elif L==7	// it
	 "Avvisi";
#elif L==8	// pl
	 "Ogloszenia";
#elif L==9	// pt
	 "Avisos";
#endif

const char *Txt_NOTIF_new_SINGULAR =
#if   L==1	// ca
	"nueva";	// Necessita traduccio
#elif L==2	// de
	"neue";
#elif L==3	// en
	"new";
#elif L==4	// es
	"nueva";
#elif L==5	// fr
	"nouvelle";
#elif L==6	// gn
	"nueva";	// Okoteve traduccin
#elif L==7	// it
	"nuova";
#elif L==8	// pl
	"nowy";
#elif L==9	// pt
	"nova";
#endif

const char *Txt_NOTIF_new_PLURAL =
#if   L==1	// ca
	"nuevas";	// Necessita traduccio
#elif L==2	// de
	"neue";
#elif L==3	// en
	"new";
#elif L==4	// es
	"nuevas";
#elif L==5	// fr
	"nouvelles";
#elif L==6	// gn
	"nuevas";	// Okoteve traduccin
#elif L==7	// it
	"nuove";
#elif L==8	// pl
	"nowy";
#elif L==9	// pt
	"novas";
#endif

const char *Txt_NOTIFICATION_STATUS[Ntf_NUM_STATUS_TXT] =
   {
#if   L==1	// ca	// Ntf_STATUS_NO_EMAIL
	 "No"	// Necessita traduccio
#elif L==2	// de
	 "Nein"
#elif L==3	// en
	 "No"
#elif L==4	// es
	 "No"
#elif L==5	// fr
	 "Non"
#elif L==6	// gn
	 "Nah&aacute;niri"
#elif L==7	// it
	 "No"
#elif L==8	// pl
	 "No"
#elif L==9	// pt
	 "N&atilde;o"
#endif
	 ,
#if   L==1	// ca	// Ntf_STATUS_EMAIL_PENDING
	 "Pendiente"	// Necessita traduccio
#elif L==2	// de
	 "Offen"
#elif L==3	// en
	 "Pending"
#elif L==4	// es
	 "Pendiente"
#elif L==5	// fr
	 "En attente"
#elif L==6	// gn
	 "Pendiente"	// Okoteve traduccin
#elif L==7	// it
	 "In attesa"
#elif L==8	// pl
	 "Do czasu"
#elif L==9	// pt
	 "Pendente"
#endif
	 ,
#if   L==1	// ca	// Ntf_STATUS_EMAIL_CANCELLED
	 "Cancelado"	// Necessita traduccio
#elif L==2	// de
	 "Abgebrochen"
#elif L==3	// en
	 "Cancelled"
#elif L==4	// es
	 "Cancelado"
#elif L==5	// fr
	 "Annul&eacute;"
#elif L==6	// gn
	 "Cancelado"	// Okoteve traduccin
#elif L==7	// it
	 "Rimosso"
#elif L==8	// pl
	 "Odwolany"
#elif L==9	// pt
	 "Cancelado"
#endif
	 ,
#if   L==1	// ca	// Ntf_STATUS_EMAIL_SENT
	 "Enviado"	// Necessita traduccio
#elif L==2	// de
	 "Gesendet"
#elif L==3	// en
	 "Sent"
#elif L==4	// es
	 "Enviado"
#elif L==5	// fr
	 "Envoy&eacute;e"
#elif L==6	// gn
	 "Enviado"	// Okoteve traduccin
#elif L==7	// it
	 "Inviata"
#elif L==8	// pl
	 "Wyslane"
#elif L==9	// pt
	 "Enviada"
#endif
	};

const char *Txt_notification =
#if   L==1	// ca
	"notificaci&oacute;";
#elif L==2	// de
	"Anmeldung";
#elif L==3	// en
	"notification";
#elif L==4	// es
	"notificaci&oacute;n";
#elif L==5	// fr
	"notification";
#elif L==6	// gn
	"momarandu";
#elif L==7	// it
	"notifica";
#elif L==8	// pl
	"powiadomienie";
#elif L==9	// pt
	"notifica&ccedil;&atilde;o";
#endif

const char *Txt_Notifications =
#if   L==1	// ca
	"Notificacions";
#elif L==2	// de
	"Anmeldungen";
#elif L==3	// en
	"Notifications";
#elif L==4	// es
	"Notificaciones";
#elif L==5	// fr
	"Notifications";
#elif L==6	// gn
	"Momarandu";
#elif L==7	// it
	"Notifiche";
#elif L==8	// pl
	"Powiadomienia";
#elif L==9	// pt
	"Notifica&ccedil;&otilde;es";
#endif

const char *Txt_notifications =
#if   L==1	// ca
	"notificacions";
#elif L==2	// de
	"Anmeldungen";
#elif L==3	// en
	"notifications";
#elif L==4	// es
	"notificaciones";
#elif L==5	// fr
	"notifications";
#elif L==6	// gn
	"momarandu";
#elif L==7	// it
	"notifiche";
#elif L==8	// pl
	"powiadomienia";
#elif L==9	// pt
	"notifica&ccedil;&otilde;es";
#endif

const char *Txt_NOTIFY_EVENTS_PLURAL[Ntf_NUM_NOTIFY_EVENTS] =
   {
#if   L==1	// ca	// Ntf_EVENT_UNKNOWN
	 "Esdeveniments desconeguts"
#elif L==2	// de
	 "Unbekannte Ereignisse"
#elif L==3	// en
	 "Unknown events"
#elif L==4	// es
	 "Eventos desconocidos"
#elif L==5	// fr
	 "&Eacute;v&eacute;nements inconnus"
#elif L==6	// gn
	 "Eventos desconocidos"	// Okoteve traduccin
#elif L==7	// it
	 "Eventi sconosciuto"
#elif L==8	// pl
	 "Brak wydarzen"
#elif L==9	// pt
	 "Eventos desconhecidos"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_DOCUMENT_FILE
	 "Nous documents"
#elif L==2	// de
	 "Neue Dokumentdateien"
#elif L==3	// en
	 "New document files"
#elif L==4	// es
	 "Nuevos documentos"
#elif L==5	// fr
	 "Nouveaux fichiers de documents"
#elif L==6	// gn
	 "Nuevos documentos"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi documenti"
#elif L==8	// pl
	 "Nowy plik dokumentu"
#elif L==9	// pt
	 "Novos arquivos de documento"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_TEACHERS_FILE
	 "Nous arxiu de professors"
#elif L==2	// de
	 "New Lehrer Dateien"
#elif L==3	// en
	 "New teachers' files"
#elif L==4	// es
	 "Nuevos archivos de profesores"
#elif L==5	// fr
	 "Nouveaux fichiers d'enseignants"
#elif L==6	// gn
	 "Nuevos archivos de profesores"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi file di professori"
#elif L==8	// pl
	 "Nowy Nauczyciela plik"
#elif L==9	// pt
	 "Novos arquivos dos professores"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_SHARED_FILE
	 "Nous arxiu compartit"
#elif L==2	// de
	 "New freigegebene Dateien"
#elif L==3	// en
	 "New shared files"
#elif L==4	// es
	 "Nuevos archivos compartidos"
#elif L==5	// fr
	 "Nouveaux fichiers partag&eacute;s"
#elif L==6	// gn
	 "Nuevos archivos compartidos"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi file condiviso"
#elif L==8	// pl
	 "Nowy Udost&eogon;pniony plik"
#elif L==9	// pt
	 "Novos arquivos compartilhado"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_ASSIGNMENT
	 "Noves activitats"
#elif L==2	// de
	 "Neue Aufgaben"
#elif L==3	// en
	 "New assignments"
#elif L==4	// es
	 "Nuevas actividades"
#elif L==5	// fr
	 "Nouvelles activit&eacute;s"
#elif L==6	// gn
	 "Nuevas actividades"	// Okoteve traduccin
#elif L==7	// it
	 "Nuove attivit&agrave;"
#elif L==8	// pl
	 "Nowe zadania"
#elif L==9	// pt
	 "Novas atividades"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_EXAM_ANNOUNCEMENT
	 "Noves convocat&ograve;ries d'examen"
#elif L==2	// de
	 "Neue Aufrufe f&uuml;r Pr&uuml;fung"
#elif L==3	// en
	 "New announcements of exam"
#elif L==4	// es
	 "Nuevas convocatorias de examen"
#elif L==5	// fr
	 "Nouvelles convocations &agrave; un examen"
#elif L==6	// gn
	 "Nuevas convocatorias de examen"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi appelli d'esame"
#elif L==8	// pl
	 "Nowe anonse egzamin"
#elif L==9	// pt
	 "Novas chamadas para exame"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_MARKS_FILE
	 "Nous fitxers de qualificacions"
#elif L==2	// de
	 "Neue Dateien mit Bewertung"
#elif L==3	// en
	 "New files with marks"
#elif L==4	// es
	 "Nuevos archivos de calificaciones"
#elif L==5	// fr
	 "Nouveaux fichiers avec des notes"
#elif L==6	// gn
	 "Nuevos archivos de calificaciones"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi file con risultati"
#elif L==8	// pl
	 "Nowe pliki ze znakami"
#elif L==9	// pt
	 "Novos arquivos de notas"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_ENROLMENT_STD
	 "Novas inscripci&oacute;s com a estudiant"
#elif L==2	// de
	 "New Einschreibungen als Student"
#elif L==3	// en
	 "New enrolments as a student"
#elif L==4	// es
	 "Nuevas inscripciones como estudiante"
#elif L==5	// fr
	 "Nouvelles inscriptions en tant qu'&eacute;tudiant"
#elif L==6	// gn
	 "Nuevas inscripciones como estudiante"	// Okoteve traduccin
#elif L==7	// it
	 "Nuova iscrizione come studente"
#elif L==8	// pl
	 "Nowe zapisy jako student"
#elif L==9	// pt
	 "Novas inscri&ccedil;&otilde;es como estudante"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_ENROLMENT_TCH
	 "Novas inscripci&oacute;s com a professor"
#elif L==2	// de
	 "New Einschreibungen als Lehrkraft"
#elif L==3	// en
	 "New enrolments as a teacher"
#elif L==4	// es
	 "Nuevas inscripciones como profesor/a"
#elif L==5	// fr
	 "Nouvelles inscriptions en tant qu'enseignant"
#elif L==6	// gn
	 "Nuevas inscripciones como profesor/a"	// Okoteve traduccin
#elif L==7	// it
	 "Nuova iscrizione come professore"
#elif L==8	// pl
	 "Nowe zapisy jako nauczyciel"
#elif L==9	// pt
	 "Novas inscri&ccedil;&otilde;es como professor"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_ENROLMENT_REQUEST
	 "Novas petici&oacute;s d'inscripci&oacute;"
#elif L==2	// de
	 "New Registrierungsanforderungen"
#elif L==3	// en
	 "New enrolment requests"
#elif L==4	// es
	 "Nuevas peticiones de inscripci&oacute;n"
#elif L==5	// fr
	 "Nouvelles demandes d'inscription"
#elif L==6	// gn
	 "Nuevas peticiones de inscripci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Nuova richiesta di iscrizione"
#elif L==8	// pl
	 "Nowe pro&sacute;by o rejestracji"
#elif L==9	// pt
	 "Novos pedidos de inscri&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_TIMELINE_COMMENT
	 "Nous comentaris als meus missatges socials"
#elif L==2	// de
	 "Neue Kommentare zu meinem sozialen Beitr&auml;ge"
#elif L==3	// en
	 "New comments to my social posts"
#elif L==4	// es
	 "Nuevos comentarios a mis mensajes sociales"
#elif L==5	// fr
	 "Nouveaux commentaires &agrave; mes messages sociaux"
#elif L==6	// gn
	 "Nuevos comentarios a mis mensajes sociales"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi commenti ai miei post sociali"
#elif L==8	// pl
	 "Nowe komentarze do moich post spo&lstrok;ecznej"
#elif L==9	// pt
	 "Novos coment&aacute;rio &agrave;s minhos posts sociais"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_TIMELINE_FAV
	 "Nous favorits als meus missatges socials"
#elif L==2	// de
	 "Neue Favorit zu meinem sozialen Beitr&auml;ge"
#elif L==3	// en
	 "New favourite to my social posts"
#elif L==4	// es
	 "Nuevos favoritos a mis mensajes sociales"
#elif L==5	// fr
	 "Nouveaux favori &agrave; mes messages sociaux"
#elif L==6	// gn
	 "Nuevos favoritos a mis mensajes sociales"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi preferito ai miei post sociali"
#elif L==8	// pl
	 "Nowe ulubiony do moich post spo&lstrok;ecznej"
#elif L==9	// pt
	 "Novos favoritos &agrave;s minhos posts sociais"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_TIMELINE_SHARE
	 "Noves comparticions dels meus missatges socials"
#elif L==2	// de
	 "Neue gemeinsame Nutzung zu meinem sozialen Beitr&auml;ge"
#elif L==3	// en
	 "New sharings of my social posts"
#elif L==4	// es
	 "Nuevas comparticiones de mis mensajes sociales"
#elif L==5	// fr
	 "Nouveaux partages de mes messages sociaux"
#elif L==6	// gn
	 "Nuevas comparticiones de mis mensajes sociales"	// Okoteve traduccin
#elif L==7	// it
	 "Nuove condivisioni dei miei post sociali"
#elif L==8	// pl
	 "Nowe podzial moich post spo&lstrok;ecznej"
#elif L==9	// pt
	 "Novas partilhas de minhos posts sociais"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_TIMELINE_MENTION
	 "Noves mencions"
#elif L==2	// de
	 "Neue Erw&auml;hnungen"
#elif L==3	// en
	 "New mentions"
#elif L==4	// es
	 "Nuevas menciones"
#elif L==5	// fr
	 "Nouvelles mentions"
#elif L==6	// gn
	 "Nuevas menciones"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi menzioni"
#elif L==8	// pl
	 "Nowe wzmianki"
#elif L==9	// pt
	 "Novas men&ccedil;&otilde;es"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_FOLLOWER
	 "Nous seguidors"
#elif L==2	// de
	 "Neue Anh&auml;nger"
#elif L==3	// en
	 "New followers"
#elif L==4	// es
	 "Nuevos seguidores"
#elif L==5	// fr
	 "Nouveaux suiveurs"
#elif L==6	// gn
	 "Nuevos seguidores"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi followers"
#elif L==8	// pl
	 "Nowe obserwuj&aogon;"
#elif L==9	// pt
	 "Novos seguidores"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_FORUM_POST_COURSE
	 "Novos comentaris en f&ograve;rums"
#elif L==2	// de
	 "Neue Beitr&auml;ge in den Foren"
#elif L==3	// en
	 "New posts in course forums"
#elif L==4	// es
	 "Nuevos comentarios en foros de asignaturas"
#elif L==5	// fr
	 "Nouveaux posts dans les forums"
#elif L==6	// gn
	 "Nuevos comentarios en foros de asignaturas"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi post nei forum"
#elif L==8	// pl
	 "Nowe post&oacute;w na forum"
#elif L==9	// pt
	 "Novos posts em f&oacute;runs"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_FORUM_REPLY
	 "Noves respostes als meus comentaris en f&ograve;rums"
#elif L==2	// de
	 "Neue Antworten auf meine Beitr&auml;ge in den Foren"
#elif L==3	// en
	 "New replies to my posts in forums"
#elif L==4	// es
	 "Nuevas respuestas a mis comentarios en foros"
#elif L==5	// fr
	 "Nouvelles r&eacute;ponses &agrave; mes posts dans les forums"
#elif L==6	// gn
	 "Nuevas respuestas a mis comentarios en foros"	// Okoteve traduccin
#elif L==7	// it
	 "Nuove risposte ai miei post nei forum"
#elif L==8	// pl
	 "Nowe podejscie do moich post&oacute;w na forum"
#elif L==9	// pt
	 "Novas respostas aos meus posts em f&oacute;runs"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_NOTICE
	 "Nous avisos"
#elif L==2	// de
	 "Neue Ank&uuml;ndigungen"
#elif L==3	// en
	 "New notices"
#elif L==4	// es
	 "Nuevos avisos"
#elif L==5	// fr
	 "Nouveaux avis"
#elif L==6	// gn
	 "Nuevos avisos"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi avvisi"
#elif L==8	// pl
	 "Nowe ogloszenia"
#elif L==9	// pt
	 "Novos avisos"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_MESSAGE
	 "Nous missatges"
#elif L==2	// de
	 "Neue Nachrichten"
#elif L==3	// en
	 "New messages"
#elif L==4	// es
	 "Nuevos mensajes"
#elif L==5	// fr
	 "Nouveaux messages"
#elif L==6	// gn
	 "Nuevos mensajes"	// Okoteve traduccin
#elif L==7	// it
	 "Nuovi messaggi"
#elif L==8	// pl
	 "Nowa wiadomosc"
#elif L==9	// pt
	 "Novas mensagens"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_SURVEY		// TODO: Move to assessment tab (also necessary in database) !!!!!!!!!
	 "Noves enquestes"
#elif L==2	// de
	 "Neue Umfragen"
#elif L==3	// en
	 "New surveys"
#elif L==4	// es
	 "Nuevas encuestas"
#elif L==5	// fr
	 "Nouveaux sondages"
#elif L==6	// gn
	 "Nuevas encuestas"	// Okoteve traduccin
#elif L==7	// it
	 "Nuove sondaggi"
#elif L==8	// pl
	 "Nowe badania"
#elif L==9	// pt
	 "Novos inqu&eacute;ritos"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_ENROLMENT_NET	// TODO: Move to users tab (also necessary in database) !!!!!!!!!
	 "Novas inscripci&oacute;s com a professor/a no editor/a"
#elif L==2	// de
	 "New Einschreibungen als nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	 "New enrolments as a non-editing teacher"
#elif L==4	// es
	 "Nuevas inscripciones como profesor/a/a no editor/a"
#elif L==5	// fr
	 "Nouvelles inscriptions en tant qu'enseignant/e non-&eacute;diteur/trice"
#elif L==6	// gn
	 "Nuevas inscripciones como profesor/a no editor/a"	// Okoteve traduccin
#elif L==7	// it
	 "Nuova iscrizione come professore/ssa non-editing"
#elif L==8	// pl
	 "Nowe zapisy jako nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	 "Novas inscri&ccedil;&otilde;es como professor/a n&atilde;o editor/a"
#endif
	};

const char *Txt_NOTIFY_EVENTS_SINGULAR[Ntf_NUM_NOTIFY_EVENTS] =
   {
#if   L==1	// ca	// Ntf_EVENT_UNKNOWN
	 "Esdeveniment desconegut"
#elif L==2	// de
	 "Unbekannt Ereignis"
#elif L==3	// en
	 "Unknown event"
#elif L==4	// es
	 "Evento desconocido"
#elif L==5	// fr
	 "&Eacute;v&eacute;nement inconnu"
#elif L==6	// gn
	 "Evento desconocido"	// Okoteve traduccin
#elif L==7	// it
	 "Evento sconosciuto"
#elif L==8	// pl
	 "Nieznane zdarzenia"
#elif L==9	// pt
	 "Evento desconhecido"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_DOCUMENT_FILE
	 "Document"
#elif L==2	// de
	 "Dokumentdatei"
#elif L==3	// en
	 "Document file"
#elif L==4	// es
	 "Documento"
#elif L==5	// fr
	 "Fichier de document"
#elif L==6	// gn
	 "Documento"	// Okoteve traduccin
#elif L==7	// it
	 "Documento"
#elif L==8	// pl
	 "Plik dokumentu"
#elif L==9	// pt
	 "Arquivo de documento"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_TEACHERS_FILE
	 "Arxiu de professors"
#elif L==2	// de
	 "Lehrer Datei"
#elif L==3	// en
	 "Teachers' file"
#elif L==4	// es
	 "Archivo de profesores"
#elif L==5	// fr
	 "Fichier d'enseignants"
#elif L==6	// gn
	 "Archivo de profesores"	// Okoteve traduccin
#elif L==7	// it
	 "File di professori"
#elif L==8	// pl
	 "Nauczyciela plik"
#elif L==9	// pt
	 "Arquivo dos professores"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_SHARED_FILE
	 "Arxiu compartit"
#elif L==2	// de
	 "Freigegebene Datei"
#elif L==3	// en
	 "Shared file"
#elif L==4	// es
	 "Archivo compartido"
#elif L==5	// fr
	 "Fichier partag&eacute;"
#elif L==6	// gn
	 "Archivo compartido"	// Okoteve traduccin
#elif L==7	// it
	 "File condiviso"
#elif L==8	// pl
	 "Udost&eogon;pniony plik"
#elif L==9	// pt
	 "Arquivo compartilhado"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_ASSIGNMENT
	 "Activitat"
#elif L==2	// de
	 "Aufgabe"
#elif L==3	// en
	 "Assignment"
#elif L==4	// es
	 "Actividad"
#elif L==5	// fr
	 "Activit&eacute;"
#elif L==6	// gn
	 "Tar&eacute;a"
#elif L==7	// it
	 "Attivit&agrave;"
#elif L==8	// pl
	 "Przyporzadkowanie"
#elif L==9	// pt
	 "Atividade"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_EXAM_ANNOUNCEMENT
	 "Convocat&ograve;ria d'examen"
#elif L==2	// de
	 "Aufrufe fr Pr&uuml;fung"
#elif L==3	// en
	 "Announcement of exam"
#elif L==4	// es
	 "Convocatoria de examen"
#elif L==5	// fr
	 "Convocation &agrave; un examen"
#elif L==6	// gn
	 "Convocatoria de examen"	// Okoteve traduccin
#elif L==7	// it
	 "Appello d'esame"
#elif L==8	// pl
	 "Ogloszenie egzamin"
#elif L==9	// pt
	 "Chamada para exame"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_MARKS_FILE
	 "Fitxer de qualificacions"
#elif L==2	// de
	 "Datei mit Bewertung"
#elif L==3	// en
	 "File with marks"
#elif L==4	// es
	 "Archivo de calificaciones"
#elif L==5	// fr
	 "Fichier avec des notes"
#elif L==6	// gn
	 "Archivo de calificaciones"	// Okoteve traduccin
#elif L==7	// it
	 "File con risultati"
#elif L==8	// pl
	 "Plik ze znakami"
#elif L==9	// pt
	 "Arquivo de notas"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_ENROLMENT_STD
	 "Inscripci&oacute; com a estudiant"
#elif L==2	// de
	 "Einschreibung als Student"
#elif L==3	// en
	 "Enrolment as a student"
#elif L==4	// es
	 "Inscripci&oacute;n como estudiante"
#elif L==5	// fr
	 "Inscription en tant qu'&eacute;tudiant"
#elif L==6	// gn
	 "Inscripci&oacute;n como estudiante"	// Okoteve traduccin
#elif L==7	// it
	 "Iscrizione come studente"
#elif L==8	// pl
	 "Zapisy jako student"
#elif L==9	// pt
	 "Inscri&ccedil;&atilde;o como estudante"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_ENROLMENT_TCH
	 "Inscripci&oacute; com a professor/a"
#elif L==2	// de
	 "Einschreibung als Lehrkraft"
#elif L==3	// en
	 "Enrolment as a teacher"
#elif L==4	// es
	 "Inscripci&oacute;n como profesor/a"
#elif L==5	// fr
	 "Inscription en tant qu'enseignant"
#elif L==6	// gn
	 "Inscripci&oacute;n como profesor/a"	// Okoteve traduccin
#elif L==7	// it
	 "Iscrizione come professore/ssa"
#elif L==8	// pl
	 "Zapisy jako nauczyciel"
#elif L==9	// pt
	 "Inscri&ccedil;&atilde;o como professor/a"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_ENROLMENT_REQUEST
	 "Petici&oacute; d'inscripci&oacute;"
#elif L==2	// de
	 "Registrierungsanforderung"
#elif L==3	// en
	 "Enrolment request"
#elif L==4	// es
	 "Petici&oacute;n de inscripci&oacute;n"
#elif L==5	// fr
	 "Demande d'inscription"
#elif L==6	// gn
	 "Petici&oacute;n de inscripci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Richiesta di iscrizione"
#elif L==8	// pl
	 "&Zdot;&aogon;danie rejestracji"
#elif L==9	// pt
	 "Pedido de inscri&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_TIMELINE_COMMENT
	 "Comentari a un missatge social"
#elif L==2	// de
	 "Kommentar zu soziale Beitrag"
#elif L==3	// en
	 "Comment to social post"
#elif L==4	// es
	 "Comentario a un mensaje social"
#elif L==5	// fr
	 "Commentaire &agrave; message social"
#elif L==6	// gn
	 "Comentario a un mensaje social"	// Okoteve traduccin
#elif L==7	// it
	 "Commento a un post sociale"
#elif L==8	// pl
	 "Komentarz do post spo&lstrok;eczna"
#elif L==9	// pt
	 "Coment&aacute;rio a um post sociais"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_TIMELINE_FAV
	 "Missatge social marcat com favorit"
#elif L==2	// de
	 "Social Beitrag als Favorit markiert"
#elif L==3	// en
	 "Social post marked as favourite"
#elif L==4	// es
	 "Mensaje social marcado como favorito"
#elif L==5	// fr
	 "Message social marqu&eacute;e comme favori"
#elif L==6	// gn
	 "Mensaje social marcado como favorito"	// Okoteve traduccin
#elif L==7	// it
	 "Post sociale segnato come preferito"
#elif L==8	// pl
	 "Post spo&lstrok;ecznej oznaczone jako ulubiony"
#elif L==9	// pt
	 "Post sociais marcado como favorito"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_TIMELINE_SHARE
	 "Missatge social compartit"
#elif L==2	// de
	 "Social Beitrag gemeinsamer"
#elif L==3	// en
	 "Social post shared"
#elif L==4	// es
	 "Mensaje social compartido"
#elif L==5	// fr
	 "Message social partag&eacute;"
#elif L==6	// gn
	 "Mensaje social compartido"	// Okoteve traduccin
#elif L==7	// it
	 "Post sociale condiviso"
#elif L==8	// pl
	 "Post spo&lstrok;ecznej wsp&oacute;lne"
#elif L==9	// pt
	 "Post sociais compartido"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_TIMELINE_MENTION
	 "Menci&oacute;"
#elif L==2	// de
	 "Erw&auml;hnung"
#elif L==3	// en
	 "Mention"
#elif L==4	// es
	 "Menci&oacute;n"
#elif L==5	// fr
	 "Mention"
#elif L==6	// gn
	 "Menci&oacute;n"			// Okoteve traduccin
#elif L==7	// it
	 "Menzione"
#elif L==8	// pl
	 "Wzmianka"
#elif L==9	// pt
	 "Men&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_FOLLOWER
	"Seguidor"
#elif L==2	// de
	"Anh&auml;nger"
#elif L==3	// en
	"Follower"
#elif L==4	// es
	"Seguidor"
#elif L==5	// fr
	"Suiveur"
#elif L==6	// gn
	"Seguidor"	// Okoteve traduccin
#elif L==7	// it
	"Follower"
#elif L==8	// pl
	"Obserwuj&aogon;"
#elif L==9	// pt
	"Seguidor"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_FORUM_POST_COURSE
	 "Comentari en un f&ograve;rum"
#elif L==2	// de
	 "Beitr&auml;ge in einem Forum"
#elif L==3	// en
	 "Post in a forum"
#elif L==4	// es
	 "Comentario en un foro"
#elif L==5	// fr
	 "Post dans un forum"
#elif L==6	// gn
	 "Comentario en un foro"	// Okoteve traduccin
#elif L==7	// it
	 "Post in un forum"
#elif L==8	// pl
	 "Post na forum"
#elif L==9	// pt
	 "Post em um f&oacute;rum"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_FORUM_REPLY
	 "Resposta en un f&ograve;rum"
#elif L==2	// de
	 "Antwort in einem Forum"
#elif L==3	// en
	 "Response in a forum"
#elif L==4	// es
	 "Respuesta en un foro"
#elif L==5	// fr
	 "R&eacute;ponse dans un forum"
#elif L==6	// gn
	 "Respuesta en un foro"	// Okoteve traduccin
#elif L==7	// it
	 "Risposta in un forum"
#elif L==8	// pl
	 "Odpowiedz na forum"
#elif L==9	// pt
	 "Resposta em um f&oacute;rum"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_NOTICE
	 "Av&iacute;s"
#elif L==2	// de
	 "Ank&uuml;ndigungen"
#elif L==3	// en
	 "Notice"
#elif L==4	// es
	 "Aviso"
#elif L==5	// fr
	 "Avis"
#elif L==6	// gn
	 "Marandu"
#elif L==7	// it
	 "Avviso"
#elif L==8	// pl
	 "Informacja"
#elif L==9	// pt
	 "An&uacute;ncio"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_MESSAGE
	 "Missatge"
#elif L==2	// de
	 "Nachricht"
#elif L==3	// en
	 "Message"
#elif L==4	// es
	 "Mensaje"
#elif L==5	// fr
	 "Message"
#elif L==6	// gn
	 "Marandu"
#elif L==7	// it
	 "Messaggio"
#elif L==8	// pl
	 "Wiadomo&sacute;&cacute;"
#elif L==9	// pt
	 "Mensagem"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_SURVEY		// TODO: Move to assessment tab (also necessary in database) !!!!!!!!!
	 "Enquesta"
#elif L==2	// de
	 "Umfrage"
#elif L==3	// en
	 "Survey"
#elif L==4	// es
	 "Encuesta"
#elif L==5	// fr
	 "Sondage"
#elif L==6	// gn
	 "Encuesta"	// Okoteve traduccin
#elif L==7	// it
	 "Sondaggio"
#elif L==8	// pl
	 "Badania"
#elif L==9	// pt
	 "Inqu&eacute;rito"
#endif
	 ,
#if   L==1	// ca	// Ntf_EVENT_ENROLMENT_NET	// TODO: Move to users tab (also necessary in database) !!!!!!!!!
	 "Inscripci&oacute; com a professor/a no editor/a"
#elif L==2	// de
	 "Einschreibung als nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	 "Enrolment as a non-editing teacher"
#elif L==4	// es
	 "Inscripci&oacute;n como profesor/a no editor/a"
#elif L==5	// fr
	 "Inscription en tant qu'enseignant non-&eacute;diteur/trice"
#elif L==6	// gn
	 "Inscripci&oacute;n como profesor/a no editor/a"	// Okoteve traduccin
#elif L==7	// it
	 "Iscrizione come professore/ssa non-editing"
#elif L==8	// pl
	 "Zapisy jako nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	 "Inscri&ccedil;&atilde;o como professor/a n&atilde;o editor/a"
#endif
	};

const char *Txt_Notify_me_BR_by_email =
#if   L==1	// ca
	"Avisar-me<br />per correu";
#elif L==2	// de
	"Benachrichtigt mich<br />per E-Mail";
#elif L==3	// en
	"Notify me<br />by email";
#elif L==4	// es
	"Avisarme<br />por correo";
#elif L==5	// fr
	"Avertissez-moi<br />par email";
#elif L==6	// gn
	"Avisarme<br />por correo";		// Okoteve traduccin
#elif L==7	// it
	"Notificatami<br />via email";
#elif L==8	// pl
	"Informuj<br />mnie emailem";
#elif L==9	// pt
	"Notifique-me<br />por email";
#endif

const char *Txt_Now_each_student_can_belong_to_multiple_groups_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";		//
#elif L==3	// en
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";
#elif L==4	// es
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";
#elif L==5	// fr
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";		//
#elif L==6	// gn
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Ora ogni studente pu&ograve; appartenere a gruppi multipli di tipo <strong>%s</strong>.";
#elif L==8	// pl
	"Teraz kazdy student moze nalezec do wielu grup typu <strong>%s</strong>.";
#elif L==9	// pt
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";		//
#endif

const char *Txt_Now_each_student_can_only_belong_to_a_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ahora cada estudiante solo puede pertenecer a un grupo de tipo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Now each student can only belong to a group of type <strong>%s</strong>.";
#elif L==3	// en
	"Now each student can only belong to a group of type <strong>%s</strong>.";
#elif L==4	// es
	"Ahora cada estudiante solo puede pertenecer a un grupo de tipo <strong>%s</strong>.";
#elif L==5	// fr
	"Now each student can only belong to a group of type <strong>%s</strong>.";		//
#elif L==6	// gn
	"Ahora cada estudiante solo puede pertenecer a un grupo de tipo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Ora ogni studente pu&ograve; solo appartenere a un gruppo di tipo <strong>%s</strong>.";
#elif L==8	// pl
	"Teraz kazdy student moze nalezec tylko do grupy typu <strong>%s</strong>.";
#elif L==9	// pt
	"Now each student can only belong to a group of type <strong>%s</strong>.";		//
#endif

const char *Txt_Number_of_BR_assignments =
#if   L==1	// ca
	"Nombre<br />d'activitats";
#elif L==2	// de
	"Anzahl der<br />Aufgaben";
#elif L==3	// en
	"Number of<br />assignments";
#elif L==4	// es
	"N&ordm; de<br />actividades";
#elif L==5	// fr
	"Nombre<br />d'activit&eacute;s";
#elif L==6	// gn
	"N&ordm; de<br />actividades";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />attivit&agrave;";
#elif L==8	// pl
	"Number of<br />assignments";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&ordm; de<br />atividades";
#endif

const char *Txt_Number_of_BR_courses_with_BR_assignments =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb activitats";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Aufgaben";
#elif L==3	// en
	"Number of<br />courses with<br />assignments";
#elif L==4	// es
	"N&ordm; de<br />asignaturas<br />con actividades";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec activit&eacute;s";
#elif L==6	// gn
	"N&ordm; de<br />asignaturas<br />con actividades";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />attivit&agrave;";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />zadania";
#elif L==9	// pt
	"N&ordm; de<br />disciplinas<br />com atividades";
#endif

const char *Txt_Number_of_BR_courses_with_BR_exportable_BR_test_BR_questions =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb preguntes<br />de test<br />exportables";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit exportierbar<br />Testfragen";
#elif L==3	// en
	"Number of<br />courses with<br />exportable<br />test<br />questions";
#elif L==4	// es
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test<br />exportables";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec questions<br />de test<br />exportables";
#elif L==6	// gn
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test<br />exportables";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />domande<br />de test<br />esportabili";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w<br />z eksportowane<br />pyta&nacute;<br />testowych";
#elif L==9	// pt
	"N&ordm; de<br />disciplinas<br />com<br />quest&otilde;es<br />de test<br />export&aacute;veis";
#endif

const char *Txt_Number_of_BR_courses_with_BR_games =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb jocs";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Spiele";
#elif L==3	// en
	"Number of<br />courses with<br />games";
#elif L==4	// es
	"N&ordm; de<br />asignaturas<br />con juegos";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec jeux";
#elif L==6	// gn
	"N&ordm; de<br />asignaturas<br />con juegos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />giochi";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />gry";
#elif L==9	// pt
	"N&ordm; de<br />disciplinas<br />com jogos";
#endif

const char *Txt_Number_of_BR_courses_with_BR_projects =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb projectes";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Projekte";
#elif L==3	// en
	"Number of<br />courses with<br />projects";
#elif L==4	// es
	"N&ordm; de<br />asignaturas<br />con proyectos";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec projets";
#elif L==6	// gn
	"N&ordm; de<br />asignaturas<br />con proyectos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />progetti";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />projekty";
#elif L==9	// pt
	"N&ordm; de<br />disciplinas<br />com projetos";
#endif

const char *Txt_Number_of_BR_courses_BR_with_test_BR_questions =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb preguntes<br />de test";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Testfragen";
#elif L==3	// en
	"Number of<br />courses<br />with test<br />questions";
#elif L==4	// es
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec questions<br />de test";
#elif L==6	// gn
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />domande<br />de test";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w<br />z pyta&nacute;<br />testowych";
#elif L==9	// pt
	"N&ordm; de<br />disciplinas<br />com<br />quest&otilde;es<br />de test";
#endif

const char *Txt_Number_of_BR_courses_with_BR_surveys =
#if   L==1	// ca
	"Nombre<br />d'assignatures<br />amb enquestes";
#elif L==2	// de
	"Anzahl der<br />Kursen<br />mit Umfragen";
#elif L==3	// en
	"Number of<br />courses with<br />surveys";
#elif L==4	// es
	"N&ordm; de<br />asignaturas<br />con encuestas";
#elif L==5	// fr
	"Nombre de<br />mati&egrave;res<br />avec sondages";
#elif L==6	// gn
	"N&ordm; de<br />asignaturas<br />con encuestas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />corsi con<br />sondaggi";
#elif L==8	// pl
	"Liczba<br />kurs&oacute;w z<br />badania";
#elif L==9	// pt
	"N&ordm; de<br />disciplinas<br />com inqu&eacute;ritos";
#endif

const char *Txt_Number_of_BR_events =
#if   L==1	// ca
	"N&ordm; d'<br />esdeveniments.";
#elif L==2	// de
	"Anzahl der<br />Ereignisse";
#elif L==3	// en
	"Number of<br />events";
#elif L==4	// es
	"N&ordm; de<br />eventos";
#elif L==5	// fr
	"Nombre<br />d'&eacute;v&eacute;nements";
#elif L==6	// gn
	"N&ordm; de<br />eventos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />eventi";
#elif L==8	// pl
	"Liczba of<br />zdarzenia";
#elif L==9	// pt
	"N&uacute;mero de<br />eventos";
#endif

const char *Txt_Number_of_BR_emails =
#if   L==1	// ca
	"N&ordm; de<br />correos";	// Necessita traduccio
#elif L==2	// de
	"Anzahl der<br />E-Mails";
#elif L==3	// en
	"Number of<br />emails";
#elif L==4	// es
	"N&ordm; de<br />correos";
#elif L==5	// fr
	"Nombre<br />d'emails";
#elif L==6	// gn
	"N&ordm; de<br />correos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />email";
#elif L==8	// pl
	"Liczba<br />email";
#elif L==9	// pt
	"N&uacute;mero de<br />emails";
#endif

const char *Txt_Number_of_BR_games =
#if   L==1	// ca
	"Nombre<br />de jocs.";
#elif L==2	// de
	"Anzahl der<br />Spiele";
#elif L==3	// en
	"Number of<br />games";
#elif L==4	// es
	"N&ordm; de<br />juegos";
#elif L==5	// fr
	"Nombre<br />de jeux";
#elif L==6	// gn
	"N&ordm; de<br />juegos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />giochi";
#elif L==8	// pl
	"Liczba<br />gier.";
#elif L==9	// pt
	"N&uacute;mero de<br />jogos";
#endif

const char *Txt_Number_of_BR_notifications =
#if   L==1	// ca
	"Nombre de<br />notificacions";
#elif L==2	// de
	"Anzahl der<br />Anmeldungen";
#elif L==3	// en
	"Number of<br />notifications";
#elif L==4	// es
	"N&ordm; de<br />notificaciones";
#elif L==5	// fr
	"Nombre de<br />notifications";
#elif L==6	// gn
	"N&ordm; de<br />notificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />notifiche";
#elif L==8	// pl
	"Liczba<br />zgloszen";
#elif L==9	// pt
	"N&uacute;mero de<br />notifica&ccedil;&otilde;es";
#endif

const char *Txt_Number_of_BR_projects =
#if   L==1	// ca
	"Nombre<br />de projectes.";
#elif L==2	// de
	"Anzahl der<br />Projekte";
#elif L==3	// en
	"Number of<br />projects";
#elif L==4	// es
	"N&ordm; de<br />proyectos";
#elif L==5	// fr
	"Nombre<br />de projets";
#elif L==6	// gn
	"N&ordm; de<br />proyectos";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />progetti";
#elif L==8	// pl
	"Liczba<br />projekt&oacute;w.";
#elif L==9	// pt
	"N&uacute;mero de<br />projetos";
#endif

const char *Txt_Number_of_students =
#if   L==1	// ca
	"Nombre d'estudiants";
#elif L==2	// de
	"Anzahl der Studenten";
#elif L==3	// en
	"Number of students";
#elif L==4	// es
	"N&ordm; de estudiantes";
#elif L==5	// fr
	"Nombre d'&eacute;tudiants";
#elif L==6	// gn
	"N&ordm; de estudiantes";	// Okoteve traduccin
#elif L==7	// it
	"Numero di studenti";
#elif L==8	// pl
	"Liczba studenci";
#elif L==9	// pt
	"N&ordm; de estudantes";
#endif

const char *Txt_Number_of_BR_surveys =
#if   L==1	// ca
	"Nombre<br />d'enquestes";
#elif L==2	// de
	"Anzahl der<br />Umfragen";
#elif L==3	// en
	"Number of<br />surveys";
#elif L==4	// es
	"N&ordm; de<br />encuestas";
#elif L==5	// fr
	"Nombre de<br />sondages";
#elif L==6	// gn
	"N&ordm; de<br />encuestas";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />sondaggi";
#elif L==8	// pl
	"Liczba<br />badania";
#elif L==9	// pt
	"N&ordm; de<br />inqu&eacute;ritos";
#endif

const char *Txt_Number_BR_of_test_BR_questions =
#if   L==1	// ca
	"Nombre de<br />preguntes<br />de test";
#elif L==2	// de
	"Anzahl der<br />Testfragen";
#elif L==3	// en
	"Number<br />of test<br />questions";
#elif L==4	// es
	"N&ordm; de<br />preguntas<br />de test";
#elif L==5	// fr
	"Nombre de<br />questions<br />de test";
#elif L==6	// gn
	"N&ordm; de<br />preguntas<br />de test";	// Okoteve traduccin
#elif L==7	// it
	"Numero di<br />domande<br />de test";
#elif L==8	// pl
	"Liczba<br />pyta&nacute;<br />testowych";
#elif L==9	// pt
	"N&ordm; de<br />quest&otilde;es<br />de test";
#endif

const char *Txt_Numeric_BR_code_BR_ISO_3166_1 =
#if   L==1	// ca
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";	// Necessita traduccio
#elif L==2	// de
	"Numerischen<br />Code<br />ISO 3166-1";
#elif L==3	// en
	"Numeric<br />code<br />ISO 3166-1";
#elif L==4	// es
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";
#elif L==5	// fr
	"Code<br />num&eacute;rique<br />ISO 3166-1";
#elif L==6	// gn
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";	// Okoteve traduccin
#elif L==7	// it
	"Codice<br />numerico<br />ISO 3166-1";
#elif L==8	// pl
	"Nnumerycznych<br />kod<br />ISO 3166-1";
#elif L==9	// pt
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";
#endif

const char *Txt_of_PART_OF_A_TOTAL =	// Example: transferred 2 of 3 files
#if   L==1	// ca
	"de";	// Necessita traduccio
#elif L==2	// de
	"von";
#elif L==3	// en
	"of";
#elif L==4	// es
	"de";
#elif L==5	// fr
	"de";
#elif L==6	// gn
	"de";	// Okoteve traduccin
#elif L==7	// it
	"di";
#elif L==8	// pl
	"z";
#elif L==9	// pt
	"de";
#endif

const char *Txt_Office =
#if   L==1	// ca
	"Despatx";
#elif L==2	// de
	"Raum";
#elif L==3	// en
	"Office";
#elif L==4	// es
	"Despacho";
#elif L==5	// fr
	"Bureau";
#elif L==6	// gn
	"Mba'apoha";
#elif L==7	// it
	"Ufficio";
#elif L==8	// pl
	"Biuro";
#elif L==9	// pt
	"Gabinete";
#endif

const char *Txt_One_message_has_been_deleted =
#if   L==1	// ca
	"Se ha eliminado un mensaje.";			// Necessita traduccio
#elif L==2	// de
	"One message has been deleted.";		// Need bersetzung
#elif L==3	// en
	"One message has been deleted.";
#elif L==4	// es
	"Se ha eliminado un mensaje.";
#elif L==5	// fr
	"One message has been deleted.";		// Besoin de traduction
#elif L==6	// gn
	"Se ha eliminado un mensaje.";			// Okoteve traduccin
#elif L==7	// it
	"Un messaggio &egrave; stato rimosso.";
#elif L==8	// pl
	"Jedna wiadomosc zostala usunieta.";
#elif L==9	// pt
	"Uma mensagem foi removida.";
#endif

const char *Txt_One_user_has_been_eliminated =
#if   L==1	// ca
	"Se ha eliminado un usuario.";			// Necessita traduccio
#elif L==2	// de
	"One user has been eliminated.";		// Need bersetzung
#elif L==3	// en
	"One user has been eliminated.";
#elif L==4	// es
	"Se ha eliminado un usuario.";
#elif L==5	// fr
	"One user has been eliminated.";		// Besoin de traduction
#elif L==6	// gn
	"Se ha eliminado un usuario.";			// Okoteve traduccin
#elif L==7	// it
	"Un utente &egrave; stato rimosso.";
#elif L==8	// pl
	"Jeden uzytkownik zostal usuniety.";
#elif L==9	// pt
	"Um usu&aacute;rio foi removido.";
#endif

const char *Txt_One_user_has_been_enroled =
#if   L==1	// ca
	"Se ha inscrito un usuario.";		// Necessita traduccio
#elif L==2	// de
	"One user has been enroled.";		// Need bersetzung
#elif L==3	// en
	"One user has been enroled.";
#elif L==4	// es
	"Se ha inscrito un usuario.";
#elif L==5	// fr
	"One user has been enroled.";		// Besoin de traduction
#elif L==6	// gn
	"Se ha inscrito un usuario.";		// Okoteve traduccin
#elif L==7	// it
	"Un utente &egrave; stato registrato.";
#elif L==8	// pl
	"Jeden uzytkownik zostal zarejestrowany.";
#elif L==9	// pt
	"Um usu&aacute;rio se registrou.";
#endif

const char *Txt_One_user_has_been_removed =
#if   L==1	// ca
	"Se ha eliminado un usuario.";		// Necessita traduccio
#elif L==2	// de
	"One user has been removed.";		// Need bersetzung
#elif L==3	// en
	"One user has been removed.";
#elif L==4	// es
	"Se ha eliminado un usuario.";
#elif L==5	// fr
	"One user has been removed.";		// Besoin de traduction
#elif L==6	// gn
	"Se ha eliminado un usuario.";		// Okoteve traduccin
#elif L==7	// it
	"Un utente &egrave; stato rimosso.";
#elif L==8	// pl
	"Jeden z uzytkownik&oacute;w zostaly usuniete.";
#elif L==9	// pt
	"Um usu&aacute;rio foi removido.";
#endif

const char *Txt_Online_tutoring =
#if   L==1	// ca
	"Tutor&iacute;as virtuales";	// Necessita traduccio
#elif L==2	// de
	"Virtual Anleitungen";
#elif L==3	// en
	"Online tutoring";
#elif L==4	// es
	"Tutor&iacute;as virtuales";
#elif L==5	// fr
	"Tutorat en ligne";
#elif L==6	// gn
	"Tutor&iacute;as virtuales";	// Okoteve traduccin
#elif L==7	// it
	"Tutoraggio on-line";
#elif L==8	// pl
	"Korepetycje online";
#elif L==9	// pt
	"Tutoria on-line";
#endif

const char *Txt_Only_courses_with_more_than_X_clicks_are_shown =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Nom&eacute;s es mostren les assignatures amb m&eacute;s de %u clics.";
#elif L==2	// de
	"Nur Kurse mit mehr als %u Klicks angezeigt.";
#elif L==3	// en
	"Only courses with more than %u clicks are shown.";
#elif L==4	// es
	"Solo se muestran las asignaturas con m&aacute;s de %u clics.";
#elif L==5	// fr
	"Seuls les cours avec plus de %u clics sont pr&eacute;sent&eacute;s.";
#elif L==6	// gn
	"Solo se muestran las asignaturas con m&aacute;s de %u clics.";	// Okoteve traduccin
#elif L==7	// it
	"Solo i corsi con pi&ugrave; di %u scatti sono mostrati.";
#elif L==8	// pl
	"Tylko kursy z ponad 100 klikni&eogon;&cacute; s&aogon; pokazane.";
#elif L==9	// pt
	"Somente disciplinas com mais de %u cliques s&atilde;o mostradas.";
#endif

const char *Txt_only_if_the_scope_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"nom&eacute;s si l'&agrave;mbit &eacute;s %s";
#elif L==2	// de
	"nur dann, wenn der Geltungsbereich ist %s";
#elif L==3	// en
	"only if the scope is %s";
#elif L==4	// es
	"solo si el &aacute;mbito es %s";
#elif L==5	// fr
	"seulement si le port&eacute;e est %s";
#elif L==6	// gn
	"solo si el &aacute;mbito es %s";	// Okoteve traduccin
#elif L==7	// it
	"solo se l'ambito &egrave; %s";
#elif L==8	// pl
	"tylko wtedy, gdy zakres jest %s";
#elif L==9	// pt
	"somente se o &acirc;mbito &eacute; %s";
#endif

const char *Txt_only_students =
#if   L==1	// ca
	", solo estudiantes";	// Necessita traduccio
#elif L==2	// de
	", nur Studenten";
#elif L==3	// en
	", only students";
#elif L==4	// es
	", solo estudiantes";
#elif L==5	// fr
	", seulement &eacute;tudiants";
#elif L==6	// gn
	", solo estudiantes";	// Okoteve traduccin
#elif L==7	// it
	", solo studenti";
#elif L==8	// pl
	", tylko studenci";
#elif L==9	// pt
	", apenas estudantes";
#endif

const char *Txt_only_teachers =
#if   L==1	// ca
	", professors";
#elif L==2	// de
	", Lehrkr&auml;fte";
#elif L==3	// en
	", teachers";
#elif L==4	// es
	", profesores";
#elif L==5	// fr
	", enseignants";
#elif L==6	// gn
	", profesores";	// Okoteve traduccin
#elif L==7	// it
	", professori";
#elif L==8	// pl
	", nauczycieli";
#elif L==9	// pt
	", professores";
#endif

const char *Txt_only_unread_messages =
#if   L==1	// ca
	"nom&eacute;s els missatges no llegits";
#elif L==2	// de
	"nur ungelesene Nachrichten";
#elif L==3	// en
	"only unread messages";
#elif L==4	// es
	"&uacute;nicamente los mensajes no le&iacute;dos";
#elif L==5	// fr
	"uniquement les messages non lus";
#elif L==6	// gn
	"&uacute;nicamente los mensajes no le&iacute;dos";	// Okoteve traduccin
#elif L==7	// it
	"solo i messaggi non letti";
#elif L==8	// pl
	"only unread messages";					// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"apenas mensagens n&atilde;o lidas";
#endif

const char *Txt_open_document =
#if   L==1	// ca
	"document obert";
#elif L==2	// de
	"ge&ouml;ffnet Dokumente";
#elif L==3	// en
	"open document";
#elif L==4	// es
	"documento abierto";
#elif L==5	// fr
	"document ouvert";
#elif L==6	// gn
	"documento abierto";	// Okoteve traduccin
#elif L==7	// it
	"documento aperto";
#elif L==8	// pl
	"otwarty dokument";
#elif L==9	// pt
	"documento aberto";
#endif

const char *Txt_open_documents =
#if   L==1	// ca
	"documents oberts";
#elif L==2	// de
	"ge&ouml;ffnete Dokumente";
#elif L==3	// en
	"open documents";
#elif L==4	// es
	"documentos abiertos";
#elif L==5	// fr
	"documents ouverts";
#elif L==6	// gn
	"documentos abiertos";	// Okoteve traduccin
#elif L==7	// it
	"documenti aperti";
#elif L==8	// pl
	"otwarte dokumenty";
#elif L==9	// pt
	"documentos abertos";
#endif

const char *Txt_Open_game =
#if   L==1	// ca
	"Joc oberto";
#elif L==2	// de
	"Ge&ouml;ffneten Spiel";
#elif L==3	// en
	"Open game";
#elif L==4	// es
	"Juego abierto";
#elif L==5	// fr
	"Jeu ouvert";
#elif L==6	// gn
	"Juego abierto";	// Okoteve traduccin
#elif L==7	// it
	"Gioco aperto";
#elif L==8	// pl
	"Otwarte gra";
#elif L==9	// pt
	"Jogo aberto";
#endif

const char *Txt_Open_survey =
#if   L==1	// ca
	"Enquesta oberta";
#elif L==2	// de
	"Ge&ouml;ffneten Umfrage";
#elif L==3	// en
	"Open survey";
#elif L==4	// es
	"Encuesta abierta";
#elif L==5	// fr
	"Sondage ouvert";
#elif L==6	// gn
	"Encuesta abierta";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio aperto";
#elif L==8	// pl
	"Otwarte badania";
#elif L==9	// pt
	"Inqu&eacute;rito aberta";
#endif

const char *Txt_Opening_of_groups =
#if   L==1	// ca
	"Obertura de grups";
#elif L==2	// de
	"Opening of groups";		// Need bersetzung
#elif L==3	// en
	"Opening of groups";
#elif L==4	// es
	"Apertura de grupos";
#elif L==5	// fr
	"Opening of groups";		// Besoin de traduction
#elif L==6	// gn
	"Apertura de grupos";		// Okoteve traduccin
#elif L==7	// it
	"Apertura di gruppi";
#elif L==8	// pl
	"Opening of groups";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Abertura de grupos";
#endif

const char *Txt_Option_under_development =
#if   L==1	// ca
	"Opci&oacute; en desenvolupament.";
#elif L==2	// de
	"Option in der Entwicklung.";
#elif L==3	// en
	"Option under development.";
#elif L==4	// es
	"Opci&oacute;n en desarrollo.";
#elif L==5	// fr
	"Option en cours de d&eacute;veloppement.";
#elif L==6	// gn
	"Opci&oacute;n en desarrollo.";			// Okoteve traduccin
#elif L==7	// it
	"Opzione in fase di sviluppo.";
#elif L==8	// pl
	"Opcja w fazie rozwoju.";
#elif L==9	// pt
	"Op&ccedil;&atilde;o em desenvolvimento.";
#endif

const char *Txt_optional =
#if   L==1	// ca
	"opcional";
#elif L==2	// de
	"wahlweise";
#elif L==3	// en
	"optional";
#elif L==4	// es
	"opcional";
#elif L==5	// fr
	"en option";
#elif L==6	// gn
	"opcional";			// Okoteve traduccin
#elif L==7	// it
	"opzionale";
#elif L==8	// pl
	"do wyboru";
#elif L==9	// pt
	"opcional";
#endif

const char *Txt_or_you_can_create_a_new_link_inside_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&hellip;o pot crear un novo enlla&ccedil;"
	" dins de la carpeta <strong>%s</strong>:";
#elif L==2	// de
	"&hellip;Oder Sie k&ouml;nnen ein neues Link"
	" im Verzeichnis <strong>%s</strong> anlegen:";
#elif L==3	// en
	"&hellip;or you can create a new link"
	" inside the folder <strong>%s</strong>:";
#elif L==4	// es
	"&hellip;o puede crear un nuevo enlace"
	" dentro de la carpeta <strong>%s</strong>:";
#elif L==5	// fr
	"&hellip;ou vous pouvez cr&eacute;er un nouveau lien"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6	// gn
	"&hellip;o puede crear un nuevo enlace"
	" dentro de la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7	// it
	"&hellip;oppure &egrave; possibile creare una nuova cartella"
	" all'interno della cartella <strong>%s</strong>:";
#elif L==8	// pl
	"&hellip;czy mo&zdot;esz utworzy&cacute; nowy link"
	" wewn&aogon;trz folderu <strong>%s</strong>:";
#elif L==9	// pt
	"&hellip;ou voc&ecirc; pode criar uma nova liga&ccedil;&atilde;o"
	" dentro do diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_or_you_can_make_a_file_copy_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&hellip;o pot realitzar una c&ograve;pia d'arxius"
	" a la carpeta <strong>%s</strong>:";
#elif L==2	// de
	"&hellip;Oder Sie k&ouml;nnen die Kopie einer Datei"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==3	// en
	"&hellip;or you can make a file copy"
	" to the folder <strong>%s</strong>:";
#elif L==4	// es
	"&hellip;o puede realizar una copia de archivos"
	" a la carpeta <strong>%s</strong>:";
#elif L==5	// fr
	"&hellip;ou vous pouvez faire une copie de fichiers"
	" vers le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6	// gn
	"&hellip;o puede realizar una copia de archivos"
	" a la carpeta <strong>%s</strong>:";		// Okoteve traduccin
#elif L==7	// it
	"&hellip;oppure &egrave; possibile effettuare una copia dei file"
	" nella cartella <strong>%s</strong>:";
#elif L==8	// pl
	"&hellip;czy mozna zrobic kopie pliku"
	" do folderu <strong>%s</strong>:";
#elif L==9	// pt
	"&hellip;ou voc&ecirc; pode fazer uma c&oacute;pia de arquivo"
	" na diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_or_you_can_upload_a_new_file_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&hellip;o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";	// Necessita traduccio
#elif L==2	// de
	"&hellip;oder senden Sie eine neue Datei"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==3	// en
	"&hellip;or you can upload a new file"
	" to the folder <strong>%s</strong>:";
#elif L==4	// es
	"&hellip;o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";
#elif L==5	// fr
	"&hellip;ou vous pouvez envoyer un nouveau fichier"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6	// gn
	"&hellip;o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7	// it
	"&hellip;oppure si pu&ograve; inviare un nuovo file"
	" nella cartella <strong>%s</strong>:";
#elif L==8	// pl
	"&hellip;lub wyslac nowy plik"
	" do folderu <strong>%s</strong>:";
#elif L==9	// pt
	"&hellip;ou voc&ecirc; pode enviar um novo arquivo"
	" para o diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_or_you_can_upload_new_files_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"&hellip;o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";	// Necessita traduccio
#elif L==2	// de
	"&hellip;oder senden Sie neue Dateien"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==3	// en
	"&hellip;or you can upload new files"
	" to the folder <strong>%s</strong>:";
#elif L==4	// es
	"&hellip;o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";
#elif L==5	// fr
	"&hellip;ou vous pouvez envoyer des nouveaux fichiers"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6	// gn
	"&hellip;o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7	// it
	"&hellip;oppure si pu&ograve; inviare nuovi file"
	" nella cartella <strong>%s</strong>:";
#elif L==8	// pl
	"&hellip;lub wyslac nowy plik"
	" do folderu <strong>%s</strong>:";
#elif L==9	// pt
	"&hellip;ou voc&ecirc; pode enviar novos arquivos"
	" para o diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_Original_message =
#if   L==1	// ca
	"Mensaje original";	// Necessita traduccio
#elif L==2	// de
	"Original-Nachricht";
#elif L==3	// en
	"Original message";
#elif L==4	// es
	"Mensaje original";
#elif L==5	// fr
	"Message d'origine";
#elif L==6	// gn
	"Mensaje original";	// Okoteve traduccin
#elif L==7	// it
	"Messaggio originale";
#elif L==8	// pl
	"Oryginalna wiadomo&sacute;&cacute;";
#elif L==9	// pt
	"Mensagem original";
#endif

const char *Txt_Other_actions =
#if   L==1	// ca
	"Altres accions";
#elif L==2	// de
	"Andere Handlungen";
#elif L==3	// en
	"Other actions";
#elif L==4	// es
	"Otras acciones";
#elif L==5	// fr
	"Autres actions";
#elif L==6	// gn
	"Otras acciones";	// Okoteve traduccin
#elif L==7	// it
	"Altri azioni";
#elif L==8	// pl
	"Inne dzia&lstrok;ania";
#elif L==9	// pt
	"Outras a&ccedil;&otilde;es";
#endif

const char *Txt_Other_countries =
#if   L==1	// ca
	"Altres pa&iuml;sos";
#elif L==2	// de
	"Andere L&auml;nder";
#elif L==3	// en
	"Other countries";
#elif L==4	// es
	"Otros pa&iacute;ses";
#elif L==5	// fr
	"Autres pays";
#elif L==6	// gn
	"Otros pa&iacute;ses";	// Okoteve traduccin
#elif L==7	// it
	"Altri paesi";
#elif L==8	// pl
	"Inne kraje";
#elif L==9	// pt
	"Outros pa&iacute;ses";
#endif

const char *Txt_Other_departments =
#if   L==1	// ca
	"Altres departaments";
#elif L==2	// de
	"Andere Abteilung";
#elif L==3	// en
	"Other departments";
#elif L==4	// es
	"Otros departamentos";
#elif L==5	// fr
	"Autres d&eacute;partements";
#elif L==6	// gn
	"Otros departamentos";	// Okoteve traduccin
#elif L==7	// it
	"Altri dipartimenti";
#elif L==8	// pl
	"Inne wydzialy";
#elif L==9	// pt
	"Outros departamentos";
#endif

const char *Txt_Other_emails =
#if   L==1	// ca
	"Altres correus";
#elif L==2	// de
	"Andere E-Mails";
#elif L==3	// en
	"Other emails";
#elif L==4	// es
	"Otros correos";
#elif L==5	// fr
	"Autres emails";
#elif L==6	// gn
	"Otros correos";	// Okoteve traduccin
#elif L==7	// it
	"Altre email";
#elif L==8	// pl
	"Inne email";
#elif L==9	// pt
	"Outros emails";
#endif

const char *Txt_Other_nicknames =
#if   L==1	// ca
	"Altres sobrenoms";
#elif L==2	// de
	"Andere Benutzername";
#elif L==3	// en
	"Other nicknames";
#elif L==4	// es
	"Otros apodos";
#elif L==5	// fr
	"Autres surnoms";
#elif L==6	// gn
	"Otros apodos";	// Okoteve traduccin
#elif L==7	// it
	"Altri nomi utenti";
#elif L==8	// pl
	"Inne przydomki";
#elif L==9	// pt
	"Outras alcunhas";
#endif

const char *Txt_Other_places =
#if   L==1	// ca
	"Altres llocs";
#elif L==2	// de
	"Andere Standorte";
#elif L==3	// en
	"Other places";
#elif L==4	// es
	"Otros lugares";
#elif L==5	// fr
	"Autres emplacements";
#elif L==6	// gn
	"Otros lugares";	// Okoteve traduccin
#elif L==7	// it
	"Altre localit&agrave;";
#elif L==8	// pl
	"Inne miejsca";
#elif L==9	// pt
	"Outras localiza&ccedil;oes";
#endif

const char *Txt_Other_recipients =
#if   L==1	// ca
	"Altres destinataris";
#elif L==2	// de
	"Andere Empf&auml;nger";
#elif L==3	// en
	"Other recipients";
#elif L==4	// es
	"Otros destinatarios";
#elif L==5	// fr
	"Autres destinataires";
#elif L==6	// gn
	"Otros destinatarios";	// Okoteve traduccin
#elif L==7	// it
	"Altri destinatari";
#elif L==8	// pl
	"Inne odbiorc&oacute;w";
#elif L==9	// pt
	"Outros destinat&aacute;rios";
#endif

const char *Txt_out_of_PART_OF_A_SCORE =	// Example: score is 2 out of 10
#if   L==1	// ca
	"sobre";	// Necessita traduccio
#elif L==2	// de
	"von";
#elif L==3	// en
	"out of";
#elif L==4	// es
	"sobre";
#elif L==5	// fr
	"sur";
#elif L==6	// gn
	"sobre";	// Okoteve traduccin
#elif L==7	// it
	"su";
#elif L==8	// pl
	"obecnie";
#elif L==9	// pt
	"out of";
#endif

const char *Txt_page =
#if   L==1	// ca
	"p&agrave;g.";
#elif L==2	// de
	"Page";
#elif L==3	// en
	"page";
#elif L==4	// es
	"p&aacute;g.";
#elif L==5	// fr
	"page";
#elif L==6	// gn
	"kuatiarogue";
#elif L==7	// it
	"pag.";
#elif L==8	// pl
	"strona";
#elif L==9	// pt
	"p&aacute;g.";
#endif

const char *Txt_Page_X_of_Y =	// Warning: it is very important to include two %u in the following sentences
#if   L==1	// ca
	"P&agrave;gina %u de %u";
#elif L==2	// de
	"Seite %u von %u";
#elif L==3	// en
	"Page %u of %u";
#elif L==4	// es
	"P&aacute;gina %u de %u";
#elif L==5	// fr
	"Page %u de %u";
#elif L==6	// gn
	"P&aacute;gina %u de %u";	// Okoteve traduccin
#elif L==7	// it
	"Pagina %u di %u";
#elif L==8	// pl
	"Strona %u z %u";
#elif L==9	// pt
	"P&aacute;gina %u de %u";
#endif

const char *Txt_PAGE1_Page_generated_in =
#if   L==1	// ca
	"P&agrave;gina generada en";
#elif L==2	// de
	"Page generated in";
#elif L==3	// en
	"Page generated in";
#elif L==4	// es
	"P&aacute;gina generada en";
#elif L==5	// fr
	"Page g&eacute;n&eacute;r&eacute;e en";
#elif L==6	// gn
	"P&aacute;gina generada en";	// Okoteve traduccin
#elif L==7	// it
	"Pagina generata in";
#elif L==8	// pl
	"Strona wygenerowana w";
#elif L==9	// pt
	"P&aacute;gina gerada em";
#endif

const char *Txt_PAGE2_and_sent_in =
#if   L==1	// ca
	"i enviat en";
#elif L==2	// de
	"gesendet und in";
#elif L==3	// en
	"and sent in";
#elif L==4	// es
	"y enviada en";
#elif L==5	// fr
	"et envoy&eacute;e en";
#elif L==6	// gn
	"y enviada en";	// Okoteve traduccin
#elif L==7	// it
	"e inviata in";
#elif L==8	// pl
	"i wysylane w";
#elif L==9	// pt
	"e enviada em";
#endif

const char *Txt_PAGES_Previous =
#if   L==1	// ca
	"Anteriores";	// Necessita traduccio
#elif L==2	// de
	"Zur&uuml;ck";
#elif L==3	// en
	"Previous";
#elif L==4	// es
	"Anteriores";
#elif L==5	// fr
	"Pr&eacute;c&eacute;dentes";
#elif L==6	// gn
	"Anteriores";	// Okoteve traduccin
#elif L==7	// it
	"Precedenti";
#elif L==8	// pl
	"Poprzedni";
#elif L==9	// pt
	"Anteriores";
#endif

const char *Txt_PAGES_Next =
#if   L==1	// ca
	"Siguientes";	// Necessita traduccio
#elif L==2	// de
	"N&auml;chste";
#elif L==3	// en
	"Next";
#elif L==4	// es
	"Siguientes";
#elif L==5	// fr
	"Suivantes";
#elif L==6	// gn
	"Siguientes";	// Okoteve traduccin
#elif L==7	// it
	"Seguenti";
#elif L==8	// pl
	"Nastepny";
#elif L==9	// pt
	"Seguintes";
#endif

const char *Txt_Password =
#if   L==1	// ca
	"Contrasenya";
#elif L==2	// de
	"Passwort";
#elif L==3	// en
	"Password";
#elif L==4	// es
	"Contrase&ntilde;a";
#elif L==5	// fr
	"Mot de passe";
#elif L==6	// gn
	"Tera&ntilde;emi";
#elif L==7	// it
	"Password";
#elif L==8	// pl
	"Has&lstrok;o";
#elif L==9	// pt
	"Senha";
#endif

const char *Txt_password =
#if   L==1	// ca
	"contrasenya";
#elif L==2	// de
	"Passwort";
#elif L==3	// en
	"password";
#elif L==4	// es
	"contrase&ntilde;a";
#elif L==5	// fr
	"mot de passe";
#elif L==6	// gn
	"tera&ntilde;emi";
#elif L==7	// it
	"password";
#elif L==8	// pl
	"has&lstrok;o";
#elif L==9	// pt
	"senha";
#endif

const char *Txt_Paste =
#if   L==1	// ca
	"Pegar";	// Necessita traduccio
#elif L==2	// de
	"Einf&uuml;gen";
#elif L==3	// en
	"Paste";
#elif L==4	// es
	"Pegar";
#elif L==5	// fr
	"Coller";
#elif L==6	// gn
	"Pegar";	// Okoteve traduccin
#elif L==7	// it
	"Incolla";
#elif L==8	// pl
	"Wklej";
#elif L==9	// pt
	"Colar";
#endif

const char *Txt_Paste_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Pegar en %s";	// Necessita traduccio
#elif L==2	// de
	"Einf&uuml;gen an %s";
#elif L==3	// en
	"Paste in %s";
#elif L==4	// es
	"Pegar en %s";
#elif L==5	// fr
	"Coller dans %s";
#elif L==6	// gn
	"Pegar en %s";	// Okoteve traduccin
#elif L==7	// it
	"Incolla in %s";
#elif L==8	// pl
	"Wklej w %s";
#elif L==9	// pt
	"Colar em %s";
#endif

const char *Txt_PERCENT_of_users =
#if   L==1	// ca
	"% de usuarios";	// Necessita traduccio
#elif L==2	// de
	"% der Benutzer";
#elif L==3	// en
	"% of users";
#elif L==4	// es
	"% de usuarios";
#elif L==5	// fr
	"% d'utilisateurs";
#elif L==6	// gn
	"% de usuarios";	// Okoteve traduccin
#elif L==7	// it
	"% di utenti";
#elif L==8	// pl
	"% uzytkownik&oacute;w";
#elif L==9	// pt
	"% de utilizadores";
#endif

const char *Txt_Permalink =
#if   L==1	// ca
	"Enlla&ccedil; permanent";
#elif L==2	// de
	"Permalink";
#elif L==3	// en
	"Permalink";
#elif L==4	// es
	"Permalink";
#elif L==5	// fr
	"Permalien";
#elif L==6	// gn
	"Permalink";	// Okoteve traduccin
#elif L==7	// it
	"Permalink";
#elif L==8	// pl
	"Odno&sacute;nik bezpo&sacute;redni";
#elif L==9	// pt
	"Liga&ccedil;&atilde;o permanente";
#endif

const char *Txt_Personal_information =
#if   L==1	// ca
	"Dades personals";
#elif L==2	// de
	"Pers&ouml;nliche Angaben";
#elif L==3	// en
	"Personal information";
#elif L==4	// es
	"Datos personales";
#elif L==5	// fr
	"Informations personnelles";
#elif L==6	// gn
	"Datos personales";	// Okoteve traduccin
#elif L==7	// it
	"Informazione personale";
#elif L==8	// pl
	"Informacje osobiste";
#elif L==9	// pt
	"Informa&ccedil;&atilde;o pessoal";
#endif

const char *Txt_Phone =
#if   L==1	// ca
	"Tel&egrave;fon";
#elif L==2	// de
	"Telefon";
#elif L==3	// en
	"Phone";
#elif L==4	// es
	"Tel&eacute;fono";
#elif L==5	// fr
	"T&eacute;l&eacute;phone";
#elif L==6	// gn
	"Pumbyry";
#elif L==7	// it
	"Telefono";
#elif L==8	// pl
	"Telefon";
#elif L==9	// pt
	"Telefone";
#endif

const char *Txt_Photo =
#if   L==1	// ca
	"Foto";
#elif L==2	// de
	"Foto";
#elif L==3	// en
	"Photo";
#elif L==4	// es
	"Foto";
#elif L==5	// fr
	"Photo";
#elif L==6	// gn
	"Ta'anga";
#elif L==7	// it
	"Foto";
#elif L==8	// pl
	"Zdj&eogon;cie";
#elif L==9	// pt
	"Foto";
#endif

const char *Txt_Photo_has_been_updated =
#if   L==1	// ca
	"La fotografia s'ha actualitzat.";
#elif L==2	// de
	"Foto wurde aktualisiert.";
#elif L==3	// en
	"Photo has been updated.";
#elif L==4	// es
	"La fotograf&iacute;a se ha actualizado.";
#elif L==5	// fr
	"La photo a &eacute;t&eacute; mise &agrave; jour.";
#elif L==6	// gn
	"La fotograf&iacute;a se ha actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"La foto &egrave; stata aggiornata.";
#elif L==8	// pl
	"Zdjecie zostalo zaktualizowane.";
#elif L==9	// pt
	"A fotografia foi atualizada.";
#endif

const char *Txt_Photo_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Se ha eliminado la foto de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Photo of <strong>%s</strong> has been removed.";	// Need bersetzung
#elif L==3	// en
	"Photo of <strong>%s</strong> has been removed.";
#elif L==4	// es
	"Se ha eliminado la foto de <strong>%s</strong>.";
#elif L==5	// fr
	"Photo of <strong>%s</strong> has been removed.";	// Besoin de traduction
#elif L==6	// gn
	"Se ha eliminado la foto de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La foto de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==8	// pl
	"Zdjecie <strong>%s</strong> zostalo usuniete.";
#elif L==9	// pt
	"A foto de <strong>%s</strong> foi removida.";
#endif

const char *Txt_PHOTO_PROCESSING_CAPTIONS[3] =
   {
#if   L==1	// ca
	 "Rostre seleccionat"
#elif L==2	// de
	 "Gesicht ausgew&auml;hlt"
#elif L==3	// en
	 "Face selected"
#elif L==4	// es
	 "Rostro seleccionado"
#elif L==5	// fr
	 "Visage s&eacute;lectionn&eacute;"
#elif L==6	// gn
	 "Rostro seleccionado"	// Okoteve traduccin
#elif L==7	// it
	 "Viso selezionato"
#elif L==8	// pl
	 "Twarze wybranych"
#elif L==9	// pt
	 "Rosto selecionado"
#endif
	 ,
#if   L==1	// ca
	 "Contraste y saturaci&oacute;n mejorados"	// Necessita traduccio
#elif L==2	// de
	 "Contrast and saturation enhanced"		// Need bersetzung
#elif L==3	// en
	 "Contrast and saturation enhanced"
#elif L==4	// es
	 "Contraste y saturaci&oacute;n mejorados"
#elif L==5	// fr
	 "Contrast and saturation enhanced"		// Besoin de traduction
#elif L==6	// gn
	 "Contraste y saturaci&oacute;n mejorados"	// Okoteve traduccin
#elif L==7	// it
	 "Contrasto e saturazione migliorata"
#elif L==8	// pl
	 "Zwiekszenie kontrastu i nasycenia"
#elif L==9	// pt
	 "Contraste e satura&ccedil;&atilde;o aprimorados"
#endif
	 ,
#if   L==1	// ca
	 "Balance de blancos realizado"		// Necessita traduccio
#elif L==2	// de
	 "White balance made"			// Need bersetzung
#elif L==3	// en
	 "White balance made"
#elif L==4	// es
	 "Balance de blancos realizado"
#elif L==5	// fr
	 "White balance made"			// Besoin de traduction
#elif L==6	// gn
	 "Balance de blancos realizado"		// Okoteve traduccin
#elif L==7	// it
	 "Bilanciamento di bianco realizzato"
#elif L==8	// pl
	 "Balans bieli sie"
#elif L==9	// pt
	 "Balan&ccedil;o de branco realizado"
#endif
	};

const char *Txt_Photo_removed =
#if   L==1	// ca
	"Fotograf&iacute;a eliminada.";	// Necessita traduccio
#elif L==2	// de
	"Foto entfernt.";
#elif L==3	// en
	"Photo removed.";
#elif L==4	// es
	"Fotograf&iacute;a eliminada.";
#elif L==5	// fr
	"Photo removed.";		// Besoin de traduction
#elif L==6	// gn
	"Fotograf&iacute;a eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Foto rimossa.";
#elif L==8	// pl
	"Zdjecie usuniete.";
#elif L==9	// pt
	"Fotografia removida.";
#endif

const char *Txt_photos =
#if   L==1	// ca
	"fotos";
#elif L==2	// de
	"Fotos";
#elif L==3	// en
	"photos";
#elif L==4	// es
	"fotos";
#elif L==5	// fr
	"photos";
#elif L==6	// gn
	"fotos";	// Okoteve traduccin
#elif L==7	// it
	"foto";
#elif L==8	// pl
	"zdjecia";
#elif L==9	// pt
	"fotos";
#endif

const char *Txt_XxY_pixels_or_higher =
#if   L==1	// ca
	"p&iacute;xels o m&eacute;s gran";
#elif L==2	// de
	"Pixel oder h&ouml;her";
#elif L==3	// en
	"pixels or higher";
#elif L==4	// es
	"p&iacute;xeles o mayor";
#elif L==5	// fr
	"pixels ou plus";
#elif L==6	// gn
	"p&iacute;xeles o mayor";	// Okoteve traduccin
#elif L==7	// it
	"pixel o superiore";
#elif L==8	// pl
	"pikseli lub wy&zdot;szej";
#elif L==9	// pt
	"pixels ou superior";
#endif

const char *Txt_Place =		// A particular spot or area
#if   L==1	// ca
	"Lloc";
#elif L==2	// de
	"Standort";
#elif L==3	// en
	"Place";
#elif L==4	// es
	"Lugar";
#elif L==5	// fr
	"Emplacement";
#elif L==6	// gn
	"Tenda";
#elif L==7	// it
	"Luogo";
#elif L==8	// pl
	"Miejsce";
#elif L==9	// pt
	"Localiza&ccedil;&atilde;o";
#endif

const char *Txt_Place_of_origin =
#if   L==1	// ca
	"Lugar de origen";	// Necessita traduccio
#elif L==2	// de
	"Herkunftsort";
#elif L==3	// en
	"Place of origin";
#elif L==4	// es
	"Lugar de origen";
#elif L==5	// fr
	"Lieu d'origine";
#elif L==6	// gn
	"Lugar de origen";	// Okoteve traduccin
#elif L==7	// it
	"Luogo d'origine";
#elif L==8	// pl
	"Miejsce pochodzenia";
#elif L==9	// pt
	"Lugar de origem";
#endif

const char *Txt_Place_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Lloc <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Standort <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Place <strong>%s</strong> removed.";
#elif L==4	// es
	"Lugar <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Emplacement <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Lugar <strong>%s</strong> eliminado.";		// Okoteve traduccin
#elif L==7	// it
	"Luogo <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Miejsce <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Lugar <strong>%s</strong> removido.";
#endif

const char *Txt_Place_unspecified =
#if   L==1	// ca
	"Lloc sense especificar";
#elif L==2	// de
	"Standort nicht spezifiziert";
#elif L==3	// en
	"Place unspecified";
#elif L==4	// es
	"Lugar sin especificar";
#elif L==5	// fr
	"Emplacement non sp&eacute;cifi&eacute;";
#elif L==6	// gn
	"Lugar sin especificar";	// Okoteve traduccin
#elif L==7	// it
	"Luogo non specificato";
#elif L==8	// pl
	"Miejsce nieokreslone";
#elif L==9	// pt
	"Localiza&ccedil;&atilde;o indeterminada";
#endif

const char *Txt_Places =
#if   L==1	// ca
	"Llocs";
#elif L==2	// de
	"Standorte";
#elif L==3	// en
	"Places";
#elif L==4	// es
	"Lugares / sedes";
#elif L==5	// fr
	"Emplacements";
#elif L==6	// gn
	"Tenda";
#elif L==7	// it
	"Localit&grave;";
#elif L==8	// pl
	"Miejsca";
#elif L==9	// pt
	"Localiza&ccedil;oes";
#endif

const char *Txt_PLACES_HELP_ORDER[Plc_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Ordenar por lugar"	// Necessita traduccio
#elif L==2	// de
	 "Nach Standort sortieren"
#elif L==3	// en
	 "Sort by place"
#elif L==4	// es
	 "Ordenar por lugar"
#elif L==5	// fr
	 "Trier par emplacement"
#elif L==6	// gn
	 "Ordenar por lugar"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per localit&grave;"
#elif L==8	// pl
	 "Sortuj wedlug miejsca"
#elif L==9	// pt
	 "Classificar por localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "Ordenar per nombre de titulacions"
#elif L==2	// de
	 "Nach Anzahl der Studieng&auml;nge sortieren"
#elif L==3	// en
	 "Sort by no. of degrees"
#elif L==4	// es
	 "Ordenar por n&ordm; de titulaciones"
#elif L==5	// fr
	 "Trier par nombre d'&eacute;tudes"
#elif L==6	// gn
	 "Ordenar por n&ordm; de titulaciones"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per numero di lauree"
#elif L==8	// pl
	 "Sortuj wedlug stopni"
#elif L==9	// pt
	 "Classificar por n&uacute;mero de graus"
#endif
	};

const char *Txt_PLACES_ORDER[Plc_NUM_ORDERS] =
   {
#if   L==1	// ca
	 "Lloc"
#elif L==2	// de
	 "Standort"
#elif L==3	// en
	 "Place"
#elif L==4	// es
	 "Lugar"
#elif L==5	// fr
	 "Emplacement"
#elif L==6	// gn
	 "Tenda"
#elif L==7	// it
	 "Luogo"
#elif L==8	// pl
	 "Miejsce"
#elif L==9	// pt
	 "Localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca
	"Centres"
#elif L==2	// de
	"Lehrinstitute"
#elif L==3	// en
	"Centres"
#elif L==4	// es
	"Centros"
#elif L==5	// fr
	"Centres"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Centri"
#elif L==8	// pl
	"Centra"
#elif L==9	// pt
	"Centros"
#endif
	};

const char *Txt_Play =	// To play a game
#if   L==1	// ca
	"Jugar";
#elif L==2	// de
	"Spielen";
#elif L==3	// en
	"Play";
#elif L==4	// es
	"Jugar";
#elif L==5	// fr
	"Jouer";
#elif L==6	// gn
	"Jugar";	// Okoteve traduccin
#elif L==7	// it
	"Giocare";
#elif L==8	// pl
	"Gra&cacute;";
#elif L==9	// pt
	"Jogar";
#endif

const char *Txt_Please_check_your_email_address =
#if   L==1	// ca
	"Si us plau, comproveu la seva adre&ccedil;a de correu.";
#elif L==2	// de
	"Bitte &uuml;berpr&uuml;fen Ihre E-Mail-Adresse.";
#elif L==3	// en
	"Please check your email address.";
#elif L==4	// es
	"Por favor, compruebe su direcci&oacute;n de correo.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, v&eaxcute;rifiez votre adresse email.";
#elif L==6	// gn
	"Por favor, compruebe su direcci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, verifica il tuo indirizzo email.";
#elif L==8	// pl
	"Prosz&eogon; sprawd&zacute; sw&oacute;j adres email.";
#elif L==9	// pt
	"Por favor, verifique seu endere&ccedil;o de email.";
#endif

const char *Txt_Please_confirm_your_email_address =
#if   L==1	// ca
	"Si us plau, confirmeu la seva adre&ccedil;a de correu.";
#elif L==2	// de
	"Bitte best&auml;tige Ihre E-Mail-Adressen.";
#elif L==3	// en
	"Please confirm your email address.";
#elif L==4	// es
	"Por favor, confirme su direcci&oacute;n de correo.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, confirmez votre adresse email.";
#elif L==6	// gn
	"Por favor, confirme su direcci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, conferma il tuo indirizzo email.";
#elif L==8	// pl
	"Prosz&eogon; potwierdzi&cacute; sw&oacute;j adres email.";
#elif L==9	// pt
	"Por favor, confirme seu endere&ccedil;o de email.";
#endif

const char *Txt_Please_enter_your_ID =
#if   L==1	// ca
	"Si us plau, escriviu el seu ID (DNI/c&egrave;dula).";
#elif L==2	// de
	"Bitte geben Sie Ihre Ausweis-Nr. &auml;nder.";
#elif L==3	// en
	"Please enter your ID.";
#elif L==4	// es
	"Por favor, escriba su ID (DNI/c&eacute;dula).";
#elif L==5	// fr
	"S'il vous pla&iacute;t, entrez votre ID.";
#elif L==6	// gn
	"Por favor, escriba su ID (DNI/c&eacute;dula).";		// Okoteve traduccin
#elif L==7	// it
	"Per favore, inserisci il tuo ID.";
#elif L==8	// pl
	"Prosz&eogon;, poda&cacute; swoje zmie&nacute; identyfikatory.";
#elif L==9	// pt
	"Por favor, digite n&ordm; de identifica&ccedil;&atilde;o.";
#endif

const char *Txt_Please_fill_in_your_ID =
#if   L==1	// ca
	"Si us plau, ompli el seu ID (DNI/c&egrave;dula)."
	" &Eacute;s una dada opcional,"
	" per&ograve; &eacute;s recomanable que l'ompli"
        " perqu&egrave; facilita el proc&eacute;s d'inscripci&oacute;"
        " dels usuaris en les assignatures.";
#elif L==2	// de
	"Bitte geben Sie Ihre Ausweis-Nr. &auml;nder."
	" Es ist eine optionale Daten,"
	" aber es wird empfohlen, sie zu f&uuml;llen,"
	" weil sie die Registrierung"
	" der Benutzer in den Kursen erleichtert.";
#elif L==3	// en
	"Please fill in your ID."
	" It is an optional data,"
	" but it is recommended that you fill it"
	" because it facilitates the registration process"
	" of users in the courses.";
#elif L==4	// es
	"Por favor, rellene su ID (DNI/c&eacute;dula)."
	" Es un dato opcional,"
	" pero es recomendable que lo rellene"
	" porque facilita el proceso de inscripci&oacute;n"
	" de los usuarios en las asignaturas.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, indiquez votre ID."
	" C'est une donn&eacute;e facultative,"
	" mais il est recommand&eacute; que vous la remplissez"
	" parce qu'il facilite le processus d'enregistrement"
	" des utilisateurs dans les mati&egrave;res.";
#elif L==6	// gn
	"Por favor, rellene su ID (DNI/c&eacute;dula)."
	" Es un dato opcional,"
	" pero es recomendable que lo rellene"
	" porque facilita el proceso de inscripci&oacute;n"
	" de los usuarios en las asignaturas.";		// Okoteve traduccin
#elif L==7	// it
	"Per favore, inserisci il tuo ID."
	" Si tratta di un dato facoltativo,"
	" ma si raccomanda che si riempie"
	" perch&eacute; facilita il processo di registrazione"
	" degli utenti ai corsi.";
#elif L==8	// pl
	"Prosz&eogon;, wpisz swoje zmie&nacute; identyfikatory."
	" Jest to opcja danych, ale zaleca si&eogon;,"
	" aby wype&lstrok;ni&cacute; go, poniewa&zdot; u&lstrok;atwia proces rejestracji"
	" u&zdot;ytkownik&oacute;w w kursach.";
#elif L==9	// pt
	"Por favor, preencha seu n&ordm; de identifica&ccedil;&atilde;o"
	" &Eacute; uma dada opcional,"
	" mas &eacute; recomend&aacute;vel que voc&ecirc; preench&ecirc;-lo,"
	" pois facilita o processo de registro"
	" de usu&aacute;rios nas disciplinas.";
#endif

const char *Txt_Please_fill_in_your_record_card_including_your_country_nationality =
#if   L==1	// ca
	"Si us plau, ompli la seva fitxa,"
	" incloent el seu pais (nacionalitat).";
#elif L==2	// de
	"Bitte geben Sie Ihre Karteikarte zu f&uuml;llen,"
	" einschlie&szlig;lich Ihrem Land (Nationalit&auml;t).";
#elif L==3	// en
	"Please fill in your country (nationality).";
#elif L==4	// es
	"Por favor, rellene su ficha,"
	" incluyendo su pa&iacute;s (nacionalidad).";
#elif L==5	// fr
	"S'il vous pla&iacute;t, remplissez votre carte d'enregistrement,"
	" y compris votre pays (nationalit&eacute;).";
#elif L==6	// gn
	"Por favor, rellene su ficha,"
	" incluyendo su pa&iacute;s (nacionalidad).";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, compila la tua scheda,"
	" compreso il tuo paese (nazionalit&agrave;).";
#elif L==8	// pl
	"Prosz&eogon;, wpisz swoje karte zapisu,"
	" w tym kraju (obywatelstwo";
#elif L==9	// pt
	"Por favor, preencha o seu cart&atilde;o,"
	" incluindo seu pa&iacute;s (nacionalidade).";
#endif

const char *Txt_Please_fill_in_your_record_card_including_your_name =
#if   L==1	// ca
	"Si us plau, ompli la seva fitxa,"
	" incloent el seu nom i cognom(s).";
#elif L==2	// de
	"Bitte geben Sie Ihre Karteikarte zu f&uuml;llen,"
	" mit Ihrem Namen.";
#elif L==3	// en
	"Please fill in your record card,"
	" including your name.";
#elif L==4	// es
	"Por favor, rellene su ficha,"
	" incluyendo su nombre y apellido(s).";
#elif L==5	// fr
	"S'il vous pla&iacute;t, remplissez votre carte d'enregistrement,"
	" y compris votre nom.";
#elif L==6	// gn
	"Por favor, rellene su ficha,"
	" incluyendo su nombre y apellido(s).";		// Okoteve traduccin
#elif L==7	// it
	"Per favore, compila la tua scheda,"
	" compresi il tuo nome e cognome.";
#elif L==8	// pl
	"Prosz&eogon;, wpisz swoje karte zapisu,"
	" w tym imie i nazwisko.";
#elif L==9	// pt
	"Por favor, preencha o cart&atilde;o,"
	" incluindo seu nome.";
#endif

const char *Txt_Please_fill_in_your_record_card_including_your_sex =
#if   L==1	// ca
	"Si us plau, ompli la seva fitxa,"
	" incloent el sexe.";
#elif L==2	// de
	"Bitte geben Sie Ihre Karteikarte zu f&uuml;llen,"
	" einschlie&szlig;lich Ihrer Sex.";
#elif L==3	// en
	"Please fill in your record card,"
	" including your sex.";
#elif L==4	// es
	"Por favor, rellene su ficha,"
	" incluyendo el sexo.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, remplissez votre carte d'enregistrement,"
	" y compris votre sexe.";
#elif L==6	// gn
	"Por favor, rellene su ficha,"
	" incluyendo el sexo.";				// Okoteve traduccin
#elif L==7	// it
	"Per favore, compila la tua scheda,"
	" compreso il sesso.";
#elif L==8	// pl
	"Prosz&eogon;, wpisz swoje karte zapisu,"
	" w tym ple&cacute;.";
#elif L==9	// pt
	"Por favor, preencha o cart&atilde;o,"
	" incluindo o seu sexo.";
#endif

const char *Txt_Please_review_your_privacy_preferences =
#if   L==1	// ca
	"Si us plau, revisi les seves prefer&egrave;ncies de privacitat.";
#elif L==2	// de
	"Bitte &uuml;berpr&uuml;fen Sie Ihre Datenschutzeinstellungen.";
#elif L==3	// en
	"Please review your privacy preferences.";
#elif L==4	// es
	"Por favor, revise sus preferencias de privacidad.";
#elif L==5	// fr
	"S'il vous pla&icirc;t revoir vos pr&eacute;f&eacute;rences de privacit&eacute;.";
#elif L==6	// gn
	"Por favor, revise sus preferencias de privacidad.";	// Okoteve traduccin
#elif L==7	// it
	"Si prega di rivedere le vostre preferenze di privatezza.";
#elif L==8	// pl
	"Prosz&eogon; sprawdzi&cacute; ustawienia prywatno&sacute;ci.";
#elif L==9	// pt
	"Por favor, revise suas prefer&ecirc;ncias de privacidade.";
#endif

const char *Txt_Please_select_the_country_of_your_institution =
#if   L==1	// ca
	"Si us plau, seleccioneu el pa&iacute;s de la seva instituci&oacute;.";
#elif L==2	// de
	"Bitte w&auml;hlen Sie das Land Ihrer Hochschule aus";
#elif L==3	// en
	"Please, select the country of your institution.";
#elif L==4	// es
	"Por favor, seleccione el pa&iacute;s de su instituci&oacute;n.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, s&eacute;lectionnez le pays de votre &eacute;tablissement.";
#elif L==6	// gn
	"Por favor, seleccione el pa&iacute;s de su instituci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, selezionare il paese di la tua istituzione accademica.";
#elif L==8	// pl
	"Prosz&eogon;, wybra&cacute; kraj swojej instytucji.";
#elif L==9	// pt
	"Por favor, selecione o pa&iacute;s da sua institui&ccedil;&atilde;o.";
#endif

const char *Txt_Please_select_your_centre =
#if   L==1	// ca
	"Si us plau, seleccioneu el seu centre.";
#elif L==2	// de
	"Bitte w&auml;hlen Sie Ihre Zentrum aus.";
#elif L==3	// en
	"Please, select your centre.";
#elif L==4	// es
	"Por favor, seleccione su centro.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, s&eacute;lectionnez votre centre.";
#elif L==6	// gn
	"Por favor, seleccione su centro.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, seleziona il tuo centro.";
#elif L==8	// pl
	"Prosz&eogon;, wybra&cacute; swoje tym centrum.";
#elif L==9	// pt
	"Por favor, selecione seu centro.";
#endif

const char *Txt_Please_select_your_department =
#if   L==1	// ca
	"Si us plau, seleccioneu el seu department.";
#elif L==2	// de
	"Bitte w&auml;hlen Sie Ihre Abteilung aus.";
#elif L==3	// en
	"Please, select your department.";
#elif L==4	// es
	"Por favor, seleccione su departamento.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, s&eacute;lectionnez votre d&eacute;partement.";
#elif L==6	// gn
	"Por favor, seleccione su departamento.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, seleziona il tuo dipartimento.";
#elif L==8	// pl
	"Prosz&eogon;, wybra&cacute; swoje tym dzia&lstrok;u.";
#elif L==9	// pt
	"Por favor, selecione seu departamento.";
#endif

const char *Txt_Please_select_your_institution =
#if   L==1	// ca
	"Si us plau, seleccioneu la seva instituci&oacute;.";
#elif L==2	// de
	"Bitte w&auml;hlen Sie Ihre Hochschule aus.";
#elif L==3	// en
	"Please, select your institution.";
#elif L==4	// es
	"Por favor, seleccione su instituci&oacute;n.";
#elif L==5	// fr
	"S'il vous pla&iacute;t, s&eacute;lectionnez votre &eacute;tablissement.";
#elif L==6	// gn
	"Por favor, seleccione su instituci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Per favore, seleziona la tua istituzione accademica.";
#elif L==8	// pl
	"Prosz&eogon;, wybra&cacute; swoj&aogon; instytucj&eogon;.";
#elif L==9	// pt
	"Por favor, selecione sua institui&ccedil;&atilde;o.";
#endif

const char *Txt_Please_specify_if_you_allow_access_to_test_questions_from_mobile_applications =
#if   L==1	// ca
	"Por favor, especifique si permite el acceso"
	" a las preguntas de test desde aplicaciones m&oacute;viles.";	// Necessita traduccio
#elif L==2	// de
	"Please specify if you allow access"
	" to test questions from mobile applications.";			// Need bersetzung
#elif L==3	// en
	"Please specify if you allow access"
	" to test questions from mobile applications.";
#elif L==4	// es
	"Por favor, especifique si permite el acceso"
	" a las preguntas de test desde aplicaciones m&oacute;viles.";
#elif L==5	// fr
	"Please specify if you allow access"
	" to test questions from mobile applications.";			// Besoin de traduction
#elif L==6	// gn
	"Por favor, especifique si permite el acceso"
	" a las preguntas de test desde aplicaciones m&oacute;viles.";	// Okoteve traduccin
#elif L==7	// it
	"Please specify if you allow access"
	" to test questions from mobile applications.";			// Bisogno di traduzione
#elif L==8	// pl
	"Please specify if you allow access"
	" to test questions from mobile applications.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Por favor, especifique se voc&ecirc; permite acesso"
	" para perguntas de teste de aplicativos para dispositivos m&oacute;veis.";
#endif

const char *Txt_Plugin =
#if   L==1	// ca
	"Complement";
#elif L==2	// de
	"Plugin";
#elif L==3	// en
	"Plugin";
#elif L==4	// es
	"Complemento";
#elif L==5	// fr
	"Plugin";
#elif L==6	// gn
	"Complemento";	// Okoteve traduccin
#elif L==7	// it
	"Plugin";
#elif L==8	// pl
	"Wtyczka";
#elif L==9	// pt
	"Plugin";
#endif

const char *Txt_Plugin_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Complemento <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2	// de
	"Plugin <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Plugin <strong>%s</strong> removed.";
#elif L==4	// es
	"Complemento <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Plugin <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6	// gn
	"Complemento <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Plugin <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Wtyczka <strong>%s</strong> usunieta.";
#elif L==9	// pt
	"Plugin <strong>%s</strong> removido.";
#endif

const char *Txt_Plugins =
#if   L==1	// ca
	"Complements";
#elif L==2	// de
	"Plugins";
#elif L==3	// en
	"Plugins";
#elif L==4	// es
	"Complementos";
#elif L==5	// fr
	"Plugins";
#elif L==6	// gn
	"Complementos";		// Okoteve traduccin
#elif L==7	// it
	"Plugin";
#elif L==8	// pl
	"wtyczki";
#elif L==9	// pt
	"Plugins";
#endif

const char *Txt_Possibly_duplicate_users =
#if   L==1	// ca
	"Usuaris possiblement duplicats";
#elif L==2	// de
	"M&ouml;gliche doppelte Benutzer";
#elif L==3	// en
	"Possibly duplicate users";
#elif L==4	// es
	"Usuarios posiblemente duplicados";
#elif L==5	// fr
	"Possibles utilisateurs en double";
#elif L==6	// gn
	"Usuarios posiblemente duplicados";	// Okoteve traduccin
#elif L==7	// it
	"Possibili utenti duplicati";
#elif L==8	// pl
	"Mo&zdot;liwe duplikaty u&zdot;ytkownik&oacute;w";
#elif L==9	// pt
	"Usu&aacute;rios possivelmente duplicados";
#endif


const char *Txt_Post =	// Publish (verb)
#if   L==1	// ca
	"Publicar";
#elif L==2	// de
	"Ver&ouml;ffentlichen";
#elif L==3	// en
	"Post";
#elif L==4	// es
	"Publicar";
#elif L==5	// fr
	"Publier";
#elif L==6	// gn
	"Publicar";	// Okoteve traduccin
#elif L==7	// it
	"Pubblicare";
#elif L==8	// pl
	"Publikowa&cacute;";
#elif L==9	// pt
	"Publicar";
#endif

const char *Txt_Preassigned_QUESTION =
#if   L==1	// ca
	"Preassignat?";
#elif L==2	// de
	"Vorbelegt?";
#elif L==3	// en
	"Preassigned?";
#elif L==4	// es
	"&iquest;Preasignado?";
#elif L==5	// fr
	"Pr&eacute;assign&eacute;?";
#elif L==6	// gn
	"&iquest;Preasignado?";	// Okoteve traduccin
#elif L==7	// it
	"Preassegnato?";
#elif L==8	// pl
	"Zadany?";
#elif L==9	// pt
	"Pr&eacute;-atribu&iacute;do?";
#endif

const char *Txt_Present =
#if   L==1	// ca
	"Present";
#elif L==2	// de
	"Vorhanden";
#elif L==3	// en
	"Present";
#elif L==4	// es
	"Presente";
#elif L==5	// fr
	"Pr&eacute;sent";
#elif L==6	// gn
	"Presente";	// Okoteve traduccin
#elif L==7	// it
	"Presente";
#elif L==8	// pl
	"Obecny";
#elif L==9	// pt
	"Presente";
#endif

const char *Txt_Presents =
#if   L==1	// ca
	"Presents";
#elif L==2	// de
	"Vorhanden";
#elif L==3	// en
	"Presents";
#elif L==4	// es
	"Presentes";
#elif L==5	// fr
	"Pr&eacute;sents";
#elif L==6	// gn
	"Presentes";	// Okoteve traduccin
#elif L==7	// it
	"Presenti";
#elif L==8	// pl
	"Obecny";
#elif L==9	// pt
	"Presentes";
#endif

const char *Txt_Print =
#if   L==1	// ca
	"Imprimir";
#elif L==2	// de
	"Ausdrucken";
#elif L==3	// en
	"Print";
#elif L==4	// es
	"Imprimir";
#elif L==5	// fr
	"Imprimer";
#elif L==6	// gn
	"Imprimir";	// Okoteve traduccin
#elif L==7	// it
	"Stampare";
#elif L==8	// pl
	"Wydrukuj";
#elif L==9	// pt
	"Imprimir";
#endif

const char *Txt_Privacy =
#if   L==1	// ca
	"Privacitat";
#elif L==2	// de
	"Geheimhaltung";
#elif L==3	// en
	"Privacy";
#elif L==4	// es
	"Privacidad";
#elif L==5	// fr
	"Privacit&eacute;";
#elif L==6	// gn
	"Privacidad";	// Okoteve traduccin
#elif L==7	// it
	"Privatezza";
#elif L==8	// pl
	"Prywatno&sacute;&cacute;";
#elif L==9	// pt
	"Privacidade";
#endif

const char *Txt_PRIVACY_OPTIONS[Pri_NUM_OPTIONS_PRIVACY] =
   {		// Pri_VISIBILITY_UNKNOWN
#if   L==1	// ca
	 "Desconeguda"
#elif L==2	// de
	 "Unbekannt"
#elif L==3	// en
	 "Unknown"
#elif L==4	// es
	 "Desconocida"
#elif L==5	// fr
	 "Inconnu"
#elif L==6	// gn
	 "Desconocida"	// Okoteve traduccin
#elif L==7	// it
	 "Sconosciuta"
#elif L==8	// pl
	 "Nieznany"
#elif L==9	// pt
	 "Desconhecida"
#endif
	 ,	// Pri_VISIBILITY_USER
#if   L==1	// ca
	 "Visible por m&iacute; y mis profesores / mis estudiantes"	// Necessita traduccio
#elif L==2	// de
	 "Visible by me and my teachers / my students"			// Need bersetzung
#elif L==3	// en
	 "Visible by me and my teachers / my students"
#elif L==4	// es
	 "Visible por m&iacute; y mis profesores / mis estudiantes"
#elif L==5	// fr
	 "Visible by me and my teachers / my students"			// Besoin de traduction
#elif L==6	// gn
	 "Visible por m&iacute; y mis profesores / mis estudiantes"	// Okoteve traduccin
#elif L==7	// it
	 "Visible by me and my teachers / my students"			// Bisogno di traduzione
#elif L==8	// pl
	 "Visible by me and my teachers / my students"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Vis&iacute;vel por mim e meus professores / meus estudantes"
#endif
	 ,	// Pri_VISIBILITY_COURSE
#if   L==1	// ca
	 "Visible por usuarios con los que comparto asignaturas"	// Necessita traduccio
#elif L==2	// de
	 "Visible by users with whom I share courses"			// Need bersetzung
#elif L==3	// en
	 "Visible by users with whom I share courses"
#elif L==4	// es
	 "Visible por usuarios con los que comparto asignaturas"
#elif L==5	// fr
	 "Visible by users with whom I share courses"			// Besoin de traduction
#elif L==6	// gn
	 "Visible por usuarios con los que comparto asignaturas"	// Okoteve traduccin
#elif L==7	// it
	 "Visible by users with whom I share courses"			// Bisogno di traduzione
#elif L==8	// pl
	 "Visible by users with whom I share courses"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Vis&iacute;vel por usu&aacute;rios com quem compartilho disciplinas"
#endif
	 ,	// Pri_VISIBILITY_SYSTEM
#if   L==1	// ca
	 "Visible por cualquier usuario identificado"	// Necessita traduccio
#elif L==2	// de
	 "Visible by any logged user"			// Need bersetzung
#elif L==3	// en
	 "Visible by any logged user"
#elif L==4	// es
	 "Visible por cualquier usuario identificado"
#elif L==5	// fr
	 "Visible by any logged user"			// Besoin de traduction
#elif L==6	// gn
	 "Visible por cualquier usuario identificado"	// Okoteve traduccin
#elif L==7	// it
	 "Visible by any logged user"			// Bisogno di traduzione
#elif L==8	// pl
	 "Visible by any logged user"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Vis&iacute;vel por qualquer usu&aacute;rio logado"
#endif
	 ,	// Pri_VISIBILITY_WORLD
#if   L==1	// ca
	 "Visible por todo el mundo"	// Necessita traduccio
#elif L==2	// de
	 "Visible by everyone"		// Need bersetzung
#elif L==3	// en
	 "Visible by everyone"
#elif L==4	// es
	 "Visible por todo el mundo"
#elif L==5	// fr
	 "Visible by everyone"		// Besoin de traduction
#elif L==6	// gn
	 "Visible por todo el mundo"	// Okoteve traduccin
#elif L==7	// it
	 "Visible by everyone"		// Bisogno di traduzione
#elif L==8	// pl
	 "Visible by everyone"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Vis&iacute;vel em todo o mundo"
#endif
	};

const char *Txt_Private_available_to_certain_users_identified =
#if   L==1	// ca
	"Privat, disponible per a certs usuaris identificats";
#elif L==2	// de
	"Private, verf&uuml;gbar auf bestimmte Benutzer identifiziert";
#elif L==3	// en
	"Private, available to certain users identified";
#elif L==4	// es
	"Privado, disponible para ciertos usuarios identificados";
#elif L==5	// fr
	"Priv&eacute;, &agrave; la disposition de certains utilisateurs identifi&eacute;s";
#elif L==6	// gn
	"Privado, disponible para ciertos usuarios identificados";	// Okoteve traduccin
#elif L==7	// it
	"Privato, a disposizione degli utenti individuati alcuni";
#elif L==8	// pl
	"Prywatne, dost&eogon;pne dla okre&sacute;lonych u&zdot;ytkownik&oacute;w zidentyfikowanych";
#elif L==9	// pt
	"Privado, dispon&iacute;vel para alguns usu&aacute;rios identificados";
#endif

const char *Txt_project =
#if   L==1	// ca
	"projecte";
#elif L==2	// de
	"Projekt";
#elif L==3	// en
	"project";
#elif L==4	// es
	"proyecto";
#elif L==5	// fr
	"project";
#elif L==6	// gn
	"proyecto";	// Okoteve traduccin
#elif L==7	// it
	"progetto";
#elif L==8	// pl
	"projektowe";
#elif L==9	// pt
	"projeto";
#endif

const char *Txt_Project_assessment =
#if   L==1	// ca
	"Avaluaci&oacute; del projecte";
#elif L==2	// de
	"Projektbewertung";
#elif L==3	// en
	"Project assessment";
#elif L==4	// es
	"Evaluaci&oacute;n del proyecto";
#elif L==5	// fr
	"&Eacute;valuation du project";
#elif L==6	// gn
	"Evaluaci&oacute;n del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"Valutazione di progetto";
#elif L==8	// pl
	"Ocena projektowe";
#elif L==9	// pt
	"Avalia&ccedil;&atilde;o do projeto";
#endif

const char *Txt_project_assessment =
#if   L==1	// ca
	"avaluaci&oacute; del projecte";
#elif L==2	// de
	"Projektbewertung";
#elif L==3	// en
	"project assessment";
#elif L==4	// es
	"evaluaci&oacute;n del proyecto";
#elif L==5	// fr
	"&eacute;valuation du project";
#elif L==6	// gn
	"evaluaci&oacute;n del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"valutazione di progetto";
#elif L==8	// pl
	"ocena projektowe";
#elif L==9	// pt
	"avalia&ccedil;&atilde;o do projeto";
#endif

const char *Txt_Project_documents =
#if   L==1	// ca
	"Documents del projecte";
#elif L==2	// de
	"Projektdokumente";
#elif L==3	// en
	"Project documents";
#elif L==4	// es
	"Documentos del proyecto";
#elif L==5	// fr
	"Documents du project";
#elif L==6	// gn
	"Documentos del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"Documenti di progetto";
#elif L==8	// pl
	"Dokumenty projektowe";
#elif L==9	// pt
	"Documentos do projeto";
#endif

const char *Txt_project_documents =
#if   L==1	// ca
	"documents del projecte";
#elif L==2	// de
	"Projektdokumente";
#elif L==3	// en
	"project documents";
#elif L==4	// es
	"documentos del proyecto";
#elif L==5	// fr
	"documents du project";
#elif L==6	// gn
	"documentos del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"documenti di progetto";
#elif L==8	// pl
	"dokumenty projektowe";
#elif L==9	// pt
	"documentos do projeto";
#endif

const char *Txt_Project_files =
#if   L==1	// ca
	"Fitxers del projecte";
#elif L==2	// de
	"Projektdateien";
#elif L==3	// en
	"Project files";
#elif L==4	// es
	"Archivos del proyecto";
#elif L==5	// fr
	"Fichiers du project";
#elif L==6	// gn
	"Archivos del proyecto";	// Okoteve traduccin
#elif L==7	// it
	"File di progetto";
#elif L==8	// pl
	"Pliki projektowe";
#elif L==9	// pt
	"Arquivos do projeto";
#endif

const char *Txt_Project_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El projecte <strong>%s</strong> ara est&agrave; ocult.";
#elif L==2	// de
	"Die Projekt <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==3	// en
	"Project <strong>%s</strong> is now hidden.";
#elif L==4	// es
	"El proyecto <strong>%s</strong> ahora est&aacute; oculto.";
#elif L==5	// fr
	"Le project <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==6	// gn
	"El proyecto <strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==7	// it
	"Il progetto <strong>%s</strong> &egrave; ora nascosto.";
#elif L==8	// pl
	"Projekt <strong>%s</strong> jest ukryte.";
#elif L==9	// pt
	"O projeto <strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_Project_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El projecte <strong>%s</strong> ara est&agrave; visible.";
#elif L==2	// de
	"Die Projekt <strong>%s</strong> ist jetzt sichtbar.";
#elif L==3	// en
	"Project <strong>%s</strong> is now visible.";
#elif L==4	// es
	"El proyecto <strong>%s</strong> ahora est&aacute; visible.";
#elif L==5	// fr
	"Le project <strong>%s</strong> est maintenant visible.";
#elif L==6	// gn
	"El proyecto <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7	// it
	"Il progetto <strong>%s</strong> &egrave; ora nascosto.";
#elif L==8	// pl
	"Projekt <strong>%s</strong> jest teraz widoczne.";
#elif L==9	// pt
	"O projeto <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_PROJECT_ORDER[Prj_NUM_ORDERS] =
   {
	 // Prj_ORDER_START_TIME
#if   L==1	// ca
	"Data de creaci&oacute;"
#elif L==2	// de
	"Datum der Erstellung"
#elif L==3	// en
	"Date of creation"
#elif L==4	// es
	"Fecha de creaci&oacute;n"
#elif L==5	// fr
	"Date de cr&eacute;ation"
#elif L==6	// gn
	"Fecha de creaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Data di creazione"
#elif L==8	// pl
	"Data utworzenia"
#elif L==9	// pt
	"Data da cria&ccedil;&atilde;o"
#endif
	 ,
	 // Prj_ORDER_END_TIME
#if   L==1	// ca
	"&Uacute;ltima edici&oacute;"
#elif L==2	// de
	"Letzte Ausgabe"
#elif L==3	// en
	"Last edition"
#elif L==4	// es
	"&Uacute;ltima edici&oacute;n"
#elif L==5	// fr
	"Derni&egrave;re &eacute;dition"
#elif L==6	// gn
	"&Uacute;ltima edici&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ultima edizione"
#elif L==8	// pl
	"Ostatnie wydanie"
#elif L==9	// pt
	"&Uacute;ltima edi&ccedil;&atilde;o"
#endif
	 ,
	 // Prj_ORDER_TITLE
#if   L==1	// ca
	"T&iacute;tol"
#elif L==2	// de
	"Titel"
#elif L==3	// en
	"Title"
#elif L==4	// es
	"T&iacute;tulo"
#elif L==5	// fr
	"Titre"
#elif L==6	// gn
	"Moak&atilde;ha"
#elif L==7	// it
	"Titolo"
#elif L==8	// pl
	"Tytu&lstrok;"
#elif L==9	// pt
	"T&iacute;tulo"
#endif
	 ,
	 // Prj_ORDER_DEPARTMENT
#if   L==1	// ca
	"Departament"
#elif L==2	// de
	"Abteilung"
#elif L==3	// en
	"Department"
#elif L==4	// es
	"Departamento"
#elif L==5	// fr
	"D&eacute;partement"
#elif L==6	// gn
	"Peh&#7869;"
#elif L==7	// it
	"Dipartimento"
#elif L==8	// pl
	"Wydzia&lstrok;"
#elif L==9	// pt
	"Departamento"
#endif
	};

const char *Txt_PROJECT_ORDER_HELP[Prj_NUM_ORDERS] =
   {
	 // Prj_ORDER_START_TIME
#if   L==1	// ca
	 "Ordenar per data de creaci&oacute;"
#elif L==2	// de
	 "Sortieren nach Erstellungsdatum"
#elif L==3	// en
	 "Sort by date of creation"
#elif L==4	// es
	 "Ordenar por fecha de creaci&oacute;n"
#elif L==5	// fr
	 "Trier par date de cr&eacute;ation"
#elif L==6	// gn
	 "Ordenar por fecha de creaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per data di creazione"
#elif L==8	// pl
	 "Sortuj wed&lstrok;ug daty utworzenia"
#elif L==9	// pt
	 "Classificar por data de cria&ccedil;&atilde;o"
#endif
	 ,
	 // Prj_ORDER_END_TIME
#if   L==1	// ca
	 "Ordenar per data de l'&uacute;ltima edici&oacute;"
#elif L==2	// de
	 "Sortieren nach letzten Ausgabe"
#elif L==3	// en
	 "Sort by date of last edition"
#elif L==4	// es
	 "Ordenar por fecha de &uacute;ltima edici&oacute;n"
#elif L==5	// fr
	 "Trier par date de derni&egrave;re &eacute;dition"
#elif L==6	// gn
	 "Ordenar por fecha de &uacute;ltima edici&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per data dell'ultima edizione"
#elif L==8	// pl
	 "Sortuj wed&lstrok;ug daty ostatniej edycji"
#elif L==9	// pt
	 "Classificar por data da &uacute;ltima edi&ccedil;&atilde;o"
#endif
	 ,
	 // Prj_ORDER_TITLE
#if   L==1	// ca
	 "Ordenar per t&iacute;tol"
#elif L==2	// de
	 "Sortieren nach Titel"
#elif L==3	// en
	 "Sort by title"
#elif L==4	// es
	 "Ordenar por t&iacute;tulo"
#elif L==5	// fr
	 "Trier par date de d&eacute;but"
#elif L==6	// gn
	 "Ordenar por t&iacute;tulo"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per titolo"
#elif L==8	// pl
	 "Sortuj wedlug tytu&lstrok;"
#elif L==9	// pt
	 "Classificar por t&iacute;tulo"
#endif
	 ,
	 // Prj_ORDER_DEPARTMENT
#if   L==1	// ca
	 "Ordenar per departament"
#elif L==2	// de
	 "Sortieren nach Abteilung"
#elif L==3	// en
	 "Sort by department"
#elif L==4	// es
	 "Ordenar por departamento"
#elif L==5	// fr
	 "Trier par d&eacute;partement"
#elif L==6	// gn
	 "Ordenar por departamento"	// Okoteve traduccin
#elif L==7	// it
	 "Ordina per dipartimento"
#elif L==8	// pl
	 "Sortuj wedlug wydzia&lstrok;"
#elif L==9	// pt
	 "Classificar por departamento"
#endif
	};

const char *Txt_Project_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Projecte <strong>%s</strong> eliminat.";
#elif L==2	// de
	"Projekt <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Project <strong>%s</strong> removed.";
#elif L==4	// es
	"Proyecto <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Projet <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Proyecto <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Progetto <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Projekt <strong>%s</strong> usuniety.";
#elif L==9	// pt
	"Projeto <strong>%s</strong> removido.";
#endif

const char *Txt_PROJECT_ROLES_SINGUL_Abc[Prj_NUM_ROLES_IN_PROJECT] =
	{
#if   L==1	// ca	// Prj_ROLE_UNK
	"Desconegut/da"
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido/a"
#elif L==5	// fr
	"Inconnu/e"
#elif L==6	// gn
	"Desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto/a"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido/a"
#endif
	,
#if   L==1	// ca	// Prj_ROLE_STD
	"Estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"Student"
#elif L==4	// es
	"Estudiante"
#elif L==5	// fr
	"&Eacute;tudiant/e"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studente"
#elif L==8	// pl
	"Student"
#elif L==9	// pt
	"Estudante"
#endif
	,
#if   L==1	// ca	// Prj_ROLE_TUT
	"Tutor/a"
#elif L==2	// de
	"Tutor"
#elif L==3	// en
	"Tutor"
#elif L==4	// es
	"Tutor/a"
#elif L==5	// fr
	"Tuteur"
#elif L==6	// gn
	"Tutor/a"	// Okoteve traduccin
#elif L==7	// it
	"Precettore"
#elif L==8	// pl
	"Nauczyciel"
#elif L==9	// pt
	"Tutor/a"
#endif
	,
#if   L==1	// ca	// Prj_ROLE_EVA
	"Avaluador/a"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"Evaluator"
#elif L==4	// es
	"Evaluador/a"
#elif L==5	// fr
	"&Eacute;valuateur/trice"
#elif L==6	// gn
	"Evaluador/a"	// Okoteve traduccin
#elif L==7	// it
	"Valutatore"
#elif L==8	// pl
	"Ewaluator"
#elif L==9	// pt
	"Avaliador/a"
#endif
	};

const char *Txt_PROJECT_ROLES_SINGUL_abc[Prj_NUM_ROLES_IN_PROJECT] =
	{
#if   L==1	// ca	// Prj_ROLE_UNK
	"desconegut/da"
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocido/a"
#elif L==5	// fr
	"inconnu/e"
#elif L==6	// gn
	"desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuto/a"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecido/a"
#endif
	,
#if   L==1	// ca	// Prj_ROLE_STD
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiant/e"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente"
#elif L==8	// pl
	"student"
#elif L==9	// pt
	"estudante"
#endif
	,
#if   L==1	// ca	// Prj_ROLE_TUT
	"tutor/a"
#elif L==2	// de
	"Tutor"
#elif L==3	// en
	"tutor"
#elif L==4	// es
	"tutor/a"
#elif L==5	// fr
	"tuteur"
#elif L==6	// gn
	"tutor/a"	// Okoteve traduccin
#elif L==7	// it
	"precettore"
#elif L==8	// pl
	"nauczyciel"
#elif L==9	// pt
	"tutor/a"
#endif
	,
#if   L==1	// ca	// Prj_ROLE_EVA
	"avaluador/a"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"evaluator"
#elif L==4	// es
	"evaluador/a"
#elif L==5	// fr
	"&eacute;valuateur"
#elif L==6	// gn
	"evaluador/a"	// Okoteve traduccin
#elif L==7	// it
	"valutatore"
#elif L==8	// pl
	"ewaluator"
#elif L==9	// pt
	"avaliador/a"
#endif
	};

const char *Txt_PROJECT_ROLES_PLURAL_Abc[Prj_NUM_ROLES_IN_PROJECT] =
	{
#if   L==1	// ca	// Prj_ROLE_UNK
	"Desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidos"
#elif L==5	// fr
	"Inconnus"
#elif L==6	// gn
	"Desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecidos"
#endif
	,
#if   L==1	// ca	// Prj_ROLE_STD
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiants"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studenti"
#elif L==8	// pl
	"Studenci"
#elif L==9	// pt
	"Estudantes"
#endif
	,
#if   L==1	// ca	// Prj_ROLE_TUT
	"Tutors"
#elif L==2	// de
	"Tutoren"
#elif L==3	// en
	"Tutors"
#elif L==4	// es
	"Tutores"
#elif L==5	// fr
	"Tuteurs"
#elif L==6	// gn
	"Tutores"	// Okoteve traduccin
#elif L==7	// it
	"Precettori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Tutores"
#endif
	,
#if   L==1	// ca	// Prj_ROLE_EVA
	"Avaluadors"
#elif L==2	// de
	"Auswerter"
#elif L==3	// en
	"Evaluators"
#elif L==4	// es
	"Evaluadores"
#elif L==5	// fr
	"&Eacute;valuateurs"
#elif L==6	// gn
	"Evaluadores"	// Okoteve traduccin
#elif L==7	// it
	"Valutatori"
#elif L==8	// pl
	"Ewaluator&oacute;w"
#elif L==9	// pt
	"Avaliadores"
#endif
	};

const char *Txt_PROJECT_STATUS[Prj_NUM_PROPOSAL_TYPES] =
	{
#if   L==1	// ca	// Prj_PROPOSAL_NEW
	"Projecte nou"
#elif L==2	// de
	"Neues Projekt"
#elif L==3	// en
	"New project"
#elif L==4	// es
	"Proyecto nuevo"
#elif L==5	// fr
	"Projet nouveau"
#elif L==6	// gn
	"Proyecto nuevo"		// Okoteve traduccin
#elif L==7	// it
	"Progetto nuovo"
#elif L==8	// pl
	"Nowy projekt"
#elif L==9	// pt
	"Projeto novo"
#endif
	,
#if   L==1	// ca	// Prj_PROPOSAL_MODIFIED
	"Projecte ja proposat en convocat&ograve;ries anteriors, amb modificacions"
#elif L==2	// de
	"Projekt in fr&uuml;heren Anrufen vorgeschlagen, mit &Auml;nderungen"
#elif L==3	// en
	"Project proposed in previous calls, with modifications"
#elif L==4	// es
	"Proyecto propuesto en convocatorias anteriores, con modificaciones"
#elif L==5	// fr
	"Projet propos&eacute; dans les appels pr&eacute;c&eacute;dents, avec modifications"
#elif L==6	// gn
	"Proyecto propuesto en convocatorias anteriores, con modificaciones"	// Okoteve traduccin
#elif L==7	// it
	"Progetto proposto nelle chiamate precedenti, con modifiche"
#elif L==8	// pl
	"Proponowany projekt we wcze&sacute;niejszych rozmowach z modyfikacjami"
#elif L==9	// pt
	"Projeto proposto em chamadas anteriores, com modifica&ccedil;&otilde;es"
#endif
	,
#if   L==1	// ca	// Prj_PROPOSAL_UNMODIFIED
	"Projecte ja proposat en convocat&ograve;ries anteriors, sense modificacions"
#elif L==2	// de
	"Projekt in fr&uuml;heren Anrufen vorgeschlagen, ohne &Auml;nderungen"
#elif L==3	// en
	"Project proposed in previous calls, without modifications"
#elif L==4	// es
	"Proyecto propuesto en convocatorias anteriores, sin modificaciones"
#elif L==5	// fr
	"Projet propos&eacute; dans les appels pr&eacute;c&eacute;dents, sans modifications"
#elif L==6	// gn
	"Proyecto propuesto en convocatorias anteriores, sin modificaciones"	// Okoteve traduccin
#elif L==7	// it
	"Progetto proposto nelle chiamate precedenti, senza modifiche"
#elif L==8	// pl
	"Proponowany projekt we wcze&sacute;niejszych rozmowach bez modyfikacji"
#elif L==9	// pt
	"Projeto proposto em chamadas anteriores, sem modifica&ccedil;&otilde;es"
#endif
	};

const char *Txt_PROJECT_MY_ALL_PROJECTS[Prj_NUM_WHOSE_PROJECTS] =
	{
	 // Prj_MY__PROJECTS
#if   L==1	// ca
	"Nom&eacute;s els meus projectes"
#elif L==2	// de
	"Nur meine Projekte"
#elif L==3	// en
	"Only my projects"
#elif L==4	// es
	"Solo mis proyectos"
#elif L==5	// fr
	"Uniquement mes projets"
#elif L==6	// gn
	"Solo mis proyectos"	// Okoteve traduccin
#elif L==7	// it
	"Solo i miei progetti"
#elif L==8	// pl
	"Tylko moje projekty"
#elif L==9	// pt
	"Apenas os meus projetos"
#endif
	,
	// Prj_ALL_PROJECTS
#if   L==1	// ca
	"Tots els projectes"
#elif L==2	// de
	"Alle Projekte"
#elif L==3	// en
	"All projects"
#elif L==4	// es
	"Todos los proyectos"
#elif L==5	// fr
	"Tous les projets"
#elif L==6	// gn
	"Todos los proyectos"	// Okoteve traduccin
#elif L==7	// it
	"Tutti i progetti"
#elif L==8	// pl
	"Wszystkie projekty"
#elif L==9	// pt
	"Todos os projetos"
#endif
	};

const char *Txt_PROJECT_PREASSIGNED_NONPREASSIGNED_PLURAL[Prj_NUM_PREASSIGNED_NONPREASSIG] =
	{
	 // Prj_PREASSIGNED
#if   L==1	// ca
	"Projectes preassignats"
#elif L==2	// de
	"Vorbelegt Projekte"
#elif L==3	// en
	"Preassigned projects"
#elif L==4	// es
	"Proyectos preasignados"
#elif L==5	// fr
	"Projets pr&eacute;assign&eacute;s"
#elif L==6	// gn
	"Proyectos preasignados"	// Okoteve traduccin
#elif L==7	// it
	"Progetti preassegnati"
#elif L==8	// pl
	"Zadany projekty"
#elif L==9	// pt
	"Projetos pr&eacute;-atribu&iacute;dos"
#endif
	,
	// Prj_NONPREASSIG
#if   L==1	// ca
	"Projectes no preassignats"
#elif L==2	// de
	"Nicht vorbelegt Projekte"
#elif L==3	// en
	"Non-preassigned projects"
#elif L==4	// es
	"Proyectos no preasignados"
#elif L==5	// fr
	"Projets non pr&eacute;assign&eacute;s"
#elif L==6	// gn
	"Proyectos no preasignados"	// Okoteve traduccin
#elif L==7	// it
	"Progetti non-preassegnati"
#elif L==8	// pl
	"Nieprzypisane projekty"
#elif L==9	// pt
	"Projetos n&atilde;o pr&eacute;-atribu&iacute;dos"
#endif
	};

const char *Txt_PROJECT_PREASSIGNED_NONPREASSIGNED_SINGUL[Prj_NUM_PREASSIGNED_NONPREASSIG] =
	{
	 // Prj_PREASSIGNED
#if   L==1	// ca
	"Projecte preassignat"
#elif L==2	// de
	"Vorbelegt Projekt"
#elif L==3	// en
	"Preassigned project"
#elif L==4	// es
	"Proyecto preasignado"
#elif L==5	// fr
	"Projet pr&eacute;assign&eacute;"
#elif L==6	// gn
	"Proyecto preasignado"	// Okoteve traduccin
#elif L==7	// it
	"Progetto preassegnato"
#elif L==8	// pl
	"Zadany projekt"
#elif L==9	// pt
	"Projeto pr&eacute;-atribu&iacute;do"
#endif
	,
	// Prj_NONPREASSIG
#if   L==1	// ca
	"Projecte no preassignat"
#elif L==2	// de
	"Nicht vorbelegt Projekt"
#elif L==3	// en
	"Non-preassigned project"
#elif L==4	// es
	"Proyecto no preasignado"
#elif L==5	// fr
	"Projet non pr&eacute;assign&eacute;"
#elif L==6	// gn
	"Proyecto no preasignado"	// Okoteve traduccin
#elif L==7	// it
	"Progetto non-preassegnato"
#elif L==8	// pl
	"Nieprzypisane projekt"
#elif L==9	// pt
	"Projeto n&atilde;o pr&eacute;-atribu&iacute;do"
#endif
	};

const char *Txt_PROJECT_HIDDEN_VISIBL_PROJECTS[Prj_NUM_HIDDEN_VISIBL] =
	{
	// Prj_HIDDEN
#if   L==1	// ca
	"Projectes ocults"
#elif L==2	// de
	"Unsichtbare Projekte"
#elif L==3	// en
	"Hidden projects"
#elif L==4	// es
	"Proyectos ocultos"
#elif L==5	// fr
	"Projets cach&eacute;s"
#elif L==6	// gn
	"Proyectos ocultos"	// Okoteve traduccin
#elif L==7	// it
	"Progetti nascosti"
#elif L==8	// pl
	"Ukryte projekty"
#elif L==9	// pt
	"Projetos ocultos"
#endif
	,
	// Prj_VISIBL
#if   L==1	// ca
	"Projectes visibles"
#elif L==2	// de
	"Sichtbar Projekte"
#elif L==3	// en
	"Visible projects"
#elif L==4	// es
	"Proyectos visibles"
#elif L==5	// fr
	"Projets visibles"
#elif L==6	// gn
	"Proyectos visibles"	// Okoteve traduccin
#elif L==7	// it
	"Progetti visibili"
#elif L==8	// pl
	"Widoczne projekty"
#elif L==9	// pt
	"Projetos vis&iacute;veis"
#endif
	};

const char *Txt_Projects =
#if   L==1	// ca
	"Projectes (treballs fi de grau/m&agrave;ster, tesi&hellip;)";
#elif L==2	// de
	"Projekte (Abschlussarbeiten, Diplomarbeiten&hellip;)";
#elif L==3	// en
	"Projects (end-of-degree projects, thesis&hellip;)";
#elif L==4	// es
	"Proyectos (trabajos fin de grado/m&aacute;ster, tesis&hellip;)";
#elif L==5	// fr
	"Projets (projets de fin de cours, th&egrave;se&hellip;)";
#elif L==6	// gn
	"Proyectos (trabajos fin de grado/m&aacute;ster, tesis&hellip;)";	// Okoteve traduccin
#elif L==7	// it
	"Progetti (progetti di fine corso, tesi&hellip;)";
#elif L==8	// pl
	"Projekty (projekty ko&nacute;cowe, praca dyplomowa&hellip;)";
#elif L==9	// pt
	"Projetos (projetos de fim de grau, tese&hellip;)";
#endif

const char *Txt_Proposal =
#if   L==1	// ca
	"Proposta";
#elif L==2	// de
	"Vorschlag";
#elif L==3	// en
	"Proposal";
#elif L==4	// es
	"Propuesta";
#elif L==5	// fr
	"Proposition";
#elif L==6	// gn
	"Propuesta";	// Okoteve traduccin
#elif L==7	// it
	"Proposta";
#elif L==8	// pl
	"Wniosek";
#elif L==9	// pt
	"Proposta";
#endif

const char *Txt_Public_agenda_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Agenda p&uacute;blica de %s";
#elif L==2	// de
	"&Ouml;ffentliche Organizer von %s";
#elif L==3	// en
	"%s's public agenda";
#elif L==4	// es
	"Agenda p&uacute;blica de %s";
#elif L==5	// fr
	"Agenda publique de %s";
#elif L==6	// gn
	"Agenda p&uacute;blica de %s";	// Okoteve traduccin
#elif L==7	// it
	"Agenda pubblica di %s";
#elif L==8	// pl
	"Pami&eogon;tnik publiczny %s";
#elif L==9	// pt
	"Agenda p&uacute;blica de %s";
#endif

const char *Txt_public_FILES =
#if   L==1	// ca
	"p&uacute;blics";
#elif L==2	// de
	"&ouml;ffentliche";
#elif L==3	// en
	"public";
#elif L==4	// es
	"p&uacute;blicos";
#elif L==5	// fr
	"publics";
#elif L==6	// gn
	"p&uacute;blicos";	// Okoteve traduccin
#elif L==7	// it
	"pubblici";
#elif L==8	// pl
	"publicznych";
#elif L==9	// pt
	"p&uacute;blicos";
#endif

const char *Txt_Public_open_educational_resource_OER_for_everyone =
#if   L==1	// ca
	"P&uacute;blic, recurs educatiu obert (OER) per a tothom";
#elif L==2	// de
	"&Ouml;ffentliche, Open Educational Resource (OER) f&uuml;r jedermann";
#elif L==3	// en
	"Public, open educational resource for everyone";
#elif L==4	// es
	"P&uacute;blico, recurso educativo abierto (OER) para todo el mundo";
#elif L==5	// fr
	"Public, ressource &eacute;ducatif libre (REL) pour tout le monde";
#elif L==6	// gn
	"P&uacute;blico, recurso educativo abierto (OER) para todo el mundo";	// Okoteve traduccin
#elif L==7	// it
	"Pubblico, risorse didattici aperto (OER) per tutti";
#elif L==8	// pl
	"Publiczne, otwarte zasoby edukacyjne (OER) dla ka&zdot;dego";
#elif L==9	// pt
	"P&uacute;blico, recurso educacional aberto (OER) para todos";
#endif

const char *Txt_Public_views =
#if   L==1	// ca
	"Accessos p&uacute;blics";
#elif L==2	// de
	"&Ouml;ffentliche Ansichten";
#elif L==3	// en
	"Public views";
#elif L==4	// es
	"Accesos p&uacute;blicos";
#elif L==5	// fr
	"Vues publics";
#elif L==6	// gn
	"Accesos p&uacute;blicos";	// Okoteve traduccin
#elif L==7	// it
	"Visualizzazioni pubblici";
#elif L==8	// pl
	"Publiczne ods&lstrok;on";
#elif L==9	// pt
	"Acessos p&uacute;blicos";
#endif

const char *Txt_Publish_announcement_OF_EXAM =
#if   L==1	// ca
	"Publicar convocat&ograve;ria";
#elif L==2	// de
	"Ver&ouml;ffentlichen Aufruf";
#elif L==3	// en
	"Publish announcement";
#elif L==4	// es
	"Publicar convocatoria";
#elif L==5	// fr
	"Publier convocation";
#elif L==6	// gn
	"Publicar convocatoria";	// Okoteve traduccin
#elif L==7	// it
	"Pubblicare sessione";
#elif L==8	// pl
	"Publikuje zaproszenie";
#elif L==9	// pt
	"Publicar chamada";
#endif

const char *Txt_QR_code =
#if   L==1	// ca
	"Codi QR";
#elif L==2	// de
	"QR-Code";
#elif L==3	// en
	"QR code";
#elif L==4	// es
	"C&oacute;digo QR";
#elif L==5	// fr
	"QR Code";
#elif L==6	// gn
	"C&oacute;digo QR";	// Okoteve traduccin
#elif L==7	// it
	"Codice QR";
#elif L==8	// pl
	"QR Code";
#elif L==9	// pt
	"C&oacute;digo QR";
#endif

const char *Txt_Question =
#if   L==1	// ca
	"Pregunta";	// Necessita traduccio
#elif L==2	// de
	"Frage";
#elif L==3	// en
	"Question";
#elif L==4	// es
	"Pregunta";
#elif L==5	// fr
	"Question";
#elif L==6	// gn
	"Pregunta";	// Okoteve traduccin
#elif L==7	// it
	"Domanda";
#elif L==8	// pl
	"Pytanie";
#elif L==9	// pt
	"Quest&atilde;o";
#endif

const char *Txt_Question_code_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Pregunta amb codi %ld";
#elif L==2	// de
	"Frage Code %ld";
#elif L==3	// en
	"Code %ld";
#elif L==4	// es
	"Pregunta con c&oacute;digo %ld";
#elif L==5	// fr
	"Code de question %ld";
#elif L==6	// gn
	"Pregunta con c&oacute;digo %ld";	// Okoteve traduccin
#elif L==7	// it
	"Codice di domanda %ld";
#elif L==8	// pl
	"Kodeks pytanie %ld";
#elif L==9	// pt
	"Quest&atilde;o com c&oacute;digo %ld";
#endif

const char *Txt_Question_modified =
#if   L==1	// ca
	"Pregunta modificada.";
#elif L==2	// de
	"Frage modifiziert.";
#elif L==3	// en
	"Question modified.";
#elif L==4	// es
	"Pregunta modificada.";
#elif L==5	// fr
	"Question modifi&eacute;e.";
#elif L==6	// gn
	"Pregunta modificada.";		// Okoteve traduccin
#elif L==7	// it
	"Domanda modificata.";
#elif L==8	// pl
	"Pytanie zmodyfikowane.";
#elif L==9	// pt
	"Quest&atilde;o modificada.";
#endif

const char *Txt_Question_removed =
#if   L==1	// ca
	"Pregunta eliminada.";
#elif L==2	// de
	"Frage entfernt.";
#elif L==3	// en
	"Question removed.";
#elif L==4	// es
	"Pregunta eliminada.";
#elif L==5	// fr
	"Question supprim&eacute;e.";
#elif L==6	// gn
	"Pregunta eliminada.";		// Okoteve traduccin
#elif L==7	// it
	"Domanda rimossa.";
#elif L==8	// pl
	"Pytanie usuniete.";
#elif L==9	// pt
	"Quest&atilde;o removioda.";
#endif

const char *Txt_Questions =
#if   L==1	// ca
	"Preguntes";
#elif L==2	// de
	"Fragen";
#elif L==3	// en
	"Questions";
#elif L==4	// es
	"Preguntas";
#elif L==5	// fr
	"Questions";
#elif L==6	// gn
	"Preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Domande";
#elif L==8	// pl
	"Pyta&nacute;";
#elif L==9	// pt
	"Quest&otilde;es";
#endif

const char *Txt_Questions_and_problems =
#if   L==1	// ca
	"Consultes i problemes";
#elif L==2	// de
	"Fragen und Probleme";
#elif L==3	// en
	"Questions and problems";
#elif L==4	// es
	"Consultas y problemas";
#elif L==5	// fr
	"Questions et probl&egrave;mes";
#elif L==6	// gn
	"Consultas y problemas";		// Okoteve traduccin
#elif L==7	// it
	"Domande e problemi";
#elif L==8	// pl
	"Pytania i problemy";
#elif L==9	// pt
	"Quest&otilde;es e problemas";
#endif

const char *Txt_Quota_exceeded =
#if   L==1	// ca
	"&iexcl;Cuota excedida!";	// Necessita traduccio
#elif L==2	// de
	"Kontingent &uuml;berschritten!";
#elif L==3	// en
	"Quota exceeded!";
#elif L==4	// es
	"&iexcl;Cuota excedida!";
#elif L==5	// fr
	"Quota d&eacute;pass&eacute;!";
#elif L==6	// gn
	"&iexcl;Cuota excedida!";	// Okoteve traduccin
#elif L==7	// it
	"Quota superata!";
#elif L==8	// pl
	"Przekroczono przydzial!";
#elif L==9	// pt
	"Quota excedida!";
#endif

const char *Txt_ReaBRders =
#if   L==1	// ca
	"Lec-<br />tores";	// Necessita traduccio
#elif L==2	// de
	"Leser";
#elif L==3	// en
	"Rea-<br />ders";
#elif L==4	// es
	"Lec-<br />tores";
#elif L==5	// fr
	"Lec-<br />teurs";
#elif L==6	// gn
	"Lec-<br />tores";	// Okoteve traduccin
#elif L==7	// it
	"Le-<br />ttori";
#elif L==8	// pl
	"Czytel-<br />nik&oacute;w";
#elif L==9	// pt
	"Lei-<br />tores";
#endif

const char *Txt_Real_number_between_A_and_B_1 =
#if   L==1	// ca
	"Nombre real entre";
#elif L==2	// de
	"Reelle Zahl zwischen";
#elif L==3	// en
	"Real number between";
#elif L==4	// es
	"N&ordm; real entre";
#elif L==5	// fr
	"Nombre r&eacute;el entre";
#elif L==6	// gn
	"N&ordm; real entre";	// Okoteve traduccin
#elif L==7	// it
	"Numero reale compreso tra";
#elif L==8	// pl
	"Prawdziwa liczbe miedzy";
#elif L==9	// pt
	"N&ordm; real entre";
#endif

const char *Txt_Real_number_between_A_and_B_2 =	// Must start by space
#if   L==1	// ca
	" i";
#elif L==2	// de
	" und";
#elif L==3	// en
	" and";
#elif L==4	// es
	" y";
#elif L==5	// fr
	" et";
#elif L==6	// gn
	" y";	// Okoteve traduccin
#elif L==7	// it
	" e";
#elif L==8	// pl
	" i";
#elif L==9	// pt
	" e";
#endif

const char *Txt_Recommended_aspect_ratio =
#if   L==1	// ca
	"Relaci&oacute; d'aspecte recomanada";
#elif L==2	// de
	"Empfohlene Seitenverh&auml;ltnisr";
#elif L==3	// en
	"Recommended aspect ratio";
#elif L==4	// es
	"Relaci&oacute;n de aspecto recomendada";
#elif L==5	// fr
	"Rapport d'aspect recommand&eacute;";
#elif L==6	// gn
	"Relaci&oacute;n de aspecto recomendada";	// Okoteve traduccin
#elif L==7	// it
	"Rapporto aspetto consigliato";
#elif L==8	// pl
	"Zalecane proporcje obrazu";
#elif L==9	// pt
	"Propor&ccedil;&atilde;o recomendada";
#endif

const char *Txt_Recommended_resolution =
#if   L==1	// ca
	"Resoluci&oacute; recomanada";
#elif L==2	// de
	"Empfohlene Aufl&ouml;sung";
#elif L==3	// en
	"Recommended resolution";
#elif L==4	// es
	"Resoluci&oacute;n recomendada";
#elif L==5	// fr
	"R&eacute;solution recommand&eacute;e";
#elif L==6	// gn
	"Resoluci&oacute;n recomendada";	// Okoteve traduccin
#elif L==7	// it
	"Risoluzione consigliata";
#elif L==8	// pl
	"Zalecana rozdzielczo&sacute;&cacute;";
#elif L==9	// pt
	"Resolu&ccedil;&atilde;o recomendada";
#endif

const char *Txt_Record_card_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Se ha eliminado la ficha de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Record card of <strong>%s</strong> has been removed.";	// Need bersetzung
#elif L==3	// en
	"Record card of <strong>%s</strong> has been removed.";
#elif L==4	// es
	"Se ha eliminado la ficha de <strong>%s</strong>.";
#elif L==5	// fr
	"Record card of <strong>%s</strong> has been removed.";	// Besoin de traduction
#elif L==6	// gn
	"Se ha eliminado la ficha de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La scheda de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==8	// pl
	"Karta Rekord <strong>%s</strong> zostala usunieta.";
#elif L==9	// pt
	"Cart&atilde;o de registro de <strong>%s</strong> foi removido.";
#endif

const char *Txt_record_cards_per_page =
#if   L==1	// ca
	"fichas por p&aacute;gina";		// Necessita traduccio
#elif L==2	// de
	"record cards per page";		// Need bersetzung
#elif L==3	// en
	"record cards per page";
#elif L==4	// es
	"fichas por p&aacute;gina";
#elif L==5	// fr
	"record cards per page";		// Besoin de traduction
#elif L==6	// gn
	"fichas por p&aacute;gina";		// Okoteve traduccin
#elif L==7	// it
	"scheda per pagina";
#elif L==8	// pl
	"karty zapisu na stronie";
#elif L==9	// pt
	"cart&otilde;es por p&aacute;gina";
#endif

const char *Txt_Record_field_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Campo de ficha <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2	// de
	"Datenfeld <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Record field <strong>%s</strong> removed.";
#elif L==4	// es
	"Campo de ficha <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Record field <strong>%s</strong> removed.";	// Besoin de traduction
#elif L==6	// gn
	"Campo de ficha <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Campo scheda <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Pole rekordu <strong>%s</strong> usuniete.";
#elif L==9	// pt
	"Campo de cart&atilde;o <strong>%s</strong> removido.";
#endif

const char *Txt_RECORD_FIELD_VISIBILITY_MENU[Rec_NUM_TYPES_VISIBILITY] =
   {
#if   L==1	// ca
	 "Oculto"	// Necessita traduccio
#elif L==2	// de
	 "Versteckt"
#elif L==3	// en
	 "Hidden"
#elif L==4	// es
	 "Oculto"
#elif L==5	// fr
	 "Cach&eacute;"
#elif L==6	// gn
	 "Oculto"	// Okoteve traduccin
#elif L==7	// it
	 "Nascosto"
#elif L==8	// pl
	 "Ukryty"
#elif L==9	// pt
	 "Oculto"
#endif
	 ,
#if   L==1	// ca
	 "Visible pero no editable"	// Necessita traduccio
#elif L==2	// de
	 "Sichtbar, aber nicht bearbeitet werden"
#elif L==3	// en
	 "Visible but not editable"
#elif L==4	// es
	 "Visible pero no editable"
#elif L==5	// fr
	 "Visible mais non modifiable"
#elif L==6	// gn
	 "Visible pero no editable"	// Okoteve traduccin
#elif L==7	// it
	 "Visibile ma non modificabile"
#elif L==8	// pl
	 "Widoczne, ale nie mozna edytowa&cacute;"
#elif L==9	// pt
	 "Vis&iacute;vel, mas n&atilde;o edit&aacute;vel"
#endif
	 ,
#if   L==1	// ca
	 "Visible y editable"	// Necessita traduccio
#elif L==2	// de
	 "Sichtbar und bearbeitet werde"
#elif L==3	// en
	 "Visible and editable"
#elif L==4	// es
	 "Visible y editable"
#elif L==5	// fr
	 "Visible et modifiable"
#elif L==6	// gn
	 "Visible y editable"	// Okoteve traduccin
#elif L==7	// it
	 "Visibile e modificabile"
#elif L==8	// pl
	 "Widzialne i mozna edytowa&cacute;"
#elif L==9	// pt
	 "Vis&iacute;vel e edit&aacute;vel"
#endif
	};

const char *Txt_RECORD_FIELD_VISIBILITY_MSG[Rec_NUM_TYPES_VISIBILITY] =	// Warning: it is very important to include %s in the following sentences
	{
#if   L==1	// ca
	"A partir de ahora los estudiantes"
	" no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."// Necessita traduccio
#elif L==2	// de
	"Von nun an k&ouml;nnen die Studenten"
	" nicht das Feld <strong>%s</strong> der Profile."
#elif L==3	// en
	"From now on students"
	" can not see the field <strong>%s</strong> of the records."
#elif L==4	// es
	"A partir de ahora los estudiantes"
	" no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."
#elif L==5	// fr
	"A partir de maintenant, les &eacute;tudiants"
	" ne peuvent pas voir le champ <strong>%s</strong> de les cartes."
#elif L==6	// gn
	"A partir de ahora los estudiantes"
	" no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==7	// it
	"Da ora in poi gli studenti"
	" non possono visualizzare il campo <strong>%s</strong> delle schede."
#elif L==8	// pl
	"Od teraz studenci"
	" nie widac pola <strong>%s</strong> zapis&oacute;w."
#elif L==9	// pt
	"De agora em diante,"
	" os estudantes n&atilde;o podem ver o campo <strong>%s</strong> dos cart&otilde;es."
#endif
	,
#if   L==1	// ca
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."// Necessita traduccio
#elif L==2	// de
	"Von nun an k&ouml;nnen sich Studenten,"
	" aber nicht bearbeiten das Feld <strong>%s</strong> der Profile."
#elif L==3	// en
	"From now on students"
	" can view but not edit the field <strong>%s</strong> of the records."
#elif L==4	// es
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."
#elif L==5	// fr
	"A partir de maintenant, les &eacute;tudiants"
	" peuvent consulter mais pas modifier le champ <strong>%s</strong> de les cartes."
#elif L==6	// gn
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==7	// it
	"Da ora in poi gli studenti"
	" possono visualizzare ma non modificare il campo <strong>%s</strong> delle schede."
#elif L==8	// pl
	"Od teraz studenci"
	" moga przegladac, ale nie edytowa&cacute; pole <strong>%s</strong> zapis&oacute;w."
#elif L==9	// pt
	"De agora em diante,"
	" os estudantes podem visualizar, mas n&atilde;o editar o campo <strong>%s</strong> dos cart&otilde;es."
#endif
	,
#if   L==1	// ca
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."// Necessita traduccio
#elif L==2	// de
	"Von nun an k&ouml;nnen die Studenten"
	" anzuzeigen und zu bearbeiten das Feld <strong>%s</strong> der Profile."
#elif L==3	// en
	"From now on students"
	" can view and edit the field <strong>%s</strong> of the records."
#elif L==4	// es
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."
#elif L==5	// fr
	"A partir de maintenant,"
	" les &eacute;tudiants peuvent consulter et modifier le champ <strong>%s</strong> de les cartes."
#elif L==6	// gn
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==7	// it
	"Da ora in poi"
	" gli studenti possono visualizzare e modificare il campo <strong>%s</strong> delle schede."
#elif L==8	// pl
	"Od teraz studenci"
	" moga ogladac i edytowa&cacute; pola <strong>%s</strong> zapis&oacute;w."
#elif L==9	// pt
	"De agora em diante,"
	" os estudantes podem visualizar e editar o campo <strong>%s</strong> dos cart&otilde;es."
#endif
	};

const char *Txt_RECORD_FIELD_VISIBILITY_RECORD[Rec_NUM_TYPES_VISIBILITY] =
   {
#if   L==1	// ca
	 "oculto"	// Necessita traduccio
#elif L==2	// de
	 "versteckt"
#elif L==3	// en
	 "hidden"
#elif L==4	// es
	 "oculto"
#elif L==5	// fr
	 "cach&eacute;"
#elif L==6	// gn
	 "oculto"	// Okoteve traduccin
#elif L==7	// it
	 "nascosto"
#elif L==8	// pl
	 "ukryty"
#elif L==9	// pt
	 "oculto"
#endif
	 ,
#if   L==1	// ca
	 "visible"	// Necessita traduccio
#elif L==2	// de
	 "sichtbar"
#elif L==3	// en
	 "visible"
#elif L==4	// es
	 "visible"
#elif L==5	// fr
	 "visible"
#elif L==6	// gn
	 "visible"	// Okoteve traduccin
#elif L==7	// it
	 "visibile"
#elif L==8	// pl
	 "widoczny"
#elif L==9	// pt
	 "vis&iacute;vel"
#endif
	 ,
#if   L==1	// ca
	 "editable"	// Necessita traduccio
#elif L==2	// de
	 "bearbeitet werde"
#elif L==3	// en
	 "editable"
#elif L==4	// es
	 "editable"
#elif L==5	// fr
	 "modifiable"
#elif L==6	// gn
	 "editable"	// Okoteve traduccin
#elif L==7	// it
	 "modificabile"
#elif L==8	// pl
	 "edytowalne"
#elif L==9	// pt
	 "edit&aacute;vel"
#endif
	};

const char *Txt_Record_fields =
#if   L==1	// ca
	"Camps de fitxes";
#elif L==2	// de
	"Record fields";	// Need bersetzung
#elif L==3	// en
	"Record fields";
#elif L==4	// es
	"Campos de fichas";
#elif L==5	// fr
	"Record fields";	// Besoin de traduction
#elif L==6	// gn
	"Campos de fichas";	// Okoteve traduccin
#elif L==7	// it
	"Campo schede";
#elif L==8	// pl
	"Rekord w dziedzinie";
#elif L==9	// pt
	"Campos de cart&otilde;es";
#endif

const char *Txt_Recipients =
#if   L==1	// ca
	"Destinatarios";	// Necessita traduccio
#elif L==2	// de
	"Empf&auml;nger";
#elif L==3	// en
	"Recipients";
#elif L==4	// es
	"Destinatarios";
#elif L==5	// fr
	"Destinataires";
#elif L==6	// gn
	"Destinatarios";	// Okoteve traduccin
#elif L==7	// it
	"Destinatari";
#elif L==8	// pl
	"Odbiorcy";
#elif L==9	// pt
	"Destinat&aacute;rios";
#endif

const char *Txt_Register =
#if   L==1	// ca
	"Inscriure";
#elif L==2	// de
	"Anmeldung";
#elif L==3	// en
	"Register";
#elif L==4	// es
	"Inscribir";
#elif L==5	// fr
	"Inscrire";
#elif L==6	// gn
	"Inscribir";	// Okoteve traduccin
#elif L==7	// it
	"Registrare";
#elif L==8	// pl
	"Zarejestruj sie";
#elif L==9	// pt
	"Registrar";
#endif

const char *Txt_Register_students =
#if   L==1	// ca
	"Inscriure estudiants";
#elif L==2	// de
	"Anmeldung Studenten";
#elif L==3	// en
	"Register students";
#elif L==4	// es
	"Inscribir estudiantes";
#elif L==5	// fr
	"Inscrire &eacute;tudiants";
#elif L==6	// gn
	"Inscribir estudiantes";	// Okoteve traduccin
#elif L==7	// it
	"Registrare studenti";
#elif L==8	// pl
	"Rejestracja student&oacute;w";
#elif L==9	// pt
	"Registrar estudantes";
#endif

const char *Txt_Register_students_in_COURSE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriure estudiants en <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung Studenten im <strong>%s</strong>";
#elif L==3	// en
	"Register students in <strong>%s</strong>";
#elif L==4	// es
	"Inscribir estudiantes en <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire &eacute;tudiants dans <strong>%s</strong>";
#elif L==6	// gn
	"Inscribir estudiantes en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrare studenti in <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja student&oacute;w w <strong>%s</strong>";
#elif L==9	// pt
	"Registrar estudantes em <strong>%s</strong>";
#endif

const char *Txt_Register_the_users_indicated_in_step_1 =
#if   L==1	// ca
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";	// Necessita traduccio
#elif L==2	// de
	"Anmeldung Sie die Benutzer, die auf der Liste";
#elif L==3	// en
	"Add (register the users indicated in step 1)";
#elif L==4	// es
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";
#elif L==5	// fr
	"Inscrire les utilisateurs qui sont sur la liste";
#elif L==6	// gn
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";	// Okoteve traduccin
#elif L==7	// it
	"Registrare gli utenti che sono in lista";
#elif L==8	// pl
	"Rejestracja uzytkownik&oacute;w, kt&oacute;rzy sa na liscie";
#elif L==9	// pt
	"Registrar os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Register_the_users_indicated_in_step_1_and_remove_the_users_not_indicated =
#if   L==1	// ca
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";	// Necessita traduccio
#elif L==2	// de
	"Entfernen Sie die Benutzer, die <strong>nicht</strong> auf der Liste,"
	" und Anmeldung Sie die Benutzer, die auf der Liste";
#elif L==3	// en
	"Update (register the users indicated in step 1"
	" and remove the users <strong>not</strong> indicated)";
#elif L==4	// es
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";
#elif L==5	// fr
	"Enlever les utilisateurs qui <strong>ne</strong> sont <strong>pas</strong> sur la liste,"
	" et inscrire les utilisateurs qui sont sur la liste";
#elif L==6	// gn
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere gli utenti che <strong>non</strong> sono sulla lista,"
	" e registrare gli utenti che invece lo sono";
#elif L==8	// pl
	"Usun u&zdot;ytkownikami, kt&oacute;rzy sa <strong>nie</strong> na liscie,"
	"i  zarejestrowac uzytkownik&oacute;w, kt&oacute;rzy sa na liscie";
#elif L==9	// pt
	"Tirar os utilizadores que <strong>n&atilde;o</strong> est&atilde;o na lista,"
	" e registrar os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Register_teacher =
#if   L==1	// ca
	"Inscriure professor/a";
#elif L==2	// de
	"Anmeldung Lehrkraft";
#elif L==3	// en
	"Register teacher";
#elif L==4	// es
	"Inscribir profesor/a";
#elif L==5	// fr
	"Inscrire enseignant/e";
#elif L==6	// gn
	"Inscribir profesor/a";	// Okoteve traduccin
#elif L==7	// it
	"Registrare professore/ssa";
#elif L==8	// pl
	"Rejestracja nauczyciel";
#elif L==9	// pt
	"Registrar professor/a";
#endif

const char *Txt_Register_user_IN_A_COURSE_OR_DEGREE =
#if   L==1	// ca
	"Inscriure usuari/a";
#elif L==2	// de
	"Anmeldung Benutzer";
#elif L==3	// en
	"Register user";
#elif L==4	// es
	"Inscribir usuario/a";
#elif L==5	// fr
	"Inscrire utilisateur/se";
#elif L==6	// gn
	"Inscribir usuario/a";	// Okoteve traduccin
#elif L==7	// it
	"Registrare utente";
#elif L==8	// pl
	"Rejestracja u&zdot;ytkownika";
#elif L==9	// pt
	"Registrar utilizador/a";
#endif

const char *Txt_Register_me_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriureme en <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung Sie mir im <strong>%s</strong>";
#elif L==3	// en
	"Register me in <strong>%s</strong>";
#elif L==4	// es
	"Inscribirme en <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire-moi dans <strong>%s</strong>";
#elif L==6	// gn
	"Inscribirme en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrami in <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja mnie w <strong>%s</strong>";
#elif L==9	// pt
	"Registrar me em <strong>%s</strong>";
#endif

const char *Txt_Register_USER_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriure en l'assignatura <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung im Kurs <strong>%s</strong>";
#elif L==3	// en
	"Register in the course <strong>%s</strong>";
#elif L==4	// es
	"Inscribir en la asignatura <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire dans la mati&egrave;re <strong>%s</strong>";
#elif L==6	// gn
	"Inscribir en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrare nel corso <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja w kurs <strong>%s</strong>";
#elif L==9	// pt
	"Registrar na disciplina <strong>%s</strong>";
#endif

const char *Txt_Register_USER_as_an_administrator_of_the_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriure com a administrador/a"
	" del centre <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung als Administrator"
	" des Lehrinstitut <strong>%s</strong>";
#elif L==3	// en
	"Register as an administrator"
	" of the centre <strong>%s</strong>";
#elif L==4	// es
	"Inscribir como administrador/a"
	" del centro <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire en tant qu'administrateur"
	" du centre <strong>%s</strong>";
#elif L==6	// gn
	"Inscribir como administrador/a"
	" del centro <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrare come amministratore"
	" del centro <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja jako administrator"
	" centrum <strong>%s</strong>";
#elif L==9	// pt
	"Registrar como administrador"
	" do centro <strong>%s</strong>";
#endif

const char *Txt_Register_USER_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriure com a administrador/a"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==3	// en
	"Register as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==4	// es
	"Inscribir como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==6	// gn
	"Inscribir como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrare come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==9	// pt
	"Registrar como administrador"
	" do grau <strong>%s</strong>";
#endif

const char *Txt_Register_USER_as_an_administrator_of_the_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Inscriure com a administrador/a"
	" de la instituci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Anmeldung als Administrator"
	" des Hochschule <strong>%s</strong>";
#elif L==3	// en
	"Register as an administrator"
	" of the institution <strong>%s</strong>";
#elif L==4	// es
	"Inscribir como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Inscrire en tant qu'administrateur"
	" du &eacute;tablissement <strong>%s</strong>";
#elif L==6	// gn
	"Inscribir como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Registrare come amministratore"
	" della istituzione <strong>%s</strong>";
#elif L==8	// pl
	"Rejestracja jako administrator"
	" instytucje <strong>%s</strong>";
#elif L==9	// pt
	"Registrar como administrador"
	" da institui&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Reject =
#if   L==1	// ca
	"Rebutjar";
#elif L==2	// de
	"Ablehnen";
#elif L==3	// en
	"Reject";
#elif L==4	// es
	"Rechazar";
#elif L==5	// fr
	"Rejeter";
#elif L==6	// gn
	"Rechazar";	// Okoteve traduccin
#elif L==7	// it
	"Rifiutare";
#elif L==8	// pl
	"Odrzuca&cacute;";
#elif L==9	// pt
	"Rejeitar";
#endif

const char *Txt_Removal_not_allowed =
#if   L==1	// ca
	"Eliminaci&oacute; no perm&egrave;s";
#elif L==2	// de
	"Entfernen nicht erlaubt";
#elif L==3	// en
	"Removal not allowed";
#elif L==4	// es
	"Eliminaci&oacute;n no permitida";
#elif L==5	// fr
	"Suppression pas autoris&eacute;";
#elif L==6	// gn
	"Eliminaci&oacute;n no permitida";	// Okoteve traduccin
#elif L==7	// it
	"Rimozione non ammessi";
#elif L==8	// pl
	"Usuwanie nie wolno";
#elif L==9	// pt
	"Remo&ccedil;&atilde;o n&atilde;o permitida";
#endif

const char *Txt_Remove =
#if   L==1	// ca
	"Eliminar";
#elif L==2	// de
	"Entfernen";
#elif L==3	// en
	"Remove";
#elif L==4	// es
	"Eliminar";
#elif L==5	// fr
	"Supprimer";
#elif L==6	// gn
	"Mbogue";
#elif L==7	// it
	"Rimuovere";
#elif L==8	// pl
	"Usu&nacute;";
#elif L==9	// pt
	"Remover";
#endif

const char *Txt_Remove_account =
#if   L==1	// ca
	"Eliminar compte";
#elif L==2	// de
	"Benutzerkonto entfernen";
#elif L==3	// en
	"Remove account";
#elif L==4	// es
	"Eliminar cuenta";
#elif L==5	// fr
	"Supprimer compte";
#elif L==6	// gn
	"Eliminar cuenta";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere account";
#elif L==8	// pl
	"Usu&nacute; konto";
#elif L==9	// pt
	"Remover conta";
#endif

const char *Txt_Remove_all_students =
#if   L==1	// ca
	"Eliminar tots estudiants";
#elif L==2	// de
	"Alle Studenten entfernen";
#elif L==3	// en
	"Remove all students";
#elif L==4	// es
	"Eliminar todos los estudiantes";
#elif L==5	// fr
	"Enlever tous &eacute;tudiants";
#elif L==6	// gn
	"Eliminar todos los estudiantes";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere tutti gli stud.";
#elif L==8	// pl
	"Usu&nacute; wszystkich student&oacute;w";
#elif L==9	// pt
	"Tirar todos estudantes";
#endif

const char *Txt_Remove_assignment =
#if   L==1	// ca
	"Eliminar activitat";
#elif L==2	// de
	"Aufgabe entfernen";
#elif L==3	// en
	"Remove assignment";
#elif L==4	// es
	"Eliminar actividad";
#elif L==5	// fr
	"Supprimer activit&eacute;";
#elif L==6	// gn
	"Eliminar actividad";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere attivit&agrave;";
#elif L==8	// pl
	"Usu&nacute; przypisanie";
#elif L==9	// pt
	"Remover atividade";
#endif

const char *Txt_Remove_event =
#if   L==1	// ca
	"Eliminar evento";	// Necessita traduccio
#elif L==2	// de
	"Ereignis Entfernen";
#elif L==3	// en
	"Remove event";
#elif L==4	// es
	"Eliminar evento";
#elif L==5	// fr
	"Supprimer &eacute;v&eacute;nement";
#elif L==6	// gn
	"Eliminar evento";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere evento";
#elif L==8	// pl
	"Usu&nacute; wydarzenie";
#elif L==9	// pt
	"Remover evento";
#endif

const char *Txt_Remove_file =
#if   L==1	// ca
	"Eliminar fitxer";
#elif L==2	// de
	"Datei Entfernen";
#elif L==3	// en
	"Remove file";
#elif L==4	// es
	"Eliminar archivo";
#elif L==5	// fr
	"Supprimer fichier";
#elif L==6	// gn
	"Eliminar archivo";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere file";
#elif L==8	// pl
	"Usu&nacute; plik";
#elif L==9	// pt
	"Remover arquivo";
#endif

const char *Txt_Remove_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar %s";
#elif L==2	// de
	"Datei %s";
#elif L==3	// en
	"Remove %s";
#elif L==4	// es
	"Eliminar %s";
#elif L==5	// fr
	"Supprimer %s";
#elif L==6	// gn
	"Eliminar %s";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere %s";
#elif L==8	// pl
	"Usu&nacute; %s";
#elif L==9	// pt
	"Remover %s";
#endif

const char *Txt_Remove_files_older_than_PART_1_OF_2 =
#if   L==1	// ca
	"Eliminar els arxius amb m&eacute;s de";
#elif L==2	// de
	"Dateien zu entfernen, die &auml;lter als";
#elif L==3	// en
	"Remove files older than";
#elif L==4	// es
	"Eliminar los archivos con m&aacute;s de";
#elif L==5	// fr
	"Supprimer les fichiers de plus de";
#elif L==6	// gn
	"Eliminar los archivos con m&aacute;s de";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere i file pi&ugrave; vecchi di";
#elif L==8	// pl
	"Remove files older than";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Remover arquivos com mais de";
#endif

const char *Txt_Remove_files_older_than_PART_2_OF_2 =
#if   L==1	// ca
	"mesos d'antiguitat.";
#elif L==2	// de
	"Monate.";
#elif L==3	// en
	"months.";
#elif L==4	// es
	"meses de antig&uuml;edad.";
#elif L==5	// fr
	"mois.";
#elif L==6	// gn
	"meses de antig&uuml;edad.";	// Okoteve traduccin
#elif L==7	// it
	"mesi.";
#elif L==8	// pl
	"months.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"meses.";
#endif

const char *Txt_Remove_folder =
#if   L==1	// ca
	"Eliminar carpeta";
#elif L==2	// de
	"Verzeichnis entfernen";
#elif L==3	// en
	"Remove folder";
#elif L==4	// es
	"Eliminar carpeta";
#elif L==5	// fr
	"Supprimer r&eacute;pertoire";
#elif L==6	// gn
	"Eliminar carpeta";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere cartella";
#elif L==8	// pl
	"Usu&nacute; folder";
#elif L==9	// pt
	"Remover diret&oacute;rio";
#endif

const char *Txt_Remove_folder_X = // Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar carpeta %s";
#elif L==2	// de
	"Verzeichnis entfernen %s";
#elif L==3	// en
	"Remove folder %s";
#elif L==4	// es
	"Eliminar carpeta %s";
#elif L==5	// fr
	"Supprimer r&eacute;pertoire %s";
#elif L==6	// gn
	"Eliminar carpeta %s";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere cartella %s";
#elif L==8	// pl
	"Usu&nacute; folder %s";
#elif L==9	// pt
	"Remover diret&oacute;rio %s";
#endif

const char *Txt_Remove_game =
#if   L==1	// ca
	"Eliminar joc";
#elif L==2	// de
	"Entfernen Spiel";
#elif L==3	// en
	"Remove game";
#elif L==4	// es
	"Eliminar juego";
#elif L==5	// fr
	"Supprimer jeu";
#elif L==6	// gn
	"Eliminar juego";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere gioco";
#elif L==8	// pl
	"Usu&nacute; gra";
#elif L==9	// pt
	"Remover jogo";
#endif

const char *Txt_Remove_group =
#if   L==1	// ca
	"Eliminar grupo";	// Necessita traduccio
#elif L==2	// de
	"Entfernen Gruppe";
#elif L==3	// en
	"Remove group";
#elif L==4	// es
	"Eliminar grupo";
#elif L==5	// fr
	"Supprimer groupe";
#elif L==6	// gn
	"Eliminar grupo";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere gruppo";
#elif L==8	// pl
	"Usu&nacute; gruper";
#elif L==9	// pt
	"Remover grupo";
#endif

const char *Txt_Remove_link =
#if   L==1	// ca
	"Eliminar enlla&ccedil;";
#elif L==2	// de
	"Entfernen Link";
#elif L==3	// en
	"Remove link";
#elif L==4	// es
	"Eliminar enlace";
#elif L==5	// fr
	"Supprimer lien";
#elif L==6	// gn
	"Eliminar enlace";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere link";
#elif L==8	// pl
	"Usu&nacute; link";
#elif L==9	// pt
	"Remover liga&ccedil;&atilde;o";
#endif

const char *Txt_Remove_logo =
#if   L==1	// ca
	"Eliminar logo";
#elif L==2	// de
	"Entfernen Logo";
#elif L==3	// en
	"Remove logo";
#elif L==4	// es
	"Eliminar logo";
#elif L==5	// fr
	"Supprimer logo";
#elif L==6	// gn
	"Eliminar logo";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere logo";
#elif L==8	// pl
	"Usu&nacute; logo";
#elif L==9	// pt
	"Remover logotipo";
#endif

const char *Txt_Remove_me =
#if   L==1	// ca
	"Eliminarme";
#elif L==2	// de
	"Entfernen Sie";
#elif L==3	// en
	"Remove me";
#elif L==4	// es
	"Eliminarme";
#elif L==5	// fr
	"Enlever-moi";
#elif L==6	// gn
	"Eliminarme";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi";
#elif L==8	// pl
	"Usu&ntilde;";
#elif L==9	// pt
	"Tirar me";
#endif

const char *Txt_Remove_me_as_an_administrator =
#if   L==1	// ca
	"Eliminarme com a administrador/a";
#elif L==2	// de
	"Entfernen Sie mir als Administrator";
#elif L==3	// en
	"Remove me as an administrator";
#elif L==4	// es
	"Eliminarme como administrador/a";
#elif L==5	// fr
	"Enlever-moi en tant qu'administrateur";
#elif L==6	// gn
	"Eliminarme como administrador/a";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi come amministratore";
#elif L==8	// pl
	"Usu&nacute; mnie jako administrator";
#elif L==9	// pt
	"Tirar me como administrador";
#endif

const char *Txt_Remove_me_as_an_administrator_of_the_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminarme com a administrador/a"
	" del centre <strong>%s</strong>";
#elif L==2	// de
	"Entfernen Sie mir als Administrator"
	" des Lehrinstitute <strong>%s</strong>";
#elif L==3	// en
	"Remove me as an administrator"
	" of the centre <strong>%s</strong>";
#elif L==4	// es
	"Eliminarme como administrador/a"
	" del centro <strong>%s</strong>";
#elif L==5	// fr
	"Enlever-moi en tant qu'administrateur"
	" du centre <strong>%s</strong>";
#elif L==6	// gn
	"Eliminarme como administrador/a"
	" del centro <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi come amministratore"
	" del centro <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; mnie jako administrator"
	" centrum <strong>%s</strong>";
#elif L==9	// pt
	"Tirar me como administrador"
	" do centro <strong>%s</strong>";
#endif

const char *Txt_Remove_me_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminarme com a administrador/a"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Entfernen Sie mir als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==3	// en
	"Remove me as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==4	// es
	"Eliminarme como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Enlever-moi en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==6	// gn
	"Eliminarme como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; mnie jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==9	// pt
	"Tirar me como administrador"
	" do grau <strong>%s</strong>";
#endif

const char *Txt_Remove_me_as_an_administrator_of_the_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminarme com a administrador/a"
	" de la instituci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Entfernen Sie mir als Administrator"
	" des Hochschule <strong>%s</strong>";
#elif L==3	// en
	"Remove me as an administrator"
	" of the institution <strong>%s</strong>";
#elif L==4	// es
	"Eliminarme como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Enlever-moi en tant qu'administrateur"
	" du &eacute;tablissement <strong>%s</strong>";
#elif L==6	// gn
	"Eliminarme como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi come amministratore"
	" della istituzione <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; mnie jako administrator"
	" instytucji <strong>%s</strong>";
#elif L==9	// pt
	"Tirar me como administrador"
	" da institu&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Remove_me_from_this_course =
#if   L==1	// ca
	"Eliminarme d'aquesta assignatura";
#elif L==2	// de
	"Entfernen Sie diesen Kurs";
#elif L==3	// en
	"Remove me from this course";
#elif L==4	// es
	"Eliminarme de esta asignatura";
#elif L==5	// fr
	"Enlever-moi de cette mati&egrave;re";
#elif L==6	// gn
	"Eliminarme de esta asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi da questo corso";
#elif L==8	// pl
	"Usu&ntilde; z kursu";
#elif L==9	// pt
	"Tirar me desta disciplina";
#endif

const char *Txt_Remove_me_from_this_project =
#if   L==1	// ca
	"Eliminarme d'aquest projecte";
#elif L==2	// de
	"Entfernen Sie diesen Projekt";
#elif L==3	// en
	"Remove me from this project";
#elif L==4	// es
	"Eliminarme de este proyecto";
#elif L==5	// fr
	"Enlever-moi de ce projet";
#elif L==6	// gn
	"Eliminarme de este proyecto";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi da questo progetto";
#elif L==8	// pl
	"Usu&ntilde; z projekt";
#elif L==9	// pt
	"Tirar me deste projeto";
#endif

const char *Txt_Remove_me_from_THE_COURSE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminarme de <strong>%s</strong>";
#elif L==2	// de
	"Entfernen Sie mir aus <strong>%s</strong>";
#elif L==3	// en
	"Remove me from <strong>%s</strong>";
#elif L==4	// es
	"Eliminarme de <strong>%s</strong>";
#elif L==5	// fr
	"Enlever-moi de <strong>%s</strong>";
#elif L==6	// gn
	"Eliminarme de <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovimi di <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; mnie w <strong>%s</strong>";
#elif L==9	// pt
	"Tirar me de <strong>%s</strong>";
#endif

const char *Txt_Remove_old_files =
#if   L==1	// ca
	"Eliminar arxius antics";
#elif L==2	// de
	"Entfernen Sie alte Dateien";
#elif L==3	// en
	"Remove old files";
#elif L==4	// es
	"Eliminar archivos antiguos";
#elif L==5	// fr
	"Supprimer anciens fichiers";
#elif L==6	// gn
	"Eliminar archivos antiguos";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere vecchi file";
#elif L==8	// pl
	"Usu&nacute; stare pliki";
#elif L==9	// pt
	"Remover arquivos antigos";
#endif

const char *Txt_Remove_photo =
#if   L==1	// ca
	"Eliminar foto";
#elif L==2	// de
	"Entfernen Foto";
#elif L==3	// en
	"Remove photo";
#elif L==4	// es
	"Eliminar foto";
#elif L==5	// fr
	"Supprimer photo";
#elif L==6	// gn
	"Eliminar foto";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere foto";
#elif L==8	// pl
	"Usu&nacute; zdjecie";
#elif L==9	// pt
	"Remover foto";
#endif

const char *Txt_Remove_project =
#if   L==1	// ca
	"Eliminar projecte";
#elif L==2	// de
	"Entfernen Projekt";
#elif L==3	// en
	"Remove project";
#elif L==4	// es
	"Eliminar proyecto";
#elif L==5	// fr
	"Supprimer projet";
#elif L==6	// gn
	"Eliminar proyecto";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere progetto";
#elif L==8	// pl
	"Usu&nacute; projekt";
#elif L==9	// pt
	"Remover projeto";
#endif

const char *Txt_Remove_question =
#if   L==1	// ca
	"Eliminar pregunta";
#elif L==2	// de
	"Entfernen Frage";
#elif L==3	// en
	"Remove question";
#elif L==4	// es
	"Eliminar pregunta";
#elif L==5	// fr
	"Supprimer question";
#elif L==6	// gn
	"Eliminar pregunta";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere domanda";
#elif L==8	// pl
	"Usu&nacute; pytanie";
#elif L==9	// pt
	"Remover quest&atilde;o";
#endif

const char *Txt_Remove_record_field =
#if   L==1	// ca
	"Eliminar campo de ficha";	// Necessita traduccio
#elif L==2	// de
	"Entfernen Datenfeld";
#elif L==3	// en
	"Remove record field";
#elif L==4	// es
	"Eliminar campo de ficha";
#elif L==5	// fr
	"Supprimer champ de carte";
#elif L==6	// gn
	"Eliminar campo de ficha";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere campo di scheda";
#elif L==8	// pl
	"Usu&nacute; pola rekordu";
#elif L==9	// pt
	"Remover campo de cart&atilde;o";
#endif

const char *Txt_Remove_survey =
#if   L==1	// ca
	"Eliminar enquesta";
#elif L==2	// de
	"Entfernen Umfrage";
#elif L==3	// en
	"Remove survey";
#elif L==4	// es
	"Eliminar encuesta";
#elif L==5	// fr
	"Supprimer sondage";
#elif L==6	// gn
	"Eliminar encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere sondaggio";
#elif L==8	// pl
	"Usu&nacute; badania";
#elif L==9	// pt
	"Remover inqu&eacute;rito";
#endif

const char *Txt_Remove_thread =
#if   L==1	// ca
	"Eliminar discusi&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Entfernen Thread";
#elif L==3	// en
	"Remove thread";
#elif L==4	// es
	"Eliminar discusi&oacute;n";
#elif L==5	// fr
	"Supprimer fil";
#elif L==6	// gn
	"Eliminar discusi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere discussione";
#elif L==8	// pl
	"Usu&nacute; watkek";
#elif L==9	// pt
	"Remover discuss&atilde;o";
#endif

const char *Txt_Remove_the_users_not_indicated_in_step_1 =
#if   L==1	// ca
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";	// Necessita traduccio
#elif L==2	// de
	"Entfernen Sie die Benutzer, die <strong>nicht</strong> auf der Liste";
#elif L==3	// en
	"Remove the users <strong>not</strong> indicated in step 1";
#elif L==4	// es
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";
#elif L==5	// fr
	"Enlever les utilisateurs qui <strong>ne</strong> sont <strong>pas</strong> sur la liste";
#elif L==6	// gn
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere gli utenti che <strong>non</strong> sono sulla lista";
#elif L==8	// pl
	"Usu&nacute; u&zdot;ytkownikami, kt&oacute;rzy sa <strong>nie</strong> na liscie";
#elif L==9	// pt
	"Tirar os utilizadores que <strong>n&atilde;o</strong> est&atilde;o na lista";
#endif

const char *Txt_Remove_the_users_indicated_in_step_1 =
#if   L==1	// ca
	"Eliminar los usuarios indicados en el paso 1";	// Necessita traduccio
#elif L==2	// de
	"Entfernen Sie die Benutzer, die auf der Liste";
#elif L==3	// en
	"Remove the users indicated in step 1";
#elif L==4	// es
	"Eliminar los usuarios indicados en el paso 1";
#elif L==5	// fr
	"Enlever les utilisateurs qui sont sur la liste";
#elif L==6	// gn
	"Eliminar los usuarios indicados en el paso 1";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere gli utenti che sono in lista";
#elif L==8	// pl
	"Usu&nacute; u&zdot;ytkownikami, kt&oacute;rzy sa na liscie";
#elif L==9	// pt
	"Tirar os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Remove_type_of_group =
#if   L==1	// ca
	"Eliminar tipo de grupo";	// Necessita traduccio
#elif L==2	// de
	"Entfernen Art der Gruppe";
#elif L==3	// en
	"Remove type of group";
#elif L==4	// es
	"Eliminar tipo de grupo";
#elif L==5	// fr
	"Supprimer type de groupe";
#elif L==6	// gn
	"Eliminar tipo de grupo";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovere tipo di gruppo";
#elif L==8	// pl
	"Usu&nacute; typ grupy";
#elif L==9	// pt
	"Remover tipo de grupo";
#endif

const char *Txt_Remove_user_from_this_course =
#if   L==1	// ca
	"Eliminar usuari d'aquesta assignatura";
#elif L==2	// de
	"Entfernen Benutzer Sie diesen Kurs";
#elif L==3	// en
	"Remove user from this course";
#elif L==4	// es
	"Eliminar usuario de esta asignatura";
#elif L==5	// fr
	"Enlever utilisateur de cette mati&egrave;re";
#elif L==6	// gn
	"Eliminar usuario de esta asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi utente da questo corso";
#elif L==8	// pl
	"Usu&ntilde; u&zdot;ytkownika z kursu";
#elif L==9	// pt
	"Tirar utilizador desta disciplina";
#endif

const char *Txt_Remove_user_from_this_project =
#if   L==1	// ca
	"Eliminar usuari d'aquest projecte";
#elif L==2	// de
	"Entfernen Benutzer Sie diesen Projekt";
#elif L==3	// en
	"Remove user from this project";
#elif L==4	// es
	"Eliminar usuario de este proyecto";
#elif L==5	// fr
	"Enlever utilisateur de ce projet";
#elif L==6	// gn
	"Eliminar usuario de este proyecto";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi utente da questo progetto";
#elif L==8	// pl
	"Usu&ntilde; u&zdot;ytkownika z projekt";
#elif L==9	// pt
	"Tirar utilizador deste projeto";
#endif

const char *Txt_Remove_USER_as_an_administrator =
#if   L==1	// ca
	"Eliminar com a administrador/a";
#elif L==2	// de
	"Entfernen als Administrator";
#elif L==3	// en
	"Remove as an administrator";
#elif L==4	// es
	"Eliminar como administrador/a";
#elif L==5	// fr
	"Enlever en tant qu'administrateur";
#elif L==6	// gn
	"Eliminar como administrador/a";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi come amministratore";
#elif L==8	// pl
	"Usu&nacute; jako administrator";
#elif L==9	// pt
	"Tirar como administrador";
#endif

const char *Txt_Remove_USER_as_an_administrator_of_the_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar com a administrador/a"
	" del centre <strong>%s</strong>";
#elif L==2	// de
	"Entfernen als Administrator"
	" des Lehrinstitute <strong>%s</strong>";
#elif L==3	// en
	"Remove as an administrator"
	" of the centre <strong>%s</strong>";
#elif L==4	// es
	"Eliminar como administrador/a"
	" del centro <strong>%s</strong>";
#elif L==5	// fr
	"Enlever en tant qu'administrateur"
	" du centre <strong>%s</strong>";
#elif L==6	// gn
	"Eliminar como administrador/a"
	" del centro <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi come amministratore"
	" del centro <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; jako administrator"
	" centrum <strong>%s</strong>";
#elif L==9	// pt
	"Tirar como administrador"
	" do centro <strong>%s</strong>";
#endif

const char *Txt_Remove_USER_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar com a administrador/a"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Entfernen als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==3	// en
	"Remove as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==4	// es
	"Eliminar como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Enlever en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==6	// gn
	"Eliminar como administrador/a"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==9	// pt
	"Tirar como administrador"
	" do grau <strong>%s</strong>";
#endif

const char *Txt_Remove_USER_as_an_administrator_of_the_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar com a administrador/a"
	" de la instituci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"Entfernen als Administrator"
	" des Hochschule <strong>%s</strong>";
#elif L==3	// en
	"Remove as an administrator"
	" of the institution <strong>%s</strong>";
#elif L==4	// es
	"Eliminar como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"Enlever en tant qu'administrateur"
	" du &eacute;tablissement <strong>%s</strong>";
#elif L==6	// gn
	"Eliminar como administrador/a"
	" de la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi come amministratore"
	" della istituzione <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; jako administrator"
	" instytucji <strong>%s</strong>";
#elif L==9	// pt
	"Tirar como administrador"
	" da institu&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Remove_USER_from_THE_COURSE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Eliminar de <strong>%s</strong>";
#elif L==2	// de
	"Entfernen aus <strong>%s</strong>";
#elif L==3	// en
	"Remove from <strong>%s</strong>";
#elif L==4	// es
	"Eliminar de <strong>%s</strong>";
#elif L==5	// fr
	"Enlever de <strong>%s</strong>";
#elif L==6	// gn
	"Eliminar de <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Rimuovi di <strong>%s</strong>";
#elif L==8	// pl
	"Usu&nacute; w <strong>%s</strong>";
#elif L==9	// pt
	"Tirar de <strong>%s</strong>";
#endif

const char *Txt_Reply =
#if   L==1	// ca
	"Respondre";
#elif L==2	// de
	"Antworten";
#elif L==3	// en
	"Reply";
#elif L==4	// es
	"Responder";
#elif L==5	// fr
	"R&eacute;pondre";
#elif L==6	// gn
	"Responder";	// Okoteve traduccin
#elif L==7	// it
	"Rispondere";
#elif L==8	// pl
	"Odpowiedz";
#elif L==9	// pt
	"Responder";
#endif

const char *Txt_Reply_again =
#if   L==1	// ca
	"Respondre de nou";
#elif L==2	// de
	"Antworten wieder";
#elif L==3	// en
	"Reply again";
#elif L==4	// es
	"Responder otra vez";
#elif L==5	// fr
	"R&eacute;pondre &agrave; nouveau";
#elif L==6	// gn
	"Responder otra vez";		// Okoteve traduccin
#elif L==7	// it
	"Rispondere nuovamente";
#elif L==8	// pl
	"Odpowiedz ponownie";
#elif L==9	// pt
	"Responder novamente";
#endif

const char *Txt_Reply_message =	// "Reply" as a noun, not as a verb
#if   L==1	// ca
	"Missatge de resposta";
#elif L==2	// de
	"Antwortnachricht";
#elif L==3	// en
	"Reply message";
#elif L==4	// es
	"Mensaje de respuesta";
#elif L==5	// fr
	"Message de r&eacute;ponse";
#elif L==6	// gn
	"Mensaje de respuesta";	// Okoteve traduccin
#elif L==7	// it
	"Messaggio di risposta";
#elif L==8	// pl
	"Wiadomo&sacute;&cacute; odpowiedz";
#elif L==9	// pt
	"Mensagem de resposta";
#endif

const char *Txt_Report =
#if   L==1	// ca
	"Informe";
#elif L==2	// de
	"Nutzungsbericht";
#elif L==3	// en
	"Report";
#elif L==4	// es
	"Informe";
#elif L==5	// fr
	"Rapport";
#elif L==6	// gn
	"Informe";	// Okoteve traduccin
#elif L==7	// it
	"Rapporto";
#elif L==8	// pl
	"Raport";
#elif L==9	// pt
	"Relat&oacute;rio";
#endif

const char *Txt_Report_of_use_of_PLATFORM =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Informe d'&uacute;s de %s";
#elif L==2	// de
	"Bericht der Nutzung der %s";
#elif L==3	// en
	"Report of use of %s";
#elif L==4	// es
	"Informe de uso de %s";
#elif L==5	// fr
	"Rapport d'utilisation de %s";
#elif L==6	// gn
	"Informe de uso de %s";	// Okoteve traduccin
#elif L==7	// it
	"Rapporto di utilizzo de %s";
#elif L==8	// pl
	"Sprawozdanie z wykorzystaniem %s";
#elif L==9	// pt
	"Relat&oacute;rio de utiliza&ccedil;&atilde;o de %s";
#endif

const char *Txt_Report_possible_duplicate_user =
#if   L==1	// ca
	"Informar possible usuari duplicat";
#elif L==2	// de
	"Melden Sie die doppelte Benutzer";
#elif L==3	// en
	"Report possible duplicate user";
#elif L==4	// es
	"Informar de posible usuario duplicado";
#elif L==5	// fr
	"Signaler possible utilisateur en double";
#elif L==6	// gn
	"Informar de posible usuario duplicado";	// Okoteve traduccin
#elif L==7	// it
	"Segnalare eventuali utente duplicato";
#elif L==8	// pl
	"Zg&lstrok;o&sacute; mo&zdot;liwy duplikat u&zdot;ytkownika";
#elif L==9	// pt
	"Denunciar poss&iacute;vel usu&aacute;rio duplicado";
#endif

const char *Txt_Requester =
#if   L==1	// ca
	"Sol&middot;licitant";
#elif L==2	// de
	"Requester";
#elif L==3	// en
	"Requester";
#elif L==4	// es
	"Solicitante";
#elif L==5	// fr
	"Demandeur";
#elif L==6	// gn
	"Solicitante";	// Okoteve traduccin
#elif L==7	// it
	"Richiedente";
#elif L==8	// pl
	"&Zdot;&aogon;daj&aogon;cego";
#elif L==9	// pt
	"Requerente";
#endif

const char *Txt_Required_knowledge =
#if   L==1	// ca
	"Coneixements necessaris";
#elif L==2	// de
	"Notwendiges Wissen";
#elif L==3	// en
	"Required knowledge";
#elif L==4	// es
	"Conocimientos necesarios";
#elif L==5	// fr
	"Connaissances n&eacute;cessaires";
#elif L==6	// gn
	"Conocimientos necesarios";	// Okoteve traduccin
#elif L==7	// it
	"Nozioni necessarie";
#elif L==8	// pl
	"Niezb&eogon;dna wiedza";
#elif L==9	// pt
	"Conhecimento necess&aacute;rio";
#endif

const char *Txt_Required_materials =
#if   L==1	// ca
	"Materials necessaris";
#elif L==2	// de
	"Ben&ouml;tigte Materialien";
#elif L==3	// en
	"Required materials";
#elif L==4	// es
	"Materiales necesarios";
#elif L==5	// fr
	"Mat&eacute;riel n&eacute;cessaire";
#elif L==6	// gn
	"Materiales necesarios";	// Okoteve traduccin
#elif L==7	// it
	"Materiali necessari";
#elif L==8	// pl
	"Potrzebne materia&lstrok;y";
#elif L==9	// pt
	"Materiais necess&aacute;rios";
#endif

const char *Txt_Required_reading =
#if   L==1	// ca
	"Lectura obligat&ograve;ria";
#elif L==2	// de
	"Erforderliches Lesen";
#elif L==3	// en
	"Required reading";
#elif L==4	// es
	"Lectura obligatoria";
#elif L==5	// fr
	"Lecture obligatoire";
#elif L==6	// gn
	"Lectura obligatoria";	// Okoteve traduccin
#elif L==7	// it
	"Lettura obbligatoria";
#elif L==8	// pl
	"Obowi&aogon;zkowa lektura";
#elif L==9	// pt
	"Leitura obrigat&oacute;ria";
#endif

const char *Txt_Reset =
#if   L==1	// ca
	"Posar a zero";
#elif L==2	// de
	"Reset";
#elif L==3	// en
	"Reset";
#elif L==4	// es
	"Poner a cero";
#elif L==5	// fr
	"Reset";
#elif L==6	// gn
	"Poner a cero";	// Okoteve traduccin
#elif L==7	// it
	"Resettare";
#elif L==8	// pl
	"Reset";
#elif L==9	// pt
	"Reiniciar";
#endif

const char *Txt_Reset_game =
#if   L==1	// ca
	"Posar joc a zero";
#elif L==2	// de
	"Reset Spiel";
#elif L==3	// en
	"Reset game";
#elif L==4	// es
	"Poner juego a cero";
#elif L==5	// fr
	"Reset jeu";
#elif L==6	// gn
	"Poner juego a cero";	// Okoteve traduccin
#elif L==7	// it
	"Resettare gioco";
#elif L==8	// pl
	"Reset gr&eogon;";
#elif L==9	// pt
	"Reiniciar jogo";
#endif

const char *Txt_Reset_survey =
#if   L==1	// ca
	"Posar enquesta a zero";
#elif L==2	// de
	"Reset Umfrage";
#elif L==3	// en
	"Reset survey";
#elif L==4	// es
	"Poner encuesta a cero";
#elif L==5	// fr
	"Reset sondage";
#elif L==6	// gn
	"Poner encuesta a cero";	// Okoteve traduccin
#elif L==7	// it
	"Resettare sondaggio";
#elif L==8	// pl
	"Reset badania";
#elif L==9	// pt
	"Reiniciar inqu&eacute;rito";
#endif

const char *Txt_results_per_page =
#if   L==1	// ca
	"resultats per p&agrave;gina";
#elif L==2	// de
	"Ergebnisse pro Seite";
#elif L==3	// en
	"results per page";
#elif L==4	// es
	"resultados por p&aacute;gina";
#elif L==5	// fr
	"r&eacute;sultats par page";
#elif L==6	// gn
	"resultados por p&aacute;gina";	// Okoteve traduccin
#elif L==7	// it
	"risultati per pagina";
#elif L==8	// pl
	"wynik&oacute;w na stronie";
#elif L==9	// pt
	"resultados por p&aacute;gina";
#endif

const char *Txt_Retype_new_password =
#if   L==1	// ca
	"Reescriviu la nova contrasenya";
#elif L==2	// de
	"Wiederholung neues Passwort";
#elif L==3	// en
	"Retype new password";
#elif L==4	// es
	"Reescriba la nueva contrase&ntilde;a";
#elif L==5	// fr
	"Confirmez votre nouveau mot de passe";
#elif L==6	// gn
	"Reescriba la nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Riscrivi la nuova password";
#elif L==8	// pl
	"Powt&oacute;rz nowe has&lstrok;o";
#elif L==9	// pt
	"Redigite a nova senha";
#endif

const char *Txt_Role =
#if   L==1	// ca
	"Rol";
#elif L==2	// de
	"Rolle";
#elif L==3	// en
	"Role";
#elif L==4	// es
	"Rol";
#elif L==5	// fr
	"R&ocirc;le";
#elif L==6	// gn
	"Rol";	// Okoteve traduccin
#elif L==7	// it
	"Ruolo";
#elif L==8	// pl
	"Rol&eogon;";
#elif L==9	// pt
	"Papel";
#endif

const char *Txt_ROLE_STATS[Sta_NUM_ROLES_STAT] =
   {	// Sta_ROLE_IDENTIFIED_USRS
#if   L==1	// ca
	"identificats"
#elif L==2	// de
	"Identifiziert"
#elif L==3	// en
	"identified"
#elif L==4	// es
	"identificados"
#elif L==5	// fr
	"identifi&eacute;s"
#elif L==6	// gn
	"identificados"	// Okoteve traduccin
#elif L==7	// it
	"identificati"
#elif L==8	// pl
	"zidentyfikowane"
#elif L==9	// pt
	"identificados"
#endif
	,	// Sta_ROLE_ALL_USRS
#if   L==1	// ca
	"identificats o desconeguts"
#elif L==2	// de
	"Identifiziert oder Unbekannte"
#elif L==3	// en
	"identified or unknown"
#elif L==4	// es
	"identificados o desconocidos"
#elif L==5	// fr
	"identifi&eacute;s ou inconnus"
#elif L==6	// gn
	"identificados o desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"identificati o sconosciuti"
#elif L==8	// pl
	"zidentyfikowanej lub nieznane"
#elif L==9	// pt
	"identificados ou desconhecidos"
#endif
	,	// Sta_ROLE_INS_ADMINS
#if   L==1	// ca
	"administradors d'institucions"
#elif L==2	// de
	"Administratoren von Hochschulen"
#elif L==3	// en
	"administrators of institutions"
#elif L==4	// es
	"administradores de instituciones"
#elif L==5	// fr
	"administrateurs d'&eacute;tablissements"
#elif L==6	// gn
	"administradores de instituciones"	// Okoteve traduccin
#elif L==7	// it
	"amministratori di istituzioni"
#elif L==8	// pl
	"Administratorzy instytucje"
#elif L==9	// pt
	"administradores de institu&ccedil;&otilde;es"
#endif
	,	// Sta_ROLE_CTR_ADMINS
#if   L==1	// ca
	"administradors de centres"
#elif L==2	// de
	"Administratoren von Lehrinstitute"
#elif L==3	// en
	"administrators of centres"
#elif L==4	// es
	"administradores de centros"
#elif L==5	// fr
	"administrateurs de centres"
#elif L==6	// gn
	"administradores de centros"	// Okoteve traduccin
#elif L==7	// it
	"amministratori di centri"
#elif L==8	// pl
	"Administratorzy centra"
#elif L==9	// pt
	"administradores de centros"
#endif
	,	// Sta_ROLE_DEG_ADMINS
#if   L==1	// ca
	"administradors de titulacions"
#elif L==2	// de
	"Administratoren von Studieng&auml;nge"
#elif L==3	// en
	"administrators of degrees"
#elif L==4	// es
	"administradores de titulaciones"
#elif L==5	// fr
	"administrateurs de degr&eacute;s"
#elif L==6	// gn
	"administradores de titulaciones"	// Okoteve traduccin
#elif L==7	// it
	"amministratori di laurea"
#elif L==8	// pl
	"Administratorzy stopni"
#elif L==9	// pt
	"administradores de graus"
#endif
	,	// Sta_ROLE_TEACHERS
#if   L==1	// ca
	"professors"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"teachers"
#elif L==4	// es
	"profesores"
#elif L==5	// fr
	"enseignants"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professori"
#elif L==8	// pl
	"nauczycieli"
#elif L==9	// pt
	"professores"
#endif
	,	// Sta_ROLE_NON_EDITING_TEACHERS
#if   L==1	// ca
	"professors no editors"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teachers"
#elif L==4	// es
	"profesores no editores"
#elif L==5	// fr
	"enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"professori non-editing"
#elif L==8	// pl
	"nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"professores n&atilde;o editores"
#endif
	,	// Sta_ROLE_STUDENTS
#if   L==1	// ca
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiants"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studenti"
#elif L==8	// pl
	"studenci"
#elif L==9	// pt
	"estudantes"
#endif
	,	// Sta_ROLE_USERS
#if   L==1	// ca
	 "usuaris"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "users"
#elif L==4	// es
	 "usuarios"
#elif L==5	// fr
	 "utilisateurs"
#elif L==6	// gn
	 "puruh&aacute;ra"
#elif L==7	// it
	 "utenti"
#elif L==8	// pl
	 "u&zdot;ytkownicy"
#elif L==9	// pt
	 "utilizadores"
#endif
	,	// Sta_ROLE_GUESTS
#if   L==1	// ca
	"convidats (sense assignatures)"
#elif L==2	// de
	"Eingeladen (ohne Kursen)"
#elif L==3	// en
	"guests (without courses)"
#elif L==4	// es
	"invitados (sin asignaturas)"
#elif L==5	// fr
	"invit&eacute;s (sans mati&egrave;res)"
#elif L==6	// gn
	"invitados (sin asignaturas)"	// Okoteve traduccin
#elif L==7	// it
	"ospiti (senza corsi)"
#elif L==8	// pl
	"zaproszeny (bez kurs&oacute;w)"
#elif L==9	// pt
	"convidados (sem disciplinas)"
#endif
	,	// Sta_ROLE_UNKNOWN_USRS
#if   L==1	// ca
	"desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidos"
#elif L==5	// fr
	"inconnus"
#elif L==6	// gn
	"desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecidos"
#endif
	,	// Sta_ROLE_ME
#if   L==1	// ca
	"jo"
#elif L==2	// de
	"Mich"
#elif L==3	// en
	"me"
#elif L==4	// es
	"yo"
#elif L==5	// fr
	"moi"
#elif L==6	// gn
	"che"
#elif L==7	// it
	"me"
#elif L==8	// pl
	"ja"
#elif L==9	// pt
	"eu"
#endif
	};

const char *Txt_ROLES_SINGUL_Abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
{
	{
#if   L==1	// ca	// Rol_UNK - Usr_SEX_UNKNOWN
	"Desconegut/da"
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido/a"
#elif L==5	// fr
	"Inconnu/e"
#elif L==6	// gn
	"Desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto/a"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido/a"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_FEMALE
	"Desconeguda"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocida"
#elif L==5	// fr
	"Inconnue"
#elif L==6	// gn
	"Desconocida"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuta"
#elif L==8	// pl
	"Nieznana"
#elif L==9	// pt
	"Desconhecida"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_MALE
	"Desconegut"
#elif L==2	// de
	"Unbekannter"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido"
#elif L==5	// fr
	"Inconnu"
#elif L==6	// gn
	"Desconocido"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecido"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_ALL
	"Desconegut/da"
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocido/a"
#elif L==5	// fr
	"Inconnu/e"
#elif L==6	// gn
	"Desconocido/a"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto/a"
#elif L==8	// pl
	"Nieznani"
#elif L==9	// pt
	"Desconhecido/a"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_GST - Usr_SEX_UNKNOWN
	"Convidat/da"
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"Guest"
#elif L==4	// es
	"Invitado/a"
#elif L==5	// fr
	"Invit&eacute;/e"
#elif L==6	// gn
	"Invitado/a"	// Okoteve traduccin
#elif L==7	// it
	"Ospite"
#elif L==8	// pl
	"Go&sacute;&cacute;"
#elif L==9	// pt
	"Convidado/a"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_FEMALE
	"Convidada"
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"Guest"
#elif L==4	// es
	"Invitada"
#elif L==5	// fr
	"Invit&eacute;e"
#elif L==6	// gn
	"Invitada"	// Okoteve traduccin
#elif L==7	// it
	"Ospite"
#elif L==8	// pl
	"Opinia"
#elif L==9	// pt
	"Convidada"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_MALE
	"Convidat"
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"Guest"
#elif L==4	// es
	"Invitado"
#elif L==5	// fr
	"Invit&eacute;"
#elif L==6	// gn
	"Invitado"	// Okoteve traduccin
#elif L==7	// it
	"Ospite"
#elif L==8	// pl
	"Opinia"
#elif L==9	// pt
	"Convidado"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_ALL
	"Convidat/da"
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"Guest"
#elif L==4	// es
	"Invitado/a"
#elif L==5	// fr
	"Invit&eacute;/e"
#elif L==6	// gn
	"Invitado/a"	// Okoteve traduccin
#elif L==7	// it
	"Ospite"
#elif L==8	// pl
	"Opinie"
#elif L==9	// pt
	"Convidado/a"
#endif
	},
	{
#if   L==1	// ca	// Rol_USR - Usr_SEX_UNKNOWN
	"Usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario/a"
#elif L==5	// fr
	"Utilisateur/se"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador/a"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_FEMALE
	"Usuaria"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuaria"
#elif L==5	// fr
	"Utilisateuse"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizadora"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_MALE
	"Usuari"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario"
#elif L==5	// fr
	"Utilisateur"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_ALL
	"Usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario/a"
#elif L==5	// fr
	"Utilisateur/se"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador/a"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_STD - Usr_SEX_UNKNOWN
	"Estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"Student"
#elif L==4	// es
	"Estudiante"
#elif L==5	// fr
	"&Eacute;tudiant/e"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studente"
#elif L==8	// pl
	"Student"
#elif L==9	// pt
	"Estudante"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_FEMALE
	"Estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"Student"
#elif L==4	// es
	"Estudiante"
#elif L==5	// fr
	"&Eacute;tudiante"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studentessa"
#elif L==8	// pl
	"Studentka"
#elif L==9	// pt
	"Estudante"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_MALE
	"Estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"Student"
#elif L==4	// es
	"Estudiante"
#elif L==5	// fr
	"&Eacute;tudiant"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studente"
#elif L==8	// pl
	"Student"
#elif L==9	// pt
	"Estudante"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_ALL
	"Estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"Student"
#elif L==4	// es
	"Estudiante"
#elif L==5	// fr
	"&Eacute;tudiant/e"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studente"
#elif L==8	// pl
	"Studenci"
#elif L==9	// pt
	"Estudante"
#endif
	},
	{
#if   L==1	// ca	// Rol_NET - Usr_SEX_UNKNOWN
	"Professor/a no editor/a"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teacher"
#elif L==4	// es
	"Profesor/a no editor/a"
#elif L==5	// fr
	"Enseignant/e non-&eacute;diteur/trice"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editor/a"	// Okoteve traduccin
#elif L==7	// it
	"Professore/ssa non-editing"
#elif L==8	// pl
	"Nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professor/a n&atilde;o editor/a"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_FEMALE
	"Professora no editora"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teacher"
#elif L==4	// es
	"Profesora no editora"
#elif L==5	// fr
	"Enseignante non-&eacute;ditrice"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editora"	// Okoteve traduccin
#elif L==7	// it
	"Professoressa non-editing"
#elif L==8	// pl
	"Nauczycielka nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professora n&atilde;o editora"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_MALE
	"Professor no editor"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teacher"
#elif L==4	// es
	"Profesor no editor"
#elif L==5	// fr
	"Enseignant non-&eacute;diteur"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editor"	// Okoteve traduccin
#elif L==7	// it
	"Professore non-editing"
#elif L==8	// pl
	"Nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professor n&atilde;o editor"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_ALL
	"Professor/a no editor/a"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teacher"
#elif L==4	// es
	"Profesor/a no editor/a"
#elif L==5	// fr
	"Enseignant/e non-&eacute;diteur/trice"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editor/a"	// Okoteve traduccin
#elif L==7	// it
	"Professore/ssa non-editing"
#elif L==8	// pl
	"Nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professor/a n&atilde;o editor/a"
#endif
	},
	{
#if   L==1	// ca	// Rol_TCH - Usr_SEX_UNKNOWN
	"Professor/a"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teacher"
#elif L==4	// es
	"Profesor/a"
#elif L==5	// fr
	"Enseignant/e"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professore/ssa"
#elif L==8	// pl
	"Nauczyciel"
#elif L==9	// pt
	"Professor/a"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_FEMALE
	"Professora"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teacher"
#elif L==4	// es
	"Profesora"
#elif L==5	// fr
	"Enseignante"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professoressa"
#elif L==8	// pl
	"Nauczycielka"
#elif L==9	// pt
	"Professora"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_MALE
	"Professor"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teacher"
#elif L==4	// es
	"Profesor"
#elif L==5	// fr
	"Enseignant"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professore"
#elif L==8	// pl
	"Nauczyciel"
#elif L==9	// pt
	"Professor"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_ALL
	"Professor/a"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teacher"
#elif L==4	// es
	"Profesor/a"
#elif L==5	// fr
	"Enseignant/e"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professore/ssa"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Professor/a"
#endif
	},
	{
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_UNKNOWN
	"Admin. titulaci&oacute;"
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"Admin. degree"
#elif L==4	// es
	"Admin. titulaci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tude"
#elif L==6	// gn
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. laurea"
#elif L==8	// pl
	"Admin. stopien"
#elif L==9	// pt
	"Admin. grau"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_FEMALE
	"Admin. titulaci&oacute;"
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"Admin. degree"
#elif L==4	// es
	"Admin. titulaci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tude"
#elif L==6	// gn
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. laurea"
#elif L==8	// pl
	"Admin. stopien"
#elif L==9	// pt
	"Admin. grau"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_MALE
	"Admin. titulaci&oacute;"
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"Admin. degree"
#elif L==4	// es
	"Admin. titulaci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tude"
#elif L==6	// gn
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. laurea"
#elif L==8	// pl
	"Admin. stopien"
#elif L==9	// pt
	"Admin. grau"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_ALL
	"Admin. titulaci&oacute;"
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"Admin. degree"
#elif L==4	// es
	"Admin. titulaci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tude"
#elif L==6	// gn
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. laurea"
#elif L==8	// pl
	"Admin. stopien"
#elif L==9	// pt
	"Admin. grau"
#endif
	},
	{
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_UNKNOWN
	"Admin. centre"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"Admin. centre"
#elif L==4	// es
	"Admin. centro"
#elif L==5	// fr
	"Admin. centre"
#elif L==6	// gn
	"Admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. centro"
#elif L==8	// pl
	"Admin. centrum"
#elif L==9	// pt
	"Admin. centro"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_FEMALE
	"Admin. centre"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"Admin. centre"
#elif L==4	// es
	"Admin. centro"
#elif L==5	// fr
	"Admin. centre"
#elif L==6	// gn
	"Admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. centro"
#elif L==8	// pl
	"Admin. centrum"
#elif L==9	// pt
	"Admin. centro"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_MALE
	"Admin. centre"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"Admin. centre"
#elif L==4	// es
	"Admin. centro"
#elif L==5	// fr
	"Admin. centre"
#elif L==6	// gn
	"Admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. centro"
#elif L==8	// pl
	"Admin. centrum"
#elif L==9	// pt
	"Admin. centro"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_ALL
	"Admin. centre"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"Admin. centre"
#elif L==4	// es
	"Admin. centro"
#elif L==5	// fr
	"Admin. centre"
#elif L==6	// gn
	"Admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. centro"
#elif L==8	// pl
	"Admin. centrum"
#elif L==9	// pt
	"Admin. centro"
#endif
	},
	{
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_UNKNOWN
	"Admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"Admin. institution"
#elif L==4	// es
	"Admin. instituci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tablissement"
#elif L==6	// gn
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. istituzione"
#elif L==8	// pl
	"Admin. instytucji"
#elif L==9	// pt
	"Admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_FEMALE
	"Admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"Admin. institution"
#elif L==4	// es
	"Admin. instituci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tablissement"
#elif L==6	// gn
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. istituzione"
#elif L==8	// pl
	"Admin. instytucji"
#elif L==9	// pt
	"Admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_MALE
	"Admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"Admin. institution"
#elif L==4	// es
	"Admin. instituci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tablissement"
#elif L==6	// gn
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. istituzione"
#elif L==8	// pl
	"Admin. instytucji"
#elif L==9	// pt
	"Admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_ALL
	"Admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"Admin. institution"
#elif L==4	// es
	"Admin. instituci&oacute;n"
#elif L==5	// fr
	"Admin. &eacute;tablissement"
#elif L==6	// gn
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"Ammin. istituzione"
#elif L==8	// pl
	"Admin. instytucji"
#elif L==9	// pt
	"Admin. institu&ccedil;&atilde;o"
#endif
	},
	{
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_UNKNOWN
	"Superusuari/a"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superuser"
#elif L==4	// es
	"Superusuario/a"
#elif L==5	// fr
	"Superutilisateur/se"
#elif L==6	// gn
	"Superusuario/a"	// Okoteve traduccin
#elif L==7	// it
	"Superutente"
#elif L==8	// pl
	"Superuzytkownik"
#elif L==9	// pt
	"Superutilizador/a"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_FEMALE
	"Superusuaria"	// Necessita traduccio
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superuser"
#elif L==4	// es
	"Superusuaria"
#elif L==5	// fr
	"Superutilisateuse"
#elif L==6	// gn
	"Superusuaria"	// Okoteve traduccin
#elif L==7	// it
	"Superutente"
#elif L==8	// pl
	"Superuzytkownik"
#elif L==9	// pt
	"Superutilizadora"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_MALE
	"Superusuari"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superuser"
#elif L==4	// es
	"Superusuario"
#elif L==5	// fr
	"Superutilisateur"
#elif L==6	// gn
	"Superusuario"	// Okoteve traduccin
#elif L==7	// it
	"Superutente"
#elif L==8	// pl
	"Superuzytkownik"
#elif L==9	// pt
	"Superutilizador"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_ALL
	"Superusuari/a"	// Necessita traduccio
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superuser"
#elif L==4	// es
	"Superusuario/a"
#elif L==5	// fr
	"Superutilisateur/se"
#elif L==6	// gn
	"Superusuario/a"		// Okoteve traduccin
#elif L==7	// it
	"Superutente"
#elif L==8	// pl
	"Superuzytkownicy"
#elif L==9	// pt
	"Superutilizador/a"
#endif
	 }
};

const char *Txt_ROLES_SINGUL_abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
{
	{
#if   L==1	// ca	// Rol_UNK - Usr_SEX_UNKNOWN
	"desconocido/a"		// Necessita traduccio
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocido/a"
#elif L==5	// fr
	"inconnu/e"
#elif L==6	// gn
	"desconocido/a"		// Okoteve traduccin
#elif L==7	// it
	"sconosciuto/a"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecido/a"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_FEMALE
	"desconocida"	// Necessita traduccio
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocida"
#elif L==5	// fr
	"inconnue"
#elif L==6	// gn
	"desconocida"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuta"
#elif L==8	// pl
	"nieznana"
#elif L==9	// pt
	"desconhecida"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_MALE
	"desconocido"	// Necessita traduccio
#elif L==2	// de
	"Unbekannter"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocido"
#elif L==5	// fr
	"inconnu"
#elif L==6	// gn
	"desconocido"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuto"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecido"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_ALL
	"desconocido/a"		// Necessita traduccio
#elif L==2	// de
	"Unbekannter/e"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocido/a"
#elif L==5	// fr
	"inconnu/e"
#elif L==6	// gn
	"desconocido/a"		// Okoteve traduccin
#elif L==7	// it
	"sconosciuto/a"
#elif L==8	// pl
	"nieznani"
#elif L==9	// pt
	"desconhecido/a"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_GST - Usr_SEX_UNKNOWN
	"invitado/a"	// Necessita traduccio
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"guest"
#elif L==4	// es
	"invitado/a"
#elif L==5	// fr
	"invit&eacute;/e"
#elif L==6	// gn
	"invitado/a"	// Okoteve traduccin
#elif L==7	// it
	"ospite"
#elif L==8	// pl
	"go&sacute;&cacute;"
#elif L==9	// pt
	"convidado/a"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_FEMALE
	"invitada"	// Necessita traduccio
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"guest"
#elif L==4	// es
	"invitada"
#elif L==5	// fr
	"invit&eacute;e"
#elif L==6	// gn
	"invitada"	// Okoteve traduccin
#elif L==7	// it
	"ospite"
#elif L==8	// pl
	"goscinna"
#elif L==9	// pt
	"convidada"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_MALE
	"invitado"	// Necessita traduccio
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"guest"
#elif L==4	// es
	"invitado"
#elif L==5	// fr
	"invit&eacute;"
#elif L==6	// gn
	"invitado"	// Okoteve traduccin
#elif L==7	// it
	"ospite"
#elif L==8	// pl
	"goscinny"
#elif L==9	// pt
	"convidado"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_ALL
	"invitado/a"	// Necessita traduccio
#elif L==2	// de
	"Gast"
#elif L==3	// en
	"guest"
#elif L==4	// es
	"invitado/a"
#elif L==5	// fr
	"invit&eacute;/e"
#elif L==6	// gn
	"invitado/a"	// Okoteve traduccin
#elif L==7	// it
	"ospite"
#elif L==8	// pl
	"goscinni"
#elif L==9	// pt
	"convidado/a"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_USR - Usr_SEX_UNKNOWN
	"usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario/a"
#elif L==5	// fr
	"utilisateur/se"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador/a"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_FEMALE
	"usuaria"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuaria"
#elif L==5	// fr
	"utilisateuse"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizadora"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_MALE
	"usuari"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario"
#elif L==5	// fr
	"utilisateur"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_ALL
	"usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario/a"
#elif L==5	// fr
	"utilisateur/se"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador/a"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_STD - Usr_SEX_UNKNOWN
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiant/e"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente"
#elif L==8	// pl
	"student"
#elif L==9	// pt
	"estudante"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_FEMALE
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiante"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studentessa"
#elif L==8	// pl
	"studentka"
#elif L==9	// pt
	"estudante"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_MALE
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiant"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente"
#elif L==8	// pl
	"student"
#elif L==9	// pt
	"estudante"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_ALL
	"estudiant"
#elif L==2	// de
	"Student"
#elif L==3	// en
	"student"
#elif L==4	// es
	"estudiante"
#elif L==5	// fr
	"&eacute;tudiant/e"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studente/ssa"
#elif L==8	// pl
	"studenci"
#elif L==9	// pt
	"estudante"
#endif
	 },
	{
#if   L==1	// ca	// Rol_NET - Usr_SEX_UNKNOWN
	"professor/a no editor/a"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teacher"
#elif L==4	// es
	"profesor/a no editor/a"
#elif L==5	// fr
	"enseignant/e non-&eacute;diteur/trice"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editor/a"	// Okoteve traduccin
#elif L==7	// it
	"professore/ssa non-editing"
#elif L==8	// pl
	"nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	"professor/a n&atilde;o editor/a"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_FEMALE
	"professora no editora"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teacher"
#elif L==4	// es
	"profesora no editora"
#elif L==5	// fr
	"enseignante non-&eacute;ditrice"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editora"	// Okoteve traduccin
#elif L==7	// it
	"professoressa non-editing"
#elif L==8	// pl
	"nauczycielka nie edytuj&aogon;cy"
#elif L==9	// pt
	"professora n&atilde;o editora"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_MALE
	"professor no editor"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teacher"
#elif L==4	// es
	"profesor no editor"
#elif L==5	// fr
	"enseignant non-&eacute;diteur"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editor"	// Okoteve traduccin
#elif L==7	// it
	"professore non-editing"
#elif L==8	// pl
	"nauczyciel nie edytuj&aogon;cy"
#elif L==9	// pt
	"professor n&atilde;o editor"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_ALL
	"professor/a no editor/a"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teacher"
#elif L==4	// es
	"profesor/a no editor/a"
#elif L==5	// fr
	"enseignant/e non-&eacute;diteur/trice"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editor/a"	// Okoteve traduccin
#elif L==7	// it
	"professore/ssa non-editing"
#elif L==8	// pl
	"nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"professor/a n&atilde;o editor/a"
#endif
	},
	 {
#if   L==1	// ca	// Rol_TCH - Usr_SEX_UNKNOWN
	"professor/a"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"teacher"
#elif L==4	// es
	"profesor/a"
#elif L==5	// fr
	"enseignant/e"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"Professore/ssa"
#elif L==8	// pl
	"nauczyciel"
#elif L==9	// pt
	"professor/a"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_FEMALE
	"professora"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"teacher"
#elif L==4	// es
	"profesora"
#elif L==5	// fr
	"enseignante"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"Professoressa"
#elif L==8	// pl
	"nauczycielka"
#elif L==9	// pt
	"professora"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_MALE
	"professor"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"teacher"
#elif L==4	// es
	"profesor"
#elif L==5	// fr
	"enseignant"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professore"
#elif L==8	// pl
	"nauczyciel"
#elif L==9	// pt
	"professor"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_ALL
	"professor/a"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"teacher"
#elif L==4	// es
	"profesor/a"
#elif L==5	// fr
	"enseignant/e"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professore/ssa"
#elif L==8	// pl
	"nauczyciele"
#elif L==9	// pt
	"professor/a"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_UNKNOWN
	"admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"admin. degree"
#elif L==4	// es
	"admin. titulaci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tude"
#elif L==6	// gn
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. laurea"
#elif L==8	// pl
	"admin. stopien"
#elif L==9	// pt
	"admin. grau"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_FEMALE
	"admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"admin. degree"
#elif L==4	// es
	"admin. titulaci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tude"
#elif L==6	// gn
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. laurea"
#elif L==8	// pl
	"admin. stopien"
#elif L==9	// pt
	"admin. grau"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_MALE
	"admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"admin. degree"
#elif L==4	// es
	"admin. titulaci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tude"
#elif L==6	// gn
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. laurea"
#elif L==8	// pl
	"admin. stopien"
#elif L==9	// pt
	"admin. grau"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_ALL
	"admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	"Admin. Studiengang"
#elif L==3	// en
	"admin. degree"
#elif L==4	// es
	"admin. titulaci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tude"
#elif L==6	// gn
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. laurea"
#elif L==8	// pl
	"admin. stopien"
#elif L==9	// pt
	"admin. grau"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_UNKNOWN
	"admin. centre"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"admin. centre"
#elif L==4	// es
	"admin. centro"
#elif L==5	// fr
	"admin. centre"
#elif L==6	// gn
	"admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"ammin. centro"
#elif L==8	// pl
	"admin. centrum"
#elif L==9	// pt
	"admin. centro"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_FEMALE
	"admin. centre"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"admin. centre"
#elif L==4	// es
	"admin. centro"
#elif L==5	// fr
	"admin. centre"
#elif L==6	// gn
	"admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"ammin. centro"
#elif L==8	// pl
	"admin. centrum"
#elif L==9	// pt
	"admin. centro"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_MALE
	"admin. centre"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"admin. centre"
#elif L==4	// es
	"admin. centro"
#elif L==5	// fr
	"admin. centre"
#elif L==6	// gn
	"admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"ammin. centro"
#elif L==8	// pl
	"admin. centrum"
#elif L==9	// pt
	"admin. centro"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_ALL
	"admin. centre"
#elif L==2	// de
	"Admin. Lehrinstitut"
#elif L==3	// en
	"admin. centre"
#elif L==4	// es
	"admin. centro"
#elif L==5	// fr
	"admin. centre"
#elif L==6	// gn
	"admin. centro"	// Okoteve traduccin
#elif L==7	// it
	"ammin. centro"
#elif L==8	// pl
	"admin. centrum"
#elif L==9	// pt
	"admin. centro"
#endif
	},
	{
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_UNKNOWN
	"admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"admin. institution"
#elif L==4	// es
	"admin. instituci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tablissement"
#elif L==6	// gn
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. istituzione"
#elif L==8	// pl
	"admin. instytucji"
#elif L==9	// pt
	"admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_FEMALE
	"admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"admin. institution"
#elif L==4	// es
	"admin. instituci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tablissement"
#elif L==6	// gn
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. istituzione"
#elif L==8	// pl
	"admin. instytucji"
#elif L==9	// pt
	"admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_MALE
	"admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"admin. institution"
#elif L==4	// es
	"admin. instituci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tablissement"
#elif L==6	// gn
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. istituzione"
#elif L==8	// pl
	"admin. instytucji"
#elif L==9	// pt
	"admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_ALL
	"admin. instituci&oacute;"
#elif L==2	// de
	"Admin. Hochschule"
#elif L==3	// en
	"admin. institution"
#elif L==4	// es
	"admin. instituci&oacute;n"
#elif L==5	// fr
	"admin. &eacute;tablissement"
#elif L==6	// gn
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	"ammin. istituzione"
#elif L==8	// pl
	"admin. instytucji"
#elif L==9	// pt
	"admin. institu&ccedil;&atilde;o"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_UNKNOWN
	"superusuari/a"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superuser"
#elif L==4	// es
	"superusuario/a"
#elif L==5	// fr
	"superutilisateur/se"
#elif L==6	// gn
	"superusuario/a"	// Okoteve traduccin
#elif L==7	// it
	"superutente"
#elif L==8	// pl
	"superuzytkownik"
#elif L==9	// pt
	"superutilizador/a"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_FEMALE
	"superusuaria"		// Necessita traduccio
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superuser"
#elif L==4	// es
	"superusuaria"
#elif L==5	// fr
	"superutilisateuse"
#elif L==6	// gn
	"superusuaria"		// Okoteve traduccin
#elif L==7	// it
	"superutente"
#elif L==8	// pl
	"superuzytkowniczka"
#elif L==9	// pt
	"superutilizadora"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_MALE
	"superusuari"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superuser"
#elif L==4	// es
	"superusuario"
#elif L==5	// fr
	"superutilisateur"
#elif L==6	// gn
	"superusuario"	// Okoteve traduccin
#elif L==7	// it
	"superutente"
#elif L==8	// pl
	"superu&zdot;ytkownika"
#elif L==9	// pt
	"superutilizador"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_ALL
	"superusuari/a"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superuser"
#elif L==4	// es
	"superusuario/a"
#elif L==5	// fr
	"superutilisateur/se"
#elif L==6	// gn
	"superusuario/a"	// Okoteve traduccin
#elif L==7	// it
	"superutente"
#elif L==8	// pl
	"superuzytkownicy"
#elif L==9	// pt
	"superutilizador/a"
#endif
	 }
};

const char *Txt_ROLES_PLURAL_Abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
{
	{
#if   L==1	// ca	// Rol_UNK - Usr_SEX_UNKNOWN
	"Desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidos"
#elif L==5	// fr
	"Inconnus"
#elif L==6	// gn
	"Desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecidos"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_FEMALE
	"Desconegudes"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidas"
#elif L==5	// fr
	"Inconnues"
#elif L==6	// gn
	"Desconocidas"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznana"
#elif L==9	// pt
	"Desconhecidas"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_MALE
	"Desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidos"
#elif L==5	// fr
	"Inconnus"
#elif L==6	// gn
	"Desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznany"
#elif L==9	// pt
	"Desconhecidos"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_ALL
	"Desconeguts"
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"Unknown"
#elif L==4	// es
	"Desconocidos"
#elif L==5	// fr
	"Inconnus"
#elif L==6	// gn
	"Desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuti"
#elif L==8	// pl
	"Nieznani"
#elif L==9	// pt
	"Desconhecidos"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_GST - Usr_SEX_UNKNOWN
	"Convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"Guests"
#elif L==4	// es
	"Invitados"
#elif L==5	// fr
	"Invit&eacute;s"
#elif L==6	// gn
	"Invitados"	// Okoteve traduccin
#elif L==7	// it
	"Ospiti"
#elif L==8	// pl
	"Zaproszenie"
#elif L==9	// pt
	"Convidados"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_FEMALE
	"Convidadas"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"Guests"
#elif L==4	// es
	"Invitadas"
#elif L==5	// fr
	"Invit&eacute;es"
#elif L==6	// gn
	"Invitadas"	// Okoteve traduccin
#elif L==7	// it
	"Ospiti"
#elif L==8	// pl
	"Zaproszona"
#elif L==9	// pt
	"Convidadas"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_MALE
	"Convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"Guests"
#elif L==4	// es
	"Invitados"
#elif L==5	// fr
	"Invit&eacute;s"
#elif L==6	// gn
	"Invitados"	// Okoteve traduccin
#elif L==7	// it
	"Ospiti"
#elif L==8	// pl
	"Zaproszony"
#elif L==9	// pt
	"Convidados"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_ALL
	"Convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"Guests"
#elif L==4	// es
	"Invitados"
#elif L==5	// fr
	"Invit&eacute;s"
#elif L==6	// gn
	"Invitados"	// Okoteve traduccin
#elif L==7	// it
	"Ospiti"
#elif L==8	// pl
	"Zaproszeni"
#elif L==9	// pt
	"Convidados"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_USR - Usr_SEX_UNKNOWN
	 "Usuaris"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "Users"
#elif L==4	// es
	 "Usuarios"
#elif L==5	// fr
	 "Utilisateurs"
#elif L==6	// gn
	 "Puruh&aacute;ra"
#elif L==7	// it
	 "Utenti"
#elif L==8	// pl
	 "U&zdot;ytkownicy"
#elif L==9	// pt
	 "Utilizadores"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_FEMALE
	 "Usu&agrave;ries"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "Users"
#elif L==4	// es
	 "Usuarias"
#elif L==5	// fr
	 "Utilisateurs"
#elif L==6	// gn
	 "Puruh&aacute;ra"
#elif L==7	// it
	 "Utenti"
#elif L==8	// pl
	 "U&zdot;ytkownicy"
#elif L==9	// pt
	 "Utilizadoras"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_MALE
	 "Usuaris"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "Users"
#elif L==4	// es
	 "Usuarios"
#elif L==5	// fr
	 "Utilisateurs"
#elif L==6	// gn
	 "Puruh&aacute;ra"
#elif L==7	// it
	 "Utenti"
#elif L==8	// pl
	 "U&zdot;ytkownicy"
#elif L==9	// pt
	 "Utilizadores"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_ALL
	 "Usuaris"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "Users"
#elif L==4	// es
	 "Usuarios"
#elif L==5	// fr
	 "Utilisateurs"
#elif L==6	// gn
	 "Puruh&aacute;ra"
#elif L==7	// it
	 "Utenti"
#elif L==8	// pl
	 "U&zdot;ytkownicy"
#elif L==9	// pt
	 "Utilizadores"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_STD - Usr_SEX_UNKNOWN
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiants"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studenti"
#elif L==8	// pl
	"Studenci"
#elif L==9	// pt
	"Estudantes"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_FEMALE
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiantes"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studentesse"
#elif L==8	// pl
	"Studentki"
#elif L==9	// pt
	"Estudantes"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_MALE
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiants"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studenti"
#elif L==8	// pl
	"Studenci"
#elif L==9	// pt
	"Estudantes"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_ALL
	"Estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"Students"
#elif L==4	// es
	"Estudiantes"
#elif L==5	// fr
	"&Eacute;tudiants"
#elif L==6	// gn
	"Temimbo'e"
#elif L==7	// it
	"Studenti"
#elif L==8	// pl
	"Studenci"
#elif L==9	// pt
	"Estudantes"
#endif
	},
	{
#if   L==1	// ca	// Rol_NET - Usr_SEX_UNKNOWN
	"Professors no editors"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teachers"
#elif L==4	// es
	"Profesores no editores"
#elif L==5	// fr
	"Enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"Professori non-editing"
#elif L==8	// pl
	"Nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professores n&atilde;o editores"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_FEMALE
	"Professores no editores"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teachers"
#elif L==4	// es
	"Profesoras no editoras"
#elif L==5	// fr
	"Enseignantes non-&eacute;ditrices"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editoras"	// Okoteve traduccin
#elif L==7	// it
	"Professoresse non-editing"
#elif L==8	// pl
	"Nauczycielki nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professoras n&atilde;o editoras"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_MALE
	"Professors no editors"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teachers"
#elif L==4	// es
	"Profesores no editores"
#elif L==5	// fr
	"Enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"Professori non-editing"
#elif L==8	// pl
	"Nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professores n&atilde;o editores"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_ALL
	"Professors no editors"
#elif L==2	// de
	"Nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"Non-editing teachers"
#elif L==4	// es
	"Profesores no editores"
#elif L==5	// fr
	"Enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"Mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"Professori non-editing"
#elif L==8	// pl
	"Nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"Professores n&atilde;o editores"
#endif
	},
	{
#if   L==1	// ca	// Rol_TCH - Usr_SEX_UNKNOWN
	"Professors"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesores"
#elif L==5	// fr
	"Enseignants"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Professores"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_FEMALE
	"Professores"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesoras"
#elif L==5	// fr
	"Enseignantes"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professoresse"
#elif L==8	// pl
	"Nauczycielki"
#elif L==9	// pt
	"Professoras"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_MALE
	"Professors"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesores"
#elif L==5	// fr
	"Enseignants"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Professores"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_ALL
	"Professors"
#elif L==2	// de
	"Lehrkraft"
#elif L==3	// en
	"Teachers"
#elif L==4	// es
	"Profesores"
#elif L==5	// fr
	"Enseignants"
#elif L==6	// gn
	"Mbo'eh&aacute;ra"
#elif L==7	// it
	"Professori"
#elif L==8	// pl
	"Nauczyciele"
#elif L==9	// pt
	"Professores"
#endif
	},
	{
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_UNKNOWN
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_FEMALE
	"Administradoras"	// Necessita traduccio
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradoras"
#elif L==5	// fr
	"Administrateuses"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorki"
#elif L==9	// pt
	"Administradoras"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_MALE
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_ALL
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_UNKNOWN
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_FEMALE
	"Administradoras"	// Necessita traduccio
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradoras"
#elif L==5	// fr
	"Administrateuses"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorki"
#elif L==9	// pt
	"Administradoras"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_MALE
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_ALL
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_UNKNOWN
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_FEMALE
	"Administradoras"	// Necessita traduccio
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradoras"
#elif L==5	// fr
	"Administrateuses"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorki"
#elif L==9	// pt
	"Administradoras"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_MALE
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_ALL
	"Administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"Administrators"
#elif L==4	// es
	"Administradores"
#elif L==5	// fr
	"Administrateurs"
#elif L==6	// gn
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"Amministratori"
#elif L==8	// pl
	"Administratorzy"
#elif L==9	// pt
	"Administradores"
#endif
	},
	{
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_UNKNOWN
	"Superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superusers"
#elif L==4	// es
	"Superusuarios"
#elif L==5	// fr
	"Superutilisateurs"
#elif L==6	// gn
	"Superusuarios"	// Okoteve traduccin
#elif L==7	// it
	"Superutenti"
#elif L==8	// pl
	"Superu&zdot;ytkownika"
#elif L==9	// pt
	"Superutilizadores"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_FEMALE
	"Superusuarias"	// Necessita traduccio
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superusers"
#elif L==4	// es
	"Superusuarias"
#elif L==5	// fr
	"Superutilisateuses"
#elif L==6	// gn
	"Superusuarias"	// Okoteve traduccin
#elif L==7	// it
	"Superutenti"
#elif L==8	// pl
	"Superuzytkowniczka"
#elif L==9	// pt
	"Superutilizadoras"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_MALE
	"Superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superusers"
#elif L==4	// es
	"Superusuarios"
#elif L==5	// fr
	"Superutilisateurs"
#elif L==6	// gn
	"Superusuarios"	// Okoteve traduccin
#elif L==7	// it
	"Superutenti"
#elif L==8	// pl
	"Superuzytkownik"
#elif L==9	// pt
	"Superutilizadores"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_ALL
	"Superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"Superusers"
#elif L==4	// es
	"Superusuarios"
#elif L==5	// fr
	"Superutilisateurs"
#elif L==6	// gn
	"Superusuarios"	// Okoteve traduccin
#elif L==7	// it
	"Superutenti"
#elif L==8	// pl
	"Superuzytkownicy"
#elif L==9	// pt
	"Superutilizadores"
#endif
	 }
   };

const char *Txt_ROLES_PLURAL_BRIEF_Abc[Rol_NUM_ROLES] =
   {
#if   L==1	// ca	// Rol_UNK
	"Desc."
#elif L==2	// de
	"Unbe."
#elif L==3	// en
	"Unkn."
#elif L==4	// es
	"Desc."
#elif L==5	// fr
	"Inco."
#elif L==6	// gn
	"Desc."	// Okoteve traduccin
#elif L==7	// it
	"Scon."
#elif L==8	// pl
	"Niez."
#elif L==9	// pt
	"Desc."
#endif
	,
#if   L==1	// ca	// Rol_GST
	"Conv."
#elif L==2	// de
	"Eing."
#elif L==3	// en
	"Gues."
#elif L==4	// es
	"Invi."
#elif L==5	// fr
	"Invi."
#elif L==6	// gn
	"Invi."	// Okoteve traduccin
#elif L==7	// it
	"Ospi."
#elif L==8	// pl
	"Zapr."
#elif L==9	// pt
	"Conv."
#endif
	,
#if   L==1	// ca	// Rol_USR
	 "Usua."
#elif L==2	// de
	 "Benu."
#elif L==3	// en
	 "Users"
#elif L==4	// es
	 "Usua."
#elif L==5	// fr
	 "Util."
#elif L==6	// gn
	 "Puru."
#elif L==7	// it
	 "Uten."
#elif L==8	// pl
	 "U&zdot;yt."
#elif L==9	// pt
	 "Util."
#endif
	,
#if   L==1	// ca	// Rol_STD
	"Estu."
#elif L==2	// de
	"Stud."
#elif L==3	// en
	"Stud."
#elif L==4	// es
	"Estu."
#elif L==5	// fr
	"&Eacute;tu."
#elif L==6	// gn
	"Temi."
#elif L==7	// it
	"Stud."
#elif L==8	// pl
	"Stud."
#elif L==9	// pt
	"Estu."
#endif
	,
#if   L==1	// ca	// Rol_NET
	"Prof.n.e."
#elif L==2	// de
	"N.b.Lehr."
#elif L==3	// en
	"N.e.teac."
#elif L==4	// es
	"Prof.n.e."
#elif L==5	// fr
	"Ense.n.&eacute;."
#elif L==6	// gn
	"Mbo'e.n.e."	// Okoteve traduccin
#elif L==7	// it
	"Prof.n.e."
#elif L==8	// pl
	"Nauc.n.e."
#elif L==9	// pt
	"Prof.n.e."
#endif
	,
#if   L==1	// ca	// Rol_TCH
	"Prof."
#elif L==2	// de
	"Lehr."
#elif L==3	// en
	"Teac."
#elif L==4	// es
	"Prof."
#elif L==5	// fr
	"Ense."
#elif L==6	// gn
	"Mbo'e."
#elif L==7	// it
	"Prof."
#elif L==8	// pl
	"Nauc."
#elif L==9	// pt
	"Prof."
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM
	"Admi."
#elif L==2	// de
	"Admi."
#elif L==3	// en
	"Admi."
#elif L==4	// es
	"Admi."
#elif L==5	// fr
	"Admi."
#elif L==6	// gn
	"&Ntilde;ang."
#elif L==7	// it
	"Ammi."
#elif L==8	// pl
	"Admi."
#elif L==9	// pt
	"Admi."
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM
	"Admi."
#elif L==2	// de
	"Admi."
#elif L==3	// en
	"Admi."
#elif L==4	// es
	"Admi."
#elif L==5	// fr
	"Admi."
#elif L==6	// gn
	"&Ntilde;ang."
#elif L==7	// it
	"Ammi."
#elif L==8	// pl
	"Admi."
#elif L==9	// pt
	"Admi."
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM
	"Admi."
#elif L==2	// de
	"Admi."
#elif L==3	// en
	"Admi."
#elif L==4	// es
	"Admi."
#elif L==5	// fr
	"Admi."
#elif L==6	// gn
	"&Ntilde;ang."
#elif L==7	// it
	"Ammi."
#elif L==8	// pl
	"Admi."
#elif L==9	// pt
	"Admi."
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM
	"Supe."
#elif L==2	// de
	"Supe."
#elif L==3	// en
	"Supe."
#elif L==4	// es
	"Supe."
#elif L==5	// fr
	"Supe."
#elif L==6	// gn
	"Supe."	// Okoteve traduccin
#elif L==7	// it
	"Supe."
#elif L==8	// pl
	"Supe."
#elif L==9	// pt
	"Supe."
#endif
	};

const char *Txt_ROLES_PLURAL_abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
{
	{
#if   L==1	// ca	// Rol_UNK - Usr_SEX_UNKNOWN
	"desconocidos"	// Necessita traduccio
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidos"
#elif L==5	// fr
	"inconnus"
#elif L==6	// gn
	"desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecidos"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_FEMALE
	"desconocidas"	// Necessita traduccio
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidas"
#elif L==5	// fr
	"inconnues"
#elif L==6	// gn
	"desconocidas"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznana"
#elif L==9	// pt
	"desconhecidas"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_MALE
	"desconocidos"	// Necessita traduccio
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidos"
#elif L==5	// fr
	"inconnus"
#elif L==6	// gn
	"desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznany"
#elif L==9	// pt
	"desconhecidos"
#endif
	,
#if   L==1	// ca	// Rol_UNK - Usr_SEX_ALL
	"desconocidos"	// Necessita traduccio
#elif L==2	// de
	"Unbekannte"
#elif L==3	// en
	"unknown"
#elif L==4	// es
	"desconocidos"
#elif L==5	// fr
	"inconnus"
#elif L==6	// gn
	"desconocidos"	// Okoteve traduccin
#elif L==7	// it
	"sconosciuti"
#elif L==8	// pl
	"nieznani"
#elif L==9	// pt
	"desconhecidos"
#endif
	},
	{
#if   L==1	// ca	// Rol_GST - Usr_SEX_UNKNOWN
	"convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"guests"
#elif L==4	// es
	"invitados"
#elif L==5	// fr
	"invit&eacute;s"
#elif L==6	// gn
	"invitados"	// Okoteve traduccin
#elif L==7	// it
	"ospiti"
#elif L==8	// pl
	"zaproszenie"
#elif L==9	// pt
	"convidados"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_FEMALE
	"invitadas"	// Necessita traduccio
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"guests"
#elif L==4	// es
	"invitadas"
#elif L==5	// fr
	"invit&eacute;es"
#elif L==6	// gn
	"invitadas"	// Okoteve traduccin
#elif L==7	// it
	"ospiti"
#elif L==8	// pl
	"zaproszona"
#elif L==9	// pt
	"convidadas"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_MALE
	"convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"guests"
#elif L==4	// es
	"invitados"
#elif L==5	// fr
	"invit&eacute;s"
#elif L==6	// gn
	"invitados"	// Okoteve traduccin
#elif L==7	// it
	"ospiti"
#elif L==8	// pl
	"zaproszony"
#elif L==9	// pt
	"convidados"
#endif
	,
#if   L==1	// ca	// Rol_GST - Usr_SEX_ALL
	"convidats"
#elif L==2	// de
	"Eingeladen"
#elif L==3	// en
	"guests"
#elif L==4	// es
	"invitados"
#elif L==5	// fr
	"invit&eacute;s"
#elif L==6	// gn
	"invitados"	// Okoteve traduccin
#elif L==7	// it
	"ospiti"
#elif L==8	// pl
	"zaproszeni"
#elif L==9	// pt
	"convidados"
#endif
	},
	{
#if   L==1	// ca	// Rol_USR - Usr_SEX_UNKNOWN
	 "usuaris"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "users"
#elif L==4	// es
	 "usuarios"
#elif L==5	// fr
	 "utilisateurs"
#elif L==6	// gn
	 "puruh&aacute;ra"
#elif L==7	// it
	 "utenti"
#elif L==8	// pl
	 "u&zdot;ytkownicy"
#elif L==9	// pt
	 "utilizadores"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_FEMALE
	 "usu&agrave;ries"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "users"
#elif L==4	// es
	 "usuarias"
#elif L==5	// fr
	 "utilisateurs"
#elif L==6	// gn
	 "puruh&aacute;ra"
#elif L==7	// it
	 "utenti"
#elif L==8	// pl
	 "u&zdot;ytkownicy"
#elif L==9	// pt
	 "utilizadoras"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_MALE
	 "usuaris"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "users"
#elif L==4	// es
	 "usuarios"
#elif L==5	// fr
	 "utilisateurs"
#elif L==6	// gn
	 "puruh&aacute;ra"
#elif L==7	// it
	 "utenti"
#elif L==8	// pl
	 "u&zdot;ytkownicy"
#elif L==9	// pt
	 "utilizadores"
#endif
	,
#if   L==1	// ca	// Rol_USR - Usr_SEX_ALL
	 "usuaris"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "users"
#elif L==4	// es
	 "usuarios"
#elif L==5	// fr
	 "utilisateurs"
#elif L==6	// gn
	 "puruh&aacute;ra"
#elif L==7	// it
	 "utenti"
#elif L==8	// pl
	 "u&zdot;ytkownicy"
#elif L==9	// pt
	 "utilizadores"
#endif
	},
	{
#if   L==1	// ca	// Rol_STD - Usr_SEX_UNKNOWN
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiants"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studenti"
#elif L==8	// pl
	"studenci"
#elif L==9	// pt
	"estudantes"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_FEMALE
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiantes"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studentesse"
#elif L==8	// pl
	"studenka"
#elif L==9	// pt
	"estudantes"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_MALE
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiants"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studenti"
#elif L==8	// pl
	"studentci"
#elif L==9	// pt
	"estudantes"
#endif
	,
#if   L==1	// ca	// Rol_STD - Usr_SEX_ALL
	"estudiants"
#elif L==2	// de
	"Studenten"
#elif L==3	// en
	"students"
#elif L==4	// es
	"estudiantes"
#elif L==5	// fr
	"&eacute;tudiants"
#elif L==6	// gn
	"temimbo'e"
#elif L==7	// it
	"studenti"
#elif L==8	// pl
	"studentci"
#elif L==9	// pt
	"estudantes"
#endif
	},
	{
#if   L==1	// ca	// Rol_NET - Usr_SEX_UNKNOWN
	"professors no editors"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teachers"
#elif L==4	// es
	"profesores no editores"
#elif L==5	// fr
	"enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"professori non-editing"
#elif L==8	// pl
	"nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"professores n&atilde;o editores"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_FEMALE
	"professores no editores"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teachers"
#elif L==4	// es
	"profesoras no editoras"
#elif L==5	// fr
	"enseignantes non-&eacute;ditrices"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editoras"	// Okoteve traduccin
#elif L==7	// it
	"professoresse non-editing"
#elif L==8	// pl
	"nauczycielki nie edytuj&aogon;cy"
#elif L==9	// pt
	"professoras n&atilde;o editoras"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_MALE
	"professors no editors"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teachers"
#elif L==4	// es
	"profesores no editores"
#elif L==5	// fr
	"enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"professori non-editing"
#elif L==8	// pl
	"nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"professores n&atilde;o editores"
#endif
	,
#if   L==1	// ca	// Rol_NET - Usr_SEX_ALL
	"professors no editors"
#elif L==2	// de
	"nicht bearbeiteter Lehrkraft"
#elif L==3	// en
	"non-editing teachers"
#elif L==4	// es
	"profesores no editores"
#elif L==5	// fr
	"enseignants non-&eacute;diteurs"
#elif L==6	// gn
	"mbo'eh&aacute;ra no editores"	// Okoteve traduccin
#elif L==7	// it
	"professori non-editing"
#elif L==8	// pl
	"nauczyciele nie edytuj&aogon;cy"
#elif L==9	// pt
	"professores n&atilde;o editores"
#endif
	},
	{
#if   L==1	// ca	// Rol_TCH - Usr_SEX_UNKNOWN
	"professors"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"teachers"
#elif L==4	// es
	"profesores"
#elif L==5	// fr
	"enseignants"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professori"
#elif L==8	// pl
	"nauczyciele"
#elif L==9	// pt
	"professores"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_FEMALE
	"professores"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"teachers"
#elif L==4	// es
	"profesoras"
#elif L==5	// fr
	"enseignantes"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professoresse"
#elif L==8	// pl
	"nauczycielki"
#elif L==9	// pt
	"professoras"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_MALE
	"professors"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"teachers"
#elif L==4	// es
	"profesores"
#elif L==5	// fr
	"enseignants"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professori"
#elif L==8	// pl
	"nauczyciele"
#elif L==9	// pt
	"professores"
#endif
	,
#if   L==1	// ca	// Rol_TCH - Usr_SEX_ALL
	"professors"
#elif L==2	// de
	"Lehrkr&auml;fte"
#elif L==3	// en
	"teachers"
#elif L==4	// es
	"profesores"
#elif L==5	// fr
	"enseignants"
#elif L==6	// gn
	"mbo'eh&aacute;ra"
#elif L==7	// it
	"professori"
#elif L==8	// pl
	"nauczyciele"
#elif L==9	// pt
	"professores"
#endif
	},
	{
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_UNKNOWN
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_FEMALE
	"administradoras"	// Necessita traduccio
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradoras"
#elif L==5	// fr
	"administrateuses"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorki"
#elif L==9	// pt
	"administradoras"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_MALE
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#endif
	,
#if   L==1	// ca	// Rol_DEG_ADM - Usr_SEX_ALL
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#endif
	},
	{
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_UNKNOWN
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_FEMALE
	"administradoras"	// Necessita traduccio
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradoras"
#elif L==5	// fr
	"administrateuses"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorki"
#elif L==9	// pt
	"administradoras"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_MALE
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#endif
	,
#if   L==1	// ca	// Rol_CTR_ADM - Usr_SEX_ALL
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#endif
	 },
	 {
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_UNKNOWN
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_FEMALE
	"administradoras"	// Necessita traduccio
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradoras"
#elif L==5	// fr
	"administrateuses"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorki"
#elif L==9	// pt
	"administradoras"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_MALE
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#endif
	,
#if   L==1	// ca	// Rol_INS_ADM - Usr_SEX_ALL
	"administradors"
#elif L==2	// de
	"Administratoren"
#elif L==3	// en
	"administrators"
#elif L==4	// es
	"administradores"
#elif L==5	// fr
	"administrateurs"
#elif L==6	// gn
	"&ntilde;angarekoh&aacute;ra"
#elif L==7	// it
	"amministratori"
#elif L==8	// pl
	"administratorzy"
#elif L==9	// pt
	"administradores"
#endif
	},
	{
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_UNKNOWN
	"superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superusers"
#elif L==4	// es
	"superusuarios"
#elif L==5	// fr
	"superutilisateurs"
#elif L==6	// gn
	"superusuarios"		// Okoteve traduccin
#elif L==7	// it
	"superutenti"
#elif L==8	// pl
	"superusers"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"superutilizadores"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_FEMALE
	"superusuarias"		// Necessita traduccio
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superusers"
#elif L==4	// es
	"superusuarias"
#elif L==5	// fr
	"superutilisateuses"
#elif L==6	// gn
	"superusuarias"		// Okoteve traduccin
#elif L==7	// it
	"superutenti"
#elif L==8	// pl
	"superusers"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"superutilizadoras"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_MALE
	"superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superusers"
#elif L==4	// es
	"superusuarios"
#elif L==5	// fr
	"superutilisateurs"
#elif L==6	// gn
	"superusuarios"		// Okoteve traduccin
#elif L==7	// it
	"superutenti"
#elif L==8	// pl
	"superusers"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"superutilizadores"
#endif
	,
#if   L==1	// ca	// Rol_SYS_ADM - Usr_SEX_ALL
	"superusuaris"
#elif L==2	// de
	"Superuser"
#elif L==3	// en
	"superusers"
#elif L==4	// es
	"superusuarios"
#elif L==5	// fr
	"superutilisateurs"
#elif L==6	// gn
	"superusuarios"		// Okoteve traduccin
#elif L==7	// it
	"superutenti"
#elif L==8	// pl
	"superusers"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"superutilizadores"
#endif
	 }
   };

const char *Txt_Rooms_with_users =
#if   L==1	// ca
	"Salas con usuarios";		// Necessita traduccio
#elif L==2	// de
	"Rooms with users";		// Need bersetzung
#elif L==3	// en
	"Rooms with users";
#elif L==4	// es
	"Salas con usuarios";
#elif L==5	// fr
	"Rooms with users";		// Besoin de traduction
#elif L==6	// gn
	"Salas con usuarios";		// Okoteve traduccin
#elif L==7	// it
	"Rooms con utenti";
#elif L==8	// pl
	"Rooms with users";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Quartos com usu&aacute;rios";
#endif

const char *Txt_ROOT_FOLDER_EXTERNAL_NAMES[Brw_NUM_TYPES_FILE_BROWSER] =
   {
	 // Brw_UNKNOWN
	 "",
	 // Brw_SHOW_DOC_CRS
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_SHOW_MRK_CRS
#if   L==1	// ca
	 "qualificacions"
#elif L==2	// de
	 "Bewertungen"
#elif L==3	// en
	 "marks"
#elif L==4	// es
	 "calificaciones"
#elif L==5	// fr
	 "notes"
#elif L==6	// gn
	 "calificaciones"	// Okoteve traduccin
#elif L==7	// it
	 "risultati"
#elif L==8	// pl
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "notas"
#endif
	 ,
	 // Brw_ADMI_DOC_CRS
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_SHR_CRS
#if   L==1	// ca
	 "compartits"
#elif L==2	// de
	 "Freigegebene"
#elif L==3	// en
	 "shared"
#elif L==4	// es
	 "compartidos"
#elif L==5	// fr
	 "partages"
#elif L==6	// gn
	 "compartidos"			// Okoteve traduccin
#elif L==7	// it
	 "condivisi"
#elif L==8	// pl
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "comum"
#endif
	 ,
	 // Brw_ADMI_SHR_GRP
#if   L==1	// ca
	 "compartits"
#elif L==2	// de
	 "Freigegebene"
#elif L==3	// en
	 "shared"
#elif L==4	// es
	 "compartidos"
#elif L==5	// fr
	 "partages"
#elif L==6	// gn
	 "compartidos"			// Okoteve traduccin
#elif L==7	// it
	 "condivisi"
#elif L==8	// pl
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "comum"
#endif
	 ,
	 // Brw_ADMI_WRK_USR
#if   L==1	// ca
	 "treballs"
#elif L==2	// de
	 "Hausarbeiten"
#elif L==3	// en
	 "homework"
#elif L==4	// es
	 "trabajos"
#elif L==5	// fr
	 "travaux"
#elif L==6	// gn
	 "tembiapo"
#elif L==7	// it
	 "lavori"
#elif L==8	// pl
	 "homework"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "trabalhos"
#endif
	 ,
	 // Brw_ADMI_WRK_CRS
#if   L==1	// ca
	 "treballs"
#elif L==2	// de
	 "Hausarbeiten"
#elif L==3	// en
	 "homework"
#elif L==4	// es
	 "trabajos"
#elif L==5	// fr
	 "travaux"
#elif L==6	// gn
	 "tembiapo"
#elif L==7	// it
	 "lavori"
#elif L==8	// pl
	 "homework"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "trabalhos"
#endif
	 ,
	 // Brw_ADMI_MRK_CRS
#if   L==1	// ca
	 "qualificacions"
#elif L==2	// de
	 "Bewertungen"
#elif L==3	// en
	 "marks"
#elif L==4	// es
	 "calificaciones"
#elif L==5	// fr
	 "notes"
#elif L==6	// gn
	 "calificaciones"	// Okoteve traduccin
#elif L==7	// it
	 "risultati"
#elif L==8	// pl
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "notas"
#endif
	 ,
	 // Brw_ADMI_BRF_USR
#if   L==1	// ca
	 "maleti"
#elif L==2	// de
	 "Aktentasche"
#elif L==3	// en
	 "briefcase"
#elif L==4	// es
	 "maletin"
#elif L==5	// fr
	 "porte-documents"
#elif L==6	// gn
	 "maletin"		// Okoteve traduccin
#elif L==7	// it
	 "cartella"
#elif L==8	// pl
	 "briefcase"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "pasta"
#endif
	 ,
	 // Brw_SHOW_DOC_GRP
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_DOC_GRP
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_SHOW_MRK_GRP
#if   L==1	// ca
	 "qualificacions"	// Necessita traduccio
#elif L==2	// de
	 "Bewertungen"
#elif L==3	// en
	 "marks"
#elif L==4	// es
	 "calificaciones"
#elif L==5	// fr
	 "notes"
#elif L==6	// gn
	 "calificaciones"	// Okoteve traduccin
#elif L==7	// it
	 "risultati"
#elif L==8	// pl
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "notas"
#endif
	 ,
	 // Brw_ADMI_MRK_GRP
#if   L==1	// ca
	 "qualificacions"
#elif L==2	// de
	 "Bewertungen"
#elif L==3	// en
	 "marks"
#elif L==4	// es
	 "calificaciones"
#elif L==5	// fr
	 "notes"
#elif L==6	// gn
	 "calificaciones"	// Okoteve traduccin
#elif L==7	// it
	 "risultati"
#elif L==8	// pl
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "notas"
#endif
	 ,
	 // Brw_ADMI_ASG_USR
#if   L==1	// ca
	 "activitats"
#elif L==2	// de
	 "Aufgaben"
#elif L==3	// en
	 "assignments"
#elif L==4	// es
	 "actividades"
#elif L==5	// fr
	 "activites"
#elif L==6	// gn
	 "tar&eacute;a"
#elif L==7	// it
	 "attivit&agrave;"
#elif L==8	// pl
	 "assignments"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "atividades"
#endif
	 ,
	 // Brw_ADMI_ASG_CRS
#if   L==1	// ca
	 "activitats"
#elif L==2	// de
	 "Aufgaben"
#elif L==3	// en
	 "assignments"
#elif L==4	// es
	 "actividades"
#elif L==5	// fr
	 "activites"
#elif L==6	// gn
	 "tar&eacute;a"
#elif L==7	// it
	 "attivit&agrave;"
#elif L==8	// pl
	 "assignments"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "atividades"
#endif
	 ,
	 // Brw_SHOW_DOC_DEG
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_DOC_DEG
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_SHOW_DOC_CTR
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_DOC_CTR
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_SHOW_DOC_INS
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_DOC_INS
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_SHR_DEG
#if   L==1	// ca
	 "compartits"
#elif L==2	// de
	 "Freigegebene"
#elif L==3	// en
	 "shared"
#elif L==4	// es
	 "compartidos"
#elif L==5	// fr
	 "partages"
#elif L==6	// gn
	 "compartidos"			// Okoteve traduccin
#elif L==7	// it
	 "condivisi"
#elif L==8	// pl
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "comum"
#endif
	 ,
	 // Brw_ADMI_SHR_CTR
#if   L==1	// ca
	 "compartits"
#elif L==2	// de
	 "Freigegebene"
#elif L==3	// en
	 "shared"
#elif L==4	// es
	 "compartidos"
#elif L==5	// fr
	 "partages"
#elif L==6	// gn
	 "compartidos"			// Okoteve traduccin
#elif L==7	// it
	 "condivisi"
#elif L==8	// pl
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "comum"
#endif
	 ,
	 // Brw_ADMI_SHR_INS
#if   L==1	// ca
	 "compartits"
#elif L==2	// de
	 "Freigegebene"
#elif L==3	// en
	 "shared"
#elif L==4	// es
	 "compartidos"
#elif L==5	// fr
	 "partages"
#elif L==6	// gn
	 "compartidos"			// Okoteve traduccin
#elif L==7	// it
	 "condivisi"
#elif L==8	// pl
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "comum"
#endif
	 ,
	 // Brw_ADMI_TCH_CRS
#if   L==1	// ca
	"privats"
#elif L==2	// de
	"Private"
#elif L==3	// en
	"private"
#elif L==4	// es
	"privados"
#elif L==5	// fr
	"priv&eacute;s"
#elif L==6	// gn
	"privados"	// Okoteve traduccin
#elif L==7	// it
	"privati"
#elif L==8	// pl
	"prywatne"
#elif L==9	// pt
	"privados"
#endif
	 ,
	 // Brw_ADMI_TCH_GRP
#if   L==1	// ca
	"privats"
#elif L==2	// de
	"Private"
#elif L==3	// en
	"private"
#elif L==4	// es
	"privados"
#elif L==5	// fr
	"priv&eacute;s"
#elif L==6	// gn
	"privados"	// Okoteve traduccin
#elif L==7	// it
	"privati"
#elif L==8	// pl
	"prywatne"
#elif L==9	// pt
	"privados"
#endif
	 ,
	 // Brw_ADMI_DOC_PRJ
#if   L==1	// ca
	 "documents"
#elif L==2	// de
	 "Dokumente"
#elif L==3	// en
	 "documents"
#elif L==4	// es
	 "documentos"
#elif L==5	// fr
	 "documents"
#elif L==6	// gn
	 "kuatia"
#elif L==7	// it
	 "documenti"
#elif L==8	// pl
	 "dokumenty"
#elif L==9	// pt
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_ASS_PRJ
#if   L==1	// ca
	 "avaluacio"
#elif L==2	// de
	 "Bewertung"
#elif L==3	// en
	 "assessment"
#elif L==4	// es
	 "evaluacion"
#elif L==5	// fr
	 "evaluation"
#elif L==6	// gn
	 "mbohepy"
#elif L==7	// it
	 "valutazione"
#elif L==8	// pl
	 "ocena"
#elif L==9	// pt
	 "avaliacao"
#endif
	};

const char *Txt_Save_as =
#if   L==1	// ca
	"Guardar com";
#elif L==2	// de
	"Speichern unter";
#elif L==3	// en
	"Save as";
#elif L==4	// es
	"Guardar como";
#elif L==5	// fr
	"Enregistrer sous";
#elif L==6	// gn
	"Guardar como";	// Okoteve traduccin
#elif L==7	// it
	"Salva come";
#elif L==8	// pl
	"Zapisz jako";
#elif L==9	// pt
	"Salvar como";
#endif

const char *Txt_Save_changes =
#if   L==1	// ca
	"Desar canvis";
#elif L==2	// de
	"&Auml;nderungen speichern";
#elif L==3	// en
	"Save changes";
#elif L==4	// es
	"Guardar cambios";
#elif L==5	// fr
	"Enregistrer les modifications";
#elif L==6	// gn
	"&Ntilde;ongatu";
#elif L==7	// it
	"Salva modifiche";
#elif L==8	// pl
	"Zapisz zmiany";
#elif L==9	// pt
	"Salvar altera&ccedil;&otilde;es";
#endif

const char *Txt_Save_file_properties =
#if   L==1	// ca
	"Guardar les propietats del fitxer";
#elif L==2	// de
	"Speichern Dateieigenschaften";
#elif L==3	// en
	"Save file properties";
#elif L==4	// es
	"Guardar propiedades del archivo";
#elif L==5	// fr
	"Enregistrer les propri&eacute;t&eacute;s du fichier";
#elif L==6	// gn
	"Guardar propiedades del archivo";	// Okoteve traduccin
#elif L==7	// it
	"Salva le propriet&agrave; dei file";
#elif L==8	// pl
	"Zapisz w&lstrok;a&sacute;ciwo&sacute;ci pliku";
#elif L==9	// pt
	"Salvar as propriedades do arquivo";
#endif

const char *Txt_Scope =
#if   L==1	// ca
	"&Agrave;mbit";
#elif L==2	// de
	"Geltungsbereich";
#elif L==3	// en
	"Scope";
#elif L==4	// es
	"&Aacute;mbito";
#elif L==5	// fr
	"Port&eacute;e";
#elif L==6	// gn
	"&Aacute;mbito";	// Okoteve traduccin
#elif L==7	// it
	"Ambito";
#elif L==8	// pl
	"Scope";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"&Acirc;mbito";
#endif

const char *Txt_Score =
#if   L==1	// ca
	"Puntuaci&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Ergebnis";
#elif L==3	// en
	"Score";
#elif L==4	// es
	"Puntuaci&oacute;n";
#elif L==5	// fr
	"Note";
#elif L==6	// gn
	"Puntuaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Punteggio";
#elif L==8	// pl
	"Score";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o";
#endif

const char *Txt_Search =
#if   L==1	// ca
	"Cercar";
#elif L==2	// de
	"Suchen";
#elif L==3	// en
	"Search";
#elif L==4	// es
	"Buscar";
#elif L==5	// fr
	"Chercher";
#elif L==6	// gn
	"Heka";
#elif L==7	// it
	"Cercare";
#elif L==8	// pl
	"Szukaj";
#elif L==9	// pt
	"Procurar";
#endif

const char *Txt_Search_courses =
#if   L==1	// ca
	"Cercar assignatures";
#elif L==2	// de
	"Suchen Kursen";
#elif L==3	// en
	"Search courses";
#elif L==4	// es
	"Buscar asignaturas";
#elif L==5	// fr
	"Chercher mati&egrave;res";
#elif L==6	// gn
	"Heka mbo'esyry";
#elif L==7	// it
	"Cercare corsi";
#elif L==8	// pl
	"Szukaj kurs&oacute;w";
#elif L==9	// pt
	"Procurar disciplinas";
#endif

const char *Txt_See_full_notice =
#if   L==1	// ca
	"Veure avs complet";
#elif L==2	// de
	"Siehe voll Ank&uuml;ndigung";
#elif L==3	// en
	"See full notice";
#elif L==4	// es
	"Ver aviso completo";
#elif L==5	// fr
	"Voir l'avis complet";
#elif L==6	// gn
	"Ver aviso completo";	// Okoteve traduccin
#elif L==7	// it
	"Vedi avviso completo ";
#elif L==8	// pl
	"Wiadomo&sacute;ci ca&lstrok;y obwieszczenie";
#elif L==9	// pt
	"Ver aviso completo";
#endif

const char *Txt_See_less =
#if   L==1	// ca
	 "Veure menys";
#elif L==2	// de
	 "Zeige weniger";
#elif L==3	// en
	 "See less";
#elif L==4	// es
	 "Ver menos";
#elif L==5	// fr
	 "Voir moins";
#elif L==6	// gn
	 "Ver menos";	// Okoteve traduccin
#elif L==7	// it
	 "Vedi meno";
#elif L==8	// pl
	 "Zobacz mniej";
#elif L==9	// pt
	 "Ver menos";
#endif

const char *Txt_See_message =
#if   L==1	// ca
	"Veure missatge";
#elif L==2	// de
	"Siehe Nachricht";
#elif L==3	// en
	"See message";
#elif L==4	// es
	"Ver mensaje";
#elif L==5	// fr
	"Voir message";
#elif L==6	// gn
	"Ver mensaje";	// Okoteve traduccin
#elif L==7	// it
	"Vedi messaggio";
#elif L==8	// pl
	"See message";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ver mensagem";
#endif

const char *Txt_See_messages_received =
#if   L==1	// ca
	"Veure missatges rebuts";
#elif L==2	// de
	"Siehe Empfangene Nachrichten";
#elif L==3	// en
	"See messages received";
#elif L==4	// es
	"Ver mensajes recibidos";
#elif L==5	// fr
	"Voir messages re&ccedil;us";
#elif L==6	// gn
	"Ver mensajes recibidos";	// Okoteve traduccin
#elif L==7	// it
	"Vedi messaggi ricevuti";
#elif L==8	// pl
	"Wiadomo&sacute;ci odebrane";
#elif L==9	// pt
	"Ver mensagens recebidas";
#endif

const char *Txt_See_more =
#if   L==1	// ca
	 "Veure m&eacute;s";
#elif L==2	// de
	 "Siehe mehr";
#elif L==3	// en
	 "See more";
#elif L==4	// es
	 "Ver m&aacute;s";
#elif L==5	// fr
	 "Voir plus";
#elif L==6	// gn
	 "Ver m&aacute;s";	// Okoteve traduccin
#elif L==7	// it
	 "Vedi pi&ugrave;";
#elif L==8	// pl
	 "Zobacz wi&eogon;cej";
#elif L==9	// pt
	 "Ver mais";
#endif

const char *Txt_See_new_activity =
#if   L==1	// ca
	 "Veure nova activitat";
#elif L==2	// de
	 "Siehe neue Aktivit&auml;t";
#elif L==3	// en
	 "See new activity";
#elif L==4	// es
	 "Ver nueva actividad";
#elif L==5	// fr
	 "Voir nouvelle activit&eacute;";
#elif L==6	// gn
	 "Ver nueva actividad";	// Okoteve traduccin
#elif L==7	// it
	 "Vedi nuova attivit&agrave;";
#elif L==8	// pl
	 "Zobacz now&aogon; aktywno&sacute;&cacute;";
#elif L==9	// pt
	 "Ver nova atividade";
#endif

const char *Txt_See_notifications =
#if   L==1	// ca
	"Ver notificaciones";	// Necessita traduccio
#elif L==2	// de
	"Siehe Anmeldungen";
#elif L==3	// en
	"See notifications";
#elif L==4	// es
	"Ver notificaciones";
#elif L==5	// fr
	"Voir notifications";
#elif L==6	// gn
	"Ver notificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Vedi notifiche";
#elif L==8	// pl
	"See notifications";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ver notifica&ccedil;&otilde;es";
#endif

const char *Txt_See_only_the_latest_COMMENTS =
#if   L==1	// ca
	 "Veure nom&eacute;s els &uacute;ltims";
#elif L==2	// de
	 "Nur die neuesten anzeigen";
#elif L==3	// en
	 "See only the latest";
#elif L==4	// es
	 "Ver s&oacute;lo los &uacute;ltimos";
#elif L==5	// fr
	 "Voir seulement les derni&egrave;res";
#elif L==6	// gn
	 "Ver s&oacute;lo los &uacute;ltimos";	// Okoteve traduccin
#elif L==7	// it
	 "Vedi solo le ultime";
#elif L==8	// pl
	 "Zobacz tylko najnowsze";
#elif L==9	// pt
	 "Ver apenas os &uacute;ltimos";
#endif

const char *Txt_See_the_previous_X_COMMENTS =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	 "Veure els %u anteriors";
#elif L==2	// de
	 "Siehe die vorherigen %u";
#elif L==3	// en
	 "See the previous %u";
#elif L==4	// es
	 "Ver los %u anteriores";
#elif L==5	// fr
	 "Voir les %u pr&eacute;c&eacute;dents";
#elif L==6	// gn
	 "Ver los %u anteriores";	// Okoteve traduccin
#elif L==7	// it
	 "Vedi i precedente %u";
#elif L==8	// pl
	 "Zobacz poprzednie %u";
#elif L==9	// pt
	 "Ver os %u anteriores";
#endif

const char *Txt_Select_another_country =
#if   L==1	// ca
	"Seleccionar un altre pas";
#elif L==2	// de
	"W&auml;hlen Sie eine anderes Land";
#elif L==3	// en
	"Select another country";
#elif L==4	// es
	"Seleccionar otro pa&iacute;s";
#elif L==5	// fr
	"S&eacute;lectionnez un autre pays";
#elif L==6	// gn
	"Seleccionar otro pa&iacute;s";		// Okoteve traduccin
#elif L==7	// it
	"Selezionare un altro paese";
#elif L==8	// pl
	"Wybierz inny kraj";
#elif L==9	// pt
	"Selecionar outro pa&iacute;s";
#endif

const char *Txt_Select_one_country =
#if   L==1	// ca
	"Seleccionar un pa&iacute;s";
#elif L==2	// de
	"W&auml;hlen Sie ein Land";
#elif L==3	// en
	"Select one country";
#elif L==4	// es
	"Seleccionar un pa&iacute;s";
#elif L==5	// fr
	"S&eacute;lectionnez un pays";
#elif L==6	// gn
	"Seleccionar un pa&iacute;s";		// Okoteve traduccin
#elif L==7	// it
	"Selezionare un paese";
#elif L==8	// pl
	"Wybierz jeden kraj";
#elif L==9	// pt
	"Selecionar um pa&iacute;s";
#endif

const char *Txt_Select_or_create_another_centre_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear un altre centre en la instituci&oacute; <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie einen anderen Lehrinstitut der Hochschule <strong>%s</strong>";
#elif L==3	// en
	"Select or create another centre in the institution <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear otro centro en la instituci&oacute;n <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez un autre &eacute;tude du &eacute;tablissement <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear otro centro en la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare un altro centro di istituzione <strong>%s</strong>";
#elif L==8	// pl
	"Select or create another centre in the institution <strong>%s</strong>";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Selecionar ou criar um outro centro da institu&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Select_create_course_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar/crear assignatura en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen/erstellen Kurs von <strong>%s</strong>";
#elif L==3	// en
	"Select/create in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar/crear asignatura en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionner/cr&eacute;er mati&egrave;re dans <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar/crear asignatura en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare/creare corso di <strong>%s</strong>";
#elif L==8	// pl
	"Select/create course in <strong>%s</strong>";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Selecionar/criar disciplina de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_another_degree_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear una altra titulaci&oacute; en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie einen anderen Studiengang von <strong>%s</strong>";
#elif L==3	// en
	"Select or create another degree in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear otra titulaci&oacute;n en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez un autre &eacute;tude dans <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear otra titulaci&oacute;n en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare una altra laurea di <strong>%s</strong>";
#elif L==8	// pl
	"Select or create another degree in <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Selecionar ou criar outro grau de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_another_institution_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear una altra instituci&oacute; en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie einen anderen Hochschule von <strong>%s</strong>";
#elif L==3	// en
	"Select or create another institution in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear otra instituci&oacute;n en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez un autre &eacute;tablissement de <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear otra instituci&oacute;n en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare una altra istituzione di <strong>%s</strong>";
#elif L==8	// pl
	"Select or create another institution in <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Selecionar ou criar outra institu&ccedil;&atilde;o de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_one_centre_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear un centre en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie einen Lehrinstitut von <strong>%s</strong>";
#elif L==3	// en
	"Select or create one centre in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear un centro en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez un &eacute;tude dans <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear un centro en <strong>%s</strong>";		// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare un centro di <strong>%s</strong>";
#elif L==8	// pl
	"Select or create one degree in <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Selecionar ou criar um centro de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_one_course_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear una assignatura en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie eine Kurs von <strong>%s</strong>";
#elif L==3	// en
	"Select or create one course in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear una asignatura en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez une mati&egrave;re dans <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear una asignatura en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare un corso di <strong>%s</strong>";
#elif L==8	// pl
	"Select or create one course in <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Selecionar ou criar uma disciplina de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_one_degree_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear una titulaci&oacute; en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie einen Studiengang von <strong>%s</strong>";
#elif L==3	// en
	"Select or create one degree in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear una titulaci&oacute;n en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez un &eacute;tude dans <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear una titulaci&oacute;n en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare una laurea di <strong>%s</strong>";
#elif L==8	// pl
	"Select or create one degree in <strong>%s</strong>";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Selecionar ou criar um grau de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_one_institution_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Seleccionar o crear una instituci&oacute; en <strong>%s</strong>";
#elif L==2	// de
	"W&auml;hlen oder erstellen Sie einen Hochschule von <strong>%s</strong>";
#elif L==3	// en
	"Select or create one institution in <strong>%s</strong>";
#elif L==4	// es
	"Seleccionar o crear una instituci&oacute;n en <strong>%s</strong>";
#elif L==5	// fr
	"S&eacute;lectionnez ou cr&eacute;ez un &eacute;tablissement de <strong>%s</strong>";
#elif L==6	// gn
	"Seleccionar o crear una instituci&oacute;n en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare o creare una istituzione di <strong>%s</strong>";
#elif L==8	// pl
	"Select or create one institution in <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Selecionar ou criar uma institu&ccedil;&atilde;o de <strong>%s</strong>";
#endif

const char *Txt_Select_one_or_more_files_from_your_computer_or_drag_and_drop_here =
#if   L==1	// ca
	"Seleccioneu un o m&eacute;s fitxers del seu ordinador<br />"
	"o arrossegueu i deixeu-los anar aqu&iacute;";
#elif L==2	// de
	"W&auml;hlen Sie eine oder mehrere Dateien von Ihrem Computer<br />"
	"oder per Drag & Drop hier";
#elif L==3	// en
	"Select one or more files from your computer<br />"
	"or drag and drop here";
#elif L==4	// es
	"Seleccione uno o m&aacute;s archivos de su ordenador<br />"
	"o arr&aacute;strelos y su&eacute;ltelos aqu&iacute;";
#elif L==5	// fr
	"S&eacute;lectionnez un ou plusieurs fichiers &agrave; partir de votre ordinateur<br />"
	"ou glisser-d&eacute;poser ici";
#elif L==6	// gn
	"Seleccione uno o m&aacute;s archivos de su ordenador<br />"
	"o arr&aacute;strelos y su&eacute;ltelos aqu&iacute;";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare uno o pi&ugrave; file dal tuo computer<br />"
	"o trascinare e rilasciare qui";
#elif L==8	// pl
	"Wybierz jeden lub wi&eogon;cej plik&oacute;w z komputera<br />"
	"lub przeci&aogon;gnij i upu&sacute;&cacute; tutaj";
#elif L==9	// pt
	"Selecione um ou mais arquivos de seu computador<br />"
	"ou arrastar e soltar aqui";
#endif

const char *Txt_Select_questions =
#if   L==1	// ca
	"Seleccionar preguntes";
#elif L==2	// de
	"W&auml;hlen Sie Fragen";
#elif L==3	// en
	"Select questions";
#elif L==4	// es
	"Seleccionar preguntas";
#elif L==5	// fr
	"Choisir questions";
#elif L==6	// gn
	"Seleccionar preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Selezionare domande";
#elif L==8	// pl
	"Wybierz pytania";
#elif L==9	// pt
	"Selecionar quest&otilde;es";
#endif

const char *Txt_Select_the_groups_in_from_which_you_want_to_register_remove_users_ =
#if   L==1	// ca
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";	// Necessita traduccio
#elif L==2	// de
	"Select the groups in/from which you want to register/remove users.<br />"
	"If you select any groups, users will be enroled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enroled in the course but not in specific groups,"
	" or removed from the course and from all groups.";		// Need bersetzung
#elif L==3	// en
	"Select the groups in/from which you want to register/remove users.<br />"
	"If you select any groups, users will be enroled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enroled in the course but not in specific groups,"
	" or removed from the course and from all groups.";
#elif L==4	// es
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";
#elif L==5	// fr
	"Select the groups in/from which you want to register/remove users.<br />"
	"If you select any groups, users will be enroled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enroled in the course but not in specific groups,"
	" or removed from the course and from all groups.";		// Besoin de traduction
#elif L==6	// gn
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";	// Okoteve traduccin
#elif L==7	// it
	"Seleziona i gruppi nel/dal quale vuoi registrare/rimuovere utenti.<br />"
	"Se selezioni qualche gruppo, gli utenti saranno registrati nel corso e nel gruppo selezionato,"
	" o rimossi dai gruppi selezionati ma non dal corso.<br />"
	"Se non selezioni alcun gruppo, gli utenti saranno registrati nel corso ma non in gruppi specifici,"
	" o rimossi dal corso e da tutti i gruppi.";
#elif L==8	// pl
	"Select the groups in/from which you want to register/remove users.<br />"
	"If you select any groups, users will be enroled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enroled in the course but not in specific groups,"
	" or removed from the course and from all groups.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Selecione os grupos em/dos quais voc&ecirc; deseja registrar/remover usu&aacute;rios.<br />"
	"Se voc&ecirc; selecionar grupos, os usu&aacute;rios ser&atilde;o inscritos na disciplina e nos grupos selecionados"
	" ou removidos dos grupos selecionados, mas n&atilde;o da disciplina.<br />"
	"Se voc&ecirc; n&atilde;o selecionar nenhum grupo, os usu&aacute;rios ser&atilde;o inscritos na disciplina, mas n&atilde;o em grupos espec&iacute;ficos,"
	" ou ser&atilde;o removidos da disciplina e de todos os grupos.";
#endif

const char *Txt_Select_users_X_Followed_Y_Not_followed_Z =	// Warning: it is very important to include three %u in the following sentences
#if   L==1	// ca
	"Usuaris seleccionats: %u<br />"
	"&bull; Seguits: %u<br />"
	"&bull; No seguits: %u";
#elif L==2	// de
	"Ausgew&auml;hlte Benutzer: %u<br />"
	"&bull; Gefolgt: %u<br />"
	"&bull; Nicht gefolgt: %u";
#elif L==3	// en
	"Selected users: %u<br />"
	"&bull; Followed: %u<br />"
	"&bull; Not followed: %u";
#elif L==4	// es
	"Usuarios seleccionados: %u<br />"
	"&bull; Seguidos: %u<br />"
	"&bull; No seguidos: %u";
#elif L==5	// fr
	"Utilisateurs s&eacute;lectionn&eacute;s: %u<br />"
	"&bull; Suivis: %u<br />"
	"&bull; Non suivis: %u";
#elif L==6	// gn
	"Usuarios seleccionados: %u<br />"
	"&bull; Seguidos: %u<br />"
	"&bull; No seguidos: %u";	// Okoteve traduccin
#elif L==7	// it
	"Utenti selezionati: %u<br />"
	"&bull; Seguiti: %u<br />"
	"&bull; Non seguiti: %u";
#elif L==8	// pl
	"Wybrani u&zdot;ytkownicy: %u<br />"
	"&bull; Nast&eogon;pnie: %u<br />"
	"&bull; Nie przestrzegano: %u";
#elif L==9	// pt
	"Usu&aacute;rios selecionados: %u<br />"
	"&bull; Seguidos: %u<br />"
	"&bull; N&atilde;o seguidos: %u";
#endif

const char *Txt_Send =
#if   L==1	// ca
	"Enviar";
#elif L==2	// de
	"Senden";
#elif L==3	// en
	"Send";
#elif L==4	// es
	"Enviar";
#elif L==5	// fr
	"Envoyer";
#elif L==6	// gn
	"Mondo";
#elif L==7	// it
	"Invia";
#elif L==8	// pl
	"Send";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Enviar";
#endif

const char *Txt_Send_message =
#if   L==1	// ca
	"Enviar missatge";
#elif L==2	// de
	"Senden Nachricht";
#elif L==3	// en
	"Send message";
#elif L==4	// es
	"Enviar mensaje";
#elif L==5	// fr
	"Envoyer message";
#elif L==6	// gn
	"Enviar mensaje";	// Okoteve traduccin
#elif L==7	// it
	"Invia messaggio";
#elif L==8	// pl
	"Wy&sacute;lij wiadomo&sacute;&cacute;";
#elif L==9	// pt
	"Enviar mensagem";
#endif

const char *Txt_Send_survey =
#if   L==1	// ca
	"Enviar encuesta";	// Necessita traduccio
#elif L==2	// de
	"Senden Umfrage";
#elif L==3	// en
	"Send survey";
#elif L==4	// es
	"Enviar encuesta";
#elif L==5	// fr
	"Envoyer sondage";
#elif L==6	// gn
	"Enviar encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Invia sondaggio";
#elif L==8	// pl
	"Send survey";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Enviar inqu&eacute;rito";
#endif

const char *Txt_Send_URL =
#if   L==1	// ca
	"Enviar URL";		// Necessita traduccio
#elif L==2	// de
	"Senden URL";
#elif L==3	// en
	"Send URL";
#elif L==4	// es
	"Enviar URL";
#elif L==5	// fr
	"Envoyer URL";
#elif L==6	// gn
	"Enviar URL";		// Okoteve traduccin
#elif L==7	// it
	"Invia URL";
#elif L==8	// pl
	"Send URL";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Enviar URL";
#endif

const char *Txt_Sender_banned_click_to_unban_him =
#if   L==1	// ca
	"Remitente bloqueado/a; pulse para desbloquearlo/a";	// Necessita traduccio
#elif L==2	// de
	"Sender banned; click to unban her/him";		// Need bersetzung
#elif L==3	// en
	"Sender banned; click to unban her/him";
#elif L==4	// es
	"Remitente bloqueado/a; pulse para desbloquearlo/a";
#elif L==5	// fr
	"Sender banned; click to unban her/him";		// Besoin de traduction
#elif L==6	// gn
	"Remitente bloqueado/a; pulse para desbloquearlo/a";	// Okoteve traduccin
#elif L==7	// it
	"Sender banned; click to unban her/him";		// Bisogno di traduzione
#elif L==8	// pl
	"Sender banned; click to unban her/him";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Remetente banido; clique para desbanir ela/ele";
#endif

const char *Txt_Sender_permitted_click_to_ban_him =
#if   L==1	// ca
	"Remitente permitido/a; pulse para bloquearlo/a";	// Necessita traduccio
#elif L==2	// de
	"Sender permitted; click to ban her/him";		// Need bersetzung
#elif L==3	// en
	"Sender permitted; click to ban her/him";
#elif L==4	// es
	"Remitente permitido/a; pulse para bloquearlo/a";
#elif L==5	// fr
	"Sender permitted; click to ban her/him";		// Besoin de traduction
#elif L==6	// gn
	"Remitente permitido/a; pulse para bloquearlo/a";	// Okoteve traduccin
#elif L==7	// it
	"Sender permitted; click to ban her/him";		// Bisogno di traduzione
#elif L==8	// pl
	"Sender permitted; click to ban her/him";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Remetente permitido; clique para banir ela/ele";
#endif

const char *Txt_Session =
#if   L==1	// ca
	"Sessi&oacute;";
#elif L==2	// de
	"Sitzung";
#elif L==3	// en
	"Session";
#elif L==4	// es
	"Sesi&oacute;n";
#elif L==5	// fr
	"Session";
#elif L==6	// gn
	"Sesi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Sessione";
#elif L==8	// pl
	"Sesja";
#elif L==9	// pt
	"Sess&atilde;o";
#endif

const char *Txt_session =
#if   L==1	// ca
	"sessi&oacute;";
#elif L==2	// de
	"Sitzung";
#elif L==3	// en
	"session";
#elif L==4	// es
	"sesi&oacute;n";
#elif L==5	// fr
	"session";
#elif L==6	// gn
	"sesi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"sessione";
#elif L==8	// pl
	"sesja";
#elif L==9	// pt
	"sess&atilde;o";
#endif

const char *Txt_sessions =
#if   L==1	// ca
	"sessions";
#elif L==2	// de
	"Sitzungen";
#elif L==3	// en
	"sessions";
#elif L==4	// es
	"sesiones";
#elif L==5	// fr
	"sessions";
#elif L==6	// gn
	"sesiones";	// Okoteve traduccin
#elif L==7	// it
	"sessioni";
#elif L==8	// pl
	"sessions";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"sess&otilde;es";
#endif

const char *Txt_Set_password =
#if   L==1	// ca
	"Establir contrasenya";
#elif L==2	// de
	"Passwort festlegen";
#elif L==3	// en
	"Set password";
#elif L==4	// es
	"Establecer contrase&ntilde;a";
#elif L==5	// fr
	"D&eacute;finir mot de passe";
#elif L==6	// gn
	"Establecer contrase&ntilde;a";	// Okoteve traduccin
#elif L==7	// it
	"Impostare password";
#elif L==8	// pl
	"Ustaw has&lstrok;o";
#elif L==9	// pt
	"Definir senha";
#endif

const char *Txt_Settings =
#if   L==1	// ca
	"Configuraci&oacute;";
#elif L==2	// de
	"Einstellungen";
#elif L==3	// en
	"Settings";
#elif L==4	// es
	"Ajustes";
#elif L==5	// fr
	"R&eacute;glages";
#elif L==6	// gn
	"Mbohoryha";
#elif L==7	// it
	"Impostazioni";
#elif L==8	// pl
	"Ustawienia";
#elif L==9	// pt
	"Configura&ccedil;&atilde;o";
#endif

const char *Txt_Sex =
#if   L==1	// ca
	"Sexe";
#elif L==2	// de
	"Sex";
#elif L==3	// en
	"Sex";
#elif L==4	// es
	"Sexo";
#elif L==5	// fr
	"Sexe";
#elif L==6	// gn
	"Me&ntilde;a";
#elif L==7	// it
	"Sesso";
#elif L==8	// pl
	"Ple&cacute;";
#elif L==9	// pt
	"Sexo";
#endif

const char *Txt_SEX_PLURAL_Abc[Usr_NUM_SEXS] =
	{
#if   L==1	// ca	// Usr_SEX_UNKNOWN
	 "Desconocidos"	// Necessita traduccio
#elif L==2	// de
	 "Unbekannt"
#elif L==3	// en
	 "Unknown"
#elif L==4	// es
	 "Desconocidos"
#elif L==5	// fr
	 "Inconnus"
#elif L==6	// gn
	 "Desconocidos"	// Okoteve traduccin
#elif L==7	// it
	 "Sconosciuti"
#elif L==8	// pl
	 "Nieznany"
#elif L==9	// pt
	 "Desconhecidos"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_FEMALE
	 "Dones"	// Necessita traduccio
#elif L==2	// de
	 "Frauen"
#elif L==3	// en
	 "Women"
#elif L==4	// es
	 "Mujeres"
#elif L==5	// fr
	 "Femmes"
#elif L==6	// gn
	 "Mujeres"	// Okoteve traduccin
#elif L==7	// it
	 "Donne"
#elif L==8	// pl
	 "Kobiety"
#elif L==9	// pt
	 "Mulheres"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_MALE
	 "Homes"
#elif L==2	// de
	 "M&auml;nner"
#elif L==3	// en
	 "Men"
#elif L==4	// es
	 "Hombres"
#elif L==5	// fr
	 "Hommes"
#elif L==6	// gn
	 "Hombres"	// Okoteve traduccin
#elif L==7	// it
	 "Uomini"
#elif L==8	// pl
	 "Mezczyzni"
#elif L==9	// pt
	 "Homens"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_ALL
	 "Todos"	// Necessita traduccio
#elif L==2	// de
	 "Alle"
#elif L==3	// en
	 "All"
#elif L==4	// es
	 "Todos"
#elif L==5	// fr
	 "Tous"
#elif L==6	// gn
	 "Todos"	// Okoteve traduccin
#elif L==7	// it
	 "Tutti"
#elif L==8	// pl
	 "Wszyscy"
#elif L==9	// pt
	 "Todos"
#endif
	};

const char *Txt_SEX_PLURAL_abc[Usr_NUM_SEXS] =
	{
#if   L==1	// ca	// Usr_SEX_UNKNOWN
	 "desconocidos"	// Necessita traduccio
#elif L==2	// de
	 "unbekannt"
#elif L==3	// en
	 "unknown"
#elif L==4	// es
	 "desconocidos"
#elif L==5	// fr
	 "inconnus"
#elif L==6	// gn
	 "desconocidos"	// Okoteve traduccin
#elif L==7	// it
	 "sconosciuti"
#elif L==8	// pl
	 "nieznany"
#elif L==9	// pt
	 "desconhecidos"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_FEMALE
	 "dones"
#elif L==2	// de
	 "Frauen"
#elif L==3	// en
	 "women"
#elif L==4	// es
	 "mujeres"
#elif L==5	// fr
	 "femmes"
#elif L==6	// gn
	 "mujeres"	// Okoteve traduccin
#elif L==7	// it
	 "donne"
#elif L==8	// pl
	 "kobiety"
#elif L==9	// pt
	 "mulheres"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_MALE
	 "homes"
#elif L==2	// de
	 "M&auml;nner"
#elif L==3	// en
	 "men"
#elif L==4	// es
	 "hombres"
#elif L==5	// fr
	 "hommes"
#elif L==6	// gn
	 "hombres"	// Okoteve traduccin
#elif L==7	// it
	 "uomini"
#elif L==8	// pl
	 "mezczyzni"
#elif L==9	// pt
	 "homens"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_ALL
	 "todos"	// Necessita traduccio
#elif L==2	// de
	 "alle"
#elif L==3	// en
	 "all"
#elif L==4	// es
	 "todos"
#elif L==5	// fr
	 "tous"
#elif L==6	// gn
	 "todos"	// Okoteve traduccin
#elif L==7	// it
	 "tutti"
#elif L==8	// pl
	 "wszyscy"
#elif L==9	// pt
	 "todos"
#endif
	};

const char *Txt_SEX_SINGULAR_Abc[Usr_NUM_SEXS] =
	{
#if   L==1	// ca	// Usr_SEX_UNKNOWN
	 "Desconocido"	// Necessita traduccio
#elif L==2	// de
	 "Unbekannt"
#elif L==3	// en
	 "Unknown"
#elif L==4	// es
	 "Desconocido"
#elif L==5	// fr
	 "Inconnu"
#elif L==6	// gn
	 "Desconocido"	// Okoteve traduccin
#elif L==7	// it
	 "Sconosciuto"
#elif L==8	// pl
	 "Nieznany"
#elif L==9	// pt
	 "Desconhecido"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_FEMALE
	 "Dona"
#elif L==2	// de
	 "Frau"
#elif L==3	// en
	 "Woman"
#elif L==4	// es
	 "Mujer"
#elif L==5	// fr
	 "Femme"
#elif L==6	// gn
	 "Ku&ntilde;a"
#elif L==7	// it
	 "Donna"
#elif L==8	// pl
	 "Kobieta"
#elif L==9	// pt
	 "Mulher"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_MALE
	 "Home"
#elif L==2	// de
	 "Mann"
#elif L==3	// en
	 "Man"
#elif L==4	// es
	 "Hombre"
#elif L==5	// fr
	 "Homme"
#elif L==6	// gn
	 "Kuimba'e"
#elif L==7	// it
	 "Uomo"
#elif L==8	// pl
	 "M&eogon;&zdot;czyzna"
#elif L==9	// pt
	 "Homem"
#endif
	 ,
#if   L==1	// ca	// Usr_SEX_ALL
	 "Todos"		// Necessita traduccio
#elif L==2	// de
	 "Alle"
#elif L==3	// en
	 "All"
#elif L==4	// es
	 "Todos"
#elif L==5	// fr
	 "Tous"
#elif L==6	// gn
	 "Todos"		// Okoteve traduccin
#elif L==7	// it
	 "Tutti"
#elif L==8	// pl
	 "Wwszyscy"
#elif L==9	// pt
	 "Todos"
#endif
	};

const char *Txt_Share =
#if   L==1	// ca
	"Compartir";
#elif L==2	// de
	"Teilen";
#elif L==3	// en
	"Share";
#elif L==4	// es
	"Compartir";
#elif L==5	// fr
	"Partager";
#elif L==6	// gn
	"Compartir";	// Okoteve traduccin
#elif L==7	// it
	"Condividere";
#elif L==8	// pl
	"Dzieli&cacute;";
#elif L==9	// pt
	"Compartilhar";
#endif

const char *Txt_Shared_files_area =
#if   L==1	// ca
	"Zona d'arxius compartits";
#elif L==2	// de
	"Freigegebene Dateien-Bereich";
#elif L==3	// en
	"Shared files area";
#elif L==4	// es
	"Zona de archivos compartidos";
#elif L==5	// fr
	"Zone de fichiers partag&eacute;s";
#elif L==6	// gn
	"Zona de archivos compartidos";	// Okoteve traduccin
#elif L==7	// it
	"Area comune";
#elif L==8	// pl
	"Wsp&oacute;lnej dla";
#elif L==9	// pt
	"Zona comum";
#endif

const char *Txt_shared_files_area =
#if   L==1	// ca
	"zona d'arxius compartits";
#elif L==2	// de
	"Freigegebene Dateien-Bereich";
#elif L==3	// en
	"shared files area";
#elif L==4	// es
	"zona de archivos compartidos";
#elif L==5	// fr
	"zone de fichiers partag&eacute;s";
#elif L==6	// gn
	"zona de archivos compartidos";	// Okoteve traduccin
#elif L==7	// it
	"zona di file condivisi";
#elif L==8	// pl
	"Powierzchnia udost&eogon;pnionych plik&oacute;w";
#elif L==9	// pt
	"zona de arquivos compartilhados";
#endif

const char *Txt_Short_name =
#if   L==1	// ca
	"Nom breu";
#elif L==2	// de
	"Kurzname";
#elif L==3	// en
	"Short name";
#elif L==4	// es
	"Nombre breve";
#elif L==5	// fr
	"Nom abr&eacute;g&eacute;";
#elif L==6	// gn
	"Nombre breve";		// Okoteve traduccin
#elif L==7	// it
	"Nome breve";
#elif L==8	// pl
	"Nazwa skr&oacute;cona";
#elif L==9	// pt
	"Nome abreviado";
#endif

const char *Txt_Short_name_of_the_degree =
#if   L==1	// ca
	"Nom breu de la titulaci&oacute;";
#elif L==2	// de
	"Kurzname der Studiengang";
#elif L==3	// en
	"Short name of the degree";
#elif L==4	// es
	"Nombre breve de la titulaci&oacute;n";
#elif L==5	// fr
	"Nom abr&eacute;g&eacute; du &eacute;tude";
#elif L==6	// gn
	"Nombre breve de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nome breve della laurea";
#elif L==8	// pl
	"Skr&oacute;cona nazwa stopie&nacute;";
#elif L==9	// pt
	"Nome abreviado do grau";
#endif

const char *Txt_Short_name_of_the_centre =
#if   L==1	// ca
	"Nom breu del centre";
#elif L==2	// de
	"Kurzname der Lehrinstitut";
#elif L==3	// en
	"Short name of the centre";
#elif L==4	// es
	"Nombre breve del centro";
#elif L==5	// fr
	"Nom abr&eacute;g&eacute; du centre";
#elif L==6	// gn
	"Nombre breve del centro";	// Okoteve traduccin
#elif L==7	// it
	"Nome breve del centro";
#elif L==8	// pl
	"Skr&oacute;cona nazwa centrum";
#elif L==9	// pt
	"Nome abreviado do centro";
#endif

const char *Txt_Short_name_of_the_course =
#if   L==1	// ca
	"Nom breu de l'assignatura";
#elif L==2	// de
	"Kurzname der Kurs";
#elif L==3	// en
	"Short name of the course";
#elif L==4	// es
	"Nombre breve de la asignatura";
#elif L==5	// fr
	"Nom abr&eacute;g&eacute; de la mati&egrave;re";
#elif L==6	// gn
	"Nombre breve de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Nome breve del corso";
#elif L==8	// pl
	"Skr&oacute;cona nazwa kurs";
#elif L==9	// pt
	"Nome abreviado da disciplina";
#endif

const char *Txt_Short_name_of_the_institution =
#if   L==1	// ca
	"Nom breu de la instituci&oacute;";
#elif L==2	// de
	"Kurzname der Institution";
#elif L==3	// en
	"Short name of the institution";
#elif L==4	// es
	"Nombre breve de la instituci&oacute;n";
#elif L==5	// fr
	"Nom abr&eacute;g&eacute; de l'institution";
#elif L==6	// gn
	"Nombre breve de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Nome breve dell'istituzione";
#elif L==8	// pl
	"Skr&oacute;cona nazwa instytucji";
#elif L==9	// pt
	"Nome abreviado da institui&ccedil;&atilde;o";
#endif

const char *Txt_Shortcut =
#if   L==1	// ca
	"Acc&eacute;s directe";
#elif L==2	// de
	"Direktzugriff";
#elif L==3	// en
	"Shortcut";
#elif L==4	// es
	"Acceso directo";
#elif L==5	// fr
	"Acc&egrave;s direct";
#elif L==6	// gn
	"Acceso directo";	// Okoteve traduccin
#elif L==7	// it
	"Accesso diretto";
#elif L==8	// pl
	"Bezpo&sacute;redni dost&eogon;p";
#elif L==9	// pt
	"Acesso directo";
#endif

const char *Txt_Show =
#if   L==1	// ca
	"Mostra";
#elif L==2	// de
	"Zeige";
#elif L==3	// en
	"Show";
#elif L==4	// es
	"Mostrar";
#elif L==5	// fr
	"Voir";
#elif L==6	// gn
	"Mostrar";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza";
#elif L==8	// pl
	"Pokaza&cacute;";
#elif L==9	// pt
	"Mostrar";
#endif

const char *Txt_Show_FOLDER_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Mostra %s";
#elif L==2	// de
	"Zeige %s";
#elif L==3	// en
	"Show %s";
#elif L==4	// es
	"Mostrar %s";
#elif L==5	// fr
	"Voir %s";
#elif L==6	// gn
	"Mostrar %s";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza %s";
#elif L==8	// pl
	"Pokaza&cacute; %s";
#elif L==9	// pt
	"Mostrar %s";
#endif

const char *Txt_Show_all_data_in_a_table =
#if   L==1	// ca
	"Mostrar totes les dades en una taula";
#elif L==2	// de
	"Alle Daten in einer Tabelle anzeigen";
#elif L==3	// en
	"Show all data in a table";
#elif L==4	// es
	"Mostrar todos los datos en una tabla";
#elif L==5	// fr
	"Voir toutes les donn&eacute;es dans une table";
#elif L==6	// gn
	"Mostrar todos los datos en una tabla";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza tutti i dati in una tabella";
#elif L==8	// pl
	"Poka&zdot; wszystkie dane w tabeli";
#elif L==9	// pt
	"Mostrar todos os dados em uma tabela";
#endif

const char *Txt_Show_all_files =
#if   L==1	// ca
	"Mostrar tots els fitxers";
#elif L==2	// de
	"Alle Dateien anzeigen";
#elif L==3	// en
	"Show all files";
#elif L==4	// es
	"Mostrar todos los archivos";
#elif L==5	// fr
	"Afficher tous les fichiers";
#elif L==6	// gn
	"Mostrar todos los archivos";	// Okoteve traduccin
#elif L==7	// it
	"Mostra tutti i file";
#elif L==8	// pl
	"Poka&zdot; wszystkie pliki";
#elif L==9	// pt
	"Mostrar todos os arquivos";
#endif

const char *Txt_Show_all_NOTIFICATIONS =
#if   L==1	// ca
	"Mostrar tots";
#elif L==2	// de
	"Alle anzeigen";
#elif L==3	// en
	"Show all";
#elif L==4	// es
	"Mostrar todas";
#elif L==5	// fr
	"Afficher tous";
#elif L==6	// gn
	"Mostrar todas";	// Okoteve traduccin
#elif L==7	// it
	"Mostra tutti";
#elif L==8	// pl
	"Poka&zdot; wszystkie";
#elif L==9	// pt
	"Mostrar todas";
#endif

const char *Txt_Show_all_notifications =
#if   L==1	// ca
	"Mostrar tots les notificacions";
#elif L==2	// de
	"Alle Anmeldungen anzeigen";
#elif L==3	// en
	"Show all notifications";
#elif L==4	// es
	"Mostrar todas las notificaciones";
#elif L==5	// fr
	"Afficher tous les notifications";
#elif L==6	// gn
	"Mostrar todas las notificaciones";	// Okoteve traduccin
#elif L==7	// it
	"Mostra tutti le notifiche";
#elif L==8	// pl
	"Poka&zdot; wszystkie powiadomienia";
#elif L==9	// pt
	"Mostrar todas as notifica&ccedil;&otilde;es";
#endif

const char *Txt_Show_agenda =
#if   L==1	// ca
	"Mostra agenda";
#elif L==2	// de
	"Organizer anzeigen";
#elif L==3	// en
	"Show agenda";
#elif L==4	// es
	"Ver agenda";
#elif L==5	// fr
	"Afficher agenda";
#elif L==6	// gn
	"Ver agenda";	// Okoteve traduccin
#elif L==7	// it
	"Mostra agenda";
#elif L==8	// pl
	"Poka&zdot; pami&eogon;tnik";
#elif L==9	// pt
	"Mostrar agenda";
#endif

const char *Txt_Show_anyway =
#if   L==1	// ca
	"Mostra de totes maneres";
#elif L==2	// de
	"Zeigesowieso";
#elif L==3	// en
	"Show anyway";
#elif L==4	// es
	"Mostrar de todos modos";
#elif L==5	// fr
	"Voir de toute faon";
#elif L==6	// gn
	"Mostrar de todos modos";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza comunque";
#elif L==8	// pl
	"Show anyway";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Mostrar de qualquer maneira";
#endif

const char *Txt_Show_calendar =
#if   L==1	// ca
	"Mostra calendari";
#elif L==2	// de
	"Kalender anzeigen";
#elif L==3	// en
	"Show calendar";
#elif L==4	// es
	"Ver calendario";
#elif L==5	// fr
	"Afficher calendrier";
#elif L==6	// gn
	"Ver calendario";	// Okoteve traduccin
#elif L==7	// it
	"Mostra calendario";
#elif L==8	// pl
	"Poka&zdot; kalendarz";
#elif L==9	// pt
	"Mostrar calend&aacute;rio";
#endif

const char *Txt_Show_hits =	// hits = visits, clicks, page views...
#if   L==1	// ca
	"Veure accessos";
#elif L==2	// de
	"Zeige Anmeldungen";
#elif L==3	// en
	"Show hits";
#elif L==4	// es
	"Ver accesos";
#elif L==5	// fr
	"Voir visites";
#elif L==6	// gn
	"Ver accesos";		// Okoteve traduccin
#elif L==7	// it
	"Visualizza visite";
#elif L==8	// pl
	"Show hits";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Mostrar visitas";
#endif

const char *Txt_Show_more_details =
#if   L==1	// ca
	"Mostra m&eacute;s detalls";
#elif L==2	// de
	"Mehr Details";
#elif L==3	// en
	"Show more details";
#elif L==4	// es
	"Mostrar m&aacute;s detalles";
#elif L==5	// fr
	"Montrer plus de d&eacute;tails";
#elif L==6	// gn
	"Mostrar m&aacute;s detalles";	// Okoteve traduccin
#elif L==7	// it
	"Mostra pi&ugrave; dettagli";
#elif L==8	// pl
	"Poka&zdot; wi&eogon;cej szczeg&oacute;&lstrok;&oacute;w";
#elif L==9	// pt
	"Mostrar mais detalhes";
#endif

const char *Txt_Show_more_recipients =
#if   L==1	// ca
	"Veure m&eacute;s recipients";
#elif L==2	// de
	"Mehrere Empf&auml;nger anzeigen";
#elif L==3	// en
	"Show more recipients";
#elif L==4	// es
	"Ver m&aacute;s destinatarios";
#elif L==5	// fr
	"Montrer plus de destinataires";
#elif L==6	// gn
	"Ver m&aacute;s destinatarios";	// Okoteve traduccin
#elif L==7	// it
	"Mostra pi&ugrave; destinatari";
#elif L==8	// pl
	"Poka&zdot; wi&eogon;cej odbiorc&oacute;w";
#elif L==9	// pt
	"Ver mais destinat&aacute;rios";
#endif

const char *Txt_Show_next_X_clicks =	// Warning: it is very important to include %lu in the following sentences
#if   L==1	// ca
	"Mostrar %lu clics posteriores";	// Necessita traduccio
#elif L==2	// de
	"Show next %lu clicks";			// Need bersetzung
#elif L==3	// en
	"Show next %lu clicks";
#elif L==4	// es
	"Mostrar %lu clics posteriores";
#elif L==5	// fr
	"Show next %lu clicks";			// Besoin de traduction
#elif L==6	// gn
	"Mostrar %lu clics posteriores";	// Okoteve traduccin
#elif L==7	// it
	"Mostra %lu scatti seguenti";
#elif L==8	// pl
	"Show next %lu clicks";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Mostrar %lu cliques subsequentes";
#endif

const char *Txt_Show_office_hours =
#if   L==1	// ca
	"Mostra horaris tutories";
#elif L==2	// de
	"Gesch&auml;ftszeiten anzeigen";
#elif L==3	// en
	"Show office hours";
#elif L==4	// es
	"Ver horarios tutor&iacute;as";
#elif L==5	// fr
	"Voir horaires tuteur";
#elif L==6	// gn
	"Ver horarios tutor&iacute;as";	// Okoteve traduccin
#elif L==7	// it
	"Mostra orari tutoraggi";
#elif L==8	// pl
	"Poka&zdot; godzin pracy biura";
#elif L==9	// pt
	"Mostrar hor&aacute;rios tutor";
#endif

const char *Txt_Show_previous_X_clicks =	// Warning: it is very important to include %lu in the following sentences
#if   L==1	// ca
	"Mostrar %lu clics anteriores";		// Necessita traduccio
#elif L==2	// de
	"Show previous %lu clicks";		// Need bersetzung
#elif L==3	// en
	"Show previous %lu clicks";
#elif L==4	// es
	"Mostrar %lu clics anteriores";
#elif L==5	// fr
	"Show previous %lu clicks";		// Besoin de traduction
#elif L==6	// gn
	"Mostrar %lu clics anteriores";		// Okoteve traduccin
#elif L==7	// it
	"Mostra %lu scatti precedenti";
#elif L==8	// pl
	"Show previous %lu clicks";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Mostrar %lu cliques anteriores";
#endif

const char *Txt_Show_questions =
#if   L==1	// ca
	"Mostrar preguntes";
#elif L==2	// de
	"Zeige Testfragen";
#elif L==3	// en
	"Show questions";
#elif L==4	// es
	"Mostrar preguntas";
#elif L==5	// fr
	"Voir les questions";
#elif L==6	// gn
	"Mostrar preguntas";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza le domande";
#elif L==8	// pl
	"Show questions";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Mostrar quest&otilde;es";
#endif

const char *Txt_Show_statistic =
#if   L==1	// ca
	"Mostrar estad&iacute;stica";
#elif L==2	// de
	"Zeige Statistik";
#elif L==3	// en
	"Show statistic";
#elif L==4	// es
	"Mostrar estad&iacute;stica";
#elif L==5	// fr
	"Voir statistique";
#elif L==6	// gn
	"Mostrar estad&iacute;stica";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza statistica";
#elif L==8	// pl
	"Show statistic";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Mostrar estat&iacute;stica";
#endif

const char *Txt_Shuffle =
#if   L==1	// ca
	"Barrejar";
#elif L==2	// de
	"Shuffle";
#elif L==3	// en
	"Shuffle";
#elif L==4	// es
	"Barajar";
#elif L==5	// fr
	"M&eacute;langer";
#elif L==6	// gn
	"Barajar";	// Okoteve traduccin
#elif L==7	// it
	"Mischiare";
#elif L==8	// pl
	"Shuffle";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Baralhar";
#endif

const char *Txt_Sign_up =
#if   L==1	// ca
	"Sol&middot;licitar inscripci&oacute;";
#elif L==2	// de
	"Registrierung beantragen";
#elif L==3	// en
	"Sign up";
#elif L==4	// es
	"Solicitar inscripci&oacute;n";
#elif L==5	// fr
	"Demander inscription";
#elif L==6	// gn
	"Solicitar inscripci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Richiedere iscrizione";
#elif L==8	// pl
	"Wniosku rejestracj&eogon;";
#elif L==9	// pt
	"Solicitar inscri&ccedil;&atilde;o";
#endif

const char *Txt_Similar_users =
#if   L==1	// ca
	"Usuaris similars";
#elif L==2	// de
	"&Auml;hnliche Benutzer";
#elif L==3	// en
	"Similar users";
#elif L==4	// es
	"Usuarios similares";
#elif L==5	// fr
	"Utilisateurs similaires";
#elif L==6	// gn
	"Usuarios similares";	// Okoteve traduccin
#elif L==7	// it
	"Utenti simili";
#elif L==8	// pl
	"Podobne u&zdot;ytkownik&oacute;w";
#elif L==9	// pt
	"Usu&aacute;rios similares";
#endif

const char *Txt_Size =		// ...of a file zone
#if   L==1	// ca
	"Mida";
#elif L==2	// de
	"Gr&ouml;&szlig;e";
#elif L==3	// en
	"Size";
#elif L==4	// es
	"Tama&ntilde;o";
#elif L==5	// fr
	"Taille";
#elif L==6	// gn
	"Tama&ntilde;o";	// Okoteve traduccin
#elif L==7	// it
	"Dimensione";
#elif L==8	// pl
	"Size";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tamanho";
#endif

const char *Txt_Size_of_photos =
#if   L==1	// ca
	"Mida de les fotos";
#elif L==2	// de
	"Gr&ouml;&szlig;e des Fotos";
#elif L==3	// en
	"Size of photos";
#elif L==4	// es
	"Tama&ntilde;o de las fotos";
#elif L==5	// fr
	"Taille des photos";
#elif L==6	// gn
	"Tama&ntilde;o de las fotos";	// Okoteve traduccin
#elif L==7	// it
	"Dimensioni delle foto";
#elif L==8	// pl
	"Rozmiar zdj&eogon;&cacute;";
#elif L==9	// pt
	"Tamanho das fotos";
#endif

const char *Txt_Sort_degrees_by =
#if   L==1	// ca
	"Ordenar titulacions per";
#elif L==2	// de
	"Sortieren Studieng&auml;nge nach";
#elif L==3	// en
	"Sort degrees by";
#elif L==4	// es
	"Ordenar titulaciones por";
#elif L==5	// fr
	"Trier &eacute;tudes par";
#elif L==6	// gn
	"Ordenar titulaciones por";	// Okoteve traduccin
#elif L==7	// it
	"Ordina laurea per";
#elif L==8	// pl
	"Sortuj stopni wed&lstrok;ug";
#elif L==9	// pt
	"Classificar graus por";
#endif

const char *Txt_Source_of_information =
#if   L==1	// ca
	"Font d'informaci&oacute;";
#elif L==2	// de
	"Quelle von Informationen";
#elif L==3	// en
	"Source of information";
#elif L==4	// es
	"Fuente de informaci&oacute;n";
#elif L==5	// fr
	"Source d'information";
#elif L==6	// gn
	"Fuente de informaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Fonte delle informazioni";
#elif L==8	// pl
	"&Zacute;r&oacute;d&lstrok;o informacji";
#elif L==9	// pt
	"Fonte de informa&ccedil;&atilde;o";
#endif

const char *Txt_STAT_CLICKS_GROUPED_BY[Sta_NUM_CLICKS_GROUPED_BY] =
   {
		   // Sta_CLICKS_CRS_DETAILED_LIST
#if   L==1	// ca
	 "clics detallats"
#elif L==2	// de
	 "detaillierte Klicks"
#elif L==3	// en
	 "detailed clicks"
#elif L==4	// es
	 "clics detallados"
#elif L==5	// fr
	 "clics d&eacute;taill&eacute;es"
#elif L==6	// gn
	 "clics detallados"	// Okoteve traduccin
#elif L==7	// it
	 "dettagli di scatto"
#elif L==8	// pl
	 "szczeg&oacute;&lstrok;owe klikni&eogon;&cacute;"
#elif L==9	// pt
	 "detalhadamente cliques"
#endif
	 ,	// Sta_CLICKS_CRS_PER_USR
#if   L==1	// ca
	 "usuari"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "user"
#elif L==4	// es
	 "usuario"
#elif L==5	// fr
	 "utilisateur"
#elif L==6	// gn
	 "puruh&aacute;ra"
#elif L==7	// it
	 "utente"
#elif L==8	// pl
	 "u&zdot;ytkownik"
#elif L==9	// pt
	 "utilizador"
#endif
	 ,	// Sta_CLICKS_CRS_PER_DAY
#if   L==1	// ca
	 "dia"
#elif L==2	// de
	 "Tag"
#elif L==3	// en
	 "day"
#elif L==4	// es
	 "d&iacute;a"
#elif L==5	// fr
	 "jour"
#elif L==6	// gn
	 "&aacute;ra"
#elif L==7	// it
	 "giorno"
#elif L==8	// pl
	 "dzie&nacute;"
#elif L==9	// pt
	 "dia"
#endif
	 ,	// Sta_CLICKS_CRS_PER_DAY_AND_HOUR
#if   L==1	// ca
	 "dia i hora"
#elif L==2	// de
	 "Tag und Stunde"
#elif L==3	// en
	 "day and hour"
#elif L==4	// es
	 "d&iacute;a y hora"
#elif L==5	// fr
	 "jour et heure"
#elif L==6	// gn
	 "&aacute;ra ha aravo"
#elif L==7	// it
	 "giorno e ora"
#elif L==8	// pl
	 "dzie&nacute; i godzin&eogon;"
#elif L==9	// pt
	 "dia e hora"
#endif
	 ,	// Sta_CLICKS_CRS_PER_WEEK
#if   L==1	// ca
	 "setmana"
#elif L==2	// de
	 "Woche"
#elif L==3	// en
	 "week"
#elif L==4	// es
	 "semana"
#elif L==5	// fr
	 "semaine"
#elif L==6	// gn
	 "arapok&otilde;indy"
#elif L==7	// it
	 "settimana"
#elif L==8	// pl
	 "tydzie&nacute;"
#elif L==9	// pt
	 "semana"
#endif
	 ,	// Sta_CLICKS_CRS_PER_MONTH
#if   L==1	// ca
	 "mes"
#elif L==2	// de
	 "Monat"
#elif L==3	// en
	 "month"
#elif L==4	// es
	 "mes"
#elif L==5	// fr
	 "mois"
#elif L==6	// gn
	 "jasy"
#elif L==7	// it
	 "mese"
#elif L==8	// pl
	 "miesi&aogon;c"
#elif L==9	// pt
	 "m&ecirc;s"
#endif
	 ,	// Sta_CLICKS_CRS_PER_YEAR
#if   L==1	// ca
	"any"
#elif L==2	// de
	"Jahr"
#elif L==3	// en
	"year"
#elif L==4	// es
	"a&ntilde;o"
#elif L==5	// fr
	"ann&eacute;e"
#elif L==6	// gn
	"ary"
#elif L==7	// it
	"anno"
#elif L==8	// pl
	"rok"
#elif L==9	// pt
	"ano"
#endif
	 ,	// Sta_CLICKS_CRS_PER_HOUR
#if   L==1	// ca
	 "hora"
#elif L==2	// de
	 "Stunde"
#elif L==3	// en
	 "hour"
#elif L==4	// es
	 "hora"
#elif L==5	// fr
	 "heure"
#elif L==6	// gn
	 "aravo"
#elif L==7	// it
	 "ora"
#elif L==8	// pl
	 "godzina"
#elif L==9	// pt
	 "hora"
#endif
	 ,	// Sta_CLICKS_CRS_PER_MINUTE
#if   L==1	// ca
	 "minut"
#elif L==2	// de
	 "Minute"
#elif L==3	// en
	 "minute"
#elif L==4	// es
	 "minuto"
#elif L==5	// fr
	 "minute"
#elif L==6	// gn
	 "aravo'i"
#elif L==7	// it
	 "minuto"
#elif L==8	// pl
	 "minut&eogon;"
#elif L==9	// pt
	 "minuto"
#endif
	 ,	// Sta_CLICKS_CRS_PER_ACTION
#if   L==1	// ca
	 "acci&oacute;"
#elif L==2	// de
	 "Handlung"
#elif L==3	// en
	 "action"
#elif L==4	// es
	 "acci&oacute;n"
#elif L==5	// fr
	 "action"
#elif L==6	// gn
	 "tembiapo"
#elif L==7	// it
	 "azione"
#elif L==8	// pl
	 "dzia&lstrok;anie"
#elif L==9	// pt
	 "a&ccedil;&atilde;o"
#endif
	 ,	// Sta_CLICKS_GBL_PER_DAY
#if   L==1	// ca
	 "dia"
#elif L==2	// de
	 "Tag"
#elif L==3	// en
	 "day"
#elif L==4	// es
	 "d&iacute;a"
#elif L==5	// fr
	 "jour"
#elif L==6	// gn
	 "&aacute;ra"
#elif L==7	// it
	 "giorno"
#elif L==8	// pl
	 "dzie&nacute;"
#elif L==9	// pt
	 "dia"
#endif
	 ,	// Sta_CLICKS_GBL_PER_DAY_AND_HOUR
#if   L==1	// ca
	 "dia i hora"
#elif L==2	// de
	 "Tag und Stunde"
#elif L==3	// en
	 "day and hour"
#elif L==4	// es
	 "d&iacute;a y hora"
#elif L==5	// fr
	 "jour et heure"
#elif L==6	// gn
	 "&aacute;ra ha aravo"
#elif L==7	// it
	 "giorno e ora"
#elif L==8	// pl
	 "dzie&nacute; i godzin&eogon;"
#elif L==9	// pt
	 "dia e hora"
#endif
	 ,	// Sta_CLICKS_GBL_PER_WEEK
#if   L==1	// ca
	 "setmana"
#elif L==2	// de
	 "Woche"
#elif L==3	// en
	 "week"
#elif L==4	// es
	 "semana"
#elif L==5	// fr
	 "semaine"
#elif L==6	// gn
	 "arapok&otilde;indy"
#elif L==7	// it
	 "settimana"
#elif L==8	// pl
	 "tydzie&nacute;"
#elif L==9	// pt
	 "semana"
#endif
	 ,	// Sta_CLICKS_GBL_PER_MONTH
#if   L==1	// ca
	 "mes"
#elif L==2	// de
	 "Monat"
#elif L==3	// en
	 "month"
#elif L==4	// es
	 "mes"
#elif L==5	// fr
	 "mois"
#elif L==6	// gn
	 "jasy"
#elif L==7	// it
	 "mese"
#elif L==8	// pl
	 "miesi&aogon;c"
#elif L==9	// pt
	 "m&ecirc;s"
#endif
	 ,	// Sta_CLICKS_GBL_PER_YEAR
#if   L==1	// ca
	"any"
#elif L==2	// de
	"Jahr"
#elif L==3	// en
	"year"
#elif L==4	// es
	"a&ntilde;o"
#elif L==5	// fr
	"ann&eacute;e"
#elif L==6	// gn
	"ary"
#elif L==7	// it
	"anno"
#elif L==8	// pl
	"rok"
#elif L==9	// pt
	"ano"
#endif
	 ,	// Sta_CLICKS_GBL_PER_HOUR
#if   L==1	// ca
	 "hora"
#elif L==2	// de
	 "Stunde"
#elif L==3	// en
	 "hour"
#elif L==4	// es
	 "hora"
#elif L==5	// fr
	 "heure"
#elif L==6	// gn
	 "aravo"
#elif L==7	// it
	 "ora"
#elif L==8	// pl
	 "godzina"
#elif L==9	// pt
	 "hora"
#endif
	 ,	// Sta_CLICKS_GBL_PER_MINUTE
#if   L==1	// ca
	 "minut"
#elif L==2	// de
	 "Minute"
#elif L==3	// en
	 "minute"
#elif L==4	// es
	 "minuto"
#elif L==5	// fr
	 "minute"
#elif L==6	// gn
	 "aravo'i"
#elif L==7	// it
	 "minuto"
#elif L==8	// pl
	 "minut&eogon;"
#elif L==9	// pt
	 "minuto"
#endif
	 ,	// Sta_CLICKS_GBL_PER_ACTION
#if   L==1	// ca
	 "acci&oacute;"
#elif L==2	// de
	 "Handlung"
#elif L==3	// en
	 "action"
#elif L==4	// es
	 "acci&oacute;n"
#elif L==5	// fr
	 "action"
#elif L==6	// gn
	 "tembiapo"
#elif L==7	// it
	 "azione"
#elif L==8	// pl
	 "dzia&lstrok;anie"
#elif L==9	// pt
	 "a&ccedil;&atilde;o"
#endif
	 ,	// Sta_CLICKS_GBL_PER_PLUGIN
#if   L==1	// ca
	 "complement"
#elif L==2	// de
	 "Plugin"
#elif L==3	// en
	 "plugin"
#elif L==4	// es
	 "complemento"
#elif L==5	// fr
	 "plugin"
#elif L==6	// gn
	 "complemento"	// Okoteve traduccin
#elif L==7	// it
	 "plugin"
#elif L==8	// pl
	 "wtyczka"
#elif L==9	// pt
	 "plugin"
#endif
	 ,	// Sta_CLICKS_GBL_PER_API_FUNCTION
#if   L==1	// ca
	 "funci&oacute;"
#elif L==2	// de
	 "Funktion"
#elif L==3	// en
	 "function"
#elif L==4	// es
	 "funci&oacute;n"
#elif L==5	// fr
	 "fonction"
#elif L==6	// gn
	 "funci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "funzione"
#elif L==8	// pl
	 "funkcja"
#elif L==9	// pt
	 "fun&ccedil;&atilde;o"
#endif
	 ,	// Sta_CLICKS_GBL_PER_BANNER
#if   L==1	// ca
	 "banner"
#elif L==2	// de
	 "Werbebanner"
#elif L==3	// en
	 "banner"
#elif L==4	// es
	 "banner"
#elif L==5	// fr
	 "banni&eacute;re"
#elif L==6	// gn
	 "banner"	// Okoteve traduccin
#elif L==7	// it
	 "banner"
#elif L==8	// pl
	 "banner"
#elif L==9	// pt
	 "banner"
#endif
	 ,	// Sta_CLICKS_GBL_PER_COUNTRY
#if   L==1	// ca
	"pa&iacute;s"
#elif L==2	// de
	"Land"
#elif L==3	// en
	"country"
#elif L==4	// es
	"pa&iacute;s"
#elif L==5	// fr
	"pays;"
#elif L==6	// gn
	"tet&atilde;"
#elif L==7	// it
	"paese"
#elif L==8	// pl
	"kraj"
#elif L==9	// pt
	"pa&iacute;s"
#endif
	 ,	// Sta_CLICKS_GBL_PER_INSTITUTION
#if   L==1	// ca
	"instituci&oacute;"
#elif L==2	// de
	"Hochschule"
#elif L==3	// en
	"institution"
#elif L==4	// es
	"instituci&oacute;n"
#elif L==5	// fr
	"&eacute;tablissement"
#elif L==6	// gn
	"mbo'ehao"
#elif L==7	// it
	"istituzione"
#elif L==8	// pl
	"instytucji"
#elif L==9	// pt
	"institu&ccedil;&atilde;o"
#endif
	 ,	// Sta_CLICKS_GBL_PER_CENTRE
#if   L==1	// ca
	"centre"
#elif L==2	// de
	"Lehrinstitut"
#elif L==3	// en
	"centre"
#elif L==4	// es
	"centro"
#elif L==5	// fr
	"centre"
#elif L==6	// gn
	"mbo'ehao"
#elif L==7	// it
	"centro"
#elif L==8	// pl
	"centrum"
#elif L==9	// pt
	"centro"
#endif
	 ,	// Sta_CLICKS_GBL_PER_DEGREE
#if   L==1	// ca
	 "titulaci&oacute;"
#elif L==2	// de
	 "Studiengang"
#elif L==3	// en
	 "degree"
#elif L==4	// es
	 "titulaci&oacute;n"
#elif L==5	// fr
	 "&eacute;tude"
#elif L==6	// gn
	 "arandur&atilde;"
#elif L==7	// it
	 "laurea"
#elif L==8	// pl
	 "stopie&nacute;"
#elif L==9	// pt
	 "grau"
#endif
	 ,	// Sta_CLICKS_GBL_PER_COURSE
#if   L==1	// ca
	 "assignatura"
#elif L==2	// de
	 "Kurs"
#elif L==3	// en
	 "course"
#elif L==4	// es
	 "asignatura"
#elif L==5	// fr
	 "mati&egrave;re"
#elif L==6	// gn
	 "mbo'esyry"
#elif L==7	// it
	 "corso"
#elif L==8	// pl
	 "kurs"
#elif L==9	// pt
	 "disciplina"
#endif
	};

const char *Txt_STAT_COLOR_TYPES[Sta_NUM_COLOR_TYPES] =
   {
#if   L==1	// ca
	 "color"
#elif L==2	// de
	 "Farbe"
#elif L==3	// en
	 "color"
#elif L==4	// es
	 "color"
#elif L==5	// fr
	 "couleur"
#elif L==6	// gn
	 "color"	// Okoteve traduccin
#elif L==7	// it
	 "colore"
#elif L==8	// pl
	 "kolor"
#elif L==9	// pt
	 "cor"
#endif
	 ,
#if   L==1	// ca
	 "de negre a blanc"
#elif L==2	// de
	 "schwarz zu wei&szlig;"
#elif L==3	// en
	 "black to white"
#elif L==4	// es
	 "de negro a blanco"
#elif L==5	// fr
	 "du noir au blanc"
#elif L==6	// gn
	 "de negro a blanco"	// Okoteve traduccin
#elif L==7	// it
	 "dal nero al bianco"
#elif L==8	// pl
	 "czarny na bia&lstrok;y"
#elif L==9	// pt
	 "do preto ao branco"
#endif
	 ,
#if   L==1	// ca
	 "de blanc a negre"
#elif L==2	// de
	 "wei&szlig; zu schwarz"
#elif L==3	// en
	 "white to black"
#elif L==4	// es
	 "de blanco a negro"
#elif L==5	// fr
	 "du blanc au noir"
#elif L==6	// gn
	 "de blanco a negro"	// Okoteve traduccin
#elif L==7	// it
	 "dal bianco al nero"
#elif L==8	// pl
	 "bia&lstrok;y do czarny"
#elif L==9	// pt
	 "do branco ao preto"
#endif
	};

const char *Txt_STAT_COURSE_FILE_ZONES[Fig_NUM_STAT_CRS_FILE_ZONES] =	// Use &nbsp; instead of space
	 {
#if   L==1	// ca	// Brw_ADMI_DOC_CRS
	 "Documents&nbsp;(ass.)"
#elif L==2	// de
	 "Dokumente&nbsp;(Ver.)"
#elif L==3	// en
	 "Documents&nbsp;(crs.)"
#elif L==4	// es
	 "Documentos&nbsp;(asg.)"
#elif L==5	// fr
	 "Documents&nbsp;(mat.)"
#elif L==6	// gn
	 "Documentos&nbsp;(asg.)"		// Okoteve traduccin
#elif L==7	// it
	 "Documenti&nbsp;(crs.)"
#elif L==8	// pl
	 "Dokumenty&nbsp;(kurs)"
#elif L==9	// pt
	"Documentos&nbsp;(crs.)"
#endif
	,
#if   L==1	// ca	// Brw_ADMI_DOC_GRP
	"Documents&nbsp;(grp.)"
#elif L==2	// de
	"Dokumente&nbsp;(Grp.)"
#elif L==3	// en
	"Documents&nbsp;(grp.)"
#elif L==4	// es
	"Documentos&nbsp;(grp.)"
#elif L==5	// fr
	"Documents&nbsp;(grp.)"
#elif L==6	// gn
	"Documentos&nbsp;(grp.)"	// Okoteve traduccin
#elif L==7	// it
	"Documenti&nbsp;(grp.)"
#elif L==8	// pl
	"Dokumenty&nbsp;(grp.)"
#elif L==9	// pt
	"Documentos&nbsp;(grp.)"
#endif
	,
#if   L==1	// ca	// Brw_ADMI_TCH_CRS
	"Privats&nbsp;(ass.)"
#elif L==2	// de
	"Private&nbsp;(Ver.)"
#elif L==3	// en
	"Private&nbsp;(crs.)"
#elif L==4	// es
	"Privados&nbsp;(asg.)"
#elif L==5	// fr
	"Priv&eacute;s&nbsp;(mat.)"
#elif L==6	// gn
	"Privados&nbsp;(asg.)"	// Okoteve traduccin
#elif L==7	// it
	"Privati&nbsp;(crs.)"
#elif L==8	// pl
	"Prywatne&nbsp;(kurs)"
#elif L==9	// pt
	"Privadoss&nbsp;(crs.)"
#endif
	,
#if   L==1	// ca	// Brw_ADMI_TCH_GRP
	"Privats&nbsp;(grp.)"
#elif L==2	// de
	"Private&nbsp;(Grp.)"
#elif L==3	// en
	"Private&nbsp;(grp.)"
#elif L==4	// es
	"Privados&nbsp;(grp.)"
#elif L==5	// fr
	"Priv&eacute;s&nbsp;(grp.)"
#elif L==6	// gn
	"Privados&nbsp;(grp.)"	// Okoteve traduccin
#elif L==7	// it
	"Privati&nbsp;(grp.)"
#elif L==8	// pl
	"Prywatne&nbsp;(grp)"
#elif L==9	// pt
	"Privadoss&nbsp;(grp.)"
#endif
	,
#if   L==1	// ca	// Brw_ADMI_SHR_CRS
	"Compartits&nbsp;(asg.)"
#elif L==2	// de
	"Freigegebene&nbsp;(Ver.)"
#elif L==3	// en
	"Shared&nbsp;(crs.)"
#elif L==4	// es
	"Compartidos&nbsp;(asg.)"
#elif L==5	// fr
	"Partag&eacute;s&nbsp;(mat.)"
#elif L==6	// gn
	"Compartidos&nbsp;(asg.)"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi&nbsp;(crs.)"
#elif L==8	// pl
	"Wsp&oacute;lna&nbsp;(kurs)"
#elif L==9	// pt
	"Compartilhados&nbsp;(crs.)"
#endif
	,
#if   L==1	// ca	// Brw_ADMI_SHR_GRP
	"Compartits&nbsp;(grp.)"
#elif L==2	// de
	"Freigegebene&nbsp;(Grp.)"
#elif L==3	// en
	"Shared&nbsp;(grp.)"
#elif L==4	// es
	"Compartidos&nbsp;(grp.)"
#elif L==5	// fr
	"Partag&eacute;s&nbsp;(grp.)"
#elif L==6	// gn
	"Compartidos&nbsp;(grp.)"	// Okoteve traduccin
#elif L==7	// it
	"Condivisi&nbsp;(grp.)"
#elif L==8	// pl
	"Wsp&oacute;lna&nbsp;(grp.)"
#elif L==9	// pt
	"Compartilhados&nbsp;(grp.)"
#endif
	,
#if   L==1	// ca	// Brw_ADMI_MRK_CRS
	"Qualificacions&nbsp;(ass.)"
#elif L==2	// de
	"Bewertungen&nbsp;(Ver.)"
#elif L==3	// en
	"Marks&nbsp;(crs.)"
#elif L==4	// es
	"Calificaciones&nbsp;(asg.)"
#elif L==5	// fr
	"Notes&nbsp;(mat.)"
#elif L==6	// gn
	"Calificaciones&nbsp;(asg.)"		// Okoteve traduccin
#elif L==7	// it
	"Risultati&nbsp;(crs.)"
#elif L==8	// pl
	"Marks&nbsp;(crs.)"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Notas&nbsp;(crs.)"
#endif
	,
#if   L==1	// ca	// Brw_ADMI_MRK_GRP
	"Qualificacions&nbsp;(grp.)"
#elif L==2	// de
	"Bewertungen&nbsp;(Grp.)"
#elif L==3	// en
	"Marks&nbsp;(grp.)"
#elif L==4	// es
	"Calificaciones&nbsp;(grp.)"
#elif L==5	// fr
	"Notes&nbsp;(grp.)"
#elif L==6	// gn
	"Calificaciones&nbsp;(grp.)"		// Okoteve traduccin
#elif L==7	// it
	"Risultati&nbsp;(grp.)"
#elif L==8	// pl
	"Marks&nbsp;(grp.)"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Notas&nbsp;(grp.)"
#endif
	,
#if   L==1	// ca	// Brw_ADMI_ASG_USR
	"Activitats"
#elif L==2	// de
	"Aufgaben"
#elif L==3	// en
	"Assignments"
#elif L==4	// es
	"Actividades"
#elif L==5	// fr
	"Activit&eacute;s"
#elif L==6	// gn
	"Tar&eacute;a"
#elif L==7	// it
	"Attivit&agrave;"
#elif L==8	// pl
	"Zadania"
#elif L==9	// pt
	"Atividades"
#endif
	,
#if   L==1	// ca	// Brw_ADMI_WRK_USR
	"Treballs"
#elif L==2	// de
	"Hausarbeiten"
#elif L==3	// en
	"Homework"
#elif L==4	// es
	"Trabajos"
#elif L==5	// fr
	"Travails"
#elif L==6	// gn
	"Tembiapo"
#elif L==7	// it
	"Lavori"
#elif L==8	// pl
	"Praca domowa"
#elif L==9	// pt
	"Trabalhos"
#endif
	,
#if   L==1	// ca	// Brw_UNKNOWN
	"Total"
#elif L==2	// de
	"Gesamtzahl"
#elif L==3	// en
	"Total"
#elif L==4	// es
	"Total"
#elif L==5	// fr
	"Total"
#elif L==6	// gn
	"Total"		// Okoteve traduccin
#elif L==7	// it
	"Totale"
#elif L==8	// pl
	"Ca&lstrok;kowity"
#elif L==9	// pt
	"Total"
#endif
	,
#if   L==1	// ca	// Brw_ADMI_BRF_USR
	"Maletins"
#elif L==2	// de
	"Aktentasche"
#elif L==3	// en
	"Briefcases"
#elif L==4	// es
	"Maletines"
#elif L==5	// fr
	"Porte-documents"
#elif L==6	// gn
	"Maletines"	// Okoteve traduccin
#elif L==7	// it
	"Penna USB"
#elif L==8	// pl
	"Briefcases"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pasta"
#endif
	};

const char *Txt_STAT_DEGREE_PHOTO_SIZE[Pho_NUM_HOW_COMPUTE_PHOTO_SIZES] =
   {
#if   L==1	// ca
	 "proporcional al n&ordm; de estudiantes"	// Necessita traduccio
#elif L==2	// de
	 "proportional to the number of students"	// Need bersetzung
#elif L==3	// en
	 "proportional to the number of students"
#elif L==4	// es
	 "proporcional al n&ordm; de estudiantes"
#elif L==5	// fr
	 "proportional to the number of students"	// Besoin de traduction
#elif L==6	// gn
	 "proporcional al n&ordm; de estudiantes"	// Okoteve traduccin
#elif L==7	// it
	 "proporzionale al numero di studenti"
#elif L==8	// pl
	 "proportional to the number of students"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "proporcional ao n&uacute;mero de estudantes"
#endif
	 ,
#if   L==1	// ca
	 "proporcional al n&ordm; de estudiantes con fotograf&iacute;a"	// Necessita traduccio
#elif L==2	// de
	 "proportional to the number of students with photo"		// Need bersetzung
#elif L==3	// en
	 "proportional to the number of students with photo"
#elif L==4	// es
	 "proporcional al n&ordm; de estudiantes con fotograf&iacute;a"
#elif L==5	// fr
	 "proportional to the number of students with photo"		// Besoin de traduction
#elif L==6	// gn
	 "proporcional al n&ordm; de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	 "proporzionale al numero di studenti con foto"
#elif L==8	// pl
	 "proportional to the number of students with photo"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "proporcional ao n&uacute;mero de estudantes com foto"
#endif
	 ,
#if   L==1	// ca
	 "proporcional al porcentaje de estudiantes con fotograf&iacute;a"	// Necessita traduccio
#elif L==2	// de
	 "proportional to the percentage of students with photo"		// Need bersetzung
#elif L==3	// en
	 "proportional to the percentage of students with photo"
#elif L==4	// es
	 "proporcional al porcentaje de estudiantes con fotograf&iacute;a"
#elif L==5	// fr
	 "proportional to the percentage of students with photo"		// Besoin de traduction
#elif L==6	// gn
	 "proporcional al porcentaje de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	 "proporzionale alla percentuale di studenti con foto"
#elif L==8	// pl
	 "proportional to the percentage of students with photo"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "proporcional ao porcentagem de estudantes com foto"
#endif
	 ,
#if   L==1	// ca
	 "igual para todas las titulaciones"	// Necessita traduccio
#elif L==2	// de
	 "same for all degrees"			// Need bersetzung
#elif L==3	// en
	 "same for all degrees"
#elif L==4	// es
	 "igual para todas las titulaciones"
#elif L==5	// fr
	 "same for all degrees"			// Besoin de traduction
#elif L==6	// gn
	 "igual para todas las titulaciones"	// Okoteve traduccin
#elif L==7	// it
	 "uguale per tutte le lauree"
#elif L==8	// pl
	 "same for all degrees"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "mesmo para todos os graus"
#endif
	};

const char *Txt_STAT_DEGREE_PHOTO_ORDER[Pho_NUM_HOW_ORDER_DEGREES] =
   {
#if   L==1	// ca
	 "nombre d'estudiants"
#elif L==2	// de
	 "Anzahl der Studenten"
#elif L==3	// en
	 "number of students"
#elif L==4	// es
	 "n&ordm; de estudiantes"
#elif L==5	// fr
	 "nombre d'&eacute;tudiants"
#elif L==6	// gn
	 "n&ordm; de estudiantes"	// Okoteve traduccin
#elif L==7	// it
	 "numero di studenti"
#elif L==8	// pl
	 "liczba student&oacute;w"
#elif L==9	// pt
	 "n&uacute;mero de estudantes"
#endif
	 ,
#if   L==1	// ca
	 "nombre d'estudiants amb fotografia"
#elif L==2	// de
	 "Anzahl der Studenten mit Foto"
#elif L==3	// en
	 "number of students with photo"
#elif L==4	// es
	 "n&ordm; de estudiantes con fotograf&iacute;a"
#elif L==5	// fr
	 "nombre d'&eacute;tudiants avec photo"
#elif L==6	// gn
	 "n&ordm; de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	 "numero di studenti con foto"
#elif L==8	// pl
	 "liczba student&oacute;w ze zdj&eogon;ciem"
#elif L==9	// pt
	 "n&uacute;mero de estudantes com foto"
#endif
	 ,
#if   L==1	// ca
	 "porcentaje de estudiantes con fotograf&iacute;a"	// Necessita traduccio
#elif L==2	// de
	 "percentage of students with photo"			// Need bersetzung
#elif L==3	// en
	 "percentage of students with photo"
#elif L==4	// es
	 "porcentaje de estudiantes con fotograf&iacute;a"
#elif L==5	// fr
	 "percentage of students with photo"			// Besoin de traduction
#elif L==6	// gn
	 "porcentaje de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7	// it
	 "percentuale di studenti con foto"
#elif L==8	// pl
	 "percentage of students with photo"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "porcentagem de estudantes com foto"
#endif
	 ,
#if   L==1	// ca
	 "nombre de titulaci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	 "degree name"			// Need bersetzung
#elif L==3	// en
	 "degree name"
#elif L==4	// es
	 "nombre de titulaci&oacute;n"
#elif L==5	// fr
	 "degree name"			// Besoin de traduction
#elif L==6	// gn
	 "nombre de titulaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "nome laurea"
#elif L==8	// pl
	 "degree name"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "nome do grau"
#endif
	};

const char *Txt_STAT_TYPE_COUNT_CAPS[Sta_NUM_COUNT_TYPES] =
   {
#if   L==1	// ca
	 "N&ordm; total de clics"	// Necessita traduccio
#elif L==2	// de
	 "Gesamtzahl der Klicks"
#elif L==3	// en
	 "Total number of clicks"
#elif L==4	// es
	 "N&ordm; total de clics"
#elif L==5	// fr
	 "Nombre total de clics"
#elif L==6	// gn
	 "N&ordm; total de clics"	// Okoteve traduccin
#elif L==7	// it
	 "Numero totale di scatti"
#elif L==8	// pl
	 "Total number of clicks"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "N&uacute;mero total de cliques"
#endif
	 ,
#if   L==1	// ca
	 "N&ordm; de usuarios distintos"		// Necessita traduccio
#elif L==2	// de
	 "Anzahl der unterschiedlichen Benutzer"
#elif L==3	// en
	 "Number of distinct users"
#elif L==4	// es
	 "N&ordm; de usuarios distintos"
#elif L==5	// fr
	 "Nombre des diff&eacute;rents utilisateurs"
#elif L==6	// gn
	 "N&ordm; de usuarios distintos"		// Okoteve traduccin
#elif L==7	// it
	 "Numero di utenti distinti"
#elif L==8	// pl
	 "Number of distinct users"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "N&uacute;mero de utilizadores diferentes"
#endif
	 ,
#if   L==1	// ca
	 "N&ordm; de clics por usuario"		// Necessita traduccio
#elif L==2	// de
	 "Anzahl der Klicks pro Benutzer"
#elif L==3	// en
	 "Number of clicks per user"
#elif L==4	// es
	 "N&ordm; de clics por usuario"
#elif L==5	// fr
	 "Nombre de clics par utilisateur"
#elif L==6	// gn
	 "N&ordm; de clics por usuario"		// Okoteve traduccin
#elif L==7	// it
	 "Numero di scatti per utente"
#elif L==8	// pl
	 "Number of clicks per user"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "n&uacute;mero de cliques por utilizador"
#endif
	 ,
#if   L==1	// ca
	 "Tiempo de generaci&oacute;n de p&aacute;gina"	// Necessita traduccio
#elif L==2	// de
	 "Zeit zur Erzeugung der Seite"
#elif L==3	// en
	 "Time to generate page"
#elif L==4	// es
	 "Tiempo de generaci&oacute;n de p&aacute;gina"
#elif L==5	// fr
	 "Temps pour g&eacute;n&eacute;rer la page"
#elif L==6	// gn
	 "Tiempo de generaci&oacute;n de p&aacute;gina"	// Okoteve traduccin
#elif L==7	// it
	 "Tempo per generare pagina"
#elif L==8	// pl
	 "Time to generate page"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Tempo para gerar p&aacute;gina"
#endif
	 ,
#if   L==1	// ca
	 "Tiempo de env&iacute;o de p&aacute;gina"	// Necessita traduccio
#elif L==2	// de
	 "Zeit, um die Seite zu senden"
#elif L==3	// en
	 "Time to send page"
#elif L==4	// es
	 "Tiempo de env&iacute;o de p&aacute;gina"
#elif L==5	// fr
	 "Temps pour envoyer la page"
#elif L==6	// gn
	 "Tiempo de env&iacute;o de p&aacute;gina"	// Okoteve traduccin
#elif L==7	// it
	 "Tempo d'invio della pagina"
#elif L==8	// pl
	 "Time to send page"				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Tempo para enviar a p&aacute;gina"
#endif
	};

const char *Txt_STAT_TYPE_COUNT_SMALL[Sta_NUM_COUNT_TYPES] =
   {
#if   L==1	// ca
	 "n&ordm; total de clics"	// Necessita traduccio
#elif L==2	// de
	 "Gesamtzahl der Klicks"
#elif L==3	// en
	 "total number of clicks"
#elif L==4	// es
	 "n&ordm; total de clics"
#elif L==5	// fr
	 "nombre total de clics"
#elif L==6	// gn
	 "n&ordm; total de clics"	// Okoteve traduccin
#elif L==7	// it
	 "numero totale di scatti"
#elif L==8	// pl
	 "total number of clicks"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "n&uacute;mero total de cliques"
#endif
	 ,
#if   L==1	// ca
	 "n&ordm; de usuarios distintos"		// Necessita traduccio
#elif L==2	// de
	 "Anzahl der unterschiedliche Benutzer"
#elif L==3	// en
	 "number of distinct users"
#elif L==4	// es
	 "n&ordm; de usuarios distintos"
#elif L==5	// fr
	 "nombre des diff&eacute;rents utilisateurs"
#elif L==6	// gn
	 "n&ordm; de usuarios distintos"		// Okoteve traduccin
#elif L==7	// it
	 "numero di utenti distinti"
#elif L==8	// pl
	 "number of distinct users"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "n&uacute;mero de utilizadores diferentes"
#endif
	 ,
#if   L==1	// ca
	 "n&ordm; de clics por usuario"		// Necessita traduccio
#elif L==2	// de
	 "Anzahl der Klicks pro Benutzer"
#elif L==3	// en
	 "number of clicks per user"
#elif L==4	// es
	 "n&ordm; de clics por usuario"
#elif L==5	// fr
	 "nombre de clics par utilisateur"
#elif L==6	// gn
	 "n&ordm; de clics por usuario"		// Okoteve traduccin
#elif L==7	// it
	 "numero di scatti per utente"
#elif L==8	// pl
	 "number of clicks per user"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "n&uacute;mero de cliques por utilizador"
#endif
	 ,
#if   L==1	// ca
	 "tiempo de generaci&oacute;n"		// Necessita traduccio
#elif L==2	// de
	 "Zeit zur Erzeugung"
#elif L==3	// en
	 "time to generate"
#elif L==4	// es
	 "tiempo de generaci&oacute;n"
#elif L==5	// fr
	 "temps pour g&eacute;n&eacute;rer"
#elif L==6	// gn
	 "tiempo de generaci&oacute;n"		// Okoteve traduccin
#elif L==7	// it
	 "tempo per generare"
#elif L==8	// pl
	 "time to generate"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "tempo para gerar"
#endif
	 ,
#if   L==1	// ca
	 "tiempo de env&iacute;o"	// Necessita traduccio
#elif L==2	// de
	 "Zeit, um Seite"
#elif L==3	// en
	 "time to send"
#elif L==4	// es
	 "tiempo de env&iacute;o"
#elif L==5	// fr
	 "temps pour envoyer"
#elif L==6	// gn
	 "tiempo de env&iacute;o"	// Okoteve traduccin
#elif L==7	// it
	 "tempo d'invio"
#elif L==8	// pl
	 "time to send"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Tempo para enviar"
#endif
	};

const char *Txt_FIGURE_TYPES[Fig_NUM_FIGURES] =
   {
#if   L==1	// ca				// Fig_USERS
	 "Usuaris"
#elif L==2	// de
	 "Benutzer"
#elif L==3	// en
	 "Users"
#elif L==4	// es
	 "Usuarios"
#elif L==5	// fr
	 "Utilisateurs"
#elif L==6	// gn
	 "Puruh&aacute;ra"
#elif L==7	// it
	 "Utenti"
#elif L==8	// pl
	 "U&zdot;ytkownicy"
#elif L==9	// pt
	 "Utilizadores"
#endif
	 ,
#if   L==1	// ca				// Fig_USERS_RANKING
	 "R&agrave;nquing"
#elif L==2	// de
	 "Rang"
#elif L==3	// en
	 "Ranking"
#elif L==4	// es
	 "Ranking"
#elif L==5	// fr
	 "Classement"
#elif L==6	// gn
	 "Ranking"	// Okoteve traduccin
#elif L==7	// it
	 "Posto"
#elif L==8	// pl
	 "Ranking"
#elif L==9	// pt
	 "Posi&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// ca				// Fig_HIERARCHY
	 "Jerarquia"
#elif L==2	// de
	 "Hierarchie"
#elif L==3	// en
	 "Hierarchy"
#elif L==4	// es
	 "Jerarqu&iacute;a"
#elif L==5	// fr
	 "Hi&eacute;rarchie"
#elif L==6	// gn
	 "S&atilde;mbyhyha reko"
#elif L==7	// it
	 "Gerarchia"
#elif L==8	// pl
	 "Hierarchia"
#elif L==9	// pt
	 "Hierarquia"
#endif
	 ,
#if   L==1	// ca				// Fig_INSTITS
	"Institucions"
#elif L==2	// de
	"Hochschulen"
#elif L==3	// en
	"Institutions"
#elif L==4	// es
	"Instituciones"
#elif L==5	// fr
	"&Eacute;tablissements"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Istituzioni"
#elif L==8	// pl
	"Instytucje"
#elif L==9	// pt
	"Institu&ccedil;&otilde;es"
#endif
	 ,
#if   L==1	// ca				// Fig_DEGREE_TYPES
	"Tipus de titulaci&oacute;"
#elif L==2	// de
	"Abschlussart"
#elif L==3	// en
	"Types of degree"
#elif L==4	// es
	"Tipos de titulaci&oacute;n"
#elif L==5	// fr
	"Types d'&eacute;tude"
#elif L==6	// gn
	"Arandur&atilde; l&aacute;ja"
#elif L==7	// it
	"Tipi di laurea"
#elif L==8	// pl
	"Rodzaje stopni"
#elif L==9	// pt
	"Tipos de grau"
#endif
	 ,
#if   L==1	// ca				// Fig_FOLDERS_AND_FILES
	 "Carpetes i fitxers"
#elif L==2	// de
	 "Verzeichnisse und Dateien"
#elif L==3	// en
	 "Folders and files"
#elif L==4	// es
	 "Carpetas y archivos"
#elif L==5	// fr
	 "R&eacute;pertoires et fichiers"
#elif L==6	// gn
	 "Carpetas y archivos"		// Okoteve traduccin
#elif L==7	// it
	 "Cartelle e file"
#elif L==8	// pl
	 "Folders and files"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Diret&oacute;rios e arquivos"
#endif
	 ,
#if   L==1	// ca				// Fig_OER
	 "Recursos Educatius Oberts (OER)"
#elif L==2	// de
	 "Open Educational Resources (OER)"
#elif L==3	// en
	 "Open Educational Resources (OER)"
#elif L==4	// es
	 "Recursos Educativos Abiertos (OER)"
#elif L==5	// fr
	 "Ressources &Eacute;ducatives Libres (REL)"
#elif L==6	// gn
	 "Recursos Educativos Abiertos (OER)"	// Okoteve traduccin
#elif L==7	// it
	 "Risorse Didattiche Aperte (OER)"
#elif L==8	// pl
	 "Otwarte Zasoby Edukacyjne (OER)"
#elif L==9	// pt
	 "Recursos Educacionais Abertos (OER)"
#endif
	 ,
#if   L==1	// ca				// Fig_ASSIGNMENTS
	 "Activitats"
#elif L==2	// de
	 "Aufgaben"
#elif L==3	// en
	 "Assignments"
#elif L==4	// es
	 "Actividades"
#elif L==5	// fr
	 "Activit&eacute;s"
#elif L==6	// gn
	 "Tar&eacute;a"
#elif L==7	// it
	 "Attivit&agrave;"
#elif L==8	// pl
	 "Zadania"
#elif L==9	// pt
	 "Atividades"
#endif
	 ,
#if   L==1	// ca				// Fig_PROJECTS
	"Projectes"
#elif L==2	// de
	"Projekte"
#elif L==3	// en
	"Projects"
#elif L==4	// es
	"Proyectos"
#elif L==5	// fr
	"Projets"
#elif L==6	// gn
	"Proyectos"	// Okoteve traduccin
#elif L==7	// it
	"Progetti"
#elif L==8	// pl
	"Projekty"
#elif L==9	// pt
	"Projetos"
#endif
	 ,
#if   L==1	// ca				// Fig_TESTS
	 "Tests"
#elif L==2	// de
	 "Tests"
#elif L==3	// en
	 "Tests"
#elif L==4	// es
	 "Test"
#elif L==5	// fr
	 "Tests"
#elif L==6	// gn
	 "Test"		// Okoteve traduccin
#elif L==7	// it
	 "Test"
#elif L==8	// pl
	 "Testy"
#elif L==9	// pt
	 "Testes"
#endif
	 ,
#if   L==1	// ca				// Fig_GAMES
	"Jocs"
#elif L==2	// de
	"Spiele"
#elif L==3	// en
	"Games"
#elif L==4	// es
	"Juegos"
#elif L==5	// fr
	"Jeux"
#elif L==6	// gn
	"Juegos"	// Okoteve traduccin
#elif L==7	// it
	"Giochi"
#elif L==8	// pl
	"Gry"
#elif L==9	// pt
	"Jogos"
#endif
	 ,
#if   L==1	// ca				// Fig_SURVEYS
	 "Enquestes"
#elif L==2	// de
	 "Umfragen"
#elif L==3	// en
	 "Surveys"
#elif L==4	// es
	 "Encuestas"
#elif L==5	// fr
	 "Sondages"
#elif L==6	// gn
	 "Encuestas"	// Okoteve traduccin
#elif L==7	// it
	 "Sondaggi"
#elif L==8	// pl
	 "Badania"
#elif L==9	// pt
	 "Inqu&eacute;ritos"
#endif
	 ,
#if   L==1	// ca				// Fig_SOCIAL_ACTIVITY
	 "Timeline"
#elif L==2	// de
	 "Timeline"
#elif L==3	// en
	 "Timeline"
#elif L==4	// es
	 "Timeline"
#elif L==5	// fr
	 "Chronologie"
#elif L==6	// gn
	 "Timeline"	// Okoteve traduccin
#elif L==7	// it
	 "Timeline"
#elif L==8	// pl
	 "Timeline"
#elif L==9	// pt
	 "Timeline"
#endif
	 ,
#if   L==1	// ca				// Fig_FOLLOW
	"Seguits / Seguidors"
#elif L==2	// de
	"Gefolgt / Anh&auml;nger"
#elif L==3	// en
	"Followed / Followers"
#elif L==4	// es
	"Seguidos / Seguidores"
#elif L==5	// fr
	"Suivis / Suiveurs"
#elif L==6	// gn
	"Seguidos / Seguidores"	// Okoteve traduccin
#elif L==7	// it
	"Seguiti / Seguaci"
#elif L==8	// pl
	"Obserwowani / Obserwuj&aogon;"
#elif L==9	// pt
	"Seguidos / Seguidores"
#endif
	 ,
#if   L==1	// ca				// Fig_FORUMS
	 "F&ograve;rums"
#elif L==2	// de
	 "Foren"
#elif L==3	// en
	 "Forums"
#elif L==4	// es
	 "Foros"
#elif L==5	// fr
	 "Forums"
#elif L==6	// gn
	 "Foros"		// Okoteve traduccin
#elif L==7	// it
	 "Forum"
#elif L==8	// pl
	 "Forums"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "F&oacute;runs"
#endif
	 ,
#if   L==1	// ca				// Fig_NOTIFY_EVENTS
	"Notificacions enviades per correu"
#elif L==2	// de
	"Anmeldungen per E-Mail gesendet"
#elif L==3	// en
	"Notifications sent by email"
#elif L==4	// es
	"Notificaciones enviadas por correo"
#elif L==5	// fr
	"Notifications envoy&eacute;es par courriel"
#elif L==6	// gn
	"Notificaciones enviadas por correo"	// Okoteve traduccin
#elif L==7	// it
	"Notifiche inviate via email"
#elif L==8	// pl
	"Powiadomienia wysy&lstrok;ane przez email"
#elif L==9	// pt
	"Notifica&ccedil;&otilde;es enviadas por email"
#endif
	 ,
#if   L==1	// ca				// Fig_NOTICES
	 "Avisos"
#elif L==2	// de
	 "Ank&uuml;ndigungen"
#elif L==3	// en
	 "Notices"
#elif L==4	// es
	 "Avisos"
#elif L==5	// fr
	 "Avis"
#elif L==6	// gn
	 "Marandu"
#elif L==7	// it
	 "Avvisi"
#elif L==8	// pl
	 "Ogloszenia"
#elif L==9	// pt
	 "Avisos"
#endif
	 ,
#if   L==1	// ca				// Fig_MESSAGES
	 "Missatges"
#elif L==2	// de
	 "Nachrichten"
#elif L==3	// en
	 "Messages"
#elif L==4	// es
	 "Mensajes"
#elif L==5	// fr
	 "Messages"
#elif L==6	// gn
	 "Marandu"
#elif L==7	// it
	 "Messaggi"
#elif L==8	// pl
	 "Wiadomo&sacute;ci"
#elif L==9	// pt
	 "Mensagens"
#endif
	 ,
#if   L==1	// ca				// Fig_SOCIAL_NETWORKS
	 "Webs / xarxes socials"
#elif L==2	// de
	 "Webs / Social Networks"
#elif L==3	// en
	 "Webs / social networks"
#elif L==4	// es
	 "Webs / redes sociales"
#elif L==5	// fr
	 "Webs / r&eacute;seaux sociaux"
#elif L==6	// gn
	 "Webs / redes sociales"	// Okoteve traduccin
#elif L==7	// it
	 "Webs / reti sociali"
#elif L==8	// pl
	 "Webs / sieci spo&lstrok;eczne"
#elif L==9	// pt
	 "Webs / redes sociais"
#endif
	 ,
#if   L==1	// ca				// Fig_LANGUAGES
	 "Idioma"
#elif L==2	// de
	 "Sprache"
#elif L==3	// en
	 "Language"
#elif L==4	// es
	 "Idioma"
#elif L==5	// fr
	 "Langue"
#elif L==6	// gn
	 "&Ntilde;e'&#7869;"
#elif L==7	// it
	 "Lingua"
#elif L==8	// pl
	 "Language"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Lingua"
#endif
	 ,
#if   L==1	// ca				// Fig_FIRST_DAY_OF_WEEK
	"Calendari"
#elif L==2	// de
	"Kalender"
#elif L==3	// en
	"Calendar"
#elif L==4	// es
	"Calendario"
#elif L==5	// fr
	"Calendrier"
#elif L==6	// gn
	"Arapapaha"
#elif L==7	// it
	"Calendario"
#elif L==8	// pl
	"Kalendarz"
#elif L==9	// pt
	"Calend&aacute;rio"
#endif
	 ,
#if   L==1	// ca				// Fig_DATE_FORMAT
	"Dates"
#elif L==2	// de
	"Datum"
#elif L==3	// en
	"Dates"
#elif L==4	// es
	"Fechas"
#elif L==5	// fr
	"Dates"
#elif L==6	// gn
	"Arange"
#elif L==7	// it
	"Date"
#elif L==8	// pl
	"Daty"
#elif L==9	// pt
	"Datas"
#endif
	 ,
#if   L==1	// ca				// Fig_ICON_SETS
	 "Icones"
#elif L==2	// de
	 "Symbole"
#elif L==3	// en
	 "Icons"
#elif L==4	// es
	 "Iconos"
#elif L==5	// fr
	 "Ic&ocirc;nes"
#elif L==6	// gn
	 "Iconos"		// Okoteve traduccin
#elif L==7	// it
	 "Icone"
#elif L==8	// pl
	 "Ikony"
#elif L==9	// pt
	 "&Iacute;cones"
#endif
	 ,
#if   L==1	// ca				// Fig_MENUS
	"Men&uacute;"
#elif L==2	// de
	"Men&uuml;"
#elif L==3	// en
	"Menu"
#elif L==4	// es
	"Men&uacute;"
#elif L==5	// fr
	"Menu"
#elif L==6	// gn
	"Men&uacute;"	// Okoteve traduccin
#elif L==7	// it
	"Menu"
#elif L==8	// pl
	"Menu"
#elif L==9	// pt
	"Menu"
#endif
	 ,
#if   L==1	// ca				// Fig_THEMES
	 "Tema (colors)"
#elif L==2	// de
	 "Skin"
#elif L==3	// en
	 "Theme"
#elif L==4	// es
	 "Tema (colores)"
#elif L==5	// fr
	 "Th&egrave;me"
#elif L==6	// gn
	 "Tema (colores)"	// Okoteve traduccin
#elif L==7	// it
	 "Tema"
#elif L==8	// pl
	 "Temat"
#elif L==9	// pt
	 "Skin"
#endif
	 ,
#if   L==1	// ca				// Fig_SIDE_COLUMNS
	 "Columnes"
#elif L==2	// de
	 "Spalten"
#elif L==3	// en
	 "Columns"
#elif L==4	// es
	 "Columnas"
#elif L==5	// fr
	 "Colonnes"
#elif L==6	// gn
	 "Columnas"	// Okoteve traduccin
#elif L==7	// it
	 "Colonne"
#elif L==8	// pl
	 "Kolumny"
#elif L==9	// pt
	 "Colunas"
#endif
	 ,
#if   L==1	// ca				// Fig_PRIVACY
	"Privacitat"
#elif L==2	// de
	"Geheimhaltung"
#elif L==3	// en
	"Privacy"
#elif L==4	// es
	"Privacidad"
#elif L==5	// fr
	"Privacit&eacute;"
#elif L==6	// gn
	"Privacidad"	// Okoteve traduccin
#elif L==7	// it
	"Privatezza"
#elif L==8	// pl
	"Prywatno&sacute;&cacute;"
#elif L==9	// pt
	"Privacidade"
#endif
	 ,
#if   L==1	// ca				// Fig_COOKIES
	"Cookies"
#elif L==2	// de
	"Cookies"
#elif L==3	// en
	"Cookies"
#elif L==4	// es
	"Cookies"
#elif L==5	// fr
	"Cookies"
#elif L==6	// gn
	"Cookies"
#elif L==7	// it
	"Cookies"
#elif L==8	// pl
	"Cookies"
#elif L==9	// pt
	"Cookies"
#endif
	};

const char *Txt_Statistic =
#if   L==1	// ca
	 "Estad&iacute;stique";
#elif L==2	// de
	 "Statistik";
#elif L==3	// en
	 "Statistic";
#elif L==4	// es
	 "Estad&iacute;stica";
#elif L==5	// fr
	 "Statistique";
#elif L==6	// gn
	 "Papy'aporeko";
#elif L==7	// it
	 "Statistica";
#elif L==8	// pl
	 "Statystyki";
#elif L==9	// pt
	 "Estat&iacute;sticas";
#endif

const char *Txt_Statistics_of_all_visits =
#if   L==1	// ca
	"Estad&iacute;stica de todos los accesos";	// Necessita traduccio
#elif L==2	// de
	"Statistics of all visits";			// Need bersetzung
#elif L==3	// en
	"Statistics of all visits";
#elif L==4	// es
	"Estad&iacute;stica de todos los accesos";
#elif L==5	// fr
	"Statistics of all visits";			// Besoin de traduction
#elif L==6	// gn
	"Estad&iacute;stica de todos los accesos";	// Okoteve traduccin
#elif L==7	// it
	"Statistiche di tutte gli accessi";
#elif L==8	// pl
	"Statistics of all visits";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Estat&iacute;sticas de todas as visitas";
#endif

const char *Txt_Statistics_of_visits_to_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Estad&iacute;stica d'accessos a l'assignatura %s";
#elif L==2	// de
	"Statistik der Besuche des Kurses %s";
#elif L==3	// en
	"Statistics of visits to the course %s";
#elif L==4	// es
	"Estad&iacute;stica de accesos a la asignatura %s";
#elif L==5	// fr
	"Statistiques de visites a la mati&egrave;re %s";
#elif L==6	// gn
	"Estad&iacute;stica de accesos a la asignatura %s";	// Okoteve traduccin
#elif L==7	// it
	"Statistiche delle visite al corso %s";
#elif L==8	// pl
	"Statystyki odwiedzin kursu %s";
#elif L==9	// pt
	"Estat&iacute;sticas de visitas &agrave; disciplina %s";
#endif

const char *Txt_Status =
#if   L==1	// ca
	"Estado";	// Necessita traduccio
#elif L==2	// de
	"Status";
#elif L==3	// en
	"Status";
#elif L==4	// es
	"Estado";
#elif L==5	// fr
	"Statut";
#elif L==6	// gn
	"O&#297; h&aacute;icha";
#elif L==7	// it
	"Stato";
#elif L==8	// pl
	"Status";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Status";
#endif

const char *Txt_Stay_connected_with_SWADroid =
#if   L==1	// ca
	"Seguiu connectat/da amb <strong>SWADroid</strong>.";
#elif L==2	// de
	"Stay connected with <strong>SWADroid</strong>.";		// Need bersetzung
#elif L==3	// en
	"Stay connected with <strong>SWADroid</strong>.";
#elif L==4	// es
	"Siga conectado/a con <strong>SWADroid</strong>.";
#elif L==5	// fr
	"Stay connected with <strong>SWADroid</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"Siga conectado/a con <strong>SWADroid</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"Resta in contatto con <strong>SWADroid</strong>.";
#elif L==8	// pl
	"Stay connected with <strong>SWADroid</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Fique ligado/a com <strong>SWADroid</strong>.";
#endif

const char *Txt_Stem =		// Stem of a test question
#if   L==1	// ca
	"Enunciat";
#elif L==2	// de
	"Stamm";
#elif L==3	// en
	"Stem";
#elif L==4	// es
	"Enunciado";
#elif L==5	// fr
	"Question";
#elif L==6	// gn
	"Enunciado";	// Okoteve traduccin
#elif L==7	// it
	"Domanda";
#elif L==8	// pl
	"Stem";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Quest&atilde;o";
#endif

const char *Txt_Step_1_Provide_a_list_of_users =
#if   L==1	// ca
	"Paso 1: Proporcione una lista de usuarios";	// Necessita traduccio
#elif L==2	// de
	"Step 1: Provide a list of users";		// Need bersetzung
#elif L==3	// en
	"Step 1: Provide a list of users";
#elif L==4	// es
	"Paso 1: Proporcione una lista de usuarios";
#elif L==5	// fr
	"Step 1: Provide a list of users";		// Besoin de traduction
#elif L==6	// gn
	"Paso 1: Proporcione una lista de usuarios";	// Okoteve traduccin
#elif L==7	// it
	"Passo 1: Fornisci una lista di utenti";
#elif L==8	// pl
	"Step 1: Provide a list of users";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Etapa 1: Forne&ccedil;a uma lista de usu&aacute;rios";
#endif

const char *Txt_Step_2_Select_the_desired_action =
#if   L==1	// ca
	"Paso 2: Seleccione la acci&oacute;n que desea realizar";	// Necessita traduccio
#elif L==2	// de
	"Step 2: Select the desired action";				// Need bersetzung
#elif L==3	// en
	"Step 2: Select the desired action";
#elif L==4	// es
	"Paso 2: Seleccione la acci&oacute;n que desea realizar";
#elif L==5	// fr
	"Step 2: Select the desired action";				// Besoin de traduction
#elif L==6	// gn
	"Paso 2: Seleccione la acci&oacute;n que desea realizar";	// Okoteve traduccin
#elif L==7	// it
	"Passo 2: Seleziona l'azione desiderata";
#elif L==8	// pl
	"Step 2: Select the desired action";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Etapa 2: Selecione a a&ccedil;&atilde;o desejada";
#endif

const char *Txt_Step_3_Optionally_select_groups =
#if   L==1	// ca
	"Paso 3: Opcionalmente, seleccione grupos";	// Necessita traduccio
#elif L==2	// de
	"Step 3: Optionally, select groups";		// Need bersetzung
#elif L==3	// en
	"Step 3: Optionally, select groups";
#elif L==4	// es
	"Paso 3: Opcionalmente, seleccione grupos";
#elif L==5	// fr
	"Step 3: Optionally, select groups";		// Besoin de traduction
#elif L==6	// gn
	"Paso 3: Opcionalmente, seleccione grupos";	// Okoteve traduccin
#elif L==7	// it
	"Passo 3: A scelta, seleziona i gruppi";
#elif L==8	// pl
	"Step 3: Optionally, select groups";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Etapa 3: Opcionalmente, selecione grupos";
#endif

const char *Txt_Step_4_Confirm_the_enrolment_removing =
#if   L==1	// ca
	"Paso 4: Confirme la inscripci&oacute;n / eliminaci&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Step 4: Confirm the enrolment / removing";			// Need bersetzung
#elif L==3	// en
	"Step 4: Confirm the enrolment / removing";
#elif L==4	// es
	"Paso 4: Confirme la inscripci&oacute;n / eliminaci&oacute;n";
#elif L==5	// fr
	"Step 4: Confirm the enrolment / removing";			// Besoin de traduction
#elif L==6	// gn
	"Paso 4: Confirme la inscripci&oacute;n / eliminaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Passo 4: Conferma la registrazione / rimozione";
#elif L==8	// pl
	"Step 4: Confirm the enrolment / removing";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Etapa 4: Confirmar a inscri&ccedil;&atilde;o / remo&ccedil;&atilde;o";
#endif

const char *Txt_Student_comment =
#if   L==1	// ca
	"Comentari de l'estudiant";
#elif L==2	// de
	"Student-Kommentar";
#elif L==3	// en
	"Student's comment";
#elif L==4	// es
	"Comentario del estudiante";
#elif L==5	// fr
	"Commentaire de l'&eacute;tudiant";
#elif L==6	// gn
	"Comentario del estudiante";	// Okoteve traduccin
#elif L==7	// it
	"Commento di student";
#elif L==8	// pl
	"Komentarz studenta";
#elif L==9	// pt
	"Coment&aacute;rio do estudante";
#endif

const char *Txt_students_ABBREVIATION =	// Abbreviation of "students"
#if   L==1	// ca
	"estu.";
#elif L==2	// de
	"Stud.";
#elif L==3	// en
	"stud.";
#elif L==4	// es
	"estu.";
#elif L==5	// fr
	"&eacute;tu.";
#elif L==6	// gn
	"Temi.";
#elif L==7	// it
	"stud.";
#elif L==8	// pl
	"stud.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"estu.";
#endif

const char *Txt_Student_record_card_in_this_course_has_been_updated =
#if   L==1	// ca
	"La ficha del estudiante en la asignatura se ha actualizado.";	// Necessita traduccio
#elif L==2	// de
	"Student record card in this course has been updated.";		// Need bersetzung
#elif L==3	// en
	"Student record card in this course has been updated.";
#elif L==4	// es
	"La ficha del estudiante en la asignatura se ha actualizado.";
#elif L==5	// fr
	"Student record card in this course has been updated.";		// Besoin de traduction
#elif L==6	// gn
	"La ficha del estudiante en la asignatura se ha actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"La scheda dello/a studente/ssa in questo corso &egrave; stata aggiornata.";
#elif L==8	// pl
	"Student record card in this course has been updated.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O cart&atilde;o do estudante nesta disciplina foi atualizado.";
#endif

const char *Txt_Students_are_no_longer_obliged_to_read_this_information =
#if   L==1	// ca
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"Students are no longer obliged to read this information.";			// Need bersetzung
#elif L==3	// en
	"Students are no longer obliged to read this information.";
#elif L==4	// es
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";
#elif L==5	// fr
	"Students are no longer obliged to read this information.";			// Besoin de traduction
#elif L==6	// gn
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Gli studenti non sono obbligati a leggere questa informazione.";
#elif L==8	// pl
	"Students are no longer obliged to read this information.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Os estudantes n&atilde;o precisam mais ler essas informa&ccedil;&otilde;es.";
#endif

const char *Txt_Students_now_are_required_to_read_this_information =
#if   L==1	// ca
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"Students now are required to read this information.";				// Need bersetzung
#elif L==3	// en
	"Students now are required to read this information.";
#elif L==4	// es
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";
#elif L==5	// fr
	"Students now are required to read this information.";				// Besoin de traduction
#elif L==6	// gn
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Gli studenti ora sono obbligati a leggere questa informazione.";
#elif L==8	// pl
	"Students now are required to read this information.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Os estudantes agora precisam ler essas informa&ccedil;&otilde;es.";
#endif

const char *Txt_X_students_who_have_email =	// Warning: it is very important to include %u and %f in the following sentences
#if   L==1	// ca
	"%u estudiantes con correo (%.1f%% de %u estudiantes)";	// Necessita traduccio
#elif L==2	// de
	"%u Studenten mit E-Mail (%.1f%% von %u Studenten)";
#elif L==3	// en
	"%u students who have email (%.1f%% of %u students)";
#elif L==4	// es
	"%u estudiantes con correo (%.1f%% de %u estudiantes)";
#elif L==5	// fr
	"%u &eacute;tudiants qui ont l'email (%.1f%% de %u &eacute;tudiants)";
#elif L==6	// gn
	"%u estudiantes con correo (%.1f%% de %u estudiantes)";	// Okoteve traduccin
#elif L==7	// it
	"%u studenti che hanno email (%.1f%% di %u studenti)";
#elif L==8	// pl
	"%u students who have email (%.1f%% of %u students)";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"%u estudantes que t&ecirc;m email (%.1f%% dos %u estudantes)";
#endif

const char *Txt_Students_who_have_accepted_and_who_have_email =
#if   L==1	// ca
	"Estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo";	// Necessita traduccio
#elif L==2	// de
	"Studenten die akzeptiert wurden"
	" und die E-Mail";
#elif L==3	// en
	"Students who have accepted"
	" and who have email";
#elif L==4	// es
	"Estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo";
#elif L==5	// fr
	"&Eacute;tudiants qui ont accept&eacute;"
	" et qui ont l'email";
#elif L==6	// gn
	"Estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo";	// Okoteve traduccin
#elif L==7	// it
	"Studenti che hanno aderito"
	" e che hanno email";
#elif L==8	// pl
	"Students who have accepted"
	" and who have email";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Estudantes que aceitaram"
	" e que t&ecirc;m email";
#endif

const char *Txt_X_students_who_have_accepted_and_who_have_email =	// Warning: it is very important to include %u and %f in the following sentences
#if   L==1	// ca
	"%u estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo (%.1f%% de %u estudiantes)";	// Necessita traduccio
#elif L==2	// de
	"%u Studenten die akzeptiert wurden"
	" und die E-Mail (%.1f%% von %u Studenten)";
#elif L==3	// en
	"%u students who have accepted"
	" and who have email (%.1f%% of %u students)";
#elif L==4	// es
	"%u estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo (%.1f%% de %u estudiantes)";
#elif L==5	// fr
	"%u &eacute;tudiants qui ont accept&eacute;"
	" et qui ont l'email (%.1f%% de %u &eacute;tudiants)";
#elif L==6	// gn
	"%u estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo (%.1f%% de %u estudiantes)";	// Okoteve traduccin
#elif L==7	// it
	"%u studenti che hanno aderito"
	" e che hanno email (%.1f%% di %u studenti)";
#elif L==8	// pl
	"%u students who have accepted"
	" and who have email (%.1f%% of %u students)";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"%u estudantes que aceitaram"
	" e que t&ecirc;m email (%.1f%% dos %u estudantes)";
#endif

const char *Txt_Surname_1 =
#if   L==1	// ca
	"Primer cognom";
#elif L==2	// de
	"Nachname 1";
#elif L==3	// en
	"Surname 1";
#elif L==4	// es
	"Primer apellido";
#elif L==5	// fr
	"Nom de famille 1";
#elif L==6	// gn
	"Terajoapy 1";
#elif L==7	// it
	"Primo cognome";
#elif L==8	// pl
	"Nazwisko 1";
#elif L==9	// pt
	"Primeiro apelido";
#endif

const char *Txt_Surname_2 =
#if   L==1	// ca
	"Segon cognom";
#elif L==2	// de
	"Nachname 2";
#elif L==3	// en
	"Surname 2";
#elif L==4	// es
	"Segundo apellido";
#elif L==5	// fr
	"Nom de famille 2";
#elif L==6	// gn
	"Terajoapy 2";
#elif L==7	// it
	"Secondo cognome";
#elif L==8	// pl
	"Nazwisko 2";
#elif L==9	// pt
	"Segundo apelido";
#endif

const char *Txt_Survey =
#if   L==1	// ca
	"Encuesta";	// Necessita traduccio
#elif L==2	// de
	"Umfrage";
#elif L==3	// en
	"Survey";
#elif L==4	// es
	"Encuesta";
#elif L==5	// fr
	"Sondage";
#elif L==6	// gn
	"Encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio";
#elif L==8	// pl
	"Survey";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Inqu&eacute;rito";
#endif

const char *Txt_SURVEY_You_belong_to_the_scope_of_the_survey =
#if   L==1	// ca
	"Pertany a l'&agrave;mbit de l'enquesta";
#elif L==2	// de
	"Sie geh&ouml;ren zum Umfang der Umfrage";
#elif L==3	// en
	"You belong to the scope of the survey";
#elif L==4	// es
	"Pertenece al &aacute;mbito de la encuesta";
#elif L==5	// fr
	"Vous appartenez &agrave; la port&eacute;e de l'enqu&ecirc;te";
#elif L==6	// gn
	"Pertenece al &aacute;mbito de la encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Voi appartenete al campo d'indagine";
#elif L==8	// pl
	"Nale&zdot;ysz do zakresu badania";
#elif L==9	// pt
	"Voc&ecirc; pertence ao &acirc;mbito do inqu&eacute;rito";
#endif

const char *Txt_SURVEY_You_dont_belong_to_the_scope_of_the_survey =
#if   L==1	// ca
	"No pertany a l'&agrave;mbit de l'enquesta";
#elif L==2	// de
	"Sie geh&ouml;ren nicht zum Umfang der Befragung";
#elif L==3	// en
	"You don't belong to the scope of the survey";
#elif L==4	// es
	"No pertenece al &aacute;mbito de la encuesta";
#elif L==5	// fr
	"Vous n'appartenez pas &agrave; la port&eacute;e de l'enqu&ecirc;te";
#elif L==6	// gn
	"No pertenece al &aacute;mbito de la encuesta";	// Okoteve traduccin
#elif L==7	// it
	"Voi non appartieni al campo d'indagine";
#elif L==8	// pl
	"Nie nale&zdot;&aogon; do zakresu badania";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pertence ao &acirc;mbito do inqu&eacute;rito";
#endif

const char *Txt_SURVEY_You_have_already_answered =
#if   L==1	// ca
	"Ya ha respondido";		// Necessita traduccio
#elif L==2	// de
	"You have already answered";	// Need bersetzung
#elif L==3	// en
	"You have already answered";
#elif L==4	// es
	"Ya ha respondido";
#elif L==5	// fr
	"You have already answered";	// Besoin de traduction
#elif L==6	// gn
	"Ya ha respondido";		// Okoteve traduccin
#elif L==7	// it
	"Hai gi&agrave; risposto";
#elif L==8	// pl
	"You have already answered";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; j&aacute; respondeu";
#endif

const char *Txt_SURVEY_You_have_not_answered =
#if   L==1	// ca
	"No ha respondido";		// Necessita traduccio
#elif L==2	// de
	"You have not answered";	// Need bersetzung
#elif L==3	// en
	"You have not answered";
#elif L==4	// es
	"No ha respondido";
#elif L==5	// fr
	"You have not answered";	// Besoin de traduction
#elif L==6	// gn
	"No ha respondido";		// Okoteve traduccin
#elif L==7	// it
	"Non hai risposto";
#elif L==8	// pl
	"You have not answered";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o respondeu";
#endif

const char *Txt_SURVEY_Type_of_user_allowed =
#if   L==1	// ca
	"Tipo de usuario permitido";	// Necessita traduccio
#elif L==2	// de
	"Type of user allowed";		// Need bersetzung
#elif L==3	// en
	"Type of user allowed";
#elif L==4	// es
	"Tipo de usuario permitido";
#elif L==5	// fr
	"Type of user allowed";		// Besoin de traduction
#elif L==6	// gn
	"Tipo de usuario permitido";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di utente autorizzato";
#elif L==8	// pl
	"Type of user allowed";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipo de usu&aacute;rio permitido";
#endif

const char *Txt_SURVEY_Type_of_user_not_allowed =
#if   L==1	// ca
	"Tipo de usuario no permitido";		// Necessita traduccio
#elif L==2	// de
	"Type of user not allowed";		// Need bersetzung
#elif L==3	// en
	"Type of user not allowed";
#elif L==4	// es
	"Tipo de usuario no permitido";
#elif L==5	// fr
	"Type of user not allowed";		// Besoin de traduction
#elif L==6	// gn
	"Tipo de usuario no permitido";		// Okoteve traduccin
#elif L==7	// it
	"Tipo di utente non autorizzato";
#elif L==8	// pl
	"Type of user not allowed";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipo de usu&aacute;rio n&atilde;o permitido";
#endif

const char *Txt_Survey_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'enquesta <strong>%s</strong> ara est&agrave; oculta.";
#elif L==2	// de
	"Die Umfrage <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==3	// en
	"Survey <strong>%s</strong> is now hidden.";
#elif L==4	// es
	"La encuesta <strong>%s</strong> ahora est&aacute; oculta.";
#elif L==5	// fr
	"Le sondage <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==6	// gn
	"La encuesta <strong>%s</strong> ahora est&aacute; oculta.";	// Okoteve traduccin
#elif L==7	// it
	"Il sondaggio <strong>%s</strong> &egrave; ora nascosto.";
#elif L==8	// pl
	"Badanie <strong>%s</strong> jest teraz ukryta.";
#elif L==9	// pt
	"O inqu&eacute;rito <strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_Survey_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'enquesta <strong>%s</strong> ara est&agrave; visible.";
#elif L==2	// de
	"Die Umfrage <strong>%s</strong> ist jetzt sichtbar.";
#elif L==3	// en
	"Survey <strong>%s</strong> is now visible.";
#elif L==4	// es
	"La encuesta <strong>%s</strong> ahora est&aacute; visible.";
#elif L==5	// fr
	"Le sondage <strong>%s</strong> est maintenant visible.";
#elif L==6	// gn
	"La encuesta <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7	// it
	"Il sondaggio <strong>%s</strong> &egrave; ora visibile.";
#elif L==8	// pl
	"Badanie <strong>%s</strong> jest teraz widoczne.";
#elif L==9	// pt
	"O inqu&eacute;rito <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_Survey_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Enquesta <strong>%s</strong> eliminada.";
#elif L==2	// de
	"Umfrage <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Survey <strong>%s</strong> removed.";
#elif L==4	// es
	"Encuesta <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Sondage <strong>%s</strong> supprim&eacute;.";
#elif L==6	// gn
	"Encuesta <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Badanie <strong>%s</strong> usuni&eogon;te.";
#elif L==9	// pt
	"Inqu&eacute;rito <strong>%s</strong> removido.";
#endif

const char *Txt_Survey_X_reset =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Enquesta <strong>%s</strong> posada a zero..";
#elif L==2	// de
	"Umfrage <strong>%s</strong> zur&uuml;ckgesetzt.";
#elif L==3	// en
	"Survey <strong>%s</strong> reset.";
#elif L==4	// es
	"Encuesta <strong>%s</strong> puesta a cero.";
#elif L==5	// fr
	"Sondage <strong>%s</strong> remis &agrave; z&eacute;ro.";
#elif L==6	// gn
	"Encuesta <strong>%s</strong> puesta a cero.";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio <strong>%s</strong> resettato.";
#elif L==8	// pl
	"Badanie <strong>%s</strong> zresetowane.";
#elif L==9	// pt
	"Inqu&eacute;rito <strong>%s</strong> fixado em zero.";
#endif

const char *Txt_SURVEY_STR_ANSWER_TYPES[Svy_NUM_ANS_TYPES] =
   {
#if   L==1	// ca
	 "Elecci&oacute; &uacute;nica"
#elif L==2	// de
	 "Unique-Choice"
#elif L==3	// en
	 "Unique choice"
#elif L==4	// es
	 "Elecci&oacute;n &uacute;nica"
#elif L==5	// fr
	 "Choix unique"
#elif L==6	// gn
	 "Elecci&oacute;n &uacute;nica"		// Okoteve traduccin
#elif L==7	// it
	 "Scelta unica"
#elif L==8	// pl
	 "Unique choice"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "&Uacute;nica escolha"
#endif
	 ,
#if   L==1	// ca
	 "Elecci&oacute; m&uacute;ltiple"
#elif L==2	// de
	 "Multiple-Choice"
#elif L==3	// en
	 "Multiple choice"
#elif L==4	// es
	 "Elecci&oacute;n m&uacute;ltiple"
#elif L==5	// fr
	 "Choix multiple"
#elif L==6	// gn
	 "Elecci&oacute;n m&uacute;ltiple"	// Okoteve traduccin
#elif L==7	// it
	 "Scelta multipla"
#elif L==8	// pl
	 "Multiple choice"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "M&uacute;ltipla escolha"
#endif
	};

const char *Txt_Surveys =
#if   L==1	// ca
	"Enquestes";
#elif L==2	// de
	"Umfragen";
#elif L==3	// en
	"Surveys";
#elif L==4	// es
	"Encuestas";
#elif L==5	// fr
	"Sondages";
#elif L==6	// gn
	"Encuestas";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggi";
#elif L==8	// pl
	"Ankiety";
#elif L==9	// pt
	"Inqu&eacute;ritos";
#endif

const char *Txt_Syllabus =
#if   L==1	// ca
	"Programa";
#elif L==2	// de
	"Studienplan";
#elif L==3	// en
	"Syllabus";
#elif L==4	// es
	"Programa";
#elif L==5	// fr
	"Plan";
#elif L==6	// gn
	"Programa";	// Okoteve traduccin
#elif L==7	// it
	"Programma";
#elif L==8	// pl
	"Syllabus";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Programa";
#endif

const char *Txt_Syllabus_of_the_course =
#if   L==1	// ca
	"Programa de la asignatura";	// Necessita traduccio
#elif L==2	// de
	"Studienplan der Kurs";
#elif L==3	// en
	"Syllabus of the course";
#elif L==4	// es
	"Programa de la asignatura";
#elif L==5	// fr
	"Plan de la mati&egrave;re";
#elif L==6	// gn
	"Programa de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Programma del corso";
#elif L==8	// pl
	"Syllabus of the course";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Programa da disciplina";
#endif

const char *Txt_SYLLABUS_WHICH_SYLLABUS[Syl_NUM_WHICH_SYLLABUS] =
	 {
	 // Lectures
#if   L==1	// ca
	 "Teoria"
#elif L==2	// de
	 "Vorlesung"
#elif L==3	// en
	 "Lectures"
#elif L==4	// es
	 "Teor&iacute;a"
#elif L==5	// fr
	 "Th&eacute;orie"
#elif L==6	// gn
	 "Teor&iacute;a"		// Okoteve traduccin
#elif L==7	// it
	 "Teoria"
#elif L==8	// pl
	 "Wyklady"
#elif L==9	// pt
	 "Teoria"
#endif
	 ,
	 // Practicals
#if   L==1	// ca
	 "Pr&agrave;ctiques"
#elif L==2	// de
	 "&Uuml;bung"
#elif L==3	// en
	 "Practicals"
#elif L==4	// es
	 "Pr&aacute;cticas"
#elif L==5	// fr
	 "Pratique"
#elif L==6	// gn
	 "Pr&aacute;cticas"		// Okoteve traduccin
#elif L==7	// it
	 "Pratica"
#elif L==8	// pl
	 "Praktyki"
#elif L==9	// pt
	 "Pr&aacute;tica"
#endif
	 ,
	 };

const char *Txt_System =
#if   L==1	// ca
	"Plataforma";
#elif L==2	// de
	"Plattform";
#elif L==3	// en
	"System";
#elif L==4	// es
	"Plataforma";
#elif L==5	// fr
	"Plate-forme";
#elif L==6	// gn
	"Ypykatu";
#elif L==7	// it
	"Piattaforma";
#elif L==8	// pl
	"System";
#elif L==9	// pt
	"Sistema";
#endif

const char *Txt_TABLE_Footer =
#if   L==1	// ca
	"Pie";		// Necessita traduccio
#elif L==2	// de
	"Fu&szlig;";
#elif L==3	// en
	"Footer";
#elif L==4	// es
	"Pie";
#elif L==5	// fr
	"Pied";
#elif L==6	// gn
	"Pie";		// Okoteve traduccin
#elif L==7	// it
	"Piede";
#elif L==8	// pl
	"Footer";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"P&eacute;";
#endif

const char *Txt_TABLE_Header =
#if   L==1	// ca
	"Cabecera";	// Necessita traduccio
#elif L==2	// de
	"Header";
#elif L==3	// en
	"Header";
#elif L==4	// es
	"Cabecera";
#elif L==5	// fr
	"T&ecirc;te";
#elif L==6	// gn
	"Cabecera";	// Okoteve traduccin
#elif L==7	// it
	"Intestazione";
#elif L==8	// pl
	"Header";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Cabe&ccedil;alho";
#endif

const char *Txt_Table_not_found_in_the_file_of_marks =
#if   L==1	// ca
	"No hay ninguna tabla en el archivo de calificaciones.";	// Necessita traduccio
#elif L==2	// de
	"Table not found in the file of marks.";			// Need bersetzung
#elif L==3	// en
	"Table not found in the file of marks.";
#elif L==4	// es
	"No hay ninguna tabla en el archivo de calificaciones.";
#elif L==5	// fr
	"Table not found in the file of marks.";			// Besoin de traduction
#elif L==6	// gn
	"No hay ninguna tabla en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==7	// it
	"Tabella non trovata nel file dei risultati.";
#elif L==8	// pl
	"Table not found in the file of marks.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; tabela no arquivo de classifica&ccedil;&otilde;es.";
#endif

const char *Txt_TABS_TXT[Tab_NUM_TABS] =
   {
	// TabUnk
	NULL
	,
	// TabSta
#if   L==1	// ca
	"Inici"
#elif L==2	// de
	"Start"
#elif L==3	// en
	"Start"
#elif L==4	// es
	"Inicio"
#elif L==5	// fr
	"D&eacute;marrer"
#elif L==6	// gn
	"&Ntilde;epyr&utilde;"
#elif L==7	// it
	"Inizio"
#elif L==8	// pl
	"Start"
#elif L==9	// pt
	"Inicio"
#endif
	,
	// TabSys
#if   L==1	// ca
	"Plataforma"
#elif L==2	// de
	"Plattform"
#elif L==3	// en
	"System"
#elif L==4	// es
	"Plataforma"
#elif L==5	// fr
	"Plate-forme"
#elif L==6	// gn
	"Ypykatu"
#elif L==7	// it
	"Piattaforma"
#elif L==8	// pl
	"System"
#elif L==9	// pt
	"Sistema"
#endif
	,
	// TabCty
#if   L==1	// ca
	"Pa&iacute;s"
#elif L==2	// de
	"Land"
#elif L==3	// en
	"Country"
#elif L==4	// es
	"Pa&iacute;s"
#elif L==5	// fr
	"Pays"
#elif L==6	// gn
	"Tet&atilde;"
#elif L==7	// it
	"Paese"
#elif L==8	// pl
	"Kraj"
#elif L==9	// pt
	"Pa&iacute;s"
#endif
	,
	// TabIns
#if   L==1	// ca
	"Instituci&oacute;"
#elif L==2	// de
	"Hochschule"
#elif L==3	// en
	"Institution"
#elif L==4	// es
	"Instituci&oacute;n"
#elif L==5	// fr
	"&Eacute;tablissement"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Istituzione"
#elif L==8	// pl
	"Instytucji"
#elif L==9	// pt
	"Institu&ccedil;&atilde;o"
#endif
	,
	// TabCtr
#if   L==1	// ca
	"Centre"
#elif L==2	// de
	"Lehrinstitut"
#elif L==3	// en
	"Centre"
#elif L==4	// es
	"Centro"
#elif L==5	// fr
	"Centre"
#elif L==6	// gn
	"Mbo'ehao"
#elif L==7	// it
	"Centro"
#elif L==8	// pl
	"Centrum"
#elif L==9	// pt
	"Centro"
#endif
	,
	// TabDeg
#if   L==1	// ca
	"Titulaci&oacute;"
#elif L==2	// de
	"Studiengang"
#elif L==3	// en
	"Degree"
#elif L==4	// es
	"Titulaci&oacute;n"
#elif L==5	// fr
	"&Eacute;tude"
#elif L==6	// gn
	"Arandur&atilde;"
#elif L==7	// it
	"Laurea"
#elif L==8	// pl
	"Stopien"
#elif L==9	// pt
	"Grau"
#endif
	,
	// TabCrs
#if   L==1	// ca
	"Assignatura"
#elif L==2	// de
	"Kurs"
#elif L==3	// en
	"Course"
#elif L==4	// es
	"Asignatura"
#elif L==5	// fr
	"Mati&egrave;re"
#elif L==6	// gn
	"Mbo'esyry"
#elif L==7	// it
	"Corso"
#elif L==8	// pl
	"Kurs"
#elif L==9	// pt
	"Disciplina"
#endif
	,
	// TabAss
#if   L==1	// ca
	"Avaluaci&oacute;"
#elif L==2	// de
	"Bewertung"
#elif L==3	// en
	"Assessment"
#elif L==4	// es
	"Evaluaci&oacute;n"
#elif L==5	// fr
	"&Eacute;valuation"
#elif L==6	// gn
	"Mbohepy"
#elif L==7	// it
	"Valutazione"
#elif L==8	// pl
	"Ocena"
#elif L==9	// pt
	"Avalia&ccedil;&atilde;o"
#endif
	,
	// TabFil
#if   L==1	// ca
	"Fitxers"
#elif L==2	// de
	"Dateien"
#elif L==3	// en
	"Files"
#elif L==4	// es
	"Archivos"
#elif L==5	// fr
	"Fichiers"
#elif L==6	// gn
	"Archivos"	// Okoteve traduccin
#elif L==7	// it
	"File"
#elif L==8	// pl
	"Pliki"
#elif L==9	// pt
	"Arquivos"
#endif
	,
	// TabUsr
#if   L==1	// ca
	"Usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"Users"
#elif L==4	// es
	"Usuarios"
#elif L==5	// fr
	"Utilisateurs"
#elif L==6	// gn
	"Puruh&aacute;ra"
#elif L==7	// it
	"Utenti"
#elif L==8	// pl
	"U&zdot;ytkownicy"
#elif L==9	// pt
	"Utilizadores"
#endif
	,
	// TabMsg
#if   L==1	// ca
	"Missatges"
#elif L==2	// de
	"Nachrichten"
#elif L==3	// en
	"Messages"
#elif L==4	// es
	"Mensajes"
#elif L==5	// fr
	"Messages"
#elif L==6	// gn
	"Marandu"
#elif L==7	// it
	"Messaggi"
#elif L==8	// pl
	"Wiadomo&sacute;ci"
#elif L==9	// pt
	"Mensagens"
#endif
	,
#if   L==1	// ca
	"An&agrave;lisi"
#elif L==2	// de
	"Analytik"
#elif L==3	// en
	"Analytics"
#elif L==4	// es
	"An&aacute;lisis"
#elif L==5	// fr
	"Analytiques"
#elif L==6	// gn
	"&Ntilde;ehesa'&yuml;ijo"
#elif L==7	// it
	"Analitiche"
#elif L==8	// pl
	"Analizy"
#elif L==9	// pt
	"An&aacute;lises"
#endif
	,
	// TabPrf
#if   L==1	// ca
	"Perfil"
#elif L==2	// de
	"Profil"
#elif L==3	// en
	"Profile"
#elif L==4	// es
	"Perfil"
#elif L==5	// fr
	"Profil"
#elif L==6	// gn
	"Che"
#elif L==7	// it
	"Profilo"
#elif L==8	// pl
	"Profil"
#elif L==9	// pt
	"Perfil"
#endif
	};

const char *Txt_Tag_allowed =
#if   L==1	// ca
	"Descriptor permitido";	// Necessita traduccio
#elif L==2	// de
	"Tag erlaubt";
#elif L==3	// en
	"Tag allowed";
#elif L==4	// es
	"Descriptor permitido";
#elif L==5	// fr
	"Descripteur admis";
#elif L==6	// gn
	"Descriptor permitido";	// Okoteve traduccin
#elif L==7	// it
	"Etichetta ammessa";
#elif L==8	// pl
	"Tag allowed";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Descritor permitido";
#endif

const char *Txt_Tag_X_allowed_Click_to_disable_it =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Descriptor %s permitido. Pulse para prohibirlo.";	// Necessita traduccio
#elif L==2	// de
	"Tag %s allowed. Click to disable it.";			// Need bersetzung
#elif L==3	// en
	"Tag %s allowed. Click to disable it.";
#elif L==4	// es
	"Descriptor %s permitido. Pulse para prohibirlo.";
#elif L==5	// fr
	"Tag %s allowed. Click to disable it.";			// Besoin de traduction
#elif L==6	// gn
	"Descriptor %s permitido. Pulse para prohibirlo.";	// Okoteve traduccin
#elif L==7	// it
	"Etichetta %s autorizzata. Clicca per censurarla.";
#elif L==8	// pl
	"Tag %s allowed. Click to disable it.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Descritor %s permitido. Clique para proibir.";
#endif

const char *Txt_Tag_not_allowed =
#if   L==1	// ca
	"Descriptor no permitido";	// Necessita traduccio
#elif L==2	// de
	"Tag nicht erlaubt";
#elif L==3	// en
	"Tag not allowed";
#elif L==4	// es
	"Descriptor no permitido";
#elif L==5	// fr
	"Descripteur non admis";
#elif L==6	// gn
	"Descriptor no permitido";	// Okoteve traduccin
#elif L==7	// it
	"Etichetta non autorizzata";
#elif L==8	// pl
	"Tag not allowed";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Descritor n&atilde;o permitido";
#endif

const char *Txt_Tag_X_not_allowed_Click_to_allow_it =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Descriptor %s no permitido. Pulse para permitirlo.";	// Necessita traduccio
#elif L==2	// de
	"Tag %s not allowed. Click to allow it.";		// Need bersetzung
#elif L==3	// en
	"Tag %s not allowed. Click to allow it.";
#elif L==4	// es
	"Descriptor %s no permitido. Pulse para permitirlo.";
#elif L==5	// fr
	"Tag %s not allowed. Click to allow it.";		// Besoin de traduction
#elif L==6	// gn
	"Descriptor %s no permitido. Pulse para permitirlo.";	// Okoteve traduccin
#elif L==7	// it
	"Etichetta %s censurata. Clicca per autorizzarla.";
#elif L==8	// pl
	"Tag %s not allowed. Click to allow it.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Descritor %s n&atilde;o permitido. Clique para permitir.";
#endif

#if defined LOCALHOST_UBUNTU		// localhost (Ubuntu) *****************

const char *Txt_TAGLINE =
# if   L==1
	"plataforma educativa";
# elif L==2
	"Bildungsplattform";
# elif L==3
	"social learning platform";
# elif L==4
	"plataforma educativa";
# elif L==5
	"plate-forme &eacute;ducative";
# elif L==6
	"plataforma educativa";	// Okoteve traduccin
# elif L==7
	"piattaforma educativa";
# elif L==8
	"platforma edukacyjna";
# elif L==9
	"plataforma educacional";
# endif

const char *Txt_TAGLINE_BR =
# if   L==1
	"plataforma<br />educativa";
# elif L==2
	"Bildungs-<br />plattform";
# elif L==3
	"social learning<br />platform";
# elif L==4
	"plataforma<br />educativa";
# elif L==5
	"plate-forme<br />&eacute;ducative";
# elif L==6
	"plataforma<br />educativa";	// Okoteve traduccin
# elif L==7
	"piattaforma<br />educativa";
# elif L==8
	"platforma<br />edukacyjna";
# elif L==9
	"plataforma<br />educacional";
# endif

#elif defined OPENSWAD_ORG		// openswad.org ***********************

const char *Txt_TAGLINE =
# if   L==1
	"plataforma educativa";
# elif L==2
	"Bildungsplattform";
# elif L==3
	"social learning platform";
# elif L==4
	"plataforma educativa";
# elif L==5
	"plate-forme &eacute;ducative";
# elif L==6
	"plataforma educativa";	// Okoteve traduccin
# elif L==7
	"piattaforma educativa";
# elif L==8
	"platforma edukacyjna";
# elif L==9
	"plataforma educacional";
# endif

const char *Txt_TAGLINE_BR =
# if   L==1
	"plataforma<br />educativa";
# elif L==2
	"Bildungs-<br />plattform";
# elif L==3
	"social learning<br />platform";
# elif L==4
	"plataforma<br />educativa";
# elif L==5
	"plate-forme<br />&eacute;ducative";
# elif L==6
	"plataforma<br />educativa";	// Okoteve traduccin
# elif L==7
	"piattaforma<br />educativa";
# elif L==8
	"platforma<br />edukacyjna";
# elif L==9
	"plataforma<br />educacional";
# endif

#elif defined SWAD_UGR_ES		// swad.ugr.es ************************

const char *Txt_TAGLINE =
# if   L==1
	"plataforma de suport a la doc&egrave;ncia / UGR /";
# elif L==2
	"Plattform zur Unterst&uuml;tzung der Lehre / UGR /";
# elif L==3
	"platform to support teaching / UGR /";
# elif L==4
	"plataforma de apoyo a la docencia / UGR /";
# elif L==5
	"plate-forme pour appuyer l'enseignement / UGR /";
# elif L==6
	"plataforma de apoyo a la docencia / UGR /";	// Okoteve traduccin
# elif L==7
	"piattaforma per sostenere l'insegnamento / UGR /";
# elif L==8
	"platforma do wspierania nauczania / UGR /";
# elif L==9
	"plataforma de apoio ao ensino / UGR /";
# endif

const char *Txt_TAGLINE_BR =
# if   L==1
	"/ UGR / plataforma de<br />suport a la doc&egrave;ncia";
# elif L==2
	"/ UGR / Plattform zur<br />Unterst&uuml;tzung der Lehre";
# elif L==3
	"/ UGR / platform to<br />support teaching";
# elif L==4
	"/ UGR / plataforma de<br />apoyo a la docencia";
# elif L==5
	"/ UGR / plate-forme pour<br />appuyer l'enseignement";
# elif L==6
	"/ UGR / plataforma de<br />apoyo a la docencia";	// Okoteve traduccin
# elif L==7
	"/ UGR / piattaforma per<br />sostenere l'insegnamento";
# elif L==8
	"/ UGR / platforma do<br />wspierania nauczania";
# elif L==9
	"/ UGR / plataforma<br />de apoio ao ensino";
# endif

#elif defined SWADBERRY_UGR_ES		// swadberry.ugr.es *******************

const char *Txt_TAGLINE =
# if   L==1
	"plataforma de suport a la doc&egrave;ncia (Raspberry Pi)";
# elif L==2
	"Plattform zur Unterst&uuml;tzung der Lehre (Raspberry Pi)";
# elif L==3
	"platform to support teaching (Raspberry Pi)";
# elif L==4
	"plataforma de apoyo a la docencia (Raspberry Pi)";
# elif L==5
	"plate-forme pour appuyer l'enseignement (Raspberry Pi)";
# elif L==6
	"plataforma de apoyo a la docencia (Raspberry Pi)";	// Okoteve traduccin
# elif L==7
	"piattaforma per sostenere l'insegnamento (Raspberry Pi)";
# elif L==8
	"platforma do wspierania nauczania (Raspberry Pi)";
# elif L==9
	"plataforma de apoio ao ensino (Raspberry Pi)";
# endif

const char *Txt_TAGLINE_BR =
# if   L==1
	"plataforma de suport a la<br />doc&egrave;ncia (Raspberry Pi)";
# elif L==2
	"Plattform zur Unterst&uuml;tzung<br />der Lehre (Raspberry Pi)";
# elif L==3
	"platform to support<br />teaching (Raspberry Pi)";
# elif L==4
	"plataforma de apoyo a la<br />docencia (Raspberry Pi)";
# elif L==5
	"plate-forme pour appuyer<br />l'enseignement (Raspberry Pi)";
# elif L==6
	"plataforma de apoyo a la<br />docencia (Raspberry Pi)";	// Okoteve traduccin
# elif L==7
	"piattaforma per sostenere<br />l'insegnamento (Raspberry Pi)";
# elif L==8
	"platforma do wspierania<br />nauczania (Raspberry Pi)";
# elif L==9
	"plataforma de apoio<br />ao ensino (Raspberry Pi)";
# endif

#endif

const char *Txt_Tags =		// Tags used in test
#if   L==1	// ca
	"Descriptors";
#elif L==2	// de
	"Tags";
#elif L==3	// en
	"Tags";
#elif L==4	// es
	"Descriptores";
#elif L==5	// fr
	"Descripteurs";
#elif L==6	// gn
	"Descriptores";	// Okoteve traduccin
#elif L==7	// it
	"Etichette";
#elif L==8	// pl
	"Tags";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Descritores";
#endif

const char *Txt_Take_a_test =
#if   L==1	// ca
	"Fer un test";
#elif L==2	// de
	"F&uuml;hren Sie einen Test";
#elif L==3	// en
	"Take a test";
#elif L==4	// es
	"Hacer un test";
#elif L==5	// fr
	"Passer un test";
#elif L==6	// gn
	"Hacer un test";	// Okoteve traduccin
#elif L==7	// it
	"Fare un test";
#elif L==8	// pl
	"Zr&oacute;b test";
#elif L==9	// pt
	"Fazer um teste";
#endif

const char *Txt_teachers_ABBREVIATION =	// Abbreviation of "Teachers"
#if   L==1	// ca
	"prof.";
#elif L==2	// de
	"Lehr.";
#elif L==3	// en
	"teac.";
#elif L==4	// es
	"prof.";
#elif L==5	// fr
	"ense.";
#elif L==6	// gn
	"prof.";	// Okoteve traduccin
#elif L==7	// it
	"prof.";
#elif L==8	// pl
	"nauc.";
#elif L==9	// pt
	"prof.";
#endif

const char *Txt_Teachers_files_area =
#if   L==1	// ca
	"Zona d'arxius de professors";
#elif L==2	// de
	"Lehrer Dateien-Bereich";
#elif L==3	// en
	"Teachers' files area";
#elif L==4	// es
	"Zona de archivos de profesores";
#elif L==5	// fr
	"Zone de fichiers d'enseignants";
#elif L==6	// gn
	"Zona de archivos de profesores";	// Okoteve traduccin
#elif L==7	// it
	"Zona di file di professori";
#elif L==8	// pl
	"Nauczyciela udost&eogon;pnionych plik&oacute;w";
#elif L==9	// pt
	"Zona de arquivos dos professores";
#endif

const char *Txt_teachers_files_area =
#if   L==1	// ca
	"zona d'arxius de professors";
#elif L==2	// de
	"Lehrer Dateien-Bereich";
#elif L==3	// en
	"teachers' files area";
#elif L==4	// es
	"zona de archivos de profesores";
#elif L==5	// fr
	"zone de fichiers d'enseignants";
#elif L==6	// gn
	"zona de archivos de profesores";	// Okoteve traduccin
#elif L==7	// it
	"zona di file di professori";
#elif L==8	// pl
	"nauczyciela udost&eogon;pnionych plik&oacute;w";
#elif L==9	// pt
	"zona de arquivos dos professores";
#endif

const char *Txt_Teaching_platform =
#if   L==1	// ca
	"Plataforma docent";
#elif L==2	// de
	"Lernplattform ";
#elif L==3	// en
	"Teaching platform";
#elif L==4	// es
	"Plataforma docente";
#elif L==5	// fr
	"Plate-forme d'enseignement";
#elif L==6	// gn
	"Plataforma docente";	// Okoteve traduccin
#elif L==7	// it
	"Piattaforma didattica";
#elif L==8	// pl
	"Platforma nauczania";
#elif L==9	// pt
	"Plataforma de ensino";
#endif

const char *Txt_Temporary_private_storage_area =
#if   L==1	// ca
	"Zona d'emmagatzematge privada";
#elif L==2	// de
	"Privater Speicherbereich";
#elif L==3	// en
	"Temporary private storage area";
#elif L==4	// es
	"Zona de almacenamiento privada temporal";
#elif L==5	// fr
	"Zone de stockage priv&eacute;e";
#elif L==6	// gn
	"Zona de almacenamiento privada temporal";	// Okoteve traduccin
#elif L==7	// it
	"Area di archiviazione privata";
#elif L==8	// pl
	"Prywatny magazyn";
#elif L==9	// pt
	"Zona privada de armazenamento";
#endif

const char *Txt_temporary_private_storage_area =
#if   L==1	// ca
	"zona d'emmagatzematge privada";
#elif L==2	// de
	"privater Speicherbereich";
#elif L==3	// en
	"temporary private storage area";
#elif L==4	// es
	"zona de almacenamiento privada temporal";
#elif L==5	// fr
	"zone de stockage priv&eacute;e";
#elif L==6	// gn
	"zona de almacenamiento privada temporal";	// Okoteve traduccin
#elif L==7	// it
	"area di archiviazione privata";
#elif L==8	// pl
	"obszar prywatnego przechowywania";
#elif L==9	// pt
	"zona privada de armazenamento";
#endif

const char *Txt_Test =
#if   L==1	// ca
	"Test";
#elif L==2	// de
	"Test";
#elif L==3	// en
	"Test";
#elif L==4	// es
	"Test";
#elif L==5	// fr
	"Test";
#elif L==6	// gn
	"Test";	// Okoteve traduccin
#elif L==7	// it
	"Test";
#elif L==8	// pl
	"Test";
#elif L==9	// pt
	"Teste";
#endif

const char *Txt_Test_No_X_that_you_make_in_this_course =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Test n&ordm; %u que realiza usted en esta asignatura";	// Necessita traduccio
#elif L==2	// de
	"Der %u. Test, den Sie in dieser Kurs machen";
#elif L==3	// en
	"Test No. %u that you make in this course";
#elif L==4	// es
	"Test n&ordm; %u que realiza usted en esta asignatura";
#elif L==5	// fr
	"Test n&ordm; %u que vous faites dans cette mati&egrave;re";
#elif L==6	// gn
	"Test n&ordm; %u que realiza usted en esta asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Test n. %u che fai in questo corso";
#elif L==8	// pl
	"Test No. %u that you make in this course";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Teste n&ordm; %u que voc&ecirc; faz nesta disciplina";
#endif

const char *Txt_Test_result =
#if   L==1	// ca
	"Resultat del test";
#elif L==2	// de
	"Testergebnis";
#elif L==3	// en
	"Test result";
#elif L==4	// es
	"Resultado del test";
#elif L==5	// fr
	"R&eacute;sultat du test";
#elif L==6	// gn
	"Resultado del test";	// Okoteve traduccin
#elif L==7	// it
	"Risultato del test";
#elif L==8	// pl
	"Wynik testu";
#elif L==9	// pt
	"Resultado do teste";
#endif

const char *Txt_Test_results =
#if   L==1	// ca
	"Resultats de tests";
#elif L==2	// de
	"Testergebnisse";
#elif L==3	// en
	"Test results";
#elif L==4	// es
	"Resultados de los test";
#elif L==5	// fr
	"R&eacute;sultats des tests";
#elif L==6	// gn
	"Resultados de los test";	// Okoteve traduccin
#elif L==7	// it
	"Risultati dei test";
#elif L==8	// pl
	"Wyniki test&oacute;w";
#elif L==9	// pt
	"Resultados dos testes";
#endif

const char *Txt_Text =
#if   L==1	// ca
	"Text";
#elif L==2	// de
	"Text";
#elif L==3	// en
	"Text";
#elif L==4	// es
	"Texto";
#elif L==5	// fr
	"Texte";
#elif L==6	// gn
	"Texto";	// Okoteve traduccin
#elif L==7	// it
	"Testo";
#elif L==8	// pl
	"Tekst";
#elif L==9	// pt
	"Texto";
#endif

const char *Txt_TEXT_plain =
#if   L==1	// ca
	 "pla";
#elif L==2	// de
	 "Plain";
#elif L==3	// en
	 "plain";
#elif L==4	// es
	 "plano";
#elif L==5	// fr
	 "plain";
#elif L==6	// gn
	 "plano";	// Okoteve traduccin
#elif L==7	// it
	 "normale";
#elif L==8	// pl
	 "zwykly";
#elif L==9	// pt
	 "plano";
#endif

const char *Txt_Thank_you_for_reporting_a_possible_duplicate_user =
#if   L==1	// ca
	"Gr&agrave;cies per informar d'un usuari possiblement duplicat.";
#elif L==2	// de
	"Vielen Dank fr eine m&ouml;gliche doppelte Benutzer berichten.";
#elif L==3	// en
	"Thank you for reporting a possible duplicate user.";
#elif L==4	// es
	"Gracias por informar de un usuario posiblemente duplicado.";
#elif L==5	// fr
	"Merci d'avoir signal&eacute; un &eacute;ventuel utilisateur en double.";
#elif L==6	// gn
	"Gracias por informar de un usuario posiblemente duplicado.";	// Okoteve traduccin
#elif L==7	// it
	"Grazie per aver segnalato un possibile utente duplicato.";
#elif L==8	// pl
	"Dzi&eogon;kujemy za zg&lstrok;oszenie ewentualnego duplikat u&zdot;ytkownika.";
#elif L==9	// pt
	"Obrigado por relatar um possvel usu&aacute;rio duplicado.";
#endif

const char *Txt_Thanks_for_playing_the_game =
#if   L==1	// ca
	"Gr&agrave;cies per jugar al joc.";
#elif L==2	// de
	"Vielen Dank f&uuml;r das Spiel.";
#elif L==3	// en
	"Thanks for playing the game.";
#elif L==4	// es
	"Gracias por jugar al juego.";
#elif L==5	// fr
	"Merci d'avoir jou&eacute; le jeu.";
#elif L==6	// gn
	"Gracias por jugar al juego.";	// Okoteve traduccin
#elif L==7	// it
	"Grazie per il gioco.";
#elif L==8	// pl
	"TDzi&eogon;ki za gr&eogon;.";
#elif L==9	// pt
	"Obrigado por jogar o jogo.";
#endif

const char *Txt_Thanks_for_answering_the_survey =
#if   L==1	// ca
	"Gracias por responder la encuesta.";	// Necessita traduccio
#elif L==2	// de
	"Vielen Dank f&uuml;r die Beantwortung der Umfrage.";
#elif L==3	// en
	"Thanks for answering the survey.";
#elif L==4	// es
	"Gracias por responder la encuesta.";
#elif L==5	// fr
	"Merci de r&eacute;pondre au sondage.";
#elif L==6	// gn
	"Gracias por responder la encuesta.";	// Okoteve traduccin
#elif L==7	// it
	"Grazie per la risposta al sondaggio.";
#elif L==8	// pl
	"Thanks for answering the survey.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Obrigado por responder o inqu&eacute;rito.";
#endif

const char *Txt_The_alphabetical_code_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";	// Necessita traduccio
#elif L==2	// de
	"The ISO 3166-1 alphabetical code %s already exists.";		// Need bersetzung
#elif L==3	// en
	"The ISO 3166-1 alphabetical code %s already exists.";
#elif L==4	// es
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";
#elif L==5	// fr
	"The ISO 3166-1 alphabetical code %s already exists.";		// Besoin de traduction
#elif L==6	// gn
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il codice alfabetico ISO 3166-1 %s gi&agrave; esiste.";
#elif L==8	// pl
	"The ISO 3166-1 alphabetical code %s already exists.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O c&oacute;digo alfab&eacute;tico ISO 3166-1 %s j&aacute; existe.";
#endif

const char *Txt_The_alphabetical_code_X_is_not_correct =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";	// Necessita traduccio
#elif L==2	// de
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Need bersetzung
#elif L==3	// en
	"The ISO 3166-1 alphabetical code %s is not correct.";
#elif L==4	// es
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";
#elif L==5	// fr
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Besoin de traduction
#elif L==6	// gn
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";	// Okoteve traduccin
#elif L==7	// it
	"Il codice alfabetico ISO 3166-1 %s non &egrave; corretto.";
#elif L==8	// pl
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O c&oacute;digo alfab&eacute;tico ISO 3166-1 %s n&atilde;o est&aacute; correto.";
#endif

const char *Txt_The_announcement_of_exam_has_been_successfully_updated =
#if   L==1	// ca
	"La convocat&ograve;ria d'examen s'ha actualitzat correctament.";
#elif L==2	// de
	"Der Aufruf zur Pr&uuml;fung wurde erfolgreich aktualisiert.";
#elif L==3	// en
	"The announcement of exam has been successfully updated.";
#elif L==4	// es
	"La convocatoria de examen se ha actualizado correctamente.";
#elif L==5	// fr
	"La convocation pour l'examen a &eacute;t&eacute; mis &agrave; jour avec succ&egrave;s.";
#elif L==6	// gn
	"La convocatoria de examen se ha actualizado correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"L'apello d'esame &egrave; stata aggiornata con successo.";
#elif L==8	// pl
	"Ogloszenie egzamin zosta&lstrok; pomy&sacute;lnie zaktualizowany.";
#elif L==9	// pt
	"A chamada para exame foi atualizada com sucesso.";
#endif

const char *Txt_The_announcement_of_exam_is_now_hidden =
#if   L==1	// ca
	"La convocat&ograve;ria d'examen ara est&agrave; oculta.";
#elif L==2	// de
	"Der Aufruf zur Pr&uuml;fung ist jetzt ausgeblendet.";
#elif L==3	// en
	"The announcement of exam is now hidden.";
#elif L==4	// es
	"La convocatoria de examen ahora est&aacute; oculta.";
#elif L==5	// fr
	"La convocation pour l'examen est maintenant cach&eacute;.";
#elif L==6	// gn
	"La convocatoria de examen ahora est&aacute; oculta.";	// Okoteve traduccin
#elif L==7	// it
	"L'apello d'esame &egrave; ora nascosto.";
#elif L==8	// pl
	"Ogloszenie egzamin jest teraz ukryte.";
#elif L==9	// pt
	"A chamada para exame &eacute; agora oculta.";
#endif

const char *Txt_The_announcement_of_exam_is_now_visible =
#if   L==1	// ca
	"La convocat&ograve;ria d'examen ara est&agrave; visible.";
#elif L==2	// de
	"Der Aufruf zur Pr&uuml;fung ist jetzt sichtbar.";
#elif L==3	// en
	"The announcement of exam is now visible.";
#elif L==4	// es
	"La convocatoria de examen ahora est&aacute; visible.";
#elif L==5	// fr
	"La convocation pour l'examen est maintenant visible.";
#elif L==6	// gn
	"La convocatoria de examen ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7	// it
	"L'apello d'esame &egrave; ora visibile.";
#elif L==8	// pl
	"Ogloszenie egzamin jest teraz widoczny.";
#elif L==9	// pt
	"A chamada para exame &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_The_answers_of_the_question_with_code_X_will_appear_shuffled =	// Warning: it is very important to include %ld in the following sentences
#if   L==1	// ca
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";	// Necessita traduccio
#elif L==2	// de
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Need bersetzung
#elif L==3	// en
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";
#elif L==4	// es
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";
#elif L==5	// fr
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Besoin de traduction
#elif L==6	// gn
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";	// Okoteve traduccin
#elif L==7	// it
	"Le risposte alla domande con il codice <strong>%ld</strong>"
	" appariranno mischiate.";
#elif L==8	// pl
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"As respostas da pergunta com o c&oacute;digo <strong>%ld</strong>"
	" aparecer&atilde;o embaralhadas.";
#endif

const char *Txt_The_answers_of_the_question_with_code_X_will_appear_without_shuffling =	// Warning: it is very important to include %ld in the following sentences
#if   L==1	// ca
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";	// Necessita traduccio
#elif L==2	// de
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Need bersetzung
#elif L==3	// en
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";
#elif L==4	// es
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";
#elif L==5	// fr
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Besoin de traduction
#elif L==6	// gn
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";	// Okoteve traduccin
#elif L==7	// it
	"Le risposte della domanda con il codice <strong>%ld</strong>"
	" appariranno senza essere mischiate.";
#elif L==8	// pl
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"As respostas da pergunta com o c&oacute;digo <strong>%ld</strong>"
	" aparecer&atilde;o sem embaralhar.";
#endif

const char *Txt_The_assignment_has_been_modified =
#if   L==1	// ca
	"L'activitat ha estat modificada.";
#elif L==2	// de
	"Die Aufgabe wurde ge&auml;ndert.";
#elif L==3	// en
	"The assignment has been modified.";
#elif L==4	// es
	"La actividad ha sido modificada.";
#elif L==5	// fr
	"L'activit&eacute; a &eacute;t&eacute; modifi&eacute;e.";
#elif L==6	// gn
	"La actividad ha sido modificada.";	// Okoteve traduccin
#elif L==7	// it
	"L'attivit&agrave; &egrave; stata modificata.";
#elif L==8	// pl
	"Dzia&lstrok;anie zosta&lstrok;o zmodyfikowane.";
#elif L==9	// pt
	"A atividade foi modificada.";
#endif

const char *Txt_The_banner_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El banner <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==2	// de
	"Der Werbebanner <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The banner <strong>%s</strong> already exists.";
#elif L==4	// es
	"El banner <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The banner <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El banner <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"Il banner <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The banner <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O banner <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_banner_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The Werbebanner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il banner <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O banner <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_banner_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El banner <strong>%s</strong> ara est&agrave; ocult.";
#elif L==2	// de
	"Der Werbebanner <strong>%s</strong> ist jetzt ausgeblendet.";	// Need bersetzung
#elif L==3	// en
	"The banner <strong>%s</strong> is now hidden.";
#elif L==4	// es
	"El banner <strong>%s</strong> ahora est&aacute; oculto.";
#elif L==5	// fr
	"Le banner <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==6	// gn
	"El banner <strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==7	// it
	"Il banner <strong>%s</strong> &egrave; ora nascosto.";
#elif L==8	// pl
	"The banner <strong>%s</strong> jest ukryte.";
#elif L==9	// pt
	"O banner <strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_The_banner_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El banner <strong>%s</strong> ara est&agrave; visible.";
#elif L==2	// de
	"Der Werbebanner <strong>%s</strong> ist jetzt sichtbar.";	// Need bersetzung
#elif L==3	// en
	"The banner <strong>%s</strong> is now visible.";
#elif L==4	// es
	"El banner <strong>%s</strong> ahora est&aacute; visible.";
#elif L==5	// fr
	"Le banner <strong>%s</strong> est maintenant visible.";
#elif L==6	// gn
	"El banner <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7	// it
	"Il banner <strong>%s</strong> &egrave; ora visible.";
#elif L==8	// pl
	"The banner <strong>%s</strong> jest teraz widoczny.";
#elif L==9	// pt
	"O banner <strong>%s</strong> &eacute; agora visible.";
#endif

const char *Txt_The_capacity_of_classroom_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'aforament de l'aula <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Die Kapazit&auml;t des Klassenzimmers <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The capacity of classroom <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El aforo del aula <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"La capacit&eacute; de la salle de classe <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El aforo del aula <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"La capacit&agrave; della aula <strong>%s</strong> non &egrave; cambiata.";
#elif L==8	// pl
	"Pojemno&sacute;&cacute; klasy <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"A capacidade da sala de aula <strong>%s</strong> n&atilde;o mudou.";
#endif

const char *Txt_The_capacity_of_classroom_X_is_now_Y =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1	// ca
	"L'aforament de l'aula <strong>%s</strong> ara &eacute;s <strong>%u</strong>.";
#elif L==2	// de
	"Die Kapazit&auml;t des Klassenzimmers <strong>%s</strong> betr&auml;gt jetzt <strong>%u</strong>.";
#elif L==3	// en
	"The capacity of classroom <strong>%s</strong> is now <strong>%u</strong>.";
#elif L==4	// es
	"El aforo del aula <strong>%s</strong> ahora es <strong>%u</strong>.";
#elif L==5	// fr
	"La capacit&eacute; de la salle de classe <strong>%s</strong> est maintenant de <strong>%u</strong>.";
#elif L==6	// gn
	"El aforo del aula <strong>%s</strong> ahora es <strong>%u</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La capacit&agrave; della aula <strong>%s</strong> &egrave; ora <strong>%u</strong>.";
#elif L==8	// pl
	"Pojemno&sacute;&cacute; klasy <strong>%s</strong> wynosi teraz <strong>%u</strong>.";
#elif L==9	// pt
	"A capacidade da sala de aula <strong>%s</strong> &eacute; agora <strong>%u</strong>.";
#endif

const char *Txt_The_centre_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El centro <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==2	// de
	"Es wurde bereits ein Lehrinstitut mit dem Namen <strong>%s</strong> eingegeben.";
#elif L==3	// en
	"The centre <strong>%s</strong> already exists.";
#elif L==4	// es
	"El centro <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The centre <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El centro <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"Il centro <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The centre <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O centro <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_centre_X_has_been_moved_to_the_institution_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El centro <strong>%s</strong> se ha movido"
	" a la instituci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The centre <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"The centre <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";
#elif L==4	// es
	"El centro <strong>%s</strong> se ha movido"
	" a la instituci&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"The centre <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"El centro <strong>%s</strong> se ha movido"
	" a la instituci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il centre <strong>%s</strong> &egrave; stato spostato"
	" alla istituzione <strong>%s</strong>.";
#elif L==8	// pl
	"The centre <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O centro <strong>%s</strong> foi transferido"
	" para a institui&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_The_centre_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il centro <strong>%s</strong> &egrave; stato rinominato <strong>%s</strong>.";
#elif L==8	// pl
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O centro <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_classroom_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'aula <strong>%s</strong> ja existeix.";
#elif L==2	// de
	"Das Klassenzimmer <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The classroom <strong>%s</strong> already exists.";
#elif L==4	// es
	"El aula <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"La salle de classe <strong>%s</strong> existe d&eacute;j&agrave;.";
#elif L==6	// gn
	"El aula <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"L'aula <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"Klasa <strong>%s</strong> ju&zdot; istnieje.";
#elif L==9	// pt
	"A sala de clase <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_classroom_X_does_not_have_a_limited_capacity_now =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'aula <strong>%s</strong> ja no t&eacute; un aforament limitat.";
#elif L==2	// de
	"Das Klassenzimmer <strong>%s</strong> hat jetzt keine begrenzte Kapazit&auml;t.";
#elif L==3	// en
	"The classroom <strong>%s</strong> does not have a limited capacity now.";
#elif L==4	// es
	"El aula <strong>%s</strong> ya no tiene un aforo limitado.";
#elif L==5	// fr
	"La salle de classe <strong>%s</strong> n'a plus de capacit&eacute; limit&eacute;e.";
#elif L==6	// gn
	"El aula <strong>%s</strong> ya no tiene un aforo limitado.";	// Okoteve traduccin
#elif L==7	// it
	"L'aula <strong>%s</strong> non ha una capacit&agrave; limitata ora.";
#elif L==8	// pl
	"Klasa <strong>%s</strong> nie ma teraz ograniczonej pojemno&sacute;ci.";
#elif L==9	// pt
	"A sala de clase <strong>%s</strong> n&atilde;o tem capacidade limitada agora.";
#endif

const char *Txt_The_classroom_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"L'aula <strong>%s</strong> ha passat a denominar-se <strong>%s</strong>.";
#elif L==2	// de
	"Der Standort <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"Das Klassenzimmer <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El aula <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"La salle de classe <strong>%s</strong> a &eacute;t&eacute; renomm&eacute;e en <strong>%s</strong>.";
#elif L==6	// gn
	"El aula <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"L'aula <strong>%s</strong> &egrave; stata rinominata come <strong>%s</strong>.";
#elif L==8	// pl
	"Klasa <strong>%s</strong> zosta&lstrok;o przemianowane na <strong>%s</strong>.";
#elif L==9	// pt
	"A sala de clase <strong>%s</strong> foi renomeada como <strong>%s</strong>.";
#endif

const char *Txt_The_classroom_assigned_to_the_group_X_has_changed =
#if   L==1	// ca
	"L'aula assignada al grup <strong>%s</strong> ha canviat.";
#elif L==2	// de
	"Das der Gruppe <strong>%s</strong> zugeordnete Klassenzimmer hat sich ge&auml;ndert.";
#elif L==3	// en
	"The classroom assigned to the group <strong>%s</strong> has changed.";
#elif L==4	// es
	"El aula asignada al grupo <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"La salle de classe affect&eacute;e au groupe <strong>%s</strong> a chang&eacute;.";
#elif L==6	// gn
	"El aula asignada al grupo <strong>%s</strong> ha cambiado.";		// Okoteve traduccin
#elif L==7	// it
	"L'aula assegnata al gruppo <strong>%s</strong> &egrave; cambiata.";
#elif L==8	// pl
	"Klasa przypisana do grupy <strong>%s</strong> uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"A sala de aula atribu&iacute;da ao grupo <strong>%s</strong> mudou.";
#endif

const char *Txt_The_comment_no_longer_exists =
#if   L==1	// ca
	"El comentari ja no existeix.";
#elif L==2	// de
	"Der Kommentar nicht mehr existiert.";
#elif L==3	// en
	"The comment no longer exists.";
#elif L==4	// es
	"El comentario ya no existe.";
#elif L==5	// fr
	"Le commentaire a disparu.";
#elif L==6	// gn
	"El comentario ya no existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il commento non esiste pi&ugrave;.";
#elif L==8	// pl
	"Komentarz ju&zdot; nie istnieje.";
#elif L==9	// pt
	"O coment&aacute;rio n&atilde;o existe mais.";
#endif

const char *Txt_The_contents_of_the_folder_are_too_big =
#if   L==1	// ca
	"El contingut de la carpeta &eacute;s massa gran.";
#elif L==2	// de
	"Der Inhalt der Ordner sind zu gro&szlig;.";
#elif L==3	// en
	"The contents of the folder are too big.";
#elif L==4	// es
	"El contenido de la carpeta es demasiado grande.";
#elif L==5	// fr
	"Le contenu du dossier est trop gros.";
#elif L==6	// gn
	"El contenido de la carpeta es demasiado grande.";	// Okoteve traduccin
#elif L==7	// it
	"Il contenuto della cartella &egrave; troppo grande.";
#elif L==8	// pl
	"Zawarto&sacute;&cacute; folderu s&aogon; zbyt du&zdot;e.";
#elif L==9	// pt
	"O conte&uacute;do do diret&oacute;rio &eacute; muito grande.";
#endif

const char *Txt_The_copy_has_been_successful =
#if   L==1	// ca
	"La copia se ha realizado con &eacute;xito.";	// Necessita traduccio
#elif L==2	// de
	"Die Kopie wurde erfolgreich.";
#elif L==3	// en
	"The copy has been successful.";
#elif L==4	// es
	"La copia se ha realizado con &eacute;xito.";
#elif L==5	// fr
	"La copie a &eacute;t&eacute; un succ&egrave;s.";
#elif L==6	// gn
	"La copia se ha realizado con &eacute;xito.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; avvenuta con successo.";
#elif L==8	// pl
	"The copy has been successful.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia foi feita com sucesso.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Necessita traduccio
#elif L==2	// de
	"Der Kopiervorgang wurde aus Platzgr&uuml;nden abgebrochen, als versucht wurde,"
	" die Datei <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3	// en
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5	// fr
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; esso superava la quota di gestione del disco.";
#elif L==8	// pl
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar o arquivo <strong>%s</strong>"
	" porque ele excederia a cota de disco.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Necessita traduccio
#elif L==2	// de
	"Der Kopiervorgang wurde aus Platzgr&uuml;nden abgebrochen, als versucht wurde,"
	" das Verzeichnis <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3	// en
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5	// fr
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare la cartella<strong>%s</strong>"
	" perch&egrave; essa superava la quota di gestione del disco.";
#elif L==8	// pl
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar a pasta <strong>%s</strong>"
	" porque ele excederia a cota de disco.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_link_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Necessita traduccio
#elif L==2	// de
	"Der Kopiervorgang wurde aus Platzgr&uuml;nden abgebrochen, als versucht wurde,"
	" das Link <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3	// en
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5	// fr
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare il link<strong>%s</strong>"
	" perch&egrave; essa superava la quota di gestione del disco.";
#elif L==8	// pl
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar o link <strong>%s</strong>"
	" porque ele excederia a cota de disco.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_maximum_allowed_number_of_levels =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Necessita traduccio
#elif L==2	// de
	"Zu viele Ebenen. Der Kopiervorgang wurde abgebrochen, als versucht wurde,"
	" die Datei <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3	// en
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";
#elif L==5	// fr
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; esso superava il massimo numero di livelli permesso.";
#elif L==8	// pl
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar o arquivo <strong>%s</strong>"
	" porque ele excederia o n&uacute;mero m&aacute;ximo permitido de n&iacute;veis.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_maximum_allowed_number_of_levels =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Necessita traduccio
#elif L==2	// de
	"Zu viele Ebenen. Der Kopiervorgang wurde abgebrochen, als versucht wurde,"
	" die Datei <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3	// en
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";
#elif L==5	// fr
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; superava il massimo numero di livelli permessi.";
#elif L==8	// pl
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar o arquivo <strong>%s</strong>"
	" porque ele excederia o n&uacute;mero m&aacute;ximo permitido de n&iacute;veis.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_link_X_because_it_would_exceed_the_maximum_allowed_number_of_levels =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Necessita traduccio
#elif L==2	// de
	"Zu viele Ebenen. Der Kopiervorgang wurde abgebrochen, als versucht wurde,"
	" die Link <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3	// en
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";
#elif L==5	// fr
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare il link <strong>%s</strong>"
	" perch&egrave; superava il massimo numero di livelli permessi.";
#elif L==8	// pl
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar o link <strong>%s</strong>"
	" porque ele excederia o n&uacute;mero m&aaxute;ximo permitido de n&iacute;veis.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_there_is_already_an_object_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";	// Necessita traduccio
#elif L==2	// de
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, die Datei <strong>%s</strong>"
	" einzuf&uuml;gen. Der Objektname wird bereits verwendet.";
#elif L==3	// en
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";
#elif L==5	// fr
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";	// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque ya existe otro objecto con ese nombre.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; c'era gi&agrave; un oggetto con quel nome.";
#elif L==8	// pl
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar o arquivo <strong>%s</strong>"
	" porque j&aacute; existe um objeto com esse nome.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_link_X_because_there_is_already_an_object_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";	// Necessita traduccio
#elif L==2	// de
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, die Link <strong>%s</strong>"
	" einzuf&uuml;gen. Der Objektname wird bereits verwendet.";
#elif L==3	// en
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";
#elif L==5	// fr
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";	// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque ya existe otro objecto con ese nombre.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare il link <strong>%s</strong>"
	" perch&egrave; c'era gi&agrave; un oggetto con quel nome.";
#elif L==8	// pl
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar o link <strong>%s</strong>"
	" porque j&aacute; existe um objeto com esse nome.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_you_can_not_paste_a_file_here_of_a_type_other_than_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";	// Necessita traduccio
#elif L==2	// de
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, die Datei <strong>%s</strong>"
	" einzuf&uuml;gen. Es d&uuml;rfen nur HTML-Daten eingef&uumlgt werden.";
#elif L==3	// en
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";
#elif L==4	// es
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";
#elif L==5	// fr
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// Besoin de traduction
#elif L==6	// gn
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";	// Okoteve traduccin
#elif L==7	// it
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; non puoi incollare qui un file di un tipo diverso da HTML.";
#elif L==8	// pl
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A c&oacute;pia parou ao tentar colar o arquivo <strong>%s</strong>"
	" porque voc&ecirc; n&atilde;o pode colar um arquivo aqui de um tipo diferente de HTML.";
#endif

const char *Txt_The_country_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El pa&iacute;s <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2	// de
	"Die Land <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The country <strong>%s</strong> already exists.";
#elif L==4	// es
	"El pa&iacute;s <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The country <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El pa&iacute;s <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il paese <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The country <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O pa&iacute;s <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_country_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il paese <strong>%s</strong> &egrave; stato rimosso come <strong>%s</strong>.";
#elif L==8	// pl
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O pa&iacute;s <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_country_of_the_institution_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==3	// en
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==4	// es
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==6	// gn
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il paese dell'istituzione <strong>%s</strong>"
	" &egrave; cambiato in <strong>%s</strong>.";
#elif L==8	// pl
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#endif

const char *Txt_The_course_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La asignatura <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==2	// de
	"Die Kurs <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The course <strong>%s</strong> already exists.";
#elif L==4	// es
	"La asignatura <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The course <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"La asignatura <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"Il corso <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The course <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A disciplina <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_course_X_already_exists_in_year_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La asignatura <strong>%s</strong> ya existe en el a&ntilde;o/semestre %s.";	// Necessita traduccio
#elif L==2	// de
	"Die Kurs <strong>%s</strong> wurde bereits in Studienjahr/Semester %s eingegeben.";
#elif L==3	// en
	"The course <strong>%s</strong> already exists in year/semester %s.";
#elif L==4	// es
	"La asignatura <strong>%s</strong> ya existe en el a&ntilde;o/semestre %s.";
#elif L==5	// fr
	"The course <strong>%s</strong> already exists in year/semester %s.";	// Besoin de traduction
#elif L==6	// gn
	"La asignatura <strong>%s</strong> ya existe en el a&ntilde;o/semestre %s.";	// Okoteve traduccin
#elif L==7	// it
	"Il corso <strong>%s</strong> esiste gi&agrave; nell'anno/semester %s.";
#elif L==8	// pl
	"The course <strong>%s</strong> already exists in year/semester %s.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A disciplina <strong>%s</strong> j&aacute; existe no ano/semestre %s.";
#endif

const char *Txt_The_course_X_has_been_moved_to_the_degree_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";
#elif L==4	// es
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";
#elif L==5	// fr
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il corso <strong>%s</strong> &egrave; stato spostato"
	" dalla laurea <strong>%s</strong>.";
#elif L==8	// pl
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A disciplina curso <strong>%s</strong> foi movida"
	" para o grau <strong>%s</strong>.";
#endif

const char *Txt_The_date_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4	// es
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La data delle festivit&agrave; <strong>%s</strong> &egrave; cambiata a <strong>%s</strong>.";
#elif L==8	// pl
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A data do feriado <strong>%s</strong> mudou para <strong>%s</strong>.";
#endif

const char *Txt_The_date_range_must_be_less_than_or_equal_to_X_days =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";	// Necessita traduccio
#elif L==2	// de
	"The date range must be less than or equal to %u days.";	// Need bersetzung
#elif L==3	// en
	"The date range must be less than or equal to %u days.";
#elif L==4	// es
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";
#elif L==5	// fr
	"The date range must be less than or equal to %u days.";	// Besoin de traduction
#elif L==6	// gn
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";	// Okoteve traduccin
#elif L==7	// it
	"Il range della data deve essere minore o uguale a %u giorni.";
#elif L==8	// pl
	"The date range must be less than or equal to %u days.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O intervalo de datas deve ser menor ou igual a %u dias.";
#endif

const char *Txt_The_date_time_of_opening_of_groups_has_changed =
#if   L==1	// ca
	"La data / hora d'obertura de grups s'ha modificat.";
#elif L==2	// de
	"The date / time of opening of groups has changed.";			// Need bersetzung
#elif L==3	// en
	"The date / time of opening of groups has changed.";
#elif L==4	// es
	"La fecha / hora de apertura ha cambiado.";
#elif L==5	// fr
	"The date / time of opening of groups has changed.";			// Besoin de traduction
#elif L==6	// gn
	"La fecha / hora de apertura ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"La data / ora di apertura dei gruppi &egrave; cambiata.";
#elif L==8	// pl
	"The date / time of opening of groups has changed.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A data / hora de abertura dos grupos foi alterada.";
#endif

const char *Txt_The_degree_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La titulaci&oacute;n <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2	// de
	"Der Studiengang <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The degree <strong>%s</strong> already exists.";
#elif L==4	// es
	"La titulaci&oacute;n <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The degree <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"La titulaci&oacute;n <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"La laurea <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The degree <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O grau <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_degree_X_has_been_moved_to_the_centre_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La titulaci&oacute;n <strong>%s</strong> se ha movido"
	" al centro <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The degree <strong>%s</strong> has been moved"
	" to the centre <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"The degree <strong>%s</strong> has been moved"
	" to the centre <strong>%s</strong>.";
#elif L==4	// es
	"La titulaci&oacute;n <strong>%s</strong> se ha movido"
	" al centro <strong>%s</strong>.";
#elif L==5	// fr
	"The degree <strong>%s</strong> has been moved"
	" to the centre <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"La titulaci&oacute;n <strong>%s</strong> se ha movido"
	" al centro <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"La laurea <strong>%s</strong> &egrave; stata spostata"
	" al centro <strong>%s</strong>.";
#elif L==8	// pl
	"The degree <strong>%s</strong> has been moved"
	" to the centre <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O grau <strong>%s</strong> foi movido"
	" para o centro <strong>%s</strong>.";
#endif

const char *Txt_The_department_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El departamento <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2	// de
	"Die Abteilung <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The department <strong>%s</strong> already exists.";
#elif L==4	// es
	"El departamento <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The department <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El departamento <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"IL dipartimento <strong>%s</strong> esiste gi&agrave;.";
#elif L==8	// pl
	"The department <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O departamento <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_department_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El departamento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El departamento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"El departamento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il dipartimento <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O departamento <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_email_X_has_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El correo <strong>%s</strong> ha sido confirmado.";	// Necessita traduccio
#elif L==2	// de
	"The email <strong>%s</strong> has been confirmed.";	// Need bersetzung
#elif L==3	// en
	"The email <strong>%s</strong> has been confirmed.";
#elif L==4	// es
	"El correo <strong>%s</strong> ha sido confirmado.";
#elif L==5	// fr
	"The email <strong>%s</strong> has been confirmed.";	// Besoin de traduction
#elif L==6	// gn
	"El correo <strong>%s</strong> ha sido confirmado.";	// Okoteve traduccin
#elif L==7	// it
	"The email <strong>%s</strong> has been confirmed.";	// Bisogno di traduzione
#elif L==8	// pl
	"The email <strong>%s</strong> has been confirmed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O email <strong>%s</strong> foi confirmado.";
#endif

const char *Txt_The_email_address_entered_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";	// Necessita traduccio
#elif L==2	// de
	"The email address entered"
	" <strong>%s</strong> is not valid.";		// Need bersetzung
#elif L==3	// en
	"The email address entered"
	" <strong>%s</strong> is not valid.";
#elif L==4	// es
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";
#elif L==5	// fr
	"The email address entered"
	" <strong>%s</strong> is not valid.";		// Besoin de traduction
#elif L==6	// gn
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";	// Okoteve traduccin
#elif L==7	// it
	"Il email immesso"
	" <strong>%s</strong> non &egrave; valido.";
#elif L==8	// pl
	"The email address entered"
	" <strong>%s</strong> is not valid.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O endere&ccedil;o de email inserido"
	" <strong>%s</strong> n&atilde;o &eacute; v&aacute;lido.";
#endif

const char *Txt_The_email_address_X_had_been_registered_by_another_user =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya ha sido registrada por otro usuario.";	// Necessita traduccio
#elif L==2	// de
	"The email address <strong>%s</strong>"
	" had been registered by another user.";	// Need bersetzung
#elif L==3	// en
	"The email address <strong>%s</strong>"
	" had been registered by another user.";
#elif L==4	// es
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya ha sido registrada por otro usuario.";
#elif L==5	// fr
	"The email address <strong>%s</strong>"
	" had been enroled by another user.";		// Besoin de traduction
#elif L==6	// gn
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya ha sido registrada por otro usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Il email <strong>%s</strong>"
	" &egrave; stato registrato da un altro utente.";
#elif L==8	// pl
	"The email address <strong>%s</strong>"
	" had been registered by another user.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O endere&ccedil;o de email <strong>%s</strong>"
	" foi registrado por outro usu&aacute;rio.";
#endif

const char *Txt_The_email_address_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El correo <strong>%s</strong>"
	" se ha registrado correctamente.";		// Necessita traduccio
#elif L==2	// de
	"The email address <strong>%s</strong>"
	" has been registered successfully.";		// Need bersetzung
#elif L==3	// en
	"The email address <strong>%s</strong>"
	" has been registered successfully.";
#elif L==4	// es
	"El correo <strong>%s</strong>"
	" se ha registrado correctamente.";
#elif L==5	// fr
	"The email address <strong>%s</strong>"
	" has been registered successfully.";		// Besoin de traduction
#elif L==6	// gn
	"El correo <strong>%s</strong>"
	" se ha registrado correctamente.";		// Okoteve traduccin
#elif L==7	// it
	"Il email <strong>%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==8	// pl
	"The email address <strong>%s</strong>"
	" has been registered successfully.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O endere&ccedil;o de email <strong>%s</strong>"
	" foi registrado com sucesso.";
#endif

const char *Txt_The_email_address_has_not_been_confirmed =
#if   L==1	// ca
	"La direcci&oacute;n de correo"
	" no ha podido confirmarse.";			// Necessita traduccio
#elif L==2	// de
	"The email address has not been confirmed.";	// Need bersetzung
#elif L==3	// en
	"The email address has not been confirmed.";
#elif L==4	// es
	"La direcci&oacute;n de correo"
	" no ha podido confirmarse.";
#elif L==5	// fr
	"The email address has not been confirmed.";	// Besoin de traduction
#elif L==6	// gn
	"La direcci&oacute;n de correo"
	" no ha podido confirmarse.";			// Okoteve traduccin
#elif L==7	// it
	"The email address has not been confirmed.";	// Bisogno di traduzione
#elif L==8	// pl
	"The email address has not been confirmed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O endere&ccedil;o de email n&atilde;o foi confirmado.";
#endif

const char *Txt_The_email_address_X_matches_one_previously_registered =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con una registrada previamente.";	// Necessita traduccio
#elif L==2	// de
	"The email address <strong>%s</strong>"
	" matches one previously registered.";		// Need bersetzung
#elif L==3	// en
	"The email address <strong>%s</strong>"
	" matches one previously registered.";
#elif L==4	// es
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con una registrada previamente.";
#elif L==5	// fr
	"The email address <strong>%s</strong>"
	" matches one previously registered.";		// Besoin de traduction
#elif L==6	// gn
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con una registrada previamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il email <strong>%s</strong>"
	" coincide con uno precedentemente registrato.";
#elif L==8	// pl
	"The email address <strong>%s</strong>"
	" matches one previously registered.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O endere&ccedil;o de email <strong>%s</strong>"
	" corresponde a um registrado anteriormente.";
#endif

const char *Txt_The_email_domain_X_already_exists =
#if   L==1	// ca
	"El dominio de correo ya existe.";		// Necessita traduccio
#elif L==2	// de
	"Die E-Mail-Dom&auml;ne wurde bereits eingegeben.";
#elif L==3	// en
	"The email domain already exists.";
#elif L==4	// es
	"El dominio de correo ya existe.";
#elif L==5	// fr
	"The email domain already exists.";		// Besoin de traduction
#elif L==6	// gn
	"El dominio de correo ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"Il campo mail gi&agrave; esiste.";
#elif L==8	// pl
	"The email domain already exists.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O dom&iacute;nio de email j&aacute; existe.";
#endif

const char *Txt_The_email_domain_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El dominio de correo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The email domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==3	// en
	"The email domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El dominio de correo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The email domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==6	// gn
	"El dominio de correo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il campo mail <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"The email domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O dom&iacute;nio de email <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_email_domain_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El dominio de correo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The email domain <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3	// en
	"The email domain <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El dominio de correo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The email domain <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El dominio de correo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il campo mail <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The email domain <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O dom&iacute;nio de email <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_enrolment_of_students_into_groups_of_type_X_is_now_mandatory =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es obligatoria.";	// Necessita traduccio
#elif L==2	// de
	"The enrolment of students into groups of type <strong>%s</strong> is now mandatory.";			// Need bersetzung
#elif L==3	// en
	"The enrolment of students into groups of type <strong>%s</strong> is now mandatory.";
#elif L==4	// es
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es obligatoria.";
#elif L==5	// fr
	"The enrolment of students into groups of type <strong>%s</strong> is now mandatory.";			// Besoin de traduction
#elif L==6	// gn
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es obligatoria.";	// Okoteve traduccin
#elif L==7	// it
	"La registrazione di studenti nei gruppi di tipo <strong>%s</strong> &egrave; ora obbligatoria.";
#elif L==8	// pl
	"The enrolment of students into groups of type <strong>%s</strong> is now mandatory.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A inscri&ccedil;&atilde;o de alunos em grupos do tipo <strong>%s</strong> &eacute; agora obrigat&oacute;ria.";
#endif

const char *Txt_The_enrolment_of_students_into_groups_of_type_X_is_now_voluntary =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es voluntaria.";	// Necessita traduccio
#elif L==2	// de
	"The enrolment of students into groups of type <strong>%s</strong> is now voluntary.";			// Need bersetzung
#elif L==3	// en
	"The enrolment of students into groups of type <strong>%s</strong> is now voluntary.";
#elif L==4	// es
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es voluntaria.";
#elif L==5	// fr
	"The enrolment of students into groups of type <strong>%s</strong> is now voluntary.";			// Besoin de traduction
#elif L==6	// gn
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es voluntaria.";	// Okoteve traduccin
#elif L==7	// it
	"La registrazione di studenti nei gruppi di tipo <strong>%s</strong> &egrave; ora volontaria.";
#elif L==8	// pl
	"The enrolment of students into groups of type <strong>%s</strong> is now voluntary.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A inscri&ccedil;&atilde;o de estudantes em grupos do tipo <strong>%s</strong> &eacute; agora volunt&aacute;ria.";
#endif

const char *Txt_The_event_has_been_modified =
#if   L==1	// ca
	"El evento ha sido modificado.";	// Necessita traduccio
#elif L==2	// de
	"Der Ereignis wurde ge&auml;ndert.";
#elif L==3	// en
	"The event has been modified.";
#elif L==4	// es
	"El evento ha sido modificado.";
#elif L==5	// fr
	"L'&eacute;v&eacute;nement a &eacute;t&eacute; modifi&eacute;.";
#elif L==6	// gn
	"El evento ha sido modificado.";	// Okoteve traduccin
#elif L==7	// it
	"L'evento &egrave; stata modificato.";
#elif L==8	// pl
	"The event has been modified.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O evento foi modificado.";
#endif

const char *Txt_The_file_could_not_be_processed_successfully =
#if   L==1	// ca
	"El fitxer no s'ha pogut processar correctament.";
#elif L==2	// de
	"Die Datei konnte nicht erfolgreich verarbeitet werden.";
#elif L==3	// en
	"The file could not be processed successfully.";
#elif L==4	// es
	"El archivo no ha podido procesarse correctamente.";
#elif L==5	// fr
	"Le fichier n'a pas pu &ecirc;tre trait&eacute; avec succ&egrave;s.";
#elif L==6	// gn
	"El archivo no ha podido procesarse correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il file non pu&ograve; essere elaborato con successo.";
#elif L==8	// pl
	"Plik nie m&oacute;g&lstrok; zosta&cacute; pomy&sacute;lnie przetworzony.";
#elif L==9	// pt
	"O arquivo n&atilde;o p&ocirc;de ser processado com sucesso.";
#endif

const char *Txt_The_file_X_has_been_placed_inside_the_folder_Y =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Necessita traduccio
#elif L==2	// de
	"Die Datei <strong>%s</strong> wurde"
	" in das Verzeichnis <strong>%s</strong> gesteckt.";
#elif L==3	// en
	"The file <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";
#elif L==4	// es
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";
#elif L==5	// fr
	"Le fichier <strong>%s</strong> a &eacute;t&eacute; plac&eacute;"
	" dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6	// gn
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"Il file <strong>%s</strong> &egrave; stato messo"
	" all'interno della cartella <strong>%s</strong>.";
#elif L==8	// pl
	"The file <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O arquivo <strong>%s</strong> foi colocado"
	" dentro do diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_The_file_is_not_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El archivo no es <em>%s</em>.";	// Necessita traduccio
#elif L==2	// de
	"The file is not <em>%s</em>.";		// Need bersetzung
#elif L==3	// en
	"The file is not <em>%s</em>.";
#elif L==4	// es
	"El archivo no es <em>%s</em>.";
#elif L==5	// fr
	"The file is not <em>%s</em>.";		// Besoin de traduction
#elif L==6	// gn
	"El archivo no es <em>%s</em>.";	// Okoteve traduccin
#elif L==7	// it
	"IL file non &egrave; <em>%s</em>.";
#elif L==8	// pl
	"The file is not <em>%s</em>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O arquivo n&atilde;o &eacute; <em>%s</em>.";
#endif

const char *Txt_The_file_of_folder_no_longer_exists_or_is_now_hidden =
#if   L==1	// ca
	"L'arxiu o carpeta ja no existeix o est&agrave; ara ocult.";
#elif L==2	// de
	"Die Datei oder Ordner existiert nicht mehr oder ist jetzt versteckt.";
#elif L==3	// en
	"The file or folder no longer exists or is now hidden.";
#elif L==4	// es
	"El archivo o carpeta ya no existe o est&aacute; ahora oculto.";
#elif L==5	// fr
	"Le fichier ou r&eacute;pertoire n'existe plus ou est maintenant cach&eacute;.";
#elif L==6	// gn
	"El archivo o carpeta ya no existe o est&aacute; ahora oculto.";	// Okoteve traduccin
#elif L==7	// it
	"Il file o cartella non esiste pi&ugrave; o &egrave; nascosta.";
#elif L==8	// pl
	"Plik lub folder ju&zdot; nie istnieje lub jest ukryte.";
#elif L==9	// pt
	"O arquivo ou diret&oacute;rio n&atilde;o existe mais ou est&aacute; escondido.";
#endif

const char *Txt_The_file_type_is_X_and_should_be_HTML_or_ZIP =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";	// Necessita traduccio
#elif L==2	// de
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Need bersetzung
#elif L==3	// en
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";
#elif L==4	// es
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";
#elif L==5	// fr
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Besoin de traduction
#elif L==6	// gn
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di file &egrave; %s e dovrebbe essere <em>HTML</em> o <em>ZIP</em>.";
#elif L==8	// pl
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de arquivo &eacute; %s e deve ser <em>HTML</em> ou <em>ZIP</em>.";
#endif

const char *Txt_The_file_type_should_be_HTML_or_ZIP =
#if   L==1	// ca
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";	// Necessita traduccio
#elif L==2	// de
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Need bersetzung
#elif L==3	// en
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";
#elif L==4	// es
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";
#elif L==5	// fr
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Besoin de traduction
#elif L==6	// gn
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di file dovrebbe essere <em>HTML</em> o <em>ZIP</em>.";
#elif L==8	// pl
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de arquivo deve ser <em>HTML</em> ou <em>ZIP</em>.";
#endif

const char *Txt_The_folder_X_has_been_created_inside_the_folder_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Das Verzeichnis <strong>%s</strong> wurde im Verzeichnis <strong>%s</strong> erstellt.";
#elif L==3	// en
	"The folder <strong>%s</strong> has been created inside the folder <strong>%s</strong>.";
#elif L==4	// es
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";
#elif L==5	// fr
	"Le r&eacute;pertoire <strong>%s</strong> a &eacute;t&eacute; cr&eacute;&eacute; dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6	// gn
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La cartella <strong>%s</strong> &egrave; stata creata all'interno della cartella <strong>%s</strong>.";
#elif L==8	// pl
	"The folder <strong>%s</strong> has been created inside the folder <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O diret&oacute;rio <strong>%s</strong> foi criado dentro do diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_The_folder_is_empty =
#if   L==1	// ca
	"La carpeta est&agrave; buida.";
#elif L==2	// de
	"Der Verzeichnis is leer.";
#elif L==3	// en
	"The folder is empty.";
#elif L==4	// es
	"La carpeta est&aacute; vac&iacute;a.";
#elif L==5	// fr
	"Le r&eacute;pertoire est vide.";
#elif L==6	// gn
	"La carpeta est&aacute; vac&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"La cartella &egrave; vuota.";
#elif L==8	// pl
	"Folder jest pusty.";
#elif L==9	// pt
	"O diret&oacute;rio est&aacute; vazio.";
#endif

const char *Txt_The_folder_name_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Se ha cambiado el nombre de la carpeta <strong>%s</strong> por <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Das Verzeichnis <strong>%s</strong> wurde umbenannt in <strong>%s</strong>.";
#elif L==3	// en
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4	// es
	"Se ha cambiado el nombre de la carpeta <strong>%s</strong> por <strong>%s</strong>.";
#elif L==5	// fr
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"Se ha cambiado el nombre de la carpeta <strong>%s</strong> por <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome della cartella <strong>%s</strong> &egrave; cambiato in <strong>%s</strong>.";
#elif L==8	// pl
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome da pasta <strong>%s</strong> foi alterado para <strong>%s</strong>.";
#endif

const char *Txt_The_folder_name_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"Der Name des Verzeichnisses <strong>%s</strong> wurde nicht ge&auml;ndert.";
#elif L==3	// en
	"The folder name <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The folder name <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome della cartella <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The folder name <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome da pasta <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_folder_name_X_has_not_changed_because_there_is_already_a_folder_or_a_file_with_the_name_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Der Name des Verzeichnisses <strong>%s</strong> wurde nicht ge&auml;ndert,"
	" weil bereits ein Verzeichnis oder eine Datei mit Namen <strong>%s</strong> besteht.";
#elif L==3	// en
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";
#elif L==4	// es
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";
#elif L==5	// fr
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome della cartella <strong>%s</strong> non &egrave; cambiato"
	" perch&egrave; c'&egrave; gi&agrave; una cartella o un file con il nome <strong>%s</strong>.";
#elif L==8	// pl
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome da pasta <strong>%s</strong> n&atilde;o foi alterado"
	" porque j&aacute; existe uma pasta ou um arquivo com o nome <strong>%s</strong>.";
#endif

const char *Txt_The_game_has_been_modified =
#if   L==1	// ca
	"El joc ha estat modificat.";
#elif L==2	// de
	"Das Spiel wurde ge&auml;ndert.";
#elif L==3	// en
	"The game has been modified.";
#elif L==4	// es
	"El juego ha sido modificado.";
#elif L==5	// fr
	"Le jeu a &eacute;t&eacute; modifi&eacute;.";
#elif L==6	// gn
	"El juego ha sido modificado.";	// Okoteve traduccin
#elif L==7	// it
	"Il gioco &egrave; stato modificato.";
#elif L==8	// pl
	"Gra zosta&lstrok;o zmodyfikowane.";
#elif L==9	// pt
	"O jogo foi modificado.";
#endif

const char *Txt_The_group_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El grupo <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==2	// de
	"Die Gruppe <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The group <strong>%s</strong> already exists.";
#elif L==4	// es
	"El grupo <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The group <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El grupo <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"Il gruppo <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The group <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O grupo <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_group_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il gruppo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O grupo <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_group_X_is_now_closed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";	// Necessita traduccio
#elif L==2	// de
	"The group <strong>%s</strong> is now closed.";			// Need bersetzung
#elif L==3	// en
	"The group <strong>%s</strong> is now closed.";
#elif L==4	// es
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";
#elif L==5	// fr
	"The group <strong>%s</strong> is now closed.";			// Besoin de traduction
#elif L==6	// gn
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";	// Okoteve traduccin
#elif L==7	// it
	"Il gruppo <strong>%s</strong> &egrave; ora chiuso.";
#elif L==8	// pl
	"The group <strong>%s</strong> is now closed.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O grupo <strong>%s</strong> est&aacute; agora fechado.";
#endif

const char *Txt_The_group_X_is_now_open =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";	// Necessita traduccio
#elif L==2	// de
	"The group <strong>%s</strong> is now open.";			// Need bersetzung
#elif L==3	// en
	"The group <strong>%s</strong> is now open.";
#elif L==4	// es
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";
#elif L==5	// fr
	"The group <strong>%s</strong> is now open.";			// Besoin de traduction
#elif L==6	// gn
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";	// Okoteve traduccin
#elif L==7	// it
	"Il gruppo <strong>%s</strong> &egrave; ora aperto.";
#elif L==8	// pl
	"The group <strong>%s</strong> is now open.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O grupo <strong>%s</strong> est&aacute; agora aberto.";
#endif

const char *Txt_The_group_X_does_not_have_a_student_limit_now =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El grup <strong>%s</strong> ara no t&eacute; cap l&iacute;mit d'estudiants.";
#elif L==2	// de
	"Die Gruppe <strong>%s</strong> hat jetzt kein Studentenlimit.";
#elif L==3	// en
	"The group <strong>%s</strong> does not have a student limit now.";
#elif L==4	// es
	"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de estudiantes.";
#elif L==5	// fr
	"Le groupe <strong>%s</strong> n'a plus de limite d'&eacute;tudiants.";
#elif L==6	// gn
	"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de estudiantes.";	// Okoteve traduccin
#elif L==7	// it
	"Il gruppo <strong>%s</strong> ora non ha limite di studenti.";
#elif L==8	// pl
	"Grupa <strong>%s</strong> nie ma teraz &zdot;adnych limit&oacute;w uczni&oacute;w.";
#elif L==9	// pt
	"O grupo <strong>%s</strong> n&atilde;o tem limite de estudantes agora.";
#endif

const char *Txt_The_groups_will_automatically_open =
#if   L==1	// ca
	"Els grups s'obriran autom&agrave;ticament.";
#elif L==2	// de
	"The groups will automatically open.";			// Need bersetzung
#elif L==3	// en
	"The groups will automatically open.";
#elif L==4	// es
	"Los grupos se abrir&aacute;n autom&aacute;ticamente.";
#elif L==5	// fr
	"The groups will automatically open.";			// Besoin de traduction
#elif L==6	// gn
	"Los grupos se abrir&aacute;n autom&aacute;ticamente.";	// Okoteve traduccin
#elif L==7	// it
	"I gruppi si aprir&agrave; automaticamente.";
#elif L==8	// pl
	"The groups will automatically open.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Os grupos ser&atilde;o abertos automaticamente.";
#endif

const char *Txt_The_groups_will_not_automatically_open =
#if   L==1	// ca
	"Els grups no s'obriran autom&agrave;ticament.";
#elif L==2	// de
	"The groups will not automatically open.";			// Need bersetzung
#elif L==3	// en
	"The groups will not automatically open.";
#elif L==4	// es
	"Los grupos no se abrir&aacute;n autom&aacute;ticamente.";
#elif L==5	// fr
	"The groups will not automatically open.";			// Besoin de traduction
#elif L==6	// gn
	"Los grupos no se abrir&aacute;n autom&aacute;ticamente.";	// Okoteve traduccin
#elif L==7	// it
	"I gruppi non si aprir&agrave; automaticamente.";
#elif L==8	// pl
	"The groups will not automatically open.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Os grupos n&atilde;o ser&atilde;o abertos automaticamente.";
#endif

const char *Txt_The_HTML_file_has_been_received_successfully =
#if   L==1	// ca
	"El archivo <em>HTML</em> se ha recibido correctamente.";	// Necessita traduccio
#elif L==2	// de
	"The <em>HTML</em> file has been received successfully.";	// Need bersetzung
#elif L==3	// en
	"The <em>HTML</em> file has been received successfully.";
#elif L==4	// es
	"El archivo <em>HTML</em> se ha recibido correctamente.";
#elif L==5	// fr
	"The <em>HTML</em> file has been received successfully.";	// Besoin de traduction
#elif L==6	// gn
	"El archivo <em>HTML</em> se ha recibido correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il <em>HTML</em> file &egrave; stato ricevuto con successo.";
#elif L==8	// pl
	"The <em>HTML</em> file has been received successfully.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O arquivo <em>HTML</em> foi recebido com sucesso.";
#endif

const char *Txt_The_ID_X_has_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";		// Necessita traduccio
#elif L==2	// de
	"The ID <strong>%s</strong>"
	" has been confirmed.";		// Need bersetzung
#elif L==3	// en
	"The ID <strong>%s</strong>"
	" has been confirmed.";
#elif L==4	// es
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";
#elif L==5	// fr
	"The ID <strong>%s</strong>"
	" has been confirmed.";		// Besoin de traduction
#elif L==6	// gn
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";		// Okoteve traduccin
#elif L==7	// it
	"Il ID <strong>%s</strong>"
	" &egrave; stato confermato.";
#elif L==8	// pl
	"The ID <strong>%s</strong>"
	" has been confirmed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O ID <strong>%s</strong>"
	" foi confirmado.";
#endif

const char *Txt_The_ID_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";		// Necessita traduccio
#elif L==2	// de
	"The ID <strong>%s</strong>"
	" has been registered successfully.";		// Need bersetzung
#elif L==3	// en
	"The ID <strong>%s</strong>"
	" has been registered successfully.";
#elif L==4	// es
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";
#elif L==5	// fr
	"The ID <strong>%s</strong>"
	" has been registered successfully.";		// Besoin de traduction
#elif L==6	// gn
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";		// Okoteve traduccin
#elif L==7	// it
	"Il ID <strong>%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==8	// pl
	"The ID <strong>%s</strong>"
	" has been registered successfully.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O ID <strong>%s</strong>"
	" foi registrado com sucesso.";
#endif

const char *Txt_The_ID_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido."
	" Solo puede contener n&uacute;meros (al menos uno) y letras.";		// Necessita traduccio
#elif L==2	// de
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";		// Need bersetzung
#elif L==3	// en
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";
#elif L==4	// es
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido."
	" Solo puede contener n&uacute;meros (al menos uno) y letras.";
#elif L==5	// fr
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";		// Besoin de traduction
#elif L==6	// gn
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido."
	" Solo puede contener n&uacute;meros (al menos uno) y letras.";		// Okoteve traduccin
#elif L==7	// it
	"L'ID <strong>%s</strong> non &egrave; valido."
	" Pu&ograve; contenere solo numeri (almeno uno) e lettere.";
#elif L==8	// pl
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O ID <strong>%s</strong> n&atilde;o &eacute; v&aacute;lido."
	" S&oacute; pode conter n&uacute;meros (pelo menos um) e letras.";
#endif

const char *Txt_The_ID_is_used_in_order_to_facilitate_ =
#if   L==1	// ca
	"L'ID s'utilitza per tal de facilitar"
	" la inscripci&oacute; en les assignatures"
	" i la consulta de qualificacions."
	" Si hi ha diverses versions de la ID"
	" (per ex. DNI amb o sense lletra),"
	" indiqueu totes les versions.";
#elif L==2	// de
	"Die ID wird zu erleichtern,"
	" um die Einschreibung in Kurse"
	" und Beratungskompetenz eingesetzt."
	" Wenn es mehrere Versionen der ID"
	" (zB. mit oder ohne Buchstabe),"
	" schreiben alle Versionen.";
#elif L==3	// en
	"The ID is used in order to facilitate"
	" enrolment in courses"
	" and consulting marks."
	" If there are multiple versions of the ID"
	" (eg. with or without letter),"
	" write out all versions.";
#elif L==4	// es
	"El ID se utiliza para facilitar"
	" la inscripci&oacute;n en las asignaturas"
	" y la consulta de calificaciones."
	" Si existen varias versiones del ID"
	" (por ej. DNI con o sin letra),"
	" indique todas las versiones.";
#elif L==5	// fr
	"L'ID est utilis&eacute; afin de faciliter"
	" l'inscription &agrave; des mati&egrave;res"
	" et la consulte des notes."
	" S'il existe plusieurs versions du ID"
	" (par example avec ou sans lettre),"
	" &eacute;crire toutes les versions.";
#elif L==6	// gn
	"El ID se utiliza para facilitar"
	" la inscripci&oacute;n en las asignaturas"
	" y la consulta de calificaciones."
	" Si existen varias versiones del ID"
	" (por ej. DNI con o sin letra),"
	" indique todas las versiones.";	// Okoteve traduccin
#elif L==7	// it
	"L'ID &egrave; utilizzato per agevolare"
	" l'iscrizione ai corsi"
	" e consultare dei risultati."
	" Se ci sono pi&ugrave; versioni del ID"
	" (ad es. con o senza lettera),"
	" scrivere tutte le versioni.";
#elif L==8	// pl
	"Identyfikator jest u&zdot;ywany w celu u&lstrok;atwienia"
	" rejestracji w kursach"
	" i skonsultuje znak&oacute;w."
	" Je&sacute;li istnieje wiele wersji identyfikatora"
	" (np. Z lub bez litery),"
	" wypisa&cacute; wszystkie wersje.";
#elif L==9	// pt
	"O n&ordm; de identifica&ccedil;&atilde;o &eacute; usado para facilitar"
	" a inscri&ccedil;&atilde;o em disciplinas"
	" e consultas de notas."
	" Se houver v&aacute;rias vers&otilde;es do n&ordm; de identifica&ccedil;&atilde;o"
	" (eg., com ou sem letra),"
	" escreva todas as vers&otilde;es.";
#endif

const char *Txt_The_ID_nickname_or_email_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";	// Necessita traduccio
#elif L==2	// de
	"The @nick, email or ID <strong>%s</strong> is not valid.";				// Need bersetzung
#elif L==3	// en
	"The @nick, email or ID <strong>%s</strong> is not valid.";
#elif L==4	// es
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";
#elif L==5	// fr
	"The @nick, email or ID <strong>%s</strong> is not valid.";				// Besoin de traduction
#elif L==6	// gn
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";	// Okoteve traduccin
#elif L==7	// it
	"Il @nome-utente, email o ID <strong>%s</strong> non &egrave; valido.";
#elif L==8	// pl
	"The @nick, email or ID <strong>%s</strong> is not valid.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O @nick, email ou ID <strong>%s</strong> n&atilde;o &eacute; v&aacute;lido.";
#endif

const char *Txt_The_ID_X_matches_one_of_the_existing =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";		// Necessita traduccio
#elif L==2	// de
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Need bersetzung
#elif L==3	// en
	"The ID <strong>%s</strong>"
	" matches one of the existing.";
#elif L==4	// es
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";
#elif L==5	// fr
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Besoin de traduction
#elif L==6	// gn
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";		// Okoteve traduccin
#elif L==7	// it
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Bisogno di traduzione
#elif L==8	// pl
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O ID <strong>%s</strong>"
	" corresponde a um dos existentes.";
#endif

const char *Txt_The_institution_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La instituci&oacute;n <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2	// de
	"Die Hochschule <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The institution <strong>%s</strong> already exists.";
#elif L==4	// es
	"La instituci&oacute;n <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The institution <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"La instituci&oacute;n <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"L'istituzione <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The institution <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A institui&ccedil;&atilde;o <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_institution_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La instituci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==3	// en
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"La instituci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==6	// gn
	"La instituci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"L'istituzione <strong>%s</strong> &egrave; stata rinominata <strong>%s</strong>.";
#elif L==8	// pl
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A institui&ccedil;&atilde;o <strong>%s</strong> foi renomeada como <strong>%s</strong>.";
#endif

const char *Txt_The_institution_of_the_centre_has_changed =
#if   L==1	// ca
	"La instituci&oacute;n del centro ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The institution of the centre has changed.";		// Need bersetzung
#elif L==3	// en
	"The institution of the centre has changed.";
#elif L==4	// es
	"La instituci&oacute;n del centro ha cambiado.";
#elif L==5	// fr
	"The institution of the centre .";			// Besoin de traduction
#elif L==6	// gn
	"La instituci&oacute;n del centro ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"L'istituzione del centro &egrave; cambiata.";
#elif L==8	// pl
	"The institution of the centre has changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A institui&ccedil;&atilde;o do centro mudou.";
#endif

const char *Txt_The_institution_of_the_department_has_changed =
#if   L==1	// ca
	"La instituci&oacute;n del departamento ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The institution of the department has changed.";	// Need bersetzung
#elif L==3	// en
	"The institution of the department has changed.";
#elif L==4	// es
	"La instituci&oacute;n del departamento ha cambiado.";
#elif L==5	// fr
	"The institution of the department has changed.";	// Besoin de traduction
#elif L==6	// gn
	"La instituci&oacute;n del departamento ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"L'istituzione del dipartimento &egrave; cambiata.";
#elif L==8	// pl
	"The institution of the department has changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A institui&ccedil;&atilde;o do departamento mudou.";
#endif

const char *Txt_The_institutional_code_of_the_course_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";			// Necessita traduccio
#elif L==2	// de
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong> .";		// Need bersetzung
#elif L==3	// en
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==4	// es
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";			// Besoin de traduction
#elif L==6	// gn
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";			// Okoteve traduccin
#elif L==7	// it
	"Il codice istituzionale del corso <strong>%s</strong>"
	" &egrave; cambiato a <strong>%s</strong>.";
#elif L==8	// pl
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O c&oacute;digo institucional da disciplina <strong>%s</strong>"
	" mudou para <strong>%s</strong>.";
#endif

const char *Txt_The_institutional_code_of_the_course_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3	// en
	"The institutional code of the course <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il codice istituzionale del corso <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O c&oacute;digo institucional da disciplina <strong>%s</strong> n&atilde;o mudou.";
#endif

const char *Txt_The_integrated_editor_is_not_yet_available =
#if   L==1	// ca
	"El editor integrado a&uacute;n no est&aacute; disponible.";	// Necessita traduccio
#elif L==2	// de
	"Der integrierte Editor ist noch nicht verf&uuml;gbar.";
#elif L==3	// en
	"The integrated editor is not yet available.";
#elif L==4	// es
	"El editor integrado a&uacute;n no est&aacute; disponible.";
#elif L==5	// fr
	"The integrated editor is not yet available.";			// Besoin de traduction
#elif L==6	// gn
	"El editor integrado a&uacute;n no est&aacute; disponible.";	// Okoteve traduccin
#elif L==7	// it
	"L'editor integrato non &egrave; ancora disponibile.";
#elif L==8	// pl
	"The integrated editor is not yet available.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O editor integrado ainda n&atilde;o est&aacute; dispon&iacute;vel.";
#endif

const char *Txt_The_link_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El enlace <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2	// de
	"Der Link <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The link <strong>%s</strong> already exists.";
#elif L==4	// es
	"El enlace <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The link <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El enlace <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il link <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The link <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O link <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_link_X_has_been_placed_inside_the_folder_Y =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Necessita traduccio
#elif L==2	// de
	"Die Link <strong>%s</strong> wurde"
	" in das Verzeichnis <strong>%s</strong> gesteckt.";
#elif L==3	// en
	"The link <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";
#elif L==4	// es
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";
#elif L==5	// fr
	"Le lien <strong>%s</strong> a &eacute;t&eacute; plac&eacute;"
	" dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6	// gn
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"Il link <strong>%s</strong> &egrave; stato messo"
	" all'interno della cartella <strong>%s</strong>.";
#elif L==8	// pl
	"The link <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A liga&ccedil;&atilde;o <strong>%s</strong> foi colocado"
	" dentro do diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_The_link_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il link <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O link <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_list_of_X_courses_is_too_large_to_be_displayed =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"La lista de %u asignaturas es demasiado grande para mostrarla.";	// Necessita traduccio
#elif L==2	// de
	"The list of %u courses is too large to be displayed.";			// Need bersetzung
#elif L==3	// en
	"The list of %u courses is too large to be displayed.";
#elif L==4	// es
	"La lista de %u asignaturas es demasiado grande para mostrarla.";
#elif L==5	// fr
	"The list of %u courses is too large to be displayed.";			// Besoin de traduction
#elif L==6	// gn
	"La lista de %u asignaturas es demasiado grande para mostrarla.";	// Okoteve traduccin
#elif L==7	// it
	"La lista di %u corsi &egrave; troppo grande per essere mostrata.";
#elif L==8	// pl
	"The list of %u courses is too large to be displayed.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A lista de %u disciplinas &eacute; muito grande para ser exibida.";
#endif

const char *Txt_The_list_of_X_users_is_too_large_to_be_displayed =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"La lista de %u usuarios es demasiado grande para mostrarla.";	// Necessita traduccio
#elif L==2	// de
	"The list of %u users is too large to be displayed.";		// Need bersetzung
#elif L==3	// en
	"The list of %u users is too large to be displayed.";
#elif L==4	// es
	"La lista de %u usuarios es demasiado grande para mostrarla.";
#elif L==5	// fr
	"The list of %u users is too large to be displayed.";		// Besoin de traduction
#elif L==6	// gn
	"La lista de %u usuarios es demasiado grande para mostrarla.";	// Okoteve traduccin
#elif L==7	// it
	"La lista di %u utenti &egrave; troppo grande per essere mostrata.";
#elif L==8	// pl
	"The list of %u users is too large to be displayed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A lista de %u usu&aacute;rios &eacute; muito grande para ser exibida.";
#endif

const char *Txt_The_location_of_the_classroom_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La ubicaci&oacute; de l'aula <strong>%s</strong>"
	" ha canviat a <strong>%s</strong>.";
#elif L==2	// de
	"Der Standort des Klassenzimmer <strong>%s</strong>"
	" wurde in <strong>%s</strong> ge&auml;ndert.";
#elif L==3	// en
	"The location of the classroom <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==4	// es
	"La ubicaci&oacute;n del aula <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"L'emplacement de la salle de classe <strong>%s</strong>"
	" a &eacute;t&eacute; remplac&eacute; par <strong>%s</strong>.";
#elif L==6	// gn
	"La ubicaci&oacute;n del aula <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La posizione della aula <strong>%s</strong>"
	" &egrave; cambiata in <strong>%s</strong>.";
#elif L==8	// pl
	"Lokalizacja klasy <strong>%s</strong>"
	" zmieni&lstrok;a si&eogon; na <strong>%s</strong>.";
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o da sala de clase <strong>%s</strong>"
	" mudou para <strong>%s</strong>.";
#endif

const char *Txt_The_location_of_the_classroom_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La ubicaci&oacute; de l'aula <strong>%s</strong>"
	" no ha canviat.";
#elif L==2	// de
	"Der Standort des Klassenzimmer <strong>%s</strong>"
	" hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The location of the classroom <strong>%s</strong>"
	" has not changed.";
#elif L==4	// es
	"La ubicaci&oacute;n del aula <strong>%s</strong>"
	" no ha cambiado.";
#elif L==5	// fr
	"L'emplacement de la salle de classe <strong>%s</strong>"
	" n'a pas chang&eacute;.";
#elif L==6	// gn
	"La ubicaci&oacute;n del aula <strong>%s</strong>"
	" no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"La posizione della aula <strong>%s</strong>"
	" non &egrave; cambiata.";
#elif L==8	// pl
	"Lokalizacja klasy <strong>%s</strong>"
	" nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o da sala de clase <strong>%s</strong>"
	" n&atilde;o foi alterada.";
#endif

const char *Txt_The_lower_limit_of_correct_answers_must_be_less_than_or_equal_to_the_upper_limit =
#if   L==1	// ca
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";	// Necessita traduccio
#elif L==2	// de
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Need bersetzung
#elif L==3	// en
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";
#elif L==4	// es
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";
#elif L==5	// fr
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Besoin de traduction
#elif L==6	// gn
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";	// Okoteve traduccin
#elif L==7	// it
	"Il limite inferiore di risposte corrette"
	" deve essere minore o uguale al limite superiore.";
#elif L==8	// pl
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O limite inferior de respostas corretas"
	" deve ser menor ou igual ao limite superior.";
#endif

const char *Txt_the_marks_of_a_student_chosen_at_random_ =
#if   L==1	// ca
	"se mostrar&aacute;n las calificaciones de un estudiante de la asignatura escogido al azar;<br />"
	" si ve m&aacute;s de un estudiante, significa que los n&uacute;meros de filas de cabecera o de pie no son correctos";	// Necessita traduccio
#elif L==2	// de
	"es wird die Bewertung eines nach dem Zufallsprinzip ausgew&auml;ten Studenten angezeigt;<br />"
	" wenn Sie mehr als einen Student bedeutet dies, dass die Anzahl der Kopf oder Fu von Zeilen ist nicht korrekt";
#elif L==3	// en
	"the marks of a student chosen at random from the course will be shown;<br />"
	" if you see more than one student, it means that the number of header or footer rows is not correct";
#elif L==4	// es
	"se mostrar&aacute;n las calificaciones de un estudiante de la asignatura escogido al azar;<br />"
	" si ve m&aacute;s de un estudiante, significa que los n&uacute;meros de filas de cabecera o de pie no son correctos";
#elif L==5	// fr
	"les notes d'un &eacute;tudiant choisi au hasard dans la mati&egrave;re seront pr&eacute;sent&eacute;s;<br />"
	" si vous voyez plus d'un &eacute;tudiant, cela signifie que le nombre de ligned de la t&ecirc;te ou du pied n'est pas correcte";
#elif L==6	// gn
	"se mostrar&aacute;n las calificaciones de un estudiante de la asignatura escogido al azar;<br />"
	" si ve m&aacute;s de un estudiante, significa que los n&uacute;meros de filas de cabecera o de pie no son correctos";	// Okoteve traduccin
#elif L==7	// it
	"i risultati di uno studente scelto a caso dal corso verranno mostrati;<br />"
	" se si vede pi&uacute; di uno studente, significa che il numero di righe di intestazione o a pi&egrave; di pagina non &egrave; corretto";
#elif L==8	// pl
	"the marks of a student chosen at random from the course will be shown;<br />"
	" if you see more than one student, it means that the number of header or footer rows is not correct";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"vai mostrar as notas de um estudante da disciplina escolhido ao acaso;<br />"
	" se voc&ecirc; vir mais de um estudante, isso significa que o n&uacute;mero de cabe&ccedil;a ou p&eacute; linhas n&atilde;o est&aacute; correto";
#endif

const char *Txt_The_maximum_number_of_students_in_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre m&agrave;xim d'estudiants"
	" del grup <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Die maximale Anzahl der Studenten"
	" in Gruppe <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The maximum number of students"
	" in group <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le nombre maximum d'&eacute;tudiants"
	" du groupe <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il numero massimo di studenti"
	" nel gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"Maksymalna liczba uczni&oacute;w"
	" w grupie <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"O n&uacute;mero m&aacute;ximo de estudantes"
	" no grupo <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_maximum_number_of_students_in_group_X_is_now_Y =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1	// ca
	"El nombre m&agrave;xim d'estudiants"
	" del grup <strong>%s</strong> ara &eacute;s <strong>%u</strong>.";
#elif L==2	// de
	"Die maximale Anzahl der Studenten"
	" in Gruppe <strong>%s</strong> betr&auml;gt jetzt <strong>%u</strong>.";
#elif L==3	// en
	"The maximum number of students"
	" in group <strong>%s</strong> is now <strong>%u</strong>.";
#elif L==4	// es
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> ahora es <strong>%u</strong>.";
#elif L==5	// fr
	"Le nombre maximum d'&eacute;tudiants"
	" du groupe <strong>%s</strong> est maintenant de <strong>%u</strong>.";
#elif L==6	// gn
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> ahora es <strong>%u</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"Il numero massimo di studenti"
	" nel gruppo <strong>%s</strong> &egrave; ora <strong>%u</strong>.";
#elif L==8	// pl
	"Maksymalna liczba student&oacute;w"
	" w grupie <strong>%s</strong> wynosi teraz <strong>%u</strong>.";
#elif L==9	// pt
	"O n&uacute;mero m&aacute;ximo de estudantes"
	" no grupo <strong>%s</strong> &eacute; agora <strong>%u</strong>.";
#endif

const char *Txt_The_message_has_not_been_sent_to_any_recipient =
#if   L==1	// ca
	"El missatge no ha estat enviat a cap destinatari.";
#elif L==2	// de
	"The message has not been sent to any recipient.";		// Need bersetzung
#elif L==3	// en
	"The message has not been sent to any recipient.";
#elif L==4	// es
	"El mensaje no se ha enviado a ning&uacute;n destinatario.";
#elif L==5	// fr
	"The message has not been sent to any recipient.";		// Besoin de traduction
#elif L==6	// gn
	"El mensaje no se ha enviado a ning&uacute;n destinatario.";	// Okoteve traduccin
#elif L==7	// it
	"Il messaggio non &egrave; stato inviato a nessun destinatario.";
#elif L==8	// pl
	"The message has not been sent to any recipient.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A mensagem n&atilde;o foi enviada para nenhum destinat&aacute;rio.";
#endif

const char *Txt_The_message_has_been_sent_to_1_recipient =
#if   L==1	// ca
	"El mensaje se ha enviado a 1 destinatario.";	// Necessita traduccio
#elif L==2	// de
	"The message has been sent to 1 recipient.";	// Need bersetzung
#elif L==3	// en
	"The message has been sent to 1 recipient.";
#elif L==4	// es
	"El mensaje se ha enviado a 1 destinatario.";
#elif L==5	// fr
	"The message has been sent to 1 recipient.";	// Besoin de traduction
#elif L==6	// gn
	"El mensaje se ha enviado a 1 destinatario.";	// Okoteve traduccin
#elif L==7	// it
	"Il messaggio &egrave; stato inviato a 1 destinatario.";
#elif L==8	// pl
	"The message has been sent to 1 recipient.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A mensagem foi enviada para 1 destinat&aacute;rio.";
#endif

const char *Txt_The_message_has_been_sent_to_X_recipients =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"El mensaje se ha enviado a %u destinatarios.";	// Necessita traduccio
#elif L==2	// de
	"The message has been sent to %u recipients.";	// Need bersetzung
#elif L==3	// en
	"The message has been sent to %u recipients.";
#elif L==4	// es
	"El mensaje se ha enviado a %u destinatarios.";
#elif L==5	// fr
	"The message has been sent to %u recipients.";	// Besoin de traduction
#elif L==6	// gn
	"El mensaje se ha enviado a %u destinatarios.";	// Okoteve traduccin
#elif L==7	// it
	"Il messaggio &egrave; stato inviato a %u destinatari.";
#elif L==8	// pl
	"The message has been sent to %u recipients.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A mensagem foi enviada para %u destinat&aacute;rios.";
#endif

const char *Txt_The_name_of_the_banner_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre del banner <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the banner <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3	// en
	"The name of the banner <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del banner <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the banner <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre del banner <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del banner <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the banner <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do banner <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_centre_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre del centro <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the centre <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3	// en
	"The name of the centre <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del centro <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the centre <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre del centro <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del centro <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the centre <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do centro <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_classroom_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nom de l'aula <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Der Name des Klassenzimmer <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The name of the classroom <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del aula <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le nom de la salle de classe <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El nombre del aula <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome  della aula <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"Nazwa klasy <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"O nome da sala de clase <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_country_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre del pa&iacute;s <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the country <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3	// en
	"The name of the country <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del pa&iacute;s <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the country <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El nombre del pa&iacute;s <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del paese <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the country <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do pa&iacute;s <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_course_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El nombre de la asignatura <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong> .";	// Need bersetzung
#elif L==3	// en
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4	// es
	"El nombre de la asignatura <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre de la asignatura <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del corso <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==8	// pl
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome da disciplina <strong>%s</strong> mudou para <strong>%s</strong>.";
#endif

const char *Txt_The_name_of_the_course_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre de la asignatura <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the course <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3	// en
	"The name of the course <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre de la asignatura <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the course <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El nombre de la asignatura <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del corso <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the course <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome da disciplina <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_degree_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El nombre de la titulaci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4	// es
	"El nombre de la titulaci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"El nombre de la titulaci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome della laurea <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==8	// pl
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do grau <strong>%s</strong> mudou para <strong>%s</strong>.";
#endif

const char *Txt_The_name_of_the_degree_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre de la titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the degree <strong>%s</strong> has not changed.";			// Need bersetzung
#elif L==3	// en
	"The name of the degree <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre de la titulaci&oacute;n <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the degree <strong>%s</strong> has not changed.";			// Besoin de traduction
#elif L==6	// gn
	"El nombre de la titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome della laurea <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the degree <strong>%s</strong> has not changed.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do grau <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_department_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre del departamento <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the department <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3	// en
	"The name of the department <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del departamento <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the department <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre del departamento <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del dipartimento <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the department <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do departamento <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre del campo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the field <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3	// en
	"The name of the field <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del campo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the field <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre del campo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del campo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the field <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do campo <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre del grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3	// en
	"The name of the group <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del grupo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre del grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do grupo <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El nombre de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4	// es
	"El nombre de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome della festivit&agrave; <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==8	// pl
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do feriado <strong>%s</strong> foi alterado para <strong>%s</strong>.";
#endif

const char *Txt_The_name_of_the_holiday_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre de la festividad <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the holiday <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3	// en
	"The name of the holiday <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre de la festividad <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the holiday <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El nombre de la festividad <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome della festivit&agrave; <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the holiday <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do feriado <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_institution_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre de la instituci&oacute;n <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the institution <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3	// en
	"The name of the institution <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre de la instituci&oacute;n <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the institution <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El nombre de la instituci&oacute;n <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome dell'istituzione <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the institution <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome da institui&ccedil;&atilde;o <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_link_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre del enlace <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the link <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3	// en
	"The name of the link <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del enlace <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the link <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre del enlace <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del link <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the link <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do link <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_place_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nom del lloc <strong>%s</strong> no ha canviat.";
#elif L==2	// de
	"Der Name von Standort <strong>%s</strong> hat sich nicht ge&auml;ndert.";
#elif L==3	// en
	"The name of the place <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del lugar <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"Le nom de l'emplacement <strong>%s</strong> n'a pas chang&eacute;.";
#elif L==6	// gn
	"El nombre del lugar <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del luogo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"Nazwa miejsca <strong>%s</strong> nie uleg&lstrok;a zmianie.";
#elif L==9	// pt
	"O nome da localiza&ccedil;&atilde;o <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_plugin_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre del complemento <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the plugin <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3	// en
	"The name of the plugin <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del complemento <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the plugin <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El nombre del complemento <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del plugin <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the plugin <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do plugin <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_type_of_degree_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre del tipo de titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3	// en
	"The name of the type of degree <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del tipo de titulaci&oacute;n <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El nombre del tipo de titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del tipo di laurea <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do tipo de grau <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_name_of_the_type_of_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nombre del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The name of the type of group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3	// en
	"The name of the type of group <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El nombre del tipo de grupo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The name of the type of group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6	// gn
	"El nombre del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome del tipo di gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The name of the type of group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O nome do tipo de grupo <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_new_IP_address_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The new IP address is <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"The new IP address is <strong>%s</strong>.";
#elif L==4	// es
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";
#elif L==5	// fr
	"The new IP address is <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nuovo indirizzo IP &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"The new IP address is <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O novo endere&ccedil;o IP &eacute; <strong>%s</strong>.";
#endif

const char *Txt_The_new_logo_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nuevo logo es <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The new logo is <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"The new logo is <strong>%s</strong>.";
#elif L==4	// es
	"El nuevo logo es <strong>%s</strong>.";
#elif L==5	// fr
	"The new logo is <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"El nuevo logo es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nuovo logo &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"The new logo is <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O novo logotipo &eacute; <strong>%s</strong>.";
#endif

const char *Txt_The_nickname_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El apodo <strong>@%s</strong>"
	" se ha registrado correctamente.";		// Necessita traduccio
#elif L==2	// de
	"The nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Need bersetzung
#elif L==3	// en
	"The nickname <strong>@%s</strong>"
	" has been registered successfully.";
#elif L==4	// es
	"El apodo <strong>@%s</strong>"
	" se ha registrado correctamente.";
#elif L==5	// fr
	"The nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Besoin de traduction
#elif L==6	// gn
	"El apodo <strong>@%s</strong>"
	" se ha registrado correctamente.";		// Okoteve traduccin
#elif L==7	// it
	"Il nome utente <strong>@%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==8	// pl
	"The nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A alcunha <strong>@%s</strong>"
	" foi registrada com sucesso.";
#endif

const char *Txt_The_nickname_entered_X_is_not_valid_ =	// Warning: it is very important to include %s and two %u in the following sentences
#if   L==1	// ca
	"El apodo <strong>%s</strong> no es v&aacute;lido.<br />"
	"El apodo debe tener una longitud entre %u y %u caracteres (sin contar la @ inicial).<br />"
	"Cada car&aacute;cter posterior a la @ inicial puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";	// Necessita traduccio
#elif L==2	// de
	"The nickname <strong>%s</strong> is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Need bersetzung
#elif L==3	// en
	"The nickname <strong>%s</strong> is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";
#elif L==4	// es
	"El apodo <strong>%s</strong> no es v&aacute;lido.<br />"
	"El apodo debe tener una longitud entre %u y %u caracteres (sin contar la @ inicial).<br />"
	"Cada car&aacute;cter posterior a la @ inicial puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";
#elif L==5	// fr
	"The nickname <strong>%s</strong> is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including initial the @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Besoin de traduction
#elif L==6	// gn
	"El apodo <strong>%s</strong> no es v&aacute;lido.<br />"
	"El apodo debe tener una longitud entre %u y %u caracteres (sin contar la @ inicial).<br />"
	"Cada car&aacute;cter posterior a la @ inicial puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome utente <strong>%s</strong> non &egrave; valido.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br /> "
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Bisogno di traduzione
#elif L==8	// pl
	"The nickname <strong>%s</strong> is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A alcunha <strong>%s</strong> n&atilde;o &eacute; v&aacute;lido.<br />"
	"Sua alcunha deve ter um tamanho entre %u e %u caracteres (sem incluir o inicial @).<br />"
	"Cada caractere ap&oacute;s inicial @ pode ser uma letra (a-z, A-Z),"
	" um dgito (0-9) ou &quot;_&quot;.";
#endif

const char *Txt_The_notice_will_appear_as_a_yellow_note_ =
#if   L==1	// ca
	"L'av&iacute;s apareixer&agrave; en forma de nota groga"
	" a l'esquerra de la p&agrave;gina"
	" i ser&agrave; <strong>visible per tot el m&oacute;n</strong>"
	" (no nom&eacute;s pels usuaris de l'assignatura).<br />"
	"<strong>El missatge ha de ser breu.</strong>"
	" Per crear un av&iacute;s extens"
	" o d'acc&eacute;s exclusiu per a estudiants,"
	" pot utilitzar el f&ograve;rum de l'assignatura,"
	" enviar un missatge a tots,"
	" o pujar un arxiu amb el text a la zona de documents.";
#elif L==2	// de
	"Die Ank&uuml;ndigung wird als ein gelber Hinweis"
	" auf der linken Seite der Seite erscheinen"
	" und wird <strong>weltweit</strong>"
	" (nicht nur von den Benutzern des Kurses) sichtbar sein.<br />"
	"<strong>Die Nachricht sollte kurz sein</strong>."
	" Wenn Sie eine lange Ank&uuml;ndigung"
	" oder einen Hinweis mit exklusivem Zugang f&uuml;r Studenten erstellen m&ouml;chten,"
	" k&ouml;nnen Sie das Forum des Kurses nutzen,"
	" eine Nachricht an alle senden"
	" oder eine Datei mit dem Text in den Dokumentenbereich hochladen.";
#elif L==3	// en
	"The notice will appear as a yellow note"
	" in the left of the page"
	" and will be <strong>visible worldwide</strong>"
	" (not just by users of the course).<br />"
	"<strong>The message should be short</strong>."
	" If you want to create a long notice"
	" or a notice with exclusive access for students,"
	" you can use the forum of the course,"
	" send a message to all,"
	" or upload a file with the text to the documents area.";
#elif L==4	// es
	"El aviso aparecer&aacute; en forma de nota amarilla"
	" a la izquierda de la p&aacute;gina"
	" y ser&aacute; <strong>visible por todo el mundo</strong>"
	" (no solo por los usuarios de la asignatura).<br />"
	"<strong>El mensaje debe ser breve</strong>."
	" Si desea crear un aviso extenso"
	" o de acceso exclusivo para estudiantes,"
	" puede usar el foro de la asignatura,"
	" enviar un mensaje a todos,"
	" o subir un archivo con el texto a la zona de documentos.";
#elif L==5	// fr
	"L'avis appara&icirc;tra comme une note jaune"
	" &agrave; gauche de la page"
	" et sera <strong>visible dans le monde entier</strong>"
	" (pas seulement par les utilisateurs du cours).<br />"
	"<strong>Le message doit &ecirc;tre court</strong>."
	" Si vous souhaitez cr&eacute;er un avis long"
	" ou un avis avec un acc&egrave;s exclusif pour les &eacute;tudiants,"
	" vous pouvez utiliser le forum du cours,"
	" envoyer un message &agrave; tous"
	" ou t&eacute;l&eacute;charger un fichier avec le texte dans la zone des documents.";
#elif L==6	// gn
	"El aviso aparecer&aacute; en forma de nota amarilla"
	" a la izquierda de la p&aacute;gina"
	" y ser&aacute; <strong>visible por todo el mundo</strong>"
	" (no solo por los usuarios de la asignatura).<br />"
	"<strong>El mensaje debe ser breve</strong>."
	" Si desea crear un aviso extenso"
	" o de acceso exclusivo para estudiantes,"
	" puede usar el foro de la asignatura,"
	" enviar un mensaje a todos,"
	" o subir un archivo con el texto a la zona de documentos.";	// Okoteve traduccin
#elif L==7	// it
	"L'avviso verr&agrave; visualizzato come una nota di colore giallo"
	" nella parte sinistra della pagina"
	" e sar&agrave; <strong>visibile in tutto il mondo</strong>"
	" (non solo dagli utenti del corso).<br />"
	"<strong>Il messaggio deve essere breve</strong>."
	" Se si desidera creare un lungo avviso"
	" o una comunicazione con accesso esclusivo per gli studenti,"
	" &egrave; possibile utilizzare il forum del corso,"
	" inviare un messaggio a tutti,"
	" o caricare un file con il testo all'area documenti.";
#elif L==8	// pl
	"Og&lstrok;oszenie pojawi si&eogon; jako &zdot;&oacute;&lstrok;ty notatki"
	" w lewej strony"
	" i b&eogon;dzie <strong>widoczny na ca&lstrok;ym &sacute;wiecie</strong>"
	" (nie tylko przez u&zdot;ytkownik&oacute;w oczywi&sacute;cie).<br />"
	"<strong>Wiadomo&sacute;&cacute; powinna by&cacute; kr&oacute;tka</strong>."
	" Je&sacute;li chcesz stworzy&cacute; d&lstrok;ug&aogon; zawiadomienie"
	" lub zawiadomienie z wy&lstrok;&aogon;cznego dost&eogon;pu dla student&oacute;w,"
	" mo&zdot;na korzysta&cacute; z forum kursu,"
	" wys&lstrok;a&cacute; wiadomo&sacute;&cacute; do wszystkich,"
	" lub przes&lstrok;a&cacute; plik z tekstem do obszaru dokument&oacute;w.";
#elif L==9	// pt
	"O aviso aparecer&aacute; como uma nota amarela"
	" &agrave; esquerda da p&aacute;gina"
	" e ser&aacute; <strong>vis&iacute;vel em todo o mundo</strong>"
	" (n&atilde;o apenas pelos usu&aacute;rios do curso).<br />"
	"<strong>A mensagem deve ser curta</strong>."
	" Se voc&ecirc; quiser criar um aviso longo"
	" ou um aviso com acesso exclusivo para os estudantes,"
	" voc&ecirc; pode usar o f&oacute;rum do curso,"
	" enviar uma mensagem para todos"
	" ou enviar um arquivo com o texto para a &aacute;rea de documentos.";
#endif

const char *Txt_The_new_web_address_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The new web address is <strong>%s</strong>.";			// Need bersetzung
#elif L==3	// en
	"The new web address is <strong>%s</strong>.";
#elif L==4	// es
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";
#elif L==5	// fr
	"The new web address is <strong>%s</strong>.";			// Besoin de traduction
#elif L==6	// gn
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il nuovo indirizzo web &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"The new web address is <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O novo endere&ccedil;o web &eacute; <strong>%s</strong>.";
#endif

const char *Txt_The_new_description_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La nueva descripci&oacute;n web es <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The new description is <strong>%s</strong>.";			// Need bersetzung
#elif L==3	// en
	"The new description is <strong>%s</strong>.";
#elif L==4	// es
	"La nueva descripci&oacute;n web es <strong>%s</strong>.";
#elif L==5	// fr
	"The new description is <strong>%s</strong>.";			// Besoin de traduction
#elif L==6	// gn
	"La nueva descripci&oacute;n web es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La nuova descrizione &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"The new description is <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A nova descri&ccedil;&atilde;o &eacute; <strong>%s</strong>.";
#endif

const char *Txt_The_new_image_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La nueva imagen es <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The new image is <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"The new image is <strong>%s</strong>.";
#elif L==4	// es
	"La nueva imagen es <strong>%s</strong>.";
#elif L==5	// fr
	"The new image is <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"La nueva imagen es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"La nuova image &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"The new image is <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A nova imagem &eacute; <strong>%s</strong>.";
#endif

const char *Txt_The_new_URL_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El nuevo URL es <strong>%s</strong>.";		// Necessita traduccio
#elif L==2	// de
	"The new URL is <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"The new URL is <strong>%s</strong>.";
#elif L==4	// es
	"El nuevo URL es <strong>%s</strong>.";
#elif L==5	// fr
	"The new URL is <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"El nuevo URL es <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"La nuova URL &egrave; <strong>%s</strong>.";
#elif L==8	// pl
	"The new URL is <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O novo URL &eacute; <strong>%s</strong>.";
#endif

const char *Txt_The_nickname_X_had_been_registered_by_another_user =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El apodo <strong>@%s</strong>"
	" ya hab&iacute;a sido registrado por otro usuario.";	// Necessita traduccio
#elif L==2	// de
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";		// Need bersetzung
#elif L==3	// en
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";
#elif L==4	// es
	"El apodo <strong>@%s</strong>"
	" ya hab&iacute;a sido registrado por otro usuario.";
#elif L==5	// fr
	"The nickname <strong>@%s</strong>"
	" had been enroled by another user.";			// Besoin de traduction
#elif L==6	// gn
	"El apodo <strong>@%s</strong>"
	" ya hab&iacute;a sido registrado por otro usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome utente <strong>@%s</strong>"
	" &egrave; stato registrato da un altro utente.";
#elif L==8	// pl
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A alcunha <strong>@%s</strong>"
	" foi registrada por outro usu&aacute;rio.";
#endif

const char *Txt_The_nickname_X_matches_the_one_you_had_previously_registered =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El apodo <strong>@%s</strong>"
	" coincide con el que ya ten&iacute;a registrado previamente.";	// Necessita traduccio
#elif L==2	// de
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Need bersetzung
#elif L==3	// en
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";
#elif L==4	// es
	"El apodo <strong>@%s</strong>"
	" coincide con el que ya ten&iacute;a registrado previamente.";
#elif L==5	// fr
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Besoin de traduction
#elif L==6	// gn
	"El apodo <strong>@%s</strong>"
	" coincide con el que ya ten&iacute;a registrado previamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il nome utente <strong>@%s</strong>"
	" coincide con quello che hai precedentemente registrato.";
#elif L==8	// pl
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A alcunha <strong>@%s</strong>"
	" corresponde &agrave;quela que voc&ecirc; registrou anteriormente.";
#endif

const char *Txt_The_number_of_editing_lines_in_the_record_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El n&ordm; de l&iacute;neas"
	" de edici&oacute;n del campo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The number of editing lines"
	" in the record field <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3	// en
	"The number of editing lines"
	" in the record field <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El n&ordm; de l&iacute;neas de edici&oacute;n"
	" del campo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The number of editing lines"
	" in the record field <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El n&ordm; de l&iacute;neas de edici&oacute;n"
	" del campo <strong>%s</strong> no ha cambiado.";			// Okoteve traduccin
#elif L==7	// it
	"Il numero di linee editate"
	" nel campo scheda <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The number of editing lines"
	" in the record field <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O n&uacute;mero de linhas de edi&ccedil;&atilde;o"
	" no campo de cart&atilde;o <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_number_of_questions_must_be_in_the_interval_X =	// Warning: it is very important to include two %u in the following sentences
#if   L==1	// ca
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";	// Necessita traduccio
#elif L==2	// de
	"Die Anzahl der Fragen muss sich im Intervall [%u,%u] befinden.";
#elif L==3	// en
	"The number of questions must be in the interval [%u,%u].";
#elif L==4	// es
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";
#elif L==5	// fr
	"The number of questions must be in the interval [%u,%u].";		// Besoin de traduction
#elif L==6	// gn
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";	// Okoteve traduccin
#elif L==7	// it
	"Il numero di domande dovrebbe stare nell'intervallo [%u,%u].";
#elif L==8	// pl
	"The number of questions must be in the interval [%u,%u].";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O n&uacute;mero de perguntas deve estar no intervalo [%u,%u].";
#endif

const char *Txt_The_number_of_rows_is_now_X =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"El n&uacute;mero de filas ahora es <strong>%u</strong>.";	// Necessita traduccio
#elif L==2	// de
	"Die Anzahl der Zeilen ist jetzt <strong>%u</strong>.";
#elif L==3	// en
	"The number of rows is now <strong>%u</strong>.";
#elif L==4	// es
	"El n&uacute;mero de filas ahora es <strong>%u</strong>.";
#elif L==5	// fr
	"The number of rows is now <strong>%u</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"El n&uacute;mero de filas ahora es <strong>%u</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il numero di righe &egrave; ora <strong>%u</strong>.";
#elif L==8	// pl
	"The number of rows is now <strong>%u</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O n&uacute;mero de linhas agora &eacute; <strong>%u</strong>.";
#endif

const char *Txt_The_numerical_code_X_already_exists =	// Warning: it is very important to include %03lu in the following sentences
#if   L==1	// ca
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";	// Necessita traduccio
#elif L==2	// de
	"The ISO 3166-1 numerical code %03ld already exists.";		// Need bersetzung
#elif L==3	// en
	"The ISO 3166-1 numerical code %03ld already exists.";
#elif L==4	// es
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";
#elif L==5	// fr
	"The ISO 3166-1 numerical code %03ld already exists.";		// Besoin de traduction
#elif L==6	// gn
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il codice numerico ISO 3166-1 %03ld gi&agrave; esiste.";
#elif L==8	// pl
	"The ISO 3166-1 numerical code %03ld already exists.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O c&oacute;digo num&eacute;rico %03ld da ISO 3166-1 j&aacute; existe.";
#endif

const char *Txt_The_photo_no_longer_exists =
#if   L==1	// ca
	"La foto ja no existeix.";
#elif L==2	// de
	"Das Foto nicht mehr existiert.";
#elif L==3	// en
	"The photo no longer exists.";
#elif L==4	// es
	"La foto ya no existe.";
#elif L==5	// fr
	"La photo a disparu.";
#elif L==6	// gn
	"La foto ya no existe.";	// Okoteve traduccin
#elif L==7	// it
	"La foto non esiste pi&ugrave;.";
#elif L==8	// pl
	"Zdj&eogon;cie ju&zdot; nie istnieje.";
#elif L==9	// pt
	"La foto n&atilde;o existe mais.";
#endif

const char *Txt_The_original_post_no_longer_exists =
#if   L==1	// ca
	"La publicaci&oacute; original ja no existeix.";
#elif L==2	// de
	"Der urspr&uuml;ngliche Post nicht mehr existiert.";
#elif L==3	// en
	"The original post no longer exists.";
#elif L==4	// es
	"La publicaci&oacute;n original ya no existe.";
#elif L==5	// fr
	"Le post original a disparu.";
#elif L==6	// gn
	"La publicaci&oacute;n original ya no existe.";	// Okoteve traduccin
#elif L==7	// it
	"La pubblicazione originale non esiste pi&ugrave;.";
#elif L==8	// pl
	"Oryginalna publikacja ju&zdot; nie istnieje.";
#elif L==9	// pt
	"A publica&ccedil;&atilde;o original n&atilde;o existe mais.";
#endif

const char *Txt_The_password_can_not_consist_only_of_digits =
#if   L==1	// ca
	"La contrase&ntilde;a no puede estar formada"
	" solo por d&iacute;gitos.";		// Necessita traduccio
#elif L==2	// de
	"The password can not consist"
	" only of digits.";			// Need bersetzung
#elif L==3	// en
	"The password can not consist"
	" only of digits.";
#elif L==4	// es
	"La contrase&ntilde;a no puede estar formada"
	" solo por d&iacute;gitos.";
#elif L==5	// fr
	"The password can not consist"
	" only of digits.";			// Besoin de traduction
#elif L==6	// gn
	"La contrase&ntilde;a no puede estar formada"
	" solo por d&iacute;gitos.";		// Okoteve traduccin
#elif L==7	// it
	"La password non pu&ograve; consistere"
	" solo di numeri.";
#elif L==8	// pl
	"The password can not consist"
	" only of digits.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A senha n&atilde;o pode consistir"
	" apenas em d&iacute;gitos.";
#endif

const char *Txt_The_password_can_not_contain_spaces =
#if   L==1	// ca
	"La contrase&ntilde;a no puede tener espacios.";	// Necessita traduccio
#elif L==2	// de
	"The password can not contain spaces.";			// Need bersetzung
#elif L==3	// en
	"The password can not contain spaces.";
#elif L==4	// es
	"La contrase&ntilde;a no puede tener espacios.";
#elif L==5	// fr
	"The password can not contain spaces.";			// Besoin de traduction
#elif L==6	// gn
	"La contrase&ntilde;a no puede tener espacios.";	// Okoteve traduccin
#elif L==7	// it
	"La password non pu&ograve; contenere spazi.";
#elif L==8	// pl
	"The password can not contain spaces.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A senha n&atilde;o pode conter espa&ccedil;os.";
#endif

const char *Txt_The_password_is_too_trivial_ =
#if   L==1	// ca
	"La contrase&ntilde;a es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";	// Necessita traduccio
#elif L==2	// de
	"The password is too trivial."
	" Use other harder to guess.";				// Need bersetzung
#elif L==3	// en
	"The password is too trivial."
	" Use other harder to guess.";
#elif L==4	// es
	"La contrase&ntilde;a es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";
#elif L==5	// fr
	"The password is too trivial."
	" Use other harder to guess.";				// Besoin de traduction
#elif L==6	// gn
	"La contrase&ntilde;a es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";	// Okoteve traduccin
#elif L==7	// it
	"La password &egrave; troppo semplice."
	" Utilizza un'altra pi&ugrave; difficile da individuare.";
#elif L==8	// pl
	"The password is too trivial."
	" Use other harder to guess.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A senha &eacute; muito trivial."
	" Use outro mais difcil de adivinhar.";
#endif

const char *Txt_The_password_must_be_at_least_X_characters =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"La contrase&ntilde;a ha de tener al menos %u caracteres.";	// Necessita traduccio
#elif L==2	// de
	"The password must be at least %u characters.";			// Need bersetzung
#elif L==3	// en
	"The password must be at least %u characters.";
#elif L==4	// es
	"La contrase&ntilde;a ha de tener al menos %u caracteres.";
#elif L==5	// fr
	"The password must be at least %u characters.";			// Besoin de traduction
#elif L==6	// gn
	"La contrase&ntilde;a ha de tener al menos %u caracteres.";	// Okoteve traduccin
#elif L==7	// it
	"La password dovrebbe essere di almeno %u caratteri.";
#elif L==8	// pl
	"The password must be at least %u characters.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A senha deve ter pelo menos %u caracteres.";
#endif

const char *Txt_The_password_has_been_changed_successfully =
#if   L==1	// ca
	"La contrase&ntilde;a se ha modificado con &eacute;xito.";	// Necessita traduccio
#elif L==2	// de
	"The password has been changed successfully.";			// Need bersetzung
#elif L==3	// en
	"The password has been changed successfully.";
#elif L==4	// es
	"La contrase&ntilde;a se ha modificado con &eacute;xito.";
#elif L==5	// fr
	"The %password has been changed successfully.";			// Besoin de traduction
#elif L==6	// gn
	"La contrase&ntilde;a se ha modificado con &eacute;xito.";	// Okoteve traduccin
#elif L==7	// it
	"La password &egrave; stata cambiata con successo.";
#elif L==8	// pl
	"The password has been changed successfully.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A senha foi alterada com sucesso.";
#endif

const char *Txt_The_place_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El lloc <strong>%s</strong> ja existeix.";
#elif L==2	// de
	"Der Standort <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3	// en
	"The place <strong>%s</strong> already exists.";
#elif L==4	// es
	"El lugar <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"L'emplacement <strong>%s</strong> existe d&eacute;j&agrave;.";
#elif L==6	// gn
	"El lugar <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"Il luogo <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"Miejsce <strong>%s</strong> ju&zdot; istnieje.";
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_place_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El lloc <strong>%s</strong> ha passat a denominar-se <strong>%s</strong>.";
#elif L==2	// de
	"Der Standort <strong>%s</strong> wurde in <strong>%s</strong> umbenannt.";
#elif L==3	// en
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El lugar <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"L'emplacement <strong>%s</strong> a &eacute;t&eacute; renomm&eacute; en <strong>%s</strong>.";
#elif L==6	// gn
	"El lugar <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il luogo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"Miejsce <strong>%s</strong> zosta&lstrok;o przemianowane na <strong>%s</strong>.";
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o <strong>%s</strong> foi renomeada como <strong>%s</strong>.";
#endif

const char *Txt_The_place_of_the_centre_has_changed =
#if   L==1	// ca
	"El lugar del centro ha cambiado.";		// Necessita traduccio
#elif L==2	// de
	"The place of the centre has changed.";		// Need bersetzung
#elif L==3	// en
	"The place of the centre has changed.";
#elif L==4	// es
	"El lugar del centro ha cambiado.";
#elif L==5	// fr
	"The place of the centre has changed.";		// Besoin de traduction
#elif L==6	// gn
	"El lugar del centro ha cambiado.";		// Okoteve traduccin
#elif L==7	// it
	"Il luogo del centro &egrave; cambiato.";
#elif L==8	// pl
	"The place of the centre has changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o do centro mudou.";
#endif

const char *Txt_The_place_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4	// es
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5	// fr
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il luogo della festivit&agrave; <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==8	// pl
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A localiza&ccedil;&atilde;o do feriado <strong>%s</strong> mudou para <strong>%s</strong>.";
#endif

const char *Txt_The_plugin_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El complemento <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2	// de
	"Der Plugin <strong>%s</strong> wurde bereits installiert.";
#elif L==3	// en
	"The plugin <strong>%s</strong> already exists.";
#elif L==4	// es
	"El complemento <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The plugin <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El complemento <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il plugin <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The plugin <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O plugin <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_plugin_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il plugin <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O plugin <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_project_has_been_modified =
#if   L==1	// ca
	"El projecte ha estat modificada.";
#elif L==2	// de
	"Die Projekt wurde ge&auml;ndert.";
#elif L==3	// en
	"The project has been modified.";
#elif L==4	// es
	"El proyecto ha sido modificado.";
#elif L==5	// fr
	"Le projet a &eacute;t&eacute; modifi&eacute;.";
#elif L==6	// gn
	"El proyecto ha sido modificado.";	// Okoteve traduccin
#elif L==7	// it
	"Il progetto &egrave; stato modificato.";
#elif L==8	// pl
	"Projekt zosta&lstrok; zmodyfikowany.";
#elif L==9	// pt
	"O projeto foi modificado.";
#endif

const char *Txt_The_properties_of_file_X_have_been_saved =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Las propiedades del archivo %s se han guardado.";	// Necessita traduccio
#elif L==2	// de
	"The properties of file %s have been saved.";		// Need bersetzung
#elif L==3	// en
	"The properties of file %s have been saved.";
#elif L==4	// es
	"Las propiedades del archivo %s se han guardado.";
#elif L==5	// fr
	"The properties of file %s have been saved.";		// Besoin de traduction
#elif L==6	// gn
	"Las propiedades del archivo %s se han guardado.";	// Okoteve traduccin
#elif L==7	// it
	"The properties of file %s have been saved.";		// Bisogno di traduzione
#elif L==8	// pl
	"The properties of file %s have been saved.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"As propriedades do arquivo %s foram salvas.";
#endif

const char *Txt_The_question_has_been_moved_down =
#if   L==1	// ca
	"La pregunta ha estat moguda cap avall.";
#elif L==2	// de
	"Die Frage wurde nach unten bewegt.";
#elif L==3	// en
	"The question has been moved down.";
#elif L==4	// es
	"La pregunta se ha movido hacia abajo.";
#elif L==5	// fr
	"La question a &eacute;t&eacute; d&eacute;plac&eacute;e vers le bas.";
#elif L==6	// gn
	"La pregunta se ha movido hacia arriba.";	// Okoteve traduccin
#elif L==7	// it
	"La questione &egrave; stata abbattuta.";
#elif L==8	// pl
	"Pytanie zosta&lstrok;o przeniesione.";
#elif L==9	// pt
	"A quest&atilde;o foi movida para baixo.";
#endif

const char *Txt_The_question_has_been_moved_up =
#if   L==1	// ca
	"La pregunta ha estat moguda cap amunt.";
#elif L==2	// de
	"Die Frage wurde nach oben verschoben.";
#elif L==3	// en
	"The question has been moved up.";
#elif L==4	// es
	"La pregunta se ha movido hacia arriba.";
#elif L==5	// fr
	"La question a &eacute;t&eacute; d&eacute;plac&eacute;e vers le haut.";
#elif L==6	// gn
	"La pregunta se ha movido hacia arriba.";	// Okoteve traduccin
#elif L==7	// it
	"La questione &egrave; stata spostata.";
#elif L==8	// pl
	"Pytanie zosta&lstrok;o poruszone.";
#elif L==9	// pt
	"A quest&atilde;o foi movida para cima.";
#endif

const char *Txt_The_record_field_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El campo de ficha <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2	// de
	"Das Datenfeld <strong>%s</strong> wurde bereits erstellt.";
#elif L==3	// en
	"The record field <strong>%s</strong> already exists.";
#elif L==4	// es
	"El campo de ficha <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The record field <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El campo de ficha <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il campo di scheda <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The record field <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O campo de cart&atilde;o <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_record_field_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il campo di scheda <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O campo de cart&atilde;o <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_requested_group_changes_were_successful =
#if   L==1	// ca
	"Els canvis de grup sol&middot;licitats s'han realitzat correctament.";
#elif L==2	// de
	"The requested group changes were successful.";				// Need bersetzung
#elif L==3	// en
	"The requested group changes were successful.";
#elif L==4	// es
	"Los cambios de grupo solicitados se han realizado correctamente.";
#elif L==5	// fr
	"Les changements de groupe demand&eacute;es ont r&eacute;ussi.";
#elif L==6	// gn
	"Los cambios de grupo solicitados se han realizado correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"I cambiamenti di gruppi richieste hanno avuto successo.";
#elif L==8	// pl
	"The requested group changes were successful.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"As altera&ccedil;&otilde;es do grupo solicitadas foram bem sucedidas.";
#endif

const char *Txt_The_role_of_THE_USER_X_in_the_course_Y_has_changed_from_A_to_B =	// Warning: it is very important to include four %s in the following sentences
#if   L==1	// ca
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";		// Necessita traduccio
#elif L==2	// de
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Need bersetzung
#elif L==3	// en
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";
#elif L==4	// es
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";
#elif L==5	// fr
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Besoin de traduction
#elif L==6	// gn
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";		// Okoteve traduccin
#elif L==7	// it
	"Il ruolo de <strong>%s</strong>"
	" nel corso <strong>%s</strong>"
	" &egrave; cambiato da %s a %s.";
#elif L==8	// pl
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O papel de <strong>%s</strong>"
	" na disciplina <strong>%s</strong>"
	" mudou de %s para %s.";
#endif

const char *Txt_The_X_students_who_belonged_to_the_course_Y_have_been_removed_from_it =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==1	// ca
	"Se han eliminado los %u estudiantes que hab&iacute;a"
	" en la asignatura <strong>%s</strong>.";				// Necessita traduccio
#elif L==2	// de
	"The %u students who belonged"
	" to the course <strong>%s</strong> have been removed from it.";	// Need bersetzung
#elif L==3	// en
	"The %u students who belonged"
	" to the course <strong>%s</strong> have been removed from it.";
#elif L==4	// es
	"Se han eliminado los %u estudiantes que hab&iacute;a"
	" en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"The %u students who belonged"
	" to the course <strong>%s</strong> have been removed from it.";	// Besoin de traduction
#elif L==6	// gn
	"Se han eliminado los %u estudiantes que hab&iacute;a"
	" en la asignatura <strong>%s</strong>.";				// Okoteve traduccin
#elif L==7	// it
	"Gli %u studenti che appartengono"
	" al corso <strong>%s</strong> sono stati rimossi da esso.";
#elif L==8	// pl
	"The %u students who belonged"
	" to the course <strong>%s</strong> have been removed from it.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Os %u estudantes que pertenceram"
	" &agrave; disciplina <strong>%s</strong> foram removidos dela.";
#endif

const char *Txt_The_search_text_must_be_longer =
#if   L==1	// ca
	"El text a cercar ha de ser m&eacute;s llarg.";
#elif L==2	// de
	"Der Suchtext muss l&auml;nger sein.";
#elif L==3	// en
	"The search text must be longer.";
#elif L==4	// es
	"El texto a buscar debe ser m&aacute;s largo.";
#elif L==5	// fr
	"Le texte de recherche doit &ecirc;tre plus longue.";
#elif L==6	// gn
	"El texto a buscar debe ser m&aacute;s largo.";	// Okoteve traduccin
#elif L==7	// it
	"Il testo di ricerca deve essere pi&ugrave; lungo.";
#elif L==8	// pl
	"Tekst wyszukiwania powinien by&cacute; d&lstrok;u&zdot;szy.";
#elif L==9	// pt
	"O texto de pesquisa deve ser mais longo.";
#endif

const char *Txt_The_session_has_expired =
#if   L==1	// ca
	"La sessi&oacute; ha expirat.";
#elif L==2	// de
	"Die Sitzung wurde.";
#elif L==3	// en
	"The session has expired.";
#elif L==4	// es
	"La sesi&oacute;n ha expirado.";
#elif L==5	// fr
	"La session a expir&eacute;.";
#elif L==6	// gn
	"La sesi&oacute;n ha expirado.";	// Okoteve traduccin
#elif L==7	// it
	"La sessione &egrave; scaduta.";
#elif L==8	// pl
	"Sesja wygas&lstrok;a.";
#elif L==9	// pt
	"A sess&atilde;o expirou.";
#endif

const char *Txt_The_size_of_the_file_exceeds_the_maximum_allowed_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La mida del fitxer"
	" supera el m&agrave;xim perm&egrave;s (%s).";
#elif L==2	// de
	"Die Gr&ouml;&szlig;e der Datei"
	" &uuml;berschreitet den zul&auml;ssigen H&ouml;chstwert (%s).";
#elif L==3	// en
	"The size of the file"
	" exceeds the maximum allowed (%s).";
#elif L==4	// es
	"El tama&ntilde;o del archivo"
	" excede el m&aacute;ximo permitido (%s).";
#elif L==5	// fr
	"La taille du fichier"
	" d&eacute;passe le maximum autoris&eacute; (%s).";
#elif L==6	// gn
	"El tama&ntilde;o del archivo"
	" excede el m&aacute;ximo permitido (%s).";	// Okoteve traduccin
#elif L==7	// it
	"La dimensione del file"
	" supera il massimo consentito (%s).";
#elif L==8	// pl
	"Rozmiar pliku"
	" przekracza maksymalne dozwolone (%s).";
#elif L==9	// pt
	"O tamanho do arquivo"
	" excede o m&aacute;ximo permitido (%s).";
#endif

const char *Txt_The_status_of_the_centre_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El estado del centro <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The status of the centre <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==3	// en
	"The status of the centre <strong>%s</strong> has changed.";
#elif L==4	// es
	"El estado del centro <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"The status of the centre <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==6	// gn
	"El estado del centro <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il stato del centro <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"The status of the centre <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O status do centro <strong>%s</strong> foi alterado.";
#endif

const char *Txt_The_status_of_the_course_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The status of the course <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==3	// en
	"The status of the course <strong>%s</strong> has changed.";
#elif L==4	// es
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"The status of the course <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==6	// gn
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il stato del corso <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"The status of the course <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O status do curso <strong>%s</strong> mudou.";
#endif

const char *Txt_The_status_of_the_degree_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The status of the degree <strong>%s</strong> has changed.";		// Need bersetzung
#elif L==3	// en
	"The status of the degree <strong>%s</strong> has changed.";
#elif L==4	// es
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"The status of the degree <strong>%s</strong> has changed.";		// Besoin de traduction
#elif L==6	// gn
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il stato della laurea <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"The status of the degree <strong>%s</strong> has changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O status do grau <strong>%s</strong> foi alterado.";
#endif

const char *Txt_The_status_of_the_institution_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The status of the institution <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==3	// en
	"The status of the institution <strong>%s</strong> has changed.";
#elif L==4	// es
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"The status of the institution <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==6	// gn
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il stato della istituzione <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"The status of the institution <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O status da institui&ccedil;&atilde;o <strong>%s</strong> mudou.";
#endif

const char *Txt_The_survey_has_been_modified =
#if   L==1	// ca
	"L'enquesta ha estat modificada.";
#elif L==2	// de
	"Die Umfrage wurde ge&auml;ndert.";
#elif L==3	// en
	"The survey has been modified.";
#elif L==4	// es
	"La encuesta ha sido modificada.";
#elif L==5	// fr
	"Le sondage a &eacute;t&eacute; modifi&eacute;.";
#elif L==6	// gn
	"La encuesta ha sido modificada.";	// Okoteve traduccin
#elif L==7	// it
	"Il sondaggio &egrave; stato modificato.";
#elif L==8	// pl
	"Badanie zosta&lstrok;o zmodyfikowane.";
#elif L==9	// pt
	"O inqu&eacute;rito foi modificado.";
#endif

const char *Txt_The_tag_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==3	// en
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==6	// gn
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"L'etichetta <strong>%s</strong> &egrave; stata rinominata come <strong>%s</strong>.";
#elif L==8	// pl
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O descritor <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_tag_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El descriptor <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The tag <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3	// en
	"The tag <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El descriptor <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The tag <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6	// gn
	"El descriptor <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"L'etichetta <strong>%s</strong> non &egrave; cambiata.";
#elif L==8	// pl
	"The tag <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O descritor <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_test_X_has_already_been_assessed_previously =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"El test %u ya ha sido evaluado anteriormente.";	// Necessita traduccio
#elif L==2	// de
	"The test %u has already been assessed previously.";	// Need bersetzung
#elif L==3	// en
	"The test %u has already been assessed previously.";
#elif L==4	// es
	"El test %u ya ha sido evaluado anteriormente.";
#elif L==5	// fr
	"The test %u has already been assessed previously.";	// Besoin de traduction
#elif L==6	// gn
	"El test %u ya ha sido evaluado anteriormente.";	// Okoteve traduccin
#elif L==7	// it
	"Il test %u &egrave; stato valutato precedentemente.";
#elif L==8	// pl
	"The test %u has already been assessed previously.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O teste %u j&aacute; foi avaliado anteriormente.";
#endif

const char *Txt_The_test_configuration_has_been_updated =
#if   L==1	// ca
	"La configuraci&oacute;n de tests se ha actualizado.";	// Necessita traduccio
#elif L==2	// de
	"The test configuration has been updated.";		// Need bersetzung
#elif L==3	// en
	"The test configuration has been updated.";
#elif L==4	// es
	"La configuraci&oacute;n de los test se ha actualizado.";
#elif L==5	// fr
	"The test configuration has been updated.";		// Besoin de traduction
#elif L==6	// gn
	"La configuraci&oacute;n de los test se ha actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"La configurazione di test &egrave; stata aggiornata.";
#elif L==8	// pl
	"The test configuration has been updated.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A configura&ccedil;&atilde;o de teste foi atualizada.";
#endif

const char *Txt_The_thread_is_already_in_this_forum =
#if   L==1	// ca
	"La discusi&oacute;n ya est&aacute; en este foro.";	// Necessita traduccio
#elif L==2	// de
	"The thread is already in this forum.";			// Need bersetzung
#elif L==3	// en
	"The thread is already in this forum.";
#elif L==4	// es
	"La discusi&oacute;n ya est&aacute; en este foro.";
#elif L==5	// fr
	"The thread is already in this forum.";			// Besoin de traduction
#elif L==6	// gn
	"La discusi&oacute;n ya est&aacute; en este foro.";	// Okoteve traduccin
#elif L==7	// it
	"La discussione &egrave; gi&agrave; in questo forum.";
#elif L==8	// pl
	"The thread is already in this forum.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A discuss&atilde;o j&aacute; est&aacute; neste f&oacute;rum.";
#endif

const char *Txt_The_thread_X_is_already_in_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";	// Necessita traduccio
#elif L==2	// de
	"The thread <strong>%s</strong> is already in this forum.";		// Need bersetzung
#elif L==3	// en
	"The thread <strong>%s</strong> is already in this forum.";
#elif L==4	// es
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";
#elif L==5	// fr
	"The thread <strong>%s</strong> is already in this forum.";		// Besoin de traduction
#elif L==6	// gn
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";	// Okoteve traduccin
#elif L==7	// it
	"La discussione <strong>%s</strong> &egrave; gi&agrave; in questo forum.";
#elif L==8	// pl
	"The thread <strong>%s</strong> is already in this forum.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A discuss&atilde;o <strong>%s</strong> j&aacute; est&aacute; neste f&oacute;rum.";
#endif

const char *Txt_The_timetable_is_empty =
#if   L==1	// ca
	"L'horari est&agrave; buit.";
#elif L==2	// de
	"Der Fahrplan ist leer.";
#elif L==3	// en
	"The timetable is empty.";
#elif L==4	// es
	"El horario est&aacute; vac&iacute;o.";
#elif L==5	// fr
	"L'horaire est vide.";
#elif L==6	// gn
	"El horario est&aacute; vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Il orario &egrave; vuoto.";
#elif L==8	// pl
	"Harmonogram jest pusty.";
#elif L==9	// pt
	"O hor&aacute;rio est&aacute; vazio.";
#endif

const char *Txt_The_type_of_degree_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2	// de
	"Die Abschlussart <strong>%s</strong> wurde bereits definiert.";
#elif L==3	// en
	"The type of degree <strong>%s</strong> already exists.";
#elif L==4	// es
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The type of degree <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di laurea <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The type of degree <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de grau <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_type_of_degree_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==3	// en
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==6	// gn
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di laurea <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de grau <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_type_of_degree_of_the_degree_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Need bersetzung
#elif L==3	// en
	"The type of degree of the degree <strong>%s</strong> has changed.";
#elif L==4	// es
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Besoin de traduction
#elif L==6	// gn
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di laurea <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de grau do grau <strong>%s</strong> mudou.";
#endif

const char *Txt_The_type_of_group_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipus de grup <strong>%s</strong> ja existeix.";
#elif L==2	// de
	"Der Gruppen-Typ <strong>%s</strong> wurde bereits definiert.";
#elif L==3	// en
	"The type of group <strong>%s</strong> already exists.";
#elif L==4	// es
	"El tipo de grupo <strong>%s</strong> ya existe.";
#elif L==5	// fr
	"The type of group <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6	// gn
	"El tipo de grupo <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7	// it
	"Il tipo di gruppo <strong>%s</strong> gi&agrave; esiste.";
#elif L==8	// pl
	"The type of group <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de grupo <strong>%s</strong> j&aacute; existe.";
#endif

const char *Txt_The_type_of_group_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4	// es
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5	// fr
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di gruppo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8	// pl
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de grupo <strong>%s</strong> foi renomeado como <strong>%s</strong>.";
#endif

const char *Txt_The_type_of_group_of_the_group_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";		// Necessita traduccio
#elif L==2	// de
	"The type of group of the group <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==3	// en
	"The type of group of the group <strong>%s</strong> has changed.";
#elif L==4	// es
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"The type of group of the group <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==6	// gn
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";		// Okoteve traduccin
#elif L==7	// it
	"Il tipo di gruppo del gruppo <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"The type of group of the group <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de grupo do grupo <strong>%s</strong> foi alterado.";
#endif

const char *Txt_The_type_of_enrolment_of_the_type_of_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The type of enrolment of the type of group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3	// en
	"The type of enrolment of the type of group <strong>%s</strong> has not changed.";
#elif L==4	// es
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The type of enrolment of the type of group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6	// gn
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di registrazione del tipo di gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8	// pl
	"The type of enrolment of the type of group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo de inscri&ccedil;&atilde;o do tipo de grupo <strong>%s</strong> n&atilde;o foi alterado.";
#endif

const char *Txt_The_type_of_the_holiday_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The type of the holiday <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==3	// en
	"The type of the holiday <strong>%s</strong> has changed.";
#elif L==4	// es
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"The type of the holiday <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==6	// gn
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Il tipo di festivit&agrave; <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"The type of the holiday <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O tipo do feriado <strong>%s</strong> mudou.";
#endif

const char *Txt_The_URL_X_has_been_updated =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El URL <strong>%s</strong> se ha actualizado.";	// Necessita traduccio
#elif L==2	// de
	"The URL <strong>%s</strong> has been updated.";	// Need bersetzung
#elif L==3	// en
	"The URL <strong>%s</strong> has been updated.";
#elif L==4	// es
	"El URL <strong>%s</strong> se ha actualizado.";
#elif L==5	// fr
	"The URL <strong>%s</strong> has been updated.";	// Besoin de traduction
#elif L==6	// gn
	"El URL <strong>%s</strong> se ha actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"L'URL <strong>%s</strong> &egrave; stata archiviata.";
#elif L==8	// pl
	"The URL <strong>%s</strong> has been updated.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O URL <strong>%s</strong> foi atualizado.";
#endif

const char *Txt_The_user_does_not_exist =
#if   L==1	// ca
	"El usuario/a no existe.";	// Necessita traduccio
#elif L==2	// de
	"Die Benutzer ist nicht vorhanden.";
#elif L==3	// en
	"The user does not exist.";
#elif L==4	// es
	"El usuario/a no existe.";
#elif L==5	// fr
	"L'utilisateur n'existe pas.";
#elif L==6	// gn
	"El usuario/a no existe.";	// Okoteve traduccin
#elif L==7	// it
	"Il utente non esiste.";
#elif L==8	// pl
	"The user does not exist.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O utilizador n&atilde;o existe.";
#endif

const char *Txt_The_user_does_not_exist_or_password_is_incorrect =
#if   L==1	// ca
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";	// Necessita traduccio
#elif L==2	// de
	"Die Benutzer ist nicht vorhanden"
	" oder Passwort falsch ist.";
#elif L==3	// en
	"The user does not exist"
	" or password is incorrect.";
#elif L==4	// es
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";
#elif L==5	// fr
	"L'utilisateur n'existe pas"
	" ou le mot de passe est incorrect.";
#elif L==6	// gn
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";	// Okoteve traduccin
#elif L==7	// it
	"Il utente non esiste"
	" o la password non &egrave; corretta.";
#elif L==8	// pl
	"The user does not exist"
	" or password is incorrect.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O utilizador n&atilde;o existe"
	" ou a senha est&aacute; incorreta.";
#endif

const char *Txt_THE_USER_X_already_exists_in_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ya existe en %s.";		// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> already exists in %s.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> already exists in %s.";
#elif L==4	// es
	"<strong>%s</strong> ya existe en %s.";
#elif L==5	// fr
	"<strong>%s</strong> already exists in %s.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ya existe en %s.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> esiste gi&agrave; in %s.";
#elif L==8	// pl
	"<strong>%s</strong> already exists in %s.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> j&aacute; existe em %s.";
#endif

const char *Txt_THE_USER_X_exists_in_Y_but_is_not_enroled_in_the_course_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> existe en %s,"
	" pero <strong>no</strong> est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> exists in %s,"
	" but is <strong>not</strong> enroled"
	" in the course <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"<strong>%s</strong> exists in %s,"
	" but is <strong>not</strong> enroled"
	" in the course <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> existe en %s,"
	" pero <strong>no</strong> est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> exists in %s,"
	" but is <strong>not</strong> enroled"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> existe en %s,"
	" pero <strong>no</strong> est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> esiste in %s,"
	" ma <strong>non</strong> &egrave; registrato"
	" al corso <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> exists in %s,"
	" but is <strong>not</strong> enroled"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> existe em %s,"
	" mas <strong>n&atilde;o</strong> est&aacute; inscrito"
	" na disciplina <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_has_been_enroled_as_a_Y_in_the_project =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha estat afegit/da"
	" com a %s al projecte.";
#elif L==2	// de
	"<strong>%s</strong> hat sich"
	" als %s im Projekt eingeschrieben.";
#elif L==3	// en
	"<strong>%s</strong> has been enroled"
	" as a %s in the project.";
#elif L==4	// es
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" como %s al proyecto.";
#elif L==5	// fr
	"<strong>%s</strong> a &eacute;t&eacute; inscrit"
	" en tant que %s dans le projet.";
#elif L==6	// gn
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" como %s al proyecto.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato registrato/a"
	" como %s nel progetto.";
#elif L==8	// pl
	"<strong>%s</strong> zosta&lstrok; zapisany"
	" jako %s do projektu.";
#elif L==9	// pt
	"<strong>%s</strong> foi matriculado/a"
	" como %s no projeto.";
#endif

const char *Txt_THE_USER_X_has_been_enroled_as_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido inscrito/a"
	" como administrador/a de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> has been enroled"
	" as administrator of <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been enroled"
	" as administrator of <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido inscrito/a"
	" como administrador/a de <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> has been enroled"
	" as administrator of <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido inscrito/a"
	" como administrador/a de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato registrato"
	" come amministratore de <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> has been enroled"
	" as administrator of <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi inscrito"
	" como administrador de <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_has_been_enroled_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha estat afegit/da"
	" a l'assignatura <strong>%s</strong>.";
#elif L==2	// de
	"<strong>%s</strong> hat sich"
	" im Kurs <strong>%s</strong> eingeschrieben.";
#elif L==3	// en
	"<strong>%s</strong> has been enroled"
	" in the course <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" a la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> a &eacute;t&eacute; inscrit"
	" dans la mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" a la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato registrato/a"
	" nel corso <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> zosta&lstrok; zapisany"
	"  do kursu <strong>%s</strong>.";
#elif L==9	// pt
	"<strong>%s</strong> foi matriculado/a"
	" na disciplina <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_has_been_enroled_in_the_group_of_type_Y_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";		// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> has been enroled"
	" in the group of type %s <strong>%s</strong>.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been enroled"
	" in the group of type %s <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> has been enroled"
	" in the group of type %s <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato registrato"
	" nel gruppo di tipo %s <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> has been enroled"
	" in the group of type %s <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi inscrito"
	" no grupo do tipo %s <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_has_been_removed_as_administrator =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> has been removed as administrator.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed as administrator.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed as administrator.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso come amministratore.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed as administrator.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido como administrador.";
#endif

const char *Txt_THE_USER_X_has_been_removed_as_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a como administrador/a"
	" de <strong>%s</strong>.";		// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a como administrador/a"
	" de <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a como administrador/a"
	" de <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso come amministratore"
	" de <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido como administrador"
	" de <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_has_been_removed_from_all_his_her_courses =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> has been removed from all her/his courses.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed from all her/his courses.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed from all her/his courses.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso da tutti i suoi corsi.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed from all her/his courses.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido de todas as suas disciplinas.";
#endif

const char *Txt_THE_USER_X_has_not_been_removed_from_any_group =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> has not been removed"
	" from any group.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has not been removed"
	" from any group.";
#elif L==4	// es
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";
#elif L==5	// fr
	"<strong>%s</strong> has not been removed"
	" from any group.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> non &egrave; stato rimosso"
	" da alcun gruppo.";
#elif L==8	// pl
	"<strong>%s</strong> has not been removed"
	" from any group.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> n&atilde;o foi removido"
	" de nenhum grupo.";
#endif

const char *Txt_THE_USER_X_has_been_removed_from_Y_groups =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> has been removed from %u groups.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed from %u groups.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed from %u groups.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso da %u gruppi.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed from %u groups.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido de %u grupos.";
#endif

const char *Txt_THE_USER_X_has_been_removed_from_one_group =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> has been removed from one group.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed from one group.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed from one group.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso da un gruppo.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed from one group.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido de um grupo.";
#endif

const char *Txt_THE_USER_X_has_been_removed_from_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha estat eliminat"
	" de l'assignatura <strong>%s</strong>.";
#elif L==2	// de
	"<strong>%s</strong> wurde aus"
	" dem Kurs <strong>%s</strong> entfernt.";
#elif L==3	// en
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a"
	" de la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> a &eacute;t&eacute; retir&eacute;"
	" de la mati&egrave;re <strong>%s</strong>.";
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a"
	" de la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso"
	" dal corso <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> zosta&lstrok; usuni&eogon;ty"
	" z trasy <strong>%s</strong>.";
#elif L==9	// pt
	"<strong>%s</strong> foi removido"
	" da disciplina <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_has_been_removed_from_the_group_of_type_Y_to_which_it_belonged =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";
#elif L==5	// fr
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso"
	" dal gruppo di tipo %s al quale apparteneva.";
#elif L==8	// pl
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> foi removido"
	" do grupo do tipo %s ao qual pertencia.";
#endif

const char *Txt_THE_USER_X_has_been_removed_as_a_Y_from_the_project_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ha estat eliminat"
	" como a %s del projecte <strong>%s</strong>.";
#elif L==2	// de
	"<strong>%s</strong> wurde"
	" als %s aus dem Projekt <strong>%s</strong> entfernt.";
#elif L==3	// en
	"<strong>%s</strong> has been removed"
	" as a %s from the project <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ha sido eliminado/a"
	" como %s del proyecto <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> a &eacute;t&eacute; retir&eacute;"
	" en tant que %s du projet <strong>%s</strong>.";
#elif L==6	// gn
	"<strong>%s</strong> ha sido eliminado/a"
	" como %s del proyecto <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; stato rimosso"
	" come %s dal progetto <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> zosta&lstrok; wyeliminowany"
	" jako %s z projektu <strong>%s</strong>.";
#elif L==9	// pt
	"<strong>%s</strong> foi removido"
	" como %s do projeto <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_is_already_an_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> ya es administrador/a de <strong>%s</strong>.";		// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> ya es administrador/a de <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> ya es administrador/a de <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; gi&agrave; un amministratore de <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> j&aacute; &eacute; um administrador de <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_is_in_the_course_Y_but_has_not_yet_accepted_the_enrolment =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrolment.";			// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrolment.";
#elif L==4	// es
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";
#elif L==5	// fr
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrolment.";			// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; registrato/a"
	" nel corso <strong>%s</strong>,"
	" ma non ha ancora accettato la registrazione.";
#elif L==8	// pl
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrolment.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> est&aacute; inscrito/a"
	" na disciplina <strong>%s</strong>,"
	" mas ainda n&atilde;o aceitou a inscri&ccedil;&atilde;o.";
#endif

const char *Txt_THE_USER_X_is_already_enroled_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> is enroled"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> &egrave; registrato/a"
	" nel corso <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> est&aacute; inscrito/a"
	" na disciplina <strong>%s</strong>.";
#endif

const char *Txt_The_user_is_new_not_yet_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"L'usuari &eacute;s nou, encara no existeix en %s.";
#elif L==2	// de
	"Der Benutzer ist neu, noch nicht in %s.";
#elif L==3	// en
	"The user is new, not yet in %s.";
#elif L==4	// es
	"El usuario/a es nuevo/a, a&uacute;n no existe en %s.";
#elif L==5	// fr
	"L'utilisateur est nouveau, pas encore dans %s.";
#elif L==6	// gn
	"El usuario/a es nuevo/a, a&uacute;n no existe en %s.";	// Okoteve traduccin
#elif L==7	// it
	"L'utente &egrave; nuovo, non esiste ancora in %s.";
#elif L==8	// pl
	"U&zdot;ytkownik jest nowa, jeszcze nie w %s.";
#elif L==9	// pt
	"O usu&aacute;rio &eacute; novo, ainda n&atilde;o existe o %s.";
#endif

const char *Txt_THE_USER_X_is_not_an_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong> no es administrador/a de <strong>%s</strong>.";		// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";
#elif L==4	// es
	"<strong>%s</strong> no es administrador/a de <strong>%s</strong>.";
#elif L==5	// fr
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong> no es administrador/a de <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> non &egrave; un amministratore de <strong>%s</strong>.";
#elif L==8	// pl
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> n&atilde;o &eacute; um administrador de <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_is_not_found_in_the_file_of_marks =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong> is not found in the file of marks.";			// Need bersetzung
#elif L==3	// en
	"<strong>%s</strong> is not found in the file of marks.";
#elif L==4	// es
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";
#elif L==5	// fr
	"<strong>%s</strong> is not found in the file of marks.";			// Besoin de traduction
#elif L==6	// gn
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong> non &egrave; stato trovato nel file dei risultati.";
#elif L==8	// pl
	"<strong>%s</strong> is not found in the file of marks.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong> n&atilde;o &eacute; encontrado no arquivo de notas.";
#endif

const char *Txt_The_visibility_of_the_record_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";			// Necessita traduccio
#elif L==2	// de
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3	// en
	"The visibility of the record field <strong>%s</strong> has not changed.";
#elif L==4	// es
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";
#elif L==5	// fr
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6	// gn
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";			// Okoteve traduccin
#elif L==7	// it
	"La visibilit&agrave; del campo di scheda <strong>%s</strong> non &egrave; cambiata.";
#elif L==8	// pl
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"A visibilidade do campo de cart&atilde;o <strong>%s</strong> n&atilde;o foi alterada.";
#endif

const char *Txt_The_whole_course =
#if   L==1	// ca
	"Toda la asignatura";		// Necessita traduccio
#elif L==2	// de
	"Die gesamte Kurs";
#elif L==3	// en
	"The whole course";
#elif L==4	// es
	"Toda la asignatura";
#elif L==5	// fr
	"L'ensemble de la mati&egrave;re";
#elif L==6	// gn
	"Toda la asignatura";		// Okoteve traduccin
#elif L==7	// it
	"L'intero corso";
#elif L==8	// pl
	"The whole course";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Toda a disciplina";
#endif

const char *Txt_The_year_X_is_not_allowed =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"El a&ntilde;o/semestre <strong>%u</strong> no est&aacute; permitido.";		// Necessita traduccio
#elif L==2	// de
	"The year/semester <strong>%u</strong> is not enabled.";			// Need bersetzung
#elif L==3	// en
	"The year/semester <strong>%u</strong> is not enabled.";
#elif L==4	// es
	"El a&ntilde;o/semestre <strong>%u</strong> no est&aacute; permitido.";
#elif L==5	// fr
	"The year/semester <strong>%u</strong> is not enabled.";			// Besoin de traduction
#elif L==6	// gn
	"El a&ntilde;o/semestre <strong>%u</strong> no est&aacute; permitido.";		// Okoteve traduccin
#elif L==7	// it
	"L'anno/semester <strong>%u</strong> non &egrave; consentito.";
#elif L==8	// pl
	"The year/semester <strong>%u</strong> is not allowed.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O ano/semestre <strong>%u</strong> n&atilde;o &eacute; permitido.";
#endif

const char *Txt_The_year_of_the_course_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"El a&ntilde;o/semestre de la asignatura <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2	// de
	"The year/semester of the couse <strong>%s</strong> has changed.";		// Need bersetzung
#elif L==3	// en
	"The year/semester of the couse <strong>%s</strong> has changed.";
#elif L==4	// es
	"El a&ntilde;o/semestre de la asignatura <strong>%s</strong> ha cambiado.";
#elif L==5	// fr
	"The year/semester of the couse <strong>%s</strong> has changed.";		// Besoin de traduction
#elif L==6	// gn
	"El a&ntilde;o/semestre de la asignatura <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"L'anno/semestre del corso <strong>%s</strong> &egrave; cambiato.";
#elif L==8	// pl
	"The year/semestre of the couse <strong>%s</strong> has changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O ano/semestre da disciplina <strong>%s</strong> mudou.";
#endif

const char *Txt_The_ZIP_file_has_been_received_successfully =
#if   L==1	// ca
	"El archivo <em>ZIP</em> se ha recibido correctamente.";	// Necessita traduccio
#elif L==2	// de
	"The <em>ZIP</em> file has been received successfully.";	// Need bersetzung
#elif L==3	// en
	"The <em>ZIP</em> file has been received successfully.";
#elif L==4	// es
	"El archivo <em>ZIP</em> se ha recibido correctamente.";
#elif L==5	// fr
	"The <em>ZIP</em> file has been received successfully.";	// Besoin de traduction
#elif L==6	// gn
	"El archivo <em>ZIP</em> se ha recibido correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il <em>ZIP</em> file &egrave; stato ricevuto con successo.";
#elif L==8	// pl
	"The <em>ZIP</em> file has been received successfully.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O arquivo <em>ZIP</em> foi recebido com sucesso.";
#endif

const char *Txt_The_ZIP_file_has_been_unzipped_successfully =
#if   L==1	// ca
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";	// Necessita traduccio
#elif L==2	// de
	"The <em>ZIP</em> file has been unzipped successfully.";	// Need bersetzung
#elif L==3	// en
	"The <em>ZIP</em> file has been unzipped successfully.";
#elif L==4	// es
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";
#elif L==5	// fr
	"The <em>ZIP</em> file has been unzipped successfully.";	// Besoin de traduction
#elif L==6	// gn
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";	// Okoteve traduccin
#elif L==7	// it
	"Il <em>ZIP</em> file &egrave; stato decompresso con successo.";
#elif L==8	// pl
	"The <em>ZIP</em> file has been unzipped successfully.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"O arquivo <em>ZIP</em> foi descompactado com sucesso.";
#endif

const char *Txt_Theme_SKIN =
#if   L==1	// ca
	"Tema";
#elif L==2	// de
	"Skin";
#elif L==3	// en
	"Theme";
#elif L==4	// es
	"Tema";
#elif L==5	// fr
	"Th&egrave;me";
#elif L==6	// gn
	"Tema";	// Okoteve traduccin
#elif L==7	// it
	"Tema";
#elif L==8	// pl
	"Temat";
#elif L==9	// pt
	"Skin";
#endif

const char *Txt_There_are_more_than_one_table_in_the_file_of_marks =
#if   L==1	// ca
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";	// Necessita traduccio
#elif L==2	// de
	"There are more than one table in the file of marks.";		// Need bersetzung
#elif L==3	// en
	"There are more than one table in the file of marks.";
#elif L==4	// es
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";
#elif L==5	// fr
	"There are more than one table in the file of marks.";		// Besoin de traduction
#elif L==6	// gn
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==7	// it
	"C'&egrave; pi&ugrave; di una tabella nel file dei risultati.";
#elif L==8	// pl
	"There are more than one table in the file of marks.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Existem mais de uma tabela no arquivo de notas.";
#endif

const char *Txt_There_are_more_than_one_user_with_the_ID_X_Please_type_a_nick_or_email =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";	// Necessita traduccio
#elif L==2	// de
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";		// Need bersetzung
#elif L==3	// en
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";
#elif L==4	// es
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";
#elif L==5	// fr
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";		// Besoin de traduction
#elif L==6	// gn
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";	// Okoteve traduccin
#elif L==7	// it
	"Ci sono pi&ugrave; di un utente con l'ID <strong>%s</strong>."
	" Si prega di digitare un @nome-utente o email.";
#elif L==8	// pl
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"H&aacute; mais de um usu&aacute;rio com o ID <strong>%s</strong>."
	" Por favor, digite uma @alcunha ou um email.";
#endif

const char *Txt_There_are_new_posts =
#if   L==1	// ca
	"Hi ha missatges nous";
#elif L==2	// de
	"Es gibt neue Nachrichten";
#elif L==3	// en
	"There are new posts";
#elif L==4	// es
	"Hay mensajes nuevos";
#elif L==5	// fr
	"Il y a des nouveaux messages";
#elif L==6	// gn
	"Hay mensajes nuevos";	// Okoteve traduccin
#elif L==7	// it
	"Ci sono nuovi post";
#elif L==8	// pl
	"Istnieje nowych post&oacute;w";
#elif L==9	// pt
	"H&aacute; novas posts";
#endif

const char *Txt_There_are_no_accesses_with_the_selected_search_criteria =
#if   L==1	// ca
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";	// Necessita traduccio
#elif L==2	// de
	"There are no accesses with the selected search criteria.";		// Need bersetzung
#elif L==3	// en
	"There are no accesses with the selected search criteria.";
#elif L==4	// es
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";
#elif L==5	// fr
	"There are no accesses with the selected search criteria.";		// Besoin de traduction
#elif L==6	// gn
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono accessi con il criterio di ricerca selezionato.";
#elif L==8	// pl
	"There are no accesses with the selected search criteria.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; acessos com os crit&eacute;rios de pesquisa selecionados.";
#endif

const char *Txt_There_are_no_centres_with_requests_for_degrees_to_be_confirmed =
#if   L==1	// ca
	"No hi ha centres"
	" amb sol&middot;licituds de titulacions"
	" pendents de confirmar.";
#elif L==2	// de
	"There are no centres"
	" with requests for degrees"
	" to be confirmed.";		// Need bersetzung
#elif L==3	// en
	"There are no centres"
	" with requests for degrees"
	" to be confirmed.";
#elif L==4	// es
	"No hay centros"
	" con solicitudes de titulaciones"
	" pendientes de confirmar.";
#elif L==5	// fr
	"Il n'y a pas de centres"
	" avec demandes de degr&eacute;s"
	" &agrave; confirmer.";
#elif L==6	// gn
	"No hay centros"
	" con solicitudes de titulaciones"
	" pendientes de confirmar.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono centri"
	" con richieste di gradi"
	" pendenti de confermare.";
#elif L==8	// pl
	"There are no centres"
	" with requests for degrees"
	" to be confirmed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; centros"
	" com solicita&ccedil;&otilde;es de graus"
	" pendentes de confirma&ccedil;&atilde;o.";
#endif

const char *Txt_There_are_no_countries_with_requests_for_institutions_to_be_confirmed =
#if   L==1	// ca
	"No hi ha pa&iuml;sos"
	" amb sol&middot;licituds de institucions"
	" pendents de confirmar.";
#elif L==2	// de
	"There are no countries"
	" with requests for institutions"
	" to be confirmed.";		// Need bersetzung
#elif L==3	// en
	"There are no countries"
	" with requests for institutions"
	" to be confirmed.";
#elif L==4	// es
	"No hay pa&iacute;ses"
	" con solicitudes de instituciones"
	" pendientes de confirmar.";
#elif L==5	// fr
	"Il n'y a pas de pays"
	" avec demandes d'&eacute;tablissements"
	" &agrave; confirmer.";
#elif L==6	// gn
	"No hay pa&iacute;ses"
	" con solicitudes de instituciones"
	" pendientes de confirmar.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono paesi"
	" con richieste di istituzioni"
	" pendenti de confermare.";
#elif L==8	// pl
	"There are no countries"
	" with requests for institutions"
	" to be confirmed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; pa&iacute;ses"
	" com solicita&ccedil;&otilde;es de institui&ccedil;&otilde;es"
	" pendentes de confirma&ccedil;&atilde;o.";
#endif

const char *Txt_There_are_no_degrees_with_requests_for_courses_to_be_confirmed =
#if   L==1	// ca
	"No hi ha titulacions"
	" amb sol&middot;licituds d'assignatures"
	" pendents de confirmar.";
#elif L==2	// de
	"There are no degrees"
	" with requests for courses"
	" to be confirmed.";		// Need bersetzung
#elif L==3	// en
	"There are no degrees"
	" with requests for courses"
	" to be confirmed.";
#elif L==4	// es
	"No hay titulaciones"
	" con solicitudes de asignaturas"
	" pendientes de confirmar.";
#elif L==5	// fr
	"Il n'y a pas de degr&eacute;s"
	" avec demandes de cours"
	" &agrave; confirmer.";
#elif L==6	// gn
	"No hay titulaciones"
	" con solicitudes de asignaturas"
	" pendientes de confirmar.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono gradi"
	" con richieste di corsi"
	" pendenti de confermare.";
#elif L==8	// pl
	"There are no degrees"
	" with requests for courses"
	" to be confirmed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; graus"
	" com solicita&ccedil;&otilde;es de disciplinas"
	" pendentes de confirma&ccedil;&atilde;o.";
#endif

const char *Txt_There_are_no_institutions_with_requests_for_centres_to_be_confirmed =
#if   L==1	// ca
	"No hi ha institucions"
	" amb sol&middot;licituds de centres"
	" pendents de confirmar.";
#elif L==2	// de
	"There are no institutions"
	" with requests for centres"
	" to be confirmed.";		// Need bersetzung
#elif L==3	// en
	"There are no institutions"
	" with requests for centres"
	" to be confirmed.";
#elif L==4	// es
	"No hay instituciones"
	" con solicitudes de centros"
	" pendientes de confirmar.";
#elif L==5	// fr
	"Il n'y a pas d'&eacute;tablissements"
	" avec demandes de centres"
	" &agrave; confirmer.";
#elif L==6	// gn
	"No hay instituciones"
	" con solicitudes de centros"
	" pendientes de confirmar.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono istituzioni"
	" con richieste di centri"
	" pendenti de confermare.";
#elif L==8	// pl
	"There are no institutions"
	" with requests for centres"
	" to be confirmed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; institui&ccedil;&otilde;es"
	" com solicita&ccedil;&otilde;es de centros"
	" pendentes de confirma&ccedil;&atilde;o.";
#endif

const char *Txt_There_are_no_record_fields_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"There are no record fields in the course <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"There are no record fields in the course <strong>%s</strong>.";
#elif L==4	// es
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"There are no record fields in the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono campi di schede nel corso <strong>%s</strong>.";
#elif L==8	// pl
	"There are no record fields in the course <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&atilde; campos de cart&atilde;o na disciplina <strong>%s</strong>.";
#endif

const char *Txt_There_are_no_types_of_group_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"There are no types of group in the course <strong>%s</strong>.";	// Need bersetzung
#elif L==3	// en
	"There are no types of group in the course <strong>%s</strong>.";
#elif L==4	// es
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"There are no types of group in the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono tipi di gruppi nel corso <strong>%s</strong>.";
#elif L==8	// pl
	"There are no types of group in the course <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; tipos de grupo na disciplina <strong>%s</strong>.";
#endif

const char *Txt_There_are_X_users_with_the_ID_Y =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==1	// ca
	"Hi ha %u usuaris amb el ID (DNI/c&eacute;dula) <strong>%s</strong>.";
#elif L==2	// de
	"There are %u users with the ID <strong>%s</strong>.";			// Need bersetzung
#elif L==3	// en
	"There are %u users with the ID <strong>%s</strong>.";
#elif L==4	// es
	"Hay %u usuarios con el ID (DNI/c&eacute;dula) <strong>%s</strong>.";
#elif L==5	// fr
	"Il y a %u utilisateurs avec le ID <strong>%s</strong>.";
#elif L==6	// gn
	"Hay %u usuarios con el ID (DNI/c&eacute;dula) <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Ci sono %u utenti con il ID <strong>%s</strong>.";
#elif L==8	// pl
	"There are %u users with the ID <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Existem %u usu&aacute;rios com o ID <strong>%s</strong>.";
#endif

const char *Txt_There_has_been_no_change_in_groups =
#if   L==1	// ca
	"No se ha realizado ning&uacute;n cambio de grupos.";	// Necessita traduccio
#elif L==2	// de
	"There has been no change in groups.";			// bersetzung notwendig
#elif L==3	// en
	"There has been no change in groups.";
#elif L==4	// es
	"No se ha realizado ning&uacute;n cambio de grupos.";
#elif L==5	// fr
	"There has been no change in groups.";			// Besoin de traduction
#elif L==6	// gn
	"No se ha realizado ning&uacute;n cambio de grupos.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono stati cambiamenti in gruppi.";
#elif L==8	// pl
	"There has been no change in groups.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Nenhuma altera&ccedil;&atilde;o de grupo foi feita.";
#endif

const char *Txt_There_have_been_X_errors_in_sending_the_message =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Ha habido %u errores enviando el mensaje.";		// Necessita traduccio
#elif L==2	// de
	"There have been %u errors in sending the message.";	// bersetzung notwendig
#elif L==3	// en
	"There have been %u errors in sending the message.";
#elif L==4	// es
	"Ha habido %u errores enviando el mensaje.";
#elif L==5	// fr
	"There have been %u errors in sending the message.";	// Besoin de traduction
#elif L==6	// gn
	"Ha habido %u errores enviando el mensaje.";		// Okoteve traduccin
#elif L==7	// it
	"Ci sono stati %u errori inviando il messaggio.";
#elif L==8	// pl
	"There have been %u errors in sending the message.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Houve %u erros ao enviar a mensagem.";
#endif

const char *Txt_There_is_already_a_file_named_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ya existe un archivo con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"There is already a file named <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"There is already a file named <strong>%s</strong>.";
#elif L==4	// es
	"Ya existe un archivo con el nombre <strong>%s</strong>.";
#elif L==5	// fr
	"There is already a file named <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"Ya existe un archivo con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"C'&egrave; gi&agrave; un file chiamato <strong>%s</strong>.";
#elif L==8	// pl
	"There is already a file named <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"J&aacute; existe um arquivo chamado <strong>%s</strong>.";
#endif

const char *Txt_There_is_already_a_non_empty_folder_named_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"There is already a non-empty folder named <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"There is already a non-empty folder named <strong>%s</strong>.";
#elif L==4	// es
	"Ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.";
#elif L==5	// fr
	"There is already a non-empty folder named <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"Ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"C'&egrave; gi&agrave; una cartella non vuota chiamata <strong>%s</strong>.";
#elif L==8	// pl
	"There is already a non-empty folder named <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"J&aacute; existe uma pasta n&atilde;o vazia chamada strong>%s</strong>.";
#endif

const char *Txt_There_is_no_empty_account_associated_with_your_ID_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No hi ha cap compte buida (encara no usada)"
	" associada al seu ID <strong>%s</strong>.";
#elif L==2	// de
	"Es gibt keine leeren Benutzerkonto (noch nicht verwendet)"
	" im Zusammenhang mit Ihrer ID <strong>%s</strong>.";
#elif L==3	// en
	"There is no empty (not yet used) account"
	" associated with your ID <strong>%s</strong>.";
#elif L==4	// es
	"No existe ninguna cuenta vac&iacute;a (a&uacute;n no usada)"
	" asociada a su ID <strong>%s</strong>.";
#elif L==5	// fr
	"Il n'y a pas de compte vide (pas encore utilis&eacute;)"
	" associ&eacute; &agrave; votre ID <strong>%s</strong>.";
#elif L==6	// gn
	"No existe ninguna cuenta vac&iacute;a (a&uacute;n no usada)"
	" asociada a su ID <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non vi &egrave; alcun conto vuoto (non ancora utilizzato)"
	" associato con il tuo ID <strong>%s</strong>.";
#elif L==8	// pl
	"Nie ma pusta konta (jeszcze nie u&zdot;ywany)"
	" powi&aogon;zany z identyfikatorem <strong>%s</strong>.";
#elif L==9	// pt
	"N&atilde;o existe nenhuma conta vazio (ainda n&atilde;o utilizado)"
	" associado ao seu ID <strong>%s</strong>.";
#endif

const char *Txt_There_is_no_knowing_how_many_users_not_logged_have_accessed =
#if   L==1	// ca
	"No se puede saber cu&aacute;ntos usuarios distintos"
	" han accedido sin identificarse.";	// Necessita traduccio
#elif L==2	// de
	"There is no knowing how many users"
	" not logged have accessed.";		// bersetzung notwendig
#elif L==3	// en
	"There is no knowing how many users"
	" not logged have accessed.";
#elif L==4	// es
	"No se puede saber cu&aacute;ntos usuarios distintos"
	" han accedido sin identificarse.";
#elif L==5	// fr
	"There is no knowing how many users"
	" not logged have accessed.";		// Besoin de traduction
#elif L==6	// gn
	"No se puede saber cu&aacute;ntos usuarios distintos"
	" han accedido sin identificarse.";	// Okoteve traduccin
#elif L==7	// it
	"Non si pu&ograve; sapere quanti utenti distinti"
	" hanno avuto accesso senza identificarsi.";
#elif L==8	// pl
	"There is no knowing how many users"
	" not logged have accessed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; como saber quantos usu&aacute;rios distintos"
	" n&atilde;o logados acessaram.";
#endif

const char *Txt_There_is_no_user_with_email_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"There is no user"
	" with email <strong>%s</strong>.";	// bersetzung notwendig
#elif L==3	// en
	"There is no user"
	" with email <strong>%s</strong>.";
#elif L==4	// es
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";
#elif L==5	// fr
	"There is no user"
	" with email <strong>%s</strong>.";	// Besoin de traduction
#elif L==6	// gn
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non c'&egrave; alcun utente"
	" con email <strong>%s</strong>.";
#elif L==8	// pl
	"There is no user"
	" with email <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; usu&aacute;rio"
	" com email <strong>%s</strong>.";
#endif

const char *Txt_There_is_no_user_with_ID_nick_or_email_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";			// bersetzung notwendig
#elif L==3	// en
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";
#elif L==4	// es
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";
#elif L==5	// fr
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";			// Besoin de traduction
#elif L==6	// gn
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Non c'&egrave; alcun utente"
	" con ID, nome utente o email <strong>%s</strong>.";
#elif L==8	// pl
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; usu&aacute;rio"
	" com ID, alcunha ou email <strong>%s</strong>.";
#endif

const char *Txt_There_is_no_user_with_nickname_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";		// Necessita traduccio
#elif L==2	// de
	"There is no user"
	" with nickname <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"There is no user"
	" with nickname <strong>%s</strong>.";
#elif L==4	// es
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";
#elif L==5	// fr
	"There is no user"
	" with nickname <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7	// it
	"Non c'&egrave; alcun utente"
	" con nome utente <strong>%s</strong>.";
#elif L==8	// pl
	"There is no user"
	" with nickname <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o h&aacute; usu&aacute;rio"
	" com alcunha <strong>%s</strong>.";
#endif

const char *Txt_There_was_a_problem_sending_an_email_automatically =
#if   L==1	// ca
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo.";	// Necessita traduccio
#elif L==2	// de
	"There was a problem sending an email automatically.";			// bersetzung notwendig
#elif L==3	// en
	"There was a problem sending an email automatically.";
#elif L==4	// es
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo.";
#elif L==5	// fr
	"There was a problem sending an email automatically.";			// Besoin de traduction
#elif L==6	// gn
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo.";	// Okoteve traduccin
#elif L==7	// it
	"C'&egrave; stato un problema inviando automaticamente un'email.";
#elif L==8	// pl
	"There was a problem sending an email automatically.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ocorreu um problema ao enviar um email automaticamente.";
#endif

const char *Txt_There_was_an_error_in_assessing_the_test_X =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Se ha producido un error al evaluar el test %u.";	// Necessita traduccio
#elif L==2	// de
	"There was an error in assessing the test %u.";		// Need bersetzung
#elif L==3	// en
	"There was an error in assessing the test %u.";
#elif L==4	// es
	"Se ha producido un error al evaluar el test %u.";
#elif L==5	// fr
	"There was an error in assessing the test %u.";		// Besoin de traduction
#elif L==6	// gn
	"Se ha producido un error al evaluar el test %u.";	// Okoteve traduccin
#elif L==7	// it
	"C'&egrave; stato un errore nel valutare il test %u.";
#elif L==8	// pl
	"There was an error in assessing the test %u.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Houve um erro ao avaliar o teste %u.";
#endif

const char *Txt_This_game_has_no_questions =
#if   L==1	// ca
	"Aquest joc no t&eacute; preguntes.";
#elif L==2	// de
	"Dieses Spiel hat keine Fragen.";
#elif L==3	// en
	"This game has no questions.";
#elif L==4	// es
	"Este juego no tiene preguntas.";
#elif L==5	// fr
	"Ce jeu n'a pas de questions.";
#elif L==6	// gn
	"Este juego no tiene preguntas.";	// Okoteve traduccin
#elif L==7	// it
	"Questo gioco non ha domande.";
#elif L==8	// pl
	"Ta gra nie ma pytania.";
#elif L==9	// pt
	"Este jogo n&atilde;o tem perguntas.";
#endif

const char *Txt_This_link_will_remain_active_as_long_as_your_user_s_account_exists =
#if   L==1	// ca
	"Aquest enlla&ccedil; romandr&agrave; actiu mentre existeixi el seu compte de l'usuari.";
#elif L==2	// de
	"Diese Verbindung bleibt aktiv, solange das Konto Ihres Benutzers vorhanden.";
#elif L==3	// en
	"This link will remain active as long as your user's account exists.";
#elif L==4	// es
	"Este enlace permanecer&aacute; activo mientras exista su cuenta del usuario.";
#elif L==5	// fr
	"Ce lien restera actif aussi longtemps que votre compte de l'utilisateur existe.";
#elif L==6	// gn
	"Este enlace permanecer&aacute; activo mientras exista su cuenta del usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Questo link rimarr&agrave; attiva finch&eacute; esiste il tuo account utente.";
#elif L==8	// pl
	"Ten link pozostanie aktywna tak d&lstrok;ugo, jak istnieje konto u&zdot;ytkownika.";
#elif L==9	// pt
	"Este link permanecer&aacute; ativo enquanto existir a sua conta do usu&aacute;rio.";
#endif

const char *Txt_This_post_has_been_banned_probably_for_not_satisfy_the_rules_of_the_forums =
#if   L==1	// ca
	"Este comentario ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";		// Necessita traduccio
#elif L==2	// de
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Need bersetzung
#elif L==3	// en
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";
#elif L==4	// es
	"Este comentario ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";
#elif L==5	// fr
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Besoin de traduction
#elif L==6	// gn
	"Este comentario ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";		// Okoteve traduccin
#elif L==7	// it
	"Questo post &egrave; stato censurato,"
	" probabilmente perch&egrave; non soddisfa le regole del forum.";
#elif L==8	// pl
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Este post foi censurado,"
	" provavelmente por n&atilde;o satisfazer as regras dos f&oacute;runs.";
#endif

const char *Txt_This_survey_has_no_questions =
#if   L==1	// ca
	"Aquesta enquesta no t&eacute; preguntes.";
#elif L==2	// de
	"Diese Umfrage hat keine Fragen.";
#elif L==3	// en
	"This survey has no questions.";
#elif L==4	// es
	"Esta encuesta no tiene preguntas.";
#elif L==5	// fr
	"Ce sondage n'a pas de questions.";
#elif L==6	// gn
	"Esta encuesta no tiene preguntas.";	// Okoteve traduccin
#elif L==7	// it
	"Questo sondaggio non ha domande.";
#elif L==8	// pl
	"Badanie to nie ma pytania.";
#elif L==9	// pt
	"Este inqu&eacute;rito n&atilde;o tem perguntas.";
#endif

const char *Txt_Thread =
#if   L==1	// ca
	"Discussi&oacute;";
#elif L==2	// de
	"Thread";
#elif L==3	// en
	"Thread";
#elif L==4	// es
	"Discusi&oacute;n";
#elif L==5	// fr
	"Fil";
#elif L==6	// gn
	"Discusi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Discussione";
#elif L==8	// pl
	"W&aogon;tek";
#elif L==9	// pt
	"Discuss&atilde;o";
#endif

const char *Txt_thread =
#if   L==1	// ca
	"discussi&oacute;";
#elif L==2	// de
	"Thread";
#elif L==3	// en
	"thread";
#elif L==4	// es
	"discusi&oacute;n";
#elif L==5	// fr
	"fil";
#elif L==6	// gn
	"discusi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"discussione";
#elif L==8	// pl
	"w&aogon;tek";
#elif L==9	// pt
	"discuss&atilde;o";
#endif

const char *Txt_Thread_marked_to_be_moved =
#if   L==1	// ca
	"Discusi&oacute;n marcada para moverla.";	// Necessita traduccio
#elif L==2	// de
	"Thread marked to be moved.";			// Need bersetzung
#elif L==3	// en
	"Thread marked to be moved.";
#elif L==4	// es
	"Discusi&oacute;n marcada para moverla.";
#elif L==5	// fr
	"Thread marked to be moved.";			// Besoin de traduction
#elif L==6	// gn
	"Discusi&oacute;n marcada para moverla.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione segnata per spostarla.";
#elif L==8	// pl
	"Thread marked to be moved.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Discuss&atilde;o marcada para ser movida.";
#endif

const char *Txt_Thread_X_marked_to_be_moved =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";	// Necessita traduccio
#elif L==2	// de
	"Thread <strong>%s</strong> marked to be moved.";		// Need bersetzung
#elif L==3	// en
	"Thread <strong>%s</strong> marked to be moved.";
#elif L==4	// es
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";
#elif L==5	// fr
	"Thread <strong>%s</strong> marked to be moved.";		// Besoin de traduction
#elif L==6	// gn
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione <strong>%s</strong> segnata per spostarla.";
#elif L==8	// pl
	"Thread <strong>%s</strong> marked to be moved.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Discuss&atilde;o <strong>%s</strong> marcada para ser movida.";
#endif

const char *Txt_Thread_moved_to_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Discusi&oacute;n movida a este foro.";	// Necessita traduccio
#elif L==2	// de
	"Thread moved to this forum.";		// Need bersetzung
#elif L==3	// en
	"Thread moved to this forum.";
#elif L==4	// es
	"Discusi&oacute;n movida a este foro.";
#elif L==5	// fr
	"Thread moved to this forum.";		// Besoin de traduction
#elif L==6	// gn
	"Discusi&oacute;n movida a este foro.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione spostata in questo forum.";
#elif L==8	// pl
	"Thread moved to this forum.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Discuss&atilde;o movida para este f&oacute;rum.";
#endif

const char *Txt_Thread_X_moved_to_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";	// Necessita traduccio
#elif L==2	// de
	"Thread <strong>%s</strong> moved to this forum.";		// Need bersetzung
#elif L==3	// en
	"Thread <strong>%s</strong> moved to this forum.";
#elif L==4	// es
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";
#elif L==5	// fr
	"Thread <strong>%s</strong> moved to this forum.";		// Besoin de traduction
#elif L==6	// gn
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione <strong>%s</strong> spostata in questo forum.";
#elif L==8	// pl
	"Thread <strong>%s</strong> moved to this forum.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Discuss&atilde;o <strong>%s</strong> movida para este f&oacute;rum.";
#endif

const char *Txt_Thread_removed =
#if   L==1	// ca
	"Discusi&oacute;n eliminada.";	// Necessita traduccio
#elif L==2	// de
	"Faden entfernt.";
#elif L==3	// en
	"Thread removed.";
#elif L==4	// es
	"Discusi&oacute;n eliminada.";
#elif L==5	// fr
	"Thread removed.";		// Besoin de traduction
#elif L==6	// gn
	"Discusi&oacute;n eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione rimossa.";
#elif L==8	// pl
	"Thread removed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Discuss&atilde;o removida.";
#endif

const char *Txt_Thread_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Discusi&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2	// de
	"Faden <strong>%s</strong> entfernt.";
#elif L==3	// en
	"Thread <strong>%s</strong> removed.";
#elif L==4	// es
	"Discusi&oacute;n <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Thread <strong>%s</strong> removed.";			// Besoin de traduction
#elif L==6	// gn
	"Discusi&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Thread <strong>%s</strong> removed.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Discuss&atilde;o <strong>%s</strong> removida.";
#endif

const char *Txt_Thread_with_posts_from_you =
#if   L==1	// ca
	"Discusi&oacute;n con comentarios de usted.";	// Necessita traduccio
#elif L==2	// de
	"Thread with posts from you.";			// Need bersetzung
#elif L==3	// en
	"Thread with posts from you.";
#elif L==4	// es
	"Discusi&oacute;n con comentarios de usted.";
#elif L==5	// fr
	"Thread with posts from you.";			// Besoin de traduction
#elif L==6	// gn
	"Discusi&oacute;n con comentarios de usted.";	// Okoteve traduccin
#elif L==7	// it
	"Discussione con post per te.";
#elif L==8	// pl
	"Thread with posts from you.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Discuss&atilde;o com mensagens de voc&ecirc;.";
#endif

const char *Txt_threads =
#if   L==1	// ca
	"discussions";
#elif L==2	// de
	"Threads";
#elif L==3	// en
	"threads";
#elif L==4	// es
	"discusiones";
#elif L==5	// fr
	"fils";
#elif L==6	// gn
	"discusiones";	// Okoteve traduccin
#elif L==7	// it
	"discussione";
#elif L==8	// pl
	"w&aogon;tki";
#elif L==9	// pt
	"discuss&otilde;es";
#endif

const char *Txt_TIME_Since =
#if   L==1	// ca
	"Des de";
#elif L==2	// de
	"Seit dem";
#elif L==3	// en
	"Since";
#elif L==4	// es
	"Desde";
#elif L==5	// fr
	"Depuis";
#elif L==6	// gn
	"Desde";	// Okoteve traduccin
#elif L==7	// it
	"Dal";
#elif L==8	// pl
	"Od";
#elif L==9	// pt
	"Desde";
#endif

const char *Txt_TIME_until =
#if   L==1	// ca
	"fins al";
#elif L==2	// de
	"bis zum";
#elif L==3	// en
	"until";
#elif L==4	// es
	"hasta";
#elif L==5	// fr
	"jusqu'au";
#elif L==6	// gn
	"hasta";	// Okoteve traduccin
#elif L==7	// it
	"al";
#elif L==8	// pl
	"do";
#elif L==9	// pt
	"at&eacute;";
#endif

const char *Txt_time =
#if   L==1	// ca
	"temps";
#elif L==2	// de
	"Zeit";
#elif L==3	// en
	"time";
#elif L==4	// es
	"tiempo";
#elif L==5	// fr
	"temps";
#elif L==6	// gn
	"tiempo";	// Okoteve traduccin
#elif L==7	// it
	"tempo";
#elif L==8	// pl
	"czas";
#elif L==9	// pt
	"tempo";
#endif

const char *Txt_Time_zone_used_in_the_calculation_of_these_statistics =
#if   L==1	// ca
	"Zona hor&agrave;ria usada en el c&agrave;lcul d'aquesta estad&iacute;stica";
#elif L==2	// de
	"Zeitzone in die Berechnung dieser Statistik verwendet";
#elif L==3	// en
	"Time zone used in the calculation of these statistics";
#elif L==4	// es
	"Zona horaria usada en el c&aacute;lculo de esta estad&iacute;stica";
#elif L==5	// fr
	"Fuseau horaire utilis&eacute; dans le calcul de ces statistiques";
#elif L==6	// gn
	"Zona horaria usada en el c&aacute;lculo de esta estad&iacute;stica";	// Okoteve traduccin
#elif L==7	// it
	"Fuso orario utilizzato per il calcolo di queste statistiche";
#elif L==8	// pl
	"Strefa czasowa u&zdot;ywana w obliczeniach tych statystyk";
#elif L==9	// pt
	"Fuso hor&aacute;rio utilizado no c&aacute;lculo dessas estat&iacute;sticas";
#endif

const char *Txt_Timeline =
#if   L==1	// ca
	 "Timeline";
#elif L==2	// de
	 "Timeline";
#elif L==3	// en
	 "Timeline";
#elif L==4	// es
	 "Timeline";
#elif L==5	// fr
	 "Chronologie";
#elif L==6	// gn
	 "Timeline";	// Okoteve traduccin
#elif L==7	// it
	 "Timeline";
#elif L==8	// pl
	 "Timeline";
#elif L==9	// pt
	 "Timeline";
#endif

const char *Txt_Timeline_OF_A_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	 "Timeline de %s";
#elif L==2	// de
	 "%s timeline";
#elif L==3	// en
	 "%s timeline";
#elif L==4	// es
	 "Timeline de %s";
#elif L==5	// fr
	 "Chronologie de %s";
#elif L==6	// gn
	 "Timeline de %s";	// Okoteve traduccin
#elif L==7	// it
	 "Timeline di %s";
#elif L==8	// pl
	 "Timeline %s";
#elif L==9	// pt
	 "Timeline de %s";
#endif


const char *Txt_TIMELINE_NOTE[TL_NUM_NOTE_TYPES] =
	{
#if   L==1	// ca	// TL_NOTE_UNKNOWN
	 "Esdeveniment desconegut"
#elif L==2	// de
	 "Unbekannt Ereignis"
#elif L==3	// en
	 "Unknown event"
#elif L==4	// es
	 "Evento desconocido"
#elif L==5	// fr
	 "&Eacute;v&eacute;nement inconnu"
#elif L==6	// gn
	 "Evento desconocido"	// Okoteve traduccin
#elif L==7	// it
	 "Evento sconosciuto"
#elif L==8	// pl
	 "Nieznane zdarzenia"
#elif L==9	// pt
	 "Evento desconhecido"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_INS_DOC_PUB_FILE
	 "Document p&uacute;blic (instituci&oacute;)"
#elif L==2	// de
	 "&Ouml;ffentliche Dokumentdatei (Hochschule)"
#elif L==3	// en
	 "Public document file (institution)"
#elif L==4	// es
	 "Documento p&uacute;blico (instituci&oacute;n)"
#elif L==5	// fr
	 "Fichier de document public (&eacute;tablissement)"
#elif L==6	// gn
	 "Documento p&uacute;blico (instituci&oacute;n)"	// Okoteve traduccin
#elif L==7	// it
	 "Documento pubblico (istituzione)"
#elif L==8	// pl
	 "Plik dokumentu publiczne (instytucji)"
#elif L==9	// pt
	 "Arquivo de documento p&uacute;blico (institu&ccedil;&atilde;o)"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_INS_SHA_PUB_FILE
	 "Arxiu compartit p&uacute;blic (instituci&oacute;)"
#elif L==2	// de
	 "&Ouml;ffentliche Freigegebene Datei (Hochschule)"
#elif L==3	// en
	 "Public shared file (institution)"
#elif L==4	// es
	 "Archivo compartido p&uacute;blico (instituci&oacute;n)"
#elif L==5	// fr
	 "Fichier partag&eacute; public (&eacute;tablissement)"
#elif L==6	// gn
	 "Archivo compartido p&uacute;blico (instituci&oacute;n)"	// Okoteve traduccin
#elif L==7	// it
	 "File condiviso pubblico (istituzione)"
#elif L==8	// pl
	 "Udost&eogon;pniony plik publiczne (instytucji)"
#elif L==9	// pt
	 "Arquivo compartilhado p&uacute;blico (institu&ccedil;&atilde;o)"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_CTR_DOC_PUB_FILE
	 "Document p&uacute;blic (centre)"
#elif L==2	// de
	 "&Ouml;ffentliche Dokumentdatei (Lehrinstitut)"
#elif L==3	// en
	 "Public document file (centre)"
#elif L==4	// es
	 "Documento p&uacute;blico (centro)"
#elif L==5	// fr
	 "Fichier de document public (centre)"
#elif L==6	// gn
	 "Documento p&uacute;blico (centro)"	// Okoteve traduccin
#elif L==7	// it
	 "Documento pubblico (centro)"
#elif L==8	// pl
	 "Plik dokumentu publiczne (centrum)"
#elif L==9	// pt
	 "Arquivo de documento p&uacute;blico (centro)"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_CTR_SHA_PUB_FILE
	 "Arxiu compartit p&uacute;blic (centre)"
#elif L==2	// de
	 "&Ouml;ffentliche Freigegebene Datei (Lehrinstitut)"
#elif L==3	// en
	 "Public shared file (centre)"
#elif L==4	// es
	 "Archivo compartido p&uacute;blico (centro)"
#elif L==5	// fr
	 "Fichier partag&eacute; public (centre)"
#elif L==6	// gn
	 "Archivo compartido p&uacute;blico (centro)"	// Okoteve traduccin
#elif L==7	// it
	 "File condiviso pubblico (centro)"
#elif L==8	// pl
	 "Udost&eogon;pniony plik publiczne (centrum)"
#elif L==9	// pt
	 "Arquivo compartilhado p&uacute;blico (centro)"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_DEG_DOC_PUB_FILE
	 "Document p&uacute;blic (titulaci&oacute;)"
#elif L==2	// de
	 "&Ouml;ffentliche Dokumentdatei (Studiengang)"
#elif L==3	// en
	 "Public document file (degree)"
#elif L==4	// es
	 "Documento p&uacute;blico (titulaci&oacute;n)"
#elif L==5	// fr
	 "Fichier de document public (&eacute;tude)"
#elif L==6	// gn
	 "Documento p&uacute;blico (titulaci&oacute;n)"	// Okoteve traduccin
#elif L==7	// it
	 "Documento pubblico (laurea)"
#elif L==8	// pl
	 "Plik dokumentu publiczne (stopie&nacute;)"
#elif L==9	// pt
	 "Arquivo de documento p&uacute;blico (grau)"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_DEG_SHA_PUB_FILE
	 "Arxiu compartit p&uacute;blic (titulaci&oacute;)"
#elif L==2	// de
	 "&Ouml;ffentliche Freigegebene Datei (Studiengang)"
#elif L==3	// en
	 "Public shared file (degree)"
#elif L==4	// es
	 "Archivo compartido p&uacute;blico (titulaci&oacute;n)"
#elif L==5	// fr
	 "Fichier partag&eacute; public (&eacute;tude)"
#elif L==6	// gn
	 "Archivo compartido p&uacute;blico (titulaci&oacute;n)"	// Okoteve traduccin
#elif L==7	// it
	 "File condiviso pubblico (laurea)"
#elif L==8	// pl
	 "Udost&eogon;pniony plik publiczne (stopie&nacute;)"
#elif L==9	// pt
	 "Arquivo compartilhado p&uacute;blico (grau)"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_CRS_DOC_PUB_FILE
	 "Document p&uacute;blic (assignatura)"
#elif L==2	// de
	 "&Ouml;ffentliche Dokumentdatei (Kurs)"
#elif L==3	// en
	 "Public document file (course)"
#elif L==4	// es
	 "Documento p&uacute;blico (asignatura)"
#elif L==5	// fr
	 "Fichier de document public (mati&egrave;re)"
#elif L==6	// gn
	 "Documento p&uacute;blico (asignatura)"	// Okoteve traduccin
#elif L==7	// it
	 "Documento pubblico (corso)"
#elif L==8	// pl
	 "Plik dokumentu publiczne (kurs)"
#elif L==9	// pt
	 "Arquivo de documento p&uacute;blico (disciplina)"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_CRS_SHA_PUB_FILE
	 "Arxiu compartit p&uacute;blic (assignatura)"
#elif L==2	// de
	 "&Ouml;ffentliche Freigegebene Datei (Kurs)"
#elif L==3	// en
	 "Public shared file (course)"
#elif L==4	// es
	 "Archivo compartido p&uacute;blico (asignatura)"
#elif L==5	// fr
	 "Fichier partag&eacute; public (mati&egrave;re)"
#elif L==6	// gn
	 "Archivo compartido p&uacute;blico (asignatura)"	// Okoteve traduccin
#elif L==7	// it
	 "File condiviso pubblico (corso)"
#elif L==8	// pl
	 "Udost&eogon;pniony plik publiczne (kurs)"
#elif L==9	// pt
	 "Arquivo compartilhado p&uacute;blico (disciplina)"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_EXAM_ANNOUNCEMENT
	 "Convocat&ograve;ria d'examen"
#elif L==2	// de
	 "Aufrufe fr Pr&uuml;fung"
#elif L==3	// en
	 "Announcement of exam"
#elif L==4	// es
	 "Convocatoria de examen"
#elif L==5	// fr
	 "Convocation &agrave; un examen"
#elif L==6	// gn
	 "Convocatoria de examen"	// Okoteve traduccin
#elif L==7	// it
	 "Appello d'esame"
#elif L==8	// pl
	 "Ogloszenie egzamin"
#elif L==9	// pt
	 "Chamada para exame"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_SOCIAL_POST
	 "Missatge social"
#elif L==2	// de
	 "Social Beitrag"
#elif L==3	// en
	 "Social post"
#elif L==4	// es
	 "Mensaje social"
#elif L==5	// fr
	 "Message social"
#elif L==6	// gn
	 "Mensaje social"	// Okoteve traduccin
#elif L==7	// it
	 "Post sociale"
#elif L==8	// pl
	 "Post spo&lstrok;ecznej"
#elif L==9	// pt
	 "Post sociais"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_FORUM_POST
	 "Comentari en un f&ograve;rum"
#elif L==2	// de
	 "Beitr&auml;ge in einem Forum"
#elif L==3	// en
	 "Post in a forum"
#elif L==4	// es
	 "Comentario en un foro"
#elif L==5	// fr
	 "Post dans un forum"
#elif L==6	// gn
	 "Comentario en un foro"	// Okoteve traduccin
#elif L==7	// it
	 "Post in un forum"
#elif L==8	// pl
	 "Post na forum"
#elif L==9	// pt
	 "Post em um f&oacute;rum"
#endif
	 ,
#if   L==1	// ca	// TL_NOTE_NOTICE
	 "Av&iacute;s"
#elif L==2	// de
	 "Ank&uuml;ndigungen"
#elif L==3	// en
	 "Notice"
#elif L==4	// es
	 "Aviso"
#elif L==5	// fr
	 "Avis"
#elif L==6	// gn
	 "Marandu"
#elif L==7	// it
	 "Avviso"
#elif L==8	// pl
	 "Informacja"
#elif L==9	// pt
	 "An&uacute;ncio"
#endif
	};

const char *Txt_TIMELINE_NOTE_Favourite =
#if   L==1	// ca
	"Favorit";
#elif L==2	// de
	"Favorit";
#elif L==3	// en
	"Favourite";
#elif L==4	// es
	"Favorito";
#elif L==5	// fr
	"Favori";
#elif L==6	// gn
	"Favorito";	// Okoteve traduccin
#elif L==7	// it
	"Preferito";
#elif L==8	// pl
	"Ulubiony";
#elif L==9	// pt
	"Favorito";
#endif

const char *Txt_TIMELINE_NOTE_Favourited_by_X_USERS =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Marcat com favorit per %u";
#elif L==2	// de
	"Von %u vorgemerkt";
#elif L==3	// en
	"Favourited by %u";
#elif L==4	// es
	"Marcado como favorito por %u";
#elif L==5	// fr
	"Ajout&eacute; aux favoris par %u";
#elif L==6	// gn
	"Marcado como favorito por %u";	// Okoteve traduccin
#elif L==7	// it
	"Preferiti da %u";
#elif L==8	// pl
	"Dodane do ulubionych przez %u";
#elif L==9	// pt
	"Marcado como favorito por %u";
#endif

const char *Txt_TIMELINE_NOTE_Shared =
#if   L==1	// ca
	"Compartit";
#elif L==2	// de
	"Geteilt";
#elif L==3	// en
	"Shared";
#elif L==4	// es
	"Compartido";
#elif L==5	// fr
	"Partag&eacute;";
#elif L==6	// gn
	"Compartido";	// Okoteve traduccin
#elif L==7	// it
	"Condiviso";
#elif L==8	// pl
	"Udost&eogon;pnione";
#elif L==9	// pt
	"Compartilhado";
#endif

const char *Txt_TIMELINE_NOTE_Shared_by_X_USERS =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Compartit per %u";
#elif L==2	// de
	"Von %u geteilt";
#elif L==3	// en
	"Shared by %u";
#elif L==4	// es
	"Compartido por %u";
#elif L==5	// fr
	"Partag&eacute; par %u";
#elif L==6	// gn
	"Compartido por %u";	// Okoteve traduccin
#elif L==7	// it
	"Condiviso da %u";
#elif L==8	// pl
	"Udost&eogon;pnione przez %u";
#elif L==9	// pt
	"Compartilhado por %u";
#endif

const char *Txt_TIMELINE_NOTE_Not_favourited_by_anyone =	// No longer shared
#if   L==1	// ca
	"No marcat com favorit per ning&uacute;";
#elif L==2	// de
	"Niemand hat sie als Favorit markiert";
#elif L==3	// en
	"Not favourited by anyone";
#elif L==4	// es
	"No marcado como favorito por nadie";
#elif L==5	// fr
	"Personne n'a marqu&eacute; comme favori";
#elif L==6	// gn
	"No marcado como favorito por nadie";	// Okoteve traduccin
#elif L==7	// it
	"Non preferiti da chiunque";
#elif L==8	// pl
	"Nie przez nikogo ulubionych";
#elif L==9	// pt
	"N&atilde;o marcado como favorito por qualquer pessoa";
#endif

const char *Txt_TIMELINE_NOTE_Not_shared_by_anyone =	// No longer shared
#if   L==1	// ca
	"No compartit per ning&uacute;";
#elif L==2	// de
	"Nicht durch Dritte freigegebenen";
#elif L==3	// en
	"Not shared by anyone";
#elif L==4	// es
	"No compartido por nadie";
#elif L==5	// fr
	"Personne n'a partag&eacute;";
#elif L==6	// gn
	"No compartido por nadie";	// Okoteve traduccin
#elif L==7	// it
	"Non condiviso da chiunque";
#elif L==8	// pl
	"Nie jest dzielona przez nikogo";
#elif L==9	// pt
	"N&atilde;o compartilhado por qualquer pessoa";
#endif

const char *Txt_TIMELINE_NOTE_TOP_MESSAGES[TL_NUM_TOP_MESSAGES] =
	{
	// TL_TOP_MESSAGE_NONE
	""
	,
	// TL_TOP_MESSAGE_COMMENTED:
#if   L==1	// ca
	"ha comentat"
#elif L==2	// de
	"hat kommentiert"
#elif L==3	// en
	"has commented"
#elif L==4	// es
	"ha comentado"
#elif L==5	// fr
	"a comment&eacute;"
#elif L==6	// gn
	"ha comentado"	// Okoteve traduccin
#elif L==7	// it
	"ha commentato"
#elif L==8	// pl
	"skomentowa&lstrok;"
#elif L==9	// pt
	"comentou"
#endif
	,
	// TL_TOP_MESSAGE_FAVED:
#if   L==1	// ca
	"ha marcat com favorit"
#elif L==2	// de
	"hat als Favorit markiert"
#elif L==3	// en
	"has marked as favourite"
#elif L==4	// es
	"ha marcado como favorito"
#elif L==5	// fr
	"a marqu&eacute; en tant que favori"
#elif L==6	// gn
	"ha marcado como favorito"	// Okoteve traduccin
#elif L==7	// it
	"ha segnato come preferito"
#elif L==8	// pl
	"oznaczone jako ulubiony"
#elif L==9	// pt
	"marcou como favorito"
#endif
	,
	// TL_TOP_MESSAGE_UNFAVED:
#if   L==1	// ca
	"ha desmarcat com favorit"
#elif L==2	// de
	"hat als Favorit unmarkiert"
#elif L==3	// en
	"has unmarked as favourite"
#elif L==4	// es
	"ha desmarcado como favorito"
#elif L==5	// fr
	"a pas marqu&eacute; en tant que favori"
#elif L==6	// gn
	"ha desmarcado como favorito"	// Okoteve traduccin
#elif L==7	// it
	"ha smarcato come preferito"
#elif L==8	// pl
	"nieoznaczonych jako ulubiony"
#elif L==9	// pt
	"desmarcou como favorito"
#endif
	,
	// TL_TOP_MESSAGE_SHARED:
#if   L==1	// ca
	"ha compartit"
#elif L==2	// de
	"hat sich geteilt"
#elif L==3	// en
	"has shared"
#elif L==4	// es
	"ha compartido"
#elif L==5	// fr
	"a partag&eacute;"
#elif L==6	// gn
	"ha compartido"	// Okoteve traduccin
#elif L==7	// it
	"ha condiviso"
#elif L==8	// pl
	"podzieli&lstrok;"
#elif L==9	// pt
	"compartilhou"
#endif
	,
	// TL_TOP_MESSAGE_UNSHARED:
#if   L==1	// ca
	"ha deixat de compartir"
#elif L==2	// de
	"hat Sharing gestoppt"
#elif L==3	// en
	"has stopped sharing"
#elif L==4	// es
	"ha dejado de compartir"
#elif L==5	// fr
	"a cess&eacute; de partage"
#elif L==6	// gn
	"ha dejado de compartir"	// Okoteve traduccin
#elif L==7	// it
	"ha smesso di condividere"
#elif L==8	// pl
	"przesta&lstrok; udost&eogon;pniania"
#elif L==9	// pt
	"parou compartilhar"
#endif
	,
	// TL_TOP_MESSAGE_MENTIONED:
#if   L==1	// ca
	"li ha esmentat"
#elif L==2	// de
	"hat Sie erw&auml;hnt"
#elif L==3	// en
	"has mentioned you"
#elif L==4	// es
	"le ha mencionado"
#elif L==5	// fr
	"vous a mentionn&eacute;"
#elif L==6	// gn
	"le ha mencionado"	// Okoteve traduccin
#elif L==7	// it
	"ti ha menzionato"
#elif L==8	// pl
	"wspomnia&lstrok; o Tobie"
#elif L==9	// pt
	"mencionou voc&ecirc;"
#endif
	};

const char *Txt_TIMELINE_post =
#if   L==1	// ca
	"publicaci&oacute;";
#elif L==2	// de
	"Publikation";
#elif L==3	// en
	"post";
#elif L==4	// es
	"publicaci&oacute;n";
#elif L==5	// fr
	"publication";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"pubblicazione";
#elif L==8	// pl
	"publikacja";
#elif L==9	// pt
	"publica&ccedil;&atilde;o";
#endif

const char *Txt_TIMELINE_Post_removed =
#if   L==1	// ca
	"Publicaci&oacute; eliminada.";
#elif L==2	// de
	"Publikation entfernt.";
#elif L==3	// en
	"Post removed.";
#elif L==4	// es
	"Publicaci&oacute;n eliminada.";
#elif L==5	// fr
	"Publication supprim&eacute;e.";
#elif L==6	// gn
	"Publicaci&oacute;n eliminada.";		// Okoteve traduccin
#elif L==7	// it
	"Pubblicazione rimossa.";
#elif L==8	// pl
	"Publikacja usuni&eogon;ta.";
#elif L==9	// pt
	"Publica&ccedil;&atilde;o removida.";
#endif

const char *Txt_TIMELINE_posts =
#if   L==1	// ca
	"publicacions";
#elif L==2	// de
	"Publikationen";
#elif L==3	// en
	"posts";
#elif L==4	// es
	"publicaciones";
#elif L==5	// fr
	"publications";
#elif L==6	// gn
	"marandu";
#elif L==7	// it
	"pubblicazioni";
#elif L==8	// pl
	"publikacje";
#elif L==9	// pt
	"publica&ccedil;&otilde;es";
#endif

const char *Txt_TIMELINE_WHICH_USERS[TL_NUM_WHICH_USRS] =
   {
	 // TL_USRS_UNKNOWN
	 NULL
	 ,
	 // TL_USRS_FOLLOWED
#if   L==1	// ca
	"Usuaris que segueixo"
#elif L==2	// de
	"Benutzer Ich folge"
#elif L==3	// en
	"Users I'm following"
#elif L==4	// es
	"Usuarios que sigo"
#elif L==5	// fr
	"Utilisateurs qui je suis"
#elif L==6	// gn
	"Usuarios que sigo"	// Okoteve traduccin
#elif L==7	// it
	"Utenti che sto seguenti"
#elif L==8	// pl
	"U&zdot;ytkownicy, kt&oacute;rzy mam nast&eogon;puj&aogon;ce"
#elif L==9	// pt
	"Usu&aacute;rios que estou seguindo"
#endif
	 ,
	 // TL_USRS_ALL
#if   L==1	// ca
	 "Tots els usuaris"
#elif L==2	// de
	 "Alle Benutzer"
#elif L==3	// en
	 "All users"
#elif L==4	// es
	 "Todos los usuarios"
#elif L==5	// fr
	 "Tous les utilisateurs"
#elif L==6	// gn
	 "Todos los usuarios"	// Okoteve traduccin
#elif L==7	// it
	 "Tutti gli utenti"
#elif L==8	// pl
	 "Wszyscy u&zdot;ytkownicy"
#elif L==9	// pt
	 "Todos os usu&aacute;rios"
#endif
	};

const char *Txt_TIMETABLE_TYPES[TT_NUM_TIMETABLE_TYPES] =
	{
	// TT_COURSE_TIMETABLE
#if   L==1	// ca
	"Horari de classes de l'assignatura"
#elif L==2	// de
	"Stundenplan"
#elif L==3	// en
	"Class timetable"
#elif L==4	// es
	"Horario de clases de la asignatura"
#elif L==5	// fr
	"Horaire"
#elif L==6	// gn
	"Horario de clases de la asignatura"	// Okoteve traduccin
#elif L==7	// it
	"Orario del corso"
#elif L==8	// pl
	"Harmonogram klasy"
#elif L==9	// pt
	"Hor&aacute;rio"
#endif
	 ,
	// TT_MY_TIMETABLE
#if   L==1	// ca
	"Horari de les meves assignatures"
#elif L==2	// de
	"Stundenplan"
#elif L==3	// en
	"My timetable"
#elif L==4	// es
	"Horario de mis asignaturas"
#elif L==5	// fr
	"Horaire mes mati&egrave;res"
#elif L==6	// gn
	"Horario de mis asignaturas"	// Okoteve traduccin
#elif L==7	// it
	"Orario dei miei corsi"
#elif L==8	// pl
	"M&oacute;j plan lekcji"
#elif L==9	// pt
	"Hor&aacute;rio minha disciplinas"
#endif
	 ,
	// TT_TUTORING_TIMETABLE
#if   L==1	// ca
	"Horari de tutories"
#elif L==2	// de
	"Gesch&auml;ftszeiten"
#elif L==3	// en
	"Office hours"
#elif L==4	// es
	"Horario de tutor&iacute;as"
#elif L==5	// fr
	"Heures de tuteur"
#elif L==6	// gn
	"Horario de tutor&iacute;as"	// Okoteve traduccin
#elif L==7	// it
	"Orario di tutoraggi"
#elif L==8	// pl
	"Godziny pracy"
#elif L==9	// pt
	"Hor&aacute;rio de tutor"
#endif
	};

const char *Txt_TIMETABLE_CLASS_TYPES[TT_NUM_CLASS_TYPES] =
	{
   // TT_FREE
#if   L==1	// ca
	 "Lliure"
#elif L==2	// de
	 "Keine Klasse"
#elif L==3	// en
	 "No class"
#elif L==4	// es
	 "Libre"
#elif L==5	// fr
	 "Pas de classe"
#elif L==6	// gn
	 "Libre"			// Okoteve traduccin
#elif L==7	// it
	 "Libero"
#elif L==8	// pl
	 "Bez klasy"
#elif L==9	// pt
	 "Livre"
#endif
	 ,
	 // TT_LECTURE
#if   L==1	// ca
	 "Teoria"
#elif L==2	// de
	 "Vorlesung"
#elif L==3	// en
	 "Lectures"
#elif L==4	// es
	 "Teor&iacute;a"
#elif L==5	// fr
	 "Th&eacute;orie"
#elif L==6	// gn
	 "Teor&iacute;a"		// Okoteve traduccin
#elif L==7	// it
	 "Teoria"
#elif L==8	// pl
	 "Wyklady"
#elif L==9	// pt
	 "Teoria"
#endif
	 ,
	 // TT_PRACTICAL
#if   L==1	// ca
	 "Pr&agrave;ct."
#elif L==2	// de
	 "&Uuml;bung"
#elif L==3	// en
	 "Practicals"
#elif L==4	// es
	 "Pr&aacute;ct."
#elif L==5	// fr
	 "Pratique"
#elif L==6	// gn
	 "Pr&aacute;ct."		// Okoteve traduccin
#elif L==7	// it
	 "Pratica"
#elif L==8	// pl
	 "praktyki"
#elif L==9	// pt
	 "Pr&aacute;ct."
#endif
	 ,
	 // TT_TUTORING
#if   L==1	// ca
	 "Tutories"
#elif L==2	// de
	 "Anleitungen"
#elif L==3	// en
	 "Tutoring"
#elif L==4	// es
	 "Tutor&iacute;as"
#elif L==5	// fr
	 "Tutorat"
#elif L==6	// gn
	 "Tutor&iacute;as"	// Okoteve traduccin
#elif L==7	// it
	 "Tutoraggio"
#elif L==8	// pl
	 "Korepetycje"
#elif L==9	// pt
	 "Tutoria"
#endif
	};

const char *Txt_Title =
#if   L==1	// ca
	"T&iacute;tol";
#elif L==2	// de
	"Titel";
#elif L==3	// en
	"Title";
#elif L==4	// es
	"T&iacute;tulo";
#elif L==5	// fr
	"Titre";
#elif L==6	// gn
	"Moak&atilde;ha";
#elif L==7	// it
	"Titolo";
#elif L==8	// pl
	"Tytu&lstrok;";
#elif L==9	// pt
	"T&iacute;tulo";
#endif

const char *Txt_Title_attribution =
#if   L==1	// ca
	"T&iacute;tol/atribuci&oacute;";
#elif L==2	// de
	"Titel / Namensnennung";
#elif L==3	// en
	"Title/attribution";
#elif L==4	// es
	"T&iacute;tulo/atribuci&oacute;n";
#elif L==5	// fr
	"Titre/attribution";
#elif L==6	// gn
	"T&iacute;tulo/atribuci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Titolo/attribuzione";
#elif L==8	// pl
	"Tytu&lstrok;/przypisanie";
#elif L==9	// pt
	"T&iacute;tulo/atribui&ccedil;&atilde;o";
#endif

const char *Txt_To_remove_a_centre_you_must_first_remove_all_degrees_and_teachers_in_the_centre =
#if   L==1	// ca
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";	// Necessita traduccio
#elif L==2	// de
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Need bersetzung
#elif L==3	// en
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";
#elif L==4	// es
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";
#elif L==5	// fr
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Besoin de traduction
#elif L==6	// gn
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";	// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un centro, devi prima rimuovere"
	" tutte le lauree e professori di quel centro.";
#elif L==8	// pl
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Para remover um centro, voc&ecirc; deve primeiro remover"
	" todos os graus e professores no centro.";
#endif

const char *Txt_To_remove_a_course_you_must_first_remove_all_users_in_the_course =
#if   L==1	// ca
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";		// Necessita traduccio
#elif L==2	// de
	"To remove a course, you must first remove"
	" all users in the course.";				// Need bersetzung
#elif L==3	// en
	"To remove a course, you must first remove"
	" all users in the course.";
#elif L==4	// es
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";
#elif L==5	// fr
	"To remove a course, you must first remove"
	" all users in the course.";				// Besoin de traduction
#elif L==6	// gn
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";		// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un corso, devi prima rimuovere"
	" tutti gli utenti di quel corso.";
#elif L==8	// pl
	"To remove a course, you must first remove"
	" all users in the course.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Para remover uma disciplina, voc&ecirc; deve primeiro remover"
	" todos os usu&aacute;rios da disciplina.";
#endif

const char *Txt_To_remove_a_degree_you_must_first_remove_all_courses_in_the_degree =
#if   L==1	// ca
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Need bersetzung
#elif L==3	// en
	"To remove a degree, you must first remove"
	" all courses in the degree.";
#elif L==4	// es
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";
#elif L==5	// fr
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Besoin de traduction
#elif L==6	// gn
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere una laurea, devi prima rimuovere"
	" tutti i corsi della laurea.";
#elif L==8	// pl
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Para remover um grau, voc&ecirc; deve primeiro remover"
	" todas as disciplinas do grau.";
#endif

const char *Txt_To_remove_a_department_you_must_first_remove_all_teachers_in_the_department =
#if   L==1	// ca
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";	// Necessita traduccio
#elif L==2	// de
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Need bersetzung
#elif L==3	// en
	"To remove a department, you must first remove"
	" all teachers in the department.";
#elif L==4	// es
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";
#elif L==5	// fr
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Besoin de traduction
#elif L==6	// gn
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";	// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un dipartimento, devi prima rimuovere"
	" tutti i professori di quel dipartimento.";
#elif L==8	// pl
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Para remover um departamento, voc&ecirc; deve primeiro remover"
	" todos os professores do departamento.";
#endif

const char *Txt_To_remove_a_place_you_must_first_remove_all_centres_of_that_place =
#if   L==1	// ca
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";		// Necessita traduccio
#elif L==2	// de
	"To remove a place, you must first remove"
	" all centres of that place.";			// Need bersetzung
#elif L==3	// en
	"To remove a place, you must first remove"
	" all centres of that place.";
#elif L==4	// es
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";
#elif L==5	// fr
	"To remove a place, you must first remove"
	" all centres of that place.";			// Besoin de traduction
#elif L==6	// gn
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";		// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un luogo, devi prima rimuovere"
	" tutte le centri di quel luogo.";
#elif L==8	// pl
	"To remove a place, you must first remove"
	" all centres of that place.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Para remover uma localiza&ccedil;&atilde;o, voc&ecirc; deve primeiro remover"
	" todos os centros da localiza&ccedil;&atilde;o.";
#endif

const char *Txt_To_remove_a_type_of_degree_you_must_first_remove_all_degrees_of_that_type =
#if   L==1	// ca
	"Para eliminar un tipo de titulaci&oacute;n, primero debe eliminar"
	" todas las titulaciones de ese tipo.";		// Necessita traduccio
#elif L==2	// de
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Need bersetzung
#elif L==3	// en
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";
#elif L==4	// es
	"Para eliminar un tipo de titulaci&oacute;n, primero debe eliminar"
	" todas las titulaciones de ese tipo.";
#elif L==5	// fr
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Besoin de traduction
#elif L==6	// gn
	"Para eliminar un tipo de titulaci&oacute;n, primero debe eliminar"
	" todas las titulaciones de ese tipo.";		// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un tipo di laurea, devi prima rimuovere"
	" tutte le lauree di quel tipo.";
#elif L==8	// pl
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Para remover um tipo de grau, voc&ecirc; deve primeiro remover"
	" todos os graus desse tipo.";
#endif

const char *Txt_To_remove_an_institution_you_must_first_remove_all_centres_and_users_in_the_institution =
#if   L==1	// ca
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos los centros y todos los usuarios de esa instituci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Need bersetzung
#elif L==3	// en
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";
#elif L==4	// es
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos los centros y todos los usuarios de esa instituci&oacute;n.";
#elif L==5	// fr
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Besoin de traduction
#elif L==6	// gn
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos centros y todos los usuarios de esa instituci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Per rimuovere un'istituzione, devi prima rimuovere"
	" tutti gli centri e utenti di quell'istituzione.";
#elif L==8	// pl
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Para remover uma institui&ccedil;&atilde;o, voc&ecirc; deve primeiro remover"
	" todos os centros e usu&aacute;rios da institui&ccedil;&atilde;o.";
#endif

const char *Txt_To_use_chat_you_must_have_installed_the_software_X_and_add_Y_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==1	// ca
	"Per utilitzar el xat cal tenir instal lat el programari"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" i afegir <strong>https://%s</strong>"
	" a la llista de llocs permesos en el panell de control de Java.";
#elif L==2	// de
	"So verwenden Sie chatten, m&uuml;ssen Sie die Software installiert haben"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" und f&uuml;gen Sie <strong>https://%s</strong>"
	" in die Liste der erlaubten Seiten in der Java-Bedienfeld.";
#elif L==3	// en
	"To use chat you must have installed the software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" and add <strong>https://%s</strong>"
	" to the list of allowed sites in the Java control panel.";
#elif L==4	// es
	"Para usar el chat es necesario tener instalado el software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" y a&ntilde;adir <strong>https://%s</strong>"
	" a la lista de sitios permitidos en el panel de control de Java.";
#elif L==5	// fr
	"Pour utiliser le chat, vous devez avoir install&eacute; le logiciel"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" et ajouter <strong>https://%s</strong>"
	" &agrave; la liste des sites autoris&eacute;s dans le panneau de contr&ocirc;le Java.";
#elif L==6	// gn
	"Para usar el chat es necesario tener instalado el software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" y a&ntilde;adir <strong>https://%s</strong>"
	" a la lista de sitios permitidos en el panel de control de Java.";	// Okoteve traduccin
#elif L==7	// it
	"Per utilizzare la chat devi aver installato il software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" e aggiungere <strong>https://%s</strong>"
	" alla lista dei siti consentiti nel pannello di controllo Java.";
#elif L==8	// pl
	"To use chat you must have installed the software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" and add <strong>https://%s</strong>"
	" to the list of allowed sites in the Java control panel.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Para usar o chat &eacute; necess&aacute;rio ter instalado o software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" e adicionar <strong>https://%s</strong>"
	" &agrave; lista de sites permitidos no painel de controle Java.";
#endif

const char *Txt_To_watch_multimedia_content_from_another_website_you_have_to_accept_third_party_cookies_in_your_personal_settings =
#if   L==1	// ca
	"Per veure contingut multim&egrave;dia d'un altre lloc web"
	" ha d'acceptar les cookies de tercers"
	" en les seves par&agrave;metres personals.";
#elif L==2	// de
	"Um Multimedia-Inhalte von einer anderen Website anzuzeigen,"
	" m&uuml;ssen Sie in Ihren pers&ouml;nlichen Einstellungen Cookies"
	" von Drittanbietern akzeptieren.";
#elif L==3	// en
	"To watch multimedia content from another website"
	" you have to accept third party cookies"
	" in your personal settings.";
#elif L==4	// es
	"Para ver contenido multimedia de otro sitio web"
	" tiene que aceptar las cookies de terceros"
	" en sus ajustes personales.";
#elif L==5	// fr
	"Pour regarder le contenu multim&eacute;dia d'un autre site Web,"
	" vous devez accepter les cookies de tiers"
	" dans vos r&eacute;glages personnels.";
#elif L==6	// gn
	"Para ver contenido multimedia de otro sitio web"
	" tiene que aceptar las cookies de terceros"
	" en sus ajustes personales.";
#elif L==7	// it
	"Per visualizzare i contenuti multimediali di un altro sito web"
	" devi accettare i cookie di terze parti"
	" nelle tue impostazioni personali.";
#elif L==8	// pl
	"Aby przegl&aogon;da&cacute; tre&sacute;ci multimedialne z innej strony,"
	" musisz zaakceptowa&cacute; pliki cookie innych firm"
	" w swoich ustawieniach osobistych.";
#elif L==9	// pt
	"Para visualizar o conte&uacute;do multimdia de outro site,"
	" voc&ecirc; precisa aceitar cookies de terceiros"
	" nas suas configura&ccedil;&otilde;es pessoais.";
#endif

const char *Txt_Today =
#if   L==1	// ca
	"Avui";
#elif L==2	// de
	"Heute";
#elif L==3	// en
	"Today";
#elif L==4	// es
	"Hoy";
#elif L==5	// fr
	"Aujourd'hui";
#elif L==6	// gn
	"Ko &aacute;ra";
#elif L==7	// it
	"Oggi";
#elif L==8	// pl
	"Dzisiaj";
#elif L==9	// pt
	"Hoje";
#endif

const char *Txt_Total =
#if   L==1	// ca
	"Total";
#elif L==2	// de
	"Gesamtzahl";
#elif L==3	// en
	"Total";
#elif L==4	// es
	"Total";
#elif L==5	// fr
	"Total";
#elif L==6	// gn
	"Total";	// Okoteve traduccin
#elif L==7	// it
	"Totale";
#elif L==8	// pl
	"Ca&lstrok;kowity";
#elif L==9	// pt
	"Total";
#endif

const char *Txt_Total_BR_score =
#if   L==1	// ca
	"Puntuaci&oacute;<br />total";
#elif L==2	// de
	"Gesamt";
#elif L==3	// en
	"Total<br />score";
#elif L==4	// es
	"Puntuaci&oacute;n<br />total";
#elif L==5	// fr
	"Note<br />total";
#elif L==6	// gn
	"Puntuaci&oacute;n<br />total ";	// Okoteve traduccin
#elif L==7	// it
	"Punteggio<br />totale";
#elif L==8	// pl
	"Ca&lstrok;kowity<br />wynik";
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o<br />total";
#endif

const char *Txt_TST_Answer_given_by_the_teachers =
#if   L==1	// ca
	"Resposta donada pels professors";
#elif L==2	// de
	"Antwort von den Lehrern";
#elif L==3	// en
	"Answer given by the teachers";
#elif L==4	// es
	"Respuesta dada por los profesores";
#elif L==5	// fr
	"R&eacute;ponse donn&eacute;e par les professeurs";
#elif L==6	// gn
	"Respuesta dada por los profesores";	// Okoteve traduccin
#elif L==7	// it
	"Risposta fornita dagli insegnanti";
#elif L==8	// pl
	"Odpowied&zacute; udzielona przez nauczycieli";
#elif L==9	// pt
	"Resposta dada pelos professores";
#endif

const char *Txt_TST_Answer_given_by_the_user =
#if   L==1	// ca
	"Resposta donada per l'usuari/a";
#elif L==2	// de
	"Antwort vom Benutzer";
#elif L==3	// en
	"Answer given by the user";
#elif L==4	// es
	"Respuesta dada por el usuario/a";
#elif L==5	// fr
	"R&eacute;ponse donn&eacute;e par l'utilisateur/se";
#elif L==6	// gn
	"Respuesta dada por el usuario/a";	// Okoteve traduccin
#elif L==7	// it
	"Risposta fornita dall'utente";
#elif L==8	// pl
	"Odpowied&zacute; udzielona przez u&zdot;ytkownika";
#elif L==9	// pt
	"Resposta dada pelo usu&aacute;rio/a";
#endif

const char *Txt_TST_PLUGGABLE[Tst_NUM_OPTIONS_PLUGGABLE] =
	{
	// Tst_PLUGGABLE_UNKNOWN
	"",
	// Tst_PLUGGABLE_NO
#if   L==1	// ca
	"Tests no visibles des de m&ograve;bils (SWADroid, TriSWADos)"
#elif L==2	// de
	"Tests nicht zum Download aus Handys (SWADroid, TriSWADos)"
#elif L==3	// en
	"Tests not downloadable from mobile phones (SWADroid, TriSWADos)"
#elif L==4	// es
	"Tests no visibles desde m&oacute;viles (SWADroid, TriSWADos)"
#elif L==5	// fr
	"Tests non t&eacute;l&eacute;chargeables &agrave; partir de mobiles (SWADroid, TriSWADos)"
#elif L==6	// gn
	"Tests no visibles desde m&oacute;viles (SWADroid, TriSWADos)"	// Okoteve traduccin
#elif L==7	// it
	"Test non scaricabili da cellulari (SWADroid, TriSWADos)"
#elif L==8	// pl
	"Nie Testy pobrania z kom&oacute;rkowych (SWADroid, TriSWADos)"
#elif L==9	// pt
	"Testes n&atilde;o para download de celulares (SWADroid, TriSWADos)"
#endif
	,
	// Tst_PLUGGABLE_YES
#if   L==1	// ca
	"Tests visibles des de m&ograve;bils (SWADroid, TriSWADos), opci&oacute; recomanada"
#elif L==2	// de
	"Tests zum Download aus Handys (SWADroid, TriSWADos), empfohlene Option"
#elif L==3	// en
	"Tests downloadable from mobile phones (SWADroid, TriSWADos), recommended option"
#elif L==4	// es
	"Tests visibles desde m&oacute;viles (SWADroid, TriSWADos), opci&oacute;n recomendada"
#elif L==5	// fr
	"Tests t&eacute;l&eacute;chargeables &agrave; partir de mobiles (SWADroid, TriSWADos), option recommand&eacute;e"
#elif L==6	// gn
	"Tests visibles desde m&oacute;viles (SWADroid, TriSWADos), opci&oacute;n recomendada"// Okoteve traduccin
#elif L==7	// it
	"Test scaricabili da cellulari (SWADroid, TriSWADos), opzione consigliata"
#elif L==8	// pl
	"Testy do pobrania z kom&oacute;rkowych (SWADroid, TriSWADos), zalecana opcja"
#elif L==9	// pt
	"Testes para download de celulares (SWADroid, TriSWADos), op&ccedil;&atilde;o recomendada"
#endif
	};

const char *Txt_TST_STR_ANSWER_TYPES[Tst_NUM_ANS_TYPES] =
   {
#if   L==1	// ca
	 "Nombre enter"
#elif L==2	// de
	 "Ganze Zahl"
#elif L==3	// en
	 "Integer number"
#elif L==4	// es
	 "N&ordm; entero"
#elif L==5	// fr
	 "Nombre entier"
#elif L==6	// gn
	 "N&ordm; entero"	// Okoteve traduccin
#elif L==7	// it
	 "Numero intero"
#elif L==8	// pl
	 "Integer number"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "N&ordm; inteiro"
#endif
	 ,
#if   L==1	// ca
	 "Nombre real"
#elif L==2	// de
	 "Reelle Zahl"
#elif L==3	// en
	 "Real number"
#elif L==4	// es
	 "N&ordm; real"
#elif L==5	// fr
	 "Nombre r&eacute;el"
#elif L==6	// gn
	 "N&ordm; real"	// Okoteve traduccin
#elif L==7	// it
	 "Numero reale"
#elif L==8	// pl
	 "Liczba rzeczywista"
#elif L==9	// pt
	 "N&ordm; real"
#endif
	 ,
#if   L==1	// ca
	 "V/F"
#elif L==2	// de
	 "R/F"
#elif L==3	// en
	 "T/F"
#elif L==4	// es
	 "V/F"
#elif L==5	// fr
	 "V/F"
#elif L==6	// gn
	 "V/F"	// Okoteve traduccin
#elif L==7	// it
	 "V/F"
#elif L==8	// pl
	 "P/F"
#elif L==9	// pt
	 "V/F"
#endif
	 ,
#if   L==1	// ca
	 "Elecci&oacute; &uacute;nica"
#elif L==2	// de
	 "Unique-Choice"
#elif L==3	// en
	 "Unique choice"
#elif L==4	// es
	 "Elecci&oacute;n &uacute;nica"
#elif L==5	// fr
	 "Choix unique"
#elif L==6	// gn
	 "Elecci&oacute;n &uacute;nica"		// Okoteve traduccin
#elif L==7	// it
	 "Scelta unica"
#elif L==8	// pl
	 "Unikalny wyb&oacute;r"
#elif L==9	// pt
	 "&Uacute;nica escolha"
#endif
	 ,
#if   L==1	// ca
	 "Elecci&oacute; m&uacute;ltiple"
#elif L==2	// de
	 "Multiple-Choice"
#elif L==3	// en
	 "Multiple choice"
#elif L==4	// es
	 "Elecci&oacute;n m&uacute;ltiple"
#elif L==5	// fr
	 "Choix multiple"
#elif L==6	// gn
	 "Elecci&oacute;n m&uacute;ltiple"	// Okoteve traduccin
#elif L==7	// it
	 "Scelta multipla"
#elif L==8	// pl
	 "Wielokrotnego wyboru"
#elif L==9	// pt
	 "M&uacute;ltipla escolha"
#endif
	 ,
#if   L==1	// ca
	 "Text"
#elif L==2	// de
	 "Text"
#elif L==3	// en
	 "Text"
#elif L==4	// es
	 "Texto"
#elif L==5	// fr
	 "Texte"
#elif L==6	// gn
	 "Mba'ehaipyre"
#elif L==7	// it
	 "Testo"
#elif L==8	// pl
	 "Tekst"
#elif L==9	// pt
	 "Texto"
#endif
	,
	};

const char *Txt_TST_STR_FEEDBACK[Tst_NUM_TYPES_FEEDBACK] =
	{
#if   L==1	// ca
	 "Ninguna realimentaci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	 "Keine Feedback"
#elif L==3	// en
	 "No feedback"
#elif L==4	// es
	 "Ninguna realimentaci&oacute;n"
#elif L==5	// fr
	 "Pas de feedback"
#elif L==6	// gn
	 "Ninguna realimentaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Nessun feedback"
#elif L==8	// pl
	 "No feedback"				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Nenhum feedback"
#endif
	 ,
#if   L==1	// ca
	 "Poca: solo puntuaci&oacute;n total"	// Necessita traduccio
#elif L==2	// de
	 "Kleine: nur die Gesamtnote"
#elif L==3	// en
	 "Little: only the total score"
#elif L==4	// es
	 "Poca: solo puntuaci&oacute;n total"
#elif L==5	// fr
	 "Petit: seul le note total"
#elif L==6	// gn
	 "Poca: solo puntuaci&oacute;n total"	// Okoteve traduccin
#elif L==7	// it
	 "Poco: solo il punteggio totale"
#elif L==8	// pl
	 "Little: only the total score"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Pouco: apenas a pontua&ccedil;&atilde;o total"
#endif
	 ,
#if   L==1	// ca
	 "Media: puntuaci&oacute;n de cada pregunta"	// Necessita traduccio
#elif L==2	// de
	 "Medium: Ergebnisse der einzelnen Frage"
#elif L==3	// en
	 "Medium: result of each question"
#elif L==4	// es
	 "Media: puntuaci&oacute;n de cada pregunta"
#elif L==5	// fr
	 "Medium: le r&eacute;sultat de chaque question"
#elif L==6	// gn
	 "Media: puntuaci&oacute;n de cada pregunta"	// Okoteve traduccin
#elif L==7	// it
	 "Medio: risultato di ogni domanda"
#elif L==8	// pl
	 "Medium: result of each question"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "M&eacute;dio: pontua&ccedil;&atilde;o de cada quest&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "Alta: respuesta correcta en cada pregunta"			// Necessita traduccio
#elif L==2	// de
	 "Maximale Details: die richtige Antwort f&uuml;r jede Frage"
#elif L==3	// en
	 "High: correct answer for each question"
#elif L==4	// es
	 "Alta: respuesta correcta en cada pregunta"
#elif L==5	// fr
	 "Haute: bonne r&eacute;ponse pour chaque question"
#elif L==6	// gn
	 "Alta: respuesta correcta en cada pregunta"			// Okoteve traduccin
#elif L==7	// it
	 "Alto: la corretta risposta per ogni domanda"
#elif L==8	// pl
	 "High: correct answer for each question"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Alto: resposta correta para cada quest&atilde;o"
#endif
	 ,
#if   L==1	// ca
	 "M&aacute;xima: mostrar texto de realimentaci&oacute;n"	// Necessita traduccio
#elif L==2	// de
	 "Maximale: zeigen Feedback text"
#elif L==3	// en
	 "Maximum: show feedback text"
#elif L==4	// es
	 "M&aacute;xima: mostrar texto de realimentaci&oacute;n"
#elif L==5	// fr
	 "Maximum: afficher le texte de feedback"
#elif L==6	// gn
	 "M&aacute;xima: mostrar texto de realimentaci&oacute;n"	// Okoteve traduccin
#elif L==7	// it
	 "Massimo: visualizzare il testo de feedback"
#elif L==8	// pl
	 "Maximum: show feedback text"					// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "M&aacute;ximo: mostrar o texto de feedback"
#endif
	};

const char *Txt_TST_STR_ORDER_FULL[Tst_NUM_TYPES_ORDER_QST] =
	{
#if   L==1	// ca	// Tst_ORDER_STEM
	"Ordenar por enunciado de la pregunta"	// Necessita traduccio
#elif L==2	// de
	"Sortieren nach Stammzellen in Frage"
#elif L==3	// en
	"Sort by stem of the question"
#elif L==4	// es
	"Ordenar por enunciado de la pregunta"
#elif L==5	// fr
	"Trier par question"
#elif L==6	// gn
	"Ordenar por enunciado de la pregunta"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per domanda"
#elif L==8	// pl
	"Sort by stem of the question"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Classificar por formula&ccedil;&atilde;o da quest&atilde;o"
#endif
	,
#if   L==1	// ca	// Tst_ORDER_NUM_HITS
	"Ordenar por n&ordm; de veces que se ha respondido la pregunta"	// Necessita traduccio
#elif L==2	// de
	"Sortieren nach Anzahl der mal die Frage beantwortet wurde"
#elif L==3	// en
	"Sort by number of times the question has been answered"
#elif L==4	// es
	"Ordenar por n&ordm; de veces que se ha respondido la pregunta"
#elif L==5	// fr
	"Trier par nombre de fois qu'on a r&eacute;pondu &agrave; la question"
#elif L==6	// gn
	"Ordenar por n&ordm; de veces que se ha respondido la pregunta"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di volte che hai risposto alla domanda"
#elif L==8	// pl
	"Sort by number of times the question has been answered"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Classificar por n&ordm; de vezes que a quest&atilde;o foi respondida"
#endif
	,
#if   L==1	// ca	// TST_ORDER_AVERAGE_SCORE
	"Ordenar por puntuaci&oacute;n media"	// Necessita traduccio
#elif L==2	// de
	"Sortieren nach Durchschnittsnote"
#elif L==3	// en
	"Sort by average score"
#elif L==4	// es
	"Ordenar por puntuaci&oacute;n media"
#elif L==5	// fr
	"Trier par note moyenne"
#elif L==6	// gn
	"Ordenar por puntuaci&oacute;n media"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per punteggio medio"
#elif L==8	// pl
	"Sort by average score"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Classificar por pontua&ccedil;&atilde;o m&eacute;dia"
#endif
#if   L==1	// ca	// Tst_ORDER_NUM_HITS_NOT_BLANK
	"Ordenar por n&ordm; de veces que se ha respondido"
	" la pregunta (excluyendo respuestas en blanco)"	// Necessita traduccio
#elif L==2	// de
	"Sortieren nach Anzahl der mal"
	" die Frage beantwortet wurde (nicht leer)"
#elif L==3	// en
	"Sort by number of times the question"
	" has been answered (excluding blank answers)"
#elif L==4	// es
	"Ordenar por n&ordm; de veces que se ha respondido"
	" la pregunta (excluyendo respuestas en blanco)"
#elif L==5	// fr
	"Trier par nombre de fois qu'on a r&eacute;pondu"
	" &agrave; la question (non vide)"
#elif L==6	// gn
	"Ordenar por n&ordm; de veces que se ha respondido"
	" la pregunta (excluyendo respuestas en blanco)"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per numero di volte che hai risposto"
	" alla domanda (non vuoto)"
#elif L==8	// pl
	"Sort by number of times the question"
	" has been answered (excluding blank answers)"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Classificar por n&ordm; de vezes"
	" que a quest&atilde;o foi respondida (n&atilde;o em branco)"
#endif
	,
#if   L==1	// ca	// TST_ORDER_AVERAGE_SCORE_NOT_BLANK
	"Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"	// Necessita traduccio
#elif L==2	// de
	"Sortieren nach Durchschnittsnote (nicht leer)"
#elif L==3	// en
	"Sort by average score (excluding blank answers)"
#elif L==4	// es
	"Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"
#elif L==5	// fr
	"Trier par note moyenne (non vide)"
#elif L==6	// gn
	"Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"	// Okoteve traduccin
#elif L==7	// it
	"Ordina per punteggio medio (non vuoto)"
#elif L==8	// pl
	"Sort by average score (excluding blank answers)"				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Classificar por pontua&ccedil;&atilde;o m&eacute;dia (n&atilde;o em branco)"
#endif
	};

const char *Txt_TST_STR_ORDER_SHORT[Tst_NUM_TYPES_ORDER_QST] =
	{
#if   L==1	// ca	// Tst_ORDER_STEM
	"Pregunta"
#elif L==2	// de
	"Frage"
#elif L==3	// en
	"Question"
#elif L==4	// es
	"Pregunta"
#elif L==5	// fr
	"Question"
#elif L==6	// gn
	"Pregunta"	// Okoteve traduccin
#elif L==7	// it
	"Domanda"
#elif L==8	// pl
	"Pytanie"
#elif L==9	// pt
	"Quest&atilde;o"
#endif
	,
#if   L==1	// ca	// Tst_ORDER_NUM_HITS
	"N&ordm;<br />resp."			// Necessita traduccio
#elif L==2	// de
	"Anzahl<br />Antw."
#elif L==3	// en
	"No.<br />ans."
#elif L==4	// es
	"N&ordm;<br />resp."
#elif L==5	// fr
	"Nombre<br />r&eacute;p."
#elif L==6	// gn
	"N&ordm;<br />resp."			// Okoteve traduccin
#elif L==7	// it
	"Num.<br />risp."
#elif L==8	// pl
	"No.<br />ans."			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&ordm;<br />resp."
#endif
	,
#if   L==1	// ca	// TST_ORDER_AVERAGE_SCORE
	"Puntuaci&oacute;n<br />media"	// Necessita traduccio
#elif L==2	// de
	"Durchschnitt"
#elif L==3	// en
	"Average<br />score"
#elif L==4	// es
	"Puntuaci&oacute;n<br />media"
#elif L==5	// fr
	"Note<br />moyenne"
#elif L==6	// gn
	"Puntuaci&oacute;n<br />media"	// Okoteve traduccin
#elif L==7	// it
	"Punteggio<br />medio"
#elif L==8	// pl
	"Average<br />score"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o<br />m&eacute;dia"
#endif
	,
#if   L==1	// ca	// Tst_ORDER_NUM_HITS
	"N&ordm;<br />resp.<br />no en<br />blanco"		// Necessita traduccio
#elif L==2	// de
	"Anzahl<br />Antw.<br />nicht<br />leer"
#elif L==3	// en
	"No.<br />ans.<br />not<br />blank"
#elif L==4	// es
	"N&ordm;<br />resp.<br />no en<br />blanco"
#elif L==5	// fr
	"Nombre<br />r&eacute;p.<br />non<br />vide"
#elif L==6	// gn
	"N&ordm;<br />resp.<br />no en<br />blanco"		// Okoteve traduccin
#elif L==7	// it
	"Num.<br />risp.<br />non<br />vuoto"
#elif L==8	// pl
	"No.<br />ans.<br />not<br />blank"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&ordm;<br />resp.<br />n&atilde;o em<br />branco"
#endif
	,
#if   L==1	// ca	// TST_ORDER_AVERAGE_SCORE
	"Puntuaci&oacute;n<br />media<br />no en<br />blanco"	// Necessita traduccio
#elif L==2	// de
	"Durchschnitt<br />nicht<br />leer"
#elif L==3	// en
	"Average<br />score<br />not<br />blank"
#elif L==4	// es
	"Puntuaci&oacute;n<br />media<br />no en<br />blanco"
#elif L==5	// fr
	"Note<br />moyenne<br />non<br />vide"
#elif L==6	// gn
	"Puntuaci&oacute;n<br />media<br />no en<br />blanco"	// Okoteve traduccin
#elif L==7	// it
	"Punteggio<br />medio<br />non<br />vuoto"
#elif L==8	// pl
	"Average<br />score<br />not<br />blank"			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Pontua&ccedil;&atilde;o<br />m&eacute;dia<br />n&atilde;o em<br />branco"
#endif
	};

const char *Txt_TF_QST[2] =
   {
#if   L==1	// ca	// True
	 "V"
#elif L==2	// de
	 "R"
#elif L==3	// en
	 "T"
#elif L==4	// es
	 "V"
#elif L==5	// fr
	 "V"
#elif L==6	// gn
	 "V"	// Okoteve traduccin
#elif L==7	// it
	 "V"
#elif L==8	// pl
	 "P"
#elif L==9	// pt
	 "V"
#endif
	 ,
#if   L==1	// ca	// False
	 "F"
#elif L==2	// de
	 "F"
#elif L==3	// en
	 "F"
#elif L==4	// es
	 "F"
#elif L==5	// fr
	 "F"
#elif L==6	// gn
	 "F"	// Okoteve traduccin
#elif L==7	// it
	 "F"
#elif L==8	// pl
	 "F"
#elif L==9	// pt
	 "F"
#endif
	};

const char *Txt_Type =
#if   L==1	// ca
	"Tipus";
#elif L==2	// de
	"Art";
#elif L==3	// en
	"Type";
#elif L==4	// es
	"Tipo";
#elif L==5	// fr
	"Type";
#elif L==6	// gn
	"Teko";
#elif L==7	// it
	"Tipo";
#elif L==8	// pl
	"Typ";
#elif L==9	// pt
	"Tipo";
#endif

const char *Txt_Type_of_BR_answers =		// Answers of a test
#if   L==1	// ca
	"Tipus de<br />resposte";
#elif L==2	// de
	"Antworttype";
#elif L==3	// en
	"Type of<br />answers";
#elif L==4	// es
	"Tipo de<br />respuestas";
#elif L==5	// fr
	"Type de<br />r&eacute;ponses";
#elif L==6	// gn
	"Tipo de<br />respuestas";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di<br />risposte";
#elif L==8	// pl
	"Typu<br />odpowiedzi";
#elif L==9	// pt
	"Tipo de<br />respostas";
#endif

const char *Txt_Type_of_degree =
#if   L==1	// ca
	"Tipo de titulaci&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Abschlussart";
#elif L==3	// en
	"Type of degree";
#elif L==4	// es
	"Tipo de titulaci&oacute;n";
#elif L==5	// fr
	"Type d'&eacute;tude";
#elif L==6	// gn
	"Tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di laurea";
#elif L==8	// pl
	"Type of degree";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipo de grau";
#endif

const char *Txt_Type_of_degree_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2	// de
	"Type of degree <strong>%s</strong> removed.";			// bersetzung notwendig
#elif L==3	// en
	"Type of degree <strong>%s</strong> removed.";
#elif L==4	// es
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";
#elif L==5	// fr
	"Type of degree <strong>%s</strong> removed.";			// Besoin de traduction
#elif L==6	// gn
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di laurea <strong>%s</strong> rimossa.";
#elif L==8	// pl
	"Type of degree <strong>%s</strong> removed.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipo de grau <strong>%s</strong> removido.";
#endif

const char *Txt_Type_of_group =
#if   L==1	// ca
	"Tipo de grupo";	// Necessita traduccio
#elif L==2	// de
	"Art der Gruppe";
#elif L==3	// en
	"Type of group";
#elif L==4	// es
	"Tipo de grupo";
#elif L==5	// fr
	"Type de groupe";
#elif L==6	// gn
	"Tipo de grupo";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di gruppo";
#elif L==8	// pl
	"Type of group";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipo de grupo";
#endif

const char *Txt_Type_BR_of_group =
#if   L==1	// ca
	"Tipo<br />de grupo";	// Necessita traduccio
#elif L==2	// de
	"Art<br />der Gruppe";
#elif L==3	// en
	"Type<br />of group";
#elif L==4	// es
	"Tipo<br />de grupo";
#elif L==5	// fr
	"Type<br />de groupe";
#elif L==6	// gn
	"Tipo<br />de grupo";	// Okoteve traduccin
#elif L==7	// it
	"Tipo<br />di gruppo";
#elif L==8	// pl
	"Type<br />of group";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipo<br />de grupo";
#endif

const char *Txt_Type_of_group_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Tipo de grupo <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2	// de
	"Gruppen-Typ <strong>%s</strong> wurde entfernt.";
#elif L==3	// en
	"Type of group <strong>%s</strong> removed.";
#elif L==4	// es
	"Tipo de grupo <strong>%s</strong> eliminado.";
#elif L==5	// fr
	"Type of group <strong>%s</strong> removed.";	// Besoin de traduction
#elif L==6	// gn
	"Tipo de grupo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7	// it
	"Tipo di gruppo <strong>%s</strong> rimosso.";
#elif L==8	// pl
	"Type of group <strong>%s</strong> removed.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipo de grupo <strong>%s</strong> removido.";
#endif

const char *Txt_Type_or_paste_a_list_of_IDs_nicks_or_emails_ =
#if   L==1	// ca
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" Solo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";		// Necessita traduccio
#elif L==2	// de
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Need bersetzung
#elif L==3	// en
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";
#elif L==4	// es
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" Solo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";
#elif L==5	// fr
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Besoin de traduction
#elif L==6	// gn
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" Solo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";		// Okoteve traduccin
#elif L==7	// it
	"Scrivi o incolla la lista"
	" di @nome-utenti, emails o IDs."
	" I @nome-utenti, emails o IDs"
	" possono essere separati da caratteri arbitrari"
	" (eg. nomi, luoghi, etc.)."
	" Solo le parole che sembrano"
	" @nome-utenti, emails o IDs"
	" saranno presi in considerazione;"
	" il resto del testo sar&agrave; ignorato.";
#elif L==8	// pl
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Digite ou cole uma lista"
	" de @alcunhas, emails ou IDs."
	" As @alcunhas, os emails ou os IDs"
	" podem ser separados por strings arbitr&aacute;rias"
	" (por exemplo, nomes, lugares, etc.)."
	" Apenas as palavras que parecem @alcunhas, emails ou IDs"
	" ser%atilde;o levadas em conta;"
	" o resto do texto ser&aacute; ignorado.";
#endif

const char *Txt_Types_of_degree =
#if   L==1	// ca
	"Tipus de titulaci&oacute;";
#elif L==2	// de
	"Abschlussart";
#elif L==3	// en
	"Types of degree";
#elif L==4	// es
	"Tipos de titulaci&oacute;n";
#elif L==5	// fr
	"Types d'&eacute;tude";
#elif L==6	// gn
	"Arandur&atilde; l&aacute;ja";
#elif L==7	// it
	"Tipi di laurea";
#elif L==8	// pl
	"Rodzaje stopni";
#elif L==9	// pt
	"Tipos de grau";
#endif

const char *Txt_Types_of_answers =		// Answers of a test
#if   L==1	// ca
	"Tipus de respostes";
#elif L==2	// de
	"Antworttypen";
#elif L==3	// en
	"Types of answers";
#elif L==4	// es
	"Tipos de respuestas";
#elif L==5	// fr
	"Types de r&eacute;ponses";
#elif L==6	// gn
	"Tipos de respuestas";	// Okoteve traduccin
#elif L==7	// it
	"Tipi di risposte";
#elif L==8	// pl
	"Typy odpowiedzi";
#elif L==9	// pt
	"Tipos de respostas";
#endif

const char *Txt_Types_of_group =
#if   L==1	// ca
	"Tipus de grup";
#elif L==2	// de
	"Gruppen-Typen";
#elif L==3	// en
	"Types of group";
#elif L==4	// es
	"Tipos de grupo";
#elif L==5	// fr
	"Types de groupe";
#elif L==6	// gn
	"Tipos de grupo";	// Okoteve traduccin
#elif L==7	// it
	"Tipi di gruppo";
#elif L==8	// pl
	"Types of group";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Tipos de grupo";
#endif

const char *Txt_Undo =
#if   L==1	// ca
	"Desfer";
#elif L==2	// de
	"R&uuml;ckg&auml;ngig";
#elif L==3	// en
	"Undo";
#elif L==4	// es
	"Deshacer";
#elif L==5	// fr
	"Annuler";
#elif L==6	// gn
	"Deshacer";	// Okoteve traduccin
#elif L==7	// it
	"Annulla";
#elif L==8	// pl
	"Anulowa&cacute;";
#elif L==9	// pt
	"Desfazer";
#endif

const char *Txt_Unfollow =
#if   L==1	// ca
	"Deixar de seguir";
#elif L==2	// de
	"Entfolgen";
#elif L==3	// en
	"Unfollow";
#elif L==4	// es
	"Dejar de seguir";
#elif L==5	// fr
	"Se d&eacute;sabonner";
#elif L==6	// gn
	"Dejar de seguir";	// Okoteve traduccin
#elif L==7	// it
	"Smetti";
#elif L==8	// pl
	"Nie obserwuj";
#elif L==9	// pt
	"Deixar de seguir";
#endif

const char *Txt_Unfortunately_Firefox_and_Chrome_no_longer_allow_Java_to_run_ =
#if   L==1	// ca
	"Desafortunadament Firefox i Chrome ja no permeten executar Java,"
	" de manera que el xat no estar&agrave; disponible per a aquests navegadors"
	" fins que desenvolupem un nou xat amb una altra tecnologia."
	" Sentim les mol&egrave;sties.";
#elif L==2	// de
	"Leider keinen Firefox und Chrome l&auml;nger erlauben Ihnen Java zu laufen,"
	" so dass der Chat f&uuml;r diesen Browser nicht zur Verf&uuml;gung steht,"
	" bis wir einen neuen Chat mit anderer Technologie zu entwickeln."
	" Es tut uns leid.";
#elif L==3	// en
	"Unfortunately Firefox and Chrome no longer allow Java to run,"
	" so chat will not be available for these browsers"
	" until we develop a new chat with another technology."
	" We apologize for the inconveniences.";
#elif L==4	// es
	"Desafortunadamente Firefox y Chrome ya no permiten ejecutar Java,"
	" de modo que el chat no estar&aacute; disponible para estos navegadores"
	" hasta que desarrollemos un nuevo chat con otra tecnolog&iacute;a."
	" Sentimos las molestias.";
#elif L==5	// fr
	"Malheureusement, pas Firefox et Chrome plus vous permettent d'ex&eacute;cuter Java,"
	" donc le chat n'est pas disponible pour ces navigateurs"
	" jusqu'&agrave; ce que nous d&eacute;veloppons un nouveau chat avec d'autres technologies."
	" D&eacute;sol&eacute; pour la g&ecirc;ne occasionn&eacute;e.";
#elif L==6	// gn
	"Desafortunadamente Firefox y Chrome ya no permiten ejecutar Java,"
	" de modo que el chat no estar&aacute; disponible para estos navegadores"
	" hasta que desarrollemos un nuevo chat con otra tecnolog&iacute;a."
	" Sentimos las molestias.";	// Okoteve traduccin
#elif L==7	// it
	"Purtroppo Firefox e Chrome non consentono pi&ugrave; di eseguire Java,"
	" quindi la chat non &egrave; disponibile per questi browser"
	" finch&eacute; non sviluppiamo una nuova chat con altre tecnologie."
	" Ci scusiamo per l'inconveniente.";
#elif L==8	// pl
	"Niestety Firefox i Chrome nie pozwalaj&aogon; ju&zdot; na uruchamianie Java,"
	" wi&eogon;c rozmowa nie jest dost&eogon;pny dla tych przegl&aogon;darek"
	" a&zdot; opracowa&cacute; nowy czat z innymi technologiami."
	" Przepraszamy za niedogodno&sacute;ci.";
#elif L==9	// pt
	"Infelizmente Firefox e Chrome j&aacute; n&atilde;o permitem que voc&ecirc; execute o Java,"
	" de modo que o chat n&atilde;o est&aacute; dispon&iacute;vel para estes navegadores"
	" at&eacute; que desenvolver um novo chat com outra tecnologia."
	" Desculpe o transtorno.";
#endif

const char *Txt_unknown_assignment =
#if   L==1	// ca
	"activitat desconeguda";
#elif L==2	// de
	"Aufgabe unbekannt";
#elif L==3	// en
	"unknown assignment";
#elif L==4	// es
	"actividad desconocida";
#elif L==5	// fr
	"activit&eacute; inconnue";
#elif L==6	// gn
	"actividad desconocida";	// Okoteve traduccin
#elif L==7	// it
	"attivit&agrave; sconosciuta";
#elif L==8	// pl
	"unknown assignment";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"atividade desconhecida";
#endif

const char *Txt_Unknown_FORUM =
#if   L==1	// ca
	"Desconocido";	// Necessita traduccio
#elif L==2	// de
	"Unbekannt";
#elif L==3	// en
	"Unknown";
#elif L==4	// es
	"Desconocido";
#elif L==5	// fr
	"Inconnu";
#elif L==6	// gn
	"Desconocido";	// Okoteve traduccin
#elif L==7	// it
	"Sconosciuto";
#elif L==8	// pl
	"Unknown";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Desconhecido";
#endif

const char *Txt_unknown_recipient =
#if   L==1	// ca
	"destinatario desconocido";	// Necessita traduccio
#elif L==2	// de
	"Empf&auml;nger unbekannt";
#elif L==3	// en
	"unknown recipient";
#elif L==4	// es
	"destinatario desconocido";
#elif L==5	// fr
	"destinataire inconnu";
#elif L==6	// gn
	"destinatario desconocido";	// Okoteve traduccin
#elif L==7	// it
	"destinatario sconosciuto";
#elif L==8	// pl
	"unknown recipient";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"destinat&aacute;rio desconhecido";
#endif

const char *Txt_unknown_recipients =
#if   L==1	// ca
	"destinatarios desconocidos";	// Necessita traduccio
#elif L==2	// de
	"Empf&auml;nger unbekannt";
#elif L==3	// en
	"unknown recipients";
#elif L==4	// es
	"destinatarios desconocidos";
#elif L==5	// fr
	"destinataires inconnus";
#elif L==6	// gn
	"destinatarios desconocidos";	// Okoteve traduccin
#elif L==7	// it
	"destinatari sconosciuti";
#elif L==8	// pl
	"unknown recipients";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"destinat&aacute;rios desconhecidos";
#endif

const char *Txt_unknown_removed_course =
#if   L==1	// ca
	"assignatura desconeguda / eliminada";
#elif L==2	// de
	"Kurs unbekannt / entfernt";
#elif L==3	// en
	"unknown / removed course";
#elif L==4	// es
	"asignatura desconocida / eliminada";
#elif L==5	// fr
	"mati&egrave;re inconnue / supprim&eacute;e";
#elif L==6	// gn
	"asignatura desconocida / eliminada";	// Okoteve traduccin
#elif L==7	// it
	"corso sconosciuto / rimosso";
#elif L==8	// pl
	"kurs nieznany / usuni&eogon;te";
#elif L==9	// pt
	"disciplina desconhecida / removida";
#endif

const char *Txt_unknown_TIME =
#if   L==1	// ca
	"desconocido";	// Necessita traduccio
#elif L==2	// de
	"unbekannt";
#elif L==3	// en
	"unknown";
#elif L==4	// es
	"desconocido";
#elif L==5	// fr
	"inconnu";
#elif L==6	// gn
	"desconocido";	// Okoteve traduccin
#elif L==7	// it
	"sconosciuto";
#elif L==8	// pl
	"unknown";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"desconhecido";
#endif

const char *Txt_unread_MESSAGE =
#if   L==1	// ca
	"no le&iacute;do";	// Necessita traduccio
#elif L==2	// de
	"ungelesene";
#elif L==3	// en
	"unread";
#elif L==4	// es
	"no le&iacute;do";
#elif L==5	// fr
	"non lu";
#elif L==6	// gn
	"no le&iacute;do";	// Okoteve traduccin
#elif L==7	// it
	"non letto";
#elif L==8	// pl
	"unread";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"n&atilde;o lida";
#endif

const char *Txt_unread_MESSAGES =
#if   L==1	// ca
	"no le&iacute;dos";	// Necessita traduccio
#elif L==2	// de
	"ungelesene";
#elif L==3	// en
	"unread";
#elif L==4	// es
	"no le&iacute;dos";
#elif L==5	// fr
	"non lus";
#elif L==6	// gn
	"no le&iacute;dos";	// Okoteve traduccin
#elif L==7	// it
	"non letti";
#elif L==8	// pl
	"unread";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"n&atilde;o lidas";
#endif

const char *Txt_Unread_BR_msgs =
#if   L==1	// ca
	"No<br />le&iacute;dos";	// Necessita traduccio
#elif L==2	// de
	"Ungel.<br />Nachr.";
#elif L==3	// en
	"Unread<br />msgs.";
#elif L==4	// es
	"No<br />le&iacute;dos";
#elif L==5	// fr
	"Non<br />lus";
#elif L==6	// gn
	"No<br />le&iacute;dos";	// Okoteve traduccin
#elif L==7	// it
	"Non<br />letti";
#elif L==8	// pl
	"Unread<br />msgs.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&atilde;o<br />lidas";
#endif

const char *Txt_Update =
#if   L==1	// ca
	"Actualitzar";
#elif L==2	// de
	"Update";
#elif L==3	// en
	"Update";
#elif L==4	// es
	"Actualizar";
#elif L==5	// fr
	"Mise &agrave; jour";
#elif L==6	// gn
	"Actualizar";	// Okoteve traduccin
#elif L==7	// it
	"Aggiornamento";
#elif L==8	// pl
	"Update";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Atualizar";
#endif

const char *Txt_Update_attendance =
#if   L==1	// ca
	"Actualitzar assist&egrave;ncia";
#elif L==2	// de
	"Update Anwesenheit";
#elif L==3	// en
	"Update attendance";
#elif L==4	// es
	"Actualizar asistencia";
#elif L==5	// fr
	"Mise &agrave; jour pr&eacute;sence";
#elif L==6	// gn
	"Actualizar asistencia";	// Okoteve traduccin
#elif L==7	// it
	"Aggiornamento presenza";
#elif L==8	// pl
	"Aktualizacja bytno&sacute;&cacute;";
#elif L==9	// pt
	"Atualizar presen&ccedil;a";
#endif

const char *Txt_Update_messages =
#if   L==1	// ca
	"Actualitzar missatges";
#elif L==2	// de
	"Update Nachrichten";
#elif L==3	// en
	"Update messages";
#elif L==4	// es
	"Actualizar mensajes";
#elif L==5	// fr
	"Mise &agrave; jour des messages";
#elif L==6	// gn
	"Actualizar mensajes";		// Okoteve traduccin
#elif L==7	// it
	"Aggiornamento messaggi";
#elif L==8	// pl
	"Wiadomo&sacute;ci aktualizacji";
#elif L==9	// pt
	"Atualizar mensagens";
#endif

const char *Txt_Update_users =
#if   L==1	// ca
	"Actualitzar usuaris";
#elif L==2	// de
	"Update Benutzer";
#elif L==3	// en
	"Update users";
#elif L==4	// es
	"Actualizar usuarios";
#elif L==5	// fr
	"Mise &agrave; jour des &eacute;tudiants";
#elif L==6	// gn
	"Actualizar usuarios";	// Okoteve traduccin
#elif L==7	// it
	"Aggiornamento utenti";
#elif L==8	// pl
	"Zaktualizuj u&zdot;ytkownik&oacute;w";
#elif L==9	// pt
	"Atualizar os utilizadores";
#endif

const char *Txt_UPLOAD_FILE_Invalid_link =
#if   L==1	// ca
	"Enlace no v&aacute;lido.";	// Necessita traduccio
#elif L==2	// de
	"Invalid link.";		// Need bersetzung
#elif L==3	// en
	"Invalid link.";
#elif L==4	// es
	"Enlace no v&aacute;lido.";
#elif L==5	// fr
	"Invalid link.";		// Besoin de traduction
#elif L==6	// gn
	"Enlace no v&aacute;lido.";	// Okoteve traduccin
#elif L==7	// it
	"Link non valido.";
#elif L==8	// pl
	"Invalid link.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Link n&atilde;o v&aacute;lido.";
#endif

const char *Txt_UPLOAD_FILE_Invalid_name =
#if   L==1	// ca
	"Nombre no v&aacute;lido.";	// Necessita traduccio
#elif L==2	// de
	"Invalid name.";		// Need bersetzung
#elif L==3	// en
	"Invalid name.";
#elif L==4	// es
	"Nombre no v&aacute;lido.";
#elif L==5	// fr
	"Invalid name.";		// Besoin de traduction
#elif L==6	// gn
	"Nombre no v&aacute;lido.";	// Okoteve traduccin
#elif L==7	// it
	"Nome non valido.";
#elif L==8	// pl
	"Invalid name.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Nome n&atilde;o v&aacute;lido.";
#endif

const char *Txt_UPLOAD_FILE_X_invalid_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"<strong>%s</strong>: nombre no v&aacute;lido.";	// Necessita traduccio
#elif L==2	// de
	"<strong>%s</strong>: invalid name.";			// Need bersetzung
#elif L==3	// en
	"<strong>%s</strong>: invalid name.";
#elif L==4	// es
	"<strong>%s</strong>: nombre no v&aacute;lido.";
#elif L==5	// fr
	"<strong>%s</strong>: invalid name.";			// Besoin de traduction
#elif L==6	// gn
	"<strong>%s</strong>: nombre no v&aacute;lido.";	// Okoteve traduccin
#elif L==7	// it
	"<strong>%s</strong>: nome non valido.";
#elif L==8	// pl
	"<strong>%s</strong>: invalid name.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"<strong>%s</strong>: nome n&atilde;o v&aacute;lido.";
#endif

const char *Txt_Upload_file =
#if   L==1	// ca
	"Enviar archivo";	// Necessita traduccio
#elif L==2	// de
	"Upload Datei";
#elif L==3	// en
	"Upload file";
#elif L==4	// es
	"Enviar archivo";
#elif L==5	// fr
	"T&eacute;l&eacute;charger fichier";
#elif L==6	// gn
	"Enviar archivo";	// Okoteve traduccin
#elif L==7	// it
	"Caricare file";
#elif L==8	// pl
	"Upload file";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Fazer upload de arquivo";
#endif

const char *Txt_Upload_file_or_create_folder_in_FOLDER =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"A&ntilde;adir archivos o crear carpeta en %s";	// Necessita traduccio
#elif L==2	// de
	"Datei uploaden oder Verzeichnis in %s erstellen";
#elif L==3	// en
	"Upload files or create folder in %s";
#elif L==4	// es
	"A&ntilde;adir archivos o crear carpeta en %s";
#elif L==5	// fr
	"Charger fichiers ou cr&eacute;er r&eacute;pertoire dans %s";
#elif L==6	// gn
	"A&ntilde;adir archivos o crear carpeta en %s";	// Okoteve traduccin
#elif L==7	// it
	"Caricare file o creare cartella in %s";
#elif L==8	// pl
	"Upload file or create folder in %s";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Fazer upload de arquivos ou criar diret&oacute;rio em %s";
#endif

const char *Txt_Upload_files =
#if   L==1	// ca
	"Enviar fitxers";
#elif L==2	// de
	"Upload Dateien";
#elif L==3	// en
	"Upload files";
#elif L==4	// es
	"Subir archivos";
#elif L==5	// fr
	"T&eacute;l&eacute;charger fichiers";
#elif L==6	// gn
	"Subir archivos";	// Okoteve traduccin
#elif L==7	// it
	"Caricare file";
#elif L==8	// pl
	"Upload files";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Fazer upload de arquivos";
#endif

const char *Txt_Upload_files_QUESTION =
#if   L==1	// ca
	"Enviar<br />fitxers?";
#elif L==2	// de
	"Upload<br />Dateien?";
#elif L==3	// en
	"Upload<br />files?";
#elif L==4	// es
	"&iquest;Enviar<br />archivos?";
#elif L==5	// fr
	"T&eacute;l&eacute;charger<br />fichiers?";
#elif L==6	// gn
	"&iquest;Enviar<br />archivos?";	// Okoteve traduccin
#elif L==7	// it
	"Caricare<br />file?";
#elif L==8	// pl
	"Upload<br />files?";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Fazer upload<br />de arquivos?";
#endif

const char *Txt_Upload_logo =
#if   L==1	// ca
	"Enviar logo";
#elif L==2	// de
	"Upload Logo";
#elif L==3	// en
	"Upload logo";
#elif L==4	// es
	"Subir logo";
#elif L==5	// fr
	"T&eacute;l&eacute;charger logo";
#elif L==6	// gn
	"Subir logo";		// Okoteve traduccin
#elif L==7	// it
	"Caricare logo";
#elif L==8	// pl
	"Upload logo";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Fazer upload de logotipo";
#endif

const char *Txt_Upload_my_picture =
#if   L==1	// ca
	"Pujar la meva fotografia";
#elif L==2	// de
	"Hochladen Mein Foto";
#elif L==3	// en
	"Upload my picture";
#elif L==4	// es
	"Subir mi fotograf&iacute;a";
#elif L==5	// fr
	"Ajouter ma photo";
#elif L==6	// gn
	"Subir mi fotograf&iacute;a";		// Okoteve traduccin
#elif L==7	// it
	"Caricare una foto";
#elif L==8	// pl
	"Prze&sacute;lij swoje zdj&eogon;cia";
#elif L==9	// pt
	"Carregar minha foto";
#endif

const char *Txt_Upload_photo =
#if   L==1	// ca
	"Enviar foto";
#elif L==2	// de
	"Upload Foto";
#elif L==3	// en
	"Upload photo";
#elif L==4	// es
	"Subir foto";
#elif L==5	// fr
	"T&eacute;l&eacute;charger photo";
#elif L==6	// gn
	"Subir foto";		// Okoteve traduccin
#elif L==7	// it
	"Caricare foto";
#elif L==8	// pl
	"Upload photo";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Upload foto";
#endif

const char *Txt_Uploaded_by =
#if   L==1	// ca
	"Penjat per";
#elif L==2	// de
	"Hochgeladen von";
#elif L==3	// en
	"Uploaded by";
#elif L==4	// es
	"Subido por";
#elif L==5	// fr
	"Post&eacute; par";
#elif L==6	// gn
	"Subido por";	// Okoteve traduccin
#elif L==7	// it
	"Caricato da";
#elif L==8	// pl
	"Wyslane przez";
#elif L==9	// pt
	"Enviado por";
#endif

const char *Txt_URL =		// Uniform Resource Locator
#if   L==1	// ca
	"URL";
#elif L==2	// de
	"URL";
#elif L==3	// en
	"URL";
#elif L==4	// es
	"URL";
#elif L==5	// fr
	"URL";
#elif L==6	// gn
	"URL";	// Okoteve traduccin
#elif L==7	// it
	"URL";
#elif L==8	// pl
	"URL";
#elif L==9	// pt
	"URL";
#endif

const char *Txt_Use_this_email =
#if   L==1	// ca
	"Utilitzar aquest correu";
#elif L==2	// de
	"Verwenden Sie diese E-Mail";
#elif L==3	// en
	"Use this email";
#elif L==4	// es
	"Usar este correo";
#elif L==5	// fr
	"Utiliser ce courrier";
#elif L==6	// gn
	"Usar este correo";	// Okoteve traduccin
#elif L==7	// it
	"Utilizzare questa email";
#elif L==8	// pl
	"U&zdot;yj tej email";
#elif L==9	// pt
	"Utilizar este email";
#endif

const char *Txt_Use_this_nickname =
#if   L==1	// ca
	"Utilitzar aquest sobrenom";
#elif L==2	// de
	"Verwenden Sie diese Benutzername";
#elif L==3	// en
	"Use this nickname";
#elif L==4	// es
	"Usar este apodo";
#elif L==5	// fr
	"Utiliser ce surnom";
#elif L==6	// gn
	"Usar este apodo";	// Okoteve traduccin
#elif L==7	// it
	"Utilizzare questo nome utente";
#elif L==8	// pl
	"U&zdot;yj tej nick";
#elif L==9	// pt
	"Utilizar esta alcunha";
#endif

const char *Txt_User[Usr_NUM_SEXS] =
	{	// Usr_SEX_UNKNOWN
#if   L==1	// ca
	"Usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario/a"
#elif L==5	// fr
	"Utilisateur/se"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador/a"
#endif
	,	// Usr_SEX_FEMALE
#if   L==1	// ca
	"Usuaria"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuaria"
#elif L==5	// fr
	"Utilisateuse"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizadora"
#endif
	,	// Usr_SEX_MALE
#if   L==1	// ca
	"Usuari"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario"
#elif L==5	// fr
	"Utilisateur"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador"
#endif
	,	// Usr_SEX_ALL
#if   L==1	// ca
	"Usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"User"
#elif L==4	// es
	"Usuario/a"
#elif L==5	// fr
	"Utilisateur/se"
#elif L==6	// gn
	"Puruh&atilde;ra"
#elif L==7	// it
	"Utente"
#elif L==8	// pl
	"U&zdot;ytkownika"
#elif L==9	// pt
	"Utilizador/a"
#endif
	};

const char *Txt_user[Usr_NUM_SEXS] =
	{	// Usr_SEX_UNKNOWN
#if   L==1	// ca
	"usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario/a"
#elif L==5	// fr
	"utilisateur/se"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador/a"
#endif
	,	// Usr_SEX_FEMALE
#if   L==1	// ca
	"usuaria"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuaria"
#elif L==5	// fr
	"utilisateuse"
#elif L==6	// gn
	"usuaria"	// Okoteve traduccin
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizadora"
#endif
	,	// Usr_SEX_MALE
#if   L==1	// ca
	"usuari"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario"
#elif L==5	// fr
	"utilisateur"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador"
#endif
	,	// Usr_SEX_ALL
#if   L==1	// ca
	"usuari/a"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"user"
#elif L==4	// es
	"usuario/a"
#elif L==5	// fr
	"utilisateur/se"
#elif L==6	// gn
	"puruh&atilde;ra"
#elif L==7	// it
	"utente"
#elif L==8	// pl
	"u&zdot;ytkownika"
#elif L==9	// pt
	"utilizador/a"
#endif
	};

const char *Txt_USER_comments =
#if   L==1	// ca
	"Observaciones";	// Necessita traduccio
#elif L==2	// de
	"Kommentare";
#elif L==3	// en
	"Comments";
#elif L==4	// es
	"Observaciones";
#elif L==5	// fr
	"Commentaires";
#elif L==6	// gn
	"Observaciones";	// Okoteve traduccin
#elif L==7	// it
	"Commenti";
#elif L==8	// pl
	"Comments";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Coment&aacute;rios";
#endif

const char *Txt_User_ID =
#if   L==1	// ca
	"ID (DNI/c&egrave;dula/passaport) de l'usuari";
#elif L==2	// de
	"Ausweis-Nr.";
#elif L==3	// en
	"User ID";
#elif L==4	// es
	"ID (DNI/c&eacute;dula) del usuario";
#elif L==5	// fr
	"Num&eacute;ro d'identit&eacute; d'utilisateur";
#elif L==6	// gn
	"ID (DNI/c&eacute;dula) del usuario";		// Okoteve traduccin
#elif L==7	// it
	"Numero carta d'identit&agrave; utente";
#elif L==8	// pl
	"User ID";					// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"N&ordm; de identif. do utilizador";
#endif

const char *Txt_USER_in_COURSE =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"%s en";
#elif L==2	// de
	"%s im";
#elif L==3	// en
	"%s in";
#elif L==4	// es
	"%s en";
#elif L==5	// fr
	"%s dans";
#elif L==6	// gn
	"%s en";	// Okoteve traduccin
#elif L==7	// it
	"%s in";
#elif L==8	// pl
	"%s w";
#elif L==9	// pt
	"%s em";
#endif

const char *Txt_User_not_found_or_you_do_not_have_permission_ =
#if   L==1	// ca
	"Usuari no trobat"
	" o b&eacute; vost&egrave; no t&eacute; perm&iacute;s"
	" per realitzar aquesta acci&oacute;.";
#elif L==2	// de
	"Benutzer nicht gefunden"
	" oder Sie haben keine Berechtigung,"
	" um diese Handlung durchzuf&uuml;hren.";
#elif L==3	// en
	"User not found"
	" or you do not have permission"
	" to perform this action.";
#elif L==4	// es
	"Usuario/a no encontrado/a"
	" o bien usted no tiene permiso"
	" para realizar esta acci&oacute;n.";
#elif L==5	// fr
	"Utilisateur introuvable"
	" ou vous n'avez pas l'autorisation"
	" d'effectuer cette action.";
#elif L==6	// gn
	"Usuario/a no encontrado/a"
	" o bien usted no tiene permiso"
	" para realizar esta acci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"IUtente non trovato"
	" o non si dispone dell'autorizzazione"
	" per eseguire questa azione.";
#elif L==8	// pl
	"User not found"
	" or you do not have permission"
	" to perform this action.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Utilizador n&atilde;o encontrado"
	" ou voc&ecirc; n&atilde;o tem permiss&atilde;o"
	" para realizar esta a&ccedil;&atilde;o.";
#endif

const char *Txt_Users =
#if   L==1	// ca
	"Usuaris";
#elif L==2	// de
	"Benutzer";
#elif L==3	// en
	"Users";
#elif L==4	// es
	"Usuarios";
#elif L==5	// fr
	"Utilisateurs";
#elif L==6	// gn
	"Puruh&aacute;ra";
#elif L==7	// it
	"Utenti";
#elif L==8	// pl
	"U&zdot;ytkownicy";
#elif L==9	// pt
	"Utilizadores";
#endif

const char *Txt_users[Usr_NUM_SEXS] =
	{	// Usr_SEX_UNKNOWN
#if   L==1	// ca
	"usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarios"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadores"
#endif
	,	// Usr_SEX_FEMALE
#if   L==1	// ca
	"usuarias"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarias"
#elif L==5	// fr
	"utilisateuses"
#elif L==6	// gn
	"usuarias"	// Okoteve traduccin
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadoras"
#endif
	,	// Usr_SEX_MALE
#if   L==1	// ca
	"usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarios"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadores"
#endif
	,	// Usr_SEX_ALL
#if   L==1	// ca
	"usuaris"
#elif L==2	// de
	"Benutzer"
#elif L==3	// en
	"users"
#elif L==4	// es
	"usuarios"
#elif L==5	// fr
	"utilisateurs"
#elif L==6	// gn
	"puruh&aacute;ra"
#elif L==7	// it
	"utenti"
#elif L==8	// pl
	"u&zdot;ytkownicy"
#elif L==9	// pt
	"utilizadores"
#endif
	};

const char *Txt_X_users_have_been_eliminated =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Se ha(n) eliminado %u usuario(s).";		// Necessita traduccio
#elif L==2	// de
	"%u user(s) have been eliminated.";		// Need bersetzung
#elif L==3	// en
	"%u user(s) have been eliminated.";
#elif L==4	// es
	"Se ha(n) eliminado %u usuario(s).";
#elif L==5	// fr
	"%u users have been eliminated.";		// Besoin de traduction
#elif L==6	// gn
	"Se ha(n) eliminado %u usuario(s).";		// Okoteve traduccin
#elif L==7	// it
	"%u utenti sono stati rimossi.";
#elif L==8	// pl
	"%u user(s) have been eliminated.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"%u usu&aacute;rio(s) foram eliminados.";
#endif

const char *Txt_X_users_have_been_enroled_including_possible_repetitions =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Se han inscrito %u usuarios"
	" (incluyendo posibles repeticiones).";	// Necessita traduccio
#elif L==2	// de
	"%u users have been enroled"
	" (including possible repetitions).";	// Need bersetzung
#elif L==3	// en
	"%u users have been enroled"
	" (including possible repetitions).";
#elif L==4	// es
	"Se han inscrito %u usuarios"
	" (incluyendo posibles repeticiones).";
#elif L==5	// fr
	"%u users have been enroled"
	" (including possible repetitions).";	// Besoin de traduction
#elif L==6	// gn
	"Se han inscrito %u usuarios"
	" (incluyendo posibles repeticiones).";	// Okoteve traduccin
#elif L==7	// it
	"%u utenti sono stati registrati"
	" (comprese le eventuali ripetizioni).";
#elif L==8	// pl
	"%u users have been enroled"
	" (including possible repetitions).";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"%u usu&aacute;rios foram inscritos"
	" (incluindo poss&iacute;veis repeti&ccedil;&otilde;es).";
#endif

const char *Txt_X_users_have_been_removed =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Se han eliminado %u usuarios.";	// Necessita traduccio
#elif L==2	// de
	"%u users have been removed.";		// Need bersetzung
#elif L==3	// en
	"%u users have been removed.";
#elif L==4	// es
	"Se han eliminado %u usuarios.";
#elif L==5	// fr
	"%u users have been removed.";		// Besoin de traduction
#elif L==6	// gn
	"Se han eliminado %u usuarios.";	// Okoteve traduccin
#elif L==7	// it
	"%u utenti sono stati rimossi.";
#elif L==8	// pl
	"%u users have been removed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"%u usu&aacute;rios foram removidos.";
#endif

const char *Txt_Users_in_courses =
#if   L==1	// ca
	"Usuaris en assignatures";
#elif L==2	// de
	"Benutzer in Kursen";
#elif L==3	// en
	"Users in courses";
#elif L==4	// es
	"Usuarios en asignaturas";
#elif L==5	// fr
	"Utilisateurs dans mati&egrave;res";
#elif L==6	// gn
	"Puruh&aacute;ra en asignaturas";	// Okoteve traduccin
#elif L==7	// it
	"Utenti nei corsi";
#elif L==8	// pl
	"U&zdot;ytkownicy kurs&oacute;w";
#elif L==9	// pt
	"Utilizadores em disciplinas";
#endif

const char *Txt_Users_of_the_centre =
#if   L==1	// ca
	 "Usuaris del centre";
#elif L==2	// de
	 "Benutzer der Lehrinstitut";
#elif L==3	// en
	 "Users of the centre";
#elif L==4	// es
	 "Usuarios del centro";
#elif L==5	// fr
	 "Utilisateurs du centre";
#elif L==6	// gn
	 "Usuarios del centro";	// Okoteve traduccin
#elif L==7	// it
	 "Utenti del centro";
#elif L==8	// pl
	 "U&zdot;ytkownicy centrum";
#elif L==9	// pt
	 "Utilizadores do centro";
#endif

const char *Txt_Users_of_the_country =
#if   L==1	// ca
	 "Usuaris del pa&iacute;s";
#elif L==2	// de
	 "Benutzer des Landes";
#elif L==3	// en
	 "Users of the country";
#elif L==4	// es
	 "Usuarios del pa&iacute;s";
#elif L==5	// fr
	 "Utilisateurs du pays";
#elif L==6	// gn
	 "Usuarios del pa&iacute;s";	// Okoteve traduccin
#elif L==7	// it
	 "Utenti del paese";
#elif L==8	// pl
	 "U&zdot;ytkownicy kraju";
#elif L==9	// pt
	 "Utilizadores do pa&iacute;s";
#endif

const char *Txt_Users_of_the_institution =
#if   L==1	// ca
	 "Usuaris de la instituci&oacute;";
#elif L==2	// de
	 "Benutzer der Hochschule";
#elif L==3	// en
	 "Users of the institution";
#elif L==4	// es
	 "Usuarios de la instituci&oacute;n";
#elif L==5	// fr
	 "Utilisateurs du &eacute;tablissement";
#elif L==6	// gn
	 "Usuarios de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	 "Utenti della istituzione";
#elif L==8	// pl
	 "U&zdot;ytkownicy instytucji";
#elif L==9	// pt
	 "Utilizadores da institu&ccedil;&atilde;o";
#endif

const char *Txt_users_with_no_group =
#if   L==1	// ca
	"usuaris en cap grup";
#elif L==2	// de
	"Benutzer mit nicht-Gruppe";
#elif L==3	// en
	"users with no group";
#elif L==4	// es
	"usuarios en ning&uacute;n grupo";
#elif L==5	// fr
	"utilisateurs avec aucun groupe";
#elif L==6	// gn
	"usuarios en ning&uacute;n grupo";	// Okoteve traduccin
#elif L==7	// it
	"utenti con nessun gruppo";
#elif L==8	// pl
	"u&zdot;ytkownik&oacute;w w ka&zdot;dej grupie";
#elif L==9	// pt
	"utilizadores com nenhum grupo";
#endif

const char *Txt_USR_LIST_TYPES[Usr_NUM_USR_LIST_TYPES] =
   {
	 ""	// Usr_LIST_UNKNOWN
	 ,	// Usr_LIST_AS_CLASS_PHOTO
#if   L==1	// ca
	 "Orla"
#elif L==2	// de
	 "Klass Foto"
#elif L==3	// en
	 "Class photo"
#elif L==4	// es
	 "Orla"
#elif L==5	// fr
	 "Photo de classe"
#elif L==6	// gn
	 "Orla"		// Okoteve traduccin
#elif L==7	// it
	 "Foto di classe"
#elif L==8	// pl
	 "Class photo"	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Foto de formatura"
#endif
	 ,	// Usr_LIST_AS_LISTING
#if   L==1	// ca
	 "Llista"
#elif L==2	// de
	 "Liste"
#elif L==3	// en
	 "List"
#elif L==4	// es
	 "Lista"
#elif L==5	// fr
	 "Liste"
#elif L==6	// gn
	 "Lista"	// Okoteve traduccin
#elif L==7	// it
	 "Lista"
#elif L==8	// pl
	 "List"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "Lista"
#endif
	};

const char *Txt_Vacants =
#if   L==1	// ca
	"Vacants";
#elif L==2	// de
	"Frei";
#elif L==3	// en
	"Vacants";
#elif L==4	// es
	"Vacantes";
#elif L==5	// fr
	"Vacants";
#elif L==6	// gn
	"Vacantes";	// Okoteve traduccin
#elif L==7	// it
	"Liberi";
#elif L==8	// pl
	"Wolne";
#elif L==9	// pt
	"Desocupados";
#endif

const char *Txt_View =
#if   L==1	// ca
	"Veure";
#elif L==2	// de
	"Anzeigen";
#elif L==3	// en
	"View";
#elif L==4	// es
	"Ver";
#elif L==5	// fr
	"Voir";
#elif L==6	// gn
	"Ver";	// Okoteve traduccin
#elif L==7	// it
	"Vedi";
#elif L==8	// pl
	"Wy&sacute;wietl";
#elif L==9	// pt
	"Ver";
#endif

const char *Txt_View_all_recipients =
#if   L==1	// ca
	"Ver todos los destinatarios";	// Necessita traduccio
#elif L==2	// de
	"Anzeigen alle Empf&auml;nger";
#elif L==3	// en
	"View all recipients";
#elif L==4	// es
	"Ver todos los destinatarios";
#elif L==5	// fr
	"Voir tous les destinataires";
#elif L==6	// gn
	"Ver todos los destinatarios";	// Okoteve traduccin
#elif L==7	// it
	"Vedi tutti i destinatari";
#elif L==8	// pl
	"View all recipients";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ver todos os estinat&aacute;rios";
#endif

const char *Txt_View_attendance =
#if   L==1	// ca
	"Veure assist&egrave;ncia";
#elif L==2	// de
	"Anzeigen Anwesenheit";
#elif L==3	// en
	"View attendance";
#elif L==4	// es
	"Ver asistencia";
#elif L==5	// fr
	"Voir pr&eacute;sence";
#elif L==6	// gn
	"Ver asistencia";	// Okoteve traduccin
#elif L==7	// it
	"Vedi presenza";
#elif L==8	// pl
	"Zobacz obecno&sacute;&cacute;";
#elif L==9	// pt
	"Ver presen&ccedil;a";
#endif

const char *Txt_View_data_of_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Veure dades de %s";
#elif L==2	// de
	"Siehe Daten von %s";
#elif L==3	// en
	"View data of %s";
#elif L==4	// es
	"Ver datos de %s";
#elif L==5	// fr
	"Voir les donn&eacute;es de %s";
#elif L==6	// gn
	"Ver datos de %s";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza i dati de %s";
#elif L==8	// pl
	"Zobacz dane z %s";
#elif L==9	// pt
	"Ver os dados de %s";
#endif

const char *Txt_View_event =
#if   L==1	// ca
	"Veure esdeveniment";
#elif L==2	// de
	"Anzeigen Ereignis";
#elif L==3	// en
	"View event";
#elif L==4	// es
	"Ver evento";
#elif L==5	// fr
	"Voir &eacute;v&eacute;nement";
#elif L==6	// gn
	"Ver evento";			// Okoteve traduccin
#elif L==7	// it
	"Vedi evento";
#elif L==8	// pl
	"View event";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ver evento";
#endif

const char *Txt_View_game =
#if   L==1	// ca
	"Veure joc";
#elif L==2	// de
	"Anzeigen Spiel";
#elif L==3	// en
	"View game";
#elif L==4	// es
	"Ver juego";
#elif L==5	// fr
	"Voir jeu";
#elif L==6	// gn
	"Ver juego";			// Okoteve traduccin
#elif L==7	// it
	"Vedi gioco";
#elif L==8	// pl
	"Zobacz gr&eogon;";
#elif L==9	// pt
	"Ver jogo";
#endif

const char *Txt_View_game_results =
#if   L==1	// ca
	"Veure resultats";
#elif L==2	// de
	"Anzeigen Ergebnisse";
#elif L==3	// en
	"View game results";
#elif L==4	// es
	"Ver resultados";
#elif L==5	// fr
	"Voir r&eacute;sultats";
#elif L==6	// gn
	"Ver resultados";	// Okoteve traduccin
#elif L==7	// it
	"Vedi risultati";
#elif L==8	// pl
	"Wy&sacute;wietl wyniki ankiety";
#elif L==9	// pt
	"Ver resultados";
#endif

const char *Txt_View_in_a_new_window =
#if   L==1	// ca
	"Mostrar en una finestra nova";
#elif L==2	// de
	"Ansicht in einem neuen Fenster";
#elif L==3	// en
	"View in a new window";
#elif L==4	// es
	"Mostrar en una ventana nueva";
#elif L==5	// fr
	"Voir dans une nouvelle fen&ecirc;tre";
#elif L==6	// gn
	"Mostrar en una ventana nueva";	// Okoteve traduccin
#elif L==7	// it
	"Visualizza in una nuova finestra";
#elif L==8	// pl
	"View in a new window";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ver em uma nova janela";
#endif

const char *Txt_View_record_and_office_hours =
#if   L==1	// ca
	"Veure fitxa i horari de tutories";
#elif L==2	// de
	"Anzeigen profil und Gesch&auml;ftszeiten";
#elif L==3	// en
	"View record and office hours";
#elif L==4	// es
	"Ver ficha y horario de tutor&iacute;as";
#elif L==5	// fr
	"Voir carte et heures de tuteur";
#elif L==6	// gn
	"Ver ficha y horario de tutor&iacute;as";	// Okoteve traduccin
#elif L==7	// it
	"Vedi scheda e orario di tutoraggi";
#elif L==8	// pl
	"Zobacz rekord i godziny pracy";
#elif L==9	// pt
	"Ver cart&atilde;o e hor&aacute;rio de tutor";
#endif

const char *Txt_View_record_for_this_course =
#if   L==1	// ca
	"Veure fitxa en aquesta assignatura";
#elif L==2	// de
	"Anzeigen profil bei Kurs";
#elif L==3	// en
	"View record for this course";
#elif L==4	// es
	"Ver ficha en esta asignatura";
#elif L==5	// fr
	"Voir carte dans mati&egrave;re";
#elif L==6	// gn
	"Ver ficha en esta asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Vedi scheda per questo corso";
#elif L==8	// pl
	"Zobacz rekord na ten kurs";
#elif L==9	// pt
	"Ver cart&atilde;o em disciplina";
#endif

const char *Txt_View_records =
#if   L==1	// ca
	"Veure fitxes";
#elif L==2	// de
	"Anzeigen Profile";
#elif L==3	// en
	"View records";
#elif L==4	// es
	"Ver fichas";
#elif L==5	// fr
	"Voir cartes";
#elif L==6	// gn
	"Ver fichas";	// Okoteve traduccin
#elif L==7	// it
	"Vedi schede";
#elif L==8	// pl
	"Zobacz rekordy";
#elif L==9	// pt
	"Ver cart&otilde;es";
#endif

const char *Txt_View_survey =
#if   L==1	// ca
	"Veure enquesta";
#elif L==2	// de
	"Anzeigen Umfrage";
#elif L==3	// en
	"View survey";
#elif L==4	// es
	"Ver encuesta";
#elif L==5	// fr
	"Voir sondage";
#elif L==6	// gn
	"Ver encuesta";			// Okoteve traduccin
#elif L==7	// it
	"Vedi sondaggio";
#elif L==8	// pl
	"Zobacz ankiet&eogon;";
#elif L==9	// pt
	"Ver inqu&eacute;rito";
#endif

const char *Txt_View_survey_results =
#if   L==1	// ca
	"Veure resultats";
#elif L==2	// de
	"Anzeigen Ergebnisse";
#elif L==3	// en
	"View survey results";
#elif L==4	// es
	"Ver resultados";
#elif L==5	// fr
	"Voir r&eacute;sultats";
#elif L==6	// gn
	"Ver resultados";	// Okoteve traduccin
#elif L==7	// it
	"Vedi risultati";
#elif L==8	// pl
	"Wy&sacute;wietl wyniki ankiety";
#elif L==9	// pt
	"Ver resultados";
#endif

const char *Txt_View_test =
#if   L==1	// ca
	"Veure test";
#elif L==2	// de
	"Siehe Test";
#elif L==3	// en
	"View test";
#elif L==4	// es
	"Ver test";
#elif L==5	// fr
	"Voir test";
#elif L==6	// gn
	"Ver test";	// Okoteve traduccin
#elif L==7	// it
	"Vedi test";
#elif L==8	// pl
	"Wiadomo&sacute;ci test";
#elif L==9	// pt
	"Ver teste";
#endif

const char *Txt_View_test_results =
#if   L==1	// ca
	"Veure resultats de tests";
#elif L==2	// de
	"Siehe Testergebnisse";
#elif L==3	// en
	"View test results";
#elif L==4	// es
	"Ver resultados de los test";
#elif L==5	// fr
	"Voir r&eacute;sultats de tests";
#elif L==6	// gn
	"Ver resultados de los test";	// Okoteve traduccin
#elif L==7	// it
	"Vedi risultati dei test";
#elif L==8	// pl
	"Zobacz wyniki test&oacute;w";
#elif L==9	// pt
	"Ver resultados de testes";
#endif

const char *Txt_View_homework =
#if   L==1	// ca
	"Veure treballs";
#elif L==2	// de
	"Anzeigen Hausarbeiten";
#elif L==3	// en
	"View homework";
#elif L==4	// es
	"Ver trabajos";
#elif L==5	// fr
	"Voir travaux";
#elif L==6	// gn
	"Ver trabajos";		// Okoteve traduccin
#elif L==7	// it
	"Vedi lavori";
#elif L==8	// pl
	"Zobacz prac&eogon; domow&aogon;";
#elif L==9	// pt
	"Ver trabalhos";
#endif

const char *Txt_Visible_by_BR_the_student =
#if   L==1	// ca
	"&iquest;Visible por<br />el estudiante?";	// Necessita traduccio
#elif L==2	// de
	"Sichtbare durch<br />die Student?";
#elif L==3	// en
	"Visible by<br />the student?";
#elif L==4	// es
	"&iquest;Visible por<br />el estudiante?";
#elif L==5	// fr
	"Visible par<br />l'&eacute;tudiant/e?";
#elif L==6	// gn
	"&iquest;Visible por<br />el estudiante?";	// Okoteve traduccin
#elif L==7	// it
	"Visibili dallo<br />studente?";
#elif L==8	// pl
	"Visible by<br />the student?";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Vis&iacute;vel pelo<br />estudante?";
#endif

const char *Txt_Visible_game =
#if   L==1	// ca
	"Joc visible";
#elif L==2	// de
	"Sichtbare Spiel";
#elif L==3	// en
	"Visible game";
#elif L==4	// es
	"Juego visible";
#elif L==5	// fr
	"Jeu visible";
#elif L==6	// gn
	"Juego visible";	// Okoteve traduccin
#elif L==7	// it
	"Gioco visibile";
#elif L==8	// pl
	"Widoczny gra";
#elif L==9	// pt
	"Jogo vis&iacute;vel";
#endif

const char *Txt_Visible_MALE_PLURAL =
#if   L==1	// ca
	"Visibles";
#elif L==2	// de
	"Sichtbar";
#elif L==3	// en
	"Visible";
#elif L==4	// es
	"Visibles";
#elif L==5	// fr
	"Visibles";
#elif L==6	// gn
	"Visibles";	// Okoteve traduccin
#elif L==7	// it
	"Visibili";
#elif L==8	// pl
	"Widoczne";
#elif L==9	// pt
	"Vis&iacute;veis";
#endif

const char *Txt_Visible_tests =
#if   L==1	// ca
	"Tests visibles";
#elif L==2	// de
	"Sichtbare Pr%uuml;fungen";
#elif L==3	// en
	"Visible tests";
#elif L==4	// es
	"Test visibles";
#elif L==5	// fr
	"Tests visibles";
#elif L==6	// gn
	"Test visibles";	// Okoteve traduccin
#elif L==7	// it
	"Test visibili";
#elif L==8	// pl
	"Widoczne testy";
#elif L==9	// pt
	"Testes vis&iacute;veis";
#endif

const char *Txt_Visible_survey =
#if   L==1	// ca
	"Enquesta visible";
#elif L==2	// de
	"Sichtbare Umfrage";
#elif L==3	// en
	"Visible survey";
#elif L==4	// es
	"Encuesta visible";
#elif L==5	// fr
	"Sondage visible";
#elif L==6	// gn
	"Encuesta visible";	// Okoteve traduccin
#elif L==7	// it
	"Sondaggio visibile";
#elif L==8	// pl
	"Widoczny badanie";
#elif L==9	// pt
	"Inqu&eacute;rito vis&iacute;vel";
#endif

const char *Txt_Visits_to_course =
#if   L==1	// ca
	"Accessos assignatura";
#elif L==2	// de
	"Anmeldungen bei Kurs";
#elif L==3	// en
	"Visits to course";
#elif L==4	// es
	"Accesos asignatura";
#elif L==5	// fr
	"Visites mati&egrave;re";
#elif L==6	// gn
	"Accesos asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Accessi al corso";
#elif L==8	// pl
	"Wizyty w trakcie";
#elif L==9	// pt
	"Visitas disciplina";
#endif

const char *Txt_Web =
#if   L==1	// ca
	"Web";
#elif L==2	// de
	"Web";
#elif L==3	// en
	"Web";
#elif L==4	// es
	"Web";
#elif L==5	// fr
	"Web";
#elif L==6	// gn
	"Web";	// Okoteve traduccin
#elif L==7	// it
	"Web";
#elif L==8	// pl
	"Web";
#elif L==9	// pt
	"Web";
#endif

const char *Txt_Web_page =
#if   L==1	// ca
	"P&agrave;gina web";
#elif L==2	// de
	"Webseite";
#elif L==3	// en
	"Web page";
#elif L==4	// es
	"P&aacute;gina web";
#elif L==5	// fr
	"Page Web";
#elif L==6	// gn
	"P&aacute;gina web";	// Okoteve traduccin
#elif L==7	// it
	"Pagina Web";
#elif L==8	// pl
	"Strona internetowa";
#elif L==9	// pt
	"P&aacute;gina Web";
#endif

const char *Txt_Web_page_of_the_course =
#if   L==1	// ca
	"P&aacute;gina web de la asignatura";	// Necessita traduccio
#elif L==2	// de
	"Webseite der Kurs";
#elif L==3	// en
	"Web page of the course";
#elif L==4	// es
	"P&aacute;gina web de la asignatura";
#elif L==5	// fr
	"Page Web de la mati&egrave;re";
#elif L==6	// gn
	"P&aacute;gina web de la asignatura";	// Okoteve traduccin
#elif L==7	// it
	"Pagina Web del corso";
#elif L==8	// pl
	"Web page of the course";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"P&aacute;gina Web da disciplina";
#endif

const char *Txt_Web_social_network =
#if   L==1	// ca
	"Web / xarxa social";
#elif L==2	// de
	"Web / Soziales Netzwerk";
#elif L==3	// en
	"Web / social network";
#elif L==4	// es
	"Web / red social";
#elif L==5	// fr
	"Web / r&eacute;seau social";
#elif L==6	// gn
	"Web / red social";	// Okoteve traduccin
#elif L==7	// it
	"Web / social network";
#elif L==8	// pl
	"Web / sieci spo&lstrok;eczne";
#elif L==9	// pt
	"Web / rede social";
#endif

const char *Txt_Webs_social_networks =
#if   L==1	// ca
	"Webs / xarxes socials";
#elif L==2	// de
	"Webs / Soziale Netzwerke";
#elif L==3	// en
	"Webs / social networks";
#elif L==4	// es
	"Webs / redes sociales";
#elif L==5	// fr
	"Webs / r&eacute;seaux sociaux";
#elif L==6	// gn
	"Webs / redes sociales";	// Okoteve traduccin
#elif L==7	// it
	"Webs / reti sociali";
#elif L==8	// pl
	"Webs / sieci spo&lstrok;eczne";
#elif L==9	// pt
	"Webs / redes sociais";
#endif

const char *Txt_Week =
#if   L==1	// ca
	"Semana";	// Necessita traduccio
#elif L==2	// de
	"Woche";
#elif L==3	// en
	"Week";
#elif L==4	// es
	"Semana";
#elif L==5	// fr
	"Semaine";
#elif L==6	// gn
	"Arapok&otilde;indy";
#elif L==7	// it
	"Settimana";
#elif L==8	// pl
	"Week";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Semana";
#endif

const char *Txt_What_would_you_like_to_do =
#if   L==1	// ca
	"&iquest;Qu&egrave; voleu fer?";
#elif L==2	// de
	"Was w&uuml;rden Sie tun?";
#elif L==3	// en
	"What would you like to do?";
#elif L==4	// es
	"&iquest;Qu&eacute; desea hacer?";
#elif L==5	// fr
	"Que feriez-vous?";
#elif L==6	// gn
	"&iquest;Qu&eacute; desea hacer?";	// Okoteve traduccin
#elif L==7	// it
	"Cosa si dovrebbe fare?";
#elif L==8	// pl
	"Co by&sacute; zrobi&lstrok;?";
#elif L==9	// pt
	"O que voc&ecirc; faria?";
#endif

const char *Txt_Where_s_USER =
#if   L==1	// ca
	"On &eacute;s %s?";
#elif L==2	// de
	"Wo ist %s?";
#elif L==3	// en
	"Where's %s";
#elif L==4	// es
	"&iquest;D&oacute;nde est&aacute; %s?";
#elif L==5	// fr
	"O&ugrave; est %s?";
#elif L==6	// gn
	"&iquest;D&oacute;nde est&aacute; %s?";	// Okoteve traduccin
#elif L==7	// it
	"Dove &egrave; %s?";
#elif L==8	// pl
	"Gdzie jest %s?";
#elif L==9	// pt
	"Onde est&aacute; %s?";
#endif

const char *Txt_Who_to_follow =
#if   L==1	// ca
	"A qui seguir";
#elif L==2	// de
	"Wer zu folgen";
#elif L==3	// en
	"Who to follow";
#elif L==4	// es
	"A qui&eacute;n seguir";
#elif L==5	// fr
	"Qui &agrave; suivre";
#elif L==6	// gn
	"A qui&eacute;n seguir";	// Okoteve traduccin
#elif L==7	// it
	"Chi da seguire";
#elif L==8	// pl
	"Kto do na&sacute;ladowania";
#elif L==9	// pt
	"Quem seguir";
#endif

const char *Txt_With_ =	// Example: "With students"
#if   L==1	// ca
	"Amb ";
#elif L==2	// de
	"Mit ";
#elif L==3	// en
	"With ";
#elif L==4	// es
	"Con ";
#elif L==5	// fr
	"Avec ";
#elif L==6	// gn
	"Con ";	// Okoteve traduccin
#elif L==7	// it
	"Con ";
#elif L==8	// pl
	"Z ";
#elif L==9	// pt
	"Com ";
#endif

const char *Txt_with_new_posts =
#if   L==1	// ca
	"con comentarios nuevos";	// Necessita traduccio
#elif L==2	// de
	"mit neuen Beitr&auml;ge";
#elif L==3	// en
	"with new posts";
#elif L==4	// es
	"con comentarios nuevos";
#elif L==5	// fr
	"avec de nouveaux posts";
#elif L==6	// gn
	"con comentarios nuevos";	// Okoteve traduccin
#elif L==7	// it
	"con nuovi post";
#elif L==8	// pl
	"with new posts";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"com novas posts";
#endif

const char *Txt_works_ZIP_FILE_NAME =	// Name (without extension) of the ZIP file for works. Must be a valid filename in small letters!
#if   L==1	// ca
	"treballs";
#elif L==2	// de
	"Hausarbeiten";
#elif L==3	// en
	"homework";
#elif L==4	// es
	"trabajos";
#elif L==5	// fr
	"travaux";
#elif L==6	// gn
	"tembiapo";
#elif L==7	// it
	"lavori";
#elif L==8	// pl
	"homework";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"trabalhos";
#endif

const char *Txt_Works_area =
#if   L==1	// ca
	"Zona d'altres treballs";
#elif L==2	// de
	"Hausarbeiten-Bereich";
#elif L==3	// en
	"Homework area";
#elif L==4	// es
	"Zona de otros trabajos";
#elif L==5	// fr
	"Zone de travaux";
#elif L==6	// gn
	"Zona de otros trabajos";	// Okoteve traduccin
#elif L==7	// it
	"Area lavori";
#elif L==8	// pl
	"Homework area";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Zona de trabalhos";
#endif

const char *Txt_works_area =
#if   L==1	// ca
	"zona de trabajos";	// Necessita traduccio
#elif L==2	// de
	"Hausarbeiten-Bereich";
#elif L==3	// en
	"homework area";
#elif L==4	// es
	"zona de trabajos";
#elif L==5	// fr
	"zone de travaux";
#elif L==6	// gn
	"zona de trabajos";	// Okoteve traduccin
#elif L==7	// it
	"area lavori";
#elif L==8	// pl
	"homework area";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"zona de trabalhos";
#endif

const char *Txt_WriBRters =
#if   L==1	// ca
	"Escri-<br />tores";	// Necessita traduccio
#elif L==2	// de
	"Schrift-<br />steller";
#elif L==3	// en
	"Wri-<br />ters";
#elif L==4	// es
	"Escri-<br />tores";
#elif L==5	// fr
	"&Eacute;cri-<br />vains";
#elif L==6	// gn
	"Escri-<br />tores";	// Okoteve traduccin
#elif L==7	// it
	"Scri-<br />ttori";
#elif L==8	// pl
	"Wri-<br />ters";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Escri-<br />tores";
#endif

const char *Txt_Write_a_message =
#if   L==1	// ca
	"Escriu un missatge";
#elif L==2	// de
	"Schreiben eine Nachricht";
#elif L==3	// en
	"Write a message";
#elif L==4	// es
	"Escribir un mensaje";
#elif L==5	// fr
	"&Eacute;crire un message";
#elif L==6	// gn
	"Escribir un mensaje";		// Okoteve traduccin
#elif L==7	// it
	"Scrivi un messaggio";
#elif L==8	// pl
	"Napisz wiadomo&sacute;&cacute;";
#elif L==9	// pt
	"Escrever uma mensagem";
#endif

const char *Txt_Wrong_file_type =
#if   L==1	// ca
	"Tipus d'arxiu incorrecte.";
#elif L==2	// de
	"Falsche Dateityp.";
#elif L==3	// en
	"Wrong file type.";
#elif L==4	// es
	"Tipo de archivo incorrecto.";
#elif L==5	// fr
	"Mauvais type de fichier.";
#elif L==6	// gn
	"Tipo de archivo incorrecto.";		// Okoteve traduccin
#elif L==7	// it
	"Tipo di file sbagliato.";
#elif L==8	// pl
	"Nieprawid&lstrok;owy typ pliku.";
#elif L==9	// pt
	"Tipo de arquivo errado.";
#endif

const char *Txt_WWW =		// World Wide Web
#if   L==1	// ca
	"WWW";
#elif L==2	// de
	"WWW";
#elif L==3	// en
	"WWW";
#elif L==4	// es
	"WWW";
#elif L==5	// fr
	"WWW";
#elif L==6	// gn
	"WWW";
#elif L==7	// it
	"WWW";
#elif L==8	// pl
	"WWW";
#elif L==9	// pt
	"WWW";
#endif

const char *Txt_XML_file =
#if   L==1	// ca
	"Fitxer XML";
#elif L==2	// de
	"XML-Datei";
#elif L==3	// en
	"XML file";
#elif L==4	// es
	"Archivo XML";
#elif L==5	// fr
	"Fichier XML";
#elif L==6	// gn
	"Archivo XML";	// Okoteve traduccin
#elif L==7	// it
	"File XML";
#elif L==8	// pl
	"Plik XML";
#elif L==9	// pt
	"Arquivo XML";
#endif

const char *Txt_XML_file_content =
#if   L==1	// ca
	"Contingut del fitxer XML";
#elif L==2	// de
	"XML-Datei-Inhalt";
#elif L==3	// en
	"XML file content";
#elif L==4	// es
	"Contenido del archivo XML";
#elif L==5	// fr
	"Contenu du fichier XML";
#elif L==6	// gn
	"Contenido del archivo XML";	// Okoteve traduccin
#elif L==7	// it
	"Contenuto di file XML";
#elif L==8	// pl
	"Zawarto&sacute;&cacute; pliku XML";
#elif L==9	// pt
	"Conte&uacute;do do arquivo XML";
#endif

const char *Txt_Year =	// 1984, 2011...
#if   L==1	// ca
	"Any";
#elif L==2	// de
	"Jahr";
#elif L==3	// en
	"Year";
#elif L==4	// es
	"A&ntilde;o";
#elif L==5	// fr
	"Ann&eacute;e";
#elif L==6	// gn
	"Ary";
#elif L==7	// it
	"Anno";
#elif L==8	// pl
	"Rok";
#elif L==9	// pt
	"Ano";
#endif

const char *Txt_year =	// 1984, 2011...
#if   L==1	// ca
	"any";
#elif L==2	// de
	"Jahr";
#elif L==3	// en
	"year";
#elif L==4	// es
	"a&ntilde;o";
#elif L==5	// fr
	"ann&eacute;e";
#elif L==6	// gn
	"ary";
#elif L==7	// it
	"anno";
#elif L==8	// pl
	"rok";
#elif L==9	// pt
	"ano";
#endif

const char *Txt_Year_OF_A_DEGREE =	// 1, 2, 3..., not 1984, 2011...
#if   L==1	// ca
	"Any o semestre";
#elif L==2	// de
	"Jahr oder Semester";
#elif L==3	// en
	"Year or semester";
#elif L==4	// es
	"A&ntilde;o o semestre";
#elif L==5	// fr
	"Ann&eacute;e ou semestre";
#elif L==6	// gn
	"A&ntilde;o o semestre";	// Okoteve traduccin
#elif L==7	// it
	"Anno o semestre";
#elif L==8	// pl
	"Rok lub semestr";
#elif L==9	// pt
	"Ano ou semestre";
#endif

const char *Txt_YEAR_OF_DEGREE[1 + Deg_MAX_YEARS_PER_DEGREE] =
   {
#if   L==1	// ca
	 "N.A."
#elif L==2	// de
	 "N.A."
#elif L==3	// en
	 "N.A."
#elif L==4	// es
	 "N.A."
#elif L==5	// fr
	 "N.A."
#elif L==6	// gn
	 "N.A."		// Okoteve traduccin
#elif L==7	// it
	 "N.A."
#elif L==8	// pl
	 "N.D."
#elif L==9	// pt
	 "N.A."
#endif
	 ,
#if   L==1	// ca
	 "1&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "1st"		// Need bersetzung
#elif L==3	// en
	 "1st"
#elif L==4	// es
	 "1&ordm;"
#elif L==5	// fr
	 "1st"		// Besoin de traduction
#elif L==6	// gn
	 "1&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Primo"
#elif L==8	// pl
	 "1st"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "1&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "2&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "2nd"		// Need bersetzung
#elif L==3	// en
	 "2nd"
#elif L==4	// es
	 "2&ordm;"
#elif L==5	// fr
	 "2nd"		// Besoin de traduction
#elif L==6	// gn
	 "2&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Secondo"
#elif L==8	// pl
	 "2nd"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "2&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "3&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "3rd"
#elif L==3	// en
	 "3rd"
#elif L==4	// es
	 "3&ordm;"
#elif L==5	// fr
	 "3rd"		// Besoin de traduction
#elif L==6	// gn
	 "3&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Terzo"
#elif L==8	// pl
	 "3rd"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "3&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "4&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "4th"		// Need bersetzung
#elif L==3	// en
	 "4th"
#elif L==4	// es
	 "4&ordm;"
#elif L==5	// fr
	 "Quarto"
#elif L==6	// gn
	 "4&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "4th"		// Bisogno di traduzione
#elif L==8	// pl
	 "4th"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "4&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "5&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "5th"		// Need bersetzung
#elif L==3	// en
	 "5th"
#elif L==4	// es
	 "5&ordm;"
#elif L==5	// fr
	 "5th"		// Besoin de traduction
#elif L==6	// gn
	 "5&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Quinto"
#elif L==8	// pl
	 "5th"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "5&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "6&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "6th"		// Need bersetzung
#elif L==3	// en
	 "6th"
#elif L==4	// es
	 "6&ordm;"
#elif L==5	// fr
	 "6th"		// Besoin de traduction
#elif L==6	// gn
	 "6&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Sesto"
#elif L==8	// pl
	 "6th"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "6&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "7&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "7th"		// Need bersetzung
#elif L==3	// en
	 "7th"
#elif L==4	// es
	 "7&ordm;"
#elif L==5	// fr
	 "7th"		// Besoin de traduction
#elif L==6	// gn
	 "7&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Settimo"
#elif L==8	// pl
	 "7th"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "7&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "8&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "8th"		// Need bersetzung
#elif L==3	// en
	 "8th"
#elif L==4	// es
	 "8&ordm;"
#elif L==5	// fr
	 "8th"		// Besoin de traduction
#elif L==6	// gn
	 "8&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Ottavo"
#elif L==8	// pl
	 "8th"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "8&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "9&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "9th"		// Need bersetzung
#elif L==3	// en
	 "9th"
#elif L==4	// es
	 "9&ordm;"
#elif L==5	// fr
	 "9th"		// Besoin de traduction
#elif L==6	// gn
	 "9&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Nono"
#elif L==8	// pl
	 "9th"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "9&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "10&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "10th"		// Need bersetzung
#elif L==3	// en
	 "10th"
#elif L==4	// es
	 "10&ordm;"
#elif L==5	// fr
	 "10th"		// Besoin de traduction
#elif L==6	// gn
	 "10&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Decimo"
#elif L==8	// pl
	 "10th"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "10&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "11&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "11th"		// Need bersetzung
#elif L==3	// en
	 "11th"
#elif L==4	// es
	 "11&ordm;"
#elif L==5	// fr
	 "11th"		// Besoin de traduction
#elif L==6	// gn
	 "11&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Undicesimo"
#elif L==8	// pl
	 "11th"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "11&ordm;"
#endif
	 ,
#if   L==1	// ca
	 "12&ordm;"	// Necessita traduccio
#elif L==2	// de
	 "12th"		// Need bersetzung
#elif L==3	// en
	 "12th"
#elif L==4	// es
	 "12&ordm;"
#elif L==5	// fr
	 "12th"		// Besoin de traduction
#elif L==6	// gn
	 "12&ordm;"	// Okoteve traduccin
#elif L==7	// it
	 "Sesto"
#elif L==8	// pl
	 "12th"		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	 "12&ordm;"
#endif
	};

const char *Txt_years =
#if   L==1	// ca
	"anys";
#elif L==2	// de
	"Jahre";
#elif L==3	// en
	"years";
#elif L==4	// es
	"a&ntilde;os";
#elif L==5	// fr
	"ann&eacute;es";
#elif L==6	// gn
	"ary";
#elif L==7	// it
	"anni";
#elif L==8	// pl
	"lat";
#elif L==9	// pt
	"anos";
#endif

const char *Txt_YES =
#if   L==1	// ca
	"S&Iacute;";
#elif L==2	// de
	"JA";
#elif L==3	// en
	"YES";
#elif L==4	// es
	"S&Iacute;";
#elif L==5	// fr
	"OUI";
#elif L==6	// gn
	"HE&#7868;";
#elif L==7	// it
	"SI";
#elif L==8	// pl
	"YES";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"SIM";
#endif

const char *Txt_Yes =
#if   L==1	// ca
	"S&iacute;";
#elif L==2	// de
	"Ja";
#elif L==3	// en
	"Yes";
#elif L==4	// es
	"S&iacute;";
#elif L==5	// fr
	"Oui";
#elif L==6	// gn
	"He&#7869;";
#elif L==7	// it
	"S&igrave;";
#elif L==8	// pl
	"Yes";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Sim";
#endif

const char *Txt_Yesterday =
#if   L==1	// ca
	"Ahir";
#elif L==2	// de
	"Gestern";
#elif L==3	// en
	"Yesterday";
#elif L==4	// es
	"Ayer";
#elif L==5	// fr
	"Hier";
#elif L==6	// gn
	"Kuehe";
#elif L==7	// it
	"Ieri";
#elif L==8	// pl
	"Yesterday";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Ontem";
#endif

const char *Txt_yet_unnamed =
#if   L==1	// ca
	"encara sense nom";
#elif L==2	// de
	"noch unbenannte";
#elif L==3	// en
	"yet unnamed";
#elif L==4	// es
	"a&uacute;n sin nombre";
#elif L==5	// fr
	"encore sans nom";
#elif L==6	// gn
	"a&uacute;n sin nombre";
#elif L==7	// it
	"ancora senza nome";
#elif L==8	// pl
	"jeszcze nienazwany";
#elif L==9	// pt
	"ainda sem nome";
#endif

const char *Txt_You_already_answered_this_survey_before =
#if   L==1	// ca
	"Vost&egrave; ja ha respost a aquesta enquesta abans.";
#elif L==2	// de
	"Sie haben bereits beantwortet dieser Umfrage vor.";
#elif L==3	// en
	"You already answered this survey before.";
#elif L==4	// es
	"Usted ya respondi&oacute; esta encuesta anteriormente.";
#elif L==5	// fr
	"Vous avez d&eacute;j&agrave; r&eacute;pondu &agrave; ce sondage avant.";
#elif L==6	// gn
	"Usted ya respondi&oacute; esta encuesta anteriormente.";	// Okoteve traduccin
#elif L==7	// it
	"Hai gi&agrave; risposto a questo sondaggio prima.";
#elif L==8	// pl
	"Odpowiedzia&lstrok;e&sacute; ju&zdot; wcze&sacute;niej na ten ankiet&eogon;.";
#elif L==9	// pt
	"Voc&ecirc; j&aacute; respondeu a esso inqu&eacute;rito antes.";
#endif

const char *Txt_You_already_played_this_game_before =
#if   L==1	// ca
	"Vost&egrave; ja ha jugat aquest joc abans.";
#elif L==2	// de
	"Du hast dieses Spiel schon vorher gespielt.";
#elif L==3	// en
	"You already played this game before.";
#elif L==4	// es
	"Usted ya jug&oacute; este juego anteriormente.";
#elif L==5	// fr
	"Vous avez d&eacute;j&agrave; jou&eacute; ce jeu avant.";
#elif L==6	// gn
	"Usted ya jug&oacute; este juego anteriormente.";	// Okoteve traduccin
#elif L==7	// it
	"Hai gi&agrave; giocato questo gioco prima.";
#elif L==8	// pl
	"Ju&zdot; wcze&sacute;niej gra&lstrok;e&sacute; w t&eogon; gr&eogon;.";
#elif L==9	// pt
	"Voc&ecirc; j&aacute; jogou este jogo antes.";
#endif

const char *Txt_You_are_now_LOGGED_IN_as_X =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Vost&egrave; est&agrave; ara %s com a <strong>%s</strong>.";
#elif L==2	// de
	"Sie sind nun %s als <strong>%s</strong>.";
#elif L==3	// en
	"You are now %s as <strong>%s</strong>.";
#elif L==4	// es
	"Ahora est&aacute; %s como <strong>%s</strong>.";
#elif L==5	// fr
	"Vous &ecirc;tes maintenant %s en tant que <strong>%s</strong>.";
#elif L==6	// gn
	"Ahora est&aacute; %s como <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Sei ora %s come <strong>%s</strong>.";
#elif L==8	// pl
	"Jeste&sacute; teraz %s jako <strong>%s</strong>.";
#elif L==9	// pt
	"Agora voc&ecirc; est&aacute; %s como <strong>%s</strong>.";
#endif

const char *Txt_You_are_not_enroled_in_any_course[Usr_NUM_SEXS] =
	{
#if   L==1	// ca	// Usr_SEX_UNKNOWN
	"No est&agrave; inscrit/a en cap assignatura."
#elif L==2	// de
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3	// en
	"You are not enroled in any course."
#elif L==4	// es
	"No est&aacute; inscrito/a en ninguna asignatura."
#elif L==5	// fr
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6	// gn
	"No est&aacute; inscrito/a en ninguna asignatura."	// Okoteve traduccin
#elif L==7	// it
	"Non ti sei iscritto/a ad alcun corso."
#elif L==8	// pl
	"Nie jeste&sacute; zapisany na &zdot;aden kurs."
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o est&aacute; matriculado/a em nenhuma disciplina."
#endif
	,
#if   L==1	// ca	// Usr_SEX_FEMALE
	"No est&agrave; inscrita en cap assignatura."
#elif L==2	// de
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3	// en
	"You are not enroled in any course."
#elif L==4	// es
	"No est&aacute; inscrita en ninguna asignatura."
#elif L==5	// fr
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6	// gn
	"No est&aacute; inscrita en ninguna asignatura."	// Okoteve traduccin
#elif L==7	// it
	"Non ti sei iscritta ad alcun corso."
#elif L==8	// pl
	"Nie jeste&sacute; zapisany na &zdot;aden kurs."
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o est&aacute; matriculada em nenhuma disciplina."
#endif
	,
#if   L==1	// ca	// Usr_SEX_MALE
	"No est&agrave; inscrit en cap assignatura."
#elif L==2	// de
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3	// en
	"You are not enroled in any course."
#elif L==4	// es
	"No est&aacute; inscrito en ninguna asignatura."
#elif L==5	// fr
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6	// gn
	"No est&aacute; inscrito en ninguna asignatura."	// Okoteve traduccin
#elif L==7	// it
	"Non ti sei iscritto ad alcun corso."
#elif L==8	// pl
	"Nie jeste&sacute; zapisany na &zdot;aden kurs."
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o est&aacute; matriculado em nenhuma disciplina."
#endif
	,
#if   L==1	// ca	// Usr_SEX_ALL
	"No est&agrave; inscrit/a en cap assignatura."
#elif L==2	// de
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3	// en
	"You are not enroled in any course."
#elif L==4	// es
	"No est&aacute; inscrito/a en ninguna asignatura."
#elif L==5	// fr
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6	// gn
	"No est&aacute; inscrito/a en ninguna asignatura."	// Okoteve traduccin
#elif L==7	// it
	"Non ti sei iscritto/a ad alcun corso."
#elif L==8	// pl
	"Nie jeste&sacute; zapisany na &zdot;aden kurs."
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o est&aacute; matriculado/a em nenhuma disciplina."
#endif
	};

const char *Txt_You_can_create_a_new_folder_inside_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Podeu crear una nova carpeta"
	" dins de la carpeta <strong>%s</strong>:";
#elif L==2	// de
	"Sie k&ouml;nnen ein neues Verzeichnis"
	" im Verzeichnis <strong>%s</strong> anlegen:";
#elif L==3	// en
	"You can create a new folder"
	" inside the folder <strong>%s</strong>:";
#elif L==4	// es
	"Puede crear una nueva carpeta"
	" dentro de la carpeta <strong>%s</strong>:";
#elif L==5	// fr
	"Vous pouvez cr&eacute;er un nouveau r&eacute;pertoire"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6	// gn
	"Puede crear una nueva carpeta"
	" dentro de la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; possibile creare una nuova cartella"
	" all'interno della cartella <strong>%s</strong>:";
#elif L==8	// pl
	"Mo&zdot;esz utworzy&cacute; nowy folder"
	" wewn&aogon;trz folderu <strong>%s</strong>:";
#elif L==9	// pt
	"Voc&ecirc; pode criar um novo diret&oacute;rio"
	" dentro do diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_You_can_not_create_folders_files_or_links_here =
#if   L==1	// ca
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";	// Necessita traduccio
#elif L==2	// de
	"You can not create folders, files or links here.";		// Need bersetzung
#elif L==3	// en
	"You can not create folders, files or links here.";
#elif L==4	// es
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";
#elif L==5	// fr
	"You can not create folders, files or links here.";		// Besoin de traduction
#elif L==6	// gn
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi creare cartelle, file o linki qui.";
#elif L==8	// pl
	"You can not create folders, files or links here.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode criar pastas, arquivos ou links aqui.";
#endif

const char *Txt_You_can_not_create_folders_here =
#if   L==1	// ca
	"No puede crear carpetas aqu&iacute;.";	// Necessita traduccio
#elif L==2	// de
	"You can not create folders here.";	// Need bersetzung
#elif L==3	// en
	"You can not create folders here.";
#elif L==4	// es
	"No puede crear carpetas aqu&iacute;.";
#elif L==5	// fr
	"You can not create folders here.";	// Besoin de traduction
#elif L==6	// gn
	"No puede crear carpetas aqu&iacute;.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi creare cartelle qui.";
#elif L==8	// pl
	"You can not create folders here.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode criar pastas aqui.";
#endif

const char *Txt_You_can_not_create_links_here =
#if   L==1	// ca
	"No puede crear enlaces aqu&iacute;.";	// Necessita traduccio
#elif L==2	// de
	"You can not create links here.";	// Need bersetzung
#elif L==3	// en
	"You can not create links here.";
#elif L==4	// es
	"No puede crear enlaces aqu&iacute;.";
#elif L==5	// fr
	"You can not create links here.";	// Besoin de traduction
#elif L==6	// gn
	"No puede crear enlaces aqu&iacute;.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi creare linki qui.";
#elif L==8	// pl
	"You can not create links here.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode criar links aqui.";
#endif

const char *Txt_You_can_not_delete_your_current_nickname =
#if   L==1	// ca
	"No puede eliminar su apodo actual.";		// Necessita traduccio
#elif L==2	// de
	"You can not delete your current nickname.";	// Need bersetzung
#elif L==3	// en
	"You can not delete your current nickname.";
#elif L==4	// es
	"No puede eliminar su apodo actual.";
#elif L==5	// fr
	"You can not delete your current nickname.";	// Besoin de traduction
#elif L==6	// gn
	"No puede eliminar su apodo actual.";		// Okoteve traduccin
#elif L==7	// it
	"Non puoi cancellare il tuo nome utente attuale.";
#elif L==8	// pl
	"You can not delete your current nickname.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode excluir sua alcunha atual.";
#endif

const char *Txt_You_can_not_delete_this_ID =
#if   L==1	// ca
	"No puede eliminar este ID (DNI/c&eacute;dula).";	// Necessita traduccio
#elif L==2	// de
	"You can not delete this ID.";				// Need bersetzung
#elif L==3	// en
	"You can not delete this ID.";
#elif L==4	// es
	"No puede eliminar este ID (DNI/c&eacute;dula).";
#elif L==5	// fr
	"You can not delete this ID.";				// Besoin de traduction
#elif L==6	// gn
	"No puede eliminar este ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi cancellare questo ID.";
#elif L==8	// pl
	"You can not delete this ID.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode excluir este ID.";
#endif

const char *Txt_You_can_not_disable_file_uploading_once_folders_have_been_created =
#if   L==1	// ca
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";	// Necessita traduccio
#elif L==2	// de
	"Sie k&ouml;nnen den Upload nicht deaktivieren,"
	" wenn bereits Verzeichnisse angelegt worden sind.";
#elif L==3	// en
	"You can not disable file uploading"
	" once folders have been created.";
#elif L==4	// es
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";
#elif L==5	// fr
	"Vous ne pouvez pas d&eacute;sactiver le t&eacute;l&eacute;chargement de fichiers,"
	" une fois des r&eacute;pertoires ont &eacute;t&eacute; cr&eacute;&eacute;s.";
#elif L==6	// gn
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";	// Okoteve traduccin
#elif L==7	// it
	"Non &egrave; possibile disabilitare il caricamento dei file"
	" una volta che le cartelle sono state create.";
#elif L==8	// pl
	"You can not disable file uploading"
	" once folders have been created.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode desativar o upload de arquivos,"
	" uma vez diret&oacute;rios foram criados.";
#endif

const char *Txt_You_can_not_leave_empty_intermediate_answers =
#if   L==1	// ca
	"No puede dejar respuestas intermedias sin rellenar.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave empty intermediate answers.";	// Need bersetzung
#elif L==3	// en
	"You can not leave empty intermediate answers.";
#elif L==4	// es
	"No puede dejar respuestas intermedias sin rellenar.";
#elif L==5	// fr
	"You can not leave empty intermediate answers.";	// Besoin de traduction
#elif L==6	// gn
	"No puede dejar respuestas intermedias sin rellenar.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare vuote domande intermedie.";
#elif L==8	// pl
	"You can not leave empty intermediate answers.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar respostas intermedi&aacute;rias vazias.";
#endif

const char *Txt_You_can_not_leave_the_description_empty =
#if   L==1	// ca
	"No puede dejar la descripci&oacute;n vac&iacute;a.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the description empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the description empty.";
#elif L==4	// es
	"No puede dejar la descripci&oacute;n vac&iacute;a.";
#elif L==5	// fr
	"You can not leave the description empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar la descripci&oacute;n vac&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare vuota la descrizione.";
#elif L==8	// pl
	"You can not leave the description empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar a descri&ccedil;&atilde;o vazia.";
#endif

const char *Txt_You_can_not_leave_the_image_empty =
#if   L==1	// ca
	"No puede dejar la imagen vac&iacute;a.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the image empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the image empty.";
#elif L==4	// es
	"No puede dejar la imagen vac&iacute;a.";
#elif L==5	// fr
	"You can not leave the image empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar la imagen vac&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare l'image vuota.";
#elif L==8	// pl
	"You can not leave the image empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar a imagem vazia.";
#endif

const char *Txt_You_can_not_leave_the_IP_address_empty =
#if   L==1	// ca
	"No puede dejar la direcci&oacute;n IP vac&iacute;a.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the IP address empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the IP address empty.";
#elif L==4	// es
	"No puede dejar la direcci&oacute;n IP vac&iacute;a.";
#elif L==5	// fr
	"You can not leave the IP address empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar la direcci&oacute;n IP vac&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare vuoto l'indirizzo IP.";
#elif L==8	// pl
	"You can not leave the IP address empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o endere&ccedil;o IP vazio.";
#endif

const char *Txt_You_can_not_leave_the_logo_empty =
#if   L==1	// ca
	"No puede dejar el logo vac&iacute;o.";		// Necessita traduccio
#elif L==2	// de
	"You can not leave the logo empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the logo empty.";
#elif L==4	// es
	"No puede dejar el logo vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the logo empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el logo vac&iacute;o.";		// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare vuoto il logo.";
#elif L==8	// pl
	"You can not leave the logo empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o logotipo vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_banner_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del banner <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del banner <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del banner <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del banner <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do banner <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_centre_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del centro <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del centro <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del centro <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del centro <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do centro <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_classroom_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No pot deixar el nom"
	" de l'aula <strong>%s</strong> buit.";
#elif L==2	// de
	"Sie k&ouml;nnen den Namen"
	" des Klassenzimmer <strong>%s</strong> nicht leer lassen.";
#elif L==3	// en
	"You can not leave the name"
	" of the classroom <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del aula <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"Vous ne pouvez pas laisser le nom"
	" de la salle de classe <strong>%s</strong> vide.";
#elif L==6	// gn
	"No puede dejar el nombre"
	" del aula <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" della aula <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"Nie mo&zdot;esz zostawi&cacute;"
	" pustego imienia w klasie <strong>%s</strong>.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" da sala de clase <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_course_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" de la asignatura <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" de la asignatura <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" de la asignatura <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del corso <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" da disciplina <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_country_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del pa%iacute;s <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del pa%iacute;s <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del pa%iacute;s <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del paese <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do pa&iacute;s <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_degree_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" de la titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" de la titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" de la titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" della laurea <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do grau <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_department_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del departamento <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del departamento <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del departamento <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del dipartimento <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do departamento <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_email_domain_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre "
	"del dominio de correo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the email domain <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the email domain <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del dominio de correo <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the email domain <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del dominio de correo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del campo mail <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the email domain <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do dom&iacute;nio de email <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_field_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del campo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del campo <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del campo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del campo <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do campo <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_group_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del grupo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del grupo <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del grupo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del gruppo <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do grupo <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_holiday_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" de la festividad <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" de la festividad <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" de la festividad <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" della festivit&agrave; <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do feriado <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_institution_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" de la instituci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" de la instituci&oacute;n <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" de la instituci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" dell'istituzione <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" da institui&ccedil;&atilde;o <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_link_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del enlace <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del enlace <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del enlace <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del link <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do link <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_place_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No pot deixar el nom"
	" del lloc <strong>%s</strong> buit.";
#elif L==2	// de
	"Sie k&ouml;nnen den Namen"
	" des Standorte <strong>%s</strong> nicht leer lassen.";
#elif L==3	// en
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del lugar <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"Vous ne pouvez pas laisser le nom"
	" du emplacement <strong>%s</strong> vide.";
#elif L==6	// gn
	"No puede dejar el nombre"
	" del lugar <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del luogo <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"Nie mo&zdot;esz zostawi&cacute; nazwy"
	" miejsca <strong>%s</strong> pustego.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" da localiza&ccedil;&atilde;o <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_plugin_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del complemento <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del complemento <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del complemento <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del plugin <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do plugin <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_tag_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del descriptor <strong>%s</strong> vac&iacute;o.";		// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del descriptor <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del descriptor <strong>%s</strong> vac&iacute;o.";		// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" dell'etichetta <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do descritor <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_type_of_degree_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del tipo de titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del tipo de titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del tipo de titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del tipo di laurea <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do tipo de grau <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_name_of_the_type_of_group_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"No puede dejar el nombre"
	" del tipo de grupo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Need bersetzung
#elif L==3	// en
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";
#elif L==4	// es
	"No puede dejar el nombre"
	" del tipo de grupo <strong>%s</strong> vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el nombre"
	" del tipo de grupo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare il nome"
	" del tipo di gruppo <strong>%s</strong> vuoto.";
#elif L==8	// pl
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o nome"
	" do tipo de grupo <strong>%s</strong> vazio.";
#endif

const char *Txt_You_can_not_leave_the_URL_empty =
#if   L==1	// ca
	"No puede dejar el URL vac&iacute;o.";		// Necessita traduccio
#elif L==2	// de
	"You can not leave the URL empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the URL empty.";
#elif L==4	// es
	"No puede dejar el URL vac&iacute;o.";
#elif L==5	// fr
	"You can not leave the URL empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar el URL vac&iacute;o.";		// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare l'URL vuoto.";
#elif L==8	// pl
	"You can not leave the URL empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o URL vazio.";
#endif

const char *Txt_You_can_not_leave_the_web_address_empty =
#if   L==1	// ca
	"No puede dejar la direcci&oacute;n web vac&iacute;a.";	// Necessita traduccio
#elif L==2	// de
	"You can not leave the web address empty.";		// Need bersetzung
#elif L==3	// en
	"You can not leave the web address empty.";
#elif L==4	// es
	"No puede dejar la direcci&oacute;n web vac&iacute;a.";
#elif L==5	// fr
	"You can not leave the web address empty.";		// Besoin de traduction
#elif L==6	// gn
	"No puede dejar la direcci&oacute;n web vac&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi lasciare l'indirizzo web vuoto.";
#elif L==8	// pl
	"You can not leave the web address empty.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode deixar o endere&ccedil;o web vazio.";
#endif

const char *Txt_You_can_not_paste_file_or_folder_here =
#if   L==1	// ca
	"No puede pegar un archivo o carpeta aqu&iacute;.";		// Necessita traduccio
#elif L==2	// de
	"You can not paste file or folder here.";			// Need bersetzung
#elif L==3	// en
	"You can not paste file or folder here.";
#elif L==4	// es
	"No puede pegar un archivo o carpeta aqu&iacute;.";
#elif L==5	// fr
	"You can not paste file or folder here.";			// Besoin de traduction
#elif L==6	// gn
	"No puede pegar un archivo o carpeta aqu&iacute;.";		// Okoteve traduccin
#elif L==7	// it
	"Non puoi copiare file o cartella qui.";
#elif L==8	// pl
	"You can not paste file or folder here.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode colar arquivos ou pastas aqui.";
#endif

const char *Txt_You_can_not_remove_this_file_or_link =
#if   L==1	// ca
	"No puede eliminar este archivo o enlace.";		// Necessita traduccio
#elif L==2	// de
	"You can not remove this file or link.";		// Need bersetzung
#elif L==3	// en
	"You can not remove this file or link.";
#elif L==4	// es
	"No puede eliminar este archivo o enlace.";
#elif L==5	// fr
	"You can not remove this file or link.";		// Besoin de traduction
#elif L==6	// gn
	"No puede eliminar este archivo o enlace.";		// Okoteve traduccin
#elif L==7	// it
	"Non puoi rimuovere questo file o link.";
#elif L==8	// pl
	"You can not remove this file or link.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode remover este arquivo ou link.";
#endif

const char *Txt_You_can_not_remove_this_folder =
#if   L==1	// ca
	"No puede eliminar esta carpeta.";		// Necessita traduccio
#elif L==2	// de
	"You can not remove this folder.";		// Need bersetzung
#elif L==3	// en
	"You can not remove this folder.";
#elif L==4	// es
	"No puede eliminar esta carpeta.";
#elif L==5	// fr
	"You can not remove this folder.";		// Besoin de traduction
#elif L==6	// gn
	"No puede eliminar esta carpeta.";		// Okoteve traduccin
#elif L==7	// it
	"Non puoi rimuovere questa cartella.";
#elif L==8	// pl
	"You can not remove this folder.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode remover esta pasta.";
#endif

const char *Txt_You_can_not_remove_a_country_with_institutions_or_users =
#if   L==1	// ca
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";	// Necessita traduccio
#elif L==2	// de
	"You can not remove a country with institutions or users.";		// Need bersetzung
#elif L==3	// en
	"You can not remove a country with institutions or users.";
#elif L==4	// es
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";
#elif L==5	// fr
	"You can not remove a country with instituciones or users.";		// Besoin de traduction
#elif L==6	// gn
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi rimuovere un paese con istituzioni o utenti.";
#elif L==8	// pl
	"You can not remove a country with instituciones or users.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode remover um pa&iacute;s com instituciones ou usu&aacute;rios.";
#endif

const char *Txt_You_can_not_rename_this_folder =
#if   L==1	// ca
	"No puede cambiar el nombre de esta carpeta.";	// Necessita traduccio
#elif L==2	// de
	"You can not rename this folder.";		// Need bersetzung
#elif L==3	// en
	"You can not rename this folder.";
#elif L==4	// es
	"No puede cambiar el nombre de esta carpeta.";
#elif L==5	// fr
	"You can not rename this folder.";		// Besoin de traduction
#elif L==6	// gn
	"No puede cambiar el nombre de esta carpeta.";	// Okoteve traduccin
#elif L==7	// it
	"Non puoi rinominare questa cartella.";
#elif L==8	// pl
	"You can not rename this folder.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode renomear esta pasta.";
#endif

const char *Txt_You_can_not_send_a_message_to_so_many_recipients_ =
#if   L==1	// ca
	"No puede enviar un mensaje a tantsimos destinatarios."
	" Es mejor usar los foros.";		// Necessita traduccio
#elif L==2	// de
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Need bersetzung
#elif L==3	// en
	"You can not send a message to so many recipients."
	" Better use the forums.";
#elif L==4	// es
	"No puede enviar un mensaje a tantsimos destinatarios."
	" Es mejor usar los foros.";
#elif L==5	// fr
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Besoin de traduction
#elif L==6	// gn
	"No puede enviar un mensaje a tantsimos destinatarios."
	" Es mejor usar los foros.";		// Okoteve traduccin
#elif L==7	// it
	"Non puoi inviare un messaggio a cos tanti destinatari."
	" Meglio usare il forum.";
#elif L==8	// pl
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode enviar uma mensagem para tantos destinat&aacute;rios."
	" Melhor usar os f&oacute;runs.";
#endif

const char *Txt_You_can_not_take_a_new_test_until =
#if   L==1	// ca
	"No podr&agrave; fer un test fins";
#elif L==2	// de
	"Sie k&ouml;nnen nicht einen neuen Test bis";
#elif L==3	// en
	"You can not take a new test until";
#elif L==4	// es
	"No podr&aacute; hacer un nuevo test hasta";
#elif L==5	// fr
	"Vous ne pouvez pas passer un nouveau test avant";
#elif L==6	// gn
	"No podr&aacute; hacer un nuevo test hasta";	// Okoteve traduccin
#elif L==7	// it
	"Non si pu&ograve; fare un nuovo test fino";
#elif L==8	// pl
	"Nie mo&zdot;na wykona&cacute; nowy test do";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o pode fazer um novo teste at&eacute;";
#endif

const char *Txt_You_can_only_perform_X_further_actions_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Solo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";		// Necessita traduccio
#elif L==2	// de
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Need bersetzung
#elif L==3	// en
	"You can only perform %u further actions"
	" if you don't send your photo.";
#elif L==4	// es
	"Solo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";
#elif L==5	// fr
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Besoin de traduction
#elif L==6	// gn
	"Solo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";		// Okoteve traduccin
#elif L==7	// it
	"Puoi solo realizzare %u massime azioni"
	" se non hai inviato la tua foto.";
#elif L==8	// pl
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; s&oacute; pode executar %u a&ccedil;&otilde;es adicionais"
	" se n&atilde;o enviar sua foto.";
#endif

const char *Txt_You_can_only_receive_email_notifications_if_ =	// Warning: it is very important to include five %s in the following sentences
#if   L==1	// ca
	"Nom&eacute;s podr&agrave; rebre notificacions per correu"
	" si la seva adre&ccedil;a de correu est&agrave; confirmada (%s &gt; %s)"
	" i acaba en un dels dominis permesos (%s &gt; %s &gt; %s).";
#elif L==2	// de
	"Sie k&ouml;nnen nur E-Mail-Anmeldungen zu erhalten,"
	" wenn Sie Ihre E-Mail-Adresse best&uuml;tigt wird (%s &gt; %s),"
	" und endet in einer der erlaubt Domains (%s &gt; %s &gt; %s).";
#elif L==3	// en
	"You can only receive email notifications"
	" if your email address is confirmed (%s &gt; %s)"
	" and ends in one of the allowed domains (%s &gt; %s &gt; %s).";
#elif L==4	// es
	"Solo podr&aacute; recibir notificaciones por correo"
	" si su direcci&oacute;n de correo est&aacute; confirmada (%s &gt; %s)"
	" y termina en uno de los dominios permitidos (%s &gt; %s &gt; %s).";
#elif L==5	// fr
	"Vous ne pouvez recevoir des notifications par email"
	" si votre adresse email est confirm&eacute; (%s &gt; %s)"
	" et se termine dans l'un des domaines autoris&eacute;s (%s &gt; %s &gt; %s).";
#elif L==6	// gn
	"Solo podr&aacute; recibir notificaciones por correo"
	" si su direcci&oacute;n de correo est&aacute; confirmada (%s &gt; %s)"
	" y termina en uno de los dominios permitidos (%s &gt; %s &gt; %s).";				// Okoteve traduccin
#elif L==7	// it
	"Puoi solo ricevere email di notifica"
	" se la tua email &egrave; confermata (%s &gt; %s)"
	" e si termina in uno dei domini consentiti (%s &gt; %s &gt; %s).";
#elif L==8	// pl
	"Mo&zdot;na otrzyma&cacute; tylko informacj&eogon;,"
	" czy adres email jest potwierdzone (%s &gt; %s),"
	" a ko&nacute;czy si&eogon; w jednej z dozwolone domeny (%s &gt; %s &gt; %s).";
#elif L==9	// pt
	"Voc&ecirc; s&oacute; pode receber notifica&ccedil;&otilde;es de email"
	" se o seu endere&ccedil;o de email est&aacute; confirmado (%s &gt; %s)"
	" e termina em um dos dom&iacute;nios permitidos (%s &gt; %s &gt; %s).";
#endif

const char *Txt_You_can_register_voluntarily_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";	// Necessita traduccio
#elif L==2	// de
	"You can register voluntarily in one group of type %s.";	// Need bersetzung
#elif L==3	// en
	"You can register voluntarily in one group of type %s.";
#elif L==4	// es
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";
#elif L==5	// fr
	"You can register voluntarily in one group of type %s.";	// Besoin de traduction
#elif L==6	// gn
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==7	// it
	"Ti puoi registrare volontariamente in uno dei gruppi di tipo %s.";
#elif L==8	// pl
	"You can register voluntarily in one group of type %s.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; pode se inscrever voluntariamente em um grupo do tipo %s.";
#endif

const char *Txt_You_can_register_voluntarily_in_one_or_more_groups_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";		// Necessita traduccio
#elif L==2	// de
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Need bersetzung
#elif L==3	// en
	"You can register voluntarily"
	" in one or more groups of type %s.";
#elif L==4	// es
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";
#elif L==5	// fr
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Besoin de traduction
#elif L==6	// gn
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";		// Okoteve traduccin
#elif L==7	// it
	"Ti puoi registrare volontariamente"
	" in uno o pi&ugrave; gruppi di tipo %s.";
#elif L==8	// pl
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; pode se inscrever voluntariamente"
	" em um ou mais grupos do tipo %s.";
#endif

const char *Txt_You_can_search_for_courses_select_them_and_request_your_enrolment_in_them =
#if   L==1	// ca
	"Podeu cercar assignatures, seleccionar-les"
	" i sol&middot;licitar la seva inscripci&oacute; en elles.";
#elif L==2	// de
	"Sie k&ouml;nnen f&uuml;r Kurse zu suchen, w&auml;hlen Sie sie aus"
	" und fordern Sie Ihre Anmeldung in ihnen.";
#elif L==3	// en
	"You can search for courses, select them"
	" and request your enrolment in them.";
#elif L==4	// es
	"Puede buscar asignaturas, seleccionarlas"
	" y solicitar su inscripci&oacute;n en ellas.";
#elif L==5	// fr
	"Vous pouvez rechercher des cours, s&eacute;lectionnez-les"
	" et demander votre inscription en eux.";
#elif L==6	// gn
	"Puede buscar asignaturas, seleccionarlas"
	" y solicitar su inscripci&oacute;n en ellas.";		// Okoteve traduccin"
#elif L==7	// it
	"&Egrave; possibile cercare corsi, selezionarli"
	" e richiedere l'iscrizione in loro.";
#elif L==8	// pl
	"Mo&zdot;esz szuka&cacute; kurs&oacute;w, zaznacz je"
	" i poprosi&cacute; o rejestracj&eogon; w nich.";
#elif L==9	// pt
	"Voc&ecirc; pode procurar por cursos, selecion&aacute;-los"
	" e solicitar a sua inscri&ccedil;&atilde;o nos mesmos.";
#endif

const char *Txt_You_can_send_a_file_with_an_image_in_PNG_format_transparent_background_and_size_X_Y =	// Warning: it is very important to include two %u in the following sentences
#if   L==1	// ca
	"Puede enviar un archivo con una imagen en formato <em>PNG</em>,"
	" fondo transparente y tama&ntilde;o %u&times;%u p&iacute;xeles.";	// Necessita traduccio
#elif L==2	// de
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";			// Need bersetzung
#elif L==3	// en
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";
#elif L==4	// es
	"Puede enviar un archivo con una imagen en formato <em>PNG</em>,"
	" fondo transparente y tama&ntilde;o %u&times;%u p&iacute;xeles.";
#elif L==5	// fr
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";			// Besoin de traduction
#elif L==6	// gn
	"Puede enviar un archivo con una imagen en formato <em>PNG</em>,"
	" fondo transparente y tama&ntilde;o %u&times;%u p&iacute;xeles.";	// Okoteve traduccin
#elif L==7	// it
	"Puoi inviare un file con un'immagine in formato <em>PNG</em>,"
	" sfondo trasparente e di dimensione %u&times;%u pixel.";
#elif L==8	// pl
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; pode enviar um arquivo com uma imagem no formato <em>PNG</em>,"
	" fundo transparente e tamanho %u&times;%u pixels.";
#endif

const char *Txt_You_can_send_a_file_with_an_image_in_JPEG_format_ =
#if   L==1	// ca
	"Podeu enviar un arxiu amb una imatge en format <em>JPEG</em>.<br />"
	"La imatge ha de contenir almenys un rostre en posici&oacute; vertical i frontal,"
	" i amb el fons blanc o molt clar.";
#elif L==2	// de
	"Sie k&ouml;nnen eine Datei mit einem Bild im <em>JPEG</em>-Format senden.<br />"
	"Das Bild muss mindestens eine Fl&auml;che in vertikaler und frontaler Position enthalten,"
	" und der Hintergrund muss wei&szlig; oder sehr hell sein.";
#elif L==3	// en
	"You can send a file with an image in <em>JPEG</em> format.<br />"
	"The image must contain at least one face in vertical and frontal position,"
	" and the background must be white or very light.";
#elif L==4	// es
	"Puede enviar un archivo con una imagen en formato <em>JPEG</em>.<br />"
	"La imagen debe contener al menos un rostro en posici&oacute;n vertical y frontal,"
	" y con el fondo blanco o muy claro.";
#elif L==5	// fr
	"Vous pouvez envoyer un fichier avec une image au format JPEG.<br />"
	"L'image doit contenir au moins un visage en position verticale et frontale,"
	" et le fond doit &ecirc;tre blanc ou tr&egrave;s clair.";
#elif L==6	// gn
	"Puede enviar un archivo con una imagen en formato <em>JPEG</em>.<br />"
	"La imagen debe contener al menos un rostro en posici&oacute;n vertical y frontal,"
	" y con el fondo blanco o muy claro.";				// Okoteve traduccin
#elif L==7	// it
	"&Egrave; possibile inviare un file in un'immagine con formato <em>JPEG</em>.<br />"
	"L'immagine deve contenere almeno una faccia in posizione verticale e frontale,"
	" e lo sfondo deve essere bianco o molto chiaro.";
#elif L==8	// pl
	"Mo&zdot;esz wys&lstrok;a&cacute; plik obrazu w formacie <em>JPEG</em>.<br />"
	"Obraz musi zawiera&cacute; co najmniej jedn&aogon; twarz w pozycji pionowej jak i czo&lstrok;owego,"
	" a t&lstrok;o musi by&cacute; bia&lstrok;y lub bardzo jasnym.";
#elif L==9	// pt
	"Voc&ecirc; pode enviar um arquivo em uma imagem com formato <em>JPEG</em>.<br />"
	"A imagem deve conter pelo menos um rosto na posi&ccedil;&atilde;o vertical e frontal,"
	" eo fundo deve ser branco ou muito claro.";
#endif

const char *Txt_You_dont_follow_any_user =
#if   L==1	// ca
	"Vost&egrave; no segueix a cap usuari.";
#elif L==2	// de
	"Sie haben noch keine Benutzer folgen.";
#elif L==3	// en
	"You don't follow any user.";
#elif L==4	// es
	"Usted no sigue a ning&uacute;n usuario.";
#elif L==5	// fr
	"Vous ne suivez pas aucun utilisateur.";
#elif L==6	// gn
	"Usted no sigue a ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Non si seguono gli utenti.";
#elif L==8	// pl
	"Nie po &zdot;adnej u&zdot;ytkownika.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o segue nenhum usu&aacute;rio.";
#endif

const char *Txt_You_dont_have_permission_to_perform_this_action =
#if   L==1	// ca
	"Vost&egrave; no t&eacute; perm&iacute;s"
	" per realitzar aquesta acci&oacute;.";
#elif L==2	// de
	"Sie haben keine Berechtigung,"
	" um diese Handlung durchzuf&uuml;hren.";
#elif L==3	// en
	"You don't have permission"
	" to perform this action.";
#elif L==4	// es
	"Usted no tiene permiso"
	" para realizar esta acci&oacute;n.";
#elif L==5	// fr
	"Vous n'avez pas la permission"
	" d'effectuer cette action.";
#elif L==6	// gn
	"Usted no tiene permiso"
	" para realizar esta acci&oacute;n.";		// Okoteve traduccin
#elif L==7	// it
	"Non si dispone dell'autorizzazione"
	" per eseguire questa azione.";
#elif L==8	// pl
	"You don't have permission"
	" to perform this action.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o tem permiss&atilde;o"
	" para realizar esta a&ccedil;&atilde;o.";
#endif

const char *Txt_You_have_confirmed_that_you_have_read_this_information =
#if   L==1	// ca
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"You have confirmed that you have read this information.";	// Need bersetzung
#elif L==3	// en
	"You have confirmed that you have read this information.";
#elif L==4	// es
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";
#elif L==5	// fr
	"You have confirmed that you have read this information.";	// Besoin de traduction
#elif L==6	// gn
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Hai confermato che hai letto questa informazione.";
#elif L==8	// pl
	"You have confirmed that you have read this information.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; confirmou que leu esta informa&ccedil;&atilde;o.";
#endif

const char *Txt_You_have_confirmed_your_enrolment_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"You have confirmed your enrolment"
	" in the course <strong>%s</strong>.";		// Need bersetzung
#elif L==3	// en
	"You have confirmed your enrolment"
	" in the course <strong>%s</strong>.";
#elif L==4	// es
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"You have confirmed your enrolment"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"Hai confermato la tua registrazione"
	" in questo corso <strong>%s</strong>.";
#elif L==8	// pl
	"You have confirmed your enrolment"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; confirmou sua inscri&ccedil;&atilde;o"
	" na disciplina <strong>%s</strong>.";
#endif

const char *Txt_You_have_eliminated_the_confirmation_that_you_have_read_this_information =
#if   L==1	// ca
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"You have eliminated the confirmation"
	" that you have read this information.";	// Need bersetzung
#elif L==3	// en
	"You have eliminated the confirmation"
	" that you have read this information.";
#elif L==4	// es
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";
#elif L==5	// fr
	"You have eliminated the confirmation"
	" that you have read this information.";	// Besoin de traduction
#elif L==6	// gn
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Hai rimosso la conferma"
	" che hai letto questa informazione.";
#elif L==8	// pl
	"You have eliminated the confirmation"
	" that you have read this information.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; eliminou a confirma&ccedil;&atilde;o"
	" de que leu esta informa&ccedil;&atilde;o.";
#endif

const char *Txt_You_have_followed_one_user =
#if   L==1	// ca
	"Ha seguit a un usuari.";
#elif L==2	// de
	"Sie haben einen Benutzer verfolgt.";
#elif L==3	// en
	"You have followed one user.";
#elif L==4	// es
	"Ha seguido a un usuario.";
#elif L==5	// fr
	"Vous avez suivi un utilisateur.";
#elif L==6	// gn
	"Ha seguido a un usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Hai seguito un utente.";
#elif L==8	// pl
	"&Sacute;ledzi&lstrok;e&sacute; jednego u&zdot;ytkownika.";
#elif L==9	// pt
	"Voc&ecirc; seguiu um usu&aacute;rio.";
#endif

const char *Txt_You_have_followed_X_users =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Ha seguit a %u usuaris.";
#elif L==2	// de
	"Sie haben %u Benutzer verfolgt.";
#elif L==3	// en
	"You have followed %u users.";
#elif L==4	// es
	"Ha seguido a %u usuarios.";
#elif L==5	// fr
	"Vous avez suivi %u utilisateurs.";
#elif L==6	// gn
	"Ha seguido a %u usuarios.";	// Okoteve traduccin
#elif L==7	// it
	"Hai seguito %u utenti.";
#elif L==8	// pl
	"&Sacute;ledzi&lstrok;e&sacute; %u u&zdot;ytkownik&oacute;w.";
#elif L==9	// pt
	"Voc&ecirc; seguiu %u usu&aacute;rios.";
#endif

const char *Txt_You_have_no_notifications =
#if   L==1	// ca
	"No tens notificacions.";
#elif L==2	// de
	"Sie haben keine Benachrichtigungen.";
#elif L==3	// en
	"You have no notifications.";
#elif L==4	// es
	"No tiene notificaciones.";
#elif L==5	// fr
	"Vous n'avez aucune notification.";
#elif L==6	// gn
	"No tiene notificaciones.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono notifiche.";
#elif L==8	// pl
	"Nie masz &zdot;adnych powiadomie&nacute;.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o tem notifica&ccedil;&otilde;es.";
#endif

const char *Txt_You_have_no_unread_notifications =
#if   L==1	// ca
	"No tens notificacions sense llegir.";
#elif L==2	// de
	"Sie haben keine ungelesenen Benachrichtigungen.";
#elif L==3	// en
	"You have no unread notifications.";
#elif L==4	// es
	"No tiene notificaciones sin leer.";
#elif L==5	// fr
	"Vous n'avez aucune notification non lue.";
#elif L==6	// gn
	"No tiene notificaciones sin leer.";	// Okoteve traduccin
#elif L==7	// it
	"Non ci sono notifiche non lette.";
#elif L==8	// pl
	"Nie masz nieprzeczytanych powiadomie&nacute;.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o tem notifica&ccedil;&otilde;es n&atilde;o lidas.";
#endif

const char *Txt_You_have_not_banned_any_sender =
#if   L==1	// ca
	"No ha bloqueado ning&uacute;n remitente.";	// Necessita traduccio
#elif L==2	// de
	"You have not banned any sender.";		// Need bersetzung
#elif L==3	// en
	"You have not banned any sender.";
#elif L==4	// es
	"No ha bloqueado ning&uacute;n remitente.";
#elif L==5	// fr
	"You have not banned any sender.";		// Besoin de traduction
#elif L==6	// gn
	"No ha bloqueado ning&uacute;n remitente.";	// Okoteve traduccin
#elif L==7	// it
	"You have not banned any sender.";		// Bisogno di traduzione
#elif L==8	// pl
	"You have not banned any sender.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o baniu nenhum remetente.";
#endif

const char *Txt_You_have_not_confirmed_the_action =
#if   L==1	// ca
	"No ha confirmat l'acci&oacute;.";
#elif L==2	// de
	"Sie haben nicht die Aktion best&auml;tigt.";
#elif L==3	// en
	"You have not confirmed the action.";
#elif L==4	// es
	"No ha confirmado la acci&oacute;n.";
#elif L==5	// fr
	"Vous n'avez pas confirm&eacute; l'action.";
#elif L==6	// gn
	"No ha confirmado la acci&oacute;n.";		// Okoteve traduccin
#elif L==7	// it
	"Non hai inserito correttamente la tua password.";
#elif L==8	// pl
	"Nie potwierdzi&lstrok;y dzia&lstrok;anie.";
#elif L==9	// pt
	"Voc&ecirc; ainda n&atilde;o confirmou a a&ccedil;&atilde;o.";
#endif

const char *Txt_You_have_not_entered_your_password_correctly =
#if   L==1	// ca
	"No ha introdu&iuml;t correctament la seva contrasenya.";
#elif L==2	// de
	"Sie haben Ihr Passwort nicht korrekt eingegeben wurde.";
#elif L==3	// en
	"You have not entered your password correctly.";
#elif L==4	// es
	"No ha introducido correctamente su contrase&ntilde;a.";
#elif L==5	// fr
	"Vous n'avez pas entr&eacute; votre mot de passe correctement.";
#elif L==6	// gn
	"No ha introducido correctamente su contrase&ntilde;a.";	// Okoteve traduccin
#elif L==7	// it
	"Non hai inserito correttamente la tua password.";
#elif L==8	// pl
	"Nie wpisa&lstrok;e&sacute; poprawnie has&lstrok;o.";
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o digitou sua senha corretamente.";
#endif

const char *Txt_You_have_not_written_twice_the_same_new_password =
#if   L==1	// ca
	"No ha escrito dos veces la misma contrase&ntilde;a nueva.";	// Necessita traduccio
#elif L==2	// de
	"You have not written twice the same new password.";			// Need bersetzung
#elif L==3	// en
	"You have not written twice the same new password.";
#elif L==4	// es
	"No ha escrito dos veces la misma contrase&ntilde;a nueva.";
#elif L==5	// fr
	"You have not written twice the same new password.";			// Besoin de traduction
#elif L==6	// gn
	"No ha escrito dos veces la misma contrase&ntilde;a nueva.";	// Okoteve traduccin
#elif L==7	// it
	"Non hai scritto due volte la stessa password nuova.";
#elif L==8	// pl
	"You have not written twice the same new password.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; n&atilde;o escreveu duas vezes a mesma nova senha.";
#endif

const char *Txt_You_have_stopped_following_one_user =
#if   L==1	// ca
	"Ha deixat de seguir a un usuari.";
#elif L==2	// de
	"Sie haben aufgeh&ouml;rt, einem Benutzer zu folgen.";
#elif L==3	// en
	"You have stopped following one user.";
#elif L==4	// es
	"Ha dejado de seguir a un usuario.";
#elif L==5	// fr
	"Vous avez arr&ecirc;t&eacute; de suivre un utilisateur.";
#elif L==6	// gn
	"Ha dejado de seguir a un usuario.";	// Okoteve traduccin
#elif L==7	// it
	"Hai smesso di seguire un utente.";
#elif L==8	// pl
	"Przesta&lstrok;e&sacute; obserwowa&cacute; u&zdot;ytkownika.";
#elif L==9	// pt
	"Voc&ecirc; parou de seguir um usu&aacute;rio.";
#endif

const char *Txt_You_have_stopped_following_X_users =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Ha deixat de seguir a %u usuaris.";
#elif L==2	// de
	"Sie haben aufgeh&ouml;rt, %u Benutzern zu folgen.";
#elif L==3	// en
	"You have stopped following %u users.";
#elif L==4	// es
	"Ha dejado de seguir a %u usuarios.";
#elif L==5	// fr
	"Vous avez cess&eacute; de suivre %u utilisateurs.";
#elif L==6	// gn
	"Ha dejado de seguir a %u usuarios.";	// Okoteve traduccin
#elif L==7	// it
	"Hai smesso di seguire %u utenti.";
#elif L==8	// pl
	"Zatrzyma&lstrok;e&sacute; &sacute;ledzenie %u u&zdot;ytkownik&oacute;w.";
#elif L==9	// pt
	"Voc&ecirc; parou de seguir %u usu&aacute;rios.";
#endif

const char *Txt_You_have_to_register_compulsorily_at_least_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";	// Necessita traduccio
#elif L==2	// de
	"You have to register compulsorily at least in one group of type %s.";	// Need bersetzung
#elif L==3	// en
	"You have to register compulsorily at least in one group of type %s.";
#elif L==4	// es
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";
#elif L==5	// fr
	"You have to register compulsorily at least in one group of type %s.";	// Besoin de traduction
#elif L==6	// gn
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==7	// it
	"Devi registrarti obbligatoriamente almeno in uno dei gruppi del tipo %s.";
#elif L==8	// pl
	"You have to register compulsorily at least in one group of type %s.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; tem que se registrar obrigatoriamente pelo menos em um grupo do tipo %s.";
#endif

const char *Txt_You_have_to_register_compulsorily_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1	// ca
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";	// Necessita traduccio
#elif L==2	// de
	"You have to register compulsorily in one group of type %s.";	// Need bersetzung
#elif L==3	// en
	"You have to register compulsorily in one group of type %s.";
#elif L==4	// es
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";
#elif L==5	// fr
	"You have to register compulsorily in one group of type %s.";	// Besoin de traduction
#elif L==6	// gn
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==7	// it
	"Devi registrarti obbligatoriamente in uno dei gruppi del tipo %s.";
#elif L==8	// pl
	"You have to register compulsorily in one group of type %s.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; tem que se registrar obrigatoriamente em um grupo do tipo %s.";
#endif

const char *Txt_You_have_written_1_post_in_this_thread =
#if   L==1	// ca
	"Usted ha escrito 1 comentario en esta discusi&oacute;n";		// Necessita traduccio
#elif L==2	// de
	"Sie haben 1 Nachricht in diesem Thread";
#elif L==3	// en
	"You have written 1 post in this thread";
#elif L==4	// es
	"Usted ha escrito 1 comentario en esta discusi&oacute;n";
#elif L==5	// fr
	"Vous avez &eacute;crit 1 post dans ce fil";
#elif L==6	// gn
	"Usted ha escrito 1 comentario en esta discusi&oacute;n";		// Okoteve traduccin
#elif L==7	// it
	"Hai scritto 1 post in questa discussione";
#elif L==8	// pl
	"You have written 1 post in this thread";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; escreveu 1 post nesta discuss&atilde;o";
#endif

const char *Txt_You_have_written_X_posts_in_this_thread =		// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"Usted ha escrito %u comentarios en esta discusi&oacute;n";	// Necessita traduccio
#elif L==2	// de
	"Sie haben %u Nachrichten in diesem Thread";
#elif L==3	// en
	"You have written %u posts in this thread";
#elif L==4	// es
	"Usted ha escrito %u comentarios en esta discusi&oacute;n";
#elif L==5	// fr
	"Vous avez &eacute;crit %u posts dans ce fil";
#elif L==6	// gn
	"Usted ha escrito %u comentarios en esta discusi&oacute;n";	// Okoteve traduccin
#elif L==7	// it
	"Hai scritto %u post in questa discussione";
#elif L==8	// pl
	"You have written %u posts in this thread";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; escreveu %u posts nesta discuss&atilde;o";
#endif

const char *Txt_You_must_enter_an_integer_value_as_the_correct_answer =
#if   L==1	// ca
	"Debe escribir un valor entero como respuesta correcta.";	// Necessita traduccio
#elif L==2	// de
	"You must enter an integer value as the correct answer.";	// bersetzung notwendig
#elif L==3	// en
	"You must enter an integer value as the correct answer.";
#elif L==4	// es
	"Debe escribir un valor entero como respuesta correcta.";
#elif L==5	// fr
	"You must enter an integer value as the correct answer.";	// Besoin de traduction
#elif L==6	// gn
	"Debe escribir un valor entero como respuesta correcta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi inserire un valore intero come risposta corretta.";
#elif L==8	// pl
	"You must enter an integer value as the correct answer.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve inserir um valor inteiro como a resposta correta.";
#endif

const char *Txt_You_must_enter_the_range_of_floating_point_values_allowed_as_answer =
#if   L==1	// ca
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";	// Necessita traduccio
#elif L==2	// de
	"You must enter the range of floating point values allowed as answer.";		// bersetzung notwendig
#elif L==3	// en
	"You must enter the range of floating point values allowed as answer.";
#elif L==4	// es
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";
#elif L==5	// fr
	"You must enter the range of floating point values allowed as answer.";		// Besoin de traduction
#elif L==6	// gn
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi inserire il range di valori reali permessi come risposta.";
#elif L==8	// pl
	"You must enter the range of floating point values allowed as answer.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve inserir o intervalo de valores de ponto flutuante permitidos como resposta.";
#endif

const char *Txt_You_must_enter_your_nick_email_or_ID =
#if   L==1	// ca
	"Ha d'escriure el seu @sobrenom, correu o ID (DNI/c&egrave;dula).";
#elif L==2	// de
	"Bitte geben Sie Ihre @Benutzername, Email oder Ausweis-Nr.";
#elif L==3	// en
	"You must enter your @nick, email or ID.";
#elif L==4	// es
	"Debe escribir su @apodo, correo o ID (DNI/c&eacute;dula).";
#elif L==5	// fr
	"Vous devez entrer votre @surnom, courrier ou num&eacute;ro d'identit&eacute;.";
#elif L==6	// gn
	"Debe escribir su @apodo, correo o ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==7	// it
	"Devi inserire il tuo @nome-utente, email o numero carta d'identit&agrave;.";
#elif L==8	// pl
	"Musisz napisa&cacute; @pseudonim, email lub ID.";
#elif L==9	// pt
	"Voc&ecirc; deve digitar seu @alcunha, email ou n&ordm; identif.";
#endif

const char *Txt_You_must_mark_an_answer_as_correct =
#if   L==1	// ca
	"Debe marcar alguna respuesta como correcta.";	// Necessita traduccio
#elif L==2	// de
	"You must mark an answer as correct.";		// Need bersetzung
#elif L==3	// en
	"You must mark an answer as correct.";
#elif L==4	// es
	"Debe marcar alguna respuesta como correcta.";
#elif L==5	// fr
	"You must mark an answer as correct.";		// Besoin de traduction
#elif L==6	// gn
	"Debe marcar alguna respuesta como correcta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi segnare una risposta come corretta.";
#elif L==8	// pl
	"You must mark an answer as correct.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve marcar uma resposta como correta.";
#endif

const char *Txt_You_must_select_a_T_F_answer =
#if   L==1	// ca
	"Debe seleccionar una respuesta V o F.";	// Necessita traduccio
#elif L==2	// de
	"You must select a T/F answer.";		// Need bersetzung
#elif L==3	// en
	"You must select a T/F answer.";
#elif L==4	// es
	"Debe seleccionar una respuesta V o F.";
#elif L==5	// fr
	"You must select a T/F answer.";		// Besoin de traduction
#elif L==6	// gn
	"Debe seleccionar una respuesta V o F.";	// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare una risposta V/F.";
#elif L==8	// pl
	"You must select a T/F answer.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve selecionar uma resposta V ou F.";
#endif

const char *Txt_You_must_select_one_ore_more_questions =
#if   L==1	// ca
	"Heu de seleccionar una o m&eacute;s preguntes.";
#elif L==2	// de
	"Sie m&uuml;ssen eine oder mehrere Fragen ausw&auml;hlen.";
#elif L==3	// en
	"You must select one or more questions.";
#elif L==4	// es
	"Debe seleccionar una o m&aacute;s preguntas.";
#elif L==5	// fr
	"Vous devez s&eacute;lectionner une ou plusieurs questions.";
#elif L==6	// gn
	"Debe seleccionar una o m&aacute;s preguntas.";		// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare una o pi&ugrave; domande.";
#elif L==8	// pl
	"Musisz wybra&cacute; jedno lub wi&eogon;cej pyta&nacute;.";
#elif L==9	// pt
	"Voc&ecirc; deve selecionar uma ou mais perguntas.";
#endif

const char *Txt_You_must_select_one_ore_more_recipients =
#if   L==1	// ca
	"Debe seleccionar uno o m&aacute;s destinatarios.";	// Necessita traduccio
#elif L==2	// de
	"Es muss mindestens ein Empf&auml;nger angegeben werden.";
#elif L==3	// en
	"You must select one or more recipients.";
#elif L==4	// es
	"Debe seleccionar uno o m&aacute;s destinatarios.";
#elif L==5	// fr
	"You must select one or more recipients.";		// Besoin de traduction
#elif L==6	// gn
	"Debe seleccionar uno o m&aacute;s destinatarios.";	// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare uno o pi&ugrave; destinatari.";
#elif L==8	// pl
	"You must select one or more recipients.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve selecionar um ou mais destinat&aacute;rios.";
#endif

const char *Txt_You_must_select_one_ore_more_tags =
#if   L==1	// ca
	"Debe seleccionar uno o m&aacute;s descriptores.";	// Necessita traduccio
#elif L==2	// de
	"Es muss mindestens ein Tag ausgew&auml;hlt werden.";
#elif L==3	// en
	"You must select one or more tags.";
#elif L==4	// es
	"Debe seleccionar uno o m&aacute;s descriptores.";
#elif L==5	// fr
	"You must select one or more tags.";			// Besoin de traduction
#elif L==6	// gn
	"Debe seleccionar uno o m&aacute;s descriptores.";	// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare una o pi&ugrave; etichette.";
#elif L==8	// pl
	"You must select one or more tags.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve selecionar uma ou mais descritores.";
#endif

const char *Txt_You_must_select_one_ore_more_types_of_answer =
#if   L==1	// ca
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";	// Necessita traduccio
#elif L==2	// de
	"Es muss mindestens ein Antworttyp ausgew&auml;hlt werden.";
#elif L==3	// en
	"You must select one or more types of answer.";
#elif L==4	// es
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";
#elif L==5	// fr
	"You must select one or more types of answer.";			// Besoin de traduction
#elif L==6	// gn
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare uno o pi&ugrave; tipi di risposta.";
#elif L==8	// pl
	"You must select one or more types of answer.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve selecionar um ou mais tipos de resposta.";
#endif

const char *Txt_You_must_select_one_ore_more_users =
#if   L==1	// ca
	"Debe seleccionar uno o m&aacute;s usuarios.";	// Necessita traduccio
#elif L==2	// de
	"Es muss mindestens ein Benutzer ausgew&auml;hlt werden.";
#elif L==3	// en
	"You must select one or more users.";
#elif L==4	// es
	"Debe seleccionar uno o m&aacute;s usuarios.";
#elif L==5	// fr
	"You must select one or more users.";		// Besoin de traduction
#elif L==6	// gn
	"Debe seleccionar uno o m&aacute;s usuarios.";	// Okoteve traduccin
#elif L==7	// it
	"Devi selezionare uno o pi&ugrave; utenti.";
#elif L==8	// pl
	"You must select one or more users.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve selecionar um ou mais usu&acute;rios.";
#endif

const char *Txt_You_must_send_your_photo_because_ =
#if   L==1	// ca
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";	// Necessita traduccio
#elif L==2	// de
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Need bersetzung
#elif L==3	// en
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";
#elif L==4	// es
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";
#elif L==5	// fr
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Besoin de traduction
#elif L==6	// gn
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";	// Okoteve traduccin
#elif L==7	// it
	"Devi inviare la tua foto,"
	" perch&egrave; hai esaurito il numero di accessi"
	" permessi senza foto.";
#elif L==8	// pl
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve enviar sua foto,"
	" porque esgotou o n&uacute;mero de cliques"
	" permitidos sem foto.";
#endif

const char *Txt_You_must_specify_the_image_of_the_new_banner =
#if   L==1	// ca
	"Debe especificar la imagen"
	" del nuevo banner.";		// Necessita traduccio
#elif L==2	// de
	"You must specify the image"
	" of the new banner.";		// Need bersetzung
#elif L==3	// en
	"You must specify the image"
	" of the new banner.";
#elif L==4	// es
	"Debe especificar la imagen"
	" del nuevo banner.";
#elif L==5	// fr
	"You must specify the image"
	" of the new banner.";		// Besoin de traduction
#elif L==6	// gn
	"Debe especificar la imagen"
	" del nuevo banner.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare l'image"
	" del nuovo banner.";
#elif L==8	// pl
	"You must specify the image"
	" of the new banner.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar a imagem"
	" do novo banner.";
#endif

const char *Txt_You_must_specify_the_logo_the_application_key_the_URL_and_the_IP_address_of_the_new_plugin =
#if   L==1	// ca
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";		// Necessita traduccio
#elif L==2	// de
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Need bersetzung
#elif L==3	// en
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";
#elif L==4	// es
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";
#elif L==5	// fr
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il logo, il codice di applicazione,"
	" l'URL e l'indirizzo IP del nuovo plugin.";
#elif L==8	// pl
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o logotipo, a chave do aplicativo,"
	" o URL e o endere&ccedil;o IP do novo plugin.";
#endif

const char *Txt_You_must_specify_the_numerical_code_of_the_new_country =
#if   L==1	// ca
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Need bersetzung
#elif L==3	// en
	"You must specify the ISO 3166-1 numerical code of the new country.";
#elif L==4	// es
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";
#elif L==5	// fr
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il codice numerico ISO 3166-1 del nuovo paese.";
#elif L==8	// pl
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o c&oacute;digo num&eacute;rico ISO 3166-1 do novo pa&iacute;s.";
#endif

const char *Txt_You_must_specify_the_name_and_the_description_of_the_new_plugin =
#if   L==1	// ca
	"Debe especificar el nombre y la descripci&oacute;n del nuevo complemento.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the name and the description of the new plugin.";		// Need bersetzung
#elif L==3	// en
	"You must specify the name and the description of the new plugin.";
#elif L==4	// es
	"Debe especificar el nombre y la descripci&oacute;n del nuevo complemento.";
#elif L==5	// fr
	"You must specify the name and the description of the new plugin.";		// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre y la descripci&oacute;n del nuevo complemento.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome e la descrizione del nuovo plugin.";
#elif L==8	// pl
	"You must specify the name and the description of the new plugin.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome e a descri&ccedil;&atilde;o do novo plugin.";
#endif

const char *Txt_You_must_specify_the_name_of_the_new_country_in_all_languages =
#if   L==1	// ca
	"Debe especificar el nombre del nuevo pa&iacute;s en todos los idiomas.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the name of the new country in all languages.";		// Need bersetzung
#elif L==3	// en
	"You must specify the name of the new country in all languages.";
#elif L==4	// es
	"Debe especificar el nombre del nuevo pa&iacute;s en todos los idiomas.";
#elif L==5	// fr
	"You must specify the name of the new country in all languages.";		// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre del nuevo pa&iacute;s en todos los idiomas.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome del nuovo paese in tutte le lingue.";
#elif L==8	// pl
	"You must specify the name of the new country in all languages.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome do novo pa&iacute;s em todos os idiomas.";
#endif

const char *Txt_You_must_specify_the_name_of_the_new_group =
#if   L==1	// ca
	"Debe especificar el nombre del nuevo grupo.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the name of the new group.";	// Need bersetzung
#elif L==3	// en
	"You must specify the name of the new group.";
#elif L==4	// es
	"Debe especificar el nombre del nuevo grupo.";
#elif L==5	// fr
	"You must specify the name of the new group.";	// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre del nuevo grupo.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome del nuovo gruppo.";
#elif L==8	// pl
	"You must specify the name of the new group.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome do novo grupo.";
#endif

const char *Txt_You_must_specify_the_name_of_the_new_holiday =
#if   L==1	// ca
	"Debe especificar el nombre de la nueva festividad.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the name of the new holiday.";	// Need bersetzung
#elif L==3	// en
	"You must specify the name of the new holiday.";
#elif L==4	// es
	"Debe especificar el nombre de la nueva festividad.";
#elif L==5	// fr
	"You must specify the name of the new holiday.";	// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre de la nueva festividad.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome della nuova festivit&agrave;.";
#elif L==8	// pl
	"You must specify the name of the new holiday.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome do novo feriado.";
#endif

const char *Txt_You_must_specify_the_name_of_the_new_record_field =
#if   L==1	// ca
	"Debe especificar el nombre del nuevo campo de ficha.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the name of the new record field.";	// Need bersetzung
#elif L==3	// en
	"You must specify the name of the new record field.";
#elif L==4	// es
	"Debe especificar el nombre del nuevo campo de ficha.";
#elif L==5	// fr
	"You must specify the name of the new record field.";	// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre del nuevo campo de ficha.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome del nuovo campo di scheda.";
#elif L==8	// pl
	"You must specify the name of the new record field.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome do novo campo de cart&atilde;o.";
#endif

const char *Txt_You_must_specify_the_name_of_the_new_type_of_degree =
#if   L==1	// ca
	"Debe especificar el nombre del nuevo tipo de titulaci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the name of the new type of degree.";			// Need bersetzung
#elif L==3	// en
	"You must specify the name of the new type of degree.";
#elif L==4	// es
	"Debe especificar el nombre del nuevo tipo de titulaci&oacute;n.";
#elif L==5	// fr
	"You must specify the name of the new type of degree.";			// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre del nuevo tipo de titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome del nuovo tipo di laurea.";
#elif L==8	// pl
	"You must specify the name of the new type of degree.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome do novo tipo de grau.";
#endif

const char *Txt_You_must_specify_the_name_of_the_new_type_of_group =
#if   L==1	// ca
	"Debe especificar el nombre del nuevo tipo de grupo.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the name of the new type of group.";	// Need bersetzung
#elif L==3	// en
	"You must specify the name of the new type of group.";
#elif L==4	// es
	"Debe especificar el nombre del nuevo tipo de grupo.";
#elif L==5	// fr
	"You must specify the name of the new type of group.";	// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre del nuevo tipo de grupo.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome del nuovo tipo di gruppo.";
#elif L==8	// pl
	"You must specify the name of the new type of group.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome do novo tipo de grupo.";
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_banner =
#if   L==1	// ca
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo banner.";		// Necessita traduccio
#elif L==2	// de
	"You must specify the short name and the full name"
	" of the new banner.";		// Need bersetzung
#elif L==3	// en
	"You must specify the short name and the full name"
	" of the new banner.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo banner.";
#elif L==5	// fr
	"You must specify the short name and the full name"
	" of the new banner.";		// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo banner.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo"
	" del nuovo banner.";
#elif L==8	// pl
	"You must specify the short name and the full name"
	" of the new banner.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo"
	" do novo banner.";
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_centre =
#if   L==1	// ca
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo centro.";		// Necessita traduccio
#elif L==2	// de
	"You must specify the short name and the full name"
	" of the new centre.";		// Need bersetzung
#elif L==3	// en
	"You must specify the short name and the full name"
	" of the new centre.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo centro.";
#elif L==5	// fr
	"You must specify the short name and the full name"
	" of the new centre.";		// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo centro.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo"
	" del nuovo centro.";
#elif L==8	// pl
	"You must specify the short name and the full name"
	" of the new centre.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo"
	" do novo centro.";
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_classroom =
#if   L==1	// ca
	"Heu d'especificar el nom curt i el nom complet"
	" de la nova aula.";
#elif L==2	// de
	"Sie m&uml;ssen den Kurznamen und den vollst&auml;ndigen Namen"
	" des neuen Klassenzimmers angeben.";
#elif L==3	// en
	"You must specify the short name and the full name"
	" of the new classroom.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva aula.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le nom abr&eacute;g&eacute; et le nom complet"
	"  de la nouvelle salle de classe.";
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva aula.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo"
	" della nuova aula.";
#elif L==8	// pl
	"Musisz poda&cacute; kr&oacute;tk&aogon; nazw&eogon; i pe&lstrok;n&aogon; nazw&eogon;"
	" nowej klasy.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo"
	" da nova sala de aula.";
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_course =
#if   L==1	// ca
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva asignatura.";		// Necessita traduccio
#elif L==2	// de
	"You must specify the short name and the full name"
	" of the new course.";			// Need bersetzung
#elif L==3	// en
	"You must specify the short name and the full name"
	" of the new course.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva asignatura.";
#elif L==5	// fr
	"You must specify the short name and the full name"
	" of the new course.";			// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva asignatura.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo"
	" del nuovo corso.";
#elif L==8	// pl
	"You must specify the short name and the full name"
	" of the new course.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo"
	" da nova disciplina.";
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_degree =
#if   L==1	// ca
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva titulaci&oacute;n.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the short name and the full name"
	" of the new degree.";			// Need bersetzung
#elif L==3	// en
	"You must specify the short name and the full name"
	" of the new degree.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva titulaci&oacute;n.";
#elif L==5	// fr
	"You must specify the short name and the full name"
	" of the new degree.";			// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo"
	" della nuova laurea.";
#elif L==8	// pl
	"You must specify the short name and the full name"
	" of the new degree.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo"
	" do novo grau.";
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_department =
#if   L==1	// ca
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo departamento.";		// Necessita traduccio
#elif L==2	// de
	"You must specify the short name and the full name"
	" of the new department.";		// Need bersetzung
#elif L==3	// en
	"You must specify the short name and the full name"
	" of the new department.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo departamento.";
#elif L==5	// fr
	"You must specify the short name and the full name"
	" of the new department.";		// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo departamento.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo"
	" del nuovo dipartimento.";
#elif L==8	// pl
	"You must specify the short name and the full name"
	" of the new department.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo"
	" do novo departamento.";
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_institution =
#if   L==1	// ca
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva instituci&oacute;n.";		// Necessita traduccio
#elif L==2	// de
	"You must specify the short name and the full name"
	" of the new institution.";			// Need bersetzung
#elif L==3	// en
	"You must specify the short name and the full name"
	" of the new institution.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva instituci&oacute;n.";
#elif L==5	// fr
	"You must specify the short name and the full name"
	" of the new institution.";			// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva instituci&oacute;n.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo"
	" della nuova instituzione.";
#elif L==8	// pl
	"You must specify the short name and the full name"
	" of the new institution.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo"
	" da nova institui&ccedil;&atilde;o.";
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_link =
#if   L==1	// ca
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo enlace.";		// Necessita traduccio
#elif L==2	// de
	"You must specify the short name and the full name"
	" of the new link.";		// Need bersetzung
#elif L==3	// en
	"You must specify the short name and the full name"
	" of the new link.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo enlace.";
#elif L==5	// fr
	"You must specify the short name and the full name"
	" of the new link.";		// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo enlace.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo"
	" del nuovo link.";
#elif L==8	// pl
	"You must specify the short name and the full name"
	" of the new link.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo"
	" do novo link.";
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_email_domain =
#if   L==1	// ca
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo dominio de correo.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the short name and the full name"
	" of the new email domain.";		// Need bersetzung
#elif L==3	// en
	"You must specify the short name and the full name"
	" of the new email domain.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo dominio de correo.";
#elif L==5	// fr
	"You must specify the short name and the full name"
	" of the new email domain.";		// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo dominio de correo.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo"
	" del nuovo campo mail.";
#elif L==8	// pl
	"You must specify the short name and the full name"
	" of the new email domain.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo"
	" do novo dom&iacute;nio de email.";
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_place =
#if   L==1	// ca
	"Heu d'especificar el nom curt i el nom complet"
	" del nou lloc.";
#elif L==2	// de
	"Sie m&uml;ssen den Kurznamen und den vollst&auml;ndigen Namen"
	" des neuen Standortes angeben.";
#elif L==3	// en
	"You must specify the short name and the full name"
	" of the new place.";
#elif L==4	// es
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo lugar.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le nom abr&eacute;g&eacute; et le nom complet"
	" du nouvel emplacement.";
#elif L==6	// gn
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo lugar.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare il nome breve e completo"
	" del nuovo luogo.";
#elif L==8	// pl
	"Musisz poda&cacute; kr&oacute;tk&aogon; nazw&eogon; i pe&lstrok;n&aogon; nazw&eogon;"
	" nowego miejsca.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o nome abreviado e o nome completo"
	" da nova localiza&ccedil;&atilde;o.";
#endif

const char *Txt_You_must_specify_the_title_of_the_assignment =
#if   L==1	// ca
	"Cal especificar el t&iacute;tol de l'activitat.";
#elif L==2	// de
	"Die Aufgabe muss angegeben werden.";
#elif L==3	// en
	"You must specify the title of the assignment.";
#elif L==4	// es
	"Debe especificar el t&iacute;tulo de la actividad.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le titre de l'activit&eacute;.";
#elif L==6	// gn
	"Debe especificar el t&iacute;tulo de la actividad.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; necessario specificare il titolo dell'attivit&agrave;.";
#elif L==8	// pl
	"Musisz poda&cacute; tytu&lstrok; zadania.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o t&iacute;tulo da atividade.";
#endif

const char *Txt_You_must_specify_the_title_of_the_event =
#if   L==1	// ca
	"Cal especificar el t&iacute;tol de l'esdeveniment.";
#elif L==2	// de
	"Der Ereignis muss angegeben werden.";
#elif L==3	// en
	"You must specify the title of the event.";
#elif L==4	// es
	"Debe especificar el t&iacute;tulo del evento.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le titre du &eacute;v&eacute;nement.";
#elif L==6	// gn
	"Debe especificar el t&iacute;tulo del evento.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; necessario specificare il titolo del evento.";
#elif L==8	// pl
	"Musisz poda&cacute; tytu&lstrok; wydarzenia.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o t&iacute;tulo do evento.";
#endif

const char *Txt_You_must_specify_the_title_of_the_game =
#if   L==1	// ca
	"Cal especificar el t&iacute;tol del joc.";
#elif L==2	// de
	"Die Spiel muss angegeben werden.";
#elif L==3	// en
	"You must specify the title of the game.";
#elif L==4	// es
	"Debe especificar el t&iacute;tulo del juego.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le titre du jeu.";
#elif L==6	// gn
	"Debe especificar el t&iacute;tulo del juego.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; necessario specificare il titolo del gioco.";
#elif L==8	// pl
	"Musisz poda&cacute; tytu&lstrok; gra.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o t&iacute;tulo do jogo.";
#endif

const char *Txt_You_must_specify_the_title_of_the_project =
#if   L==1	// ca
	"Cal especificar el t&iacute;tol del projecte.";
#elif L==2	// de
	"Der Projekt muss angegeben werden.";
#elif L==3	// en
	"You must specify the title of the project.";
#elif L==4	// es
	"Debe especificar el t&iacute;tulo del proyecto.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le titre du projet.";
#elif L==6	// gn
	"Debe especificar el t&iacute;tulo del proyecto.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; necessario specificare il titolo del progetto.";
#elif L==8	// pl
	"Musisz poda&cacute; tytu&lstrok; projektu.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o t&iacute;tulo do projeto.";
#endif

const char *Txt_You_must_specify_the_title_of_the_survey =
#if   L==1	// ca
	"Cal especificar el t&iacute;tol de l'enquesta.";
#elif L==2	// de
	"Die Umfrage muss angegeben werden.";
#elif L==3	// en
	"You must specify the title of the survey.";
#elif L==4	// es
	"Debe especificar el t&iacute;tulo de la encuesta.";
#elif L==5	// fr
	"Vous devez sp&eacute;cifier le titre du sondage.";
#elif L==6	// gn
	"Debe especificar el t&iacute;tulo de la encuesta.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; necessario specificare il titolo del sondaggio.";
#elif L==8	// pl
	"Musisz poda&cacute; tytu&lstrok; badania.";
#elif L==9	// pt
	"Voc&ecirc; deve especificar o t&iacute;tulo do inqu&eacute;rito.";
#endif

const char *Txt_You_must_specify_the_URL_of_the_new_banner =
#if   L==1	// ca
	"Debe especificar el URL del nuevo banner.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the URL of the new banner.";	// Need bersetzung
#elif L==3	// en
	"You must specify the URL of the new banner.";
#elif L==4	// es
	"Debe especificar el URL del nuevo banner.";
#elif L==5	// fr
	"You must specify the URL of the new banner.";	// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el URL del nuevo banner.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare l'URL del nuovo banner.";
#elif L==8	// pl
	"You must specify the URL of the new banner.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o URL do novo banner.";
#endif

const char *Txt_You_must_specify_the_URL_of_the_new_link =
#if   L==1	// ca
	"Debe especificar el URL del nuevo enlace.";	// Necessita traduccio
#elif L==2	// de
	"You must specify the URL of the new link.";	// Need bersetzung
#elif L==3	// en
	"You must specify the URL of the new link.";
#elif L==4	// es
	"Debe especificar el URL del nuevo enlace.";
#elif L==5	// fr
	"You must specify the URL of the new link.";	// Besoin de traduction
#elif L==6	// gn
	"Debe especificar el URL del nuevo enlace.";	// Okoteve traduccin
#elif L==7	// it
	"Devi specificare l'URL del nuovo link.";
#elif L==8	// pl
	"You must specify the URL of the new link.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o URL do novo link.";
#endif

const char *Txt_You_must_specify_the_web_address_of_the_new_centre =
#if   L==1	// ca
	"Debe especificar la direcci&oacute;n web"
	" del nuevo centro.";				// Necessita traduccio
#elif L==2	// de
	"You must specify the web address"
	" of the new centre.";				// Need bersetzung
#elif L==3	// en
	"You must specify the web address"
	" of the new centre.";
#elif L==4	// es
	"Debe especificar la direcci&oacute;n web"
	" del nuevo centro.";
#elif L==5	// fr
	"You must specify the web address"
	" of the new centre.";				// Besoin de traduction
#elif L==6	// gn
	"Debe especificar la direcci&oacute;n web"
	" del nuevo centro.";				// Okoteve traduccin
#elif L==7	// it
	"Devi specificare l'indirizzo web"
	" del nuovo centro.";
#elif L==8	// pl
	"You must specify the web address"
	" of the new centre.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o endere&ccedil;o web"
	" do novo centro.";
#endif


const char *Txt_You_must_specify_the_web_address_of_the_new_degree =
#if   L==1	// ca
	"Debe especificar la direcci&oacute;n web"
	" de la nueva titulaci&oacute;n.";		// Necessita traduccio
#elif L==2	// de
	"You must specify the web address"
	" of the new degree.";				// Need bersetzung
#elif L==3	// en
	"You must specify the web address"
	" of the new degree.";
#elif L==4	// es
	"Debe especificar la direcci&oacute;n web"
	" de la nueva titulaci&oacute;n.";
#elif L==5	// fr
	"You must specify the web address"
	" of the new degree.";				// Besoin de traduction
#elif L==6	// gn
	"Debe especificar la direcci&oacute;n web"
	" de la nueva titulaci&oacute;n.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare l'indirizzo web"
	" della nuova laurea.";
#elif L==8	// pl
	"You must specify the web address"
	" of the new degree.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o endere&ccedil;o web"
	" do novo grau.";
#endif

const char *Txt_You_must_specify_the_web_address_of_the_new_department =
#if   L==1	// ca
	"Debe especificar la direcci&oacute;n web"
	" del nuevo departamento.";			// Necessita traduccio
#elif L==2	// de
	"You must specify the web address"
	" of the new department.";			// Need bersetzung
#elif L==3	// en
	"You must specify the web address"
	" of the new department.";
#elif L==4	// es
	"Debe especificar la direcci&oacute;n web"
	" del nuevo departamento.";
#elif L==5	// fr
	"You must specify the web address"
	" of the new department.";			// Besoin de traduction
#elif L==6	// gn
	"Debe especificar la direcci&oacute;n web"
	" del nuevo departamento.";			// Okoteve traduccin
#elif L==7	// it
	"Devi specificare l'indirizzo web"
	" del nuovo dipartimento.";
#elif L==8	// pl
	"You must specify the web address"
	" of the new department.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o endere&ccedil;o web"
	" do novo departamento.";
#endif

const char *Txt_You_must_specify_the_web_address_of_the_new_institution =
#if   L==1	// ca
	"Debe especificar la direcci&oacute;n web"
	" de la nueva instituci&oacute;n.";		// Necessita traduccio
#elif L==2	// de
	"You must specify the web address"
	" of the new institution.";			// Need bersetzung
#elif L==3	// en
	"You must specify the web address"
	" of the new institution.";
#elif L==4	// es
	"Debe especificar la direcci&oacute;n web"
	" de la nueva instituci&oacute;n.";
#elif L==5	// fr
	"You must specify the web address"
	" of the new institution.";			// Besoin de traduction
#elif L==6	// gn
	"Debe especificar la direcci&oacute;n web"
	" de la nueva instituci&oacute;n.";		// Okoteve traduccin
#elif L==7	// it
	"Devi specificare l'indirizzo web"
	" della nuova istituzione.";
#elif L==8	// pl
	"You must specify the web address"
	" of the new institution.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve especificar o endere&ccedil;o web"
	" da nova institui&ccedil;&atilde;o.";
#endif

const char *Txt_You_must_type_at_least_one_tag_for_the_question =
#if   L==1	// ca
	"Debe escribir al menos un descriptor para la pregunta.";	// Necessita traduccio
#elif L==2	// de
	"You must type at least one tag for the question.";		// Need bersetzung
#elif L==3	// en
	"You must type at least one tag for the question.";
#elif L==4	// es
	"Debe escribir al menos un descriptor para la pregunta.";
#elif L==5	// fr
	"You must type at least one tag for the question.";		// Besoin de traduction
#elif L==6	// gn
	"Debe escribir al menos un descriptor para la pregunta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi scrivere almeno un'etichetta per la domanda.";
#elif L==8	// pl
	"You must type at least one tag for the question.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve digitar pelo menos um descritor para a pergunta.";
#endif

const char *Txt_You_must_type_at_least_the_first_answer =
#if   L==1	// ca
	"Debe escribir al menos la primera respuesta.";	// Necessita traduccio
#elif L==2	// de
	"You must type at least the first answer.";	// Need bersetzung
#elif L==3	// en
	"You must type at least the first answer.";
#elif L==4	// es
	"Debe escribir al menos la primera respuesta.";
#elif L==5	// fr
	"You must type at least the first answer.";	// Besoin de traduction
#elif L==6	// gn
	"Debe escribir al menos la primera respuesta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi scrivere almeno la prima risposta.";
#elif L==8	// pl
	"You must type at least the first answer.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve digitar pelo menos a primeira resposta.";
#endif

const char *Txt_You_must_type_at_least_the_first_two_answers =
#if   L==1	// ca
	"Debe escribir al menos las dos primeras respuestas.";	// Necessita traduccio
#elif L==2	// de
	"You must type at least the first two answers.";	// Need bersetzung
#elif L==3	// en
	"You must type at least the first two answers.";
#elif L==4	// es
	"Debe escribir al menos las dos primeras respuestas.";
#elif L==5	// fr
	"You must type at least the first two answers.";	// Besoin de traduction
#elif L==6	// gn
	"Debe escribir al menos las dos primeras respuestas.";	// Okoteve traduccin
#elif L==7	// it
	"Devi scrivere almeno le prime due risposte.";
#elif L==8	// pl
	"You must type at least the first two answers.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve digitar pelo menos as duas primeiras respostas.";
#endif

const char *Txt_You_must_type_the_stem_of_the_question =
#if   L==1	// ca
	"Debe escribir el enunciado de la pregunta.";	// Necessita traduccio
#elif L==2	// de
	"You must type the stem of the question.";	// Need bersetzung
#elif L==3	// en
	"You must type the stem of the question.";
#elif L==4	// es
	"Debe escribir el enunciado de la pregunta.";
#elif L==5	// fr
	"You must type the stem of the question.";	// Besoin de traduction
#elif L==6	// gn
	"Debe escribir el enunciado de la pregunta.";	// Okoteve traduccin
#elif L==7	// it
	"Devi scrivere l'enunciato della domanda.";
#elif L==8	// pl
	"You must type the stem of the question.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; deve digitar a declara&ccedil;&atilde;o da pergunta.";
#endif

const char *Txt_You_need_an_XML_file_containing_a_list_of_questions =
#if   L==1	// ca
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas.";	// Necessita traduccio
#elif L==2	// de
	"You need an XML file"
	" containing a list of questions.";		// Need bersetzung
#elif L==3	// en
	"You need an XML file"
	" containing a list of questions.";
#elif L==4	// es
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas.";
#elif L==5	// fr
	"You need an XML file"
	" containing a list of questions.";		// Besoin de traduction
#elif L==6	// gn
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas.";	// Okoteve traduccin
#elif L==7	// it
	"You need an XML file"
	" containing a list of questions.";		// Bisogno di traduzione
#elif L==8	// pl
	"You need an XML file"
	" containing a list of questions.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; precisa de um arquivo XML"
	" contendo uma lista de perguntas.";
#endif

const char *Txt_You_should_read_the_following_information =
#if   L==1	// ca
	"Vost&egrave; ha de llegir la seg%uuml;ent informaci&oacute;:";
#elif L==2	// de
	"Sie sollten die folgenden Informationen lesen:";
#elif L==3	// en
	"You should read the following information:";
#elif L==4	// es
	"Debe leer la siguiente informaci&oacute;n:";
#elif L==5	// fr
	"Vous devez lire les informations suivantes:";
#elif L==6	// gn
	"Debe leer la siguiente informaci&oacute;n:";	// Okoteve traduccin
#elif L==7	// it
	"Dovresti leggere le seguenti informazioni:";
#elif L==8	// pl
	"Nale&zdot;y przeczyta&cacute; nast&eogon;puj&aogon;ce informacje:";
#elif L==9	// pt
	"Voc&ecirc; deve ler as seguintes informa&ccedil;&otilde;es:";
#endif

const char *Txt_You_were_already_enroled_as_X_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2	// de
	"You were already enroled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3	// en
	"You were already enroled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";
#elif L==4	// es
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";
#elif L==5	// fr
	"You were already enroled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6	// gn
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7	// it
	"You were already enroled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Bisogno di traduzione
#elif L==8	// pl
	"You were already enroled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Voc&ecirc; j&aacute; estava inscrito/a como <strong>%s</strong>"
	" na disciplina <strong>%s</strong>.";
#endif

const char *Txt_Your_comment_has_been_updated =
#if   L==1	// ca
	"El seu comentari ha estat actualitzat.";
#elif L==2	// de
	"Ihr Kommentar wurde aktualisiert.";
#elif L==3	// en
	"Your comment has been updated.";
#elif L==4	// es
	"Su comentario ha sido actualizado.";
#elif L==5	// fr
	"Votre commentaire a &eacute;t&eacute; mis &agrave; jour.";
#elif L==6	// gn
	"Su comentario ha sido actualizado.";		// Okoteve traduccin
#elif L==7	// it
	"Il tuo commento &egrave; stato aggiornato.";
#elif L==8	// pl
	"Komentarz zosta&lstrok; zaktualizowany.";
#elif L==9	// pt
	"O seu coment&aacute;rio foi atualizado.";
#endif

const char *Txt_Your_password_is_not_secure_enough =
#if   L==1	// ca
	"Su contrase&ntilde;a no es suficientemente segura.";	// Necessita traduccio
#elif L==2	// de
	"Your password is not secure enough.";			// Need bersetzung
#elif L==3	// en
	"Your password is not secure enough.";
#elif L==4	// es
	"Su contrase&ntilde;a no es suficientemente segura.";
#elif L==5	// fr
	"Your password is not secure enough.";			// Besoin de traduction
#elif L==6	// gn
	"Su contrase&ntilde;a no es suficientemente segura.";	// Okoteve traduccin
#elif L==7	// it
	"La tua password non &egrave; sicura abbastanza.";
#elif L==8	// pl
	"Your password is not secure enough.";			// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Sua senha n&atilde;o &eacute; segura o suficiente.";
#endif

const char *Txt_Your_password_must_be_at_least_X_characters_and_can_not_contain_spaces_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1	// ca
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use solo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido,"
	" ni una palabra conocida.";			// Necessita traduccio
#elif L==2	// de
	"Your password must be at least %u characters"
	" and can not contain spaces."
	" It's very important not to use only"
	" numeric digits, or a name or surname,"
	" or a known word.";				// Need bersetzung
#elif L==3	// en
	"Your password must be at least %u characters"
	" and can not contain spaces."
	" It's very important not to use only"
	" numeric digits, or a name or surname,"
	" or a known word.";
#elif L==4	// es
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use solo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido,"
	" ni una palabra conocida.";
#elif L==5	// fr
	"Your password must be at least %u characters"
	" and can not contain spaces."
	" It's very important not to use only"
	" numeric digits, or a name or surname,"
	" or a known word.";				// Besoin de traduction
#elif L==6	// gn
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use solo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido,"
	" ni una palabra conocida.";			// Okoteve traduccin
#elif L==7	// it
	"La tua password deve essere almeno di %u caratteri"
	" e non pu&ograve; contenere spazi."
	" E' molto importante non usare solo"
	" tasti numerici, o un nome o cognome,"
	" o una parola conosciuta.";
#elif L==8	// pl
	"Your password must be at least %u characters"
	" and can not contain spaces."
	" It's very important not to use only"
	" numeric digits, or a name or surname,"
	" or a known word.";				// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Sua senha deve ter pelo menos %u caracteres"
	" e n&atilde;o pode conter espa&ccedil;os."
	" &Eacute; muito importante n&atilde;o usar apenas"
	" dgitos num&eacute;ricos, nem um nome ou apelido,"
	" nem uma palavra conhecida.";
#endif

const char *Txt_Your_personal_data_have_been_updated =
#if   L==1	// ca
	"Sus datos personales se han actualizado.";	// Necessita traduccio
#elif L==2	// de
	"Your personal data have been updated.";	// Need bersetzung
#elif L==3	// en
	"Your personal data have been updated.";
#elif L==4	// es
	"Sus datos personales se han actualizado.";
#elif L==5	// fr
	"Your personal data have been updated.";	// Besoin de traduction
#elif L==6	// gn
	"Sus datos personales se han actualizado.";	// Okoteve traduccin
#elif L==7	// it
	"I tuoi dati personali sono stati aggiornati.";
#elif L==8	// pl
	"Your personal data have been updated.";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Seus dados pessoais foram atualizados.";
#endif

const char *Txt_Your_preferences_about_notifications_have_changed =
#if   L==1	// ca
	"Les seves prefer&egrave;ncies sobre les notificacions han canviat.";
#elif L==2	// de
	"Your preferences about notifications have changed.";		// Need bersetzung
#elif L==3	// en
	"Your preferences about notifications have changed.";
#elif L==4	// es
	"Sus preferencias sobre las notificaciones han cambiado.";
#elif L==5	// fr
	"Your preferences about notifications have changed.";		// Besoin de traduction
#elif L==6	// gn
	"Sus preferencias sobre las notificaciones han cambiado.";	// Okoteve traduccin
#elif L==7	// it
	"Le tue preferenze sulle notifiche sono cambiate.";
#elif L==8	// pl
	"Your preferences about notifications have changed.";		// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Suas prefer&ecirc;ncias sobre notifica&ccedil;&otilde;es foram alteradas.";
#endif

const char *Txt_Your_record_card_in_this_course_has_been_updated =
#if   L==1	// ca
	"Su ficha en la asignatura se ha actualizado:";		// Necessita traduccio
#elif L==2	// de
	"Your record card in this course has been updated:";	// Need bersetzung
#elif L==3	// en
	"Your record card in this course has been updated:";
#elif L==4	// es
	"Su ficha en la asignatura se ha actualizado:";
#elif L==5	// fr
	"Your record card in this course has been updated:";	// Besoin de traduction
#elif L==6	// gn
	"Su ficha en la asignatura se ha actualizado:";		// Okoteve traduccin
#elif L==7	// it
	"La tua scheda in questo corso &egrave; stata aggiornata:";
#elif L==8	// pl
	"Your record card in this course has been updated:";	// Potrzebujesz tlumaczenie
#elif L==9	// pt
	"Seu cart&atilde;o nesta disciplina foi atualizado:";
#endif

const char *Txt_Your_request_for_enrolment_as_X_in_the_course_Y_has_been_accepted_for_processing =	// Warning: it is very important to include two %s in the following sentences
#if   L==1	// ca
	"La seva petici&oacute; d'inscripci&oacute; com <strong>%s</strong>"
	" en l'assignatura <strong>%s</strong> ha estat realitzada."
	" La petici&oacute; ser&agrave; revisada per un professor o un administrador."
	" Vost&egrave; rebr&agrave; una notificaci&oacute; quan s'accepti la seva inscripci&oacute;.";
#elif L==2	// de
	"Sein Antrag auf Registrierung als <strong>%s</strong>"
	" in Kurs <strong>%s</strong> gemacht wurde."
	" Ihre Anfrage wird durch einen Lehrer oder Administrator &uuml;berpr&uuml;ft werden."
	" Sie werden benachrichtigt, sobald die Anmeldung akzeptiert wird.";
#elif L==3	// en
	"Your request for enrolment as <strong>%s</strong>"
	" in the course <strong>%s</strong> has been accepted for processing."
	" Your request will be reviewed by a teacher or administrator."
	" You will be notified when registration is accepted.";
#elif L==4	// es
	"Su petici&oacute;n de inscripci&oacute;n como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong> ha sido realizada."
	" La petici&oacute;n ser&aacute; revisada por un/a profesor/a o un/a administrador/a."
	" Usted recibir&aacute; una notificaci&oacute;n cuando se acepte su inscripci&oacute;n.";
#elif L==5	// fr
	"Votre demande d'enregistrement en tant que <strong>%s</strong>"
	" dans la mati&egrave;re <strong>%s</strong> a &eacute;t&eacute; fait."
	" Votre demande sera examin&eacute;e par un enseignant ou un administrateur."
	" Vous serez avis&eacute; lorsque l'enregistrement est accept&eacute;e.";
#elif L==6	// gn
	"Su petici&oacute;n de inscripci&oacute;n como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong> ha sido realizada."
	" La petici&oacute;n ser&aacute; revisada por un/a profesor/a o un/a administrador/a."
	" Usted recibir&aacute; una notificaci&oacute;n cuando se acepte su inscripci&oacute;n.";	// Okoteve traduccin
#elif L==7	// it
	"&Egrave; stato fatto la sua domanda di registrazione come <strong>%s</strong>"
	" nel corso <strong>%s</strong>."
	" La richiesta verr&agrave; esaminata da un insegnante o un amministratore."
	" Riceverai una notifica quando la registrazione &egrave; stata accettata.";
#elif L==8	// pl
	"Jego wniosek o rejestracj&eogon; jako <strong>%s</strong>"
	" w <strong>%s</strong> przedmiotu zosta&lstrok;a dokonana."
	" Twoja pro&sacute;ba zostanie poddane przegl&aogon;dowi przez nauczyciela lub administratora."
	" Zostaniesz powiadomiony, gdy rejestracja zostanie zaakceptowana.";
#elif L==9	// pt
	"Seu pedido de registo como <strong>%s</strong>"
	" na disciplina <strong>%s</strong> tem sido feito."
	" O pedido ser&aacute; analisado por um professor ou administrador."
	" Voc&ecirc; ser&aacute; notificado quando o registro &eacute; aceito.";
#endif

const char *Txt_ZIP_file =
#if   L==1	// ca
	"Fitxer ZIP";
#elif L==2	// de
	"ZIP-Datei";
#elif L==3	// en
	"ZIP file";
#elif L==4	// es
	"Archivo ZIP";
#elif L==5	// fr
	"Fichier ZIP";
#elif L==6	// gn
	"Archivo ZIP";	// Okoteve traduccin
#elif L==7	// it
	"File ZIP";
#elif L==8	// pl
	"Plik ZIP";
#elif L==9	// pt
	"Arquivo ZIP";
#endif
